* SUBROUTINE EILS33                ALL SYSTEMS                92/12/01
C PORTABILITY : ALL SYSTEMS
C 92/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
*  INPUT MATRICES FOR CONSTRAINED MINIMIZATION.
*  SPARSE VERSION OF QUADRATIC PROGRAMMING.
*
* PARAMETERS :
*  II  NF NUMBER OF VARIABLES.
*  II  NC NUMBER OF CONSTRAINTS.
*  II  MROWS DECLARED NUMBER OF VARIABLES.
*  II  NZ  SIZE OF ARRAYS CG/JCG.
*  IU  ICG(MROWS) VECTOR SHOWING BEGINNINGS OF THE INPUT MATRIX ROWS.
*  IU  JCG(NZ) INPUT MATRIX COLUMN INDICES.
*  RU  CG(NZ) INPUT MATRIX VALUES.
*  IO  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS:
*                                     =1 FOR LOWER BOUNDED VARIABLE.
*                                     =2 FOR UPPER BOUNDED VARIABLE.
*                                     =3 FOR LOWER AND UPPER BOUNDED.
*  RO  XL(NF)  VECTOR CONTAINING LOWER BOUNDS FOR VARIABLES.
*  RO  XU(NF)  VECTOR CONTAINING UPPER BOUNDS FOR VARIABLES.
*  IO  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS:
*                                     =1 FOR LOWER BOUNDED ROW.
*                                     =2 FOR UPPER BOUNDED ROW.
*                                     =3 FOR LOWER AND UPPER BOUNDED ROW
*  RO  CL(NC)  VECTOR CONTAINING LOWER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RO  CU(NC)  VECTOR CONTAINING UPPER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  II  NEXT  NUMBER OF THE TEST PROBLEM.
*  IO  IER  ERROR INDICATOR: =0 FOR SUCCESFUL WORK, <>0 WHEN SOME
*            ERROR IS DETECTED.
*
      SUBROUTINE EILS33(NF,NC,NCL,MROWS,NZ,ICG,JCG,CG,IX,XL,XU,IC,
     *                  CL,CU,NEXT,IER)
      INTEGER NF,NC,NCL,MROWS,NZ,NEXT,IER
      INTEGER ICG(MROWS),JCG(NZ),IX(NF),IC(NC)
      $FLOAT CG(NZ),XL(NF),XU(NF),CL(NC),CU(NC)
      $FLOAT FMIN,XMAX,ONE,TWO,THREE
      $FLOAT NGNE,NGT,FTH
      $FLOAT NTW
      INTEGER NFL,MC,NDV3
      $FLOAT ZERO,FOUR,FIVE,SIX,SEVEN,EIGHT,TEN
      PARAMETER(NTW=-1.02$P 0,ZERO=0.0$P 0)
      PARAMETER(ONE=1.0$P 0,TWO=2.0$P 0,THREE=3.0$P 0)
      PARAMETER(FOUR=4.0$P 0, FIVE=5.0$P 0,SIX=6.0$P 0)
      PARAMETER(SEVEN=7.0$P 0,EIGHT=8.0$P 0,TEN=1.0$P 1)
      PARAMETER(NGNE=-1.0$P 0,NGT=-0.2$P 0,FTH=0.004$P 0)
      INTEGER I
      $FLOAT ETA9
      PARAMETER (ETA9=1.0$P 60)
      FMIN=-ETA9
      XMAX=ETA9
      IER=0
      GO TO (10,20,30,40,50,60,70,80,90,100,110) NEXT
 10   NF=240
      NFL=NF
      NCL=2*NFL/3-1
      NC=NCL
C
C     NUMBER OF EQUATIONS IS 2*NDV3-1
C
      NDV3=NFL/3
      DO 15 I=1,NDV3-1
      CALL $UKMCI1(I,I,NGNE)
      CALL $UKMCI1(I,I+1,ONE)
      CALL $UKMCI1(I,I+NDV3,NGT)
      CALL $UKMCI1(I+NDV3,I,FTH)
15    CONTINUE
      CALL $UKMCI1(NDV3,NDV3+1,ONE)
      CALL $UKMCI1(NDV3,2*NDV3+1,NGT)
      DO 16 I=NDV3+1,2*NDV3-1
      CALL $UKMCI1(I,I,NTW)
      CALL $UKMCI1(I,I+1,ONE)
      CALL $UKMCI1(I,I+NDV3+1,NGT)
16    CONTINUE
      IX(1)=3
      XL(1)=10
      XU(1)=10
      DO 17 I=2,NDV3
      IX(I)=3
      XL(I)=-1000
      XU(I)=1000
17    CONTINUE
      DO 18 I=NDV3+1,2*NDV3-1
      IX(I)=1
      XL(I)=-1
18    CONTINUE
      IX(2*NDV3)=3
      XL(2*NDV3)=0
      XU(2*NDV3)=0
      DO 19 I=2*NDV3+1,3*NDV3
      IX(I)=3
      XL(I)=NGT
      XU(I)=-NGT
19    CONTINUE
      DO 1 I=1,NDV3-1
      CL(I)=-TWO
      CU(I)=-TWO
1     CONTINUE
      CL(NDV3)=-0.02$P 0
      CU(NDV3)=-0.02$P 0
      DO 2 I=NDV3+2,NCL
      CL(I)=-0.18$P 0
      CU(I)=-0.18$P 0
2     CONTINUE
      DO 3 I=1,NCL
      IC(I)=3
3     CONTINUE
      CL(1)=0
      CU(1)=0
      CL(NDV3+1)=10.02$P 0
      CU(NDV3+1)=10.02$P 0
      CL(2*NDV3-1)=-10.18$P 0
      CU(2*NDV3-1)=-10.18$P 0
      RETURN
   20 CONTINUE
      NC=1
      NCL=1
      NFL=2
      NF=2
      CALL $UKMCI1(1,1,10.0$P 0)
      CALL $UKMCI1(1,2,-ONE)
      IC(1)=1
      CL(1)=10.0$P 0
      IX(1)=3
      IX(2)=3
      XL(1)=2
      XU(1)=50
      XL(2)=-50
      XU(2)=50
      RETURN
   30 CONTINUE
      NC=1
      NCL=1
      CALL $UKMCI1(1,1,ONE)
      CALL $UKMCI1(1,2,TWO)
      CALL $UKMCI1(1,3,THREE)
      IC(1)=5
      CL(1)=ONE
      IX(1)=6
      IX(2)=6
      IX(3)=6
      RETURN
   40 CONTINUE
      NC=1
      NCL=1
      IC(1)=1
      CL(1)=-THREE
      CALL $UKMCI1(1,1,-ONE)
      CALL $UKMCI1(1,2,-ONE)
      CALL $UKMCI1(1,3,-TWO)
      DO 41 I=1,3
        IX(I)=1
        XL(I)=ZERO
   41 CONTINUE
      RETURN
   50 CONTINUE
      NC=6
      NCL=6
      DO 52 I=1,6
        IC(I)=1
   52 CONTINUE
      CL(1)=-8
      CL(2)=-12
      CL(3)=-12
      CL(4)=-8
      CL(5)=-8
      CL(6)=-5
      DO 51 I=1,4
        IX(I)=1
        XL(I)=ZERO
   51 CONTINUE
      CALL $UKMCI1(1,1,-ONE)
      CALL $UKMCI1(1,2,-TWO)
      CALL $UKMCI1(2,1,-FOUR)
      CALL $UKMCI1(2,2,-ONE)
      CALL $UKMCI1(3,1,-THREE)
      CALL $UKMCI1(3,2,-FOUR)
      CALL $UKMCI1(4,3,-THREE)
      CALL $UKMCI1(4,4,-ONE)
      CALL $UKMCI1(5,3,-ONE)
      CALL $UKMCI1(5,4,-TWO)
      CALL $UKMCI1(6,3,-ONE)
      CALL $UKMCI1(6,4,-ONE)
      RETURN
   60 CONTINUE
      NC=2
      NCL=2
      IC(1)=5
      IC(2)=5
      CL(1)=5
      CL(2)=-3
      DO 61 I=1,5
        IX(I)=6
   61 CONTINUE
      CALL $UKMCI1(1,1,ONE)
      CALL $UKMCI1(1,2,ONE)
      CALL $UKMCI1(1,3,ONE)
      CALL $UKMCI1(1,4,ONE)
      CALL $UKMCI1(1,5,ONE)
      CALL $UKMCI1(2,3,ONE)
      CALL $UKMCI1(2,4,-TWO)
      CALL $UKMCI1(2,5,-TWO)
      RETURN
   70 CONTINUE
      NC=3
      NCL=3
      IC(1)=5
      IC(2)=5
      IC(3)=5
      CL(1)=4
      CL(2)=ZERO
      CL(3)=ZERO
      DO 71 I=1,5
        IX(I)=6
   71 CONTINUE
      CALL $UKMCI1(1,1,ONE)
      CALL $UKMCI1(1,2,THREE)
      CALL $UKMCI1(2,3,ONE)
      CALL $UKMCI1(2,4,ONE)
      CALL $UKMCI1(2,5,-TWO)
      CALL $UKMCI1(3,2,ONE)
      CALL $UKMCI1(3,5,-ONE)
      RETURN
   80 CONTINUE
      NC=3
      NCL=3
      IC(1)=5
      IC(2)=5
      IC(3)=5
      CL(1)=ZERO
      CL(2)=ZERO
      CL(3)=ZERO
      DO 81 I=1,5
        IX(I)=6
   81 CONTINUE
      CALL $UKMCI1(1,1,ONE)
      CALL $UKMCI1(1,2,THREE)
      CALL $UKMCI1(2,3,ONE)
      CALL $UKMCI1(2,4,ONE)
      CALL $UKMCI1(2,5,-TWO)
      CALL $UKMCI1(3,2,ONE)
      CALL $UKMCI1(3,5,-ONE)
      RETURN
   90 CONTINUE
      NC=3
      NCL=3
      IC(1)=5
      IC(2)=5
      IC(3)=5
      CL(1)=ZERO
      CL(2)=ZERO
      CL(3)=ZERO
      DO 91 I=1,5
        IX(I)=3
        XL(I)=-TEN
        XU(I)=TEN
   91 CONTINUE
      CALL $UKMCI1(1,1,ONE)
      CALL $UKMCI1(1,2,THREE)
      CALL $UKMCI1(2,3,ONE)
      CALL $UKMCI1(2,4,ONE)
      CALL $UKMCI1(2,5,-TWO)
      CALL $UKMCI1(3,2,ONE)
      CALL $UKMCI1(3,5,-ONE)
      RETURN
  100 CONTINUE
      NC=3
      NCL=3
      IC(1)=1
      IC(2)=1
      IC(3)=1
      CL(1)=-FIVE
      CL(2)=-FOUR
      CL(3)=1.5$P 0
      DO 101 I=1,4
        IX(I)=1
        XL(I)=ZERO
  101 CONTINUE
      CALL $UKMCI1(1,1,-ONE)
      CALL $UKMCI1(1,2,-TWO)
      CALL $UKMCI1(1,3,-ONE)
      CALL $UKMCI1(1,4,-ONE)
      CALL $UKMCI1(2,1,-THREE)
      CALL $UKMCI1(2,2,-ONE)
      CALL $UKMCI1(2,3,-TWO)
      CALL $UKMCI1(2,4,ONE)
      CALL $UKMCI1(3,2,ONE)
      CALL $UKMCI1(3,3,FOUR)
      RETURN
  110 CONTINUE
      NC=17
      NCL=17
C
C FIRST TWELVE CONSTRAINTS
C
      DO 111 I=1,12
        IC(I)=3
        CL(I)=-SEVEN
        CU(I)=SIX
  111 CONTINUE
      DO 112 I=5,8
        CU(I)=SEVEN
  112 CONTINUE
      CALL $UKMCI1(1,1,-ONE)
      CALL $UKMCI1(1,4,ONE)
      CALL $UKMCI1(2,7,ONE)
      CALL $UKMCI1(2,4,-ONE)
      CALL $UKMCI1(3,10,ONE)
      CALL $UKMCI1(3,7,-ONE)
      CALL $UKMCI1(4,13,ONE)
      CALL $UKMCI1(4,10,-ONE)
      CALL $UKMCI1(5,5,ONE)
      CALL $UKMCI1(5,2,-ONE)
      CALL $UKMCI1(6,8,ONE)
      CALL $UKMCI1(6,5,-ONE)
      CALL $UKMCI1(7,11,ONE)
      CALL $UKMCI1(7,8,-ONE)
      CALL $UKMCI1(8,14,ONE)
      CALL $UKMCI1(8,11,-ONE)
      CALL $UKMCI1(9,6,ONE)
      CALL $UKMCI1(9,3,-ONE)
      CALL $UKMCI1(10,9,ONE)
      CALL $UKMCI1(10,6,-ONE)
      CALL $UKMCI1(11,12,ONE)
      CALL $UKMCI1(11,9,-ONE)
      CALL $UKMCI1(12,15,ONE)
      CALL $UKMCI1(12,12,-ONE)
      DO 113 I=13,17
        IC(I)=1
 113  CONTINUE
      CL(13)=60
      CL(14)=70
      CL(15)=100
      CL(16)=50
      CL(17)=85
      CALL $UKMCI1(13,1,ONE)
      CALL $UKMCI1(13,2,ONE)
      CALL $UKMCI1(13,3,ONE)
      CALL $UKMCI1(14,7,ONE)
      CALL $UKMCI1(14,8,ONE)
      CALL $UKMCI1(14,9,ONE)
      CALL $UKMCI1(15,13,ONE)
      CALL $UKMCI1(15,14,ONE)
      CALL $UKMCI1(15,15,ONE)
      CALL $UKMCI1(16,4,ONE)
      CALL $UKMCI1(16,5,ONE)
      CALL $UKMCI1(16,6,ONE)
      CALL $UKMCI1(17,10,ONE)
      CALL $UKMCI1(17,11,ONE)
      CALL $UKMCI1(17,12,ONE)
      DO 114 I=1,15
        IX(I)=3
  114 CONTINUE
      XL(1)=8
      XU(1)=21
      XL(2)=43
      XU(2)=57
      XL(3)=3
      XU(3)=16
      XL(4)=ZERO
      XU(4)=90
      XL(5)=ZERO
      XU(5)=120
      XL(6)=ZERO
      XU(6)=60
      XL(7)=ZERO
      XU(7)=90
      XL(8)=ZERO
      XU(8)=120
      XL(9)=ZERO
      XU(9)=60
      XL(10)=ZERO
      XU(10)=90
      XL(11)=ZERO
      XU(11)=120
      XL(12)=ZERO
      XU(12)=60
      XL(13)=ZERO
      XU(13)=90
      XL(14)=ZERO
      XU(14)=120
      XL(15)=ZERO
      XU(15)=60
      RETURN
      END
