* SUBROUTINE EIQS35                ALL SYSTEMS                00/12/01
C PORTABILITY : ALL SYSTEMS
C 00/12/01 RU : ORIGINAL VERSION
*
* PURPOSE :
*  INPUT OF HESSIAN MATRIX FOR SPARSE QP.
*
* PARAMETERS :
*  II  N  NUMBER OF VARIABLES.
*  II  M  NUMBER OF CONSTRAINTS.
*  II  IMAX  MAXIMUM NUMBER OF NONZERO ELEMENTS IN THE SPARSE ROW.
*  II  LMAX  MAXIMUM NUMBER OF NONZERO ELEMENTS IN THE PROBLEM MATRICES.
*  IO  MG  NUMBER OF NONZERO ELEMENTS OF THE HESSIAN MATRIX.
*  RO  VG(LMAX)  NUMERICAL VALUES OF NONZERO ELEMENTS OF THE HESSIAN
*         MATRIX.
*  IO  JG(LMAX)  COLUMN INDICES OF NONZERO ELEMENTS OF THE HESSIAN
*         MATRIX.
*  IO  KG(N+1)  ROW POINTERS OF THE HESSIAN MATRIX.
*  RO  Q(N)  GRADIENT VECTOR.
*  IO  MB  NUMBER OF NONZERO ELEMENTS OF THE JACOBIAN MATRIX.
*  RO  VB(LMAX)  NUMERICAL VALUES OF NONZERO ELEMENTS OF THE JACOBIAN
*         MATRIX.
*  IO  JB(LMAX)  COLUMN INDICES OF NONZERO ELEMENTS OF THE JACOBIAN
*         MATRIX.
*  IO  KB(N+1)  ROW POINTERS OF THE JACOBIAN MATRIX.
*  IO  IB(M)  TYPES OF CONSTRAINTS.
*  RO  D(M)  RIGHT HAND SIDES OF CONSTRAINTS.
*  RO  X(N)  THE SOLUTION VECTOR.
*  RO  FMIN  MINIMUM VALUE OF THE OBJECTIVE FUNCTION.
*  RO  LAMBDA(M) VECTOR OF THE LAGRANGE MULTIPLIERS.
*  IA  IW((N+M)*(IMAX+2)) AUXILIARY VECTOR.
*  RA  W((N+M)*(IMAX+2)) AUXILIARY VECTOR.
*  II  NEXT  NUMBER OF THE TEST PROBLEM.
*  IO  IERR  ERROR INDICATOR.
*
* METHOD :
* V.RUGGIERO, L.ZANI : ON A CLASS OF ITERATIVE METHODS FOR LARGE-SCALE
* CONVEX QUADRATIC PROGRAMS. REND. DEL CIRC. MATEM. DI PALERMO,
* SERIE II, SUPPL. 58 (1999), PP. 213-227.
* G.W.STEWART : THE EFFICIENT GENERATION OF RANDOM ORTHOGONAL MATRICES
* WITH AN APPLICATION TO CONDITION ESTIMATORS. SIAM J. NUMER. ANAL. 17,
* NO. 3 (1980), PP. 403-409.
*
      SUBROUTINE EIQS35(N,M,IMAX,LMAX,MG,VG,JG,KG,Q,MB,VB,JB,KB,IB,D,X,
     & FMIN,LAMBDA,IW,W,NEXT,IERR)
      INTEGER N,M,NEQ,NAC,CONDG,CONDB,CONDZ,CONDA,GRANK,ZRANK,INDG,INDB,
     & IRAND,NDEG,INDEQ,IMAX,LMAX,MG,JG(LMAX),KG(N+1),MB,JB(LMAX),
     & KB(M+1),IB(M),IW((N+M)*(IMAX+2)),NEXT,IERR
      $FLOAT GLMIN,BLMIN,ZLMIN,ALMIN,SPARG,SPARB,VG(LMAX),Q(N),VB(LMAX),
     & D(M),X(N),FMIN,LAMBDA(M),W((N+M)*(IMAX+2))
      INTEGER I,LG,LB,LTEMPI,LTEMPJ,LIG,LIB,LINDI,LINDJ
      IERR=0
      IF (NEXT.GT.30) THEN
      IERR=1
      RETURN
      ENDIF
      IRAND=-1
      N=MAX(N,1)
      IF ((NEXT-1)/5.EQ.0) M=N/3
      IF ((NEXT-1)/5.EQ.1) M=N/2
      IF ((NEXT-1)/5.EQ.2) M=2*N/3
      IF ((NEXT-1)/5.EQ.3) M=N
      IF ((NEXT-1)/5.EQ.4) M=2*N
      IF ((NEXT-1)/5.EQ.5) M=3*N
      IF (MOD(NEXT,5).EQ.1) THEN
      NEQ=MIN(N-2,M)
      NAC=NEQ
      ENDIF
      IF (MOD(NEXT,5).EQ.2) THEN
      NEQ=MIN(N/2,M/2)
      NAC=NEQ+1
      ENDIF
      IF (MOD(NEXT,5).EQ.3) THEN
      NEQ=MIN(N/3,M/3)
      NAC=MIN(N/2,M/2)
      ENDIF
      IF (MOD(NEXT,5).EQ.4) THEN
      NEQ=0
      NAC=MIN(2*N/3,2*M/3)
      ENDIF
      IF (MOD(NEXT,5).EQ.0) THEN
      NEQ=MIN(N/2,M/2)
      NAC=MIN(N-2,M-2)
      ENDIF
      CONDG=4
      CONDB=1
      CONDZ=2
      CONDA=1
      GLMIN=1.0$P-4
      BLMIN=1.0$P-1
      ZLMIN=1.0$P-2
      ALMIN=1.0$P-1
      INDG=1
      INDB=1
      GRANK=N
      ZRANK=3*N/4
      NDEG=1
      SPARG=1.0$P 2*(1.0$P 0-2.0$P-1*$DBLE(IMAX)/$DBLE(N))
      SPARB=1.0$P 2*(1.0$P 0-2.0$P-1*$DBLE(IMAX)/$DBLE(N))
      INDEQ=0
      LG=1
      LB=LG+N*IMAX
      LTEMPI=LB+M*IMAX
      LTEMPJ=LTEMPI+N+M
      LIG=1
      LIB=LIG+N*IMAX
      LINDI=LIB+M*IMAX
      LINDJ=LINDI+N+M
      CALL EIQSXX(N,M,NEQ,NAC,CONDG,CONDB,CONDZ,CONDA,GLMIN,BLMIN,
     & ZLMIN,ALMIN,GRANK,ZRANK,INDG,INDB,IRAND,NDEG,SPARG,SPARB,INDEQ,
     & IMAX,LMAX,MG,VG,JG,KG,Q,MB,VB,JB,KB,D,X,FMIN,LAMBDA,W(LG),
     & IW(LIG),W(LB),IW(LIB),W(LTEMPI),W(LTEMPJ),IW(LINDI),IW(LINDJ))
      DO 1 I=1,M
      IF (I.LE.NEQ) THEN
      IB(I)=5
      LAMBDA(I)=-LAMBDA(I)
      ELSE
      IB(I)=1
      ENDIF
    1 CONTINUE
      RETURN
      END
