* SUBROUTINE UA1HS3                ALL SYSTEMS                99/12/01
C PORTABILITY : ALL SYSTEMS
C 99/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* NUMERICAL COMPUTATION OF THE HESSIAN MATRIX OF THE APPROXIMATED
* FUNCTION USING ITS GRADIENTS.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  NA  NUMBER OF FUNCTIONS.
*  II  MA  NUMBER OF ELEMENTS IN THE FIELD AG.
*  II  M  NUMBER OF NONZERO ELEMENTS OF THE SPARSE HESSIAN MATRIX.
*  II  KA  INDEX OF THE SELECTED FUNCTION.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XN(NF)  VECTOR OF SCALING FACTORS.
*  RO  HA(M) HESSIAN MATRIX OF THE APPROXIMATED FUNCTION.
*  RI  GA(NF)  GRADIENT OF THE APPROXIMATED FUNCTION.
*  RA  GO(NF)  AUXILIARY VECTOR.
*  RI  IAG(NA+1)  POSITION OF THE FIRST ROWS ELEMENTS IN THE FIELD AG.
*  RI  JAG(MA)  COLUMN INDICES OF ELEMENTS IN THE FIELD AG.
*  RI  FA  VALUE OF THE SELECTED FUNCTION.
*  RI  ETA1  PRECISION OF THE COMPUTED VALUES.
*  RI  INDEX  INDEX IN THE FIELD HA.
*
* COMMON DATA :
*  II  KBF  TYPE OF BOUNDS. KBF=0-BOUNDS ARE NOT USED. KBF=1-ONE SIDED
*         BOUNDS. KBF=1-TWO SIDED BOUNDS.
*  II  NORMF  SCALING SPECIFICATION. NORMF=0-NO SCALING IS PERFORMED.
*         NORMF=1-SCALING FACTORS ARE DETERMINED AUTOMATICALLY.
*         NORMF=2-SCALING FACTORS ARE SUPPLIED BY USER.
*
* STATUS VARIABLES :
*  NS,ISP,TSS
*
* SUBPROGRAMS USED :
*  S   UXVCOP  COPYING OF A VECTOR.
*  S   UYPRO1  PROLOGUE.
*  S   UYEPI1  EPILOGUE.
*  S   UYSET0  STATUS DEFINITION.
*
      SUBROUTINE UA1HS3(NF,NA,MA,M,KA,X,IX,XN,HA,GA,GO,IAG,JAG,FA,ETA1,
     & INDEX)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER NF,NA,MA,M,KA,IX(NF),IAG(NA+1),JAG(MA),INDEX
      REAL*8 X(NF),XN(NF),HA(M),GA(NF),GO(NF),FA,ETA1
      REAL*8 XSTEP,XTEMP,FTEMP,ETA
      INTEGER I,J,IJ,IND
      INTEGER IVAR,JVAR,KVAR,LVAR,MVAR
      SAVE  XSTEP,XTEMP,FTEMP,ETA
      SAVE IVAR,JVAR,KVAR,LVAR,MVAR
      IND(I,J)=MAX(I,J)*(MAX(I,J)-1)/2+MIN(I,J)
      IF (TSS(NS).NE.'UAHX') CALL UYPRO1('UAHX',-1)
      GO TO (1,3)  ISP(NS)
    1 CONTINUE
      ETA=SQRT(ETA1)
      FTEMP=FA
      CALL UXVCOP(NF,GA,GO)
      MVAR=IAG(KA)-1
      KVAR=MVAR+1
    2 CONTINUE
      IF (KVAR.GE.IAG(KA+1)) GO TO 5
      IVAR=ABS(JAG(KVAR))
      IF (KBF.GT.0) THEN
      IF (IX(IVAR).LE.-5) THEN
      KVAR=KVAR+1
      GO TO 2
      ENDIF
      ENDIF
C
C     STEP SELECTION
C
      IF (NORMF.GT.0) THEN
      XSTEP=XN(IVAR)
      ELSE
      XSTEP=ONE
      ENDIF
      XTEMP=X(IVAR)
      IF (XTEMP.GE.ZERO) THEN
      XSTEP= ETA*MAX(ABS(XTEMP),XSTEP)
      ELSE
      XSTEP=-ETA*MAX(ABS(XTEMP),XSTEP)
      ENDIF
      X(IVAR)=XTEMP+XSTEP
      XSTEP=X(IVAR)-XTEMP
      CALL UYSET0(1,2)
      RETURN
    3 CONTINUE
C
C     NUMERICAL DIFFERENTIATION
C
      DO 4  LVAR=MVAR+1,IAG(KA+1)-1
      JVAR=ABS(JAG(LVAR))
      IF (KBF.GT.0) THEN
      IF (IX(JVAR).LE.-5) GO TO 4
      ENDIF
      IJ=IND(KVAR-MVAR,LVAR-MVAR)+INDEX
      IF (IJ.GT.M) THEN
      ITERM=-44
      TDXX='LACK SPC'
      CALL UOERR1('UA1HS3',44)
      GO TO 6
      ENDIF
      IF (LVAR .GE. KVAR)  THEN
      HA(IJ)=(GA(JVAR)-GO(JVAR))/XSTEP
      ELSE
      HA(IJ)=HALF*(HA(IJ)+(GA(JVAR)-GO(JVAR))/XSTEP)
      ENDIF
    4 CONTINUE
      X(IVAR)=XTEMP
      KVAR=KVAR+1
      GO TO 2
    5 FA=FTEMP
      CALL UXVCOP(NF,GO,GA)
    6 CALL UYEPI1(-1)
      RETURN
      END
