* SUBROUTINE UA1HS4                ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* NUMERICAL COMPUTATION OF THE HESSIAN MATRIX OF THE APPROXIMATED
* FUNCTION USING ITS GRADIENTS.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  NA  NUMBER OF FUNCTIONS.
*  II  NAL  NUMBER OF LINEAR FUNCTIONS.
*  II  MA  NUMBER OF ELEMENTS IN THE FIELD AG.
*  II  M  DIMENSION OF THE ARRAY H.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  RI  XN(NF)  VECTOR OF SCALING FACTORS.
*  RA  XO(NF)  AUXILIARY VECTOR.
*  RO  H(M)  SIMULTANEOUSLY COMPUTED SPARSE HESSIAN MATRICES.
*  RI  AG(MA)  SPARSE JACOBIAN MATRIX.
*  RA  AGO(MA)  AUXILIARY ARRAY.
*  RI  IAG(NA+1)  POSITION OF THE FIRST ROWS ELEMENTS IN THE FIELD AG.
*  RI  JAG(MA)  COLUMN INDICES OF ELEMENTS IN THE FIELD AG.
*  RI  AF(NA)  VECTOR CONTAINING VALUES OF THE APPROXIMATED
*         FUNCTIONS.
*  RO  AFO(NA)  SAVED VECTOR CONTAINING VALUES OF THE APPROXIMATED
*         FUNCTIONS.
*  II  IG(NF) VECTOR DEFINING GROUPS CONTAINING JACOBIAN COLUMNS.
*  II  NVAR  NUMBER OF GROUPS IN NUMERICAL DIFFERENTIATION.
*  RI  ETA1  PRECISION OF THE COMPUTED VALUES.
*
* COMMON DATA :
*  II  KBF  TYPE OF BOUNDS. KBF=0-BOUNDS ARE NOT USED. KBF=1-ONE SIDED
*         BOUNDS. KBF=1-TWO SIDED BOUNDS.
*  II  NORMF  SCALING SPECIFICATION. NORMF=0-NO SCALING IS PERFORMED.
*         NORMF=1-SCALING FACTORS ARE DETERMINED AUTOMATICALLY.
*         NORMF=2-SCALING FACTORS ARE SUPPLIED BY USER.
*
* STATUS VARIABLES :
*  NS,ISP,TSS
*
* SUBPROGRAMS USED :
*  S   UXVCOP  COPYING OF A VECTOR.
*  S   UYPRO1  PROLOGUE.
*  S   UYEPI1  EPILOGUE.
*  S   UYSET0  STATUS DEFINITION.
*
      SUBROUTINE UA1HS4(NF,NA,NAL,MA,M,X,XN,XO,H,AG,AGO,IAG,JAG,AF,
     & AFO,IG,NVAR,ETA1)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER NF,NA,NAL,MA,M,IAG(NA+1),JAG(MA),IG(NF),NVAR
      REAL*8 X(NF),XN(NF),XO(NF),H(M),AG(MA),AGO(MA),AF(NA),AFO(NA),ETA1
      REAL*8 XSTEP,ETA
      INTEGER I,J,K,L,II,IJ,JJ,KA,NB,IND
      INTEGER IVAR
      SAVE ETA
      SAVE IVAR
      IND(I,J)=MAX(I,J)*(MAX(I,J)-1)/2+MIN(I,J)
      IF (TSS(NS).NE.'UAHX') CALL UYPRO1('UAHX',-1)
      GO TO (1,3) ISP(NS)
    1 CONTINUE
      ETA=SQRT(ETA1)
      CALL UXVCOP(NF,X,XO)
      CALL UXVCOP(NA,AF,AFO)
      CALL UXVCOP(MA,AG,AGO)
      IVAR=1
    2 IF (IVAR.GT.NVAR) GO TO 5
C
C     STEP SELECTION
C
      DO 22 I=1,NF
      X(I)=XO(I)
      IF (IG(I).EQ.IVAR) THEN
      IF (NORMF.GT.0) THEN
      XSTEP=XN(I)
      ELSE
      XSTEP=ONE
      ENDIF
      XSTEP=ETA*MAX(ABS(X(I)),XSTEP)*SIGN(ONE,X(I))
      X(I)=X(I)+XSTEP
      ENDIF
   22 CONTINUE
      CALL UYSET0(1,2)
      RETURN
    3 CONTINUE
C
C     NUMERICAL DIFFERENTIATION
C
      NB=0
      DO 77 KA=1,NAL
      K=IAG(KA)
      L=IAG(KA+1)-K
      NB=NB+L*(L+1)/2
   77 CONTINUE
      DO 8 KA=NAL+1,NA
      K=IAG(KA)
      L=IAG(KA+1)-K
      DO 33 II=1,L
      I=ABS(JAG(K))
      IF (IG(I).EQ.IVAR) GO TO 4
      K=K+1
   33 CONTINUE
      GO TO 7
    4 K=IAG(KA)
      DO 44 JJ=1,L
      J=ABS(JAG(K))
      IJ=IND(II,JJ)
      IF(IG(J).GE.IG(I)) THEN
      H(NB+IJ)=(AG(K)-AGO(K))/(X(I)-XO(I))
      ELSE
      H(NB+IJ)=HALF*(H(NB+IJ)+(AG(K)-AGO(K))/(X(I)-XO(I)))
      ENDIF
      K=K+1
   44 CONTINUE
    7 NB=NB+L*(L+1)/2
    8 CONTINUE
      IVAR=IVAR+1
      GO TO 2
    5 CALL UXVCOP(NF,XO,X)
      CALL UXVCOP(NA,AFO,AF)
      CALL UXVCOP(MA,AGO,AG)
      CALL UYEPI1(-1)
      RETURN
      END
