* SUBROUTINE UA2DQ1             ALL SYSTEMS                 98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* COMPUTATION OF THE VALUE AND THE GRADIENT OF THE OBJECTIVE FUNCTION
* WHICH IS DEFINED AS A SUM OF SQUARES.
*
* PARAMETERS:
*  II  NF  NUMBER OF VARIABLES.
*  II  NA  NUMBER OF MESH POINTS.
*  II  NE  DECLARED NUMBER OF DIFFERENTIAL EQUATIONS.
*  II  MEL  ACTUAL NUMBER OF DIFFERENTIAL EQUATIONS.
*  IU  KE  INDEX OF THE DIFFERENTIAL EQUATION.
*  RI  AT(NA)  MESH POINTS.
*  RO  AY(NA*NE)  FIELD OF SOLUTION VECTORS IN THE MESH POINTS.
*  RU  YA(MEL)  SOLUTION VECTOR.
*  RI  EY(NE)  REQUIRED TERMINAL SOLUTION VECTOR.
*  RI  EW(NE)  VECTOR WITH WEIGHT COEFFICIENTS.
*  RO  G(NF)  GRADIENT OF THE OBJECTIVE FUNCTION.
*  RI  GE(NF)  GRADIENT OF THE RIGHT HAND SIDE FUNCTION.
*  RO  H(NF*(NF+1)/2)  APPROXIMATION OF THE HESSIAN MATRIX.
*  RI  ETA9  MAXIMUM VALUE LESS THEN INFINITY.
*  RO  F  VALUE OF THE OBJECTIVE FUNCTION.
*  RI  FE  VALUE OF THE RIGHT HAND SIDE FUNCTION.
*  RU  TA  VALUE OF THE INTEGRATION PARAMETER.
*  RU  HE  ACTUAL STEPSIZE.
*  II  MED  NUMBER OF STEPS PRECEEDING TEST OF STIFTNESS.
*  II  MED1  CHOICE OF STEP SELECTION.
*  II  MED2  NUMBER OF RESETING SOLUTIONS.
*  IO  IER  ERROR INDICATOR.
*
* COMMON DATA :
*  II  KD  DEGREE OF REQUIRED DERVATIVES.
*  II  LD  DEGREE OF PREVIOUSLY COMPUTED DERIVATIVES.
*  II  KDF  DEGREE OF ANALYTICALLY COMPUTED DERVATIVES.
*  II  KDA  DEGREE OF ANALYTICALLY COMPUTED DERVATIVES.
*
* STATUS VARIABLES :
*  NS,ISP,TSS
*
* SUBPROGRAMS USED :
*  S  UYPRO1  PROLOGUE.
*  S  UYEPI1  EPILOGUE.
*  S  UYSET0  STATUS DEFINITION.
*  S  UXDSMU  RANK ONE UPDATE OF THE SYMMETRIC MATRIX.
*  S  UXVCOP  COPY OF A VECTOR.
*  S  UXVDIR  VECTOR AUGMENTED BY THE SCALED VECTOR.
*  S  UXVSET  INITIATION OF A VECTOR.
*  S  UXVSUM  SUM OF TWO VECTORS.
*
      SUBROUTINE UA2DQ1(NF,NA,NE,MEL,KE,AT,AY,YA,EY,EW,G,GE,H,ETA9,F,
     & FE,TA,HE,MED,MED1,MED2,IER)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER NF,NA,NE,MEL,KE,MED,MED1,MED2,IER
      REAL*8 AT(NA),AY(NA*NE),YA(*),EY(NE),EW(NE),G(NF),
     & GE(NF),H(NF*(NF+1)/2),ETA9,F,FE,TA,HE
      INTEGER N2,N3,MEDS,L,M
      SAVE N2,N3,MEDS,M
      IF (TSS(NS).NE.'UAXX') CALL UYPRO1('UAXX',-1)
      GO TO (1,3,6) ISP(NS)
    1 CONTINUE
      MEDS=MED
      M=NF*(NF+1)/2
      IF (KD.LT.0) GO TO 10
      IF (KD.LE.LD) GO TO 10
      IF (KDY.LT.0) GO TO 5
      N2=NE+1
      KE=1
    2 IF (KE.GT.NE) GO TO 4
      CALL UYSET0(1,2)
      RETURN
    3 CONTINUE
      YA(KE)=FE
      IF (KD.GT.0.AND.KDY.GT.0) THEN
      CALL UXVCOP(NF,GE,YA(N2))
      N2=N2+NF
      ENDIF
      KE=KE+1
      GO TO 2
    4 CALL UXVCOP(NE,YA,AY)
    5 IF (KD.LE.0) THEN
      N2=NE
      IF (KDA.LT.0) THEN
      MEL=NE
      ELSE
      MEL=NE+1
      YA(MEL)=ZERO
      ENDIF
      ELSE
      N2=NE*(NF+1)
      IF (KDY.LE.0) CALL UXVSET(NE*NF,ZERO,YA(NE+1))
      IF (KDA.LT.0) THEN
      MEL=N2
      ELSE IF (KD.LE.1) THEN
      MEL=N2+NF+1
      CALL UXVSET(NF+1,ZERO,YA(N2+1))
      ELSE
      N3=N2+NF+1
      MEL=N3+M
      CALL UXVSET(M+NF+1,ZERO,YA(N2+1))
      ENDIF
      ENDIF
      MED=0
      MED1=1
      MED2=0
      TA=AT(1)
      IF (KDY.LT.0) CALL UXVCOP(NE,AY,YA)
      HE=ZERO
      CALL UYSET0(2,3)
      RETURN
    6 CONTINUE
      IF (IER.LT.0) THEN
      F=ETA9
      ITERM=MAX(-96,-90+IER)
      CALL UOERR1('UA2DQ1',-ITERM)
      TDXX=TEDX
      GO TO 10
      ENDIF
      IF (ITERM.LT.0) ITERM=0
      IF (KDA.LT.0) THEN
      F=ZERO
      ELSE
      F=YA(N2+1)
      ENDIF
      IF (KDF.GE.0) THEN
      DO 7 KE=1,NE
      F=F+HALF*EW(KE)*(YA(KE)-EY(KE))**2
    7 CONTINUE
      ENDIF
      IF (KD.LE.0) GO TO 10
      CALL UXVSET(NF,ZERO,G)
      IF (KDF.GE.0) THEN
      L=NE+1
      DO 8 KE=1,NE
      CALL UXVDIR(NF,EW(KE)*(YA(KE)-EY(KE)),YA(L),G,G)
      L=L+NF
    8 CONTINUE
      ENDIF
      IF (KDA.GE.0) CALL UXVSUM(NF,G,YA(N2+2),G)
      IF (KD.LE.1) GO TO 10
      CALL UXVSET(M,ZERO,H)
      IF (KDF.GE.0) THEN
      L=NE+1
      DO 9 KE=1,NE
      CALL UXDSMU(NF,H,EW(KE),YA(L))
      L=L+NF
    9 CONTINUE
      ENDIF
      IF (KDA.GE.0) CALL UXVSUM(M,H,YA(N3+1),H)
      IDECF=0
   10 CALL UYEPI1(-1)
      MED=MEDS
      LD=KD
      RETURN
      END
