* SUBROUTINE UA2SP3             ALL SYSTEMS                97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
*  COMPUTATION OF THE VALUE AND THE GRADIENT AND THE HESSIAN MATRIX
*  OF THE OBJECTIVE FUNCTION WHICH IS DEFINED AS A SUM OF POWERS.
*
* PARAMETERS:
*  II  NF  NUMBER OF VARIABLES.
*  II  NA  NUMBER OF APPROXIMATED FUNCTIONS.
*  II  NAL  NUMBER OF LINEAR APPROXIMATED FUNCTIONS.
*  II  MA  NUMBER OF ELEMENTS IN THE FIELD AG.
*  II  ME  NUMBER OF ELEMENTS OF THE MATRIX HA.
*  II  MB NUMBER OF NONZERO ELEMENTS OF THE PARTITIONED HESSIAN
*         MATRIX H.
*  II  KA  INDEX OF THE APPROXIMATED FUNCTION.
*  RI  GA(NF)  GRADIENT OF THE APPROXIMATED FUNCTION.
*  RO  G(NF)  GRADIENT OF THE OBJECTIVE FUNCTION.
*  RI  HA(ME)  HESSIAN MATRIX OF THE APPROXIMATED FUNCTION.
*  RO  AH(MB)  PARTITIONED HESSIAN MATRIX OF THE OBJECTIVE FUNCTION.
*  RI  IAG(NA+1)  POSITION OF THE FIRST ROWS ELEMENTS IN THE FIELD AG.
*  RI  JAG(MA)  COLUMN INDICES OF ELEMENTS IN THE FIELD AG.
*  RI  FA  VALUE OF THE SELECTED FUNCTION.
*  RO  F  VALUE OF THE OBJECTIVE FUNCTION.
*  RI  REXP  EXPONENT IN A SUM OF POWERS.
*
* COMMON DATA :
*  II  KDA  DEGREE OF ANALYTICALLY COMPUTED DERIVATIVES.
*  II  KD  DEGREE OF REQUIRED DERVATIVES.
*  IO  LD  DEGREE OF PREVIOUSLY COMPUTED DERIVATIVES.
*  IU  NAV  NUMBER OF APPROXIMATED FUNCTION VALUES COMPUTED.
*  IU  NAG  NUMBER OF APPROXIMATED FUNCTION GRADIENTS COMPUTED.
*  IU  NAH  NUMBER OF APPROXIMATED FUNCTION HESSIAN MATRICES COMPUTED.
*  IU  NFV  NUMBER OF OBJECTIVE FUNCTION VALUES COMPUTED.
*  IU  NFG  NUMBER OF OBJECTIVE FUNCTION GRADIENTS COMPUTED.
*  IU  NFH  NUMBER OF OBJECTIVE FUNCTION HESSIAN MATRICES COMPUTED.
*  IU  IDECF  DECOMPOSITION INDICATOR. IDECF=0-NO DECOMPOSITION.
*
* STATUS VARIABLES :
*  NS,ISP,TSS
*
* SUBPROGRAMS USED :
*  S   UYPRO1  PROLOGUE.
*  S   UYEPI1  EPILOGUE.
*  S   UYSET0  STATUS DEFINITION.
*  S   UXVSET  INITIATION OF A VECTOR.
*  S   UXDSMC  COPYING A DENSE SYMMETRIC MATRIX.
*  S   UXDSMO  INITIATION OF A DENSE SYMMETRIC MATRIX.
*  S   UXDSMS  SCALING A DENSE SYMMETRIC MATRIX.
*  S   UXBSBU  CORRECTION OF A BLOCKED SYMMETRIC MATRIX.
*
      SUBROUTINE UA2SP3(NF,NA,NAL,MA,ME,MB,KA,GA,G,HA,AH,IAG,JAG,FA,F,
     & REXP)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER NF,NA,NAL,MA,ME,MB,KA,IAG(NA+1),JAG(MA)
      REAL*8 GA(NF),G(NF),HA(ME),AH(MB),FA,F,REXP
      REAL*8 FTEMP
      INTEGER J,JP,K,L,NB
      SAVE FTEMP
      SAVE NB
      IF (TSS(NS).NE.'UAXX') CALL UYPRO1('UAXX',-1)
      GO TO (1,4,6,8) ISP(NS)
    1 CONTINUE
      IF (REXP.LE.ONE) REXP=TWO
      IF (KD.LE.LD) GO TO 12
      IF (KD.GE.0.AND.LD.LT.0) F=ZERO
      IF (KD.GE.1.AND.LD.LT.1) CALL UXVSET(NF,ZERO,G)
      NB=0
      KA=1
    2 IF (KA.GT.NA) GO TO 11
      IF (KD.LT.0) GO TO 9
      CALL UYSET0(1,2)
      RETURN
    4 CONTINUE
      FTEMP=ABS(FA)**REXP
      IF (LD.GE.0) GO TO 5
      F=F+FTEMP
    5 IF (KD.LT.1) GO TO 9
      CALL UYSET0(2,3)
      RETURN
    6 CONTINUE
      IF (FA.EQ.ZERO) THEN
      FTEMP=ZERO
      ELSE
      FTEMP=FTEMP/FA
      ENDIF
      IF (LD.GE.1) GO TO 7
      K=IAG(KA)
      L=IAG(KA+1)-K
      DO 3 J=1,L
      JP=ABS(JAG(K))
      G(JP)=G(JP)+FTEMP*GA(JP)
      K=K+1
    3 CONTINUE
    7 IF (KD.LT.2) GO TO 9
      CALL UYSET0(3,4)
      RETURN
    8 CONTINUE
      IDECF=0
      K=IAG(KA)
      L=IAG(KA+1)-K
      IF (KDA.GE.2) THEN
      IF (LDA.LE.1) THEN
      CALL UXDSMC(L,HA,AH(NB+1))
      ENDIF
      CALL UXDSMS(L,AH(NB+1),FTEMP)
      ELSE
      CALL UXDSMO(L,ZERO,AH(NB+1))
      ENDIF
      IF (FA.NE.ZERO) THEN
      FTEMP=(REXP-ONE)*FTEMP/FA
      CALL UXBSBU(L,NF,AH(NB+1),JAG(K),FTEMP,GA,1)
      ENDIF
      NB=NB+L*(L+1)/2
    9 KA=KA+1
      GO TO 2
   11 IF (KD.GE.0.AND.LD.LT.0) F=F/REXP
      NFV=NAV/(NA-NAL)
      NFG=NAG/(NA-NAL)
      NFH=NAH/(NA-NAL)
      LD=KD
   12 CALL UYEPI1(-1)
      RETURN
      END
