* SUBROUTINE UCCNS2             ALL SYSTEMS                   08/12/01
C PORTABILITY : ALL SYSTEMS
C 08/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* COMPUTATION OF THE SPARSE SCHUR COMPLEMENT STRUCTURE FOR THE
* KARUSH-KUHN-TUCKER MATRIX WITH THE UNIT HESSIAN BLOCK.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  NX  NUMBER OF BOX CONSTRAINTS.
*  II  NC  NUMBER OF GENERAL CONSTRAINTS.
*  II  MC  NUMBER OF NONZERO ELEMENTS IN THE FIELD CG.
*  II  KC  INDEX OF A GENERAL CONSTRAINT.
*  II  NCC  NUMBER OF COMPLEMENTARITY CONSTRAINTS.
*  II  ICC(2*NCC)  VECTOR CONTAINING INDICES OF COMPLEMENTRIRY
*         CONSTRAINTS.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NX)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XL(NX)  VECTOR CONTAINING LOWER BOUNDS FOR VARIABLES.
*  RI  XU(NX)  VECTOR CONTAINING UPPER BOUNDS FOR VARIABLES.
*  RI  CF(NC)  VECTOR CONTAINING VALUES OF THE CONSTRAINT FUNCTIONS.
*  II  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  RI  CL(NC)  VECTOR CONTAINING LOWER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CU(NC)  VECTOR CONTAINING UPPER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  II  CG(MC)  ELEMENTS OF THE CONSTRAINT JACOBIAN MATRIX.
*  II  ICG(NC+1)  POSITION OF THE FIRST ROWS ELEMENTS IN THE FIELD CG.
*  II  JCG(MC)  COLUMN INDICES OF ELEMENTS IN THE FIELD CG.
*  II  IDER  ORDER OF COMPUTED DERIVATIVES.
*  II  JOB  TYPE OF THE JACOBIAN MATRIX. JOB=0-STANDARD JACOBIAN
*         MATRIX. JOB=1-JACOBIAN MATRIX WITH THE MINIMAX ROW.
*
      SUBROUTINE UCCNS2(NF,NX,NC,MC,KC,NCC,ICC,X,IX,XL,XU,FC,CF,IC,
     & CL,CU,GC,CG,ICG,JCG,IDER,JOB)
      INTEGER NF,NX,NC,MC,KC,NCC,ICC(2*NCC),IX(NX),IC(NC+NCC),
     & ICG(NC+NCC+1),JCG(MC),IDER,JOB
      REAL*8 X(NX),XL(NX),XU(NX),FC,CF(NC+NCC),CL(NC),CU(NC+NCC),
     & GC(NF+JOB),CG(MC)
      INTEGER I,II,ISTRT,ISTOP,J,K,KK,KSTRT,KSTOP,L,LL,LSTRT,LSTOP
      REAL*8 FK,FL,POM,SIG
      IF (KC.LE.NC) RETURN
      I=KC-NC
      SIG=1.0D 0
      ISTRT=ICG(KC)
      ISTOP=ICG(KC+1)-1
      IF (JOB.GT.0) ISTOP=ISTOP-1
      K=ICC(I)
      L=ICC(NCC+I)
      IF (K.GT.0.AND.L.LT.0) THEN
      KK=K
      K=L
      L=KK
      ENDIF
      IF (K.LT.0) THEN
      KK=ABS(IX(-K))
      IF (KK.EQ.1) THEN
      FK=X(-K)-XL(-K)
      SIG=-SIG
      ELSE IF (KK.EQ.2) THEN
      FK=X(-K)-XU(-K)
      ENDIF
      ELSE
      KK=ABS(IC(K))
      IF (KK.EQ.1) THEN
      FK=CF(K)-CL(K)
      SIG=-SIG
      ELSE IF (KK.EQ.2) THEN
      FK=CF(K)-CU(K)
      ENDIF
      ENDIF
      IF (L.LT.0) THEN
      LL=ABS(IX(-L))
      IF (LL.EQ.1) THEN
      FL=X(-L)-XL(-L)
      SIG=-SIG
      ELSE IF (LL.EQ.2) THEN
      FL=X(-L)-XU(-L)
      ENDIF
      ELSE
      LL=ABS(IC(L))
      IF (LL.EQ.1) THEN
      FL=CF(L)-CL(L)
      SIG=-SIG
      ELSE IF (LL.EQ.2) THEN
      FL=CF(L)-CU(L)
      ENDIF
      ENDIF
      FC=SIG*FK*FL
      IF (IDER.EQ.0) RETURN
      II=ISTRT
      POM=0.0D 0
      IF (K.LT.0) THEN
      K=-K
      IF (L.LT.0) THEN
      L=-L
      GC(K)=SIG*FL
      GC(L)=SIG*FK
      ELSE
      LSTRT=ICG(L)
      LSTOP=ICG(L+1)-1
      LL=LSTRT
    1 CONTINUE
      IF (II.LE.ISTOP) THEN
      J=JCG(II)
      IF (LL.LE.LSTOP) THEN
      IF (J.EQ.JCG(LL)) THEN
      POM=POM+FK*CG(LL)
      LL=LL+1
      ENDIF
      ENDIF
      IF (J.EQ.K) POM=POM+FL
      GC(J)=SIG*POM
      II=II+1
      GO TO 1
      ENDIF
      ENDIF
      ELSE
      KSTRT=ICG(K)
      KSTOP=ICG(K+1)-1
      LSTRT=ICG(L)
      LSTOP=ICG(L+1)-1
      KK=KSTRT
      LL=LSTRT
    2 CONTINUE
      IF (II.LE.ISTOP) THEN
      J=JCG(II)
      IF (LL.LE.LSTOP) THEN
      IF (J.EQ.JCG(LL)) THEN
      POM=POM+FK*CG(LL)
      LL=LL+1
      ENDIF
      ENDIF
      IF (KK.LE.KSTOP) THEN
      IF (J.EQ.JCG(KK)) THEN
      POM=POM+FL*CG(KK)
      KK=KK+1
      ENDIF
      ENDIF
      GC(J)=SIG*POM
      II=II+1
      GO TO 2
      ENDIF
      ENDIF
      IF (JOB.GT.0) THEN
      CG(NF+1)=-1.0D 0
      ENDIF
      RETURN
      END
