* SUBROUTINE UCIMOD             ALL SYSTEMS                 03/12/01
C PORTABILITY : ALL SYSTEMS
C 03/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* DETERMINATION OF THE DIRECTIONAL DERIVATIVE.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  NX  NUMBER OF BOX CONSTRAINTS.
*  II  NC  NUMBER OF GENERAL CONSTRAINTS.
*  II  NCA  NUMBER OF ACTIVE CONSTRAINTS.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XL(NF)  VECTOR CONTAINING LOWER BOUNDS FOR VARIABLES.
*  RI  XU(NF)  VECTOR CONTAINING UPPER BOUNDS FOR VARIABLES.
*  RI  XSL(NX)  VECTOR CONTAINING LOVER SLACKS FOR VARIABLES.
*  RI  XSU(NX)  VECTOR CONTAINING UPPER SLACKS FOR VARIABLES.
*  RI  XZL(NF)  VECTOR CONTAINING LOVER MULTIPLIERS FOR VARIABLES.
*  RI  XZU(NF)  VECTOR CONTAINING UPPER MULTIPLIERS FOR VARIABLES.
*  RI  CF(NC+1)  VECTOR CONTAINING VALUES OF THE CONSTRAINT FUNCTIONS.
*  RI  CFD(NC)  VECTOR CONTAINING INCREMENTS OF CONSTRAINT FUNCTIONS.
*  II  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  RI  CL(NC)  VECTOR CONTAINING LOWER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CU(NC)  VECTOR CONTAINING UPPER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CSL(NC)  VECTOR CONTAINING LOVER SLACKS FOR CONSTRAINTS.
*  RI  CSU(NC)  VECTOR CONTAINING UPPER SLACKS FOR CONSTRAINTS.
*  RI  CZL(NC)  VECTOR CONTAINING LOVER MULTIPLIERS FOR CONSTRAINTS.
*  RI  CZU(NC)  VECTOR CONTAINING UPPER MULTIPLIERS FOR CONSTRAINTS.
*  II  ICA(NCA)  VECTOR CONTAINING INDICES OF ACTIVE CONSTRAINTS.
*  II  S(NF+NCA)  VECTOR CONTAINING NEWTON INCREMENTS.
*  RI  GS(NCA)  VECTOR CONTAINING ACTIVE SLACK INCREMENTS.
*  RI  XO(NX+NC)  VECTOR CONTAINING INCREMENTS FOR LOWER INACTIVE
*         MULTIPLIERS.
*  RI  GO(NX+NC)  VECTOR CONTAINING INCREMENTS FOR UPPER INACTIVE
*         MULTIPLIERS.
*  RI  RAT  RATIO OF TRUST REGION RADIUS.
*  RI  RPF1  OLD PENALTY COEFFICIENT.
*  RI  RPF2  NEW PENALTY COEFFICIENT.
*  RI  RPF3  BARRIER COEFFICIENT.
*  RU  P  STEPSIZE  PARAMETER.
*  RU  PP  SECOND ORDER TERM.
*  II  IPOM  CHOICE OF THE PENALTY COEFFICIENT. IPOM=1-OLD PENALTY
*         COEFFICIENT. IPOM=2-NEW PENALTY COEFFICIENT DETERMINED
*         FROM THE QUADRATIC MODEL.
*  II  JOB  CHOICE OF THE IP FORMULATION. JOB=1-PRIMAL FORMULATION.
*         JOB=2-PRIMAL-DUAL FORMULATION.
*
*  COMMON DATA :
*  II  KBF  SPECIFICATION OF SIMPLE BOUNDS. KBF=0-NO SIMPLE BOUNDS.
*         KBF=1-ONE SIDED SIMPLE BOUNDS. KBF=2=TWO SIDED SIMPLE BOUNDS.
*  II  KBC  SPECIFICATION OF LINEAR CONSTRAINTS. KBC=0-NO LINEAR
*         CONSTRAINTS. KBC=1-ONE SIDED LINEAR CONSTRAINTS. KBC=2=TWO
*         SIDED LINEAR CONSTRAINTS.
*
      SUBROUTINE UCIMOD(NF,NX,NC,NCA,X,IX,XL,XU,XSL,XSU,XZL,XZU,CF,CFD,
     & IC,CL,CU,CSL,CSU,CZL,CZU,ICA,S,GS,XO,GO,RAT,RPF1,RPF2,RPF3,P,PP,
     & IPOM,JOB)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER NF,NX,NC,NCA,IX(NX),IC(NC),ICA(NCA),IPOM,JOB
      REAL*8 X(NF),XL(NX),XU(NX),XSL(NX),XSU(NX),XZL(NX),XZU(NX),
     & CF(NC),CFD(NC),CL(NC),CU(NC),CSL(NC),CSU(NC),CZL(NC),CZU(NC),
     & S(NF+NCA),GS(NCA),XO(NX+NC),GO(NX+NC),RAT,RPF1,RPF2,RPF3,P,PP
      INTEGER I,K,KK
      REAL*8 TEMP,TEMP1,TEMP2,POM,CON
      PARAMETER (CON=0.9D 0)
      POM=ZERO
      DO 1 I=1,NCA
      TEMP=GS(I)
      K=ICA(I)
      IF (K.LT.0) THEN
      K=-K
      KK=-IX(K)
      IF (KK.EQ.1.OR.KK.EQ.3) THEN
      TEMP1=S(NF+I)
      POM=POM+(S(K)-TEMP)*(XL(K)-X(K)+XSL(K))
      P=P-RPF3*TEMP/XSL(K)+TEMP*(XZL(K)+TEMP1)-S(K)*TEMP1
      IF (JOB.EQ.1) THEN
      PP=PP+RPF3*(TEMP/XSL(K))**2
      ELSE
      PP=PP+(XZL(K)/XSL(K))*TEMP**2
      ENDIF
      ELSE IF (KK.EQ.2.OR.KK.EQ.4) THEN
      TEMP1=S(NF+I)
      POM=POM-(S(K)+TEMP)*(X(K)-XU(K)+XSU(K))
      P=P-RPF3*TEMP/XSU(K)+TEMP*(XZU(K)+TEMP1)+S(K)*TEMP1
      IF (JOB.EQ.1) THEN
      PP=PP+RPF3*(TEMP/XSU(K))**2
      ELSE
      PP=PP+(XZU(K)/XSU(K))*TEMP**2
      ENDIF
      ENDIF
      ELSE
      KK=-IC(K)
      IF (KK.EQ.1.OR.KK.EQ.3) THEN
      TEMP1=S(NF+I)
      POM=POM+(CFD(K)-TEMP)*(CL(K)-CF(K)+CSL(K))
      P=P-RPF3*TEMP/CSL(K)+TEMP*(CZL(K)+TEMP1)-CFD(K)*TEMP1
      IF (JOB.EQ.1) THEN
      PP=PP+RPF3*(TEMP/CSL(K))**2
      ELSE
      PP=PP+(CZL(K)/CSL(K))*TEMP**2
      ENDIF
      ELSE IF (KK.EQ.2.OR.KK.EQ.4) THEN
      TEMP1=S(NF+I)
      POM=POM-(CFD(K)+TEMP)*(CF(K)-CU(K)+CSU(K))
      P=P-RPF3*TEMP/CSU(K)+TEMP*(CZU(K)+TEMP1)+CFD(K)*TEMP1
      IF (JOB.EQ.1) THEN
      PP=PP+RPF3*(TEMP/CSU(K))**2
      ELSE
      PP=PP+(CZU(K)/CSU(K))*TEMP**2
      ENDIF
      ELSE IF (KK.EQ.5) THEN
      POM=POM-CFD(K)*CF(K)
      P=P+CFD(K)*S(NF+I)
      ENDIF
      ENDIF
    1 CONTINUE
      IF (KBF.GT.0) THEN
      DO 2 K=1,NX
      KK=IX(K)
      IF (KK.EQ.1.OR.KK.EQ.3.OR.ABS(KK).EQ.4) THEN
      TEMP1=XO(K)
      TEMP2=XL(K)-X(K)+XSL(K)
      TEMP=-RAT*(TEMP2-S(K))
      POM=POM+TEMP2**2
C      POM=POM+(S(K)-TEMP)*TEMP2
      P=P-RPF3*TEMP/XSL(K)+TEMP*(XZL(K)+TEMP1)-S(K)*TEMP1
      IF (JOB.EQ.1) THEN
      PP=PP+RPF3*(TEMP**2-S(K)**2)/XSL(K)**2
      ELSE
      PP=PP+(XZL(K)/XSL(K))*(TEMP**2-S(K)**2)
      ENDIF
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.4.OR.ABS(KK).EQ.3) THEN
      TEMP1=GO(K)
      TEMP2=X(K)-XU(K)+XSU(K)
      TEMP=-RAT*(TEMP2+S(K))
      POM=POM+TEMP2**2
C      POM=POM-(S(K)+TEMP)*TEMP2
      P=P-RPF3*TEMP/XSU(K)+TEMP*(XZU(K)+TEMP1)+S(K)*TEMP1
      IF (JOB.EQ.1) THEN
      PP=PP+RPF3*(TEMP**2-S(K)**2)/XSU(K)**2
      ELSE
      PP=PP+(XZU(K)/XSU(K))*(TEMP**2-S(K)**2)
      ENDIF
      ENDIF
    2 CONTINUE
      ENDIF
      IF (KBC.GT.0) THEN
      DO 3 K=1,NC
      KK=IC(K)
      IF (KK.EQ.1.OR.KK.EQ.3.OR.ABS(KK).EQ.4) THEN
      TEMP1=XO(NX+K)
      TEMP2=CL(K)-CF(K)+CSL(K)
      TEMP=-RAT*(TEMP2-CFD(K))
      POM=POM+TEMP2**2
C      POM=POM+(CFD(K)-TEMP)*TEMP2
      P=P-RPF3*TEMP/CSL(K)+TEMP*(CZL(K)+TEMP1)-CFD(K)*TEMP1
      IF (JOB.EQ.1) THEN
      PP=PP+RPF3*(TEMP**2-CFD(K)**2)/CSL(K)**2
      ELSE
      PP=PP+(CZL(K)/CSL(K))*(TEMP**2-CFD(K)**2)
      ENDIF
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.4.OR.ABS(KK).EQ.3) THEN
      TEMP1=GO(NX+K)
      TEMP2=CF(K)-CU(K)+CSU(K)
      TEMP=-RAT*(TEMP2+CFD(K))
      POM=POM+TEMP2**2
C      POM=POM-(CFD(K)+TEMP)*TEMP2
      P=P-RPF3*TEMP/CSU(K)+TEMP*(CZU(K)+TEMP1)+CFD(K)*TEMP1
      IF (JOB.EQ.1) THEN
      PP=PP+RPF3*(TEMP**2-CFD(K)**2)/CSU(K)**2
      ELSE
      PP=PP+(CZU(K)/CSU(K))*(TEMP**2-CFD(K)**2)
      ENDIF
      ENDIF
    3 CONTINUE
      ENDIF
      PP=HALF*PP
      IF (IPOM.LE.1) THEN
      RPF2=RPF1
      PP=MIN(PP,CON*ABS(P))
      ELSE
      RPF2=MAX(RPF1,1.1D 0*(P+PP)/POM)
      ENDIF
      P=P-RPF2*POM
      RETURN
      END
