* SUBROUTINE UCINS1             ALL SYSTEMS                   99/12/01
C PORTABILITY : ALL SYSTEMS
C 99/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* COMPUTATION OF THE SPARSE SCHUR COMPLEMENT FOR THE KARUSH-KUHN-TUCKER
* MATRIX WITH THE DIAGONAL HESSIAN BLOCK.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  NCA  NUMBER OF ACTIVE CONSTRAINTS.
*  II  NC  NUMBER OF CONSTRAINT FUNCTIONS.
*  II  MC  NUMBER OF NONZERO ELEMENTS IN THE FIELD CG.
*  II  M  NUMBER OF NONZERO ELEMENTS OF THE SPARSE SCHUR COMPLEMENT.
*  RO  B(M)  ELEMENTS OF THE SPARSE MATRIX B.
*  II  IB(NC+1)  POINTERS OF THE DIAGONAL ELEMENTS OF B.
*  II  JB(M)  INDICES OF THE NONZERO ELEMENTS OF B.
*  II  ICA(NCA)  VECTOR CONTAINING INDICES OF ACTIVE CONSTRAINTS.
*  II  CG(MC)  ELEMENTS OF THE CONSTRAINT JACOBIAN MATRIX.
*  II  ICG(NC+1)  POSITION OF THE FIRST ROWS ELEMENTS IN THE FIELD CG.
*  II  JCG(MC)  COLUMN INDICES OF ELEMENTS IN THE FIELD CG.
*  RI  D(NF+NCA)  DIAGONAL WEIGHTING MATRIX.
*
      SUBROUTINE UCINS1(NF,NCA,NC,MC,M,B,IB,JB,ICA,CG,ICG,JCG,
     & D)
      INTEGER NF,NCA,NC,MC,M,IB(NCA+1),JB(M),ICA(NCA),
     & ICG(NC+1),JCG(MC)
      REAL*8 B(M),CG(MC),D(NF+NCA)
      INTEGER I,II,I1,I2,IK,IL,K,KK,K1,K2,L,LL,L1,L2
      DO 5 I=1,NCA
      I1=IB(I)
      I2=IB(I+1)-1
      K=ICA(I)
      B(I1)=-D(NF+I)
      DO 4 II=I1,I2
      IF (II.NE.I1) B(II)=0.0D 0
      L=ICA(JB(II))
      IF (K.LT.0) THEN
      KK=-K
      IF (L.LT.0) THEN
      IF (L.EQ.K) B(II)=B(II)+D(KK)
      ELSE
      L1=ICG(L)
      L2=ICG(L+1)-1
      DO 1 LL=L1,L2
      IF (JCG(LL).EQ.KK) THEN
      B(II)=B(II)+D(KK)*CG(LL)
      GO TO 4
      ENDIF
    1 CONTINUE
      ENDIF
      ELSE
      K1=ICG(K)
      K2=ICG(K+1)-1
      IF (L.LT.0) THEN
      LL=-L
      DO 2 KK=K1,K2
      IF (JCG(KK).EQ.LL) THEN
      B(II)=B(II)+D(LL)*CG(KK)
      GO TO 4
      ENDIF
    2 CONTINUE
      ELSE
      KK=K1
      LL=ICG(L)
      L2=ICG(L+1)-1
      IK=JCG(KK)
      IL=JCG(LL)
    3 IF (IK.LE.0.OR.IK.LT.IL) THEN
      IF (KK.GE.K2) GO TO 4
      KK=KK+1
      IK=JCG(KK)
      GO TO 3
      ELSE IF (IL.LE.0.OR.IL.LT.IK) THEN
      IF (LL.GE.L2) GO TO 4
      LL=LL+1
      IL=JCG(LL)
      GO TO 3
      ELSE
      B(II)=B(II)+D(IK)*CG(KK)*CG(LL)
      IF(KK.GE.K2.OR.LL.GE.L2) GO TO 4
      KK=KK+1
      LL=LL+1
      IK=JCG(KK)
      IL=JCG(LL)
      GO TO 3
      ENDIF
      ENDIF
      ENDIF
    4 CONTINUE
    5 CONTINUE
      RETURN
      END
