* SUBROUTINE UCISEC             ALL SYSTEMS                 03/12/01
C PORTABILITY : ALL SYSTEMS
C 03/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* ACTIVE CONSTRAINT PART OF RIGHT HAND SIDE VECTOR.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  NX  NUMBER OF BOX CONSTRAINTS.
*  II  NC  NUMBER OF GENERAL CONSTRAINTS.
*  II  NCA  NUMBER OF ACTIVE CONSTRAINTS.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NX)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XL(NX)  VECTOR CONTAINING LOWER BOUNDS FOR VARIABLES.
*  RI  XU(NX)  VECTOR CONTAINING UPPER BOUNDS FOR VARIABLES.
*  RI  XSL(NX)  VECTOR CONTAINING LOVER SLACKS FOR VARIABLES.
*  RI  XSU(NX)  VECTOR CONTAINING UPPER SLACKS FOR VARIABLES.
*  RI  XZL(NX)  VECTOR CONTAINING LOVER MULTIPLIERS FOR VARIABLES.
*  RI  XZU(NX)  VECTOR CONTAINING UPPER MULTIPLIERS FOR VARIABLES.
*  RI  CF(NC)  VECTOR CONTAINING VALUES OF THE CONSTRAINT FUNCTIONS.
*  II  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  RI  CL(NC)  VECTOR CONTAINING LOWER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CU(NC)  VECTOR CONTAINING UPPER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CSL(NC)  VECTOR CONTAINING LOVER SLACKS FOR CONSTRAINTS.
*  RI  CSU(NC)  VECTOR CONTAINING UPPER SLACKS FOR CONSTRAINTS.
*  RI  CZL(NC)  VECTOR CONTAINING LOVER MULTIPLIERS FOR CONSTRAINTS.
*  RI  CZU(NC)  VECTOR CONTAINING UPPER MULTIPLIERS FOR CONSTRAINTS.
*  IO  ICA(NCA)  VECTOR CONTAINING INDICES OF ACTIVE CONSTRAINTS.
*  RO  CP(NCA)  RIGHT HAND SIDE VECTOR.
*  RO  D(NCA)  DIAGONAL SOUTH-EAST CORNER.
*  RO  STOL  TOLERANCE FOR MODIFIED SLACKS.
*  RI  RPF3  BARRIER COEFFICIENT.
*  RO  SIG1  SQUARED NORM OF THE PRECISION-MEASURE VECTOR.
*  II  MOS4  CHOICE OF THE PRECISION-MEASURE VECTOR. MOS4=1-RIGHT-HAND
*         SIDE VECTOR IS USED. MOS4=2-CONSTRAINT-VIOLATION VECTOR IS
*         USED.
*  II  JOB  CHOICE OF THE IP FORMULATION. JOB=1-PRIMAL FORMULATION.
*         JOB=2-PRIMAL-DUAL FORMULATION.
*
      SUBROUTINE UCISEC(NF,NX,NC,NCA,X,IX,XL,XU,XSL,XSU,XZL,XZU,CF,IC,
     & CL,CU,CSL,CSU,CZL,CZU,ICA,CP,D,STOL,RPF3,SIG1,MOS4,JOB)
      INTEGER NF,NX,NC,NCA,IX(NX),IC(NC),ICA(NCA),MOS4,JOB
      REAL*8 X(NF),XL(NX),XU(NX),XSL(NX),XSU(NX),XZL(NX),XZU(NX),CF(NC),
     & CL(NC),CU(NC),CSL(NC),CSU(NC),CZL(NC),CZU(NC),CP(NCA),D(NCA),
     & STOL,RPF3,SIG1
      INTEGER I,K
      SIG1=0.0D 0
      DO 1 I=1,NCA
      K=ICA(I)
      IF (K.LT.0) THEN
      K=-K
      IF (JOB.EQ.1) THEN
      IF (IX(K).EQ.-1.OR.IX(K).EQ.-3) THEN
      CP(I)=XL(K)-X(K)+(2.0D 0-XSL(K)*XZL(K)/RPF3)*XSL(K)
      D(I)=-XSL(K)**2/RPF3
      IF (MOS4.EQ.2) SIG1=SIG1+(XL(K)-X(K)+XSL(K))**2
      ELSE IF (IX(K).EQ.-2.OR.IX(K).EQ.-4) THEN
      CP(I)=XU(K)-X(K)-(2.0D 0-XSU(K)*XZU(K)/RPF3)*XSU(K)
      D(I)=-XSU(K)**2/RPF3
      IF (MOS4.EQ.2) SIG1=SIG1+(XU(K)-X(K)-XSU(K))**2
      ENDIF
      ELSE
      IF (IX(K).EQ.-1.OR.IX(K).EQ.-3) THEN
      CP(I)=XL(K)-X(K)+RPF3/XZL(K)
      D(I)=-MAX(STOL,XSL(K))/XZL(K)
      IF (MOS4.EQ.2) SIG1=SIG1+(XL(K)-X(K)+XSL(K))**2
      ELSE IF (IX(K).EQ.-2.OR.IX(K).EQ.-4) THEN
      CP(I)=XU(K)-X(K)-RPF3/XZU(K)
      D(I)=-MAX(STOL,XSU(K))/XZU(K)
      IF (MOS4.EQ.2) SIG1=SIG1+(XU(K)-X(K)-XSU(K))**2
      ENDIF
      ENDIF
      ELSE
      IF (JOB.EQ.1) THEN
      IF (IC(K).EQ.-1.OR.IC(K).EQ.-3) THEN
      CP(I)=CL(K)-CF(K)+(2.0D 0-CSL(K)*CZL(K)/RPF3)*CSL(K)
      D(I)=-CSL(K)**2/RPF3
      IF (MOS4.EQ.2) SIG1=SIG1+(CL(K)-CF(K)+CSL(K))**2
      ELSE IF (IC(K).EQ.-2.OR.IC(K).EQ.-4) THEN
      CP(I)=CU(K)-CF(K)-(2.0D 0-CSU(K)*CZU(K)/RPF3)*CSU(K)
      D(I)=-CSU(K)**2/RPF3
      IF (MOS4.EQ.2) SIG1=SIG1+(CU(K)-CF(K)-CSU(K))**2
      ELSE IF (IC(K).EQ.-5) THEN
      CP(I)=-CF(K)
      D(I)=0.0D 0
      IF (MOS4.EQ.2) SIG1=SIG1+CP(I)**2
      ENDIF
      ELSE
      IF (IC(K).EQ.-1.OR.IC(K).EQ.-3) THEN
      CP(I)=CL(K)-CF(K)+RPF3/CZL(K)
      D(I)=-MAX(STOL,CSL(K))/CZL(K)
      IF (MOS4.EQ.2) SIG1=SIG1+(CL(K)-CF(K)+CSL(K))**2
      ELSE IF (IC(K).EQ.-2.OR.IC(K).EQ.-4) THEN
      CP(I)=CU(K)-CF(K)-RPF3/CZU(K)
      D(I)=-MAX(STOL,CSU(K))/CZU(K)
      IF (MOS4.EQ.2) SIG1=SIG1+(CU(K)-CF(K)-CSU(K))**2
      ELSE IF (IC(K).EQ.-5) THEN
      CP(I)=-CF(K)
      D(I)=0.0D 0
      IF (MOS4.EQ.2) SIG1=SIG1+CP(I)**2
      ENDIF
      ENDIF
      ENDIF
    1 CONTINUE
      RETURN
      END
