* SUBROUTINE UCISLC             ALL SYSTEMS                 03/12/01
C PORTABILITY : ALL SYSTEMS
C 03/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* ACTIVE CONSTRAINT PART OF RIGHT HAND SIDE VECTOR.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  NX  NUMBER OF BOX CONSTRAINTS.
*  II  NC  NUMBER OF GENERAL CONSTRAINTS.
*  II  NCA  NUMBER OF ACTIVE CONSTRAINTS.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NX)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XL(NX)  VECTOR CONTAINING LOWER BOUNDS FOR VARIABLES.
*  RI  XU(NX)  VECTOR CONTAINING UPPER BOUNDS FOR VARIABLES.
*  RI  XSL(NX)  VECTOR CONTAINING LOVER SLACKS FOR VARIABLES.
*  RI  XSU(NX)  VECTOR CONTAINING UPPER SLACKS FOR VARIABLES.
*  RI  XZL(NX)  VECTOR CONTAINING LOVER MULTIPLIERS FOR VARIABLES.
*  RI  XZU(NX)  VECTOR CONTAINING UPPER MULTIPLIERS FOR VARIABLES.
*  RI  CF(NC)  VECTOR CONTAINING VALUES OF THE CONSTRAINT FUNCTIONS.
*  II  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  RI  CL(NC)  VECTOR CONTAINING LOWER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CU(NC)  VECTOR CONTAINING UPPER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CSL(NC)  VECTOR CONTAINING LOVER SLACKS FOR CONSTRAINTS.
*  RI  CSU(NC)  VECTOR CONTAINING UPPER SLACKS FOR CONSTRAINTS.
*  RI  CZL(NC)  VECTOR CONTAINING LOVER MULTIPLIERS FOR CONSTRAINTS.
*  RI  CZU(NC)  VECTOR CONTAINING UPPER MULTIPLIERS FOR CONSTRAINTS.
*  IO  ICA(NCA)  VECTOR CONTAINING INDICES OF ACTIVE CONSTRAINTS.
*  RO  CP(NCA)  ACTIVE CONSTRAINT VECTOR.
*  RO  G(NCA)  SLACK GRADIENT OF THE BARRIER FUNCTION.
*  RO  D(NCA)  DIAGONAL SOUTH-EAST CORNER.
*  RI  RPF3  BARRIER COEFFICIENT.
*  II  JOB  CHOICE OF THE IP FORMULATION. JOB=1-PRIMAL FORMULATION.
*         JOB=2-PRIMAL-DUAL FORMULATION.
*
      SUBROUTINE UCISLC(NF,NX,NC,NCA,X,IX,XL,XU,XSL,XSU,XZL,XZU,CF,IC,
     & CL,CU,CSL,CSU,CZL,CZU,ICA,CP,G,D,RPF3,JOB)
      INTEGER NF,NX,NC,NCA,IX(NX),IC(NC),ICA(NCA),JOB
      REAL*8 X(NF),XL(NX),XU(NX),XSL(NX),XSU(NX),XZL(NX),XZU(NX),
     & CF(NC),CL(NC),CU(NC),CSL(NC),CSU(NC),CZL(NC),CZU(NC),CP(NCA),
     & G(NCA),D(NCA),RPF3
      INTEGER I,K
      DO 1 I=1,NCA
      K=ICA(I)
      IF (K.LT.0) THEN
      K=-K
      IF (JOB.EQ.1) THEN
      IF (IX(K).EQ.-1.OR.IX(K).EQ.-3) THEN
      CP(I)=XL(K)-X(K)+XSL(K)
      G(I)=(XSL(K)*XZL(K)-RPF3)/SQRT(RPF3)
      D(I)=-XSL(K)**2/RPF3
      ELSE IF (IX(K).EQ.-2.OR.IX(K).EQ.-4) THEN
      CP(I)=XU(K)-X(K)-XSU(K)
      G(I)=(RPF3-XSU(K)*XZU(K))/SQRT(RPF3)
      D(I)=-XSU(K)**2/RPF3
      ENDIF
      ELSE
      IF (IX(K).EQ.-1.OR.IX(K).EQ.-3) THEN
      CP(I)=XL(K)-X(K)+XSL(K)
      G(I)=(XSL(K)*XZL(K)-RPF3)/SQRT(XSL(K)*XZL(K))
      D(I)=-XSL(K)/XZL(K)
      ELSE IF (IX(K).EQ.-2.OR.IX(K).EQ.-4) THEN
      CP(I)=XU(K)-X(K)-XSU(K)
      G(I)=(RPF3-XSU(K)*XZU(K))/SQRT(XSU(K)*XZU(K))
      D(I)=-XSU(K)/XZU(K)
      ENDIF
      ENDIF
      ELSE
      IF (JOB.EQ.1) THEN
      IF (IC(K).EQ.-1.OR.IC(K).EQ.-3) THEN
      CP(I)=CL(K)-CF(K)+CSL(K)
      G(I)=(CSL(K)*CZL(K)-RPF3)/SQRT(RPF3)
      D(I)=-CSL(K)**2/RPF3
      ELSE IF (IC(K).EQ.-2.OR.IC(K).EQ.-4) THEN
      CP(I)=CU(K)-CF(K)-CSU(K)
      G(I)=(RPF3-CSU(K)*CZU(K))/SQRT(RPF3)
      D(I)=-CSU(K)**2/RPF3
      ELSE IF (IC(K).EQ.-5) THEN
      CP(I)=-CF(K)
      G(I)=0.0D 0
      D(I)=0.0D 0
      ENDIF
      ELSE
      IF (IC(K).EQ.-1.OR.IC(K).EQ.-3) THEN
      CP(I)=CL(K)-CF(K)+CSL(K)
      G(I)=(CSL(K)*CZL(K)-RPF3)/SQRT(CSL(K)*CZL(K))
      D(I)=-CSL(K)/CZL(K)
      ELSE IF (IC(K).EQ.-2.OR.IC(K).EQ.-4) THEN
      CP(I)=CU(K)-CF(K)-CSU(K)
      G(I)=(RPF3-CSU(K)*CZU(K))/SQRT(CSU(K)*CZU(K))
      D(I)=-CSU(K)/CZU(K)
      ELSE IF (IC(K).EQ.-5) THEN
      CP(I)=-CF(K)
      G(I)=0.0D 0
      D(I)=0.0D 0
      ENDIF
      ENDIF
      ENDIF
    1 CONTINUE
      RETURN
      END
