* SUBROUTINE UCSLG1                ALL SYSTEMS                99/12/01
C PORTABILITY : ALL SYSTEMS
C 99/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* DETERMINATION OF LAGRANGE MULTIPLIERS USING LEAST SQUARES ESTIMATION.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  NC  NUMBER OF CONSTRAINTS.
*  II  MC  NUMBER OF NONZERO ELEMENTS OF THE SPARSE JACOBIAN MATRIX.
*  II  ML SIZE OF THE COMPACT FACTOR.
*  II  MMAX  LENGTH OF THE ARRAYS JB,B.
*  RU  B(MMAX)  NUMERICAL VALUES OF THE HESSIAN APPROXIMATION, ELEMENTS
*            OF THE TRIANGULAR FACTOR.
*  II  JB(MMAX) INDICES OF THE TRIANGULAR FACTOR OF THE HESSIAN MATRIX.
*  II  CG(MC)  ELEMENTS OF THE CONSTRAINT JACOBIAN MATRIX.
*  II  ICG(NC+1)  POSITION OF THE FIRST ROWS ELEMENTS IN THE FIELD CG.
*  II  JCG(MC)  COLUMN INDICES OF ELEMENTS IN THE FIELD CG.
*  II  PSL(NC+1) POINTER VECTOR OF THE SPARSE TRIANGULAR FACTOR OF THE
*           HESSIAN MATRIX.
*  RI  GA(NF) GRADIENT OF THE OBJECTIVE FUNCTION.
*  RO  CZ(NC) LAGRANGE MULTIPLIER VECTOR.
*  II  PERM(NC) PERMUTATION VECTOR.
*  IA  WN11(NC+1) AUXILIARY VECTOR.
*  IA  WN12(NC+1) AUXILIARY VECTOR.
*  RA  XS(NC) AUXILIARY VECTOR.
*  RI  ETA0  MACHINE PRECISION.
*  IU  INF  DECOMPOSITION INDICATOR. INF=0 IF THE MATRIX IS POSITIVE
*         DEFINITE.
*  II  MEP2  DETERMINATION OF THE INITIAL LAGRANGE MULTIPLIERS.
*         MEP2=1-MULTIPLIERS FROM THE PREVIOUS ITERATION.
*         MEP2=2MULTIPLIERS FROM THE LEAST SQUARES SOLUTION.
*
* COMMON DATA :
*  IO  ITERD TERMINATION INDICATOR. ITERD<0-BAD DECOMPOSITION.
*  IO  ITERM TERMINATION INDICATOR. ITERM<0-INVALID TERMINATION.
*  II  IDECC  DECOMPOSITION INDICATOR. IDECC=0-NO DECOMPOSITION.
*  TO  TDXX  TEXT INFORMATION ON THE DIRECTION DETERMINATION.
*
* SUBPROGRAMS USED :
*  S   UXSPCT  INITIATION OF THE FACTORS IN THE COMPACT FORM.
*  S   UXSPCF  SPARSE NUMERICAL GILL-MURRAY FACTORIZATION OF A
*         SYMMETRIC MATRIX USING COMPACT FORM FOR FACTORS WITH
*         THE CONTROL OF POSITIVE DEFINITENESS.
*  S   UXSPCB  SPARSE BACK SUBSTITUTION WITH FACTORS IN THE COMPACT
*         FORM.
*  S   UXVNEG  COPYING OF A VECTOR WITH A CHANGE OF THE SIGN.
*  S   UXVSFP  PERMUTATION OF A VECTOR.
*  S   UXVSBP  INVERSE PERMUTATION OF A VECTOR.
*  S   UOERR1  ERROR MESAGES.
*
      SUBROUTINE UCSLG1(NF,NC,MC,ML,MMAX,B,IB,JB,CG,ICG,JCG,G,S,PSL,
     & PERM,WN11,WN12,XS,ETA0,INF,MEP2)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER NF,NC,MC,ML,MMAX,IB(NC+1),JB(MMAX),ICG(NC+1),JCG(MC),
     & PSL(NC+1),PERM(NC),WN11(NC+1),WN12(NC+1),INF,MEP2
      REAL*8 B(MMAX),CG(MC),G(NF),S(NC),XS(NC),ETA0
      REAL*8 ALF,TAU
      INTEGER  L
      IF(IDECC.NE.0.AND.IDECC.NE.1) THEN
      ITERD=-1
      TDXX='BAD DEC1'
      CALL UOERR1('UCSLG1',1)
      GOTO 2
      ENDIF
      L=IB(NC+1)
      IF(IDECC.NE.1) THEN
      CALL UXSPCT(NC,L-1,ML,MMAX,B,JB,PSL,ITERM)
      IF (ITERM.NE.0) THEN
      TDXX='LACK SPC'
      RETURN
      ENDIF
C
C     GILL-MURRAY DECOMPOSITION
C
      ALF = ETA0
      CALL UXSPCF(NC,MMAX,B(L),PSL,JB(L),WN11,WN12,XS,INF,ALF,TAU)
      ENDIF
      IF (MEP2.EQ.2) THEN
C
C     NEWTON LIKE STEP
C
      CALL UXSRMM(NF,NC,MC,CG,ICG,JCG,G,S)
      CALL UXVNEG(NC,S,S)
      CALL UXVSFP(NC,PERM,S,XS)
      CALL UXSPCB(NC,MMAX,B(L),PSL,JB(L),S,0)
      CALL UXVSBP(NC,PERM,S,XS)
      ENDIF
    2 RETURN
      END
