* SUBROUTINE UDSLL2                ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* DETERMINATION OF THE DIRECTION VECTOR AS A SOLUTION OF THE INDEFINITE
* KARUSH-KUHN-TUCKER SYSTEM USING SPARSE BUNCH-PARLETT FACTORIZATION.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  NC  NUMBER OF CONSTRAINTS.
*  II  MC  NUMBER OF ELEMENTS IN THE FIELD CG.
*  II  M  LENGTH OF THE ARRAY H.
*  II  MMAX  LENGTH OF THE ARRAY B.
*  RI  H(M)  SPARSE SYMMETRIC MATRIX WHICH IS USED FOR DETERMINATION
*         OF THE DIRECTION VECTOR.
*  II  IH(NF+1)  POINTERS OF THE DIAGONAL ELEMENTS OF H.
*  II  JH(M)  INDICES OF THE NONZERO ELEMENTS OF H.
*  RI  G(NF)  GRADIENT OF THE OBJECTIVE FUNCTION.
*  RO  S(NF+NC)  DIRECTION VECTOR.
*  RA  XO(NF+NC)  AUXILIARY VECTOR.
*  RA  GO(NF+NC)  AUXILIARY VECTOR.
*  RA  XS(NF+NC)  AUXILIARY VECTOR.
*  II  IX(MX)  VECTOR CONTAINING TYPES OF BOUNDS.
*
* COMMON DATA :
*  IO  ITERD  TERMINATION INDICATOR. ITERD<0-BAD DECOMPOSITION.
*         ITERD=0-DESCENT DIRECTION. ITERD=1-NEWTON LIKE STEP.
*         ITERD=2-INEXACT NEWTON LIKE STEP. ITERD=3-BOUNDARY STEP.
*         ITERD=4-DIRECTION WITH THE NEGATIVE CURVATURE.
*         ITERD=5-MARQUARDT STEP.
*  IO  ITERM  TERMINATION INDICATOR. ITERM=0-SUCCESSFUL TERMINATION.
*  IU  IDECC  DECOMPOSITION INDICATOR. IDECC=0-NO DECOMPOSITION.
*         IDECC=1-GILL-MURRAY DECOMPOSITION. IDECC=2-BUNCH-PARLETT
*         DECOMPOSITION. IDECC=3-INVERSION.
*  IU  NDECC  NUMBER OF DECOMPOSITIONS.
*  TO  TDXX  TEXT INFORMATION ON THE DIRECTION VECTOR OBTAINED.
*
* SUBPROGRAMS USED :
*  S   UNSTEP  DETERMINATION OF A SCALING FACTOR FOR THE BOUNDARY STEP.
*  S   UXSSMM  MATRIX-VECTOR PRODUCT.
*  RF  UXSSMQ  VALUE OF THE QUADRATIC FORM.
*  RF  UXUDEL  NORM OF THE AUGMENTED VECTOR.
*  S   UXUDIR  VECTOR AUGMENTED BY THE SCALED VECTOR.
*  RF  UXUDOT  DOT PRODUCT OF VECTORS.
*  S   UXVNEG  COPYING OF A VECTOR WITH CHANGE OF THE SIGN.
*  S   UXVSET  INITIATION OF A VECTOR.
*  S   UOD1D1  PRINT OF ENTRY TO DIRECTION DETERMINATION.
*  S   UOD1D2  PRINT OF EXIT FROM DIRECTION DETERMINATION.
*
      SUBROUTINE UDSLL2(NF,NC,MC,M,MMAX,H,IH,CG,JCG,KCG,B,IB,JB,KB,IW,
     & IW1,IW2,G,CF,S,XO,NAU,NUP,INF)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER NF,NC,MC,M,MMAX,IH(NF+1),JCG(MC),KCG(MC),IB(NF+NC+1),
     & JB(M+MC+NC),KB(M+MC+NC),IW(MMAX),IW1(2*(NF+NC)),IW2(3*(NF+NC)),
     & NAU,NUP,INF
      REAL*8 H(M),CG(MC),B(MMAX),G(NF),CF(NC),S(NF+NC),XO(NF+NC)
      INTEGER NN,MM
      CALL UOD1D1
      IDECF=0
      M=IH(NF+1)-1
      IF(IDECF.NE.7) THEN
      CALL UXSKIN(NF,M,H,IH,NC,MC,CG,JCG,KCG,B,IB)
      NN=NF+NC
      MM=M+MC+NC
C
C     BUNCH-PARLETT DECOMPOSITION
C
      CALL UXSIBF(NN,MM,KB,JB,B,MMAX,IW,MMAX,IW1,IW2,NAU,NUP,INF)
      IF (INF.LT.0) THEN
      TDXX='LACK SPC'
      ITERM=INF
      GO TO 3
      ENDIF
      NDECF=NDECF+1
      IDECF=7
      ENDIF
      CALL UXVNEG(NF,G,S)
      CALL UXVNEG(NC,CF,S(NF+1))
      CALL UXSIBB(NN,B,MMAX,IW,MMAX,XO,S,IW1,NAU,NUP)
C
C     NEWTON LIKE STEP
C
      ITERD=1
      TDXX = 'B-P STEP'
    3 CALL UOD1D2(NF,G,S)
      RETURN
      END
