* SUBROUTINE UDSLL3                ALL SYSTEMS                97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* DETERMINATION OF THE DIRECTION VECTOR AS A SOLUTION OF THE SYSTEM OF
* NORMAL EQUATIONS USING ITERATIVE CONJUGATE GRADIENT METHOD.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  NC  NUMBER OF CONSTRAINTS.
*  II  MC  NUMBER OF ELEMENTS IN THE FIELD CG.
*  RI  CG(MC)  ELEMENTS OF THE CONSTRAINT JACOBIAN MATRIX.
*  II  ICG(NC+1)  POSITION OF THE FIRST ROWS ELEMENTS IN THE FIELD CG.
*  II  JCG(MC)  COLUMN INDICES OF ELEMENTS IN THE FIELD CG.
*  RU  B(MMAX)  NUMERICAL VALUES OF THE SPARSE MATRIX B.
*         OF THE TRIANGULAR FACTOR.
*  II  IB(NC+1) POINTER VECTOR OF THE SPARSE MATRIX A.
*  II  JB(MMAX) INDICES OF THE TRIANGULAR FACTOR OF THE HESSIAN MATRIX.
*  RI  D(NF)  DIAGONAL MATRIX.
*  RI  G(NF) GRADIENT OF THE OBJECTIVE FUNCTION.
*  RI  CF(NC) CONSTRAINT VECTOR.
*  RO  S(NF+NC) DIRECTION VECTOR.
*  II  PSL(NC+1) POINTER VECTOR OF THE SPARSE TRIANGULAR FACTOR OF THE
*         HESSIAN MATRIX.
*  II  PERM(NC) PERMUTATION VECTOR.
*  IA  WN11(NC+1) AUXILIARY VECTOR.
*  IA  WN12(NC+1) AUXILIARY VECTOR.
*  RI  ETA0  MACHINE PRECISION.
*
* COMMON DATA :
*  IO  ITERD  TERMINATION INDICATOR. ITERD<0-BAD DECOMPOSITION.
*         ITERD=0-DESCENT DIRECTION. ITERD=1-NEWTON LIKE STEP.
*         ITERD=2-INEXACT NEWTON LIKE STEP. ITERD=3-BOUNDARY STEP.
*         ITERD=4-DIRECTION WITH THE NEGATIVE CURVATURE.
*         ITERD=5-MARQUARDT STEP.
*  IU  IDECC  DECOMPOSITION INDICATOR. IDECC=0-NO DECOMPOSITION.
*         IDECC=1-GILL-MURRAY DECOMPOSITION. IDECC=2-BUNCH-PARLETT
*         DECOMPOSITION. IDECC=3-INVERSION.
*  TO  TDXX  TEXT INFORMATION ON THE DIRECTION VECTOR OBTAINED.
*
* SUBPROGRAMS USED :
*  S   UXSPCF  SPARSE NUMERICAL GILL-MURRAY FACTORIZATION OF A
*          SYMMETRIC MATRIX USING COMPACT FORM FOR FACTORS WITH
*          THE CONTROL OF POSITIVE DEFINITENESS.
*  S   UXSPCB  SPARSE BACK SUBSTITUTION WITH FACTORS IN COMPACT FORM.
*  RF  UXVDOT  DOT PRODUCT OF VECTORS.
*  S   UXVNEG  COPYING OF A VECTOR WITH A CHANGE OF THE SIGN.
*  S   UXVSFP  PERMUTATION OF A VECTOR.
*  S   UXVSBP  INVERSE PERMUTATION OF A VECTOR.
*  S   UOD1D1  PRINT OF ENTRY TO DIRECTION DETERMINATION.
*  S   UOD1D2  PRINT OF EXIT FROM DIRECTION DETERMINATION.
*  S   UOD1D5  PRINT OF INFORMATION DURING DIRECTION DETERMINATION.
*
      SUBROUTINE UDSLL3(NF,NC,MC,CG,ICG,JCG,D,G,CF,S,XO,GO,XS,GS,XP,GP,
     & ETA0,ETA9,SNORM,XDEL,MOS3)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER NF,NC,MC,ICG(NC+1),JCG(MC),MOS3
      INTEGER IMX,MMX
      REAL*8 CG(MC),D(NF),G(NF),CF(NC),S(NF+NC),XO(NC),GO(NC),
     & XS(NC),GS(NC),XP(NC),GP(NC),ETA0,ETA9,SNORM,XDEL
      REAL*8  RHO,RHO1,RHO2,ALF,TEMP2,UXVDOT,UXVNOR,BTB,BTR,RMU,PAR
      CALL UOD1D1
      IDECC=0
C
C     NEWTON LIKE STEP
C
      CALL UXVMUL(NF,D,G,S,-1)
      CALL UXSRMD(NF,NC,MC,CG,ICG,JCG,S,ONE,CF,GS)
      CALL UXVNEG(NC,GS,GS)
      RHO1 = UXVNOR(NC,GS)
      PAR = SQRT(ETA0)
      CALL UXVSET(NC,ZERO,S(NF+1))
      CALL UXVSET(NC,ZERO,XP)
      CALL UXVCOP(NC,GS,XO)
      CALL UXVCOP(NC,GS,XS)
      RHO = UXVDOT(NC,XS,XS)
      MMX=2*NC
      DO 1 IMX=1,MMX
      CALL UXSCMM(NF,NC,MC,CG,ICG,JCG,XO,S)
      CALL UXVMUL(NF,D,S,S,-1)
      CALL UXSRMM(NF,NC,MC,CG,ICG,JCG,S,GO)
      ALF = UXVDOT(NC,XO,GO)
      IF (ALF.EQ.ZERO) THEN
      ITERD=-5
      TDXX='CG BREAK'
      CALL UOERR1('UDSLL3',5)
      GO TO 3
      ENDIF
C
C     CG STEP
C
      TDXX = 'CG  STEP'
      ALF = RHO/ALF
      CALL UXVDIR(NC,ALF,XO,XP,XP)
      CALL UXVDIR(NC,-ALF,GO,XS,XS)
      IF (MOS3.LE.0) THEN
      CALL UXVCOP(NC,XS,GS)
      CALL UXVCOP(NC,XP,S(NF+1))
      ELSE
      RMU=ONE/ETA9
      CALL UXVDIF(NC,GS,XS,GP)
      BTB=UXVDOT(NC,GP,GP)
      BTR=UXVDOT(NC,GP,XS)
      RMU=-BTR/MAX(BTB,RMU)
      CALL UXVDIR(NC,RMU,GP,XS,GS)
      CALL UXVDIF(NC,S(NF+1),XP,GP)
      CALL UXVDIR(NC,RMU,GP,XP,S(NF+1))
      ENDIF
      TEMP2 = UXVNOR(NC,GS)
      CALL UOD1D4(RHO1,TEMP2,PAR,SNORM,XDEL)
      IF (TEMP2.LE.PAR*RHO1) GO TO 2
      RHO2 = UXVDOT(NC,XS,XS)
      ALF = RHO2/RHO
      CALL UXVDIR(NC,ALF,XO,XS,XO)
      RHO = RHO2
    1 CONTINUE
      TDXX = 'MAX ITER'
    2 ITERD=2
      CALL UXSCMD(NF,NC,MC,CG,ICG,JCG,S(NF+1),ONE,G,S)
      CALL UXVMUL(NF,D,S,S,-1)
    3 CALL UOD1D2(NF,G,S)
      RETURN
      END
