* SUBROUTINE UEDRD5                ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* SOLUTION OF THE INITIAL VALUE PROBLEM FOR A SET OF THE FIRST ORDER
* ORDINARY DIFFERENTIAL EQUATIONS USING THE FIFTH ORDER DORMAN AND
* PRINCE INTEGRATION FORMULA.
*
* PARAMETERS :
*  II  NA  NUMBER OF MESH POINTS.
*  II  NAL  NUMBER OF MESH POINTS.
*  II  NE  DECLARED NUMBER OF DIFFERENTIAL EQUATIONS.
*  II  MEL  ACTUAL NUMBER OF DIFFERENTIAL EQUATIONS.
*  II  MD  ACTUAL NUMBER OF DIFFERENTIAL EQUATIONS.
*  IU  KA  INDEX OF THE MESH POINT.
*  RI  AT(NA)  MESH POINTS.
*  RO  AY(NA*NE)  FIELD OF SOLUTION VECTORS IN THE MESH POINTS.
*  RU  YA(MEL)  SOLUTION VECTOR.
*  RA  YAO(MEL)  OLD SOLUTION VECTOR.
*  RI  EF(MEL)  RIGHT HAND SIDE VECTOR.
*  RA  ES(11*MEL)  AUXILIARY VECTOR.
*  RI  ED(MEL*MEL)  JACOBIAN MATRIX OF A RIGHT HAND SIDE VECTOR.
*  RA  E1(MEL,MEL)  AUXILIARY MATRIX.
*  RA  E2R(MEL,MEL)  AUXILIARY MATRIX.
*  RA  E2I(MEL,MEL)  AUXILIARY MATRIX.
*  RA  IP1(MEL)  AUXILIARY VECTOR.
*  RA  IP2(MEL)  AUXILIARY VECTOR.
*  RI  ETA0  MACHINE PRECISION.
*  RI  TET1  LOVER PARAMETER FOR STEPSIZE SELECTION.
*  RI  TET2  UPPER PARAMETER FOR STEPSIZE SELECTION.
*  RI  TET3  SAFETY FACTOR FOR STEPSIZE PREDICTION.
*  RI  ALF3  TOLERANCE FOR THE JACOBIAN MATRIX RECOMPUTATION.
*  RI  BET3  PARAMETER FOR STEPSIZE CONTROL.
*  RU  TA  VALUE OF THE INTEGRATION PARAMETER.
*  RU  TAO  OLD VALUE OF THE INTEGRATION PARAMETER.
*  RI  TAMAX  MAXIMUM VALUE OF THE INTEGRATION PARAMETER.
*  RU  HE  ACTUAL STEPSIZE.
*  RU  HEO  OLD STEPSIZE.
*  RU  HEMAX  MAXIMUM STEPSIZE.
*  RI  FMAX  UPPER BOUND FOR SOLUTION NORM.
*  RO  ERR  INTEGRATION ERROR.
*  II  MED  NUMBER OF STEPS PRECEEDING TEST OF STIFTNESS.
*  II  MED1  CHOICE OF STEP SELECTION.
*  II  MED2  NUMBER OF RESETING SOLUTIONS.
*  IO  NACC  NUMBER OF ACCEPTED INTEGRATION STEPS.
*  IO  NREJ  NUMBER OF REJECTED INTEGRATION STEPS.
*  IO  IER  ERROR INDICATOR. IER=0-SOLUTION OBTAINED. IER=-1-SYSTEM
*         IS UNSTABLE. IER=-2-MAXIMUM NUMBER OF STEPS EXCEEDED.
*         IER=-3-TOO SMALL STEP REQUIRED. IER=-4-SYSTEM IS PROBABLY
*         STIFF. IER=-5-JACOBIAN MATRIX IS SINGULAR. IER=-1-LACK
*         OF SPACE FOR OUTPUT DATA.
*
* COMMON DATA :
*  IO  NSTP  ACTUAL  NUMBER OF INTEGRATION STEPS.
*  II  MSTP  MAXIMUM NUMBER OF INTEGRATION STEPS.
*  RI  TOLR  RELATIVE PRECISION.
*  RI  TOLA  ABSOLUTE PRECISION.
*
* SUBPROGRAMS USED :
*  S   UYPRO1  PROLOGUE.
*  S   UYEPI1  EPILOGUE.
*  S   UYSET0  STATUS DEFINITION.
*  S   UNDRD5  INTERPOLATION IN MESH POINTS.
*  S   UXEGCB  COMPLEX BACK SUBSTITUTION AFTER FACTORIZATION BY UXEGCF.
*  S   UXEGCF  FACTORIZATION OF A COMPLEX DENSE GENERAL MATRIX.
*  S   UXEGGB  BACK SUBSTITUTION AFTER FACTORIZATION BY UXEGGB.
*  S   UXEGGF  FACTORIZATION A DENSE GENERAL MATRIX.
*  S   UXVCOP  COPYING OF A VECTOR.
*  S   UXVMED  MEDIAN OF TWO VECTORS.
*
* METHOD :
* E.HAIRER, G.WANNER: SOLVING ORDINARY DIFFERENTIAL
* EQUATIONS II. SPRINGER SERIES IN COMPUTATIONAL MATHEMATICS.
* SPRINGER VERLAG, BERLIN 1991.
*
      SUBROUTINE UEDRD5(NA,NAL,NE,MEL,MD,KA,AT,AY,YA,YAO,EF,ES,ED,E1,
     & E2R,E2I,IP1,IP2,ETA0,TET1,TET2,TET3,ALF3,BET3,TA,TAO,TAMAX,HE,
     & HEO,HEMAX,FMAX,ERR,MED,MED1,MED2,NACC,NREJ,IER)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER NA,NAL,NE,MEL,MD,KA,IP1(MEL),IP2(MEL),MED,MED1,
     & MED2,NACC,NREJ,IER
      REAL*8 AT(*),AY(*),YA(MEL),YAO(MEL),EF(MEL),ES(*),
     & ED(MEL*MEL),E1(MD,MEL),E2R(MD,MEL),E2I(MD,MEL),ETA0,TET1,
     & TET2,TET3,ALF3,BET3,TA,TAO,TAMAX,HE,HEO,HEMAX,FMAX,ERR
      REAL*8 SQ6,C1,C2,C1M1,C2M1,C1MC2,DD1,DD2,DD3,U1,ALFA,BETA,CNO,
     & POSNEG,HMAXN,FACCON,CFAC,HHFAC,FAC1,
     & ALFN,BETN,AK1,AK2,AK3,Z1I,Z2I,Z3I,C1Q,C2Q,C3Q,THETA,A1,A2,A3,
     & S2,S3,DYNO,DENOM,DYTH,QNEWT,DYNOLD,F1I,F2I,F3I,
     & HEE1,HEE2,HEE3,FAC,QUOT,HOPT,FJ,HNEW,AK,ACONT3,QT
      REAL*8 T11,T12,T13,T21,T22,T23,T31,TI11,TI12,TI13,TI21,TI22,
     & TI23,TI31,TI32,TI33,UXVMAX
      INTEGER NE1,NE2,NE3,NE4,NE5,NE6,NE7,NE8,NE9,NE10,NE11,NITN,I,J,
     & NSING,NDEC,NSOL,NEWT,IERR,NEGPOS
      LOGICAL REJECT,FIRST,CALJAC,STARTN
      SAVE C1,C1M1,C1MC2,C2,C2M1,DD1,DD2,DD3,U1,ALFA,BETA,CNO,HMAXN,
     & CFAC,FAC1,FACCON,ALFN,BETN,DYNOLD,HOPT,HNEW,POSNEG,THETA
      SAVE NE1,NE2,NE3,NE4,NE5,NE6,NE7,NE8,NE9,NE10,NE11,NITN,NEWT,
     & NEGPOS
      SAVE REJECT,FIRST,CALJAC,STARTN,NDEC,NSOL
      PARAMETER (T11= 9.1232394870892942792D-2,
     &           T12=-0.14125529502095420843D 0,
     &           T13=-3.0029194105147424492D-2,
     &           T21= 0.24171793270710701896D 0,
     &           T22= 0.20412935229379993199D 0,
     &           T23= 0.38294211275726193779D 0,
     &           T31= 0.96604818261509293619D 0)
      PARAMETER (TI11=4.3255798900631553510D 0,
     &           TI12= 0.33919925181580986954D 0,
     &           TI13= 0.54177053993587487119D 0,
     &           TI21=-4.1787185915519047273D 0,
     &           TI22=-0.32768282076106238708D 0,
     &           TI23= 0.47662355450055045196D 0,
     &           TI31=-0.50287263494578687595D 0,
     &           TI32= 2.5719269498556054292D 0,
     &           TI33=-0.59603920482822492497D 0)
      IF (TSS(NS).NE.'UEDX') CALL UYPRO1('UEDX',-1)
      GO TO (1,2,3,4,5,6,7,8) ISP(NS)
    1 CONTINUE
C
C     INITIALISATIONS
C
      NE1=0
      NE2=MEL
      NE3=2*MEL
      NE4=3*MEL
      NE5=4*MEL
      NE6=5*MEL
      NE7=6*MEL
      NE8=7*MEL
      NE9=8*MEL
      NE10=9*MEL
      NE11=10*MEL
      NITN=7
      NSTP=0
      NDEC=0
      NSOL=0
      STARTN=.FALSE.
C
C     CONSTANTS
C
      SQ6=SQRT(6.0D 0)
      C1=(FOUR-SQ6)/TEN
      C2=(FOUR+SQ6)/TEN
      C1M1=C1-ONE
      C2M1=C2-ONE
      C1MC2=C1-C2
      DD1=-(13.0D 0+7.0D 0*SQ6)/THREE
      DD2=(-13.0D 0+7.0D 0*SQ6)/THREE
      DD3=-ONE/THREE
      U1=(6.0D 0+81.0D 0**(ONE/THREE)-
     & 9.0D 0**(ONE/THREE))/30.0D 0
      ALFA=(12.0D 0-81.0D 0**(ONE/THREE)+
     & 9.0D 0**(ONE/THREE))/60.0D 0
      BETA=(81.0D 0**(ONE/THREE)+
     & 9.0D 0**(ONE/THREE))*SQRT(THREE)/60.0D 0
      CNO=ALFA**2+BETA**2
      POSNEG=SIGN(ONE,TAMAX-TA)
      HMAXN=MIN(ABS(HEMAX),ABS(TAMAX-TA))
      IF (ABS(HE).LE.TEN*ETA0) HE=1.0D-6
      HE=MIN(ABS(HE),HMAXN)
      HE=SIGN(HE,POSNEG)
      HEO=HE
      REJECT=.FALSE.
      FIRST=.TRUE.
      FACCON=ONE
      CFAC=TET3*(1+2*NITN)
      NSING=0
      IF (MED1.EQ.2) THEN
      NEGPOS=POSNEG
      IF (NEGPOS.GT.0) KA=1
      IF (NEGPOS.LT.0) KA=NA
      ENDIF
      TAO=TA
C
C     INITIALISATION
C
      CALL UXVCOP(MEL,YA,YAO)
      DO 21 I=1,MEL
  21  ES(NE8+I)=TOLA+TOLR*ABS(YA(I))
      HHFAC=HE
      CALL UYSET0(1,2)
      RETURN
    2 CONTINUE
      CALL UXVCOP(MEL,EF,ES(NE1+1))
      IF (MED.NE.0) CALL UNDRD5(NA,NAL,NE,MEL,KA,AT,AY,YAO,YAO,TA,TAO,
     & HE,HEO,C1M1,C2M1,IER,-MED)
C
C     BASIC INTEGRATION STEP
C
  24  CONTINUE
C
C     COMPUTATION OF THE JACOBIAN
C
      CALL UXVCOP(MEL,YAO,YA)
      TA=TAO
      CALL UYSET0(2,3)
      RETURN
    3 CONTINUE
      CALJAC=.TRUE.
  25  CONTINUE
C
C     COMPUTE THE MATRICES E1 AND E2 AND THEIR DECOMPOSITIONS
C
      FAC1=ONE/(HE*U1)
      ALFN=ALFA/(HE*CNO)
      BETN=BETA/(HE*CNO)
      DO 27 J=1,MEL
      DO 26 I=1,MEL
      FJ=-ED((J-1)*MEL+I)
      E1(J,I)=FJ
      E2R(J,I)=FJ
  26  E2I(J,I)=ZERO
      E1(J,J)=E1(J,J)+FAC1
      E2R(J,J)=E2R(J,J)+ALFN
  27  E2I(J,J)=BETN
      CALL UXEGGF(MEL,MD,E1,IP1,IERR,ZERO)
      IF (IERR.NE.0) GOTO 66
      CALL UXEGCF(MEL,MD,E2R,E2I,IP2,IERR)
      IF (IERR.NE.0) GOTO 66
      NDEC=NDEC+1
  31  CONTINUE
      IF (UXVMAX(NE,YA).GE.FMAX) GO TO 71
      IF (MED1.EQ.2) THEN
        IER=0
        KA=KA+NEGPOS
        IF (KA.LT.1.OR.KA.GT.NA) GO TO 72
        HE=AT(KA)-TAO
      ELSE
        IF (NSTP.GT.MSTP) GO TO 70
        IF (TAO+0.1D 0*HE.EQ.TAO.OR.ABS(HE).LE.ETA0) GOTO 69
      ENDIF
      NSTP=NSTP+1
C
C     STARTING VALUES FOR NEWTON ITERATION
C
      IF (FIRST.OR.STARTN) THEN
        DO 32 I=1,MEL
        ES(NE2+I)=ZERO
        ES(NE3+I)=ZERO
        ES(NE4+I)=ZERO
        ES(NE5+I)=ZERO
        ES(NE6+I)=ZERO
  32    ES(NE7+I)=ZERO
      ELSE
        C3Q=HE/HEO
        C1Q=C1*C3Q
        C2Q=C2*C3Q
        DO 33 I=1,MEL
        AK1=ES(NE9+I)
        AK2=ES(NE10+I)
        AK3=ES(NE11+I)
        Z1I=C1Q*(AK1+(C1Q-C2M1)*(AK2+(C1Q-C1M1)*AK3))
        Z2I=C2Q*(AK1+(C2Q-C2M1)*(AK2+(C2Q-C1M1)*AK3))
        Z3I=C3Q*(AK1+(C3Q-C2M1)*(AK2+(C3Q-C1M1)*AK3))
        ES(NE2+I)=Z1I
        ES(NE3+I)=Z2I
        ES(NE4+I)=Z3I
        ES(NE5+I)=TI11*Z1I+TI12*Z2I+TI13*Z3I
        ES(NE6+I)=TI21*Z1I+TI22*Z2I+TI23*Z3I
  33    ES(NE7+I)=TI31*Z1I+TI32*Z2I+TI33*Z3I
      END IF
C
C     LOOP FOR THE SIMPLIFIED NEWTON ITERATION
C
      NEWT=0
      FACCON=MAX(FACCON,ETA0)**0.8D 0
      THETA=ABS(ALF3)
  34  CONTINUE
      IF (NEWT.GE.NITN) GOTO 66
C
C     COMPUTE THE RIGHT-HAND SIDE
C
      DO 35 I=1,MEL
  35  YA(I)=YAO(I)+ES(NE2+I)
      TA=TAO+C1*HE
      CALL UYSET0(1,4)
      RETURN
    4 CONTINUE
      CALL UXVCOP(MEL,EF,ES(NE2+1))
      DO 36 I=1,MEL
  36  YA(I)=YAO(I)+ES(NE3+I)
      TA=TAO+C2*HE
      CALL UYSET0(1,5)
      RETURN
    5 CONTINUE
      CALL UXVCOP(MEL,EF,ES(NE3+1))
      DO 37 I=1,MEL
  37  YA(I)=YAO(I)+ES(NE4+I)
      TA=TAO+HE
      CALL UYSET0(1,6)
      RETURN
    6 CONTINUE
      CALL UXVCOP(MEL,EF,ES(NE4+1))
C
C     SOLVE THE LINEAR SYSTEMS
C
      DO 38 I=1,MEL
      A1=ES(NE2+I)
      A2=ES(NE3+I)
      A3=ES(NE4+I)
      S2=-ES(NE6+I)
      S3=-ES(NE7+I)
      ES(NE2+I)=-ES(NE5+I)*FAC1+TI11*A1+TI12*A2+TI13*A3
      ES(NE3+I)=S2*ALFN-S3*BETN+TI21*A1+TI22*A2+TI23*A3
  38  ES(NE4+I)=S3*ALFN+S2*BETN+TI31*A1+TI32*A2+TI33*A3
      CALL UXEGGB(MEL,MD,E1,IP1,ES(NE2+1),0)
      CALL UXEGCB(MEL,MD,E2R,E2I,ES(NE3+1),ES(NE4+1),IP2)
      DO 44 I=1,MEL
      ES(NE5+I)=ES(NE5+I)+ES(NE2+I)
      ES(NE6+I)=ES(NE6+I)+ES(NE3+I)
  44  ES(NE7+I)=ES(NE7+I)+ES(NE4+I)
      NSOL=NSOL+1
      NEWT=NEWT+1
      DYNO=ZERO
      DO 45 I=1,MEL
      DENOM=ES(NE8+I)
  45  DYNO=DYNO+(ES(NE2+I)/DENOM)**2+(ES(NE3+I)/DENOM)**2
     &    +(ES(NE4+I)/DENOM)**2
      DYNO=SQRT(DYNO/NE4)
C
C     BAD CONVERGENCE OR NUMBER OF ITERATIONS TO LARGE
C
      IF (NEWT.GE.2.AND.NEWT.LT.NITN) THEN
        THETA=DYNO/DYNOLD
        IF (THETA.LT.0.99D 0) THEN
          FACCON=THETA/(ONE-THETA)
          DYTH=FACCON*DYNO*THETA**(NITN-1-NEWT)
          QNEWT=MAX(1.0D-4,MIN(20.0D 0,DYTH/BET3))
          IF (QNEWT.GE.ONE) THEN
            HHFAC=.8D 0*QNEWT**(-ONE/(FOUR+NITN-1-NEWT))
            HE=HHFAC*HE
            REJECT=.TRUE.
            IF (CALJAC) GOTO 25
            GOTO 24
          END IF
        ELSE
          GOTO 66
        END IF
      END IF
      DYNOLD=MAX(DYNO,ETA0)
      DO 46 I=1,MEL
      F1I=ES(NE5+I)
      F2I=ES(NE6+I)
      F3I=ES(NE7+I)
      ES(NE2+I)=T11*F1I+T12*F2I+T13*F3I
      ES(NE3+I)=T21*F1I+T22*F2I+T23*F3I
  46  ES(NE4+I)=T31*F1I+    F2I
      IF (FACCON*DYNO.GT.BET3) GOTO 34
      IF (MED1.EQ.2) THEN
        NACC=NACC+1
        DO 23 I=1,MEL
        ES(NE1+I)=ES(NE2+I)
        YAO(I)=YA(I)
  23    CONTINUE
        IF (MED2.EQ.1) CALL UXVCOP(NE,AY((KA-1)*NE+1),YAO)
        IF (MED2.EQ.2) CALL UXVMED(NE,AY((KA-1)*NE+1),YAO,YAO)
        TAO=AT(KA)
        IF (CALJAC) GOTO 25
        GOTO 24
      END IF
C
C     ERROR ESTIMATION
C
      HEE1=DD1/HE
      HEE2=DD2/HE
      HEE3=DD3/HE
      DO 47 I=1,MEL
  47  ES(NE6+I)=HEE1*ES(NE2+I)+HEE2*ES(NE3+I)+HEE3*ES(NE4+I)
      DO 48 I=1,MEL
  48  YA(I)=ES(NE6+I)+ES(NE1+I)
       CALL UXEGGB(MEL,MD,E1,IP1,YA,0)
      ERR=ZERO
      DO 54 I=1,MEL
  54  ERR=ERR+(YA(I)/ES(NE8+I))**2
      ERR=MAX(SQRT(ERR/MEL),1.0D-10)
      IF (.NOT.(FIRST.OR.REJECT).OR.ERR.LT.ONE) GO TO 73
      DO 55 I=1,MEL
  55  YA(I)=YAO(I)+YA(I)
      TA=TAO
      CALL UYSET0(1,7)
      RETURN
    7 CONTINUE
      CALL UXVCOP(MEL,EF,ES(NE5+1))
      DO 56 I=1,MEL
  56  YA(I)=ES(NE5+I)+ES(NE6+I)
      CALL UXEGGB(MEL,MD,E1,IP1,YA,0)
      ERR=ZERO
      DO 62 I=1,MEL
  62  ERR=ERR+(YA(I)/ES(NE8+I))**2
      ERR=MAX(SQRT(ERR/MEL),1.0D-10)
C
C     COMPUTATION OF HNEW
C     WE REQUIRE .2<=HNEW/HE<=8.
C
  73  FAC=MIN(TET3,CFAC/(NEWT+2*NITN))
      QUOT=MAX(0.125D 0,MIN(FIVE,ERR**.25D 0/FAC))
      HNEW=HE/QUOT
C
C     IS THE ERROR SMALL ENOUGH ?
C
      IF (ERR.GE.ONE) THEN
C
C     STEP IS REJECTED
C
        REJECT=.TRUE.
        IF (FIRST) THEN
          HE=HE*0.1D 0
          HHFAC=0.1D 0
        ELSE
          HHFAC=HNEW/HE
          HE=HNEW
        END IF
        IF (NACC.GE.1) NREJ=NREJ+1
        IF (CALJAC) GOTO 25
        GOTO 24
      END IF
C
C     STEP IS ACCEPTED
C
      FIRST=.FALSE.
      NACC=NACC+1
      HEO=HE
      TAO=TA
      DO 63 I=1,MEL
      YAO(I)=YAO(I)+ES(NE4+I)
      Z2I=ES(NE3+I)
      Z1I=ES(NE2+I)
      ES(NE9+I)=(Z2I-ES(NE4+I))/C2M1
      AK=(Z1I-Z2I)/C1MC2
      ACONT3=Z1I/C1
      ACONT3=(AK-ACONT3)/C2
      ES(NE10+I)=(AK-ES(NE9+I))/C1M1
  63  ES(NE11+I)=ES(NE10+I)-ACONT3
      DO 64 I=1,MEL
  64  ES(NE8+I)=TOLA+TOLR*ABS(YAO(I))
      IF (MED.EQ.1) CALL UNDRD5(NA,NAL,NE,MEL,KA,AT,AY,YAO,YAO,TA,TAO,
     & HE,HEO,C1M1,C2M1,IER,MED)
      IF (MED.GE.2) THEN
      IF (HE*(TA-AT(KA)).GE.ZERO) THEN
      CALL UNDRD5(NA,NAL,NE,MEL,KA,AT,AY,YAO,ES(NE9+1),TA,TAO,HE,HEO,
     & C1M1,C2M1,IER,MED)
      END IF
      END IF
      IF (IER.EQ.-11) GO TO 72
      CALJAC=.FALSE.
      IF ((TAO-TAMAX)*POSNEG+ETA0.GT.ZERO) THEN
        HE=HOPT
        IER=0
        GO TO 72
      END IF
      CALL UXVCOP(MEL,YAO,YA)
      TA=TAO
      CALL UYSET0(1,8)
      RETURN
    8 CONTINUE
      CALL UXVCOP(MEL,EF,ES(NE1+1))
      HNEW=POSNEG*MIN(ABS(HNEW),HMAXN)
      HOPT=HNEW
      IF (REJECT) HNEW=POSNEG*MIN(ABS(HNEW),ABS(HE))
      REJECT=.FALSE.
      IF ((TAO+HNEW/TET1-TAMAX)*POSNEG.GT.ZERO) THEN
        HE=TAMAX-TAO
      ELSE
        QT=HNEW/HE
        HHFAC=HE
        IF (THETA.LE.ALF3.AND.QT.GE.TET1.AND.QT.LE.TET2) GOTO 31
        HE=HNEW
      END IF
      HHFAC=HE
      IF (THETA.LE.ALF3) GOTO 25
      GOTO 24
C
C     UNEXPECTED STEP-REJECTION
C
  66  CONTINUE
      IF (IERR.NE.0) THEN
        NSING=NSING+1
        IF (NSING.GE.6) GOTO 68
      END IF
      HE=HE*HALF
      HHFAC=HALF
      REJECT=.TRUE.
      IF (CALJAC) GOTO 25
      GOTO 24
C
C     FAIL EXIT
C
  68  IER=-5
      TEDX='SING JAC'
      GO TO 72
  69  IER=-3
      TEDX='SMALL ST'
      GO TO 72
  70  IER=-2
      TEDX='STEP LIM'
      GO TO 72
  71  IER=-1
      TEDX='UNSTABLE'
  72  CALL UYEPI1(-1)
      RETURN
      END
