* SUBROUTINE UEDRS4                ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* SOLUTION OF THE INITIAL VALUE PROBLEM FOR A SET OF THE FIRST ORDER
* ORDINARY DIFFERENTIAL EQUATIONS USING THE FIFTH ORDER DORMAN AND
* PRINCE INTEGRATION FORMULA.
*
* PARAMETERS :
*  II  NA  NUMBER OF MESH POINTS.
*  II  NAL  NUMBER OF MESH POINTS.
*  II  NE  DECLARED NUMBER OF DIFFERENTIAL EQUATIONS.
*  II  MEL  ACTUAL NUMBER OF DIFFERENTIAL EQUATIONS.
*  II  MD  ACTUAL NUMBER OF DIFFERENTIAL EQUATIONS.
*  IU  KA  INDEX OF THE MESH POINT.
*  RI  AT(NA)  MESH POINTS.
*  RO  AY(NA*NE)  FIELD OF SOLUTION VECTORS IN THE MESH POINTS.
*  RU  YA(MEL)  SOLUTION VECTOR.
*  RA  YAO(MEL)  OLD SOLUTION VECTOR.
*  RI  EF(MEL)  RIGHT HAND SIDE VECTOR.
*  RI  ET(MEL)  DERIVATIVE OF A RIGHT HAND SIDE VECTOR WITH RESPECT
*         TO INDEPENDENT VARIABLE T.
*  RA  ES(11*MEL)  AUXILIARY VECTOR.
*  RI  ED(MEL*MEL)  JACOBIAN MATRIX OF A RIGHT HAND SIDE VECTOR.
*  RA  E1(MD,MEL)  AUXILIARY MATRIX.
*  RU  IP(MEL)  PERMUTATION VECTOR.
*  RI  ETA0  MACHINE PRECISION.
*  RI  TET1  LOVER PARAMETER FOR STEPSIZE SELECTION.
*  RI  TET2  UPPER PARAMETER FOR STEPSIZE SELECTION.
*  RU  TA  VALUE OF THE INTEGRATION PARAMETER.
*  RU  TAO  OLD VALUE OF THE INTEGRATION PARAMETER.
*  RI  TAMAX  MAXIMUM VALUE OF THE INTEGRATION PARAMETER.
*  RU  HE  ACTUAL STEPSIZE.
*  RU  HEMAX  MAXIMUM STEPSIZE.
*  RI  FMAX  UPPER BOUND FOR SOLUTION NORM.
*  RO  ERR  INTEGRATION ERROR.
*  II  MED  NUMBER OF STEPS PRECEEDING TEST OF STIFTNESS.
*  II  MED1  CHOICE OF STEP SELECTION.
*  II  MED2  NUMBER OF RESETING SOLUTIONS.
*  II  MED3  NUMBER OF STEPS PRECEEDING TEST OF STIFTNESS.
*  IO  NACC  NUMBER OF ACCEPTED INTEGRATION STEPS.
*  IO  NREJ  NUMBER OF REJECTED INTEGRATION STEPS.
*  IO  IER  ERROR INDICATOR. IER=0-SOLUTION OBTAINED. IER=-1-SYSTEM
*         IS UNSTABLE. IER=-2-MAXIMUM NUMBER OF STEPS EXCEEDED.
*         IER=-3-TOO SMALL STEP REQUIRED. IER=-4-SYSTEM IS PROBABLY
*         STIFF. IER=-5-JACOBIAN MATRIX IS SINGULAR. IER=-1-LACK
*         OF SPACE FOR OUTPUT DATA.
*
* COMMON DATA :
*  IO  NSTP  ACTUAL  NUMBER OF INTEGRATION STEPS.
*  II  MSTP  MAXIMUM NUMBER OF INTEGRATION STEPS.
*  RI  TOLR  RELATIVE PRECISION.
*  RI  TOLA  ABSOLUTE PRECISION.
*
* SUBPROGRAMS USED :
*  S   UYPRO1  PROLOGUE.
*  S   UYEPI1  EPILOGUE.
*  S   UYSET0  STATUS DEFINITION.
*  S   UNDRS4  INTERPOLATION IN MESH POINTS.
*  S   UNCF01  COEFFICIENTS FOR THE ROSENBROCK METHOD OF ORDER 4.
*  S   UNCF02  COEFFICIENTS FOR THE ROSENBROCK METHOD OF ORDER 4.
*  S   UXEGGB  BACK SUBSTITUTION AFTER FACTORIZATION BY UXEGGB.
*  S   UXEGGF  FACTORIZATION A DENSE GENERAL MATRIX.
*  S   UXVCOP  COPYING OF A VECTOR.
*  S   UXVMED  MEDIAN OF TWO VECTORS.
*
* METHOD :
* E.HAIRER, S.P.NORSETT, G.WANNER: SOLVING ORDINARY DIFFERENTIAL
* EQUATIONS I. SPRINGER SERIES IN COMPUTATIONAL MATHEMATICS.
* SPRINGER VERLAG, BERLIN 1980.
*
      SUBROUTINE UEDRS4(NA,NAL,NE,MEL,MD,KA,AT,AY,YA,YAO,EF,ET,ES,ED,
     & E1,IP,ETA0,TET1,TET2,TA,TAO,TAMAX,HE,HEMAX,FMAX,ERR,MED,MED1,
     & MED2,MED3,NACC,NREJ,IER,IFCN)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER NA,NAL,NE,MEL,MD,KA,IP(MEL),MED,MED1,MED2,MED3,NACC,NREJ,
     & IER,IFCN
      REAL*8 AT(*),AY(*),YA(MEL),YAO(MEL),EF(MEL),ET(MEL),
     & ES(*),ED(MEL*MEL),E1(MD,MEL),ETA0,TET1,TET2,TA,TAO,TAMAX,
     & HE,HEMAX,FMAX,ERR
      REAL*8 POSNEG,HMAXN,HOPT,FAC,SK,HNEW,UXVMAX
      INTEGER NE1,NE2,NE3,NE4,NE5,NE6,NE7,NE8,NE9,NE10,NE11,NSING,
     &  IERR,NDEC,NSOL,NEGPOS,I,J
      REAL*8 A21,A31,A32,A41,A42,A43,A51,A52,A53,A54,
     &  C21,C31,C32,C41,C42,C43,C51,C52,C53,C54,C61,
     &  C62,C63,C64,C65,GAMMA,C2,C3,C4,D1,D2,D3,D4,
     &  D21,D22,D23,D24,D25,D31,D32,D33,D34,D35,
     &  HC21,HC31,HC32,HC41,HC42,HC43,HC51,HC52,HC53,HC54,HC61,
     &  HC62,HC63,HC64,HC65,HD1,HD2,HD3,HD4
      LOGICAL REJECT,LAST
      SAVE NE1,NE2,NE3,NE4,NE5,NE6,NE7,NE8,NE9,NE10,NE11,NSING,NEGPOS
      SAVE A21,A31,A32,A41,A42,A43,A51,A52,A53,A54,
     &  C21,C31,C32,C41,C42,C43,C51,C52,C53,C54,C61,
     &  C62,C63,C64,C65,GAMMA,C2,C3,C4,D1,D2,D3,D4,
     &  D21,D22,D23,D24,D25,D31,D32,D33,D34,D35,
     &  HC21,HC31,HC32,HC41,HC42,HC43,HC51,HC52,HC53,HC54,HC61,
     &  HC62,HC63,HC64,HC65,HD1,HD2,HD3,HD4,POSNEG,HMAXN
      SAVE REJECT,LAST,NDEC,NSOL
      IF (TSS(NS).NE.'UEDX') CALL UYPRO1('UEDX',-1)
      GO TO (1,2,3,4,5,6,7,8,9,10,11) ISP(NS)
    1 CONTINUE
C
C     INITIALISATIONS
C
      NE1=0
      NE2=MEL
      NE3=2*MEL
      NE4=3*MEL
      NE5=4*MEL
      NE6=5*MEL
      NE7=6*MEL
      NE8=7*MEL
      NE9=8*MEL
      NE10=9*MEL
      NE11=10*MEL
      NSTP=0
      NDEC=0
      NSOL=0
      POSNEG=SIGN(ONE,TAMAX-TA)
C
C     SET THE PARAMETERS OF THE METHOD
C
      IF(MED3.EQ.1)CALL UNCF01(A21,A31,A32,A41,A42,A43,A51,A52,A53,A54,
     &  C21,C31,C32,C41,C42,C43,C51,C52,C53,C54,C61,
     &  C62,C63,C64,C65,GAMMA,C2,C3,C4,D1,D2,D3,D4,
     &  D21,D22,D23,D24,D25,D31,D32,D33,D34,D35)
      IF(MED3.EQ.2)CALL UNCF02(A21,A31,A32,A41,A42,A43,A51,A52,A53,A54,
     &  C21,C31,C32,C41,C42,C43,C51,C52,C53,C54,C61,
     &  C62,C63,C64,C65,GAMMA,C2,C3,C4,D1,D2,D3,D4,
     &  D21,D22,D23,D24,D25,D31,D32,D33,D34,D35)
C
C     INITIAL PREPARATIONS
C
      HMAXN=MIN(ABS(HEMAX),ABS(TAMAX-TA))
      HE=MIN(MAX(1.0D-10,ABS(HE)),HMAXN)
      HE=SIGN(HE,POSNEG)
      REJECT=.FALSE.
      NSING=0
      IF (MED1.EQ.2) THEN
      NEGPOS=POSNEG
      IF (NEGPOS.GT.0) KA=1
      IF (NEGPOS.LT.0) KA=NA
      ENDIF
      TAO=TA
      CALL UXVCOP(MEL,YA,YAO)
      IF (MED.NE.0) CALL UNDRS4(NA,NAL,NE,MEL,KA,AT,AY,YAO,TA,TAO,HE,
     & IER,-MED)
C
C     BASIC INTEGRATION STEP
C
   24 CONTINUE
      IF (UXVMAX(NE,YA).GE.FMAX) GO TO 61
      IF (MED1.EQ.2) THEN
        IER=0
        KA=KA+NEGPOS
        IF (KA.LT.1.OR.KA.GT.NA) GO TO 62
        HE=AT(KA)-TAO
      ELSE
        IF (NSTP.GT.MSTP) GO TO 60
        IF (TAO+0.1D 0*HE.EQ.TAO.OR.ABS(HE).LE.ETA0) GO TO 59
        IF ((TAO+1.01D 0*HE-TAMAX)*POSNEG.GT.ZERO) THEN
          HE=TAMAX-TAO
          LAST=.TRUE.
        ELSE
          LAST=.FALSE.
        END IF
      END IF
      HOPT=HE
      IF ((TAO+HE-TAMAX)*POSNEG.GT.ZERO) HE=TAMAX-TAO
C
C     COMPUTATION OF THE JACOBIAN
C
      CALL UXVCOP(MEL,YAO,YA)
      TA=TAO
      CALL UYSET0(1,2)
      RETURN
    2 CONTINUE
      CALL UXVCOP(MEL,EF,ES(NE1+1))
C
C     COMPUTE JACOBIAN MATRIX ANALYTICALLY
C
      CALL UYSET0(2,3)
      RETURN
    3 CONTINUE
  25  CONTINUE
C
C     COMPUTE THE STAGES
C
      NDEC=NDEC+1
C
C     PREPARE FOR THE COMPUTATION OF THE 6 STAGES
C
      HC21=C21/HE
      HC31=C31/HE
      HC32=C32/HE
      HC41=C41/HE
      HC42=C42/HE
      HC43=C43/HE
      HC51=C51/HE
      HC52=C52/HE
      HC53=C53/HE
      HC54=C54/HE
      HC61=C61/HE
      HC62=C62/HE
      HC63=C63/HE
      HC64=C64/HE
      HC65=C65/HE
      FAC=ONE/(HE*GAMMA)
C
C     THIS PART COMPUTES THE STAGES IN THE CASE WHERE
C       1) THE DIFFERENTIAL EQUATION IS IN EXPLICIT FORM
C       2) THE JACOBIAN OF THE PROBLEM IS A FULL MATRIX
C
      DO 27 J=1,MEL
      DO 26 I=1,MEL
  26  E1(J,I)=-ED((J-1)*MEL+I)
  27  E1(J,J)=E1(J,J)+FAC
      CALL UXEGGF(MEL,MD,E1,IP,IERR,ZERO)
      IF (IERR.NE.0) GOTO 51
      IF (IFCN.NE.0) GO TO 84
C
C     THE DIFFERENTIAL EQUATION IS AUTONOMOUS
C
      TA=TAO
      DO 28 I=1,MEL
  28  ES(NE2+I)=ES(NE1+I)
      CALL UXEGGB(MEL,MD,E1,IP,ES(NE2+1),0)
      DO 29 I=1,MEL
  29  YA(I)=YAO(I)+A21*ES(NE2+I)
      CALL UYSET0(1,4)
      RETURN
    4 CONTINUE
      DO 30 I=1,MEL
  30  ES(NE3+I)=EF(I)+HC21*ES(NE2+I)
      CALL UXEGGB(MEL,MD,E1,IP,ES(NE3+1),0)
      DO 31 I=1,MEL
  31  YA(I)=YAO(I)+A31*ES(NE2+I)+A32*ES(NE3+I)
      CALL UYSET0(1,5)
      RETURN
    5 CONTINUE
      DO 32 I=1,MEL
  32  ES(NE4+I)=EF(I)+HC31*ES(NE2+I)+HC32*ES(NE3+I)
      CALL UXEGGB(MEL,MD,E1,IP,ES(NE4+1),0)
      DO 33 I=1,MEL
  33  YA(I)=YAO(I)+A41*ES(NE2+I)+A42*ES(NE3+I)+A43*ES(NE4+I)
      CALL UYSET0(1,6)
      RETURN
    6 CONTINUE
      DO 34 I=1,MEL
  34  ES(NE5+I)=EF(I)+HC41*ES(NE2+I)+HC42*ES(NE3+I)+HC43*ES(NE4+I)
      GO TO 81
C
C     THE DIFFERENTIAL EQUATION IS NON-AUTONOMOUS
C
  84  HD1=HE*D1
      HD2=HE*D2
      HD3=HE*D3
      HD4=HE*D4
      DO 35 I=1,MEL
  35  ES(NE2+I)=ES(NE1+I)+HD1*ET(I)
      CALL UXEGGB(MEL,MD,E1,IP,ES(NE2+1),0)
      DO 36 I=1,MEL
  36  YA(I)=YAO(I)+A21*ES(NE2+I)
      TA=TAO+C2*HE
      CALL UYSET0(1,7)
      RETURN
    7 CONTINUE
      DO 37 I=1,MEL
  37  ES(NE3+I)=EF(I)+HD2*ET(I)+HC21*ES(NE2+I)
      CALL UXEGGB(MEL,MD,E1,IP,ES(NE3+1),0)
      DO 38 I=1,MEL
  38  YA(I)=YAO(I)+A31*ES(NE2+I)+A32*ES(NE3+I)
      TA=TAO+C3*HE
      CALL UYSET0(1,8)
      RETURN
    8 CONTINUE
      DO 39 I=1,MEL
  39  ES(NE4+I)=EF(I)+HD3*ET(I)+HC31*ES(NE2+I)+HC32*ES(NE3+I)
      CALL UXEGGB(MEL,MD,E1,IP,ES(NE4+1),0)
      DO 40 I=1,MEL
  40  YA(I)=YAO(I)+A41*ES(NE2+I)+A42*ES(NE3+I)+A43*ES(NE4+I)
      TA=TAO+C4*HE
      CALL UYSET0(1,9)
      RETURN
    9 CONTINUE
      DO 41 I=1,MEL
  41  ES(NE5+I)=EF(I)+HD4*ET(I)+HC41*ES(NE2+I)+HC42*ES(NE3+I)
     &         +HC43*ES(NE4+I)
C
C     THE DIFFERENTIAL EQUATION IS AUTONOMOUS OR NON-AUTONOMOUS
C
  81  CALL UXEGGB(MEL,MD,E1,IP,ES(NE5+1),0)
      DO 42 I=1,MEL
  42  YA(I)=YAO(I)+A51*ES(NE2+I)+A52*ES(NE3+I)+A53*ES(NE4+I)
     &     +A54*ES(NE5+I)
      TA=TAO+HE
      CALL UYSET0(1,10)
      RETURN
   10 CONTINUE
      DO 43 I=1,MEL
  43  ES(NE6+I)=EF(I)+HC51*ES(NE2+I)+HC52*ES(NE3+I)+HC53*ES(NE4+I)
     &         +HC54*ES(NE5+I)
      CALL UXEGGB(MEL,MD,E1,IP,ES(NE6+1),0)
C
C     EMBEDDED SOLUTION
C
      DO 44 I=1,MEL
  44  YA(I)=YA(I)+ES(NE6+I)
      TA=TAO+HE
      CALL UYSET0(1,11)
      RETURN
   11 CONTINUE
      DO 45 I=1,MEL
  45  ES(NE7+I)=EF(I)+HC61*ES(NE2+I)+HC62*ES(NE3+I)+HC63*ES(NE4+I)
     &         +HC64*ES(NE5+I)+HC65*ES(NE6+I)
      CALL UXEGGB(MEL,MD,E1,IP,ES(NE7+1),0)
C
C     NEW SOLUTION
C
      DO 46 I=1,MEL
  46  YA(I)=YA(I)+ES(NE7+I)
      NSOL=NSOL+6
C
C     DENSE OUTPUT
C
      IF (MED.GE.2) THEN
      IF(HE*(TA-AT(KA)).GE.ZERO) THEN
        DO 47 I=1,MEL
        ES(NE8+I)=YAO(I)
        ES(NE10+I)=D21*ES(NE2+I)+D22*ES(NE3+I)+D23*ES(NE4+I)
     &            +D24*ES(NE5+I)+D25*ES(NE6+I)
        ES(NE11+I)=D31*ES(NE2+I)+D32*ES(NE3+I)+D33*ES(NE4+I)
     &            +D34*ES(NE5+I)+D35*ES(NE6+I)
  47    CONTINUE
      END IF
      END IF
      IF (MED1.EQ.2) THEN
        NACC=NACC+1
        DO 23 I=1,MEL
        ES(NE1+I)=ES(NE2+I)
        YAO(I)=YA(I)
  23    CONTINUE
        IF (MED2.EQ.1) CALL UXVCOP(NE,AY((KA-1)*NE+1),YAO)
        IF (MED2.EQ.2) CALL UXVMED(NE,AY((KA-1)*NE+1),YAO,YAO)
        TAO=AT(KA)
        GO TO 25
      ENDIF
C
C     ERROR ESTIMATION
C
      NSTP=NSTP+1
C
C     COMPUTE ERROR ESTIMATION
C
      ERR=ZERO
      DO 48 I=1,MEL
      SK=TOLA+TOLR*MAX(ABS(YAO(I)),ABS(YA(I)))
  48  ERR=ERR+(ES(NE7+I)/SK)**2
      ERR=DSQRT(ERR/MEL)
C
C     COMPUTATION OF HNEW
C     WE REQUIRE .2<=HNEW/H<=6.
C
      FAC=MAX(ONE/TET2,MIN(TET1,(ERR)**0.25D 0/0.9D 0))
      HNEW=HE/FAC
C
C     IS THE ERROR SMALL ENOUGH ?
C
      IF (ERR.LE.ONE) THEN
C
C       STEP IS ACCEPTED
C
        NACC=NACC+1
        DO 49 I=1,MEL
  49    YAO(I)=YA(I)
        TA=TAO+HE
        IF (MED.EQ.1) CALL UNDRS4(NA,NAL,NE,MEL,KA,AT,AY,YAO,TA,TAO,HE,
     &      IER,MED)
        IF (MED.GE.2) THEN
        IF (HE*(TA-AT(KA)).GE.ZERO) THEN
          DO 50 I=1,MEL
  50      ES(NE9+I)=YAO(I)
          CALL UNDRS4(NA,NAL,NE,MEL,KA,AT,AY,ES(NE8+1),TA,TAO,HE,IER,
     &    MED)
        END IF
        END IF
        IF (IER.EQ.-11) GO TO 62
        TAO=TA
C
C     NORMAL EXIT
C
        IF (LAST) THEN
          HE=HNEW
          IER=0
          GO TO 62
        END IF
        IF (ABS(HNEW).GT.HMAXN) HNEW=POSNEG*HMAXN
        IF (REJECT) HNEW=POSNEG*MIN(ABS(HNEW),ABS(HE))
        REJECT=.FALSE.
        HE=HNEW
        GOTO 24
      ELSE
C
C     STEP IS REJECTED
C
        REJECT=.TRUE.
        HE=HNEW
        IF (NACC.GE.1) NREJ=NREJ+1
        GOTO 25
      END IF
C
C     EXIT
C
  51  CONTINUE
      NSING=NSING+1
      IF (NSING.GE.5) GOTO 58
      HE=HE*HALF
      REJECT=.TRUE.
      GOTO 25
C
C     FAIL EXIT
C
  58  IER=-5
      TEDX='SING JAC'
      GO TO 62
  59  IER=-3
      TEDX='SMALL ST'
      GO TO 62
  60  IER=-2
      TEDX='STEP LIM'
      GO TO 62
  61  IER=-1
      TEDX='UNSTABLE'
  62  CALL UYEPI1(-1)
      RETURN
      END
