* SUBROUTINE UF0HS2                ALL SYSTEMS                 97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* NUMERICAL COMPUTATION OF THE HESSIAN MATRIX OF THE MODEL FUNCTION
* USING ITS VALUES.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  M  NUMBER OF NONZERO ELEMENTS OF THE SPARSE HESSIAN MATRIX.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XN(NF)  VECTOR OF SCALING FACTORS.
*  RO  HF(M) HESSIAN MATRIX OF THE MODEL FUNCTION.
*  II  IH(NF+1) POINTER VECTOR OF SPARSE HESSIAN MATRIX.
*  II  JH(M) INDEX VECTOR OF THE HESSIAN MATRIX.
*  RA  GO(NF)  AUXILIARY VECTOR.
*  RA  GS(NF)  AUXILIARY VECTOR.
*  RI  FF  VALUE OF THE MODEL FUNCTION.
*  RI  ETA1  PRECISION OF THE COMPUTED VALUES.
*
* COMMON DATA :
*  II  KBF  TYPE OF BOUNDS. KBF=0-BOUNDS ARE NOT USED. KBF=1-ONE SIDED
*         BOUNDS. KBF=1-TWO SIDED BOUNDS.
*  II  NORMF  SCALING SPECIFICATION. NORMF=0-NO SCALING IS PERFORMED.
*         NORMF=1-SCALING FACTORS ARE DETERMINED AUTOMATICALLY.
*         NORMF=2-SCALING FACTORS ARE SUPPLIED BY USER.
*
* STATUS VARIABLES :
*  NS,ISP,TSS
*
* SUBPROGRAMS USED :
*  S   UYPRO1  PROLOGUE.
*  S   UYEPI1  EPILOGUE.
*  S   UYSET0  STATUS DEFINITION.
*
      SUBROUTINE UF0HS2(NF,M,X,IX,XN,HF,IH,JH,GO,GS,FF,ETA1)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER NF,M,IX(NF)
      REAL*8 X(NF),XN(NF),HF(M),GO(NF),GS(NF),FF,ETA1
      REAL*8 XTEMPI,XTEMPJ,FTEMP,ETA
      INTEGER IH(NF+1),JH(M)
      INTEGER I,J,L,JSTRT,JSTOP,IVAR
      SAVE XTEMPI,XTEMPJ,FTEMP,ETA
      SAVE I,J,L,IVAR
      IF(ITERM.NE.0) GO TO 12
      IF (TSS(NS).NE.'UFHX') CALL UYPRO1('UFHX',-1)
      GO TO (1,3,6,8) ISP(NS)
    1 CONTINUE
      ETA=ETA1**(ONE/THREE)
      FTEMP=FF
      IVAR=0
    2 CONTINUE
      IVAR=IVAR+1
      IF (IVAR .GT. NF) GO TO 4
      IF (KBF.GT.0) THEN
      IF (IX(IVAR).LE.-5) GO TO 2
      ENDIF
      IF (NORMF.GT.0) THEN
      GO(IVAR)=XN(IVAR)
      ELSE
      GO(IVAR)=ONE
      ENDIF
      GO(IVAR)=ETA*MAX(ABS(X(IVAR)),GO(IVAR))*SIGN(ONE,X(IVAR))
      XTEMPI=X(IVAR)
      X(IVAR)=X(IVAR)+GO(IVAR)
      GO(IVAR)=X(IVAR)-XTEMPI
      CALL UYSET0(1,2)
      RETURN
    3 CONTINUE
      GS(IVAR)=FF
      X(IVAR)=XTEMPI
      GO TO 2
    4 I=0
    5 CONTINUE
      I=I+1
      IF(I.GT.NF) GO TO 10
      IF (KBF.GT.0) THEN
      IF (IX(I).LE.-7) GO TO 5
      ENDIF
      XTEMPI=X(I)
      X(I)=X(I)+TWO*GO(I)
      CALL UYSET0(1,3)
      RETURN
    6 CONTINUE
      JSTRT=IH(I)
      JSTOP=IH(I+1)-1
      HF(JSTRT)=((FTEMP-GS(I))+(FF-GS(I)))/(GO(I)*GO(I))
      X(I)=XTEMPI+GO(I)
      J=JSTRT
    7 CONTINUE
      J=J+1
      L=JH(J)
      IF (KBF.GT.0) THEN
      IF (IX(J).LE.-7) GO TO 7
      ENDIF
      IF(J.GT.JSTOP) GO TO 9
      XTEMPJ=X(L)
      X(L)=X(L)+GO(L)
      CALL UYSET0(1,4)
      RETURN
    8 CONTINUE
      HF(J)=((FTEMP-GS(I))+(FF-GS(L)))/(GO(I)*GO(L))
      X(L) =XTEMPJ
      GO TO 7
    9 X(I) =XTEMPI
      GO TO 5
   10 FF=FTEMP
      CALL UYEPI1(-1)
   12 CONTINUE
      RETURN
      END
