* SUBROUTINE UKLTSF                ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* INITIAL PROCESS OF CONSTRAINT TRANSFORMATION.
*
* PARAMETERS :
*  II  NF INPUT NUMBER OF COLUMNS.
*  II  NC INPUT NUMBER OF CONSTRAINTS.
*  IU  N NUMBER OF COLUMNS.
*  IU  M NUMBER OF ROWS.
*  II  MROWS DECLARED NUMBER OF ROWS.
*  II  MCOLS DECLARED NUMBER OF COLUMNS.
*  II  NZ  SIZE OF ARRAYS INR/CNR.
*  II  NZA SIZE OF ARRAYS RNR/A.
*  II  PINR(MINP) VECTOR SHOWING BEGINNINGS OF THE INPUT MATRIX ROWS.
*  II  INR(NZ) INPUT MATRIX COLUMN INDICES.
*  II  LENR(MINP) LENGTHS OF THE ROWS OF THE INPUT STRUCTURE INR/AI.
*  IO  RNR(NZA) ROW INDICES OF CONSTRAINT MATRIX.
*  IO  PRNR(N+1) POINTER VECTOR TO RNR.
*  RO  A(NZA) NUMERICAL VALUES OF THE CONSTRAINT MATRIX
*             CORRESPONDING TO RNR.
*  RI  AI(NZ) INPUT MATRIX VALUES.
*  RO  LB(N) LOWER BOUNDS ON VARIABLES.
*  II  TYPE(N) TYPES OF THE VARIABLES:
*                                     =1 FOR LOWER BOUNDED VARIABLE.
*                                     =2 FOR UPPER BOUNDED VARIABLE.
*                                     =3 FOR LOWER AND UPPER BOUNDED
*                                        VARIABLE.
*                                     =5 FOR FIXED VARIABLES.
*                                     =6 FOR FREE VARIABLES.
*  RO  LBC(N) LOWER BOUNDS ON MATRIX ROWS.
*  RO  UBC(N) UPPER BOUNDS ON MATRIX ROWS.
*  IU  TYPC(N) TYPES OF THE MATRIX ROWS ON INPUT:
*                                     =1 FOR LOWER BOUNDED ROW.
*                                     =2 FOR UPPER BOUNDED ROW.
*                                     =3 FOR LOWER AND UPPER BOUNDED ROW
*                                        WITH NON-EQUAL BOUNDS.
*                                     =5 FOR LOWER AND UPPER BOUNDED ROW
*                                        WITH EQUAL BOUNDS.
*              INFORMATION OF THE CONSTRAINTS TRANSFORMATION ON THE
*              OUTPUT.
*  IA  WN01(N) AUXILIARY VECTOR.
*  IA  WN02(N) AUXILIARY VECTOR.
*  II  PERMR(N) INPUT ROW PERMUTATION VECTOR.
*  II  IPRMC(N) INPUT COLUMN INVERSE PERMUTATION VECTOR.
*  II  MINP NUMBER OF ROWS ON INPUT.
*  IO  IER OUTPUT ERROR INFORMATION.
*
      SUBROUTINE UKLTSF(NF,NC,N,M,MROWS,MCOLS,NZ,NZA,PINR,INR,LENR,RNR,
     & PRNR,A,AI,LB,TYPE,LBC,UBC,TYPC,WN01,WN02,PERMR,IPRMC,MINP,IER)
      INTEGER NF,NC,N,M,MROWS,MCOLS,NZ,NZA,PINR(MROWS),INR(NZ),
     & LENR(MROWS),RNR(NZA),PRNR(MCOLS+1),TYPE(MCOLS),TYPC(MROWS),
     & WN01(MCOLS),WN02(MCOLS),PERMR(MROWS),IPRMC(MCOLS),MINP,IER
      REAL*8 A(NZA),AI(NZ),LB(MCOLS),LBC(MROWS),UBC(MROWS)
      INTEGER ENDF
      REAL*8 IA
      INTEGER I,J,K,L,I1,ITEMP
C
C     INITIALIZATION
C
      N=NF
      IER=0
      DO 100 I=1,NF
      WN01(I)=0
100   CONTINUE
C
C     COLUMN COUNTS EVALUATION
C
      DO 400 I=1,NC
         I1=PERMR(I)
      IF(TYPC(I).NE.3) THEN
         DO 200 J=PINR(I1),PINR(I1)+LENR(I1)-1
         IF(AI(J).EQ.0) GO TO 200
         K=IPRMC(INR(J))
         WN01(K)=WN01(K)+1
200      CONTINUE
      ELSE
         DO 300 J=PINR(I1),PINR(I1)+LENR(I1)-1
         IF(AI(J).EQ.0) GO TO 300
         K=IPRMC(INR(J))
         WN01(K)=WN01(K)+2
300      CONTINUE
      END IF
400   CONTINUE
C
C     SET POINTER VECTOR PRNR AND FILLING VECTOR WN02
C
      PRNR(1)=1
      L=1
      WN02(1)=1
      DO 500 I=2,NF
      L=L+WN01(I-1)
      PRNR(I)=L
      WN02(I)=L
500   CONTINUE
      ENDF=L+WN01(NF)
C
C NZA TOO SMALL
C
      IF(ENDF.GE.NZA-1) THEN
         IER=18
         RETURN
      END IF
      PRNR(NF+1)=ENDF
C
C     FILL THE VECTOR
C     ITEMP IS THE REAL NUMBER OF ROWS IN RNR/PRNR SCHEME
C
      ITEMP=1
      DO 1000 I1=1,NC
      I=PERMR(I1)
      IF(TYPC(I).EQ.1) THEN
         DO 600 J=PINR(I),PINR(I)+LENR(I)-1
         IF(AI(J).EQ.0) GOTO 600
         K=IPRMC(INR(J))
         L=WN02(K)
         RNR(L)=ITEMP
         A(L)=AI(J)
         WN02(K)=L+1
600      CONTINUE
         N=N+1
         RNR(ENDF)=ITEMP
         A(ENDF)=-1.0D 0
         PRNR(N)=ENDF
         ENDF=ENDF+1
         TYPE(N)=1
         LB(N)=0.0D 0
         ITEMP=ITEMP+1
      ELSEIF(TYPC(I).EQ.2) THEN
         DO 700 J=PINR(I),PINR(I)+LENR(I)-1
         IF(AI(J).EQ.0) GO TO 700
         K=IPRMC(INR(J))
         L=WN02(K)
         RNR(L)=ITEMP
         A(L)=AI(J)
         WN02(K)=L+1
700      CONTINUE
         N=N+1
         RNR(ENDF)=ITEMP
         A(ENDF)=1.0D 0
         PRNR(N)=ENDF
         ENDF=ENDF+1
         TYPE(N)=1
         LB(N)=0.0D 0
         ITEMP=ITEMP+1
      ELSEIF(TYPC(I).EQ.5) THEN
         DO 800 J=PINR(I),PINR(I)+LENR(I)-1
         IF(AI(J).EQ.0) GO TO 800
         K=IPRMC(INR(J))
         L=WN02(K)
         RNR(L)=ITEMP
         A(L)=AI(J)
         WN02(K)=L+1
800      CONTINUE
         ITEMP=ITEMP+1
      ELSEIF(TYPC(I).EQ.3) THEN
         DO 900 J=PINR(I),PINR(I)+LENR(I)-1
         IF(AI(J).EQ.0) GO TO 900
         K=IPRMC(INR(J))
         L=WN02(K)
         RNR(L)=ITEMP
         RNR(L+1)=ITEMP+1
         IA=AI(J)
         A(L)=IA
         A(L+1)=IA
         WN02(K)=L+2
900      CONTINUE
         N=N+2
         PRNR(N-1)=ENDF
         RNR(ENDF)=ITEMP
         A(ENDF)=-1.0D 0
         PRNR(N)=ENDF+1
         RNR(ENDF+1)=ITEMP+1
         A(ENDF+1)=1.0D 0
         ENDF=ENDF+2
         ITEMP=ITEMP+2
         TYPE(N-1)=1
         LB(N-1)=0.0D 0
         TYPE(N)=1
         LB(N)=0.0D 0
      END IF
1000  CONTINUE
C
C     NUMBER OF EQUATIONS OF THE TRANSFORMED PROBLEM
C
      M=ITEMP-1
C
C     NUMBER OF SLACK VARIABLES
C
      MINP=ENDF-PRNR(NF+1)
C
C     END OF PRNR IS SET
C
      PRNR(N+1)=ENDF
C
C SHRINK VECTORS OF BOUNDS
C
      DO 1050 I1=1,NC
      I=PERMR(I1)
      WN02(I1)=TYPC(I)
      LBC(I1)=LBC(I)
      UBC(I1)=UBC(I)
 1050 CONTINUE
C
C UPDATE OF LBC AND TYPC
C WN01 IS NOW USED TO GIVE INVERSE ROW MAPPING
C
      J=M
      DO 1100 I=NC,1,-1
      K=WN02(I)
C      TEMP1=LBC(I)
C      TEMP2=UBC(I)
      I1=PERMR(I)
      IF(K.EQ.1) THEN
C         WN02(J)=K
         LBC(J)=LBC(I)
         WN01(J)=I1
         TYPC(I1)=J
         J=J-1
      ELSEIF(K.EQ.2) THEN
C         WN02(J)=K
         LBC(J)=UBC(I)
         WN01(J)=I1
         TYPC(I1)=J
         J=J-1
      ELSEIF(K.EQ.3) THEN
C         WN02(J)=2
C         WN02(J-1)=1
         LBC(J)=UBC(I)
         LBC(J-1)=LBC(I)
         WN01(J)=I1
         WN01(J-1)=I1
         TYPC(I1)=J-1
         J=J-2
      ELSEIF(K.EQ.5) THEN
C         WN02(J)=-I
         LBC(J)=LBC(I)
         WN01(J)=I1
         TYPC(I1)=J
         J=J-1
      END IF
1100  CONTINUE
      RETURN
      END
