* SUBROUTINE UKQGLD                 ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* VECTOR OF THE RESTRICED STEP IS USED TO COMPUTE THE COMPLETE
* N-COMPONENT DIRECTION VECTOR S.
*
* PARAMETERS :
*  II  N  NUMBER OF COLUMNS OF THE SPARSE MATRIX
*  II  M  NUMBER OF ROWS OF THE SPARSE MATRIX.
*  II  MROWS  DECLARED NUMBER OF ROWS.
*  II  MCOLS  DECLARED NUMBER OF VARIABLES.
*  II  NZ SIZE OF THE INDEX ARRAY CNR
*  II  NZA SIZE OF THE INDEX ARRAY RNR.
*  II  NAU SIZE OF THE ARRAY THAT STORES THE NUMERICAL ELEMENTS OF
*          THE PROCESSED MATRIX.
*  II  NUP  LENGTH OF THE ARRAYS LI,LJ,CM.
*  II  NEL  NUMBER OF TRIPLES IN ARRAYS (LI,LJ,CM).
*  II  PTL(MROWS) POINTER VECTOR OF THE COLUMNS OF THE L-FACTOR.
*  II  LENL(MROWS) LENGTHS OF THE COLUMNS OF THE FACTOR L.
*  II  CNR(NZ) CONTAINS THE INDICES OF THE FACTORS.
*  II  PTR(MROWS) POINTER VECTOR OF THE ROWS OF THE FACTOR U.
*  II  LENR(MROWS) LENGTHS OF THE ROWS OF THE FACTOR U.
*  II  RNR(NZA) COLUMNS OF THE ORIGINAL MATRIX A.
*  II  PRNR(MCOLS+1) INDEX VECTOR FOR THE ARRAY RNR.
*  II  ORDC(MROWS) COLUMN PERMUTATION.
*  II  ORDR(MROWS) ROW PERMUTATION.
*  II  ORDL(MROWS) ROW PERMUTATION OF THE ORIGINAL FACTOR L.
*  II  LI(NUP) ROW INDICES OF THE L-UPDATES.
*  II  LJ(NUP) COLUMN INDICES OF THE L-UPDATES.
*  RI  CM(NUP) UPDATE MULTIPLIERS.
*  RI  AU(NZ) NUMERICAL VALUES OF THE FACTORS.
*  RI  A(NZA) NUMERICAL VALUES OF THE MATRIX.
*  RA  WR01(MCOLS) AUXILIARY VECTOR.
*  RA  WR02(MCOLS) AUXILIARY VECTOR.
*  II  BCOL(MROWS) PERMUTATION ARRAY: FIRST M COMPONENTS FORM BASIS,
*              NEXT LINP-M FORM SUPERBASIS AND REST CORRESPOND TO
*              THE NON-BASIC VARIABLES.
*  RU  S(MCOLS) ON INPUT :RESTRICTED DIRECTION IN THE SUPERBASICS' SPACE
*           S(1,LINP-M),ON OUTPUT: DIRECTION VECTOR S=(SB,SS,0).
*  II  LINP NUMBER OF BASICS AND SUPERBASICS.
*
      SUBROUTINE UKQGLD(N,M,MROWS,MCOLS,NZ,NZA,NAU,NUP,NEL,PTL,LENL,
     & CNR,PTR,LENR,RNR,PRNR,ORDC,ORDR,ORDL,LI,LJ,CM,AU,A,WR01,WR02,
     & BCOL,S,LINP)
      INTEGER N,M,MROWS,MCOLS,NZ,NZA,NAU,NUP,NEL,PTL(MROWS),LENL(MROWS),
     & CNR(NZ),PTR(MROWS),LENR(MROWS),RNR(NZA),PRNR(MCOLS+1),
     & ORDC(MROWS),ORDR(MROWS),ORDL(MROWS),LI(NUP),LJ(NUP),BCOL(MCOLS),
     & LINP
      REAL*8 CM(NUP),AU(NAU),A(NZA),WR01(MCOLS),WR02(MCOLS),S(MCOLS)
      INTEGER I,J
C
C     COMPUTE DIRECTION VECTOR S USING THE INFORMATION GIVEN IN
C     THE OUTPUT OF THE RESTRICTED STEP.
C
C     FIRST: COMPUTE THE PART OF THE BASIC VARIABLES
C
      CALL UKQGLG(M,MROWS,MCOLS,NZ,NZA,NAU,NUP,NEL,PTL,LENL,CNR,PTR,
     & LENR,RNR,PRNR,ORDC,ORDR,ORDL,LI,LJ,CM,AU,A,WR01,WR02,BCOL,S,LINP)
C
C     INITIALIZE S
C
      DO 100 I=1,N
      S(I)=0.0D 0
100   CONTINUE
C
C     REWRITE DIRECTION VECTOR FROM WR01 INTO S
C     - BOTH THE BASIC AND SUPERBASIC PARTS.
C
      DO 200 J=1,LINP
      I=BCOL(J)
      S(I)=WR01(J)
200   CONTINUE
      RETURN
      END
