* SUBROUTINE UKSGLC                 ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* FIND THE INPUT COLUMN INEW FOR SIMPLEX CYCLE.
*
* PARAMETERS :
*  II  N NUMBER OF COLUMNS.
*  II  MCOLS DECLARED NUMBER OF COLUMNS.
*  II  NZA  SIZE OF ARRAYS A/RNR.
*  II  PRNR(MCOLS+1) POINTER VECTOR TO RNR.
*  II  RNR(NZA) COLUMN INDICES OF THE CONSTRAINT MATRIX.
*  RI  A(NZA) NUMERICAL VALUES OF THE CONSTRAINT MATRIX.
*  RI  C(MCOLS) COST VECTOR FOR THIS PHASE.
*  II  TYPE(MCOLS) TYPE OF THE VARIABLES:
*                                    =1 FOR NONBASIC LOWER BOUNDED
*                                    =2 FOR NONBASIC UPPER BOUNDED
*                                    =3 FOR NONBASIC LOWER AND UPPER
*                     BOUNDED VARIABLE SET TO ITS LOWER BOUND
*                                    =4 FOR NONBASIC LOWER AND UPPER
*                                    =5 FOR NONBASIC FIXED
*                                    =6 FOR NONBASIC UNBOUNDED
*                     BOUNDED VARIABLE SET TO ITS UPPER BOUND
*                                    =-1 FOR BASIC LOWER BOUNDED
*                                    =-2 FOR BASIC UPPER BOUNDED
*                                    =-3 FOR BASIC LOWER AND UPPER
*                                    =-6 FOR BASIC UNBOUNDED
*                     BOUNDED VARIABLE
*  II  LAM(MCOLS) VECTOR OF LAGRANGE MULTIPLIERS.
*  IU  SCOL NUMBER OF COLUMN WHERE THE SEARCH IS COMMENCED.
*  II  PHASE LP PHASE.
*  RI  EPS9  BOUND FOR NEGLECTING SMALL TOLERANCES.
*  II  MLP1  COUNT OF DEGENERATE PIVOTS AFTER THE LAST NONDEGENERATE
*            ONE.
*  IO  INEW INPUT COLUMN.
*  II  IEXT  TYPE UF EXTREMUM. IEXT > 0 - MAXIMUM. IEXT <= 0 - MINIMUM.
*  II  IER OUTPUT VARIABLE:=0 WHEN COLUMN IS FOUND
*                          =1 WHEN SOLUTION IS OPTIMAL.
*
      SUBROUTINE UKSGLC(N,MCOLS,NZA,PRNR,RNR,A,C,TYPE,LAM,SCOL,
     & PHASE,EPS9,MLP1,INEW,IEXT,IER)
      INTEGER N,MCOLS,NZA,PRNR(MCOLS+1),RNR(NZA),
     & TYPE(MCOLS),SCOL,PHASE,MLP1,INEW,IEXT,IER
      REAL*8 A(NZA),C(MCOLS),LAM(MCOLS),EPS9
      INTEGER SMAX,BLAND,IND,I,J,JSTRT,JSTOP,TYPV
      INTEGER NLOCAL
      REAL*8 TEMP,MULT
      INTEGER K,OPFL,IEXT1
      CHARACTER CH
      PARAMETER (OPFL=30)
      IF(IEXT.EQ.0) THEN
        IEXT1=1
      ELSE
        IEXT1=-1
      ENDIF
      IF(PHASE.EQ.1) IEXT1=1
      EPS9=1.0D-13
C
C     DEFAULT FLAG IS ZERO - NON-OPTIMALITY
C
      IER=0
C
C     IND IS EQUAL TO THE INDEX OF INPUT COLUMN - IF IT EXISTS
C     - ELSE IS ZERO. THAT IS TESTED AT THE END OF THE SUBROUTINE.
C
      IND=0
C
C     MODE OF PIVOT RULE - AT THE BEGIINING IT IS THE PARTIAL
C     PIVOTING RULE WITH SMAX PIVOT FROM THE STARTING COLUMN SCOL.
C     WHEN FAILURE IS FACED OR MLP1 IS TOO HIGH - BLAND PIVOTING
C     RULE IS USED.
C
      BLAND=0
C
C     TEMPORARY VALUE OF THE RELATIVE COST.
C
      TEMP=0.0D 0
C
C     LOCAL N - WITH ARTIFICIALS.
C
      NLOCAL=N
C
C     IF NUMBER OF ZERO PIVOTS INCREASED OVER SPECIFIED RATIO
C     THEN APPLY BLAND PIVOTING RULE
C
      IF(MLP1.GT.(N*OPFL/100+1)) GO TO 700
C
C     ELSE SET THE NUMBER OF SEARCHED COLUMNS SMAX
C
C      SMAX=MIN(20,(OPFL*NLOCAL)/100+1,NLOCAL)
      SMAX=NLOCAL
100   DO 500 I=1,SMAX
C
C     INITIALIZE MULT
C
      MULT=0
C
C     DISCERN BASIC AND NONBASIC COLUMNS
C
200   TYPV=TYPE(SCOL)
      IF(TYPV.LE.0) THEN
                    SCOL=SCOL+1
                    IF(SCOL.GT.NLOCAL) SCOL=1
                    GO TO 200
      END IF
C
C    TYPV.GT.0 : NONBASIC VARIABLE
C
      JSTRT=PRNR(SCOL)
      JSTOP=PRNR(SCOL+1)-1
      DO 300 J=JSTRT,JSTOP
      K=RNR(J)
C
C     MULT IS RELATIVE COST
C
      MULT=MULT+A(J)*LAM(K)
300   CONTINUE
      MULT=IEXT1*C(SCOL)-MULT
C
C     NONBASIC AT UPPER BOUND
C
      IF(TYPV.EQ.2.OR.TYPV.EQ.4.OR.TYPV.EQ.6) THEN
          IF(MULT.GT.(TEMP+EPS9))      THEN
                                       TEMP=MULT
                                       IND=SCOL
                                       CH='U'
                                       GO TO 400
          END IF
       ENDIF
C
C     NONBASIC AT LOWER BOUND
C
      IF(TYPV.EQ.1.OR.TYPV.EQ.3.OR.TYPV.EQ.6) THEN
          IF(MULT.LT.-(TEMP+EPS9))     THEN
                                       TEMP=-MULT
                                       IND=SCOL
                                       CH='L'
                                       GO TO 400
          END IF
      END IF
      GO TO 450
C
C     TESTING OF PIVOTING RULE
C
400   IF(BLAND.EQ.1) GO TO 600
450   SCOL=SCOL+1
      IF(SCOL.GT.NLOCAL) SCOL=1
C
C     END OF CYCLE
C
500   CONTINUE
C
C     JUMP LABEL
C
600   CONTINUE
C
C     TEST WHETHER ANY IN-COLUMN WAS FOUND
C
      IF (IND.NE.0) THEN
                    INEW=IND
         IF(TYPE(IND).EQ.6) THEN
           IF(CH.EQ.'U') THEN
             TYPE(IND)=62
           ELSE
             TYPE(IND)=61
           END IF
         END IF
                       RETURN
      END IF
      IF(SMAX.EQ.NLOCAL) THEN
                         IER=1
                         RETURN
      END IF
C
C     BLAND BLOCK
C
700   CONTINUE
      SCOL=1
      SMAX=NLOCAL
      BLAND=1
      GO TO 100
      END
