* SUBROUTINE ULADF0               ALL SYSTEMS                06/12/01
C PORTABILITY : ALL SYSTEMS
C 06/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* TRIANGULAR DECOMPOSITION OF KERNEL OF THE GENERAL PROJECTION
* IS UPDATED AFTER FUNCTION OR CONSTRAINT ADDITION.
*
* PARAMETERS :
*  II  NF  DECLARED NUMBER OF VARIABLES.
*  IU  N  ACTUAL NUMBER OF VARIABLES.
*  II  NA  NUMBER OF CURRENT LINEAR APPROXIMATED FUNCTIONS.
*  II  NB  NUMBER OF DECLARED CONSTRAINTS OR APPROXIMATED FUNCTIONS.
*  II  NC  NUMBER OF LINEARIZED CONSTRAINTS.
*  II  IA(2*NA)  VECTOR CONTAINING TYPES OF DEVIATIONS.
*  IU  IAA(NF+1)  VECTOR CONTAINING INDICES OF ACTIVE FUNCTIONS.
*  RO  AG(NF*NA)  MATRIX WHOSE COLUMNS ARE GRADIENTS OF THE LINEAR
*         APPROXIMATED FUNCTIONS.
*  RI  AR((NF+1)*(NF+2)/2)  TRIANGULAR DECOMPOSITION OF KERNEL OF
*         THE ORTHOGONAL PROJECTION.
*  RI  CG(NF*NC)  MATRIX WHOSE COLUMNS ARE NORMALS OF THE LINEAR
*         CONSTRAINTS.
*  RA  S(NF)  AUXILIARY VECTOR.
*  RI  EPS7  TOLERANCE FOR LINEAR INDEPENDENCE OF CONSTRAINTS.
*  RO  GMAX  MAXIMUM ABSOLUTE VALUE OF A PARTIAL DERIVATIVE.
*  RO  UMAX  MAXIMUM ABSOLUTE VALUE OF A NEGATIVE LAGRANGE MULTIPLIER.
*  RO  E  AUXILIARY VARIABLE.
*  RI  T  AUXILIARY VARIABLE.
*  II  INEW  INDEX OF THE NEW ACTIVE CONSTRAINT.
*  II  KNEW  SIGNUM OF THE NEW ACTIVE GRADIENT.
*  IU  NADD  NUMBER OF CONSTRAINT ADDITIONS.
*  IO  IER  ERROR INDICATOR.
*
* SUBPROGRAMS USED :
*  S   UXDPRB  BACK SUBSTITUTION AFTER CHOLESKI DECOMPOSITION.
*  S   UXVCOP  COPYING OF A VECTOR.
*  RF  UXVDOT  DOT PRODUCT OF TWO VECTORS.
*  S   UXVSET  INITIATION OF A VECTOR.
*
      SUBROUTINE ULADF0(NF,N,NA,NB,NC,IA,IAA,AG,AR,CG,S,EPS7,GMAX,UMAX,
     & E,T,INEW,KNEW,NADD,IER)
      INTEGER NF,N,NA,NB,NC,IA(2*NA),IAA(NF+1),INEW,KNEW,NADD,IER
      REAL*8 AG(NF*NA),AR((NF+1)*(NF+2)/2),CG(NF*NB),S(NF),EPS7,GMAX,
     & UMAX,E,T
      REAL*8 POM,TEMP,UXVDOT
      INTEGER NAA,NAR,JAG,J,K,L
      E=0.0D 0
      IF (INEW.GT.NC) E=SIGN(1,KNEW)
      IER=0
      IF (N.LT.0) IER=2
      IF (INEW.EQ.0) IER=3
      IF (IER.NE.0) RETURN
      NAA=NF-N
      NAR=NAA*(NAA+1)/2
      IF (INEW.GT.NC) THEN
      K=INEW-NC
      IF (K.GT.NA) K=K-NA
      JAG=(K-1)*NF+1
      CALL UXVCOP(NF,AG(JAG),S)
      GMAX=UXVDOT(NF,AG(JAG),S)+T
      ELSE IF (INEW.GT.0) THEN
      JAG=(INEW-1)*NF+1
      CALL UXVCOP(NF,CG(JAG),S)
      GMAX=UXVDOT(NF,CG(JAG),S)
      ELSE
      K=-INEW
      CALL UXVSET(NF,0.0D 0,S)
      S(K)=1.0D 0
      GMAX=S(K)
      ENDIF
      POM=T*E
      DO 1 J=1,NAA
      L=IAA(J)
      IF (L.GT.NC) THEN
      L=L-NC
      K=L
      IF (L.GT.NA) L=L-NA
      AR(NAR+J)=UXVDOT(NF,AG((L-1)*NF+1),S)
      IF (INEW.GT.NC) THEN
      TEMP=POM
      IF (IA(K).EQ.-2.OR.IA(K).EQ.-4) TEMP=-TEMP
      AR(NAR+J)=AR(NAR+J)+TEMP
      ENDIF
      ELSE IF (L.GT.0) THEN
      AR(NAR+J)=UXVDOT(NF,CG((L-1)*NF+1),S)
      ELSE
      L=-L
      AR(NAR+J)=S(L)
      ENDIF
    1 CONTINUE
      IF (NAA.EQ.0) THEN
      UMAX=GMAX
      ELSE
      CALL UXDPRB(NAA,AR,AR(NAR+1),1)
      UMAX=GMAX-UXVDOT(NAA,AR(NAR+1),AR(NAR+1))
      ENDIF
      IF (UMAX.LE.EPS7*GMAX) THEN
      IER=1
      RETURN
      ELSE
      N=N-1
      NAA=NAA+1
      NAR=NAR+NAA
      IAA(NAA)=INEW
      AR(NAR)=SQRT(UMAX)
      NADD=NADD+1
      ENDIF
      RETURN
      END
