* SUBROUTINE ULADF1               ALL SYSTEMS                06/12/01
C PORTABILITY : ALL SYSTEMS
C 06/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* TRIANGULAR DECOMPOSITION OF KERNEL OF THE GENERAL PROJECTION
* IS UPDATED AFTER FUNCTION OR CONSTRAINT ADDITION.
*
* PARAMETERS :
*  II  NF  DECLARED NUMBER OF VARIABLES.
*  IU  N  ACTUAL NUMBER OF VARIABLES.
*  II  NA  NUMBER OF CURRENT LINEAR APPROXIMATED FUNCTIONS.
*  II  NB  NUMBER OF DECLARED CONSTRAINTS OR APPROXIMATED FUNCTIONS.
*  II  NC  NUMBER OF LINEARIZED CONSTRAINTS.
*  II  IA(2*NA)  VECTOR CONTAINING TYPES OF DEVIATIONS.
*  IU  IAA(NF+1)  VECTOR CONTAINING INDICES OF ACTIVE FUNCTIONS.
*  RO  AG(NF*NA)  MATRIX WHOSE COLUMNS ARE GRADIENTS OF THE LINEAR
*          APPROXIMATED FUNCTIONS.
*  RI  AR((NF+1)*(NF+2)/2)  TRIANGULAR DECOMPOSITION OF KERNEL OF
*        THE ORTHOGONAL PROJECTION.
*  RI  CG(NF*NC)  MATRIX WHOSE COLUMNS ARE NORMALS OF THE LINEAR
*         CONSTRAINTS.
*  RU  H(NF*(NF+1)/2)  TRIANGULAR DECOMPOSITION OR INVERSION OF THE
*         HESSIAN MATRIX APPROXIMATION.
*  RA  S(NF+1)  AUXILIARY VECTOR.
*  RO  G(NF+1)  VECTOR USED IN THE DUAL RANGE SPACE QUADRATIC
*        PROGRAMMING METHOD.
*  RI  EPS7  TOLERANCE FOR LINEAR INDEPENDENCE OF CONSTRAINTS.
*  RO  GMAX  MAXIMUM ABSOLUTE VALUE OF A PARTIAL DERIVATIVE.
*  RO  UMAX  MAXIMUM ABSOLUTE VALUE OF A NEGATIVE LAGRANGE MULTIPLIER.
*  RO  E  AUXILIARY VARIABLE.
*  RI  T  AUXILIARY VARIABLE.
*  IU  IDECF  DECOMPOSITION INDICATOR. IDECF=0-NO DECOMPOSITION.
*         IDECF=1-GILL-MURRAY DECOMPOSITION. IDECF=9-INVERSION.
*         IDECF=10-DIAGONAL MATRIX.
*  II  INEW  INDEX OF THE NEW ACTIVE CONSTRAINT.
*  II  KNEW  SIGNUM OF THE NEW ACTIVE GRADIENT.
*  IU  NADD  NUMBER OF CONSTRAINT ADDITIONS.
*  IO  IER  ERROR INDICATOR.
*  II  JOB  SPECIFICATION OF COMPUTATION. OUTPUT VECTOR G IS NOT OR IS
*         COMPUTED IN CASE WHEN N.LE.0 IF JOB=0 OR JOB=1 RESPECTIVELY.
*
* SUBPROGRAMS USED :
*  S   UXDPGB  BACK SUBSTITUTION AFTER GILL-MURRAY DECOMPOSITION.
*  S   UXDPRB  BACK SUBSTITUTION AFTER CHOLESKI DECOMPOSITION.
*  S   UXDSMM  MATRIX-VECTOR PRODUCT.
*  S   UXDSMV  COPYING OF A ROW OF DENSE SYMMETRIC MATRIX.
*  S   UXVCOP  COPYING OF A VECTOR.
*  RF  UXVDOT  DOT PRODUCT OF TWO VECTORS.
*
      SUBROUTINE ULADF1(NF,N,NA,NB,NC,IA,IAA,AG,AR,CG,H,S,G,EPS7,GMAX,
     & UMAX,E,T,IDECF,INEW,KNEW,NADD,IER,JOB)
      INTEGER NF,N,NA,NB,NC,IA(2*NA),IAA(NF+1),IDECF,INEW,KNEW,NADD,
     & IER,JOB
      REAL*8 AG(NF*NA),AR((NF+1)*(NF+2)/2),CG(NF*NB),H(*),S(NF+1),
     & G(NF+1),EPS7,GMAX,UMAX,E,T
      REAL*8 POM,TEMP,UXVDOT
      INTEGER NAA,NAR,JAG,J,K,L
      E=0.0D 0
      IF (INEW.GT.NC) E=SIGN(1,KNEW)
      IER=0
      IF (JOB.EQ.0.AND.N.LT.0) IER=2
      IF (INEW.EQ.0) IER=3
      IF (IDECF.GE.2.AND.IDECF.LE.8) IER=-2
      IF (IER.NE.0) RETURN
      NAA=NF-N
      NAR=NAA*(NAA+1)/2
      IF (INEW.GT.NC) THEN
      K=INEW-NC
      IF (K.GT.NA) K=K-NA
      JAG=(K-1)*NF+1
      IF (IDECF.EQ.1) THEN
      CALL UXVCOP(NF,AG(JAG),S)
      CALL UXDPGB(NF,H,S,0)
      ELSE IF (IDECF.EQ.9) THEN
      CALL UXDSMM(NF,H,AG(JAG),S)
      ELSE
      CALL UXVCOP(NF,AG(JAG),S)
      CALL UXVMUL(NF,H,S,S,-1)
      ENDIF
      GMAX=UXVDOT(NF,AG(JAG),S)+T
      ELSE IF (INEW.GT.0) THEN
      JAG=(INEW-1)*NF+1
      IF (IDECF.EQ.1) THEN
      CALL UXVCOP(NF,CG(JAG),S)
      CALL UXDPGB(NF,H,S,0)
      ELSE IF (IDECF.EQ.9) THEN
      CALL UXDSMM(NF,H,CG(JAG),S)
      ELSE
      CALL UXVCOP(NF,CG(JAG),S)
      CALL UXVMUL(NF,H,S,S,-1)
      ENDIF
      GMAX=UXVDOT(NF,CG(JAG),S)
      ELSE
      K=-INEW
      IF (IDECF.EQ.1) THEN
      CALL UXVSET(NF,0.0D 0,S)
      S(K)=1.0D 0
      CALL UXDPGB(NF,H,S,0)
      ELSE IF (IDECF.EQ.9) THEN
      CALL UXDSMV(NF,H,S,K)
      ELSE
      CALL UXVSET(NF,0.0D 0,S)
      S(K)=1.0D 0/H(K)
      ENDIF
      GMAX=S(K)
      ENDIF
      IF (NAA.GT.0) THEN
      POM=T*E
      DO 1 J=1,NAA
      L=IAA(J)
      IF (L.GT.NC) THEN
      L=L-NC
      K=L
      IF (L.GT.NA) L=L-NA
      G(J)=UXVDOT(NF,AG((L-1)*NF+1),S)
      IF (INEW.GT.NC) THEN
      TEMP=POM
      IF (IA(K).EQ.-2.OR.IA(K).EQ.-4) TEMP=-TEMP
      G(J)=G(J)+TEMP
      ENDIF
      ELSE IF (L.GT.0) THEN
      G(J)=UXVDOT(NF,CG((L-1)*NF+1),S)
      ELSE
      L=-L
      G(J)=S(L)
      ENDIF
    1 CONTINUE
      ENDIF
      IF (N.LT.0) THEN
      CALL UXDPRB(NAA,AR,G,1)
      UMAX=0.0D 0
      IER=2
      RETURN
      ELSE IF (NAA.EQ.0) THEN
      UMAX=GMAX
      ELSE
      CALL UXDPRB(NAA,AR,G,1)
      UMAX=GMAX-UXVDOT(NAA,G,G)
      CALL UXVCOP(NAA,G,AR(NAR+1))
      ENDIF
      IF (UMAX.LE.EPS7*GMAX) THEN
      IER=1
      RETURN
      ELSE
      NAA=NAA+1
      NAR=NAR+NAA
      IAA(NAA)=INEW
      AR(NAR)=SQRT(UMAX)
      IF (JOB.EQ.0) THEN
      N=N-1
      NADD=NADD+1
      ENDIF
      ENDIF
      RETURN
      END
