* SUBROUTINE ULADR0               ALL SYSTEMS                97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* TRIANGULAR DECOMPOSITION OF KERNEL OF THE ORTHOGONAL PROJECTION
* IS UPDATED AFTER CONSTRAINT ADDITION.
*
* PARAMETERS :
*  II  NF  DECLARED NUMBER OF VARIABLES.
*  IU  N  ACTUAL NUMBER OF VARIABLES.
*  II  NC  NUMBER OF LINEARIZED CONSTRAINTS.
*  IU  ICA(NF)  VECTOR CONTAINING INDICES OF ACTIVE CONSTRAINTS.
*  RI  CG(NF*NC)  MATRIX WHOSE COLUMNS ARE NORMALS OF THE LINEAR
*         CONSTRAINTS.
*  RI  CR(NF*(NF+1)/2)  TRIANGULAR DECOMPOSITION OF KERNEL OF THE
*         ORTHOGONAL PROJECTION.
*  RA  S(NF)  AUXILIARY VECTOR.
*  RI  EPS7  TOLERANCE FOR LINEAR INDEPENDENCE OF CONSTRAINTS.
*  RO  GMAX  MAXIMUM ABSOLUTE VALUE OF A PARTIAL DERIVATIVE.
*  RO  UMAX  MAXIMUM ABSOLUTE VALUE OF A NEGATIVE LAGRANGE MULTIPLIER.
*  II  INEW  INDEX OF THE NEW ACTIVE CONSTRAINT.
*  IU  NADD  NUMBER OF CONSTRAINT ADDITIONS.
*  IO  IER  ERROR INDICATOR.
*
* SUBPROGRAMS USED :
*  S   UXSPRB  SPARSE BACK SUBSTITUTION.
*  S   UXVCOP  COPYING OF A VECTOR.
*  RF  UXVDOT  DOT PRODUCT OF TWO VECTORS.
*
      SUBROUTINE ULADR0(NF,N,NC,ICA,CG,CR,S,EPS7,GMAX,UMAX,INEW,NADD,
     & IER)
      INTEGER NF,N,NC,ICA(NF),INEW,NADD,IER
      REAL*8 CG(NF*NC),CR(NF*(NF+1)/2),S(NF),EPS7,GMAX,UMAX
      REAL*8 UXVDOT
      INTEGER NCA,NCR,I,J,K,L
      IER=0
      IF (N.LE.0) IER=2
      IF (INEW.EQ.0) IER=3
      IF (IER.NE.0) RETURN
      NCA=NF-N
      NCR=NCA*(NCA+1)/2
      IF (INEW.GT.0) THEN
      CALL UXVCOP(NF,CG((INEW-1)*NF+1),S)
      GMAX=UXVDOT(NF,CG((INEW-1)*NF+1),S)
      DO 1 J=1,NCA
      L=ICA(J)
      IF (L.GT.0) THEN
      CR(NCR+J)=UXVDOT(NF,CG((L-1)*NF+1),S)
      ELSE
      I=-L
      CR(NCR+J)=S(I)
      ENDIF
    1 CONTINUE
      ELSE
      K=-INEW
      GMAX=1.0D 0
      DO 2 J=1,NCA
      L=ICA(J)
      IF (L.GT.0) THEN
      CR(NCR+J)=CG((L-1)*NF+K)*GMAX
      ELSE
      CR(NCR+J)=0.0D 0
      ENDIF
    2 CONTINUE
      ENDIF
      IF (NCA.EQ.0) THEN
      UMAX=GMAX
      ELSE
      CALL UXDPRB(NCA,CR,CR(NCR+1),1)
      UMAX=GMAX-UXVDOT(NCA,CR(NCR+1),CR(NCR+1))
      ENDIF
      IF (UMAX.LE.EPS7*GMAX) THEN
      IER=1
      RETURN
      ELSE
      N=N-1
      NCA=NCA+1
      NCR=NCR+NCA
      ICA(NCA)=INEW
      CR(NCR)=SQRT(UMAX)
      NADD=NADD+1
      ENDIF
      RETURN
      END
