* SUBROUTINE UMVLAG               ALL SYSTEMS                97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* GRADIENT OF THE OBJECTIVE FUNCTION IS PREMULTIPLIED BY TRANSPOSE
* OF THE MATRIX WHOSE COLUMNS ARE NORMALS OF CURRENT ACTIVE CONSTRAINTS
* AND GRADIENTS OF CURRENT ACTIVE FUNCTIONS.
*
* PARAMETERS :
*  II  NF  DECLARED NUMBER OF VARIABLES.
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  II  NA  NUMBER OF CURRENT LINEAR APPROXIMATED FUNCTIONS.
*  II  NB  NUMBER OF DECLARED CONSTRAINTS OR APPROXIMATED FUNCTIONS.
*  II  NC  NUMBER OF LINEARIZED CONSTRAINTS.
*  II  MA  NUMBER OF ELEMENTS IN THE FIELD AG.
*  II  MC  NUMBER OF ELEMENTS IN THE FIELD CG.
*  II  IAA(NF)  VECTOR CONTAINING INDICES OF ACTIVE FUNCTIONS.
*  RI  AG(MA)  SPARSE APPROXIMATED FUNCTION JACOBIAN MATRIX.
*  II  IAG(NA+1)  POSITION OF THE FIRST RORWS ELEMENTS IN THE FIELD AG.
*  II  JAG(MA) COLUMN INDICES OF ELEMENTS IN THE FIELD AG.
*  RI  CG(MC)  SPARSE CONSTRAINT JACOBIAN MATRIX.
*  II  ICG(NC+1)  POSITION OF THE FIRST RORWS ELEMENTS IN THE FIELD CG.
*  II  JCG(MC) COLUMN INDICES OF ELEMENTS IN THE FIELD CG.
*  RI  G(NF)  GRADIENT OF THE OBJECTIVE FUNCTION.
*  RO  GN(NF+1)  OUTPUT VECTOR.
*
* SUBPROGRAMS USED :
*  RF  UXWDOT  DOT PRODUCT OF TWO SPARSE VECTORS.
*
      SUBROUTINE UMVLAG(NF,N,NA,NB,NC,MA,MC,IAA,AG,IAG,JAG,CG,ICG,JCG,
     & G,GN)
      INTEGER NF,N,NA,NB,NC,MA,MC,IAA(NF),IAG(NA+1),JAG(MA),ICG(NB+1),
     & JCG(MC)
      REAL*8 AG(MA),CG(MC),G(NF),GN(NF)
      REAL*8 UXWDOT
      INTEGER NAA,J,K,L
      NAA=NF-N
      DO 1 J=1,NAA
      L=IAA(J)
      IF (L.GT.NC) THEN
      L=L-NC
      K=IAG(L)
      L=IAG(L+1)-K
      GN(J)=UXWDOT(L,NF,JAG(K),AG(K),G,2)
      ELSE IF (L.GT.0) THEN
      K=ICG(L)
      L=ICG(L+1)-K
      GN(J)=UXWDOT(L,NF,JCG(K),CG(K),G,2)
      ELSE
      L=-L
      GN(J)=G(L)
      ENDIF
    1 CONTINUE
      RETURN
      END
