* SUBROUTINE UNINT5                ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* EXTRAPOLATION OR INTERPOLATION FOR LINE SEARCH USED IN MINIMAX
* APPROXIMATION.
*
* PARAMETERS :
*  II  NA  NUMBER OF APPROXIMATED FUNCTIONS.
*  RI  AF(NA)  VECTOR CONTAINING NEW VALUES OF THE APPROXIMATED
*         FUNCTIONS.
*  RI  AFO(NA)  VECTOR CONTAINING OLD VALUES OF THE APPROXIMATED
*         FUNCTIONS.
*  RO  AFD(NA)  VECTOR CONTAINING INCREMENTS OF THE APPROXIMATED
*         FUNCTIONS.
*  II  IA(NA)  VECTOR CONTAINING TYPES OF DEVIATIONS.
*  RI  ETA0  MACHINE PRECISION.
*  RI  ETA9  MAXIMUM VALUE LESS THEN INFINITY.
*  RO  R  VALUE OF THE STEPSIZE PARAMETER.
*  RA  FA  AUXILIARY VARIABLE.
*  II  KAP  INDEX OF ACTIVE FUNCTION IN THE NEW POINT.
*  II  NAP  INDEX OF ACTIVE FUNCTION IN THE OLD POINT.
*  IO  MERR  ERROR INDICATOR. MERR=0 FOR NORMAL RETURN.
*  II  MODE  MODE OF INTERPOLATION.
*
* COMMON DATA :
*  TO  TSXX  TEXT INFORMATION ON THE STEPSIZE OBTAINED.
*
* METHOD :
* L.LUKSAN : AN IMPLEMENTATION OF RECURSIVE QUADRATIC PROGRAMMING
* VARIABLE METRIC METHODS FOR LINEARLY CONSTRAINED NONLINEAR MINIMAX
* APPROXIMATION, KYBERNETIKA 21,(1985),PP 22-40.
*
      SUBROUTINE UNINT5(NA,AF,AFO,AFD,IA,ETA0,ETA9,R,FA,KAP,NAP,MERR,
     & MODE)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER NA,IA(NA),KAP,NAP,MERR,MODE
      REAL*8 AF(NA),AFO(NA),AFD(NA),ETA0,ETA9,R,FA
      REAL*8 RI,AI,BI,CI,EI,RJ,AJ,BJ,CJ,EJ,RK,AK,BK,CK,EK,FK,AL,BL,CL,FL
      INTEGER I,J,K,L,MI,MJ,ML,KA
      MERR=0
      IF (MODE.LE.0) THEN
      RETURN
      ELSE IF (MODE.EQ.1) THEN
      R=TWO*R
      TSXX='BISECT. '
      RETURN
      ELSE
      TSXX='MINIMAX '
      ENDIF
      FA=AF(KAP)
      DO 1 KA=1,NA
      IF (IA(KA).NE.0) THEN
      AF(KA)=((AF(KA)-AFO(KA))/R-AFD(KA))/R
      ENDIF
    1 CONTINUE
      J=KAP
      MJ=1
      RJ=R
      IF (IA(J).EQ.1.OR.IA(J).GE.3.AND.AFO(J).LT.ZERO) THEN
      EJ=-FA
      CJ=-AFO(J)
      BJ=-AFD(J)
      AJ=-AF(J)
      ELSE
      EJ= FA
      CJ= AFO(J)
      BJ= AFD(J)
      AJ= AF(J)
      ENDIF
      FK=EJ
      IF (IA(J).GE.3) FK=ABS(FK)
      IF (FK.GT.EJ) THEN
      CJ=-CJ
      BJ=-BJ
      AJ=-AJ
      MJ=-MJ
      ENDIF
      IF (TWO*AJ*R+BJ.LT.ZERO) RETURN
      I=NAP
      MI=1
      RI=ZERO
      IF (IA(I).EQ.1.OR.IA(I).GE.3.AND.AFO(I).LT.ZERO) THEN
      CI=-AFO(I)
      BI=-AFD(I)
      AI=-AF(I)
      ELSE
      CI= AFO(I)
      BI= AFD(I)
      AI= AF(I)
      ENDIF
      IF (BI.GE.ZERO) THEN
      R =HALF*(RI+RJ)
      TSXX='BISECT. '
      RETURN
      ENDIF
    2 RK=HALF*(RI+RJ)
      K=0
      IF (I.EQ.J.AND.MI.EQ.MJ) THEN
      K=1
      IF (AI.LE.ETA0*ABS(BI)) THEN
      IF (BI.GE.ZERO) THEN
      R=RI
      ELSE
      R=RJ
      ENDIF
      ELSE
      R=-HALF*BI/AI
      ENDIF
      ELSE
      CK=CJ-CI
      BK=BJ-BI
      AK=AJ-AI
      IF (ABS(AK).LE.ETA0*ABS(BK)) THEN
      IF (ABS(BK).LE.ETA0*ABS(CK)) THEN
      IF (AI.LE.ETA0*ABS(BI)) THEN
      R=RK
      ELSE
      R=-HALF*BI/AI
      ENDIF
      ELSE
      R=-CK/BK
      K=2
      ENDIF
      ELSE
      R=-HALF*BK/AK
      BK=SQRT(MAX(ZERO,R*R-CK/AK))
      AK=R+BK
      CK=R-BK
      IF (AK.GT.RI.AND.AK.LT.RJ) THEN
      R=AK
      K=2
      ELSE IF (CK.GT.RI.AND.CK.LT.RJ) THEN
      R=CK
      K=2
      ELSE
      R=RK
      ENDIF
      ENDIF
      ENDIF
      IF (RJ-RI.LE.ETA0*RK) RETURN
      FL=-ETA9
      DO 3 KA=1,NA
      IF (IA(KA).GT.0) THEN
      IF (IA(KA).EQ.1.OR.IA(KA).GE.3.AND.AFO(KA).LT.ZERO) THEN
      CK=-AFO(KA)
      BK=-AFD(KA)
      AK=-AF(KA)
      ELSE
      CK= AFO(KA)
      BK= AFD(KA)
      AK= AF(KA)
      ENDIF
      EK=(AK*R+BK)*R+CK
      FK=EK
      IF (IA(KA).GE.3) FK=ABS(FK)
      IF (FK.GT.FL) THEN
      L=KA
      FL=FK
      IF (FK.GT.EK) THEN
      AL=-AK
      BL=-BK
      CL=-CK
      ML=-1
      ELSE
      AL= AK
      BL= BK
      CL= CK
      ML= 1
      ENDIF
      ENDIF
      ENDIF
    3 CONTINUE
      IF ((L.EQ.I.OR.L.EQ.J).AND.K.NE.0) THEN
      IF (I.EQ.J) RETURN
      EI=TWO*AI*R+BI
      EJ=TWO*AJ*R+BJ
      IF (EI*EJ.LE.ZERO) RETURN
      IF (EI.LE.ZERO) THEN
      I=J
      RI=R
      AI=AJ
      BI=BJ
      CI=CJ
      MI=MJ
      ELSE
      J=I
      RJ=R
      AJ=AI
      BJ=BI
      CJ=CI
      MJ=MI
      ENDIF
      ELSE
      IF (TWO*AL*R+BL.LT.ZERO) THEN
      I=L
      RI=R
      AI=AL
      BI=BL
      CI=CL
      MI=ML
      ELSE
      J=L
      RJ=R
      AJ=AL
      BJ=BL
      CJ=CL
      MJ=ML
      ENDIF
      ENDIF
      GO TO 2
      END
