* SUBROUTINE UNSTP5                ALL SYSTEMS                99/12/01
C PORTABILITY : ALL SYSTEMS
C 99/12/01 VL : ORIGINAL VERSION
*
* PURPOSE :
*  STEPSIZE SELECTION USING POLYHEDRAL APPROXIMATION
*  FOR NULL STEP IN NONCONVEX VARIABLE METRIC METHOD.
*
* PARAMETERS :
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  II  MA  DECLARED NUMBER OF LINEAR APPROXIMATED FUNCTIONS.
*  II  MAL  CURRENT NUMBER OF LINEAR APPROXIMATED FUNCTIONS.
*  RU  X(N)  VECTOR OF VARIABLES.
*  RI  AF(4*MA)  VECTOR OF BUNDLE FUNCTIONS VALUES.
*  RI  AG(N*MA)  MATRIX WHOSE COLUMNS ARE BUNDLE SUBGRADIENTS.
*  RI  AY(N*MA)  MATRIX WHOSE COLUMNS ARE VARIABLE VECTORS.
*  RI  S(N)  DIRECTION VECTOR.
*  RO  F  VALUE OF THE OBJECTIVE FUNCTION.
*  RU  DF  DIRECTIONAL DERIVATIVE.
*  RO  T  VALUE OF THE STEPSIZE PARAMETER.
*  RO  TB  BUNDLE PARAMETER FOR MATRIX SCALING.
*  RI  ETA5  DISTANCE MEASURE PARAMETER.
*  RI  ETA9  MAXIMUM FOR REAL NUMBERS.
*  RI  MOS3  LOCALITY MEASURE PARAMETER.
*
      SUBROUTINE UNSTP5(N,MA,MAL,X,AF,AG,AY,S,F,DF,T,TB,ETA5,ETA9,MOS3)
      INTEGER N,MA,MAL,MOS3
      REAL*8 X(N),AF(MA*3),AG(N*MA),AY(N*MA),S(N),F,DF,T,TB,ETA5,ETA9
      REAL*8 ALF,ALFL,ALFR,BET,BETL,BETR,DX,Q,R,W
      INTEGER I,J,JN,K,L,LQ
      REAL*8 ZERO,ONE,HALF
      PARAMETER (ZERO=0.0D 0,ONE=1.0D 0,HALF=0.5D 0)
      W=DF*T
C
C     INITIAL CHOICE OF POSSIBLY ACTIVE PARABOLAS
C
      K=0
      L=-1
      JN=0
      TB=SQRT(ETA9)
      BETR=-ETA9
      DO 3 J=1,MAL-1
      BET=ZERO
      R=ZERO
      ALFL=AF(J)-F
      DO 2 I=1,N
      DX=X(I)-AY(JN+I)
      R=R+DX*DX
      Q=AG(JN+I)
      ALFL=ALFL+DX*Q
    2 BET=BET+S(I)*Q
      IF(MOS3.NE.2)R=R**(DBLE(MOS3)*HALF)
      ALF=MAX(ABS(ALFL),ETA5*R)
      IF(BET+BET.GT.DF)TB=MIN(TB,ALF/(BET-DF))
      BETR=MAX(BETR,BET-ALF)
      IF(ALF.LT.BET-DF)THEN
        K=K+1
        R=T*BET-ALF
        AF(MA+K)=ALF
        AF(MA+MA+K)=BET
        IF(R.GT.W) THEN
          W=R
          L=K
        ENDIF
      ENDIF
    3 JN=JN+N
      LQ=1
      IF(BETR.LE.DF*HALF)LQ=-1
      IF(L.LT.0)RETURN
      BETR=AF(MA+MA+L)
      ALFR=AF(MA+L)
      ALF=ALFR
      BET=BETR
      ALFL=ZERO
      BETL=DF
C
C     ITERATION LOOP
C
    4 W=BET/DF
      IF(ABS(BETR-BETL)+ABS(ALFR-ALFL).LT.-1.0D-4*DF)RETURN
      IF(BETR-BETL.EQ.ZERO)STOP 11
      R=(ALFR-ALFL)/(BETR-BETL)
      IF(ABS(T-W).LT.ABS(T-R))R=W
      Q=T
      T=R
      IF(ABS(T-Q).LT.1.0D-3)RETURN
      AF(MA+L)=-ONE
      W=T*BET-ALF
      L=-1
      DO 5 J=1,K
      ALF=AF(MA+J)
      IF(ALF.LT.ZERO)GOTO5
      BET=AF(MA+MA+J)
      R=T*BET-ALF
      IF(R.GT.W) THEN
        W=R
        L=J
      ENDIF
    5 CONTINUE
      IF(L.LT.0)RETURN
      BET=AF(MA+MA+L)
      Q=BET-T*DF
      IF(Q.EQ.ZERO)RETURN
C
C     NEW INTERVAL SELECTION
C
      ALF=AF(MA+L)
      IF(Q.LT.ZERO)THEN
        ALFL=ALF
        BETL=BET
      ELSE
        ALFR=ALF
        BETR=BET
      ENDIF
      GOTO4
      END
