* SUBROUTINE UOKLQ3                ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
*  PRINT OF ITERATION OF THE SPARSE QUADRATIC PROGRAMMING.
*
* PARAMETERS:
*  II  N  TRANSFORMED NUMBER OF VARIABLES.
*  II  M  TRANSFORMED NUMBER OF CONSTRAINTS.
*  II  MCOLS DECLARED NUMBER OF COLUMNS.
*  II  NEL NUMBER OF ELEMENTS IN ARRAYS LI/LJ/CM.
*  II  LENR(MCOLS) LENGTHS OF THE ROWS OF THE FACTOR U.
*  II  TYPE(MCOLS) TYPES OF THE VARIABLES:
*                                    =1 FOR NONBASIC LOWER BOUNDED
*                                    =2 FOR NONBASIC UPPER BOUNDED
*                                    =3 FOR NONBASIC LOWER AND UPPER
*                     BOUNDED VARIABLE SET TO ITS LOWER BOUND
*                                    =4 FOR NONBASIC LOWER AND UPPER
*                     BOUNDED VARIABLE SET TO ITS UPPER BOUND
*                                    =5 FOR NONBASIC FIXED VARIABLE
*                                    =6 FOR NONBASIC FREE VARIABLE
*                                    =-1 FOR BASIC LOWER BOUNDED
*                                    =-2 FOR BASIC UPPER BOUNDED
*                                    =-3 FOR BASIC LOWER AND UPPER
*                     BOUNDED VARIABLE
*                                    =-6 FOR BASIC FREE VARIABLE.
*  II  BCOL(MCOLS) M FIRST ELEMENTS CONTAIN COLUMN INDICES OF BASIC
*              COLUMNS.
*  RI  X(MCOLS) VECTOR OF SOLUTION.
*  RI  C(MCOLS) COST VECTOR - GRADIENT OF THE LINEAR FUNCTION.
*  II  F  FUNCTIONAL VALUE.
*  RI  GNORM NORM OF THE REDUCED GRADIENT.
*  II  NIQ MINOR CYCLE COUNT.
*  II  NAQ MAJOR CYCLE COUNT.
*  II  LINP NUMBER OF BASICS AND SUPERBASICS.
*
* COMMON DATA :
*  II  NREF NUMBER OF REFACTORIZATIONS.
*  II  NCGR NUMBER OF CG ITERATIONS.
*  II  JOUT  SYSTEM VARIABLE WHICH SPECIFIES PRINT OF INFORMATION
*         DURING THE OPTIMIZATION PROCESS. JOUT=0-NO PRINT.
*         JOUT=1-PRINT IS REQUIRED.
*  II  KOUT  LEVEL OF PRINT. KOUT=0-NO PRINT. KOUT=1-PRINT OF INITIAL
*         VALUES AND FINAL RESULTS TOGETHER WITH PRINT OF ITERATIONS.
*         KOUT=2-ADDITIONAL PRINT OF STEPSIZE SELECTION.
*         KOUT=3-ADDITIONAL PRINT OF DIRECTION DETERMINATION.
*         KOUT=4-ADDITIONAL PRINT OF MATRIX CORRECTION.
*         KOUT=5-ADDITIONAL PRINT OF NUMERICAL DIFFERENTIATION.
*         KOUT=6-ADDITIONAL PRINT OF DEBUGGING.
*  II  LOUT  PRINT OF ITERATIONS. LOUT=0-STANDARD PAPER SAVING PRINT.
*         LOUT=1-ADDITIONAL PRINT OF FINAL RESULTS OF SUBROUTINES.
*         LOUT=2-ADDITIONAL PRINT OF VARIABLES. LOUT=3-ADDITIONAL
*         PRINT OF THE GRADIENT. LOUT=4-ADDITIONAL PRINT OF THE HESSIAN
*         MATRIX.
*  II  MOUT  LEVEL OF PRINT ON THE TERMINAL DEVICE.
*         ABS(MOUT)=0 - NO PRINT.
*         ABS(MOUT)=1 - PRINT OF FINAL RESULTS.
*         ABS(MOUT)=2 - ADDITIONAL PRINT OF ITERATIONS.
*
      SUBROUTINE UOKLQ3(N,M,MROWS,MCOLS,NEL,LENR,TYPE,BCOL,X,C,F,GNORM,
     & NIQ,NAQ,LINP)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER N,M,MROWS,MCOLS,NEL,BCOL(MCOLS),TYPE(MCOLS),LENR(MROWS),
     & NIQ,NAQ,LINP
      REAL*8 X(MCOLS),C(MCOLS),F,GNORM
      INTEGER I,IU,NSBSP
      REAL*8 DENSU
      LOGICAL LL
      LL=KOUT.EQ.0.AND.LOUT.GT.0
C
C NUMBER OF SUPERBASIC VARIABLES
C
      NSBSP=LINP-M
C
C COMPUTE IU,DENSITIES
C
      IU=0
      DO 100 I=1,M
        IU=IU+LENR(I)
  100 CONTINUE
      IF(ABS(MOUT).GT.3) THEN
      WRITE(ITR,9) NAQ,NIQ,NSBSP,NCGR,IU,F,GNORM
      IF(LL) WRITE(IWR,9) NAQ,NIQ,NSBSP,NCGR,IU,F,GNORM
      ENDIF
    9 FORMAT(' ','NAQ=',I3,1X,'NIQ=',I5,1X,'NSBSP=',I4,1X,
     *       'NCGR=',I3,1X,'IU=',I5,2X,'F=',D 13.7,2X,'G=',D 9.3)
      IF(JOUT.EQ.0) RETURN
      IF(ABS(KOUT).LE.3) RETURN
      IF(M.GT.0) THEN
        DENSU=2*IU-M
        DENSU=DENSU/(M*M)
      ELSE
        DENSU=0
      ENDIF
      WRITE(IWR,2) NAQ,NIQ,NSBSP,NCGR,IU,F,GNORM
    2 FORMAT(' ','NAQ=',I3,2X,'NIQ=',I4,2X,'NSBSP=',I4,2X,
     *       'NCGR=',I3,2X,'IU=',I5,2X,'F=',D 9.3,2X,
     *       'G=',D 9.3)
      IF(ABS(LOUT).GE.2) WRITE(IWR,7) DENSU,IU,NEL,NREF
    7 FORMAT (' ','DENSITY OF U =',D 9.3,2X,'ELEMENTS OF U ='
     *       ,I4,2X,'NEL =',I4,2X,'NREF =',I4)
      IF(ABS(LOUT).GE.3) WRITE(IWR,4)(BCOL(I),I=1,MIN(M,50))
    4 FORMAT(' ','BCOL=',10I7:/(6X,10I7))
      IF(ABS(LOUT).GE.3) WRITE(IWR,5)(X(I),I=1,MIN(N,25))
    5 FORMAT(' ','X   =',5D 14.5:/(6X,5D 14.5))
      IF(ABS(LOUT).GT.3.AND.KBF.GT.0)WRITE(IWR,6)(TYPE(I),I=1,MIN(N,50))
    6 FORMAT(' ','IX  =',10I7:/(6X,10I7))
      IF(ABS(LOUT).GT.3) WRITE(IWR,3)(C(I),I=1,MIN(N,25))
    3 FORMAT(' ','GF  =',5D 14.5:/(6X,5D 14.5))
      RETURN
      END
