* SUBROUTINE UP0BF2             ALL SYSTEMS                 02/12/01
C PORTABILITY : ALL SYSTEMS
C 02/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* COMPUTATION OF VALUE OF THE BARRIER FUNCTION.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  NX  NUMBER OF BOX CONSTRAINTS.
*  II  NC  NUMBER OF GENERAL CONSTRAINTS.
*  II  NCA  NUMBER OF ACTIVE CONSTRAINTS.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  RI  XP(NF)  PREVIOUS VECTOR OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XL(NF)  VECTOR CONTAINING LOWER BOUNDS FOR VARIABLES.
*  RI  XU(NF)  VECTOR CONTAINING UPPER BOUNDS FOR VARIABLES.
*  RI  XSL(NX)  VECTOR CONTAINING LOVER SLACKS FOR VARIABLES.
*  RI  XSU(NX)  VECTOR CONTAINING UPPER SLACKS FOR VARIABLES.
*  RI  XZL(NF)  VECTOR CONTAINING LOVER MULTIPLIERS FOR VARIABLES.
*  RI  XZU(NF)  VECTOR CONTAINING UPPER MULTIPLIERS FOR VARIABLES.
*  RI  CF(NC+1)  VECTOR CONTAINING VALUES OF CONSTRAINT FUNCTIONS.
*  RI  CFD(NC)  VECTOR CONTAINING INCREMENTS OF CONSTRAINT FUNCTIONS.
*  II  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  RI  CL(NC)  VECTOR CONTAINING LOWER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CU(NC)  VECTOR CONTAINING UPPER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CSL(NC)  VECTOR CONTAINING LOVER SLACKS FOR CONSTRAINTS.
*  RI  CSU(NC)  VECTOR CONTAINING UPPER SLACKS FOR CONSTRAINTS.
*  RI  CZL(NC)  VECTOR CONTAINING LOVER MULTIPLIERS FOR CONSTRAINTS.
*  RI  CZU(NC)  VECTOR CONTAINING UPPER MULTIPLIERS FOR CONSTRAINTS.
*  II  ICA(NCA)  VECTOR CONTAINING INDICES OF ACTIVE CONSTRAINTS.
*  RI  S(NCA)  VECTOR CONTAINING NEWTON INCREMENTS.
*  RI  GS(NCA)  VECTOR CONTAINING ACTIVE SLACK INCREMENTS.
*  RI  XO(NX+NC)  VECTOR CONTAINING INCREMENTS FOR LOWER INACTIVE
*         MULTIPLIERS.
*  RI  GO(NX+NC)  VECTOR CONTAINING INCREMENTS FOR UPPER INACTIVE
*         MULTIPLIERS.
*  RO  ED(NCA)  DIAGONAL ELEMENTS OF THE PERTURBATION MATRIX.
*  RI  R  STEPSIZE  PARAMETER.
*  RI  RPF2  PENALTY COEFFICIENT.
*  RI  RPF3  BARRIER COEFFICIENT.
*  RO  FC  VALUE OF THE PENALTY TERM.
*  RO  F  VALUE OF THE BARRIER FUNCTION.
*  II  IREG  OPTION. IREG=0-REGULARIZATION IS NOT USED. IREG=1-DIAGONAL
*         REGULARIZATION IS USED.
*  II  MEP  CHOICE OF A MERIT FUNCTION. MEP=0-MERIT FUNCTION IS
*            NOT USED. MEP=1-L2 AUGMENTED BARRIER FUNCTION.
*            MEP=2-L2 AUGMENTED LAGRANGIAN FUNCTION.
*
*  COMMON DATA :
*  II  KBF  SPECIFICATION OF SIMPLE BOUNDS. KBF=0-NO SIMPLE BOUNDS.
*         KBF=1-ONE SIDED SIMPLE BOUNDS. KBF=2=TWO SIDED SIMPLE BOUNDS.
*  II  KBC  SPECIFICATION OF LINEAR CONSTRAINTS. KBC=0-NO LINEAR
*         CONSTRAINTS. KBC=1-ONE SIDED LINEAR CONSTRAINTS. KBC=2=TWO
*         SIDED LINEAR CONSTRAINTS.
*
      SUBROUTINE UP0BF2(NF,NX,NC,NCA,X,XP,IX,XL,XU,XSL,XSU,XZL,XZU,CF,
     & CFD,IC,CL,CU,CSL,CSU,CZL,CZU,ICA,S,GS,XO,GO,ED,R,RPF2,RPF3,FC,F,
     & IREG,MEP)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER NF,NX,NC,NCA,IX(NX),IC(NC),ICA(NCA),IREG,MEP
      REAL*8 X(NF),XP(NF),XL(NX),XU(NX),XSL(NX),XSU(NX),XZL(NX),XZU(NX),
     & CF(NC+1),CFD(NC),CL(NC),CU(NC),CSL(NC),CSU(NC),CZL(NC),CZU(NC),
     & S(*),GS(*),XO(*),GO(*),ED(NCA),R,RPF2,RPF3,FC,F
      INTEGER I,K,KK
      REAL*8 RP,TMP,TEMP,CON
      IF (MEP.EQ.1) THEN
      CON=0.95D 0
      ELSE
      CON=0.99D 0
      ENDIF
      FC=0.0D 0
      DO 1 I=1,NCA
      RP=R
      K=ICA(I)
      IF (K.LT.0) THEN
      K=-K
      KK=-IX(K)
      IF (KK.EQ.1.OR.KK.EQ.3) THEN
      IF (GS(I).LT.ZERO) RP=MIN(RP,-CON*XSL(K)/GS(I))
      TMP=XSL(K)+RP*GS(I)
      TEMP=XL(K)-X(K)+TMP
      IF (IREG.EQ.0) THEN
      FC=FC+(XZL(K)+S(NF+I)+HALF*RPF2*TEMP)*TEMP-
     & RPF3*LOG(TMP)
      ELSE
      FC=FC+(XZL(K)+S(NF+I))*TEMP+HALF*RPF2*(TEMP-R*ED(I)*S(NF+I))**2-
     & RPF3*LOG(TMP)
      ENDIF
      ELSE IF (KK.EQ.2.OR.KK.EQ.4) THEN
      IF (GS(I).LT.ZERO) RP=MIN(RP,-CON*XSU(K)/GS(I))
      TMP=XSU(K)+RP*GS(I)
      TEMP=X(K)-XU(K)+TMP
      IF (IREG.EQ.0) THEN
      FC=FC+(XZU(K)+S(NF+I)+HALF*RPF2*TEMP)*TEMP-
     & RPF3*LOG(TMP)
      ELSE
      FC=FC+(XZU(K)+S(NF+I))*TEMP+HALF*RPF2*(TEMP-R*ED(I)*S(NF+I))**2-
     & RPF3*LOG(TMP)
      ENDIF
      ENDIF
      ELSE
      KK=-IC(K)
      IF (KK.EQ.1.OR.KK.EQ.3) THEN
      IF (GS(I).LT.ZERO) RP=MIN(RP,-CON*CSL(K)/GS(I))
      TMP=CSL(K)+RP*GS(I)
      TEMP=CL(K)-CF(K)+TMP
      IF (IREG.EQ.0) THEN
      FC=FC+(CZL(K)+S(NF+I)+HALF*RPF2*TEMP)*TEMP-
     & RPF3*LOG(TMP)
      ELSE
      FC=FC+(CZL(K)+S(NF+I))*TEMP+HALF*RPF2*(TEMP-R*ED(I)*S(NF+I))**2-
     & RPF3*LOG(TMP)
      ENDIF
      ELSE IF (KK.EQ.2.OR.KK.EQ.4) THEN
      IF (GS(I).LT.ZERO) RP=MIN(RP,-CON*CSU(K)/GS(I))
      TMP=CSU(K)+RP*GS(I)
      TEMP=CF(K)-CU(K)+TMP
      IF (IREG.EQ.0) THEN
      FC=FC+(CZU(K)+S(NF+I)+HALF*RPF2*TEMP)*TEMP-
     & RPF3*LOG(TMP)
      ELSE
      FC=FC+(CZU(K)+S(NF+I))*TEMP+HALF*RPF2*(TEMP-R*ED(I)*S(NF+I))**2-
     & RPF3*LOG(TMP)
      ENDIF
      ELSE IF (KK.EQ.5) THEN
      FC=FC+(CZL(K)+S(NF+I)+HALF*RPF2*CF(K))*CF(K)
      ENDIF
      ENDIF
    1 CONTINUE
      IF (KBF.GT.0) THEN
      DO 2 K=1,NX
      RP=R
      KK=IX(K)
      IF (KK.EQ.1.OR.KK.EQ.3.OR.ABS(KK).EQ.4) THEN
      TMP=XP(K)+S(K)-XL(K)-XSL(K)
      IF (TMP.LT.ZERO) RP=MIN(RP,-CON*XSL(K)/TMP)
      TEMP=XL(K)-X(K)+XSL(K)+RP*TMP
      FC=FC+(XZL(K)+XO(K)+HALF*RPF2*TEMP)*TEMP-
     & RPF3*LOG(XSL(K)+RP*TMP)
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.4.OR.ABS(KK).EQ.3) THEN
      TMP=XU(K)-XP(K)-S(K)-XSU(K)
      IF (TMP.LT.ZERO) RP=MIN(RP,-CON*XSU(K)/TMP)
      TEMP=X(K)-XU(K)+XSU(K)+RP*TMP
      FC=FC+(XZU(K)+GO(K)+HALF*RPF2*TEMP)*TEMP-
     & RPF3*LOG(XSU(K)+RP*TMP)
      ENDIF
    2 CONTINUE
      ENDIF
      IF (KBC.GT.0) THEN
      DO 3 K=1,NC
      RP=R
      KK=IC(K)
      IF (KK.EQ.1.OR.KK.EQ.3.OR.ABS(KK).EQ.4) THEN
      TMP=CFD(K)-CL(K)-CSL(K)
      IF (TMP.LT.ZERO) RP=MIN(RP,-CON*CSL(K)/TMP)
      TEMP=CL(K)-CF(K)+CSL(K)+RP*TMP
      FC=FC+(CZL(K)+XO(NX+K)+HALF*RPF2*TEMP)*TEMP-
     & RPF3*LOG(CSL(K)+RP*TMP)
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.4.OR.ABS(KK).EQ.3) THEN
      TMP=CU(K)-CFD(K)-CSU(K)
      IF (TMP.LT.ZERO) RP=MIN(RP,-CON*CSU(K)/TMP)
      TEMP=CF(K)-CU(K)+CSU(K)+RP*TMP
      FC=FC+(CZU(K)+GO(NX+K)+HALF*RPF2*TEMP)*TEMP-
     & RPF3*LOG(CSU(K)+RP*TMP)
      ENDIF
    3 CONTINUE
      ENDIF
      F=CF(NC+1)+FC
      RETURN
      END
