* SUBROUTINE UP2FF3                ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* COMPUTATION OF THE VALUE AND THE GRADIENT OF THE OBJECTIVE FUNCTION.
* COMPUTATION OF THE VALUES AND THE GRADIENTS OF THE CONSTRAINT
* FUNCTIONS.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  NC  NUMBER OF CONSTRAINTS.
*  II  NCL  NUMBER OF LINEAR CONSTRAINTS.
*  II  MC  NUMBER OF NONZERO ELEMENTS OF THE SPARSE JACOBIAN MATRIX.
*  II  MHC  NUMBER OF NONZERO ELEMENTS OF HC.
*  IU  KC  INDEX OF THE CONSTRAINT.
*  II  M  NUMBER OF NONZERO ELEMENTS OF THE SPARSE HESSIAN MATRIX.
*  II  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XL(NF)  VECTOR CONTAINING LOWER BOUNDS FOR VARIABLES.
*  RI  XU(NF)  VECTOR CONTAINING UPPER BOUNDS FOR VARIABLES.
*  RI  XSL(NF)  VECTOR CONTAINING LOVER SLACKS FOR VARIABLES.
*  RI  XSU(NF)  VECTOR CONTAINING UPPER SLACKS FOR VARIABLES.
*  RI  XZL(NF)  VECTOR CONTAINING LOVER MULTIPLIERS FOR VARIABLES.
*  RI  XZU(NF)  VECTOR CONTAINING UPPER MULTIPLIERS FOR VARIABLES.
*  RI  GF(NF)  GRADIENT OF THE MODEL FUNCTION.
*  II  HF(M)  SPARSE HESSIAN MATRIX OF THE MODEL FUNCTION.
*  II  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  RI  CL(NC)  VECTOR CONTAINING LOWER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CU(NC)  VECTOR CONTAINING UPPER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CSL(NC)  VECTOR CONTAINING LOVER SLACKS FOR CONSTRAINTS.
*  RI  CSU(NC)  VECTOR CONTAINING UPPER SLACKS FOR CONSTRAINTS.
*  RI  CZL(NC)  VECTOR CONTAINING LOVER MULTIPLIERS FOR CONSTRAINTS.
*  RI  CZU(NC)  VECTOR CONTAINING UPPER MULTIPLIERS FOR CONSTRAINTS.
*  RI  GC(NF)  GRADIENT OF THE SELECTED CONSTRAINT FUNCTION.
*  II  HC(ME)  SPARSE HESSIAN MATRIX OF THE CONSTRAINT FUNCTION.
*  II  ICG(NC+1)  POSITIONS OF THE FIRST ROWS ELEMENTS IN THE SPARSE
*         JACOBIAN STRUCTURE.
*  II  JCG(MC)  COLUMN INDICES OF ELEMENTS IN THE SPARSE JACOBIAN
*         STRUCTURE.
*  RO  G(NF)  GRADIENT OF THE OBJECTIVE FUNCTION.
*  RO  H(M)  SPARSE HESSIAN MATRIX OF THE LAGRANGIAN FUNCTION.
*  II  IH(N+1) POINTERS OF DIAGONAL ELEMENTS OF THE SPARSE HESSIAN
*         MATRIX.
*  II  JH(M)  COLUMN INDICES OF NONZERO ELEMENTS OF THE SPARSE HESSIAN
*         MATRIX.
*  RO  FF  VALUE OF THE MODEL FUNCTION.
*  RO  FC  VALUE OF THE CONSTRAINT TERM.
*  RO  F  VALUE OF THE PENALTY FUNCTION.
*  RO  CMAX  MAXIMUM CONSTRAINT VIOLATION.
*
* COMMON DATA :
*  II  KD  DEGREE OF REQUIRED DERIVATIVES.
*  II  LD  DEGREE OF PREVIOUSLY COMPUTED DERIVATIVES.
*  II  IEXT  TYPE OF EXTREMUM. IEXT=0-MINIMUM. IEXT=1-MAXIMUM.
*  IU  NAG  NUMBER OF APPROXIMATED FUNCTION GRADIENTS COMPUTED.
*  IU  NAH  NUMBER OF APPROXIMATED FUNCTION HESSIAN MATRICES COMPUTED.
*  IU  NFV  NUMBER OF OBJECTIVE FUNCTION VALUES COMPUTED.
*  IU  NFG  NUMBER OF OBJECTIVE FUNCTION GRADIENTS COMPUTED.
*  IU  NFH  NUMBER OF OBJECTIVE FUNCTION HESSIAN MATRICES COMPUTED.
*
* STATUS VARIABLES :
*  NS,ISP,TSS
*
* SUBPROGRAMS USED :
*  S   UYPRO1  PROLOGUE.
*  S   UYEPI1  EPILOGUE.
*  S   UYSET0  STATUS DEFINITION.
*  S   UXVCOP  COPYING OF A VECTOR.
*  S   UXVSET  INITIATION OF A VECTOR.
*  S   UASSH1  COMPUTATION OF THE SPARSE HESSIAN MATRIX FROM THE SPARSE
*         JACOBIAN MATRIX.
*  S   UASSH2  COMPUTATION OF THE SPARSE HESSIAN MATRIX FROM THE SPARSE
*         JACOBIAN MATRIX.
*
      SUBROUTINE UP2FF3(NF,NC,NCL,MC,MHC,KC,M,GF,HF,GC,HC,ICG,JCG,G,H,
     & IH,JH,CZ,FF,FC,F,CMAX)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER NF,NC,NCL,MC,MHC,KC,M,ICG(NC+1),JCG(MC),IH(NF+1),JH(M)
      REAL*8 GF(NF),HF(M),GC(NF),HC(MHC),G(NF),H(M),CZ(NC),FF,FC,
     & F,CMAX
      INTEGER J,K,L,JP
      IF (TSS(NS).NE.'UFXX') CALL UYPRO1('UFXX',-1)
      GO TO (1,2,4,6,14,16,18) ISP(NS)
    1 CONTINUE
      IF (KD.LE.LD) GO TO 20
      IF (LD.GE.0) GO TO 3
      CALL UYSET0(1,2)
      RETURN
    2 CONTINUE
      IF (IEXT.LE.0) THEN
      F= FF
      ELSE
      F=-FF
      ENDIF
      CMAX=ZERO
    3 IF (KD.LT.1) GO TO 7
      IF (LD.GE.1) GO TO 5
      CALL UYSET0(2,3)
      RETURN
    4 CONTINUE
      IF (IEXT.LE.0) THEN
      CALL UXVCOP(NF,GF,G)
      ELSE
      CALL UXVNEG(NF,GF,G)
      ENDIF
    5 IF (KD.LT.2) GO TO 7
      IF (LD.GE.2) GO TO 7
      CALL UYSET0(3,4)
      RETURN
    6 CONTINUE
      IDECF=0
      IF (IEXT.GT.0) THEN
      CALL UXVNEG(M,HF,H)
      ELSE IF (NEXT.GT.0) THEN
      CALL UXVCOP(M,HF,H)
      ENDIF
    7 IF (KD.GE.0.AND.LD.LT.0) FF=F
C      IF (KD.GE.1.AND.LD.LT.1) CALL UXVCOP(NF,G,GF)
      KC=1
   12 IF (KC.GT.NC) GO TO 20
      IF (LD.GE.0) GO TO 15
      CALL UYSET0(4,5)
      RETURN
   14 CONTINUE
      F=F+CZ(KC)*FC
      CMAX=MAX(CMAX,ABS(FC))
   15 IF (KD.LT.1) GO TO 19
      IF (LD.GE.1) GO TO 17
      CALL UYSET0(5,6)
      RETURN
   16 CONTINUE
      K=ICG(KC)
      L=ICG(KC+1)-K
      DO 13 J=1,L
      JP=ABS(JCG(K))
      G(JP)=G(JP)+CZ(KC)*GC(JP)
      K=K+1
   13 CONTINUE
   17 IF (KD.LT.2) GO TO 19
      IF (KC.LE.NCL) GO TO 19
      CALL UYSET0(6,7)
      RETURN
   18 CONTINUE
      IDECF=0
      CALL UASSH2(NF,NC,MC,MHC,M,H,IH,JH,HC,ICG,JCG,KC,CZ(KC))
   19 KC=KC+1
      GO TO 12
   20 CONTINUE
      CALL UYEPI1(-1)
      RETURN
      END
