* SUBROUTINE UP2LQ5             ALL SYSTEMS                 00/12/01
C PORTABILITY : ALL SYSTEMS
C 00/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* COMPUTATION OF THE VALUE AND THE GRADIENT OF THE OBJECTIVE FUNCTION.
* COMPUTATION OF THE VALUES AND THE GRADIENTS OF THE CONSTRAINT
* FUNCTIONS.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  NX  NUMBER OF BOX CONSTRAINTS.
*  II  NC  NUMBER OF GENERAL CONSTRAINTS.
*  II  MC  NUMBER OF NONZERO ELEMENTS OF THE SPARSE JACOBIAN MATRIX.
*  IU  KC  INDEX OF THE CONSTRAINT.
*  II  M  SIZE OF THE SPARSE MATRIX WORK SPACE.
*  II  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NX)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XL(NX)  VECTOR CONTAINING LOWER BOUNDS FOR VARIABLES.
*  RI  XU(NX)  VECTOR CONTAINING UPPER BOUNDS FOR VARIABLES.
*  RI  XSL(NX)  VECTOR CONTAINING LOVER SLACKS FOR VARIABLES.
*  RI  XSU(NX)  VECTOR CONTAINING UPPER SLACKS FOR VARIABLES.
*  RI  XZL(NX)  VECTOR CONTAINING LOVER MULTIPLIERS FOR VARIABLES.
*  RI  XZU(NX)  VECTOR CONTAINING UPPER MULTIPLIERS FOR VARIABLES.
*  RI  GF(NF)  GRADIENT OF THE MODEL FUNCTION.
*  RI  HF(M)  HESSIAN ELEMENTS OF THE QUADRATIC MODEL FUNCTION.
*  RI  CF(NC+1)  VECTOR CONTAINING VALUES OF THE CONSTRAINT FUNCTIONS.
*  II  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  RI  CL(NC)  VECTOR CONTAINING LOWER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CU(NC)  VECTOR CONTAINING UPPER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CSL(NC)  VECTOR CONTAINING LOVER SLACKS FOR CONSTRAINTS.
*  RI  CSU(NC)  VECTOR CONTAINING UPPER SLACKS FOR CONSTRAINTS.
*  RI  CZL(NC)  VECTOR CONTAINING LOVER MULTIPLIERS FOR CONSTRAINTS.
*  RI  CZU(NC)  VECTOR CONTAINING UPPER MULTIPLIERS FOR CONSTRAINTS.
*  II  CG(MC)  ELEMENTS OF THE CONSTRAINT JACOBIAN MATRIX.
*  RI  GC(NF)  GRADIENT OF THE SELECTED FUNCTION.
*  II  ICG(NC+1)  POSITIONS OF THE FIRST ROWS ELEMENTS IN THE SPARSE
*         JACOBIAN STRUCTURE.
*  II  JCG(MC)  COLUMN INDICES OF ELEMENTS IN THE SPARSE JACOBIAN
*         STRUCTURE.
*  RO  G(NF)  GRADIENT OF THE LAGRANGIAN FUNCTION.
*  RO  H(M)  SPARSE HESSIAN MATRIX OF THE LAGRANGIAN FUNCTION.
*  II  IH(NF+1)  POINTERS OF THE DIAGONAL ELEMENTS OF HESSIAN MATRIX.
*  II  JH(M)  INDICES OF THE NONZERO ELEMENTS OF HESSIAN MATRIX.
*  RI  EPS7  INITIAL VALUE OF THE SLACK VARIABLES.
*  RI  EPS8  INITIAL VALUE OF THE LAGRANGE MULTIPLIERS.
*  RI  FF  VALUE OF THE MODEL FUNCTION.
*  RO  FC  VALUE OF THE PENALTY TERM.
*  RO  F  VALUE OF THE OBJECTIVE FUNCTION.
*  RO  CMAX  MAXIMUM CONSTRAINT VIOLATION.
*
* COMMON DATA :
*  II  KBF  SPECIFICATION OF SIMPLE BOUNDS. KBF=0-NO SIMPLE BOUNDS.
*         KBF=1-ONE SIDED SIMPLE BOUNDS. KBF=2=TWO SIDED SIMPLE BOUNDS.
*  II  KBC  SPECIFICATION OF LINEAR CONSTRAINTS. KBC=0-NO LINEAR
*         CONSTRAINTS. KBC=1-ONE SIDED LINEAR CONSTRAINTS. KBC=2=TWO
*         SIDED LINEAR CONSTRAINTS.
*  II  KD  DEGREE OF REQUIRED DERIVATIVES.
*  II  LD  DEGREE OF PREVIOUSLY COMPUTED DERIVATIVES.
*  II  IEXT  TYPE OF EXTREMUM. IEXT=0-MINIMUM. IEXT=1-MAXIMUM.
*
* STATUS VARIABLES :
*  NS,ISP,TSS
*
* SUBPROGRAMS USED :
*  S   UYPRO1  PROLOGUE.
*  S   UYEPI1  EPILOGUE.
*  S   UYSET0  STATUS DEFINITION.
*  S   UXVCOP  COPYING OF A VECTOR.
*  S   UXVNEG  COPYING OF A VECTOR WITH CHANGE OF THE SIGN.
*
      SUBROUTINE UP2LQ5(NF,NX,NC,MC,KC,M,X,IX,XL,XU,XSL,XSU,XZL,XZU,GF,
     & HF,CF,IC,CL,CU,CSL,CSU,CZL,CZU,CG,GC,ICG,JCG,G,H,IH,JH,EPS7,EPS8,
     & FF,FC,F,CMAX)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER NF,NX,NC,MC,KC,M,IX(NX),IC(NC),ICG(NC+1),JCG(MC),IH(NF+1),
     & JH(M)
      REAL*8 X(NF),XL(NX),XU(NX),XSL(NX),XSU(NX),XZL(NX),XZU(NX),GF(NF),
     & HF(M),CF(NC+1),CL(NC),CU(NC),CSL(NC),CSU(NC),CZL(NC),CZU(NC),
     & CG(MC),GC(NF),G(NF),H(M),EPS7,EPS8,FF,FC,F,CMAX
      INTEGER I,J,K,KK,L,JP
      REAL*8 TEMP,UXVDOT
      IF (KD.LE.LD) GO TO 20
      IF (LD.GE.0) GO TO 3
      CALL UXSSMM(NF,IH(NF+1)-1,HF,IH,JH,X,G)
      CALL UXVSUM(NF,GF,G,G)
      F=FF+HALF*(UXVDOT(NF,X,GF)+UXVDOT(NF,X,G))
      IF (IEXT.GT.0) F=-F
      CF(NC+1)=F
      NFV=NFV+1
      NFG=NFG+1
      CMAX=ZERO
    3 IF (KD.LT.1) GO TO 7
      IF (LD.GE.1) GO TO 5
      IF (LD.GE.0) THEN
      CALL UXSSMM(NF,IH(NF+1)-1,HF,IH,JH,X,G)
      CALL UXVSUM(NF,GF,G,G)
      NFG=NFG+1
      ENDIF
      IF (IEXT.GT.0) CALL UXVNEG(NF,G,G)
    5 IF (KD.LT.2) GO TO 7
      IF (LD.GE.2) GO TO 7
      IDECF=0
      IF (IEXT.GT.0) THEN
      CALL UXVNEG(M,HF,H)
      ELSE
      CALL UXVCOP(M,HF,H)
      ENDIF
    7 IF (KBF.LE.0) GO TO 10
      IF (LD.LT.0) THEN
      DO 8 I=1,NX
      KK=ABS(IX(I))
      IF (KK.EQ.1.OR.KK.EQ.3.OR.KK.EQ.4) THEN
      IF (NIT.LE.LIT) THEN
      XSL(I)=MAX(EPS7,X(I)-XL(I))
      XZL(I)=EPS8
      ENDIF
      TEMP=XL(I)-X(I)
      F=F+XZL(I)*(TEMP+XSL(I))
      CMAX=MAX(CMAX,TEMP)
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.3.OR.KK.EQ.4) THEN
      IF (NIT.LE.LIT) THEN
      XSU(I)=MAX(EPS7,XU(I)-X(I))
      XZU(I)=EPS8
      ENDIF
      TEMP=X(I)-XU(I)
      F=F+XZU(I)*(TEMP+XSU(I))
      CMAX=MAX(CMAX,TEMP)
      ENDIF
    8 CONTINUE
      ENDIF
      IF (KD.GE.1.AND.LD.LT.1) THEN
      DO 9 I=1,NX
      KK=ABS(IX(I))
      IF (KK.EQ.1.OR.KK.EQ.3.OR.KK.EQ.4) THEN
      G(I)=G(I)-XZL(I)
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.3.OR.KK.EQ.4) THEN
      G(I)=G(I)+XZU(I)
      ENDIF
    9 CONTINUE
      ENDIF
   10 IF (KBC.LE.0) GO TO 20
      KC=1
   11 IF (KC.GT.NC) GO TO 20
      KK=ABS(IC(KC))
      IF (KK.EQ.0) GO TO 19
      IF (LD.GE.0) GO TO 13
      CALL UALNF3(NF,NC,MC,KC,X,X,CG,ICG,JCG,FC,0,0)
      CF(KC)=FC
      IF (KK.EQ.1.OR.KK.EQ.3.OR.KK.EQ.4) THEN
      IF (NIT.LE.LIT) THEN
      CSL(KC)=MAX(EPS7,FC-CL(KC))
      CZL(KC)=EPS8
      ENDIF
      TEMP=CL(KC)-FC
      F=F+CZL(KC)*(TEMP+CSL(KC))
      CMAX=MAX(CMAX,TEMP)
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.3.OR.KK.EQ.4) THEN
      IF (NIT.LE.LIT) THEN
      CSU(KC)=MAX(EPS7,CU(KC)-FC)
      CZU(KC)=EPS8
      ENDIF
      TEMP=FC-CU(KC)
      F=F+CZU(KC)*(TEMP+CSU(KC))
      CMAX=MAX(CMAX,TEMP)
      ENDIF
      IF (KK.EQ.5) THEN
      IF (NIT.LE.LIT) THEN
      CZL(KC)=ZERO
      ENDIF
      FC=FC-CL(KC)
      F=F+CZL(KC)*FC
      CMAX=MAX(CMAX,ABS(FC))
      CF(KC)=FC
      ENDIF
   13 IF (KD.LT.1) GO TO 19
      IF (LD.GE.1) GO TO 19
      CALL UALNG3(NF,NC,MC,KC,X,CG,ICG,JCG,GC,0,0)
      IF (KK.EQ.3.OR.KK.EQ.4) THEN
      TEMP= CZU(KC)-CZL(KC)
      ELSE IF (KK.EQ.1) THEN
      TEMP=-CZL(KC)
      ELSE IF (KK.EQ.2) THEN
      TEMP= CZU(KC)
      ELSE IF (KK.EQ.5) THEN
      TEMP= CZL(KC)
      ENDIF
      K=ICG(KC)
      L=ICG(KC+1)-K
      DO 15 J=1,L
      JP=ABS(JCG(K))
      G(JP)=G(JP)+TEMP*GC(JP)
      K=K+1
   15 CONTINUE
   19 KC=KC+1
      GO TO 11
   20 CONTINUE
      LD=KD
      RETURN
      END
