* SUBROUTINE UPLAG2             ALL SYSTEMS                 05/12/01
C PORTABILITY : ALL SYSTEMS
C 05/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* COMPUTATION OF VALUE OF THE AUGMENTED LAGRANGIAN FUNCTION.
*
* PARAMETERS :
*  II  NA  NUMBER OF APPROXIMATED FUNCTIONS.
*  RI  AF(NA)  VECTOR CONTAINING VALUES OF APPROXIMATED FUNCTIONS.
*  RI  AZL(NA)  VECTOR CONTAINING LOWER LAGRANGE MULTIPLIERS.
*  RI  AZU(NA)  VECTOR CONTAINING UPPER LAGRANGE MULTIPLIERS.
*  RI  ASL(NA)  VECTOR CONTAINING LOWER EXPONENTIAL TERMS.
*  RI  ASU(NA)  VECTOR CONTAINING UPPER EXPONENTIAL TERMS.
*  RI  FF  VALUE OF THE MINIMAX FUNCTION.
*  RI  RPF3  BARRIER COEFFICIENT.
*  II  IEXT  KIND OF THE MINIMAX APPROXIMATION. IEXT=0-CHEBYSHEV
*         APPROXIMATION. IEXT=-1-MINIMAX. IEXT=+1-MAXIMIN.
*  II  MEP2  SAVING EXPONENTIAL TERMS. MEP2=0-SAVING SUPPRESSED.
*         MEP2=1-SAVING EXPONENTIAL TERMS.
*
      SUBROUTINE UPLAG2(NA,AF,AZL,AZU,ASL,ASU,FF,RPF3,IEXT,MEP2)
      INTEGER NA,IEXT,MEP2
      REAL*8 AF(NA),AZL(NA),AZU(NA),ASL(NA),ASU(NA),FF,RPF3
      REAL*8 FA,TEMP
      INTEGER KA
      TEMP=0.0D 0
      DO 1 KA=1,NA
      FA=AF(KA)
      IF (IEXT.EQ.0) THEN
      IF (MEP2.EQ.0) THEN
      TEMP=TEMP+EXP(-(FF-FA)/RPF3)+EXP(-(FF+FA)/RPF3)
      ELSE
      ASU(KA)=EXP(-(FF-FA)/RPF3)
      ASL(KA)=EXP(-(FF+FA)/RPF3)
      TEMP=TEMP+ASU(KA)+ASL(KA)
      ENDIF
      ELSE IF (IEXT.LT.0) THEN
      IF (MEP2.EQ.0) THEN
      TEMP=TEMP+EXP(-(FF-FA)/RPF3)
      ELSE
      ASU(KA)=EXP(-(FF-FA)/RPF3)
      TEMP=TEMP+ASU(KA)
      ENDIF
      ELSE IF (IEXT.GT.0) THEN
      IF (MEP2.EQ.0) THEN
      TEMP=TEMP+EXP(-(FF+FA)/RPF3)
      ELSE
      ASL(KA)=EXP(-(FF+FA)/RPF3)
      TEMP=TEMP+ASL(KA)
      ENDIF
      ENDIF
    1 CONTINUE
      DO 2 KA=1,NA
      FA=AF(KA)
      IF (IEXT.EQ.0) THEN
      IF (MEP2.EQ.0) THEN
      AZL(KA)=EXP(-(FF+FA)/RPF3)/TEMP
      AZU(KA)=EXP(-(FF-FA)/RPF3)/TEMP
      ELSE
      AZL(KA)=ASL(KA)/TEMP
      AZU(KA)=ASU(KA)/TEMP
      ENDIF
      ELSE IF (IEXT.LT.0) THEN
      IF (MEP2.EQ.0) THEN
      AZU(KA)=EXP(-(FF-FA)/RPF3)/TEMP
      ELSE
      AZU(KA)=ASU(KA)/TEMP
      ENDIF
      ELSE IF (IEXT.GT.0) THEN
      IF (MEP2.EQ.0) THEN
      AZL(KA)=EXP(-(FF+FA)/RPF3)/TEMP
      ELSE
      AZL(KA)=ASL(KA)/TEMP
      ENDIF
      ENDIF
    2 CONTINUE
      RETURN
      END
