* SUBROUTINE US0L04                ALL SYSTEMS                97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* SPECIAL LINE SEARCH WITHOUT DIRECTIONAL DERIVATIVES FOR LEAST SQUARES
* APPROXIMATION.
*
* PARAMETERS :
*  II  NA  NUMBER OF APPROXIMATED FUNCTIONS.
*  RI  AF(NA)  VECTOR CONTAINING NEW VALUES OF THE APPROXIMATED
*         FUNCTIONS.
*  RI  AFO(NA)  VECTOR CONTAINING OLD VALUES OF THE APPROXIMATED
*         FUNCTIONS.
*  RO  AFD(NA)  VECTOR CONTAINING INCREMENTS OF THE APPROXIMATED
*         FUNCTIONS.
*  II  IA(NA)  VECTOR CONTAINING TYPES OF DEVIATIONS.
*  RI  EPS1  TERMINATION TOLERANCE FOR LINE SEARCH (IN TEST ON THE
*         CHANGE OF THE FUNCTION VALUE).
*  RI  ETA9  MAXIMUM MACHINE NUMBER.
*  RO  RO  INITIAL VALUE OF THE STEPSIZE PARAMETER.
*  RO  RP  PREVIOUS VALUE OF THE STEPSIZE PARAMETER.
*  RO  R  VALUE OF THE STEPSIZE PARAMETER.
*  RI  FO  INITIAL VALUE OF THE OBJECTIVE FUNCTION.
*  RO  FP  PREVIOUS VALUE OF THE OBJECTIVE FUNCTION.
*  RO  F  VALUE OF THE OBJECTIVE FUNCTION.
*  RI  PO  INITIAL VALUE OF THE DIRECTIONAL DERIVATIVE.
*  RI  PP  PREVIOUS VALUE OF THE DIRECTIONAL DERIVATIVE.
*  RI  FMIN  LOWER BOUND FOR VALUE OF THE OBJECTIVE FUNCTION.
*  RA  PAR  AUXILIARY VARIABLE.
*  RA  PAR1  AUXILIARY VARIABLE FOR CONTROLLED SCALING OF VARIABLE
*            METRIC UPDATES.
*  RA  PAR2  AUXILIARY VARIABLE FOR CONTROLLED SCALING OF VARIABLE
*            METRIC UPDATES.
*  RA  PAR3  AUXILIARY VARIABLE.
*  RI  RMAX  MAXIMUM VALUE OF THE STEPSIZE PARAMETER.
*  RI  RMIN  MINIMUM VALUE OF THE STEPSIZE PARAMETER.
*  RI  SNORM  NORM OF THE DIRECTION VECTOR.
*  IA  KA  AUXILIARY VARIABLE.
*  IO  MODE  MODE OF LINE SEARCH. MODE=0-INITIAL STEPSIZE.
*         MODE=1-EXTRAPOLATION. MODE=2-INTERPOLATION.
*  II  KTERS  TERMINATION SELECTION. KTERS=1-PERFECT LINE SEARCH.
*         KTERS=2-GOLDSTEIN STEPSIZE. KTERS=3-WEAK WOLFE STEPSIZE.
*         KTERS=4-STRONG WOLFE STEPSIZE. KTERS=5-ARMIJO STEPSIZE.
*         KTERS=6-FIRST STEPSIZE.
*  II  MES1  SWITCH FOR EXTRAPOLATION. MES1=1-CONSTANT INCREASING OF
*         THE INTERVAL. MES1=2-EXTRAPOLATION SPECIFIED BY THE PARAMETER
*         MES. MES1=3 SUPPRESSED EXTRAPOLATION.
*  II  MES2  SWITCH FOR TERMINATION. MES2=1-NORMAL TERMINATION.
*         MES2=2-TERMINATION AFTER AT LEAST TWO STEPS (ASYMPTOTICALLY
*         PERFECT LINE SEARCH).
*  II  INITS  CHOICE OF THE INITIAL STEPSIZE. INITS=0-INITIAL STEPSIZE
*         IS SPECIFIED IN THE CALLING PROGRAM. INITS=1-UNIT INITIAL
*         STEPSIZE. INITS=2-COMBINED UNIT AND QUADRATICALLY ESTIMATED
*         INITIAL STEPSIZE. INITS=3-QUADRATICALLY ESTIMATED INITIAL
*         STEPSIZE.
*  II  MRED  MAXIMUM NUMBER OF EXTRAPOLATIONS OR INTERPOLATIONS.
*
* COMMON DATA :
*  II  KIT  NUMBER OF THE ITERATION AFTER LAST RESTART.
*  IO  MAXST  MAXIMUM STEPSIZE INDICATOR. MAXST=0 OR MAXST=1 IF MAXIMUM
*         STEPSIZE WAS NOT OR WAS REACHED.
*  IO  NRED  ACTUAL NUMBER OF EXTRAPOLATIONS OR INTERPOLATIONS.
*  II  NIT  ACTUAL NUMBER OF ITERATIONS.
*  IO  ITERS  TERMINATION INDICATOR. ITERS=0-ZERO STEP. ITERS=1-PERFECT
*         LINE SEARCH. ITERS=2 GOLDSTEIN STEPSIZE. ITERS=3-WEAK WOLFE
*         STEPSIZE. ITERS=4-STRONG WOLFE STEPSIZE.
*         ITERS=5-ARMIJO STEPSIZE. ITERS=6-FIRST STEPSIZE.
*         ITERS=7-MAXIMUM STEPSIZE. ITERS=8-UNBOUNDED FUNCTION.
*         ITERS=-1-MRED REACHED. ITERS=-2-POSITIVE DIRECTIONAL
*         DERIVATIVE. ITERS=-3-ERROR IN INTERPOLATION.
*  II  IEST  LOWER BOUND SPECIFICATION. IEST=0 OR IEST=1 IF LOWER BOUND
*         IS NOT OR IS GIVEN.
*  TO  TSXX  TEXT INFORMATION ON THE STEPSIZE OBTAINED.
*
* STATUS VARIABLES :
*  NS,ISP,TSS
*
* SUBPROGRAM USED :
*  S   UNINT4  EXTRAPOLATION OR INTERPOLATION WITHOUT DIRECTIONAL
*         DERIVATIVES.
*  S   UYPRO1  PROLOGUE.
*  S   UYEPI1  EPILOGUE.
*  S   UYSET0  STATUS DEFINITION.
*  S   UYSET2  DEGREE DEFINITION OF THE COMPUTED DERIVATIVES.
*  S   UOS0L1  PRINT OF ENTRY TO STEPSIZE SELECTION.
*  S   UOS0L2  PRINT OF EXIT FROM STEPSIZE SELECTION.
*  S   UOS0L3  PRINT OF INFORMATION DURING STEPSIZE SELECTION.
*  S   UOERR1  ERROR MESSAGES.
*
* METHOD :
* SAFEGUARDED EXTRAPOLATION AND INTERPOLATION WITH STANDARD TERMINATION
* CRITERIA.
*
      SUBROUTINE US0L04(NA,AF,AFO,AFD,IA,ETA9,EPS1,RO,RP,R,FO,FP,F,PO,
     & PP,FMIN,PAR,PAR1,PAR2,PAR3,RMAX,RMIN,SNORM,KA,MODE,KTERS,MES1,
     & MES2,INITS,MRED)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER NA,IA(NA),KA,MODE,KTERS,MES1,MES2,INITS,MRED
      REAL*8 AF(NA),AFO(NA),AFD(NA)
      REAL*8 ETA9,EPS1,RO,RP,R,FO,FP,F,PO,PP,FMIN,PAR,PAR1,PAR2,PAR3,
     & RMAX,RMIN,SNORM
      REAL*8 RL,FL,RU,FU,RTEMP
      INTEGER INIT1
      LOGICAL L1,L2,L3,L4,L7
      SAVE RL,FL,RU,FU
      IF (TSS(NS).NE.'USXX') CALL UYPRO1('USXX',0)
      GO TO (1,3) ISP(NS)
    1 CONTINUE
      ITERS=0
      CALL UOS0L1(RO,FO,SNORM,RMAX)
      IF (PO.GE.ZERO) THEN
      R=ZERO
      ITERS=-2
      TSXX='P - NEG '
      CALL UOERR1('US0L04',5-ITERS)
      GO TO 4
      ENDIF
      IF(RMAX.LE.ZERO) THEN
      TSXX='RMAX=0'
      ITERS=0
      GO TO 4
      ENDIF
      PAR=ZERO
      PAR3=ZERO
      DO 5 KA=1,NA
      IF (IA(KA).GT.0) THEN
      AFO(KA)=AF(KA)
      PAR=PAR+AFO(KA)*AFD(KA)
      PAR3=PAR3+AFD(KA)*AFD(KA)
      ENDIF
    5 CONTINUE
C
C     INITIAL STEPSIZE SELECTION
C
      IF (INITS.GT.0) THEN
      RTEMP=FMIN-F
      ELSE IF (IEST.EQ.0) THEN
      RTEMP=F-FP
      ELSE
      RTEMP=MAX(F-FP,TEN*(FMIN-F))
      ENDIF
      INIT1=ABS(INITS)
      RP=ZERO
      FP=FO
      PP=PO
      TSXX='INITIAL '
      IF (INIT1.EQ.0) THEN
      ELSE IF (INIT1.EQ.1.OR.INITS.GE.1.AND.IEST.EQ.0) THEN
      R=ONE
      TSXX='INIT-1  '
      ELSE IF (INIT1.EQ.2) THEN
      R=MIN(ONE,FOUR*RTEMP/PO)
      TSXX='INIT-2  '
      ELSE IF (INIT1.EQ.3) THEN
      R=MIN(ONE, TWO*RTEMP/PO)
      TSXX='INIT-3  '
      ELSE IF (INIT1.EQ.4) THEN
      R=TWO*RTEMP/PO
      TSXX='INIT-4  '
      ENDIF
      RTEMP=R
      R=MAX(R,RMIN)
      R=MIN(R,RMAX)
      MODE=0
      RL=ZERO
      FL=FO
      RU=ZERO
      FU=FO
C
C     NEW STEPSIZE SELECTION (EXTRAPOLATION OR INTERPOLATION)
C
    2 CALL UNINT4(NA,AF,AFO,AFD,IA,ETA9,R,PAR,PAR3,MES1,MODE)
      IF (MODE.EQ.1) THEN
      NRED=NRED-1
      R=MIN(R,RMAX)
      ELSE IF (MODE.EQ.2) THEN
      NRED=NRED+1
      ENDIF
C
C     COMPUTATION OF THE NEW FUNCTION VALUE
C
      CALL UYSET2
      CALL UYSET0(1,2)
      RETURN
    3 CONTINUE
      CALL UOS0L3(R,F,SNORM,MODE)
      IF (MODE.EQ.0) THEN
      PAR1=ZERO
      PAR2=F-FO
      ENDIF
      IF (ITERS.NE.0) GO TO 4
      IF (F.LE.FMIN) THEN
      ITERS=7
      TSXX='F BOUND '
      GO TO 4
      ELSE
      L1=R.LE.RMIN.AND.NIT.NE.KIT
      L2=R.GE.RMAX
      L3=F-FO.LE.EPS1*R*PO
      L4=F-FO.GE.(ONE-EPS1)*R*PO.OR.MES2.EQ.2.AND.MODE.EQ.2
      L7=MES2.LE.2.OR.MODE.NE.0
      MAXST=0
      IF (L2) MAXST=1
      ENDIF
C
C     TEST ON TERMINATION
C
      IF (L1.AND..NOT.L3) THEN
      ITERS=0
      TSXX='ZERO  ST'
      GO TO 4
      ELSE IF (L2.AND..NOT.L4) THEN
      ITERS=7
      TSXX='R BOUND '
      GO TO 4
      ELSE IF (L3.AND.MES1.EQ.3) THEN
      ITERS=5
      TSXX='ARMIJO  '
      GO TO 4
      ELSE IF (L3.AND.L4.AND.L7) THEN
      ITERS=2
      TSXX='GOLDST. '
      GO TO 4
      ELSE IF (KTERS.LT.0.OR.KTERS.EQ.6.AND.L7) THEN
      ITERS=6
      TSXX='FIRST ST'
      GOTO 4
      ELSE IF (ABS(NRED).GE.MRED) THEN
      ITERS=-1
      TSXX='MAX  INT'
      CALL UOERR1('US0L04',5-ITERS)
      GO TO 4
      ELSE
      RP=R
      FP=F
      MODE=MAX(MODE,1)
      ENDIF
      IF (MODE.EQ.1) THEN
C
C     INTERVAL CHANGE AFTER EXTRAPOLATION
C
      RL=RU
      FL=FU
      RU=R
      FU=F
      IF (.NOT.L3) THEN
      NRED=0
      MODE=2
      ENDIF
      ELSE
C
C     INTERVAL CHANGE AFTER INTERPOLATION
C
      IF (.NOT.L3) THEN
      RU=R
      FU=F
      ELSE
      RL=R
      FL=F
      ENDIF
      ENDIF
      GOTO 2
    4 IF (MAXST.EQ.1) NMAX=NMAX+1
      CALL UOS0L2(R,F,SNORM)
      CALL UYEPI1(0)
      RETURN
      END
