* SUBROUTINE US0L07                ALL SYSTEMS                97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
*  LINE SEARCH WITHOUT DIRECTIONAL DERIVATIVES BASED ON A SQP FILTER.
*
* PARAMETERS :
*  IU  KS  NUMBER OF FILTER SAMPLES.
*  II  MS  MAXIMUM NUMBER OF FILTER SAMPLES.
*  RU  FS(MS,2)  ARRAY CONTAINING FILTER SAMPLES.
*  RI  EPS1  TERMINATION TOLERANCE FOR LINE SEARCH (IN TEST ON THE
*         CHANGE OF THE FUNCTION VALUE).
*  RI  EPS2  TERMINATION TOLERANCE FOR LINE SEARCH (IN TEST ON THE
*         CHANGE OF THE CONSTRAINT VIOLATION).
*  RO  RO  INITIAL VALUE OF THE STEPSIZE PARAMETER.
*  RO  R  VALUE OF THE STEPSIZE PARAMETER.
*  RO  FF  VALUE OF THE OBJECTIVE FUNCTION.
*  RO  FC  L-INFINITY NORM OF THE CONSTRAINT VIOLATION.
*  RI  FMIN  LOWER BOUND FOR VALUE OF THE OBJECTIVE FUNCTION.
*  RI  RMIN  MINIMUM VALUE OF THE STEPSIZE PARAMETER.
*  RI  RMAX  MAXIMUM VALUE OF THE STEPSIZE PARAMETER.
*  RI  SNORM  NORM OF THE DIRECTION VECTOR.
*  IO  MODE  MODE OF LINE SEARCH. MODE=0-INITIAL STEPSIZE.
*         MODE=1-EXTRAPOLATION. MODE=2-INTERPOLATION.
*  II  KTERS  TERMINATION SELECTION. KTERS=1-PERFECT LINE SEARCH.
*         KTERS=2-GOLDSTEIN STEPSIZE. KTERS=3-WEAK WOLFE STEPSIZE.
*         KTERS=4-STRONG WOLFE STEPSIZE. KTERS=5-ARMIJO STEPSIZE.
*         KTERS=6-FIRST STEPSIZE.
*  II  INITS  CHOICE OF THE INITIAL STEPSIZE. INITS=0-INITIAL STEPSIZE
*         IS SPECIFIED IN THE CALLING PROGRAM. INITS=1-UNIT INITIAL
*         STEPSIZE. INITS=2-COMBINED UNIT AND QUADRATICALLY ESTIMATED
*         INITIAL STEPSIZE. INITS=3-QUADRATICALLY ESTIMATED INITIAL
*         STEPSIZE.
*  II  MRED  MAXIMUM NUMBER OF EXTRAPOLATIONS OR INTERPOLATIONS.
*
* COMMON DATA :
*  IO  MAXST  MAXIMUM STEPSIZE INDICATOR. MAXST=0 OR MAXST=1 IF MAXIMUM
*         STEPSIZE WAS NOT OR WAS REACHED.
*  IO  NRED  ACTUAL NUMBER OF EXTRAPOLATIONS OR INTERPOLATIONS.
*  IO  ITERS  TERMINATION INDICATOR. ITERS=0-ZERO STEP. ITERS=1-PERFECT
*         LINE SEARCH. ITERS=2 GOLDSTEIN STEPSIZE. ITERS=3-WEAK WOLFE
*         STEPSIZE. ITERS=4-STRONG WOLFE STEPSIZE.
*         ITERS=5-ARMIJO STEPSIZE. ITERS=6-FIRST STEPSIZE.
*         ITERS=7-MAXIMUM STEPSIZE. ITERS=8-UNBOUNDED FUNCTION.
*         ITERS=-1-MRED REACHED. ITERS=-2-POSITIVE DIRECTIONAL
*         DERIVATIVE. ITERS=-3-ERROR IN INTERPOLATION.
*  TO  TSXX  TEXT INFORMATION ON THE STEPSIZE OBTAINED.
*
* STATUS VARIABLES :
*  NS,ISP,TSS
*
* SUBPROGRAM USED :
*  S   UYPRO1  PROLOGUE.
*  S   UYEPI1  EPILOGUE.
*  S   UYSET0  STATUS DEFINITION.
*  S   UYSET2  DEGREE DEFINITION OF THE COMPUTED DERIVATIVES.
*  S   UOS0F1  PRINT OF ENTRY TO STEPSIZE SELECTION.
*  S   UOS0F2  PRINT OF EXIT FROM STEPSIZE SELECTION.
*  S   UOS0F3  PRINT OF INFORMATION DURING STEPSIZE SELECTION.
*  S   UOERR1  ERROR MESSAGES.
*
* METHOD :
* SAFEGUARDED EXTRAPOLATION AND INTERPOLATION WITH EXTENDED TERMINATION
* CRITERIA.
*
      SUBROUTINE US0L07(KS,MS,FS,EPS1,EPS2,RO,R,FF,FC,FMIN,RMIN,RMAX,
     & SNORM,MODE,KTERS,INITS,MRED)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER KS,MS,MODE,KTERS,INITS,MRED
      REAL*8 FS(MS,2),EPS1,EPS2,RO,R,FF,FC,FMIN,RMIN,RMAX,SNORM
      INTEGER I,J
      REAL*8 CON
      PARAMETER(CON=1.0D 4)
      IF (TSS(NS).NE.'USXX') CALL UYPRO1('USXX',0)
      GO TO (1,3) ISP(NS)
    1 CONTINUE
      CALL UOS0F1(RO,FF,FC,SNORM,RMAX)
      IF(RMAX.LE.ZERO) THEN
      TSXX='RMAX=0'
      ITERS=0
      GO TO 10
      ENDIF
C
C     INITIAL STEPSIZE SELECTION
C
      TSXX='INITIAL '
      IF (INITS.EQ.0) THEN
      ELSE
      R=ONE
      TSXX='INIT-1  '
      ENDIF
      MAXST=0
      R=MAX(R,RMIN)
      R=MIN(R,RMAX)
      IF (R.GE.RMAX) MAXST=1
      MODE=0
C
C     NEW STEPSIZE SELECTION (EXTRAPOLATION OR INTERPOLATION)
C
    2 IF (MODE.NE.0) THEN
      R=HALF*R
      NRED=NRED+1
      ENDIF
C
C     COMPUTATION OF THE NEW FUNCTION VALUE
C
      CALL UYSET2
      CALL UYSET0(1,2)
      RETURN
    3 CONTINUE
      IF (KTERS.LT.0.OR.KTERS.EQ.6) GO TO 10
      CALL UOS0F3(R,FF,FC,SNORM,MODE)
      DO 4 I=1,KS
      IF (FF.GT.FS(I,1)-EPS1*FS(I,2).AND.FC.GT.EPS2*FS(I,2)) GO TO 9
C      IF (FF.GT.CON*ABS(FS(I,1))) GO TO 9
C      IF (FC.GT.CON*ABS(FS(I,2))) GO TO 9
    4 CONTINUE
      I=1
    5 IF (I.GT.KS) GO TO 7
      IF (FS(I,1).GT.FF.AND.FS(I,2).GT.FC) THEN
      DO 6 J=I,KS-1
      FS(J,1)=FS(J+1,1)
      FS(J,2)=FS(J+1,2)
    6 CONTINUE
      KS=KS-1
      ENDIF
      I=I+1
      GO TO 5
    7 IF (KS.GE.MS) THEN
      DO 8 J=1,KS-1
      FS(J,1)=FS(J+1,1)
      FS(J,2)=FS(J+1,2)
    8 CONTINUE
      KS=KS-1
      ENDIF
      KS=KS+1
      FS(KS,1)=FF
      FS(KS,2)=FC
      ITERS=4
      TSXX='SUCCESS '
      GO TO 10
    9 NRED=0
      MODE=2
      IF(ABS(NRED).GE.MRED) THEN
      ITERS=-1
      TSXX='MAX  INT'
      CALL UOERR1('US0L02',5-ITERS)
      GO TO 10
      ELSE IF (FF.LE.FMIN) THEN
      ITERS=7
      TSXX='F BOUND '
      GO TO 10
      ELSE IF (R.LE.RMIN) THEN
      ITERS=0
      TSXX='ZERO  ST'
      GO TO 10
      ELSE
      GO TO 2
      ENDIF
   10 IF (MAXST.EQ.1) NMAX=NMAX+1
      CALL UOS0F2(R,FF,FC,SNORM)
      CALL UYEPI1(0)
      RETURN
      END
