* SUBROUTINE US1L06                ALL SYSTEMS                97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 VL : ORIGINAL VERSION
*
* PURPOSE :
*  SPECIAL LINE SEARCH FOR NONSMOOTH VARIABLE METRIC METHODS.
*
* PARAMETERS :
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  II  MB  DECLARED NUMBER OF LINEAR APPROXIMATED FUNCTIONS
*  II  NB  CURRENT NUMBER OF LINEAR APPROXIMATED FUNCTIONS.
*  RU  X(N)  VECTOR OF VARIABLES.
*  RO  G(N)  SUBGRADIENT OF THE OBJECTIVE FUNCTION.
*  RI  S(N)  DIRECTION VECTOR.
*  RU  U(N)  PREVIOUS VECTOR OF VARIABLES.
*  RI  V(N)  PREVIOUS SUBGRADIENT OF THE OBJECTIVE FUNCTION.
*  RI  AF(4*MB)  VECTOR OF BUNDLE FUNCTIONS VALUES.
*  RI  AG(N*MB)  MATRIX WHOSE COLUMNS ARE BUNDLE SUBGRADIENTS.
*  RI  AY(N*MB)  MATRIX WHOSE COLUMNS ARE VARIABLE VECTORS.
*  RO  T  VALUE OF THE STEPSIZE PARAMETER.
*  RU  FL  PREVIOUS VALUE OF THE OBJECTIVE FUNCTION.
*  RO  F  VALUE OF THE OBJECTIVE FUNCTION.
*  RI  DF  PREVIOUS VALUE OF THE DIRECTIONAL DERIVATIVE.
*  RU  P  DIRECTIONAL DERIVATIVE.
*  RI  TMIN  MINIMUM VALUE OF THE STEPSIZE PARAMETER.
*  RI  TMAX  MAXIMUM VALUE OF THE STEPSIZE PARAMETER.
*  RI  GNORM  NORM OF THE GRADIENT VECTOR.
*  RI  SNORM  EUCLIDEAN NORM OF THE DIRECTION VECTOR.
*  RI  ETA5  DISTANCE MEASURE PARAMETER.
*  RI  ETA9  MAXIMUM FOR REAL NUMBERS.
*  IU  NNEW  NEWTON DIRECTION CASE COUNTER.
*  IU  NV  AUXILIARY FUNCTION EVALUATIONS COUNTER.
*  IO  LQ  NEWTON DIRECTION INDICATOR.
*
* COMMON DATA :
*  IO  ITERS  NULL STEP INDICATOR. ITERS=0-NULL STEP. ITERS=1-DESCENT
*         STEP.
*  TO  TSXX  TEXT INFORMATION ON THE STEPSIZE OBTAINED.
*
* STATUS VARIABLES :
*  NS,ISP,TSS
*
* SUBPROGRAMS USED :
*  S   UXVCOP  COPYING OF A VECTOR.
*  S   UXVDIR  VECTOR AUGMENTED BY THE SCALED VECTOR.
*
* METHOD :
* SPECIAL METHOD OF STEP LENGTH DETERMINATION.
*
      SUBROUTINE US1L06(N,MB,NB,X,G,S,U,V,AF,AG,AY,T,FL,F,DF,P,TMIN,
     & TMAX,GNORM,SNORM,ETA5,ETA9,NNEW,NV,LQ)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER LQ,MB,NB,N,NV,NNEW
      REAL*8 AF(MB*4),AG(N*MB),AY(N*MB),G(N),S(N),U(N),V(N),X(N)
      REAL*8 T,FL,F,DF,P,TMIN,TMAX,GNORM,SNORM,ETA5,ETA9
      REAL*8 DT,W
      INTEGER I,K
      SAVE DT
      SAVE K
      IF (TSS(NS).NE.'USXX') CALL UYPRO1('USXX',1)
      GOTO (1,8) ISP(NS)
    1 CONTINUE
      CALL UOS1L1(ZERO,FL,DF,GNORM,SNORM,TMAX)
      FL=F
      T=MIN(ONE,TMAX)
      CALL UNSTP1(N,MB,NB,X,S,AF,AG,AY,T,F,DF,TMAX,ETA5,ETA9,LQ)
      TSXX='INITIAL '
      IF (LQ.LT.0) NNEW=NNEW+1
      T=MAX(T,TMIN)
      K=0
      DT=T
C
C     FUNCTION AND GRADIENT EVALUATION AT A NEW POINT
C
    7 CALL UYSET2
      CALL UYSET0(1,2)
      RETURN
    8 CONTINUE
      CALL UOS1L3(T,F,P,SNORM,K)
      NV=NV+1
C
C     NULL/DESCENT STEP TEST (ITERS=0/1)
C
      ITERS=1
      TSXX='DESCENT '
      IF (F.GT.FL+1.0D-4*T*DF) THEN
      ITERS=0
      TSXX='NULL  ST'
      ENDIF
      W=ZERO
      DO 9 I=1,N
    9 W=W+ABS(G(I)-V(I))
      T=DT
      IF (W.EQ.ZERO.AND.ITERS.NE.0.AND.K.LE.9) THEN
C
C     EXTRAPOLATION
C
        TSXX='EXTRAPOL'
        K=K+1
        T=DT
        DT=DT+T
        FL=F
        CALL UXVCOP(N,X,U)
        GO TO 7
      ENDIF
      T=DT
      IF (T.GE.TMAX) NMAX=NMAX+1
      CALL UOS0L2(T,F,SNORM)
      CALL UYEPI1(1)
      RETURN
      END
