* SUBROUTINE UUDVI1                ALL SYSTEMS                97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 VL : ORIGINAL VERSION
*
* PURPOSE :
*  VARIABLE METRIC UPDATE OF A DENSE SYMMETRIC POSITIVE DEFINITE MATRIX
*  WITH THE POSSIBILITY OF MATRIX INNOVATION.
*
* PARAMETERS :
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  II  MB  DECLARED NUMBER OF LINEAR APPROXIMATED FUNCTIONS.
*  II  NB  CURRENT NUMBER OF LINEAR APPROXIMATED FUNCTIONS.
*  RU  S(N)  DIRECTION VECTOR.
*  RU  U(N)  AUXILIARY VECTOR.
*  RA  V(N)  AUXILIARY VECTOR.
*  RU  H(M)  POSITIVE DEFINITE APPROXIMATION OF THE INVERSE HESSIAN
*         MATRIX.
*  RI  AF(4*MB)  VECTOR OF VALUES OF BUNDLE FUNCTIONS.
*  RI  AG(N*MB)  MATRIX WHOSE COLUMNS ARE BUNDLE SUBGRADIENTS.
*  RI  AY(N*MB)  MATRIX WHOSE COLUMNS ARE VARIABLE VECTORS.
*  RI  GAM  PARAMETER FOR UPDATE.
*  RI  T  VALUE OF THE STEPSIZE PARAMETER.
*  II  MET  INDICATOR FOR UPDATE. MET=0-NO UPDATE. MET=1-VARIABLE
*         METRIC UPDATE.
*
* COMMON DATA :
*  II  LIT  NUMBER OF THE FIRST ITERATION.
*  II  NIT  ACTUAL NUMBER OF ITERATIONS.
*
* SUBPROGRAMS USED :
*  S   UXDPGS  SCALING OF A DENSE SYMMETRIC POSITIVE DEFINITE MATRIX.
*  S   UXDSMI  DENSE SYMMETRIC MATRIX A IS SET TO THE UNIT MATRIX.
*  S   UXDSMM  MATRIX-VECTOR PRODUCT.
*  RF  UXVDOT  DOT PRODUCT OF VECTORS.
*  RF  UXVSAB  L1 NORM OF A VECTOR.
*
* METHOD :
* BFGS, FOR INNOVATION WITH POWELL'S CORRECTION.
*
      SUBROUTINE UUDVI1(N,MB,NB,S,U,V,H,AF,AG,AY,GAM,T,MET)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER N,MB,NB,MET
      REAL*8 S(N),U(N),V(N),H(N*(N+1)/2),AF(MB),AG(N*MB),AY(N*MB)
      REAL*8 GAM,T
      INTEGER I,IK,IL,K,L
      REAL*8 P,Q,W,Z
      IF (MET.EQ.0) RETURN
      IF (GAM.LE.ZERO) GO TO 30
      CALL UXDSMI(N,H)
      CALL UXDPGS(N,H,GAM)
      L=1
      K=1
  10  K=K+1
      IF (AF(K).NE.ZERO) GO TO 10
  20  Z=ZERO
      W=ZERO
      DO 2 I=1,N
      IK=K*N-N+I
      IL=L*N-N+I
      P=AY(IK)-AY(IL)
      Q=AG(IK)-AG(IL)
      S(I)=P
      U(I)=Q
      W=W+ABS(Q)
    2 Z=Z+P*Q
      IF (W.NE.ZERO) THEN
        CALL UPDATE(N,NIT-LIT,H,S,U,V,ONE,Z)
        TUXX='BFGS    '
        NUPD=NUPD+1
      ENDIF
      IF (L.LT.K) THEN
        I=L
        L=K
        K=I
      ENDIF
      K=K+1
      IF (K.GE.NB) THEN
        RETURN
        TUXX='NO UPDT.'
      ENDIF
      IF (L.EQ.K) GO TO 10
      GO TO 20
  30  CONTINUE
      CALL UPDATE(N,NIT-LIT,H,S,U,V,T,-GAM)
      TUXX='BFGS    '
      NUPD=NUPD+1
      RETURN
      END
      SUBROUTINE UPDATE(N,NIT,H,S,U,V,T,Z)
      INTEGER N,NIT
      REAL*8 H(N*(N+1)/2),S(N),U(N),V(N),T,Z
      INTEGER I,J,L
      REAL*8 GAM,Q,W,UXVDOT
      CALL UXDSMM(N,H,U,V)
      W=UXVDOT(N,U,V)
C
C   POWELL'S CORRECTION
C
      IF (Z.LE.0.0D 0) THEN
        Q=0.99D 0*W/(W-T*Z)
        DO 7 I=1,N
   7    S(I)=Q*S(I)+(1.0D 0-Q)/T*V(I)
        Z=W/(1.0D 2*T)
      ENDIF
      GAM=1.0D 0
      IF (NIT.LE.1) THEN
        Q=1.0D 0
        IF (W.NE.0.0D 0) Q=T*ABS(Z)/W
        IF ((Q-0.3333333333D 0)*(Q-3.0D 0).GT.0.0D 0)
     &   GAM=MIN(3.0D 0,MAX(2.0D-2,Q))
      ENDIF
      DO 8 I=1,N
   8  U(I)=(V(I)-5D-1*(T*ABS(Z)/GAM+W)/Z*S(I))/Z
      L=1
      DO 9 I=1,N
      DO 9 J=1,I
      H(L)=(H(L)-U(I)*S(J)-S(I)*U(J))*GAM
   9  L=L+1
      RETURN
      END
