* SUBROUTINE UULBI2                ALL SYSTEMS                05/12/01
C PORTABILITY : ALL SYSTEMS
C 05/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* LIMITED STORAGE VARIABLE METRIC UPDATE.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  MF  MAXIMUM NUMBER OF VARIABLE METRIC STEPS.
*  RU  XM(NF*MF)  SET OF VECTOR OF VARIABLES DIFFERENCES.
*  RU  GM(NF*MF)  SET OF GRADIENT DIFFERENCES.
*  RU  XR(MF*(MF+1)/2)  MATRIX TRANS(XM)*XM STORED IN PACKED FORM.
*  RU  GR(MF*(MF+1)/2)  MATRIX TRANS(GM)*XM STORED IN PACKED FORM.
*  RO  HR(MF*(MF+1)/2)  MATRIX IN UPDATE REPRESENTATION.
*  RU  XO(NF)  VECTOR OF VARIABLES DIFFERENCE.
*  RI  GO(NF)  GRADIENT DIFFERENCE.
*  RO  PAR  SCALING PARAMETER.
*  RI  ETA0  MACHINE PRECISION.
*  IU  INF  DECOMPOSITION INDICATOR. INF=0 IF THE MATRIX IS POSITIVE
*         DEFINITE.
*  II  MET  METHOD SELECTION. MET=1-BFGS UPDATE. MET=4-RANK ONE UPDATE.
*  II  MET1  SCALING OPTION. MET1=1-NO SCALING. MET1=2-PRELIMINARY
*         SCALING.
*
* COMMON DATA :
*  IO  ITERH  TERMINATION INDICATOR. ITERH<0-BAD DECOMPOSITION.
*         ITERH=0-SUCCESSFUL UPDATE. ITERH>0-NONPOSITIVE PARAMETERS.
*  II  NIT  ACTUAL NUMBER OF ITERATIONS.
*  II  KIT  NUMBER OF THE ITERATION AFTER LAST RESTART.
*  TO  TUXX  TEXT INFORMATION ON THE CORRECTION USED.
*
* SUBPROGRAMS USED :
*  S   UXDPGF  GILL-MURRAY DECOMPOSITION OF A DENSE SYMMETRIC MATRIX
*         WITH CONTROLLING OF POSITIVE DEFINITENESS.
*  S   UXDPGT  TRANSFORMATION OF THE GILL-MURRAY DECOMPOSITION.
*  S   UXDSDP  GENERALIZED DOT PRODUCT OF TWO VECTORS.
*  RF  UXVDOT  DOT PRODUCT OF VECTORS.
*  S   UXVSCL  SCALING OF A VECTOR.
*  S   UOERR1  ERROR MESAGES.
*
* METHOD :
* LIMITED STORAGE VARIABLE METRIC METHODS.
*
      SUBROUTINE UULBI2(NF,MF,MF1,MM,MM1,XM,GM,XR,GR,HR,XM1,GM1,GM2,XR1,
     & GR1,HR1,XO,GO,SO,U1,U2,R,PO,PAR,ETA0,INF,MET,MET1,MET2,JOB)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER NF,MF,MF1,MM,MM1,INF,MET,MET1,MET2,JOB
      REAL*8 XM(NF*MF),GM(NF*MF),XR(MF*(MF+1)/2),GR(MF*(MF+1)/2),
     & HR(MF*(MF+1)/2),XM1(NF*MF),GM1(NF*MF),GM2(NF*MF),
     & XR1(MF*(MF+1)/2),GR1(MF*(MF+1)/2),HR1(MF*(MF+1)/2),XO(NF),GO(NF),
     & SO(NF),U1(NF),U2(NF),R,PO,PAR,ETA0
      INTEGER I,II,I1,I2,J1,J2
      REAL*8 UXVDOT
      REAL*8 A,B,ALF,BET
      ITERH=0
      B=UXVDOT(NF,XO,GO)
      IF (B.LE.ZERO) GO TO 6
      IF (JOB.EQ.0) THEN
      A=UXVDOT(NF,GO,GO)
      IF (MM.GE.MF) THEN
      I1=1
      J1=1
      DO 1 II=1,MM-1
      I2=I1+NF
      J2=J1+II
      CALL UXVCOP(NF,XM(I2),XM(I1))
      CALL UXVCOP(NF,GM(I2),GM(I1))
      CALL UXVCOP(II,XR(J2+1),XR(J1))
      CALL UXVCOP(II,GR(J2+1),GR(J1))
      I1=I2
      J1=J2
    1 CONTINUE
      J1=J1-1
      ELSE
      I1=MM*NF+1
      J1=MM*(MM+1)/2
      MM=MM+1
      ENDIF
      CALL UXVCOP(NF,XO,XM(I1))
      CALL UXVCOP(NF,GO,GM(I1))
      I1=1
      DO 2 II=1,MM-1
      XR(J1+II)=UXVDOT(NF,GM(I1),GO)
      GR(J1+II)=UXVDOT(NF,XM(I1),GO)
      I1=I1+NF
    2 CONTINUE
      XR(J1+MM)=A
      GR(J1+MM)=B
      PAR=ONE
      IF (MET1.EQ.1) PAR=(B/A)**0.7D 0
      IF (MET1.EQ.2) PAR=SQRT(-R*PO/A)**1.1D 0
      IF (MET1.EQ.3) PAR=-R*PO/B
      CALL UXVSCL(J1+MM,PAR,XR,HR)
      II=0
      DO 3 I=1,MM
      II=II+I
      HR(II)=HR(II)+GR(II)
    3 CONTINUE
      TUXX='BFGS    '
      NUPD=NUPD+1
      ELSE
      IF (MET2.EQ.0) GO TO 6
      IF (B+R*PO.LE.ZERO) GO TO 6
      CALL UXLSIN(NF,MM,0,XM,GM,GR,HR,XM1,GM1,HR1,U1,U2,GO,SO,PAR)
      A=UXVDOT(NF,GO,SO)
      IF (MM1.GE.MF1) THEN
      IF (MET2.EQ.1) GO TO 6
      I1=1
      J1=1
      DO 4 II=1,MM1-1
      I2=I1+NF
      J2=J1+II
      CALL UXVCOP(NF,XM1(I2),XM1(I1))
      CALL UXVCOP(NF,GM1(I2),GM1(I1))
      IF (MET.NE.0) CALL UXVCOP(NF,GM2(I2),GM2(I1))
      CALL UXVCOP(II,XR1(J2+1),XR1(J1))
      CALL UXVCOP(II,GR1(J2+1),GR1(J1))
      I1=I2
      J1=J2
    4 CONTINUE
      J1=J1-1
      ELSE
      I1=MM1*NF+1
      J1=MM1*(MM1+1)/2
      MM1=MM1+1
      ENDIF
      CALL UXVCOP(NF,XO,XM1(I1))
      CALL UXVCOP(NF,SO,GM1(I1))
      IF (MET.NE.0) CALL UXVCOP(NF,GO,GM2(I1))
      I1=1
      DO 5 II=1,MM1-1
      XR1(J1+II)=UXVDOT(NF,GM1(I1),GO)
      GR1(J1+II)=UXVDOT(NF,XM1(I1),GO)
      I1=I1+NF
    5 CONTINUE
      XR1(J1+MM1)=A
      GR1(J1+MM1)=B
      CALL UXVDIR(J1+MM1,-ONE,GR1,XR1,HR1)
      ALF=ETA0
      CALL UXDPGF(MM1,HR1,INF,ALF,BET)
      CALL UXDPGT(MM1,HR1)
      TUXX='RANK ONE'
      NUPD=NUPD+1
      ENDIF
      RETURN
    6 CONTINUE
      TUXX='SKIP UP.'
      RETURN
      END
