* SUBROUTINE UULSP4                ALL SYSTEMS                06/12/01
C PORTABILITY : ALL SYSTEMS
C 06/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* LIMITED STORAGE VARIABLE METRIC UPDATE.
*
* PARAMETERS :
*  II  N  NUMBER OF VARIABLES (NUMBER OF ROWS OF THE MATRIX XM).
*  IU  M  NUMBER OF COLUMNS OF THE MATRIX XM.
*  II  MF MAXIMUM NUMBER OF VARIABLE METRIC STEPS.
*  RU  XM(N*M)  RECTANGULAR MATRIX FOR THE PRODUCT FORM VM
*         METHOD (STORED COLUMNWISE): BAR(H)=XM*TRANS(XM).
*  RA  GR(M)  MATRIX TRANS(XM)*GO.
*  RU  XO(N)  VECTOR OF VARIABLES DIFFERENCE.
*  RU  GO(N)  GRADIENT DIFFERENCE.
*  RO  A     OUTPUT PARAMETER BAR(A)=TRANS(GO)*BAR(H)*GO.
*  RO  B     OUTPUT PARAMETER B=TRANS(GO)*XO.
*
* COMMON DATA :
*  IO  ITERH  TERMINATION INDICATOR. ITERH<0-BAD DECOMPOSITION.
*         ITERH=0-SUCCESSFUL UPDATE. ITERH>0-NONPOSITIVE PARAMETERS.
*  TO  TUXX  TEXT INFORMATION ON THE CORRECTION USED.
*
* SUBPROGRAMS USED :
*  S   UXDRMM  MULTIPLICATION OF A ROWWISE STORED DENSE RECTANGULAR
*         MATRIX BY A VECTOR.
*  RF  UXVDOT  DOT PRODUCT OF VECTORS.
*  S   UXVSCL  SCALING OF A VECTOR.
*
* METHOD :
* SHIFTED BFGS METHOD IN THE PRODUCT FORM.
*
      SUBROUTINE UULSP4(N,M,MF,XM,GR,XO,GO,A,B)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER N,M,MF
      REAL*8 XM(N*MF),GR(MF),XO(N),GO(N),A,B
      INTEGER I,J,L,MN
      REAL*8 POM,GAM,RHO,TMP,UXVDOT
      IF (M.GE.MF) RETURN
      CALL UOU1D1(N,XO,GO)
      RHO=1.0D 0
      GAM=1.0D 0
      B=UXVDOT(N,XO,GO)
      IF (B.LE.ZERO) THEN
      ITERH=2
      TUXX='B - NEG.'
      GO TO 22
      ENDIF
      CALL UXDRMM(N,M,XM,GO,GR)
      A=UXVDOT(M,GR,GR)
C
C     BFGS-BASED SHIFTED BFGS UPDATE
C
      POM=ONE
      TUXX='BFGS    '
      NUPD=NUPD+1
      MN=M*N
      DO 1 I=1,N
      XM(MN+I)=XO(I)*SQRT(RHO/B)
    1 CONTINUE
      L=-N
      DO 2 J=1,M
      L=L+N
      TMP=XM(J)/SQRT(RHO*B)
      DO 2 I=1,N
   2  XM(L+I)=SQRT(GAM)*(XM(L+I)-XM(MN+I)*TMP)
      M=M+1
      IF (GAM.EQ.ONE) GO TO 22
C
C     SCALING
C
      CALL UXVSCL(N*M,SQRT(GAM),XM,XM)
      NSCL=NSCL+1
   22 CONTINUE
      ITERH=0
C      CALL UOU1D3(N,H,S,RHO,GAM,PAR,A,B,C,POM,ETA9)
      RETURN
      END
