* SUBROUTINE UULSP5                ALL SYSTEMS                08/12/01
C PORTABILITY : ALL SYSTEMS
C 08/12/01 VL : ORIGINAL VERSION
*
* PURPOSE :
* LIMITED-STORAGE VARIABLE-METRIC STARTING BROYDEN CLASS UPDATE.
*
* PARAMETERS :
*  II  N  NUMBER OF VARIABLES (NUMBER OF ROWS OF XM, GM).
*  II  M  NUMBER OF COLUMNS OF XM, GM.
*  II  MM  MAXIMUM NUMBER OF COLUMNS OF XM, GM.
*  RU  XM(N*MM)  RECTANGULAR VM MATRIX STORED COLUMNWISE,
*         H=ZET*I-GM*TRANS(GM)+XM*TRANS(XM).
*  RU  GM(N*MM)  RECTANGULAR VM MATRIX STORED COLUMNWISE.
*  RI  XR(MM)  VECTOR TRANS(XM)*GO.
*  RI  GR(MM)  VECTOR TRANS(GM)*GO.
*  RI  XO(N)  VECTOR OF VARIABLES DIFFERENCE.
*  RI  GO(N)  VECTOR OF GRADIENT DIFFERENCE.
*  RA  S(N)  AUXILIARY VECTOR.
*  RA  XS(N)  AUXILIARY VECTOR.
*  RI  R  STEPSIZE PARAMETER.
*  RI  ZET  INPUT PARAMETER.
*  RI  RHO  NONQUADRATIC CORRECTION PARAMETER.
*  RI  ETA  BROYDEN PARAMETER.
*
* SUBPROGRAMS USED :
*  S   UXDCMM  RECTANGULAR MATRIX BY VECTOR MULTIPLICATION.
*  S   UXDRMM  RECTANGULAR MATRIX BY VECTOR MULTIPLICATION.
*  RF  UXVDOT  DOT PRODUCT OF TWO VECTORS.
*
* METHOD :
*
      SUBROUTINE UULSP5(N,M,MM,XM,GM,XR,GR,XO,GO,S,XS,R,ZET,RHO,ETA)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
C
C     STARTING BC UPDATE FOR MET=1/3
C
      INTEGER M,MM,N,I,J,L
      REAL*8 GM(N*MM),XM(N*MM),XR(MM),GR(MM),XO(N),GO(N),S(N),XS(N),R,
     & ZET,RHO,ETA
      REAL*8 ALF,A,AC,AH,AP,B,D,ET,GAM,OM,SN,UXVDOT
      B=UXVDOT(N,XO,GO)
      IF (B.LE.ZERO) THEN
      ITERH=2
      TUXX='B - NEG.'
      RETURN
      ENDIF
      CALL UXDRMM(N,M,XM,GO,XR)
      AH=UXVDOT(N,GO,GO)
      AP=UXVDOT(M,XR,XR)
      CALL UXDRMM(N,M,GM,GO,GR)
      CALL UXDCMM(N,M,GM,GR,XS)
      AC=ZET*AH-UXVDOT(M,GR,GR)
      IF (AC.LE.ZERO) THEN
      ITERH=1
      TUXX='A - NEG.'
      RETURN
      ENDIF
      A=AC+AP
      GAM=B/A
C      IF (GAM.LT.1.0D-3) GAM=B/AC
      ET=ETA
      IF (RHO/GAM*B/A.GT.1.0D 0) ET=MIN(ET,B/(B-A*GAM/RHO)*0.95D 0)
      D=ET+(B/A)/GAM*RHO*(1.0D 0-ET)
      CALL UXDCMM(N,M,XM,XR,S)
      OM=RHO/GAM*ET+AC/B*D
      ALF=B/A*(ET-1.0D 0)/(ET+SQRT(D))
      DO 2 I=1,N
        L=I
        SN=XO(I)-ALF*(ZET*GO(I)-XS(I)+S(I))
        DO 1 J=1,M
          GM(L)=SQRT(GAM)*GM(L)
          XM(L)=SQRT(GAM)*(XM(L)-XR(J)*SN/B*SQRT(D))
          L=L+N
    1   CONTINUE
        GM(L)=(ZET*GO(I)-XS(I))*SQRT(GAM*D/(B*OM))
        XM(L)=SN*SQRT(GAM*OM/B)-GM(L)
    2 CONTINUE
      ZET=ZET*GAM
      M=M+1
      TUXX='BR CLASS'
      NUPD=NUPD+1
      NSCL=NSCL+1
      ITERH=0
      RETURN
      END
