* SUBROUTINE UUSSD2                ALL SYSTEMS                97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* INITIATION OF A DENSE SYMMETRIC POSITIVE DEFINITE MATRIX.
*
* PARAMETERS :
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  II  NA  NUMBER OF APPROXIMATED FUNCTIONS.
*  II  MA  NUMBER OF NONZERO ELEMENTS OF THE JACOBIAN MATRIX.
*  II  M  NUMBER OF NONZERO ELEMENTS OF THE SPARSE HESSIAN MATRIX.
*  II  MB  NUMBER OF NONZERO ELEMENTS OF THE PARTITIONED HESSIAN
*         MATRIX.
*  II  IA(NA)  TYPES OF APPROXIMATED FUNCTIONS.
*  RU  H(M)  POSITIVE DEFINITE APPROXIMATION OF THE SPARSE HESSIAN
*         MATRIX
*  II  IH(NF+1)  POINTERS OF THE DIAGONAL ELEMENTS OF THE SPARSE
*         HESSIAN MATRIX.
*  II  JH(M)  INDICES OF THE NONZERO ELEMENTS OF THE SPARSE HESSIAN
*             MATRIX IN THE PACKED ROW FORM.
*  RU  AH(MB)  POSITIVE DEFINITE APPROXIMATION OF THE PARTITIONED
*         HESSIAN MATRIX.
*  II  IAG(NA+1)  POINTERS OF THE SPARSE JACOBIAN MATRIX.
*  II  JAG(MA)  COLUMN INDICES OF THE SPARSE JACOBIAN MATRIX.
*  RI  F  CURRENT VALUE OF THE OBJECTIVE FUNCTION.
*  RI  FO  PREVIOUS VALUE OF THE OBJECTIVE FUNCTION.
*  RI  ETA5  TOLERANCE FOR A HYBRID METHOD.
*  II  INITH  CHOICE OF THE INITIAL APPROXIMATION OF THE SPARSE
*         HESSIAN MATRIX. INITH=1-UNIT INITIAL APPROXIMATION.
*         INITH=2-RANDOM INITIAL APPROXIMATION.
*
* COMMON DATA :
*  RU  RAN  RANDOM NUMBER.
*  II  IDECF  DECOMPOSITION INDICATOR. IDECF=0-NO DECOMPOSITION.
*  II  IREST  RESTART SPECIFICATION. IF IREST=0 DOES NOT HOLD THEN A
*         RESTART IS PERFORMED.
*  IU  LD  DEGREE OF PREVIOUSLY COMPUTED DERIVATIVES.
*  TO  TUXX   TEXT INFORMATION ON THE RESTART USED.
*
* SUBPROGRAMS USED :
*  S   UXBSMI  GENERATION OF THE PARTITIONED UNIT MATRIX.
*  S   UXBSDO  INITIATION OF A PARTITIONED DIAGONAL MATRIX.
*  S   UXSSMI  GENERATION OF THE SPARSE UNIT MATRIX.
*  S   UXSSDO  INITIATION OF A SPARSE DIAGONAL MATRIX.
*  S   UASSH2  COMPUTATION OF THE SPARSE HESSIAN MATRIX FROM THE
*         PARTITIONED HESSIAN MATRIX.
*  RF  UNRAN1  RANDOM NUMBER GENERATOR.
*  S   UYSET3  DEFINITION OF THE RESTART VARIABLES.
*
      SUBROUTINE UUSSD2(N,NA,M,MA,MB,IA,H,IH,JH,AH,IAG,JAG,F,FO,ETA5,
     & INITH)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER N,NA,M,MA,MB,IA(NA),IH(N+1),JH(M),IAG(NA+1),JAG(MA),INITH
      REAL*8 H(M),AH(MB),F,FO,ETA5
      INTEGER IUDSD,K,L,KA,NB
      REAL*8 POM,UNRAN1
      REAL*8 CON
      PARAMETER (CON=1.0D 1)
      SAVE IUDSD
      IF (IREST.LT.0) THEN
      CALL UXBSMI(MB,NA,AH,IAG)
      TUXX='NEWTON  '
      IUDSD=1
      ELSE IF (IREST.EQ.0) THEN
      IF (IDIR.LE.0) THEN
      IF (FO-F.LE.ETA5*FO) THEN
      NB=0
      DO 2 KA=1,NA
      K=IAG(KA)
      L=IAG(KA+1)-K
      IF (IA(KA).GT.0) THEN
      CALL UASSH2(N,NA,MA,L*(L+1)/2,M,H,IH,JH,AH(NB+1),IAG,JAG,KA, ONE)
      ENDIF
      NB=NB+L*(L+1)/2
    2 CONTINUE
      LD=MIN(LD,1)
      IUDSD=0
      ENDIF
      ENDIF
      ELSEIF(IUDSD.EQ.0) THEN
      NB=0
      DO 3 KA=1,NA
      K=IAG(KA)
      L=IAG(KA+1)-K
      CALL UASSH2(N,NA,MA,L*(L+1)/2,M,H,IH,JH,AH(NB+1),IAG,JAG,KA,-ONE)
      NB=NB+L*(L+1)/2
    3 CONTINUE
      CALL UXBSMI(MB,NA,AH,IAG)
      TUXX='NEWTON  '
      IUDSD=1
      ELSEIF(INITH.EQ.1) THEN
      CALL UXSSMI(N,M,H,IH)
      LD=MIN(LD,1)
      TUXX='UNIT    '
      IUDSD=1
      CALL UYSET3
      ELSEIF(INITH.EQ.2) THEN
      POM=(CON-ONE)/(CON+ONE)
      POM=POM/DBLE(N+1)
      DO 1 K=1,MB
      H(K)=POM*(TWO*UNRAN1(RAN)-ONE)
    1 CONTINUE
      CALL UXSSDO(N,M,ONE,H,IH,JH)
      LD=MIN(LD,1)
      TUXX='RANDOM  '
      IUDSD=1
      CALL UYSET3
      ENDIF
      IDECF=0
      RETURN
      END
