* SUBROUTINE UUSSD6                ALL SYSTEMS                97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* INITIATION OF A DENSE SYMMETRIC POSITIVE DEFINITE MATRIX
*
* PARAMETERS :
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  II  NA  NUMBER OF APPROXIMATED FUNCTIONS.
*  II  MA  NUMBER OF NONZERO ELEMENTS OF THE JACOBIAN MATRIX.
*  II  M  NUMBER OF NONZERO ELEMENTS OF THE SPARSE HESSIAN MATRIX.
*  II  MB  NUMBER OF NONZERO ELEMENTS OF THE PARTITIONED HESSIAN
*         MATRIX.
*  II  IA(NA)  TYPES OF APPROXIMATED FUNCTIONS.
*  RI  AF(NA)  VECTOR CONTAINING VALUES OF THE APPROXIMATED
*         FUNCTIONS.
*  RU  H(M)  POSITIVE DEFINITE APPROXIMATION OF THE SPARSE HESSIAN
*         MATRIX
*  II  IH(NF+1)  POINTERS OF THE DIAGONAL ELEMENTS OF THE SPARSE
*         HESSIAN MATRIX.
*  II  JH(M)  INDICES OF THE NONZERO ELEMENTS OF THE SPARSE HESSIAN
*             MATRIX IN THE PACKED ROW FORM.
*  RU  AH(MB)  POSITIVE DEFINITE APPROXIMATION OF THE PARTITIONED
*         HESSIAN MATRIX.
*  II  IAG(NA+1)  POINTERS OF THE SPARSE JACOBIAN MATRIX.
*  II  JAG(MA)  COLUMN INDICES OF THE SPARSE JACOBIAN MATRIX.
*  RI  R  CURRENT STEPSIZE PARAMETER.
*  RI  F  CURRENT VALUE OF THE OBJECTIVE FUNCTION.
*  RI  FO  PREVIOUS VALUE OF THE OBJECTIVE FUNCTION.
*  RI  PO  PREVIOUS VALUE OF THE DIRECTIONAL DERIVATIVE.
*  RI  ETA5  TOLERANCE FOR A HYBRID METHOD.
*  RU  PPJ  QUADRATIC TERM WITH THE NORMAL EQUATION MATRIX.
*  RU  PPB  QUADRATIC TERM WITH A SECOND ORDER INFORMATION.
*  II  IPOM  INDICATION FOR SECOND ORDER INFORMATION.
*  II  MOT2  SWITCH FOR THE HYBRID METHOD. MOT2=0-THE FLETCHER AND XU
*         SWITCH WITH GENERAL UPDATE. MOT2=1-THE FLETCHER AND XU SWITCH
*         WITH SPECIAL UPDATE. MOT2=2-THE DENNIS AND WELSCH SWITCH.
*         MOT2=3-THE RAMSIN AND WEDIN SWITCH.
*  II  INITH  CHOICE OF THE INITIAL APPROXIMATION OF THE SPARSE
*         HESSIAN MATRIX. INITH=1-UNIT INITIAL APPROXIMATION.
*         INITH=2-RANDOM INITIAL APPROXIMATION.
* COMMON DATA :
*  RU  RAN  RANDOM NUMBER.
*  II  IDECF  DECOMPOSITION INDICATOR. IDECF=0-NO DECOMPOSITION.
*  II  IREST  RESTART SPECIFICATION. IF IREST=0 DOES NOT HOLD THEN A
*         RESTART IS PERFORMED.
*  II  LIT  NUMBER OF THE FIRST ITERATION.
*  IU  NIT  ACTUAL NUMBER OF ITERATIONS.
*  IU  LD  DEGREE OF PREVIOUSLY COMPUTED DERIVATIVES.
*  TO  TUXX   TEXT INFORMATION ON THE RESTART USED.
*
* SUBPROGRAMS USED :
*  S   UXBSMI  GENERATION OF THE UNIT MATRIX.
*  S   UXSSMI  GENERATION OF THE SPARSE UNIT MATRIX.
*  S   UXSSDO  INITIATION OF A SPARSE DIAGONAL MATRIX.
*  S   UASSH2  COMPUTATION OF THE SPARSE HESSIAN MATRIX FROM THE
*         PARTITIONED HESSIAN MATRIX.
*  RF  UNRAN1  RANDOM NUMBER GENERATOR.
*  S   UYSET3  DEFINITION OF THE RESTART VARIABLES.
*
      SUBROUTINE UUSSD6(N,NA,M,MA,MB,IA,AF,H,IH,JH,AH,IAG,JAG,R,F,FO,PO,
     & ETA5,PPJ,PPB,IPOM,MOT2,INITH)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER N,NA,M,MA,MB,IA(NA),IH(N+1),JH(M),IAG(NA+1),JAG(MA),
     & IPOM,MOT2,INITH
      REAL*8 AF(NA),H(M),AH(MB),R,F,FO,PO,ETA5,PPJ,PPB
      LOGICAL L1
      INTEGER K,L,LL,KA,NB
      REAL*8 POM,RHO,UNRAN1
      REAL*8 CON,CON2,CON3,CON4
      PARAMETER(CON=1.0D 1,CON2=0.2D 0,CON3=5.0D-3,CON4=0.5D 0)
      IF (IREST.LT.0) THEN
      CALL UXBSMI(MB,NA,AH,IAG)
      ELSE IF (IREST.EQ.0) THEN
      ELSEIF(INITH.EQ.1) THEN
      CALL UXSSMI(N,M,H,IH)
      TUXX='UNIT    '
      LD=MIN(LD,1)
      CALL UYSET3
      ELSEIF(INITH.EQ.2) THEN
      POM=(CON-ONE)/(CON+ONE)
      POM=POM/DBLE(N+1)
      DO 1 K=1,IH(N+1)-1
      H(K)=POM*(TWO*UNRAN1(RAN)-ONE)
    1 CONTINUE
      CALL UXSSDO(N,M,ONE,H,IH,JH)
      TUXX='RANDOM  '
      LD=MIN(LD,1)
      CALL UYSET3
      ENDIF
      IF (NIT.LE.LIT+1) THEN
      IPOM=0
      PPJ=ZERO
      PPB=ZERO
      RETURN
      ELSE IF (MOT2.EQ.2) THEN
      RHO=R*R
      PPJ=PPJ*RHO
      PPB=PPB*RHO
      ENDIF
      IF (IDIR.EQ.0) THEN
      IPOM=0
      IF (MOT2.EQ.3) THEN
      L1=IPOM.EQ.1.OR.PPJ.LT.CON3*FO.AND.PPJ.LT.CON4*PPB
      ELSE IF (MOT2.EQ.2) THEN
      L1=ABS((F-FO)/(PO+PPB)-ONE).LE.CON2*ABS((F-FO)/(PO+PPJ)-ONE)
      ELSE
      L1=FO-F.LE.ETA5*FO
      ENDIF
      IF (L1) IPOM=1
      IF (IPOM.EQ.1) THEN
      NB=0
      DO 2 KA=1,NA
      K=IAG(KA)
      L=IAG(KA+1)-K
      LL=L*(L+1)/2
      IF (IA(KA).GT.0) THEN
      CALL UASSH2(N,NA,MA,LL,M,H,IH,JH,AH(NB+1),IAG,JAG,KA,AF(KA))
      LD=MIN(LD,1)
      ENDIF
      NB=NB+LL
    2 CONTINUE
      ENDIF
      IDECF=0
      ENDIF
      RETURN
      END
