* SUBROUTINE UXDCQF               ALL SYSTEMS                16/12/01
C PORTABILITY : ALL SYSTEMS
C 16/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* QR DECOMPOSITION OF ROWWISE STORED DENSE RECTANGULAR MATRIX Q USING
* HOUSEHOLDER TRANSFORMATIONS WITHOUT PIVOTING.
*
* PARAMETERS :
*  II  N  NUMBER OF COLUMNS OF THE MATRIX A.
*  II  M  NUMBER OF ROWS OF THE MATRIX A.
*  RU  A(M,N)  UPPER TRIANGULAR MATRIX STORED COLUMNWISE IN THE
*         TWO-DIMENSIONAL ARRAY.
*  RO  Q(M,N)  RECTANGULAR MATRIX STORED COLUMNWISE IN THE
*         TWO-DIMENSIONAL ARRAY (PART OF THE ORTHOGONAL MATRIX).
*  RO  D(N)  VECTOR CONTAINING INFORMATION ABOUT HOUSEHOLDER
*         TRANSFORMATIONS.
*
* SUBPROGRAMS USED :
*  S   UXDRMN  EUCLIDEAN NORM OF A PART OF THE ROWWISE STORED
*         RECTANGULAR MATRIX COLUMN.
*
* METHOD :
* P.A.BUSSINGER, G.H.GOLUB : LINEAR LEAST SQUARES SOLUTION BY
* HOUSEHOLDER TRANSFORMATION. NUMER. MATH. 7 (1965) 269-276.
*
      SUBROUTINE UXDCQF(N,M,A,Q,D)
      INTEGER N,M
      REAL*8 A(N*M),Q(N*M),D(N)
      REAL*8 ALF,POM,UXVDOT,UXVNOR
      INTEGER I,K,IP,KP,NM
      REAL*8 ZERO,ONE
      PARAMETER (ZERO=0.0D 0,ONE=1.0D 0)
      CALL UXVSET(N*M,ZERO,Q)
      NM=MIN(N,M)
C
C     QR DECOMPOSITION
C
      KP=0
      DO 2 K=1,NM
      POM=UXVNOR(M-K+1,A(K+KP))
      IF (A(K+KP).NE.ZERO) POM=SIGN(POM,A(K+KP))
      IF (POM.NE.ZERO) THEN
C
C     HOUSEHOLDER TRANSFORMATION
C
      CALL UXVSCL(M-K+1,ONE/POM,A(K+KP),A(K+KP))
      A(K+KP)=A(K+KP)+ONE
      IP=KP
      DO 1 I=K+1,N
      IP=IP+M
      ALF=UXVDOT(M-K+1,A(K+KP),A(K+IP))/A(K+KP)
      CALL UXVDIR(M-K+1,-ALF,A(K+KP),A(K+IP),A(K+IP))
    1 CONTINUE
      ENDIF
      D(K)=A(K+KP)
      A(K+KP)=-POM
      Q(K+KP)=ONE
      KP=KP+M
    2 CONTINUE
      RETURN
      END
