* SUBROUTINE UXDLGU                ALL SYSTEMS                91/12/01
C PORTABILITY : ALL SYSTEMS
C 91/12/01 MA : ORIGINAL VERSION
*
* PURPOSE :
* CORRECTION OF A DENSE SYMMETRIC POSITIVE DEFINITE MATRIX A+E IN THE
* FACTORED FORM A+E=P*U*D*TRANS(U)*TRANS(P) OBTAINED BY THE SUBROUTINE
* UXDLGF. THE CORRECTION IS DEFINED AS A+E:=A+E+ALF*X*TRANS(X) WHERE
* ALF IS A GIVEN SCALING FACTOR AND X IS A GIVEN VECTOR.
*
* PARAMETERS :
*  II  N ORDER OF THE MATRIX A.
*  RU  A(N*(N+1)/2) FACTORIZATION A+E=P*U*D*TRANS(L)*TRANS(P) OBTAINED
*         BY THE SUBROUTINE UXDLGF.
*  II  IA(N)  A PERMUTATION FIELD WHICH CONTAINS AN INFORMATION ABOUT
*         PERMUTATION MATRIX P.
*  RI  ALF  SCALING FACTOR IN THE CORRECTION TERM.
*  RI  X(N)  VECTOR IN THE CORRECTION TERM.
*  RA  Y(N) AUXILIARY VECTOR.
*
C* METHOD :
C* P.E.GILL, W.MURRAY, M.SAUNDERS: METHODS FOR COMPUTING AND MODIFYING
C* THE LDV FACTORS OF A MATRIX, MATH. OF COMP. 29 (1974) PP. 1051-1077.
*
      SUBROUTINE UXDLGU(N, A, IA, ALF, X, Y)
      INTEGER N, IA(N)
      REAL*8  A(N*(N+1)/2), ALF, X(N), Y(N)
      INTEGER K, KP, J, KM1, KD0, KD1
      REAL*8 SIG, T, B1, D11
C
C     PERMUTATION OF ROWS IN MATRIX A
C
      KD1 = N*(N+1)/2
      DO K = N, 3, -1
          KD1 = KD1 - K
          DO J = K-1, 2, -1
              KP = IA(J)
              IF (KP.NE.J) CALL UNSWAP(A(KD1+KP),A(KD1+J))
          ENDDO
      ENDDO
C
C     PERMUTATION OF THE VECTOR X, THE RESULT IS A VECTOR Y
C
      CALL UXVCOP(N,X,Y)
      DO K = N, 2, -1
          KP = IA(K)
          IF (KP.NE.K) CALL UNSWAP(Y(K),Y(KP))
      ENDDO
C
C     THE PROCESSING BEGINS HERE
C
      SIG = ALF
      K = N             ! INDEX OF THE COLUMN
      KD0 = (N+1)*N/2   ! INDEX OF THE ELEMENT A(K,K)
      KD1 = KD0 - N     ! INDEX OF THE ELEMENT A(K-1,K-1)
   11 CONTINUE
      KM1 = K-1
      T = Y(K)
      B1 = SIG*T
      D11 = A(KD0) + B1*T
      DO J = 1, KM1
          Y(J) = Y(J) - A(KD1+J)*T
      ENDDO
      A(KD0) = D11
      IF (K.EQ.1) GOTO 99   ! THE DECOMPOSITION IS COMPLETE IF K=1
      SIG = SIG - B1*B1/D11
      B1 = B1/D11
C
C     UPDATE THE K-TH COLUMN OF U
C
      DO J = 1, KM1
          A(KD1+J) = A(KD1+J) + B1*Y(J)
      ENDDO
      KD0 = KD1
      K = KM1
      IF (K.GT.1) KD1 = KD0 - K
      GOTO 11
   99 CONTINUE
C
C     PERMUTATION OF ROWS IN UPDATED MATRIX A
C
      KD1 = 0
      DO K = 1, N
          DO J = 2, K-1
              KP = IA(J)
              IF (KP.NE.J) CALL UNSWAP(A(KD1+KP),A(KD1+J))
          ENDDO
          KD1 = KD1+K
      ENDDO
C
      RETURN
      END
