* SUBROUTINE UXDPGF                ALL SYSTEMS                89/12/01
C PORTABILITY : ALL SYSTEMS
C 89/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* FACTORIZATION A+E=L*D*TRANS(L) OF A DENSE SYMMETRIC POSITIVE DEFINITE
* MATRIX A+E WHERE D AND E ARE DIAGONAL POSITIVE DEFINITE MATRICES AND
* L IS A LOWER TRIANGULAR MATRIX. IF A IS SUFFICIENTLY POSITIVE
* DEFINITE THEN E=0.
*
* PARAMETERS :
*  II  N ORDER OF THE MATRIX A.
*  RU  A(N*(N+1)/2)  ON INPUT A GIVEN DENSE SYMMETRIC (USUALLY POSITIVE
*         DEFINITE) MATRIX A STORED IN THE PACKED FORM. ON OUTPUT THE
*         COMPUTED FACTORIZATION A+E=L*D*TRANS(L).
*  IO  INF  AN INFORMATION OBTAINED IN THE FACTORIZATION PROCESS. IF
*         INF=0 THEN A IS SUFFICIENTLY POSITIVE DEFINITE AND E=0. IF
*         INF<0 THEN A IS NOT SUFFICIENTLY POSITIVE DEFINITE AND E>0. IF
*         INF>0 THEN A IS INDEFINITE AND INF IS AN INDEX OF THE
*         MOST NEGATIVE DIAGONAL ELEMENT USED IN THE FACTORIZATION
*         PROCESS.
*  RU  ALF  ON INPUT A DESIRED TOLERANCE FOR POSITIVE DEFINITENESS. ON
*         OUTPUT THE MOST NEGATIVE DIAGONAL ELEMENT USED IN THE
*         FACTORIZATION PROCESS (IF INF>0).
*  RO  TAU  MAXIMUM DIAGONAL ELEMENT OF THE MATRIX E.
*
* METHOD :
* P.E.GILL, W.MURRAY : NEWTON TYPE METHODS FOR UNCONSTRAINED AND
* LINEARLY CONSTRAINED OPTIMIZATION, MATH. PROGRAMMING 28 (1974)
* PP. 311-350.
*
      SUBROUTINE UXDPGF(N,A,INF,ALF,TAU)
      INTEGER N, INF
      REAL*8  A(N*(N+1)/2), ALF, TAU
      REAL*8       BET, GAM, DEL, RHO, SIG, TOL
      INTEGER  I, J, K, L, IJ, KK, KJ, IK
      REAL*8 ZERO,ONE
      PARAMETER (ZERO=0.0D 0,ONE=1.0D 0)
      L = 0
      INF = 0
      TOL = ALF
C
C     ESTIMATION OF THE MATRIX NORM
C
      ALF = ZERO
      BET = ZERO
      GAM = ZERO
      TAU = ZERO
      KK = 0
      DO 2 K=1,N
      KK = KK + K
      BET = MAX( BET, ABS(A(KK)) )
      KJ = KK
      DO 1 J = K+1, N
      KJ = KJ + J - 1
      GAM = MAX( GAM, ABS(A(KJ)) )
    1 CONTINUE
    2 CONTINUE
      BET = MAX(TOL,BET,GAM/N)
C      DEL = TOL*BET
      DEL = TOL*MAX(BET,ONE)
      KK = 0
      DO 6 K=1,N
      KK = KK + K
C
C     DETERMINATION OF A DIAGONAL CORRECTION
C
      SIG = A(KK)
      IF (ALF .GT. SIG) THEN
      ALF = SIG
      L = K
      ENDIF
      GAM = ZERO
      KJ = KK
      DO  3 J=K+1,N
      KJ = KJ + J - 1
      GAM = MAX( GAM, ABS(A(KJ)) )
    3 CONTINUE
      GAM = GAM*GAM
      RHO = MAX( ABS(SIG), GAM/BET, DEL )
      IF (TAU.LT.RHO-SIG) THEN
      TAU=RHO-SIG
      INF=-1
      ENDIF
C
C     GAUSSIAN ELIMINATION
C
      A(KK)=RHO
      KJ = KK
      DO 5 J=K+1, N
      KJ = KJ + J - 1
      GAM = A(KJ)
      A(KJ) = GAM/RHO
      IK = KK
      IJ = KJ
      DO 4 I=K+1, J
      IK = IK + I - 1
      IJ = IJ + 1
      A(IJ) = A(IJ) - A(IK)*GAM
    4 CONTINUE
    5 CONTINUE
    6 CONTINUE
      IF (L.GT.0.AND.ABS(ALF).GT.DEL) INF = L
      RETURN
      END
