* SUBROUTINE UXDSMR               ALL SYSTEMS                02/12/01
C PORTABILITY : ALL SYSTEMS
C 02/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* PLANE ROTATION IS APPLIED TO A DENSE SYMMETRIC MATRIX A. EITHER
* L=K+1 OR K=L+1 IS REQUIRED.
*
* PARAMETERS :
*  II  N  ORDER OF THE MATRIX A.
*  RU  A(N*(N+1)/2) DENSE SYMMETRIC MATRIX STORED IN THE PACKED FORM.
*  II  K  FIRST INDEX OF PLANE ROTATION.
*  II  L  SECOND INDEX OF PLANE ROTATION.
*  RO  CK  DIAGONAL ELEMENT OF THE ELEMENTARY ORTHOGONAL MATRIX.
*  RO  CL  OFF-DIAGONAL ELEMENT OF THE ELEMENTARY ORTHOGONAL MATRIX.
*  IO  IER  INFORMATION ON THE TRANSFORMATION. IER<0-K OR L OUT OF
*         RANGE. IER=0-PLANE ROTATION. IER=1-PERMUTATION.
*         IER=2-TRANSFORMATION SUPPRESSED.
*
* SUBPROGRAMS USED :
*  S   UXVORT  COMPUTATION OF THE PLANE ROTATION MATRIX.
*  S   UXVROT  PLANE ROTATION IS APPLIED TO TWO NUMBERS.
*
      SUBROUTINE UXDSMR(N,A,K,L,CK,CL,IER)
      INTEGER N,K,L,IER
      REAL*8 A(N*(N+1)/2),CK,CL
      REAL*8 AKK,AKL,ALL,CKK,CKL,CLL
      INTEGER J,KJ,LJ,KK,KL,LL
      IF (IER.NE.0.AND.IER.NE.1) RETURN
      IF (L.EQ.K+1) THEN
      KJ=K*(K-1)/2
      DO 1 J=1,N
      IF (J.LE.K) THEN
      KJ=KJ+1
      LJ=KJ+K
      ELSE
      KJ=KJ+J-1
      LJ=KJ+1
      ENDIF
      IF (J.NE.K.AND.J.NE.L) THEN
      CALL UXVROT(A(KJ),A(LJ),CK,CL,IER)
      ENDIF
    1 CONTINUE
      IF (IER.EQ.0) THEN
      CKK=CK**2
      CKL=CK*CL
      CLL=CL**2
      KK=K*(K+1)/2
      KL=KK+K
      LL=KK+L
      AKL=(CKL+CKL)*A(KL)
      AKK=CKK*A(KK)+CLL*A(LL)+AKL
      ALL=CLL*A(KK)+CKK*A(LL)-AKL
      A(KL)=(CLL-CKK)*A(KL)+CKL*(A(KK)-A(LL))
      A(KK)=AKK
      A(LL)=ALL
      ELSE
      KK=K*(K+1)/2
      LL=KK+L
      AKK=A(KK)
      A(KK)=A(LL)
      A(LL)=AKK
      ENDIF
      ELSE IF (K.EQ.L+1) THEN
      LJ=L*(L-1)/2
      DO 2 J=1,N
      IF (J.LE.L) THEN
      LJ=LJ+1
      KJ=LJ+L
      ELSE
      LJ=LJ+J-1
      KJ=LJ+1
      ENDIF
      IF (J.NE.K.AND.J.NE.L) THEN
      CALL UXVROT(A(KJ),A(LJ),CK,CL,IER)
      ENDIF
    2 CONTINUE
      IF (IER.EQ.0) THEN
      CKK=CK**2
      CKL=CK*CL
      CLL=CL**2
      LL=L*(L+1)/2
      KL=LL+L
      KK=LL+K
      AKL=(CKL+CKL)*A(KL)
      AKK=CKK*A(KK)+CLL*A(LL)+AKL
      ALL=CLL*A(KK)+CKK*A(LL)-AKL
      A(KL)=(CLL-CKK)*A(KL)+CKL*(A(KK)-A(LL))
      A(KK)=AKK
      A(LL)=ALL
      ELSE
      LL=L*(L+1)/2
      KK=LL+K
      AKK=A(KK)
      A(KK)=A(LL)
      A(LL)=AKK
      ENDIF
      ELSE
      IER=-1
      RETURN
      ENDIF
      RETURN
      END
