* SUBROUTINE UXDTHC                ALL SYSTEMS                89/12/01
C PORTABILITY : ALL SYSTEMS
C 89/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
*  CORRECTIVE ORTHOGONAL TRANSFORMATION OF A SET OF LINEAR
*  EQUATIONS USING SQUARE ROOT FREE ROTATIONS.
*
* PARAMETERS :
*  II  N  ORDER OF THE MATRIX A.
*  IU  M  ACTUAL NUMBER OF ROWS IN THE MATRIX A.
*  RU  A(N*(N+1)/2) UPPER TRIANGULAR MATRIX OBTAINED USING ORTHOGONAL
*         TRANSFORMATION OF A SET OF LINEAR EQUATIONS.
*  RU  D(N)  DIAGONAL MATRIX OBTAINED USING ORTHOGONAL TRANSFORMATION
*         OF A SET OF LINEAR EQUATIONS.
*  RU  B(N)  RIGHT HAND SIDE VECTOR OBTAINED USING ORTHOGONAL
*         TRANSFORMATION OF A SET OF LINEAR EQUATIONS.
*  RA  X(N)  AUXILIARY VECTOR.
*  RI  Y  CORRECTIVE PARAMETER.
*
* SUBPROGRAMS :
*  S   UXVSET  INITIATION OF A VECTOR.
*  S   UXDTHR  RECURSIVE ORTHOGONAL TRANSFORMATION OF A SET OF
*              LINEAR EQUATIONS.
*
* METHOD :
*  S.HAMMARLING: A NOTE ON MODIFICATIONS TO THE GIVENS PLANE
*  ROTATIONS. J. INST. MATHS. APPLICS. (1984),13,215-218.
*
      SUBROUTINE UXDTHC(N,M,A,D,B,X,Y)
      INTEGER N,M
      REAL*8 A(N*(N+1)/2),D(N),B(N),X(N),Y
      REAL*8 TEMP,TEMP1
      INTEGER I
      REAL*8 ZERO
      PARAMETER(ZERO=0.0D 0)
      IF(Y.LE.ZERO) RETURN
      TEMP1=Y
      TEMP=SQRT(Y)
      DO 1 I=1,N
      CALL UXVSET(N,X,ZERO)
      X(I)=TEMP
      Y=ZERO
      CALL UXDTHR(N,M,A,D,B,X,Y)
    1 CONTINUE
      Y=TEMP1
      RETURN
      END
