* SUBROUTINE UXDTHF                ALL SYSTEMS                89/12/01
C PORTABILITY : ALL SYSTEMS
C 89/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
*  FINAL ORTHOGONAL TRANSFORMATION OF A SET OF LINEAR
*  EQUATIONS USING SQUARE ROOT FREE ROTATIONS.
*
* PARAMETERS :
*  II  N  ORDER OF THE MATRIX A.
*  IU  M  ON INPUT THE ACTUAL NUMBER OF ROWS IN THE MATRIX A.
*         ON OUTPUT AN INFORMATION OBTAINED DURING THE FINAL
*         ORTHOGONAL TRANSFORMATION. IF M=0 OR M=1 THEN A IS NOT
*         OR IS ALMOST SINGULAR.
*  RU  A(N*(N+1)/2) UPPER TRIANGULAR MATRIX OBTAINED USING ORTHOGONAL
*         TRANSFORMATION OF A SET OF LINEAR EQUATIONS.
*  RU  D(N)  DIAGONAL MATRIX OBTAINED USING ORTHOGONAL TRANSFORMATION
*         OF A SET OF LINEAR EQUATIONS.
*  RU  B(N)  RIGHT HAND SIDE VECTOR OBTAINED USING ORTHOGONAL
*         TRANSFORMATION OF A SET OF LINEAR EQUATIONS.
*  II  JOB  OPTION. IF JOB=0 OR JOB=1 THEN THE FINAL UPPER TRIANGULAR
*         MATRIX HAS NOT OR HAS UNIT DIAGONAL ELEMENTS.
*
* METHOD :
*  S.HAMMARLING: A NOTE ON MODIFICATIONS TO THE GIVENS PLANE
*  ROTATIONS. J. INST. MATHS. APPLICS. (1984),13,215-218.
*
      SUBROUTINE UXDTHF(N,M,A,D,B,JOB)
      INTEGER N,M,JOB
      REAL*8 A(N*(N+1)/2),D(N),B(N)
      REAL*8 A1,A2,A3,B1,B2,B3
      INTEGER I,J,K,II,IJ,JJ,IK,JK
      REAL*8 ZERO,ONE
      PARAMETER(ZERO=0.0D 0,ONE=1.0D 0)
      IF(M.LT.N) THEN
      II=M*(M+1)/2
      DO 7 I=M+1,N
      IJ=II+1
      II=II+1
      DO 6 J=I+1,N
      IJ=IJ+J-1
      A(IJ)=ZERO
    6 CONTINUE
    7 CONTINUE
      ENDIF
      M=0
      II=0
      DO 5 I=1,N
      II=II+I
      IF(A(II).EQ.ZERO) THEN
      M=1
      JJ=II
      DO 3 J=I+1,N
      IJ=JJ+I
      JJ=JJ+J
      B1=A(IJ)
      B2=B1*D(I)
      B3=B1*B2
      IF(B3.NE.ZERO) THEN
      A1=A(JJ)
      A2=A1*D(J)
      A3=A1*A2
      IF(B3.LE.A3) THEN
      B1=B1/A1
      B2=B2/A2
      B3=B3/A3
      A3=ONE/(ONE+B3)
      D(J)=A3*D(J)
      D(I)=A3*D(I)
      A(JJ)=A(JJ)+B2*A(IJ)
      A(IJ)=ZERO
      IK=IJ
      JK=JJ
      DO 1 K=J,N-1
      IK=IK+K
      JK=JK+K
      A1=B1*A(JK)
      A(JK)=A(JK)+B2*A(IK)
      A(IK)=A(IK)-A1
    1 CONTINUE
      A1=B1*B(J)
      B(J)=B(J)+B2*B(I)
      B(I)=B(I)-A1
      ELSE
      A1=A1/B1
      A2=A2/B2
      A3=A3/B3
      B3=ONE/(ONE+A3)
      B1=D(J)
      D(J)=B3*D(I)
      D(I)=B3*B1
      A(JJ)=A(IJ)+A2*A(JJ)
      A(IJ)=ZERO
      IK=IJ
      JK=JJ
      DO 2 K=J,N-1
      IK=IK+K
      JK=JK+K
      B1=A(JK)
      A(JK)=A(IK)+A2*A(JK)
      A(IK)=B1-A1*A(IK)
    2 CONTINUE
      B1=B(J)
      B(J)=B(I)+A2*B(J)
      B(I)=B1-A1*B(I)
      ENDIF
      ENDIF
    3 CONTINUE
      ELSEIF(JOB.NE.0) THEN
      A1=ONE/A(II)
      IJ=II+1
      DO 4 J=I+1,N
      IJ=IJ+J-1
      A(IJ)=A(IJ)*A1
    4 CONTINUE
      B(I)=B(I)*A1
      A(II)=D(I)*A(II)**2
      ENDIF
    5 CONTINUE
      RETURN
      END
