* SUBROUTINE UXLCMM                ALL SYSTEMS                95/12/01
C PORTABILITY : ALL SYSTEMS
C 95/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* LIMITED STORAGE QUASI-NEWTON UPDATES.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  MF  NUMBER OF QUASI-NEWTON STEPS.
*  II  NA NUMBER OF COLUMNS OF THE MATRIX A.
*  II  MA  NUMBER OF ELEMENTS IN THE FIELD A.
*  RI  A(MA)  RECTANGULAR MATRIX STORED AS A TWO-DIMENSIONAL ARRAY.
*  II  IA(NA+1)  POSITION OF THE FIRST RORWS ELEMENTS IN THE FIELD A.
*  II  JA(MA) COLUMN INDICES OF ELEMENTS IN THE FIELD A.
*  RU  XM(NF*MF)  SET OF VECTOR OF VARIABLES DIFFERENCES.
*  RU  GM(NF*MF)  SET OF GRADIENT DIFFERENCES.
*  II  IM(MF)  VECTOR OF INDICES.
*  RU  XR(MF*(MF+1)/2)  MATRIX TRANS(XM)*XM STORED IN PACKED FORM.
*  RA  UM(MF)  AUXILIARY VECTOR.
*  RI  U(NA)  INPUT VECTOR.
*  RO  V(NF)  OUTPUT VECTOR.
*  II  JOB  OPTION. JOB=1 - SCHUBERT UPDATE. JOB=2 - COLUMN UPDATE.
*
* SUBPROGRAMS USED :
*  S   UXDPRB  BACK SUBSTITUTION.
*  S   UXDCMM  VECTOR MULTIPLIED BY A COLUMNWISE STORED RECTANGULAR
*         MATRIX.
*  S   UXDCMD  VECTOR MULTIPLIED BY A COLUMNWISE STORED RECTANGULAR
*         MATRIX AND ADDED TO A SCALED VECTOR.
*  S   UXDRMM  VECTOR MULTIPLIED BY A ROWWISE STORED RECTANGULAR
*         MATRIX.
*  S   UXSRMD  VECTOR MULTIPLIED BY A ROWWISE STORED SPARSE RECTANGULAR
*         MATRIX AND ADDED TO A SCALED VECTOR.
*  S   UXVNEG  COPYING OF A VECTOR WITH THE CHANGE OF THE SIGN.
*
* METHOD :
* LIMITED STORAGE QUASI-NEWTON UPDATES.
*
      SUBROUTINE UXLCMM(NF,MF,NA,MA,A,IA,JA,XM,GM,IM,XR,UM,U,V,JOB)
      INTEGER NF,MF,NA,MA,IA(NA+1),JA(MA),IM(MF),JOB
      REAL*8 A(MA),XM(NF*MF),GM(NF*MF),XR(MF*(MF+1)/2),UM(MF),
     & U(NF),V(NA)
      INTEGER I
      REAL*8 ONE
      PARAMETER (ONE=1.0D 0)
      CALL UXSCMM(NF,NA,MA,A,IA,JA,U,V)
      IF (MF.LE.0) THEN
      ELSE
      CALL UXDRMM(NA,MF,GM,U,UM)
      CALL UXDRMD(NF,MF,XM,V,-ONE,UM,UM)
      CALL UXDPRB(MF,XR,UM,1)
      IF (JOB.EQ.1) THEN
      CALL UXDCMD(NF,MF,XM,UM,-ONE,V,V)
      CALL UXVNEG(NA,V,V)
      ELSE
      DO 1 I=1,MF
      V(IM(I))=V(IM(I))-UM(I)
    1 CONTINUE
      ENDIF
      ENDIF
      RETURN
      END
