* SUBROUTINE UXSGFP                 ALL SYSTEMS                90/12/01
C PORTABILITY : ALL SYSTEMS
C 90/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* PIVOT SEARCH
*
* PARAMETERS :
*  II  N  NUMBER OF ROWS OF THE SPARSE MATRIX
*  II  NZ SIZE OF THE INDEX ARRAY CNR
*  II  NAU SIZE OF THE ARRAY THAT STORES THE NUMERICAL ELEMENTS OF
*          THE PROCESSED MATRIX.
*  II  PTC(N) POINTER VECTOR OF THE COLUMNS OF THE SPARSE MATRIX.
*  II  LENC(N) LENGTHS OF THE COLUMNS.
*  II  LENCH(N) LENGTHS OF THE ACTIVE PARTS OF COLUMNS.
*  II  CNR(NZ) CONTAINS THE INDICES OF THE COLUMN AND ROW SCHEME
*              OF THE PROCESSED MATRIX.
*  II  PTR(N) POINTER VECTOR OF THE ROWS OF THE SPARSE MATRIX.
*  II  LENR(N) LENGTHS OF THE ROWS.
*  II  PORDR(N) INDEX VECTOR OF THE ROW ORDERING.
*  II  ORDR(N) ORDER OF THE ROWS WHICH WILL BE FURTHER UPDATED.
*  II  PORDC(N) INDEX VECTOR OF THE COLUMN ORDERING.
*  II  ORDC(N) ORDER OF THE COLUMNS WHICH WILL BE FURTHER UPDATED.
*  RI  AU(NAU) ELEMENTS OF THE PROCESSED SPARSE MATRIX.
*  RI  STFL  STABILITY FLAG.
*  RO  MIBEST THE BEST VALUE OF THE COLUMN MULTIPLIER.
*  IO  IBEST ROW COLUMN OF THE OBTAINED PIVOT.
*  IO  JBEST COLUMN INDEX OF THE OBTAINED PIVOT.
*  RI  TAU  PARAMETER SPECIFYING THE METHOD FOR PIVOT SEARCH.
*  II  JOB1  CONTROLS THE RANGE OF COLUMN SEARCH.
*  II  JOB2  CONTROLS THE RANGE OF ROW SEARCH.
*  IO  IFL1  ERROR FLAG.
*
      SUBROUTINE UXSGFP(N,NZ,NAU,PTC,LENC,LENCH,CNR,PTR,LENR,
     *                PORDR,ORDR,PORDC,ORDC,AU,STFL,MIBEST,IBEST,JBEST,
     *                TAU,JOB1,JOB2,IFL1)
      INTEGER N,NZ,NAU,IFL1
      INTEGER PTC(N),PTR(N),LENC(N),LENR(N),LENCH(N)
      INTEGER PORDR(N),ORDR(N),PORDC(N),ORDC(N)
      INTEGER CNR(NZ)
      REAL*8 MCR
      INTEGER IBEST,JBEST,JOB1,JOB2
      REAL*8 AU(NAU)
      REAL*8 STFL,ABEST,MIBEST,TAU
      REAL*8 TAUMI,TOL,AI,AJ,ACOL,AMCOL,ASTFL
      INTEGER I,J,K,L,K1,K2,K3,M1,L1,L2,IFL,JC,JCJ,JR,JRJ,K4
      INTEGER MBEST,NQ,IMAX,JSTOPC,JSTOPR
C
C     INITIALIZATION
C
      ABEST=0
      MBEST=(N+1)*(N+1)
      MIBEST=STFL
      TAUMI=TAU*STFL
      IFL=0
      IFL1=0
C
C     JOB TYPE
C
      JSTOPC=JOB1
      JSTOPR=JOB2
C
C     JC,JR ARE COUNTS OF THE COLUMNS OR ROWS PROCESSED
C     JCJ,JRJ COUNTS THE NUMBER OF THE COLUMNS OR ROWS PROCESSED
C     ACOL IS THE MAXIMUM ELEMENT IN THE COLUMN
C
      ACOL=0
      JC=1
      JCJ=0
      JR=1
      JRJ=0
C
C     COLUMN SEARCH
C
C        ALL COLUMNS THAT WAS TO PROCESS ARE PROCESSED
C
      IF (JCJ.GE.JSTOPC) GO TO 800
100   CONTINUE
C
C     FIND THE COLUMN
C
      K=PORDC(JC)
C
C     CHANGE 28.7.88 : INSTEAD OF K2=PORDC(JC+1). THE SAME FOR
C     ROWS.
C
200   IF(JC+1.GT.N) THEN
                     K2=N+1
                     ELSE
                     K2=PORDC(JC+1)
      END IF
      K4=K2
      IF(K.EQ.K2) THEN
                  JC=JC+1
                  K=K2
                  GO TO 200
      END IF
      DO 700 K=PORDC(JC),K4-1
      JCJ=JCJ+1
      K2=ORDC(K)
C
C     LENGTH OF THE ACTIVE PART
C
      L1=LENC(K2)-LENCH(K2)-1
C
C     SET ACOL
C
      DO 300 I=PTC(K2)+LENCH(K2),PTC(K2)+LENC(K2)-1
      ACOL=MAX(ACOL,ABS(AU(I)))
300   CONTINUE
C
C     MCR IS MARKOWITZ COUNT REDUCED
C
      IF(L1.EQ.0) THEN
                  MCR=N+1
                  ELSE
                  MCR=MBEST/L1
      END IF
C
C     SET BOUNDS
C
      I=PTC(K2)+LENCH(K2)
      IMAX=I+L1
C
C     TEST WHETHER THE MULTIPLIERS ARE SUFFICIENTLY SMALL
C
      ASTFL=ACOL/STFL
      IF(MIBEST.LT.TAUMI) THEN
                          AMCOL=ACOL/TAUMI
                          GO TO 600
                          ELSE
                          AMCOL=ACOL/MIBEST
      END IF
C
C     TEST WHETHER THE COLUMN IS SEARCHED THROUGH
C
400   IF(I.GT.IMAX) GO TO 500
      AI=ABS(AU(I))
      K1=CNR(I)
      L2=LENR(K1)-1
C
C     MAIN DECISION
C
      IF(L2.LT.MCR) THEN
          IF(ASTFL.GE.AI) THEN
                          I=I+1
                          GO TO 400
                          ELSE
                          MCR=L2
                          AMCOL=AI
                          IBEST=K1
                          IFL=1
                          I=I+1
                          GO TO 400
          END IF
                    ELSE
          IF(L2.EQ.MCR) THEN
            IF(AMCOL.GE.AI) THEN
                            I=I+1
                            GO TO 400
                            ELSE
                            IFL=1
                            AMCOL=AI
                            IBEST=K1
                            I=I+1
                            GO TO 400
            END IF
                        ELSE
                        I=I+1
                        GO TO 400
          END IF
      END IF
C
C     WAS THE CURRENT BEST PIVOT CHANGED
C
500   IF(IFL.EQ.1) THEN
                   IFL=0
                   MIBEST=ACOL/AMCOL
                   JBEST=K2
                   ABEST=AMCOL
                   MBEST=MCR*L1
                   IF(MBEST.EQ.0) GO TO 1500
      END IF
C
C     JUMP OVER THE SECOND BRANCH
C
      GO TO 700
C
C     IF MIBEST IS LESS THEN TAUMI
C
600   CONTINUE
      IF(I.GT.IMAX) GO TO 700
      AI=ABS(AU(I))
      K1=CNR(I)
      L2=LENR(K1)-1
C
C     MAIN DECISION
C
      IF(L2.LT.MCR) THEN
          IF(ASTFL.GE.AI) THEN
                          I=I+1
                          GO TO 400
                          ELSE
                          MCR=L2
                          AMCOL=AI
                          IBEST=K1
                          IFL=1
                          I=I+1
                          GO TO 400
          END IF
                    ELSE
          IF(L2.EQ.MCR) THEN
              IF(AMCOL.GE.AI) THEN
                              I=I+1
                              GO TO 600
                              ELSE
                  IF(AI.GT.ABEST) THEN
                                  ABEST=AI
                                  IBEST=K1
                                  MIBEST=ACOL/AI
                                  JBEST=K2
                                  I=I+1
                                  GO TO 600
                                  ELSE
                                  I=I+1
                                  GO TO 600
                  END IF
              END IF
                        ELSE
                        I=I+1
                        GO TO 600
          END IF
      END IF
700   CONTINUE
      JC=JC+1
C
C     ROW SEARCH
C
800   CONTINUE
C
C     TEST WHETHER ALL NEEDED ROWS SEARCHED THROUGH
C
      IF(JRJ.GE.JSTOPR) GO TO 1400
850   CONTINUE
      K=PORDR(JR)
900   IF(JR+1.GT.N) THEN
                    K2=N+1
                    ELSE
                    K2=PORDR(JR+1)
      END IF
      K4=K2
      IF(K.EQ.K2) THEN
                  JR=JR+1
                  K=K2
                  GO TO 900
      END IF
C
C     FIND THE ROW INDEX
C
C
C     CYCLE OF THE ROWS OF THE SAME COUNT
C
      DO 1300 K=PORDR(JR),K4-1
      JRJ=JRJ+1
      K2=ORDR(K)
      L2=LENR(K2)-1
      DO 1200 I=PTR(K2),PTR(K2)+L2
      ACOL=0
      K3=CNR(I)
      L1=LENC(K3)-LENCH(K3)-1
      M1=L1*L2
C
C     TESTED ELEMENT ON THE CROSSING OF THE ROW K2 AND
C     THE COLUMN K3 IS FOUND
C
      L=PTC(K3)+LENCH(K3)
C
C     PIVOT TEST
C
      IF(M1.GT.MBEST) THEN
                      GO TO 1200
                      ELSE
950   IF(CNR(L).EQ.K2) THEN
                       AI=ABS(AU(L))
                       ELSE
                       L=L+1
                       GO TO 950
      END IF
          IF(M1.LT.MBEST) THEN
                          TOL=AI*STFL
                          DO 1000 J=PTC(K3)+LENCH(K3),PTC(K3)+LENC(K3)-1
                          AJ=ABS(AU(J))
                          ACOL=MAX(ACOL,AJ)
                          IF(ACOL.GE.TOL) GO TO 1200
1000                      CONTINUE
                          JBEST=K3
                          IBEST=K2
                          ABEST=AI
                          MIBEST=ACOL/ABEST
                          MBEST=M1
                          ELSE
              IF(MIBEST.LT.TAUMI) THEN
                                  TOL=AI*TAUMI
                                  DO 1100 J=PTC(K3)+LENCH(K3),PTC(K3)+
     *                                      LENC(K3)-1
                                  AJ=ABS(AU(J))
                                  ACOL=MAX(ACOL,AJ)
                                  IF(ACOL.GE.TOL) GO TO 1200
1100                              CONTINUE
                                  IF(AI.GT.ABEST) THEN
                                                  JBEST=K3
                                                  IBEST=K2
                                                  ABEST=AI
                                                  MIBEST=ACOL/ABEST
                                  END IF
              END IF
          END IF
      END IF
      IF(MBEST.EQ.0) GO TO 1500
1200  CONTINUE
1300  CONTINUE
      IF(MBEST.EQ.0.AND.MBEST.LT.TAUMI) GO TO 1500
      JR=JR+1
1400  CONTINUE
C
C     TEST WHETHER WE SEARCHED ALL
C
      IF(JCJ.LT.JSTOPC) THEN
                        GO TO 100
                        ELSE
          IF(JRJ.LT.JSTOPR) THEN
                            GO TO 850
          END IF
      END IF
      NQ=(N+1)*(N+1)
      IF(MBEST.EQ.NQ) IFL1=1
1500  CONTINUE
      RETURN
      END
