* SUBROUTINE UXSIS2                 ALL SYSTEMS                95/12/01
C PORTABILITY : ALL SYSTEMS
C 95/12/01 DU : ORIGINAL VERSION
*
* PURPOSE :
* AUXILIARY SUBROUTINE FOR UXSIBS.
* ANALYSIS SUBROUTINE
*
      SUBROUTINE UXSIS2(N, IPE, IW, LW, IWFR, NV, NXT, LST, IPD, FLAG)
      INTEGER N, LW, IWFR
      INTEGER IPE(N)
      INTEGER IW(LW),NV(N),NXT(N),LST(N),IPD(N),FLAG(N)
C
C     GIVEN REPRESENTATION OF THE WHOLE MATRIX (EXCLUDING DIAGONAL)
C     PERFORM MINIMUM DEGREE ORDERING, CONSTRUCTING TREE POINTERS.
C     IT WORKS WITH SUPERVARIABLES WHICH ARE COLLECTIONS OF ONE OR MORE
C     VARIABLES, STARTING WITH SUPERVARIABLE I CONTAINING VARIABLE I FOR
C     I = 1,2,...,N. ALL VARIABLES IN A SUPERVARIABLE ARE ELIMINATED
C     TOGETHER. EACH SUPERVARIABLE HAS AS NUMERICAL NAME THAT OF ONE
C     OF ITS VARIABLES (ITS PRINCIPAL VARIABLE).
C
C     N MUST BE SET TO THE MATRIX ORDER. IT IS NOT ALTERED.
C     IPE(I) MUST BE SET TO POINT TO THE POSITION IN IW OF THE
C     START OF ROW I OR HAVE THE VALUE ZERO IF ROW I HAS NO OFF-
C     DIAGONAL NON-ZEROS. DURING EXECUTION IT IS USED AS FOLLOWS. IF
C     SUPERVARIABLE I IS ABSORBED INTO SUPERVARIABLE J THEN IPE(I)=-J.
C     IF SUPERVARIABLE I IS ELIMINATED THEN IPE(I) EITHER POINTS TO THE
C     LIST OF SUPERVARIABLES FOR CREATED ELEMENT I OR IS ZERO IF
C     THE CREATED ELEMENT IS NULL. IF ELEMENT I
C     IS ABSORBED INTO ELEMENT J THEN IPE(I)=-J.
C     IW MUST BE SET ON ENTRY TO HOLD LISTS OF VARIABLES BY
C     ROWS, EACH LIST BEING HEADED BY ITS LENGTH.
C     DURING EXECUTION THESE LISTS ARE REVISED AND HOLD
C     LISTS OF ELEMENTS AND SUPERVARIABLES. THE ELEMENTS
C     ALWAYS HEAD THE LISTS. WHEN A SUPERVARIABLE
C     IS ELIMINATED ITS LIST IS REPLACED BY A LIST OF SUPERVARIABLES
C     IN THE NEW ELEMENT.
C     LW MUST BE SET TO THE LENGTH OF IW. IT IS NOT ALTERED.
C     IWFR MUST BE SET TO THE POSITION IN IW OF THE FIRST FREE VARIABLE.
C     IT IS REVISED DURING EXECUTION AND CONTINUES TO HAVE THIS MEANING.
C     NV(JS) NEED NOT BE SET. DURING EXECUTION IT IS ZERO IF
C     JS IS NOT A PRINCIPAL VARIABLE AND IF IT IS IT HOLDS
C     THE NUMBER OF VARIABLES IN SUPERVARIABLE JS. FOR ELIMINATED
C     VARIABLES IT IS SET TO THE DEGREE AT THE TIME OF ELIMINATION.
C     NXT(JS) NEED NOT BE SET. DURING EXECUTION IT IS THE NEXT
C     SUPERVARIABLE HAVING THE SAME DEGREE AS JS, OR ZERO
C     IF IT IS LAST IN ITS LIST.
C     LST(JS) NEED NOT BE SET. DURING EXECUTION IT IS THE
C     LAST SUPERVARIABLE HAVING THE SAME DEGREE AS JS OR
C     -(ITS DEGREE) IF IT IS FIRST IN ITS LIST.
C     IPD(ID) NEED NOT BE SET. DURING EXECUTION IT
C     IS THE FIRST SUPERVARIABLE WITH DEGREE ID OR ZERO
C     IF THERE ARE NONE.
C     FLAG IS USED AS WORKSPACE FOR ELEMENT AND SUPERVARIABLE FLAGS.
C     WHILE THE CODE IS FINDING THE DEGREE OF SUPERVARIABLE IS
C     FLAG HAS THE FOLLOWING VALUES.
C     A) FOR THE CURRENT PIVOT/NEW ELEMENT ME
C        FLAG(ME)=-1
C     B) FOR VARIABLES JS
C        LAG(JS)=-1 IF JS IS NOT A PRINCIPAL VARIABLE
C        FLAG(JS)=0 IF JS IS A SUPERVARIABLE IN THE NEW ELEMENT
C        FLAG(JS)=NFLG IF JS IS A SUPERVARIABLE NOT IN THE NEW
C          ELEMENT THAT HAS BEEN COUNTED IN THE DEGREE
C          CALCULATION
C        FLAG(JS).GT.NFLG IF JS IS A SUPERVARIABLE NOT IN THE NEW
C          ELEMENT THAT HAS NOT BEEN COUNTED IN THE DEGREE
C          CALCULATION
C     C) FOR ELEMENTS IE
C        FLAG(IE)=-1 IF ELEMENT IE HAS BEEN MERGED INTO ANOTHER
C        FLAG(IE)=-NFLG IF ELEMENT IE HAS BEEN USED IN THE DEGREE
C          CALCULATION FOR IS.
C        FLAG(IE).LT.-NFLG IF ELEMENT IE HAS NOT BEEN USED IN THE
C          DEGREE CALCULATION FOR IS
C
      INTEGER I,ID,IDL,IDN, IE, IERROR,  IFRLVL,  IOVFLO,
     &        IP, IS,
     &        JP, JP1, JP2, JS, K, K1,K2,KE, KP, KP0, KP1, KP2, KS,
     &        L, LEN, LN, LS, LWFR,
     &        MD, ME, ML, MS,
     &        NCMPA, NCMPBI, NCMPBR, NEIG, NEL, NEMIN, NFLG,
     &        NIRADU, NIRBDU, NIRNEC, NIRTOT,  NP, NP0,
     &        NRLADU, NRLBDU,
     &        NRLNEC, NRLTOT, NS, NTWO, NVPIV
      DOUBLE PRECISION OPS
      INTRINSIC MIN0, IABS
C     SEE BLOCK DATA FOR COMMENTS ON VARIABLES IN COMMON
      COMMON /UMMAT2/ OPS, IERROR, NRLTOT, NIRTOT, NRLNEC, NIRNEC,
     & NRLADU, NIRADU, NRLBDU, NIRBDU, NCMPA, NCMPBR, NCMPBI, NTWO,NEIG
      COMMON /UMMAT3/ IOVFLO, NEMIN, IFRLVL(20)
C
C     IS,JS,KS,LS,MS,NS ARE USED TO REFER TO SUPERVARIABLES.
C     IE,JE,KE ARE USED TO REFER TO ELEMENTS.
C     IP,JP,KP,K,NP ARE USED TO POINT TO LISTS OF ELEMENTS.
C     OR SUPERVARIABLES.
C     ID IS USED FOR THE DEGREE OF A SUPERVARIABLE.
C     MD IS USED FOR THE CURRENT MINIMUM DEGREE.
C     IDN IS USED FOR THE NO. OF VARIABLES IN A NEWLY CREATED ELEMENT
C     NEL IS USED TO HOLD THE NO. OF VARIABLES THAT HAVE BEEN
C     ELIMINATED.
C     ME=MS IS THE NAME OF THE SUPERVARIABLE ELIMINATED AND
C     OF THE ELEMENT CREATED IN THE MAIN LOOP.
C     NFLG IS USED FOR THE CURRENT FLAG VALUE IN ARRAY FLAG. IT STARTS
C     WITH THE VALUE IOVFLO AND IS REDUCED BY 1 EACH TIME IT IS USED
C     UNTIL IT HAS THE VALUE 2 WHEN IT IS RESET TO THE VALUE IOVFLO.
C
C     INITIALIZATIONS
C
      DO 10 I=1,N
        IPD(I) = 0
        NV(I) = 1
        FLAG(I) = IOVFLO
   10 CONTINUE
      MD = 1
      NCMPA = 0
      NFLG = IOVFLO
      NEL = 0
C
C     LINK TOGETHER VARIABLES HAVING SAME DEGREE
C
      DO 30 IS=1,N
        K = IPE(IS)
        IF (K.LE.0) GO TO 20
        ID = IW(K) + 1
        NS = IPD(ID)
        IF (NS.GT.0) LST(NS) = IS
        NXT(IS) = NS
        IPD(ID) = IS
        LST(IS) = -ID
        GO TO 30
C     WE HAVE A VARIABLE THAT CAN BE ELIMINATED AT ONCE BECAUSE THERE
C     IS NO OFF-DIAGONAL NON-ZERO IN ITS ROW.
C
   20   NEL = NEL + 1
        FLAG(IS) = -1
        NXT(IS) = 0
        LST(IS) = 0
   30 CONTINUE
C
C     START OF MAIN LOOP
C
      DO 340 ML=1,N
C
C     LEAVE LOOP IF ALL VARIABLES HAVE BEEN ELIMINATED.
C
        IF (NEL.GE.N) GO TO 350
C
C     FIND NEXT SUPERVARIABLE FOR ELIMINATION.
C
        DO 40 ID=MD,N
          MS = IPD(ID)
          IF (MS.GT.0) GO TO 50
   40   CONTINUE
   50   MD = ID
C
C     NVPIV HOLDS THE NUMBER OF VARIABLES IN THE PIVOT.
C
        NVPIV = NV(MS)
C
C     STORE THE DEGREE OF THE PIVOT.
C
        NV(MS) = ID
C
C     REMOVE CHOSEN VARIABLE FROM LINKED LIST
C
        NS = NXT(MS)
        NXT(MS) = 0
        LST(MS) = 0
        IF (NS.GT.0) LST(NS) = -ID
        IPD(ID) = NS
        ME = MS
        NEL = NEL + NVPIV
C
C     IDN HOLDS THE DEGREE OF THE NEW ELEMENT.
C
        IDN = ID - NVPIV
C
C     RUN THROUGH THE LIST OF THE PIVOTAL SUPERVARIABLE, SETTING TREE
C     POINTERS AND CONSTRUCTING NEW LIST OF SUPERVARIABLES.
C     KP IS A POINTER TO THE CURRENT POSITION IN THE OLD LIST.
C
        KP = IPE(ME)
        FLAG(MS) = -1
C
C     IP POINTS TO THE START OF THE NEW LIST.
C
        IP = IWFR
C
C     LEN HOLDS THE LENGTH OF THE LIST ASSOCIATED WITH THE PIVOT.
C
        LEN = IW(KP)
        DO 140 KP1=1,LEN
          KP = KP + 1
          KE = IW(KP)
C
C     JUMP IF KE IS AN ELEMENT THAT HAS NOT BEEN MERGED INTO ANOTHER.
C
          IF (FLAG(KE).LE.-2) GO TO 60
C
C     JUMP IF KE IS AN ELEMENT THAT HAS BEEN MERGED INTO ANOTHER OR IS
C     A SUPERVARIABLE THAT HAS BEEN ELIMINATED.
C
          IF (FLAG(KE).LE.0) GO TO 140
C
C     WE HAVE A SUPERVARIABLE. PREPARE TO SEARCH REST OF LIST.
C
          JP = KP - 1
          LN = LEN - KP1 + 1
          IE = MS
          GO TO 70
C
C     SEARCH VARIABLE LIST OF ELEMENT KE, USING JP AS A POINTER TO IT.
C
   60     IE = KE
          JP = IPE(IE)
          LN = IW(JP)
C
C     SEARCH FOR DIFFERENT SUPERVARIABLES AND ADD THEM TO THE NEW LIST,
C     COMPRESSING WHEN NECESSARY. THIS LOOP IS EXECUTED ONCE FOR
C     EACH ELEMENT IN THE LIST AND ONCE FOR ALL THE SUPERVARIABLES
C     IN THE LIST.
C
   70     DO 130 JP1=1,LN
            JP = JP + 1
            IS = IW(JP)
C
C     JUMP IF IS IS NOT A PRINCIPAL VARIABLE OR HAS ALREADY BEEN
C     COUNTED.
C
            IF (FLAG(IS).LE.0) GO TO 130
            FLAG(IS) = 0
            IF (IWFR.LT.LW) GO TO 100
C
C     PREPARE FOR COMPRESSING IW BY ADJUSTING POINTERS AND
C     LENGTHS SO THAT THE LISTS BEING SEARCHED IN THE INNER AND OUTER
C     LOOPS CONTAIN ONLY THE REMAINING ENTRIES.
C
            IPE(MS) = KP
            IW(KP) = LEN - KP1
            IPE(IE) = JP
            IW(JP) = LN - JP1
C
C     COMPRESS IW
C
            CALL UXSIS3(N, IPE, IW, IP-1, LWFR)
C
C     COPY NEW LIST FORWARD
C
            JP2 = IWFR - 1
            IWFR = LWFR
            IF (IP.GT.JP2) GO TO 90
            DO 80 JP=IP,JP2
              IW(IWFR) = IW(JP)
              IWFR = IWFR + 1
   80       CONTINUE
C
C     ADJUST POINTERS FOR THE NEW LIST AND THE LISTS BEING SEARCHED.
C
   90       IP = LWFR
            JP = IPE(IE)
            KP = IPE(ME)
C
C     STORE IS IN NEW LIST.
C
  100       IW(IWFR) = IS
            IWFR = IWFR + 1
C
C     REMOVE IS FROM DEGREE LINKED LIST
C
            LS = LST(IS)
            LST(IS) = 0
            NS = NXT(IS)
            NXT(IS) = 0
            IF (NS.GT.0) LST(NS) = LS
            IF (LS.LT.0) THEN
            LS = -LS
            IPD(LS) = NS
            ELSE IF (LS.GT.0) THEN
            NXT(LS) = NS
            ENDIF
  130     CONTINUE
C
C     JUMP IF WE HAVE JUST BEEN SEARCHING THE VARIABLES AT THE END OF
C     THE LIST OF THE PIVOT.
C
          IF (IE.EQ.MS) GO TO 150
C
C     SET TREE POINTER AND FLAG TO INDICATE ELEMENT IE IS ABSORBED INTO
C     NEW ELEMENT ME.
C
          IPE(IE) = -ME
          FLAG(IE) = -1
  140   CONTINUE
C
C     JUMP IF NEW ELEMENT IS NULL.
C
  150   IF (IWFR.EQ.IP) GO TO 330
        K1 = IP
        K2 = IWFR - 1
C
C     RUN THROUGH NEW LIST OF SUPERVARIABLES REVISING EACH ASSOCIATED
C     LIST, RECALCULATING DEGREES AND REMOVING DUPLICATES.
C
        DO 310 K=K1,K2
          IS = IW(K)
          IF (NFLG.GT.2) GO TO 170
C
C     RESET FLAG VALUES TO +/-IOVFLO.
C
          DO 160 I=1,N
            IF (FLAG(I).GT.0) FLAG(I) = IOVFLO
            IF (FLAG(I).LE.-2) FLAG(I) = -IOVFLO
  160     CONTINUE
          NFLG = IOVFLO
C
C     REDUCE NFLG BY ONE TO CATER FOR THIS SUPERVARIABLE.
C
  170     NFLG = NFLG - 1
C
C     BEGIN WITH THE DEGREE OF THE NEW ELEMENT. ITS VARIABLES MUST
C     ALWAYS BE COUNTED DURING THE DEGREE CALCULATION AND THEY ARE
C     ALREADY FLAGGED WITH THE VALUE 0.
C
          ID = IDN
C
C     RUN THROUGH THE LIST ASSOCIATED WITH SUPERVARIABLE IS
C
          KP1 = IPE(IS) + 1
C
C     NP POINTS TO THE NEXT ENTRY IN THE REVISED LIST.
C
          NP = KP1
          KP2 = IW(KP1-1) + KP1 - 1
          DO 220 KP=KP1,KP2
            KE = IW(KP)
C
C     TEST WHETHER KE IS AN ELEMENT, A REDUNDANT ENTRY OR A
C     SUPERVARIABLE.
C
            IF (FLAG(KE)+1.GT.0) GO TO 230
            IF (FLAG(KE)+1.EQ.0) GO TO 220
C
C     SEARCH LIST OF ELEMENT KE, REVISING THE DEGREE WHEN NEW VARIABLES
C     FOUND.
C
            JP1 = IPE(KE) + 1
            JP2 = IW(JP1-1) + JP1 - 1
            IDL = ID
            DO 190 JP=JP1,JP2
              JS = IW(JP)
C
C     JUMP IF JS HAS ALREADY BEEN COUNTED.
C
              IF (FLAG(JS).LE.NFLG) GO TO 190
              ID = ID + NV(JS)
              FLAG(JS) = NFLG
  190       CONTINUE
C
C     JUMP IF ONE OR MORE NEW SUPERVARIABLES WERE FOUND.
C
            IF (ID.GT.IDL) GO TO 210
C
C     CHECK WHETHER EVERY VARIABLE OF ELEMENT KE IS IN NEW ELEMENT ME.
C
            DO 200 JP=JP1,JP2
              JS = IW(JP)
              IF (FLAG(JS).NE.0) GO TO 210
  200       CONTINUE
C
C     SET TREE POINTER AND FLAG TO INDICATE THAT ELEMENT KE IS ABSORBED
C     INTO NEW ELEMENT ME.
C
            IPE(KE) = -ME
            FLAG(KE) = -1
            GO TO 220
C
C     STORE ELEMENT KE IN THE REVISED LIST FOR SUPERVARIABLE IS AND FLAG
C     IT.
C
  210       IW(NP) = KE
            FLAG(KE) = -NFLG
            NP = NP + 1
  220     CONTINUE
          NP0 = NP
          GO TO 250
C
C     TREAT THE REST OF THE LIST ASSOCIATED WITH SUPERVARIABLE IS. IT
C     CONSISTS ENTIRELY OF SUPERVARIABLES.
C
  230     KP0 = KP
          NP0 = NP
          DO 240 KP=KP0,KP2
            KS = IW(KP)
            IF (FLAG(KS).LE.NFLG) GO TO 240
C
C     ADD TO DEGREE, FLAG SUPERVARIABLE KS AND ADD IT TO NEW LIST.
C
            ID = ID + NV(KS)
            FLAG(KS) = NFLG
            IW(NP) = KS
            NP = NP + 1
  240     CONTINUE
C
C     MOVE FIRST SUPERVARIABLE TO END OF LIST, MOVE FIRST ELEMENT TO
C     END OF ELEMENT PART OF LIST AND ADD NEW ELEMENT TO FRONT OF LIST.
C
  250     IW(NP) = IW(NP0)
          IW(NP0) = IW(KP1)
          IW(KP1) = ME
C
C STORE THE NEW LENGTH OF THE LIST.
C
          IW(KP1-1) = NP - KP1 + 1
C
C     CHECK WHETHER ROW IS IS IDENTICAL TO ANOTHER BY LOOKING IN LINKED
C     LIST OF SUPERVARIABLES WITH DEGREE ID AT THOSE WHOSE LISTS HAVE
C     FIRST ENTRY ME. NOTE THAT THOSE CONTAINING ME COME FIRST SO THE
C     SEARCH CAN BE TERMINATED WHEN A LIST NOT STARTING WITH ME IS
C     FOUND.
C
          JS = IPD(ID)
          DO 280 L=1,N
            IF (JS.LE.0) GO TO 300
            KP1 = IPE(JS) + 1
            IF (IW(KP1).NE.ME) GO TO 300
C
C     JS HAS SAME DEGREE AND IS ACTIVE. CHECK IF IDENTICAL TO IS.
C
            KP2 = KP1 - 1 + IW(KP1-1)
            DO 260 KP=KP1,KP2
              IE = IW(KP)
C
C     JUMP IF IE IS A SUPERVARIABLE OR AN ELEMENT NOT IN THE LIST OF IS.
C
              IF (IABS(FLAG(IE)+0).GT.NFLG) GO TO 270
  260       CONTINUE
            GO TO 290
  270       JS = NXT(JS)
  280     CONTINUE
C
C     SUPERVARIABLE AMALGAMATION. ROW IS IS IDENTICAL TO ROW JS.
C     REGARD ALL VARIABLES IN THE TWO SUPERVARIABLES AS BEING IN IS. SET
C     TREE POINTER, FLAG AND NV ENTRIES.
C
  290     IPE(JS) = -IS
          NV(IS) = NV(IS) + NV(JS)
          NV(JS) = 0
          FLAG(JS) = -1
C
C     REPLACE JS BY IS IN LINKED LIST.
C
          NS = NXT(JS)
          LS = LST(JS)
          IF (NS.GT.0) LST(NS) = IS
          IF (LS.GT.0) NXT(LS) = IS
          LST(IS) = LS
          NXT(IS) = NS
          LST(JS) = 0
          NXT(JS) = 0
          IF (IPD(ID).EQ.JS) IPD(ID) = IS
          GO TO 310
C
C     INSERT IS INTO LINKED LIST OF SUPERVARIABLES OF SAME DEGREE.
C
  300     NS = IPD(ID)
          IF (NS.GT.0) LST(NS) = IS
          NXT(IS) = NS
          IPD(ID) = IS
          LST(IS) = -ID
          MD = MIN0(MD,ID)
  310   CONTINUE
C
C     RESET FLAGS FOR SUPERVARIABLES IN NEWLY CREATED ELEMENT AND
C     REMOVE THOSE ABSORBED INTO OTHERS.
C
        DO 320 K=K1,K2
          IS = IW(K)
          IF (NV(IS).EQ.0) GO TO 320
          FLAG(IS) = NFLG
          IW(IP) = IS
          IP = IP + 1
  320   CONTINUE
        IWFR = K1
        FLAG(ME) = -NFLG
C
C     MOVE FIRST ENTRY TO END TO MAKE ROOM FOR LENGTH.
C
        IW(IP) = IW(K1)
        IW(K1) = IP - K1
C
C     SET POINTER FOR NEW ELEMENT AND RESET IWFR.
C
        IPE(ME) = K1
        IWFR = IP + 1
        GO TO 340
  330   IPE(ME) = 0
  340 CONTINUE
  350 RETURN
      END
