* SUBROUTINE UXSIS5                 ALL SYSTEMS                95/12/01
C PORTABILITY : ALL SYSTEMS
C 95/12/01 DU : ORIGINAL VERSION
*
* PURPOSE :
* AUXILIARY SUBROUTINE FOR UXSIBS.
* USING A GIVEN PIVOTAL SEQUENCE AND A REPRESENTATION OF THE MATRIX
* THAT INCLUDES ONLY NON-ZEROS OF THE STRICTLY UPPER-TRIANGULAR PART
* OF THE PERMUTED MATRIX, CONSTRUCT TREE POINTERS.
*
      SUBROUTINE UXSIS5(N, IPE, IW, LW, IWFR, IPS, IPV, NV, FLAG)
      INTEGER N, LW, IWFR
      INTEGER IPE(N)
      INTEGER IW(LW),IPS(N),IPV(N),NV(N),FLAG(N)
C
C     N MUST BE SET TO THE MATRIX ORDER. IT IS NOT ALTERED.
C     IPE(I) MUST BE SET TO POINT TO THE POSITION IN IW OF THE
C     START OF ROW I OR HAVE THE VALUE ZERO IF ROW I HAS NO OFF-
C     DIAGONAL NON-ZEROS. DURING EXECUTION IT IS USED AS FOLLOWS.
C     IF VARIABLE I IS ELIMINATED THEN IPE(I) POINTS TO THE LIST
C     OF VARIABLES FOR CREATED ELEMENT I. IF ELEMENT I IS
C     ABSORBED INTO NEWLY CREATED ELEMENT J THEN IPE(I)=-J.
C     IW MUST BE SET ON ENTRY TO HOLD LISTS OF VARIABLES BY
C     ROWS, EACH LIST BEING HEADED BY ITS LENGTH. WHEN A VARIABLE
C     IS ELIMINATED ITS LIST IS REPLACED BY A LIST OF VARIABLES
C     IN THE NEW ELEMENT.
C     LW MUST BE SET TO THE LENGTH OF IW. IT IS NOT ALTERED.
C     IWFR MUST BE SET TO THE POSITION IN IW OF THE FIRST FREE VARIABLE.
C     IT IS REVISED DURING EXECUTION, CONTINUING TO HAVE THIS MEANING.
C     IPS(I) MUST BE SET TO THE POSITION OF VARIABLE I IN THE REQUIRED
C     ORDERING. IT IS NOT ALTERED.
C     IPV NEED NOT BE SET. IPV(K) IS SET TO HOLD THE K TH VARIABLE
C     IN PIVOT ORDER.
C     NV NEED NOT BE SET. IF VARIABLE J HAS NOT BEEN ELIMINATED THEN
C     THE LAST ELEMENT WHOSE LEADING VARIABLE (VARIABLE EARLIEST
C     IN THE PIVOT SEQUENCE) IS J IS ELEMENT NV(J). IF ELEMENT J
C     EXISTS THEN THE LAST ELEMENT HAVING THE SAME LEADING
C     VARIABLE IS NV(J). IN BOTH CASES NV(J)=0 IF THERE IS NO SUCH
C     ELEMENT. IF ELEMENT J HAS BEEN MERGED INTO A LATER ELEMENT
C     THEN NV(J) IS THE DEGREE AT THE TIME OF ELIMINATION.
C     FLAG IS USED AS WORKSPACE FOR VARIABLE FLAGS.
C     FLAG(JS)=ME IF JS HAS BEEN INCLUDED IN THE LIST FOR ME.
C
      INTEGER I,IE,IERROR,  IP,
     &        J, JE, JP, JP1, JP2, JS, KDUMMY,
     &        LN,  LWFR,
     &        ME, MINJS, ML, MS,
     &        NCMPA, NCMPBI, NCMPBR, NEIG,
     &        NIRADU, NIRBDU, NIRNEC, NIRTOT,
     &        NRLADU, NRLBDU,
     &        NRLNEC, NRLTOT, NTWO
      DOUBLE PRECISION OPS
      INTRINSIC MIN0
      COMMON /UMMAT2/ OPS, IERROR, NRLTOT, NIRTOT, NRLNEC, NIRNEC,
     & NRLADU, NIRADU, NRLBDU, NIRBDU, NCMPA, NCMPBR, NCMPBI, NTWO,NEIG
C
C     INITIALIZATIONS
C
      DO 10 I=1,N
        FLAG(I) = 0
        NV(I) = 0
        J = IPS(I)
        IPV(J) = I
   10 CONTINUE
      NCMPA = 0
C
C     START OF MAIN LOOP
C
      DO 100 ML=1,N
C
C     ME=MS IS THE NAME OF THE VARIABLE ELIMINATED AND
C     OF THE ELEMENT CREATED IN THE MAIN LOOP.
C
        MS = IPV(ML)
        ME = MS
        FLAG(MS) = ME
C
C     MERGE ROW MS WITH ALL THE ELEMENTS HAVING MS AS LEADING VARIABLE.
C     IP POINTS TO THE START OF THE NEW LIST.
C
        IP = IWFR
C
C     MINJS IS SET TO THE POSITION IN THE ORDER OF THE LEADING VARIABLE
C     IN THE NEW LIST.
C
        MINJS = N
        IE = ME
        DO 70 KDUMMY=1,N
C
C     SEARCH VARIABLE LIST OF ELEMENT IE.
C     JP POINTS TO THE CURRENT POSITION IN THE LIST BEING SEARCHED.
C
          JP = IPE(IE)
C
C     LN IS THE LENGTH OF THE LIST BEING SEARCHED.
C
          LN = 0
          IF (JP.LE.0) GO TO 60
          LN = IW(JP)
C
C     SEARCH FOR DIFFERENT VARIABLES AND ADD THEM TO LIST,
C     COMPRESSING WHEN NECESSARY
C
          DO 50 JP1=1,LN
            JP = JP + 1
C
C     PLACE NEXT VARIABLE IN JS.
C
            JS = IW(JP)
C
C     JUMP IF VARIABLE HAS ALREADY BEEN INCLUDED.
C
            IF (FLAG(JS).EQ.ME) GO TO 50
            FLAG(JS) = ME
            IF (IWFR.LT.LW) GO TO 40
C
C     PREPARE FOR COMPRESSING IW BY ADJUSTING POINTER TO AND LENGTH OF
C     THE LIST FOR IE TO REFER TO THE REMAINING ENTRIES.
C
            IPE(IE) = JP
            IW(JP) = LN - JP1
C
C     COMPRESS IW.
C
            CALL UXSIS3(N, IPE, IW, IP-1, LWFR)
C
C     COPY NEW LIST FORWARD
C
            JP2 = IWFR - 1
            IWFR = LWFR
            IF (IP.GT.JP2) GO TO 30
            DO 20 JP=IP,JP2
              IW(IWFR) = IW(JP)
              IWFR = IWFR + 1
   20       CONTINUE
   30       IP = LWFR
            JP = IPE(IE)
C
C     ADD VARIABLE JS TO NEW LIST.
C
   40       IW(IWFR) = JS
            MINJS = MIN0(MINJS,IPS(JS)+0)
            IWFR = IWFR + 1
   50     CONTINUE
C
C     RECORD ABSORPTION OF ELEMENT IE INTO NEW ELEMENT.
C
   60     IPE(IE) = -ME
C
C     PICK UP NEXT ELEMENT WITH LEADING VARIABLE MS.
C
          JE = NV(IE)
C
C     STORE DEGREE OF IE.
C
          NV(IE) = LN + 1
          IE = JE
C
C     LEAVE LOOP IF THERE ARE NO MORE ELEMENTS.
C
          IF (IE.EQ.0) GO TO 80
   70   CONTINUE
   80   IF (IWFR.GT.IP) GO TO 90
C
C     DEAL WITH NULL NEW ELEMENT.
C
        IPE(ME) = 0
        NV(ME) = 1
        GO TO 100
C
C     LINK NEW ELEMENT WITH OTHERS HAVING SAME LEADING VARIABLE.
C
   90   MINJS = IPV(MINJS)
        NV(ME) = NV(MINJS)
        NV(MINJS) = ME
C
C     MOVE FIRST ENTRY IN NEW LIST TO END TO ALLOW ROOM FOR LENGTH AT
C     FRONT. SET POINTER TO FRONT.
C
        IW(IWFR) = IW(IP)
        IW(IP) = IWFR - IP
        IPE(ME) = IP
        IWFR = IWFR + 1
  100 CONTINUE
      RETURN
      END
