* SUBROUTINE UXSPCJ                ALL SYSTEMS               92/12/01
C PORTABILITY : ALL SYSTEMS
C 92/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* SPARSE MATRIX REORDER, SYMBOLIC FACTORIZATION, DATA STRUCTURES
* TRANSFORMATION - INITIATION OF THE DIRECT SPARSE SOLVER.
* MODIFIED VERSION WITH CHANGED DATA STRUCTURES.
*
*  PARAMETERS :
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  II  NJA  NUMBER OF NONZERO ELEMENTS OF THE MATRIX.
*  IO  ML  SIZE OF THE COMPACT STRUCTURE OF THE TRIANGULAR FACTOR
*         OF THE HESSIAN APPROXIMATION.
*  II  MMAX  SIZE OF THE ARRAYS JA,A.
*  RO  A(MMAX)   NUMERICAL VALUES OF THE SPARSE HESSIAN APPROXIMATION
*         STORED AT THE POSITIONS 1, ...,NJA. LOWER TRIANGULAR
*         FACTOR OF THE HESSIAN APPROXIMATION STORED AT THE
*         POSITIONS NJA+1, ..., NJA+MB.
*  II  IA(N) POINTERS OF THE DIAGONAL ELEMENTS OF THE HESSIAN MATRIX.
*  II  JA(MMAX)  INDICES OF THE NONZERO ELEMENTS OF THE HESSIAN MATRIX IN
*         THE PACKED ROW FORM AT THE FIRST NJA POSITIONS. COMPACT
*         STRUCTURE OF INDICES OF ITS TRIANGULAR FACTOR IS ROWWISE
*         STORED.
*  II  PSL(N+1)  POINTER VECTOR OF THE COMPACT FORM OF THE TRIANGULAR
*         FACTOR OF THE HESSIAN APPROXIMATION.
*  IO  PERM(N) PERMUTATION VECTOR.
*  IO  INVP(N) INVERSE PERMUTATION VECTOR.
*  IA  WN11(N) AUXILIARY VECTOR.
*  IA  WN12(N) AUXILIARY VECTOR.
*  IA  WN13(N) AUXILIARY VECTOR.
*  IO  ITERM  TERMINATION INDICATOR. TERMINATION IF ITERM .NE. 0.
*
* COMMON DATA :
*
* SUBPROGRAMS USED :
*  S   UXSTG1  WIDTHENING OF THE PACKED FORM OF THE SPARSE MATRIX.
*  S   UXSSMN  SPARSE MATRIX REORDERING.
*  S   UXVSIP  INVERSE PERMUTATION COMPUTING.
*  S   UXSPCI  SYMBOLIC FACTORIZATION.
*  S   UXSTL1  PACKING OF THE WIDTHENED FORM OF THE SPARSE MATRIX.
*
      SUBROUTINE UXSPCJ(N,NJA,ML,MMAX,A,IA,JA,PSL,PERM,INVP,WN11,WN12,
     *                  WN13,WN14,ITERM)
      INTEGER N,NJA,MMAX,ML,ITERM
      INTEGER PERM(N),INVP(N),WN11(N+1),WN12(N+1),WN13(N+1),WN14(N+1)
      INTEGER PSL(N+1),IA(N+1),JA(MMAX)
      INTEGER JSTRT,JSTOP,I,J,K,L,NJASAVE
      INTEGER LL,LL1,NJABIG,KSTRT
      REAL*8 A(MMAX)
      IF (ML.GT.0) RETURN
      IF(2*NJA.GE.MMAX) THEN
        ITERM=-41
        CALL UOERR1('UXSPCJ',41)
        GO TO 1000
      END IF
C
C WIDTHENING OF THE PACKED FORM
C
      NJASAVE=NJA
      CALL UXSTG1(N,NJA,MMAX,IA,JA,WN12,WN11)
      NJABIG=NJA
C
C REORDERING OF THE SPARSE MATRIX
C
      CALL UXSSMN( N,MMAX,IA,JA,PERM,WN11,WN12,WN13)
C
C FIND THE INVERSE PERMUTATION VECTOR INVP
C
      CALL UXVSIP(N,PERM,INVP)
C
C SHRINK THE STRUCTURE
C
      CALL UXSTL1(N,NJA,MMAX,IA,JA,WN11)
C
      DO 40 I=1,N
        WN11(I)=0
        WN12(I)=0
 40   CONTINUE
C
C WN11 CONTAINS BEGINNINGS OF THE FACTOR ROWS
C
      DO 50 I=1,N
        K=PERM(I)
        JSTRT=IA(K)
        JSTOP=IA(K+1)-1
        DO 60 J=JSTRT,JSTOP
          L=JA(J)
          L=INVP(L)
          IF(L.GE.I) THEN
            WN12(I)=WN12(I)+1
          ELSE
            WN12(L)=WN12(L)+1
          END IF
 60     CONTINUE
 50   CONTINUE
      WN11(1)=1
      DO 69 I=1,N-1
       WN11(I+1)=WN11(I)+WN12(I)
 69   CONTINUE
C
C CREATE UPPER TRIANGULAR STRUCTURE NECESSARY FOR THE TRANSFER
C
      DO 300 I=1,N
        K=PERM(I)
        JSTRT=IA(K)
        JSTOP=IA(K+1)-1
        DO 200 J=JSTRT,JSTOP
          L=JA(J)
          L=INVP(L)
          IF(L.GE.I) THEN
            LL1=WN11(I)
            WN11(I)=LL1+1
            JA(NJABIG+LL1)=L
            A(J)=LL1
            A(NJA+LL1)=J
          ELSE
            LL1=WN11(L)
            WN11(L)=LL1+1
            JA(NJABIG+LL1)=I
            A(J)=LL1
            A(NJA+LL1)=J
          END IF
 200    CONTINUE
 300  CONTINUE
C
C SORT INDICES IN THE PERMUTED UPPER TRIANGLE
C
      DO 599 I=1,N
        WN11(I)=0
 599  CONTINUE
      WN11(1)=1
      WN14(1)=1
      DO 67 I=2,N+1
        WN11(I)=WN11(I-1)+WN12(I-1)
        WN14(I)=WN11(I)
   67 CONTINUE
      DO 602 I=1,N
        WN12(I)=0
 602  CONTINUE
       JSTOP=WN11(N+1)
       DO 499 I=N,1,-1
         JSTRT=WN11(I)
         CALL UXVSR5(JSTOP-JSTRT,JSTRT-1,JA(NJABIG+JSTRT),
     *               A,A(NJASAVE+JSTRT))
         JSTOP=JSTRT
 499   CONTINUE
C
C WIDTHENING OF THE PERMUTED PACKED FORM.
C
      NJASAVE=NJA
C
      CALL UXSTG1(N,NJA,MMAX,IA,JA,WN12,WN11)
C
      NJABIG=NJA
C
C
C SYMBOLIC FACTORIZATION.
C
      CALL UXSPCI(N,ML,MMAX-NJA-1,IA,JA,PSL,A(2*NJASAVE+1),PERM,INVP,
     *            WN11,WN12,WN13,ITERM)
      IF(ITERM.NE.0) THEN
        ITERM=-42
        CALL UOERR1('UXSPCJ',42)
        GO TO 1000
      END IF
C
C
C RETRIEVE PARAMETERS
C
      CALL UXSTL1(N,NJA,MMAX,IA,JA,WN11)
C
C SHIFT PERMUTED UPPER TRIANGLE.
C
      DO 73 I=1,NJASAVE
        JA(NJA+I)=JA(NJABIG+I)
  73  CONTINUE
C
C SHIFT STRUCTURE SL.
C
      DO 70 I=1,ML
        JA(2*NJASAVE+I)=A(2*NJASAVE+I)
  70  CONTINUE
C
C SET POINTERS
C
      DO 600 I=1,N
        WN12(I)=0
 600  CONTINUE
       LL1=PSL(N)+N-1
       JSTOP=WN14(N+1)
       DO 500 I=N,1,-1
         JSTRT=WN14(I)
         DO 700 J=JSTRT,JSTOP-1
           K=JA(NJASAVE+J)
           WN12(K)=J
           LL=A(NJASAVE+J)
           WN13(K)=LL
 700     CONTINUE
         JSTOP=JSTRT
         KSTRT=JA(2*NJASAVE+I)+N+1+2*NJASAVE
         DO 800 J=KSTRT+PSL(I+1)-PSL(I)-1,KSTRT,-1
           L=JA(J)
           IF(WN12(L).NE.0) THEN
             LL=WN13(L)
             A(LL)=LL1
             WN12(L)=0
           END IF
           LL1=LL1-1
 800     CONTINUE
         K=WN12(I)
         WN12(I)=0
         LL=WN13(I)
         A(LL)=LL1
         LL1=LL1-1
 500   CONTINUE
C
      DO 76 I=1,ML
        JA(NJASAVE+I)=JA(2*NJASAVE+I)
  76  CONTINUE
C
      DO 72 I=1,NJASAVE
        JA(ML+NJASAVE+I)=A(I)
  72  CONTINUE
C
 1000 CONTINUE
      RETURN
      END
