* SUBROUTINE UXSRB7                ALL SYSTEMS               91/12/01
C PORTABILITY : ALL SYSTEMS
C 91/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
*   FIND A PSEUDO-PERIPHERAL NODE USING MODIFIED SCHEME OF
*     GIBBS-POOLE-STOCKMEYER. BASED ON SPARSPAK-A.
*
*  PARAMETERS :
*  IU  ROOT ON INPUT IT DEFINES A COMPONENT FOR WHICH THE NODE
*       IS TO BE FOUND. ON OUTPUT THE PSEUDO-PERIPHERAL NODE.
*  II  XADJ(N+1) POINTER VECTOR FOR THE ADJACENCY STRUCTURE.
*  II  ADJNCY(*) ADJACENCY STRUCTURE OF THE INPUT GRAPH.
*  II  MASK(N)  USED TO SPECIFY A SECTION SUBGRAPH.
*        NODES WITH MASK(I)=0 ARE IGNORED IN UXSRB1.
*  IO  NLVL(N)  NUMBER OF LEVELS IN THE LEVEL STRUCTURE.
*  IO  XLS(N+1) POINTER VECTOR FOR THE ROOTED LEVEL STRUCTURE.
*  IO  LS(N) INDICES OF THE ROOTED LEVEL STRUCTURE.
*
* SEE GEORGE-LIU: SPARSPAK
*
      SUBROUTINE  UXSRB7 ( ROOT, XADJ, ADJNCY, MASK, NLVL,
     1                     XLS, LS )
C
         INTEGER    ADJNCY(*), LS(*), MASK(*), XLS(*), XADJ(*)
         INTEGER    CCSIZE, J     , JSTRT , K     , KSTOP ,
     *              KSTRT , MINDEG, NABOR , NDEG  , NLVL  ,
     *              NODE  , NUNLVL, ROOT
C
C        DETERMINE THE LEVEL STRUCTURE ROOTED AT ROOT.
C
         CALL  UXSRB1 ( ROOT, XADJ, ADJNCY, MASK, NLVL, XLS, LS )
         CCSIZE = XLS(NLVL+1) - 1
         IF  ( NLVL .EQ. 1  .OR.  NLVL .EQ. CCSIZE )  RETURN
C
C PICK A NODE WITH MINIMUM DEGREE FROM THE LAST LEVEL.
C
  100    CONTINUE
             JSTRT = XLS(NLVL)
             MINDEG = CCSIZE
             ROOT = LS(JSTRT)
             IF  ( CCSIZE .EQ. JSTRT )  GO TO 400
                 DO  300  J = JSTRT, CCSIZE
                     NODE = LS(J)
                     NDEG = 0
                     KSTRT = XADJ(NODE)
                     KSTOP = XADJ(NODE+1) - 1
                     DO  200  K = KSTRT, KSTOP
                         NABOR = ADJNCY(K)
                         IF  ( MASK(NABOR) .GT. 0 )  NDEG = NDEG + 1
  200                CONTINUE
                     IF  ( NDEG .GE. MINDEG )  GO TO 300
                         ROOT = NODE
                         MINDEG = NDEG
  300            CONTINUE
  400        CONTINUE
C
C GENERATE ROOTED LEVEL STRUCTURE.
C
             CALL  UXSRB1 ( ROOT, XADJ, ADJNCY, MASK,
     1                      NUNLVL, XLS, LS )
             IF  ( NUNLVL .LE. NLVL )  RETURN
             NLVL = NUNLVL
             IF  ( NLVL .LT. CCSIZE )  GO TO 100
                 RETURN
C
      END
