* SUBROUTINE UYAGB2                ALL SYSTEMS                99/12/01
C PORTABILITY : ALL SYSTEMS
C 99/12/01 VL : ORIGINAL VERSION
*
* PURPOSE :
*  SIMPLIFIED AGGREGATION FOR NONSMOOTH VARIABLE METRIC METHOD.
*
* PARAMETERS :
*  II  NF  DECLARED NUMBER OF VARIABLES.
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  RI  H(M)  POSITIVE DEFINITE APPROXIMATION OF THE INVERSE HESSIAN
*         MATRIX.
*  RI  G(N)  SUBGRADIENT OF THE OBJECTIVE FUNCTION.
*  RU  GV(N)  AGGREGATED SUBGRADIENT OF THE OBJECTIVE FUNCTION.
*  RU  GN(N)  REDUCED AGGREGATED SUBGRADIENT.
*  RI  SN(N)  REDUCED DIRECTION VECTOR.
*  RI  CZ(NF*NF)  MATRIX WHOSE COLUMNS ARE BASIC VECTORS FROM THE
*         CURRENT REDUCED SUBSPACE.
*  RI  S(N)  REDUCED SUBGRADIENT.
*  RO  ALF  LINEARIZATION TERM.
*  RU  ALFV  AGGREGATED LINEARIZATION TERM.
*
      SUBROUTINE UYAGB2(NF,N,H,G,GV,GN,SN,CZ,S,ALF,ALFV)
      INTEGER NF,N
      REAL*8 H(N*(N+1)/2),G(NF),GV(NF),GN(N),SN(N),CZ(NF*NF),S(N),
     & ALF,ALFV
      REAL*8 LAM,P,Q,W
      INTEGER I,J,L
      REAL*8 ZERO,ONE
      PARAMETER (ZERO=0.0D 0,ONE=1.0D 0)
      CALL UXDRMM(NF,N,CZ,G,S)
      P=ALFV-ALF
      DO 1 I=1,N
      GN(I)=S(I)-GN(I)
    1 P=P+SN(I)*GN(I)
      Q=ZERO
      DO 4 I=1,N
      L=I*(I-1)/2+1
      W=ZERO
      DO 2 J=1,I-1
      W=W+H(L)*GN(J)
    2 L=L+1
      DO 3 J=I,N
      W=W+H(L)*GN(J)
    3 L=L+J
    4 Q=Q+W*GN(I)
      LAM=0.5D 0+SIGN(0.5D 0,P)
      IF(Q.GT.ZERO)LAM=MIN(ONE,MAX(ZERO,P/Q))
      P=ONE-LAM
      DO 5 I=1,N
    5 GN(I)=S(I)-P*GN(I)
      DO 6 I=1,NF
    6 GV(I)=LAM*G(I)+P*GV(I)
      ALFV=LAM*ALF+P*ALFV
      RETURN
      END
