* SUBROUTINE UYFPP1             ALL SYSTEMS                   93/12/01
C PORTABILITY : ALL SYSTEMS
C 93/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* DETERMINATION OF THE INITIAL FEASIBLE POINT USING BOTH THE SIMPLE
* BOUNDS FOR VARIABLES AND THE LINEAR CONSTRAINTS.
*
* PARAMETERS :
*  II  NF  DECLARED NUMBER OF VARIABLES.
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  II  NC  NUMBER OF CONSTRAINTS.
*  II  MC  NUMBER OF NONZERO ELEMENTS IN THE FIELD CG.
*  II  MMAX  MAXIMUM NUMBER OF NONZERO ELEMENTS IN THE FIELD CR.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XN(NF)  VECTOR OF SCALING FACTORS.
*  RI  XL(NF)  VECTOR CONTAINING LOWER BOUNDS FOR VARIABLES.
*  RI  XU(NF)  VECTOR CONTAINING UPPER BOUNDS FOR VARIABLES.
*  RI  CF(NF)  VECTOR CONTAINING VALUES OF THE CONSTRAINT FUNCYIONS.
*  RO  CFD(NF)  VECTOR CONTAINING INCREMENTS OF THE CONSTRAINT
*         FUNCTIONS.
*  II  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  II  ICA(NF)  VECTOR CONTAINING INDICES OF ACTIVE CONSTRAINTS.
*  RI  CL(NC)  VECTOR CONTAINING LOWER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CU(NC)  VECTOR CONTAINING UPPER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CG(MC)  SPARSE CONSTRAINT JACOBIAN MATRIX.
*  II  ICG(MCL+1)  POSITION OF THE FIRST RORWS ELEMENTS IN THE FIELD CG.
*  II  JCG(MC) COLUMN INDICES OF ELEMENTS IN THE FIELD CG.
*  RI  CR(MMAX)  TRIANGULAR DECOMPOSITION OF KERNEL OF THE
*         ORTHOGONAL PROJECTION.
*  II  ICR(NF+1)  POSITION OF THE FIRST RORWS ELEMENTS IN THE FIELD CR.
*  II  JCR(MMAX) COLUMN INDICES OF ELEMENTS IN THE FIELD CR.
*  RO  CZ(NF)  VECTOR OF LAGRANGE MULTIPLIERS.
*  RI  G(NF)  GRADIENT OF THE OBJECTIVE FUNCTION.
*  RO  GO(NF)  SAVED GRADIENT OF THE OBJECTIVE FUNCTION.
*  RI  S(NF)  DIRECTION VECTOR.
*  RI  ETA9  MAXIMUM FOR REAL NUMBERS.
*  RI  EPS7  TOLERANCE FOR LINEAR INDEPENDENCE OF CONSTRAINTS.
*  RI  EPS9  TOLERANCE FOR ACTIVITY OF CONSTRAINTS.
*  RU  XDEL  TRUST REGION BOUND.
*  RO  UMAX  MAXIMUM ABSOLUTE VALUE OF A NEGATIVE LAGRANGE MULTIPLIER.
*  RO  GMAX  MAXIMUM ABSOLUTE VALUE OF A PARTIAL DERIVATIVE.
*  II  MFP  TYPE OF FEASIBLE POINT. MFP=1-ARBITRARY FEASIBLE POINT.
*         MFP=2-OPTIMUM FEASIBLE POINT.
*
* COMMON DATA :
*  II  LCS  VARIABLE WHICH SPECIFIES TRUST REGION BOUNDS.
*  II  LIT  NUMBER OF THE INITIAL ITERATION.
*  II  NIT  NUMBER OF THE CURRENT ITERATION.
*  II  JOUT  SYSTEM VARIABLE WHICH SPECIFIES PRINT OF INFORMATION
*         DURING THE OPTIMIZATION PROCESS. JOUT=0-NO PRINT.
*         JOUT=1-PRINT IS REQUIRED.
*  II  KOUT  LEVEL OF PRINT. KOUT=0-NO PRINT. KOUT=1-PRINT OF INITIAL
*         VALUES AND FINAL RESULTS TOGETHER WITH PRINT OF ITERATIONS.
*         KOUT=2-ADDITIONAL PRINT OF STEPSIZE SELECTION.
*         KOUT=3-ADDITIONAL PRINT OF DIRECTION DETERMINATION.
*         KOUT=4-ADDITIONAL PRINT OF MATRIX CORRECTION.
*         KOUT=5-ADDITIONAL PRINT OF NUMERICAL DIFFERENTIATION.
*         KOUT=6-ADDITIONAL PRINT OF DEBUGGING.
*  II  KOUT1  LOWER ITERATION IN WHICH INFORMATION IS PRINTED.
*  II  KOUT2  UPPER ITERATION IN WHICH INFORMATION IS PRINTED.
*  II  KOUT3  NUMBER OF ITERATIONS AFTER THAT INFORMATION IS PRINTED.
*
* SUBPROGRAMS USED :
*  S   UMLPB2  LINEAR PROGRAMMING SUBROUTINE.
*
      SUBROUTINE UYFPP1(NF,N,NC,MC,MMAX,X,IX,XN,XL,XU,CF,CFD,IC,ICA,CL,
     & CU,CG,ICG,JCG,CR,ICR,JCR,CZ,G,GO,S,ETA9,EPS7,EPS9,XDEL,UMAX,GMAX,
     & MFP)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER NF,N,NC,MC,MMAX,IX(NF),IC(NC),ICA(NF),ICG(NC+1),JCG(MC),
     & ICR(NF+1),JCR(MMAX),MFP
      REAL*8 X(NF),XN(NF),XL(NF),XU(NF),CF(NC),CFD(NC),CL(NC),CU(NC),
     &  CG(MC),CR(MMAX),CZ(NF),G(NF),GO(NF),S(NF),ETA9,EPS7,EPS9,XDEL,
     & UMAX,GMAX
      JOUT=0
      IF(KOUT.EQ.0.OR.KOUT3.LE.0) THEN
      ELSE IF(KOUT1.LE.KOUT2) THEN
        IF(NIT-LIT.GE.KOUT1.AND.NIT-LIT.LE.KOUT2) JOUT=1
      ELSE
        IF(NIT-LIT.GE.KOUT1.OR. NIT-LIT.LE.KOUT2) JOUT=1
      ENDIF
      LCS=0
      CALL UMLPB2(NF,N,NC,MC,MMAX,X,IX,IX,XN,S,XL,XU,CF,CFD,IC,ICA,CL,
     & CU,CG,ICG,JCG,CR,ICR,JCR,CZ,GO,GO,S,ETA9,EPS7,EPS9,XDEL,UMAX,
     & GMAX,MFP,0)
      IF (ITERL.LT.0) ITERM=ITERL-10
      RETURN
      END
