* SUBROUTINE UYSORC               ALL SYSTEMS                08/10/01
C PORTABILITY : ALL SYSTEMS
C 08/10/01 MA : ORIGINAL VERSION
*
* PURPOSE :
* LSTRS METHOD: CHECK CONDITIONS FOR CONVERGENCE.
*
* PARAMETERS :
*  II  N  NUMBER OF VARIABLES.
*  II  M  NUMBER OF NONZERO ELEMENTS IN HESSIAN MATRIX.
*  RI  H(M)  ELEMENTS OF THE SPARSE SYMMETRIC MATRIX STORED IN THE
*      PACKED FORM.
*  II  IH(N+1) POINTERS OF THE HESSIAN MATRIX STORED IN A COMPACT FORM.
*  II  JH(M) COLUMN INDICES OF NONZEROES IN THE MATRIX H.
*  RI  G(N)  GRADIENT.
*  RI  GNORM  THE NORM OF A GRADIENT.
*  RI  XDEL  TRUST-REGION RADIUS.
*  RO  S(N)  DIRECTION VECTOR.
*  RO  SNORM  THE NORM OF A DIRECTION VECTOR S.
*  RI  ALPHA  SCALAR PARAMETER IN LSTRS METHOD.
*  RI  ALPHAL  LOWER BOUND FOR ALPHA.
*  RI  ALPHAU  UPPER BOUND FOR ALPHA.
*  IU  NMV  NUMBER OF MATRIX-VECTOR MULTIPLICATIONS.
*  RI  DELTAS  VALUE 1+XDEL**2
*  II  ITER  WHICH EIGENPAIR IS CONSIDERED (1-ST, 2-ND, NONE).
*  II  NCONV  NUMBER OF CONVERGED EIGENVALUES OF THE BORDERED MATRIX.
*  LI  ISZ  IF WE HAVE A CORRECTION TERM.
*  RI  Z(N)  CORRECTION TERM.
*  II  OPTS(4)  ONLY OPTS(3) AND OPTS(4) ARE CONSIDERED:
*        OPTS(3) = INTERIOR SOLUTION (1=CGM, 2=DON'T COMPUTE)
*        OPTS(4) = IN THE HARD CASE ADD CORRECTION TERM (1=YES, 2=NO)
*  RI  LAMBDA1  THE 1-ST SMALLEST EIGENVALUE OF THE BORDERED MATRIX.
*  RI  LAMBDA2  THE 2-ND SMALLEST EIGENVALUE OF THE BORDERED MATRIX.
*  RI  NU1  THE FIRST COMPONENT OF THE 1-ST EIGENVECTOR.
*  RI  NU2  THE FIRST COMPONENT OF THE 2-ND EIGENVECTOR.
*  RI  U1(N)  COMPONENTS (2,...,N+1) OF THE 1-ST EIGENVECTOR.
*  RI  U2(N)  COMPONENTS (2,...,N+1) OF THE 2-ND EIGENVECTOR.
*  RI  U1NORM  THE NORM OF THE VECTOR U1.
*  RI  U2NORM  THE NORM OF THE VECTOR U2.
*  RI  DEL1  TOLERANCE FOR A BOUNDARY SOLUTION.
*  RI  EPS6  TOLERANCE FOR A CGM METHOD IN CASE OF AN INTERIOR SOLUTION.
*  RI  ETA5  TOLERANCE DETERMINING SMALL INTERVAL FOR ALPHA.
*  RI  ETA7  TOLERANCE DETERMINING A QUASI-OPTIMAL SOLUTION
*      IN A POTENTIAL HARD CASE.
*  RI  ETA8  TOLERANCE DETERMINING AN INTERIOR SOLUTION.
*  II  NITER  NUMBER OF LSTRS ITERATIONS.
*  II  MITER  MAXIMAL NUMBER OF LSTRS ITERATIONS.
*  RA  XTILDE(N)  AUXILIARY ARRAY.
*  IO  STCOND  STOPPING CONDITION REACHED (NONE, BOUNDARY, INTERIOR,
*      QUASI-OPTIMAL SOLUTION, SMALL INT. FOR ALPHA, MAX. NUMBER OF IT.)
*
* SUBPROGRAMS USED :
*  S   UXVSCL  SCALING OF A VECTOR.
*  S   UXVSET  A SCALAR IS SET TO ALL THE ELEMENTS OF A VECTOR.
*  S   UXVNEG  CHANGE THE SIGNS OF VECTOR ELEMENTS.
*  S   UXSSMM  MULTIPLICATION OF A SPARSE SYMMETRIC MATRIX BY A VECTOR.
*  S   UXVDIR  VECTOR AUGMENTED BY THE SCALED VECTOR.
*  S   UXVCOP  COPYING OF A VECTOR.
*  RF  UXVDOT  DOT PRODUCT OF TWO VECTORS.
*  RF  UXVNOR  EUCLIDEAN NORM OF A VECTOR.
*  RF  UXSSMQ  VALUE OF A QUADRATIC FORM WITH A SPARSE SYMMETRIC MATRIX.
*
      SUBROUTINE UYSORC(N,M,H,IH,JH,G,GNORM,XDEL,S,SNORM,ALPHA,ALPHAL,
     & ALPHAU,NMV,DELTAS,ITER,NCONV,ISZ,Z,OPTS,LAMBDA1,LAMBDA2,NU1,NU2,
     & U1,U2,U1NORM,U2NORM,DEL1,EPS6,ETA5,ETA7,ETA8,NITER,MITER,XTILDE,
     & STCOND)
      INTEGER N,M,IH(N+1),JH(M),NMV,ITER,NCONV,OPTS(4),NITER,MITER,
     & STCOND
      REAL*8 H(M),G(N),GNORM,XDEL,S(N),SNORM,ALPHA,ALPHAL,ALPHAU,
     & DELTAS,Z(N),LAMBDA1,LAMBDA2,NU1,NU2,U1(N),U2(N),U1NORM,U2NORM,
     & DEL1,EPS6,ETA5,ETA7,ETA8,XTILDE(N)
      LOGICAL ISZ
      REAL*8 TEMPR,LHSEXP,RHSEXP,ETA,NORMFSQ,NORMF,PROD,SQRPROD,DEN,
     & TAU1,TAU2,QUADS,UXSSMQ,UXVDOT,UXVNOR
      REAL*8 ZERO,ONE,TWO
      PARAMETER (ZERO=0.0D0,ONE=1.0D0,TWO=2.0D0)
*
*     BOUNDARY SOLUTION
*
      STCOND = 0
      IF (ITER.EQ.0) GOTO 10
      IF (ITER.EQ.1) THEN
      SNORM = U1NORM/ABS(NU1)
      ELSE
      SNORM = U2NORM/ABS(NU2)
      ENDIF
      IF (ABS(SNORM-XDEL).LE.DEL1*XDEL.AND.LAMBDA1.LE.ZERO) THEN
      IF (ITER.EQ.1) THEN
      CALL UXVSCL(N,ONE/NU1,U1,S)
      STCOND = 11
      ELSE
      CALL UXVSCL(N,ONE/NU2,U2,S)
      STCOND = 12
      ENDIF
      ENDIF
      IF (STCOND.NE.0) GOTO 90
*
*     INTERIOR SOLUTION
*
   10 IF (NCONV.GT.0) THEN
      IF (U1NORM.LT.XDEL*ABS(NU1).AND.LAMBDA1.GT.-ETA8) THEN
      IF (OPTS(3).EQ.1) THEN
      CALL UXVSET(N,ZERO,S)
      CALL UXVNEG(N,G,Z)
      CALL UXVNEG(N,G,U1)
      TAU1 = GNORM**2
      DO WHILE (SQRT(TAU1).GT.EPS6)
      CALL UXSSMM(N,M,H,IH,JH,U1,U2)
      NMV = NMV + 1
      ALPHA = TAU1/UXVDOT(N,U1,U2)
      CALL UXVDIR(N,ALPHA,U1,S,S)
      CALL UXVDIR(N,-ALPHA,U2,Z,Z)
      TAU2 = TAU1
      TAU1 = UXVDOT(N,Z,Z)
      TEMPR = TAU1/TAU2
      CALL UXVDIR(N,TEMPR,U1,Z,U1)
      ENDDO
      SNORM = UXVNOR(N,S)
      STCOND = 20
      ELSE
      IF (ITER.EQ.1) THEN
      CALL UXVSCL(N,ONE/NU1,U1,S)
      STCOND = 21
      ELSE
      CALL UXVSCL(N,ONE/NU2,U2,S)
      STCOND = 22
      ENDIF
      ENDIF
      ENDIF
      ENDIF
      IF (STCOND.NE.0) GOTO 90
*
*     QUASIOPTIMAL SOLUTION
*
      ETA = ETA7/(ONE-ETA7)
      IF (NCONV.GE.2) THEN
      NORMFSQ = NU1**2 + NU2**2
      NORMF = SQRT(NORMFSQ)
      PROD = NORMFSQ*DELTAS
      IF (PROD.GE.ONE) THEN
      SQRPROD = SQRT(PROD-ONE)
      DEN = NORMFSQ*SQRT(DELTAS)
      IF (PROD.GT.ONE) THEN
      TAU1 = (NU1-NU2*SQRPROD)/DEN
      TAU2 = (NU2+NU1*SQRPROD)/DEN
      ELSE
      TAU1 = NU1/NORMF
      TAU2 = NU2/NORMF
      ENDIF
      TEMPR = TAU1*NU1 + TAU2*NU2
      CALL UXVSCL(N,TAU1/TEMPR,U1,XTILDE)
      CALL UXVDIR(N,TAU2/TEMPR,U2,XTILDE,XTILDE)
      QUADS = UXSSMQ(N,M,H,IH,JH,XTILDE,XTILDE)/TWO
      NMV = NMV + 1
      QUADS = QUADS + UXVDOT(N,G,XTILDE)
      LHSEXP = (LAMBDA2-LAMBDA1)*TAU2**2*DELTAS
      RHSEXP = -TWO*ETA*QUADS
      IF (LHSEXP.LE.RHSEXP) THEN
      CALL UXVCOP(N,XTILDE,S)
      SNORM = UXVNOR(N,S)
      STCOND = 31
      ELSE IF (PROD.GT.ONE) THEN
      TAU1 = (NU1+NU2*SQRPROD)/DEN
      TAU2 = (NU2-NU1*SQRPROD)/DEN
      TEMPR = TAU1*NU1 + TAU2*NU2
      CALL UXVSCL(N,TAU1/TEMPR,U1,XTILDE)
      CALL UXVDIR(N,TAU2/TEMPR,U2,XTILDE,XTILDE)
      QUADS = UXSSMQ(N,M,H,IH,JH,XTILDE,XTILDE)/TWO
      NMV = NMV + 1
      QUADS = QUADS + UXVDOT(N,G,XTILDE)
      LHSEXP = (LAMBDA2-LAMBDA1)*TAU2**2*DELTAS
      RHSEXP = -TWO*ETA*QUADS
      IF (LHSEXP.LE.RHSEXP) THEN
      CALL UXVCOP(N,XTILDE,S)
      SNORM = UXVNOR(N,S)
      STCOND = 32
      ENDIF
      ENDIF
      ENDIF
      ENDIF
      IF (STCOND.NE.0) GOTO 90
*
*     SMALL INTERVAL FOR ALPHA
*
      IF (ITER.EQ.0) THEN
      CALL UXVSET(N,ZERO,S)
      SNORM = ZERO
      STCOND = 40
      ELSE IF (ABS(ALPHAU-ALPHAL).LE.
     & ETA5*MAX(ABS(ALPHAL),ABS(ALPHAU))) THEN
      IF (ITER.EQ.1) CALL UXVSCL(N,ONE/NU1,U1,S)
      IF (ITER.EQ.2) CALL UXVSCL(N,ONE/NU2,U2,S)
      IF (SNORM.LT.XDEL) THEN
      IF (ISZ.AND.OPTS(4).EQ.1) THEN
      TEMPR = UXVNOR(N,Z)
      CALL UXVSCL(N,ONE/TEMPR,Z,Z)
      TAU1 = -UXVDOT(N,Z,S)
      TAU2 = (XDEL-SNORM)*(XDEL+SNORM)
      TAU1 = -TAU2/(TAU1+DSIGN(TAU1,ONE)*SQRT(TAU1**2+TAU2))
      CALL UXVDIR(N,TAU1,Z,S,S)
      SNORM = UXVNOR(N,S)
      IF (ITER.EQ.1) STCOND = 411
      IF (ITER.EQ.2) STCOND = 421
      ELSE
      IF (ITER.EQ.1) STCOND = 412
      IF (ITER.EQ.2) STCOND = 422
      ENDIF
      ELSE
      CALL UXVSCL(N,ONE/XDEL,S,S)
      SNORM = XDEL
      IF (ITER.EQ.1) STCOND = 413
      IF (ITER.EQ.2) STCOND = 423
      ENDIF
      ENDIF
      IF (STCOND.NE.0) GOTO 90
*
*     MAXIMAL NUMBER OF ITERATINS
*
      IF (NITER.GT.MITER) THEN
      IF (ITER.EQ.0) THEN
      CALL UXVSET(N,ZERO,S)
      SNORM = ZERO
      STCOND = 50
      ELSE IF (ITER.EQ.1) THEN
      CALL UXVSCL(N,ONE/NU1,U1,S)
      STCOND = 51
      ELSE
      CALL UXVSCL(N,ONE/NU2,U2,S)
      STCOND = 52
      ENDIF
      ENDIF
   90 CONTINUE
      RETURN
      END
