* SUBROUTINE UYSORI               ALL SYSTEMS                08/10/01
C PORTABILITY : ALL SYSTEMS
C 08/10/01 MA : ORIGINAL VERSION
*
* PURPOSE :
* LSTRS METHOD: SELECTS INTERPOLATION POINT.
*
* PARAMETERS :
*  II  N  THE NUMBER OF VARIABLES.
*  RI  G(N)  GRADIENT.
*  RI  S(N)  DIRECTION VECTOR.
*  RI  SNORM  THE NORM OF A DIRECTION VECTOR S.
*  RI  LAMBDA1  THE 1-ST SMALLEST EIGENVALUE OF THE BORDERED MATRIX.
*  RI  LAMBDA2  THE 2-ND SMALLEST EIGENVALUE OF THE BORDERED MATRIX.
*  RI  NU1  THE FIRST COMPONENT OF THE 1-ST EIGENVECTOR.
*  RI  NU2  THE FIRST COMPONENT OF THE 2-ND EIGENVECTOR.
*  RI  U1(N)  COMPONENTS (2,...,N+1) OF THE 1-ST EIGENVECTOR.
*  RI  U2(N)  COMPONENTS (2,...,N+1) OF THE 2-ND EIGENVECTOR.
*  II  ITER  IF THE 1-ST OR 2-ND EIGENPAIR IS USED.
*  RO  LAMBDAA  SELECTED PREVIOUS INTERPOLATION POINT.
*  RO  LAMBDAB  SELECTED NEW INTERPOLATION POINT.
*  RO  FIA  SELECTED PREVIOUS INTERPOLATION POINT.
*  RO  FIB  SELECTED NEW INTERPOLATION POINT.
*  RO  SNORMA  SELECTED PREVIOUS INTERPOLATION POINT.
*  RO  SNORMB  SELECTED NEW INTERPOLATION POINT.
*  II  K  K=1 - ONE-POINT INTERPOLATION AT THE START OF THE ALGORITHM
*         K=2 - TWO-POINT INTERPOLATION IN THE NEXT ITERATIONS.
*
* SUBPROGRAMS USED :
*  S   UXVSET  A SCALAR IS SET TO ALL THE ELEMENTS OF A VECTOR.
*  S   UXVCOP  COPYING OF A VECTOR.
*  S   UXVSCL  SCALING OF A VECTOR.
*
      SUBROUTINE UYSORI(N,G,S,SNORM,LAMBDA1,LAMBDA2,NU1,NU2,U1,U2,
     & ITER,LAMBDAA,LAMBDAB,FIA,FIB,SNORMA,SNORMB,K)
      INTEGER N,ITER,K
      REAL*8 G(N),S(N),SNORM,LAMBDA1,LAMBDA2,NU1,NU2,U1(N),U2(N),
     & LAMBDAA,LAMBDAB,FIA,FIB,SNORMA,SNORMB
      REAL*8 IPOINTA(3),IPOINTB(3),UXVDOT
      REAL*8 ZERO,ONE
      PARAMETER (ZERO=0.0D0,ONE=1.0D0)
      IF (K.EQ.2) CALL UXVCOP(3,IPOINTB,IPOINTA)
      IF (ITER.EQ.0) THEN
      CALL UXVSET(3,ZERO,IPOINTB)
      ELSE IF (ITER.EQ.1) THEN
      CALL UXVSCL(N,ONE/NU1,U1,S)
      IPOINTB(1) = LAMBDA1
      IPOINTB(2) = -UXVDOT(N,G,S)
      IPOINTB(3) = SNORM
      ELSEIF (ITER.EQ.2) THEN
      CALL UXVSCL(N,ONE/NU2,U2,S)
      IPOINTB(1) = LAMBDA2
      IPOINTB(2) = -UXVDOT(N,G,S)
      IPOINTB(3) = SNORM
      ENDIF
      IF (K.EQ.1) THEN
      LAMBDAB = IPOINTB(1)
      FIB = IPOINTB(2)
      SNORMB = IPOINTB(3)
      ELSE
      LAMBDAA = IPOINTA(1)
      FIA = IPOINTA(2)
      SNORMA = IPOINTA(3)
      LAMBDAB = IPOINTB(1)
      FIB = IPOINTB(2)
      SNORMB = IPOINTB(3)
      ENDIF
      RETURN
      END
