* SUBROUTINE UYTHUT                ALL SYSTEMS                02/12/01
C PORTABILITY : ALL SYSTEMS
C 02/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* LIMITED STORAGE REDUCED HESSIAN VARIABLE METRIC TRANSFORMATIONS.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  IU  N  CURRENT NUMBER OF VARIABLE METRIC STEPS.
*  II  MF  MAXIMUM NUMBER OF VARIABLE METRIC STEPS.
*  RU  XM(NF*MF)  SET OF VECTOR OF VARIABLES DIFFERENCES.
*  RU  XR(MF*(MF+1)/2)  CHOLESKI DECOMPOSITION OF THE REDUCED
*         HESSIAN MATRIX.
*  RU  HR(MF*(MF+1)/2)  TRIANGULAR TRANFORMATION MATRIX.
*  RI  G(NF) GRADIENT VECTOR.
*  RO  GR(MF) REDUCED GRADIENT VECTOR.
*  RI  GNORM  NORM OF THE GRADIENT VECTOR.
*  RI  EPS7  TOLERANCE FOR LINEAR DEPENDENCE.
*  II  MOS  OPTION. MOS=1-INVERSION OF REDUCED HESSIAN MATRIX.
*         MOS=2-REDUCED INVERSION OF HESSIAN MATRIX.
*  IO  INF  LINEAR DEPENDENCE INDICATOR. INF=0 IF THE CURRENT GRADIENT
*         IS A LINEAR COMBINATION OF BASIC VECTORS. INF=1 OTHERWISE.
*  CO  TEXT  TYPE OF DECOMPOSITION. TEXT='R'-RECURSIVE QR DECOMPOSITION
*         WITHOUT PERMUTATIONS. TEXT='I'-INVERSION.
*
* SUBPROGRAMS USED :
*  S   UXDPRB  BACK SUBSTITUTION AFTER CHOLESKI DECOMPOSITION.
*  S   UXDRMM  VECTOR MULTIPLIED BY A ROWWISE STORED RECTANGULAR
*         MATRIX.
*  S   UXVCOP  COPYING OF A VECTOR.
*  S   UXVDIF  DIFFERENCE OF TWO VECTORS.
*  RF  UXVDOT  DOT PRODUCT OF TWO VECTORS.
*  S   UXVSCL  SCALING OF A VECTOR.
*  S   UXVSET  INITIATION OF A VECTOR.
*
      SUBROUTINE UYTHUT(NF,N,MF,XM,XR,HR,G,GR,GNORM,EPS7,MOS,INF,TEXT)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER NF,N,MF,MOS,INF
      REAL*8 XM(NF*MF),XR(MF*(MF+1)/2),HR(MF*(MF+1)/2),G(NF),GR(MF),
     & GNORM,EPS7
      CHARACTER*1 TEXT
      REAL*8 RHO,UXVDOT
      INTEGER I,IER
      IF (N.LT.MF) RETURN
      RHO=ABS(GR(N))
      IF (RHO.LE.EPS7*GNORM) THEN
      INF=0
      ELSE
      INF=1
      IF (TEXT.EQ.'R') THEN
      CALL UXDPRH(N,XR,HR,GR,GR,GR,IER,1)
      ELSE
      CALL UXDSMH(N,XR,HR,GR,GR,GR,IER,MOS,1)
      ENDIF
      DO 2 I=1,NF*(N-1)
      XM(I)=XM(I+NF)
    2 CONTINUE
      RHO=SQRT(ABS(UXVDOT(NF,G,G)-UXVDOT(N-1,GR,GR)))
      GR(N)=RHO
      CALL UXVCOP(NF,G,XM((N-1)*NF+1))
      CALL UXVCOP(N,GR,HR(N*(N-1)/2+1))
      ENDIF
      RETURN
      END
