* SUBROUTINE UYTSLM                 ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* REDUCED COST AND LAGRANGE MULTIPLIERS COMPUTATION.
*
* PARAMETERS :
*  IU  N ACTUAL NUMBER OF VARIABLES.
*  II  M NUMBER OF ROWS.
*  II  MCOLS DECLARED NUMBER OF VARIABLES.
*  II  MROWS DECLARED NUMBER OF ROWS.
*  II  NZ SIZE OF THE INDEX ARRAY CNR
*  II  NZA SIZE OF THE ARRAYS A,RNR.
*  II  NAU SIZE OF THE ARRAY THAT STORES THE NUMERICAL ELEMENTS OF
*          THE PROCESSED MATRIX.
*  II  NUP LENGTH OF THE ARRAYS LI,LJ,CM.
*  II  NEL NUMBER OF TRIPLES IN ARRAYS (LI,LJ,CM).
*  II  PTL(MROWS) POINTER VECTOR OF THE COLUMNS OF THE L-FACTOR.
*  II  LENL(MROWS) LENGTHS OF THE COLUMNS OF THE FACTOR L.
*  II  CNR(NZ) CONTAINS THE INDICES OF THE FACTORS.
*  II  PTR(MROWS) POINTER VECTOR OF THE ROWS OF THE FACTOR U.
*  II  LENR(MROWS) LENGTHS OF THE ROWS OF THE FACTOR U.
*  II  ORDC(MROWS) COLUMN PERMUTATION.
*  II  ORDR(MROWS) ROW PERMUTATION.
*  II  ORDL(MROWS) ROW PERMUTATION OF THE ORIGINAL FACTOR L.
*  II  LI(NUP) ROW INDICES OF THE L-UPDATES.
*  II  LJ(NUP) COLUMN INDICES OF THE L-UPDATES.
*  RI  CM(NUP) UPDATE MULTIPLIERS.
*  RI  AU(NZ) NUMERICAL VALUES OF THE FACTORS.
*  RO  LAM(MCOLS) VECTOR OF REDUCED COST/REDUCED GRADIENT.
*  RA  WR01(MCOLS) AUXILIARY VECTOR.
*  RO  GF(MCOLS) GRADIENT VECTOR.
*  II  BCOL(MCOLS) COLUMN ORDER IN BLOCKS, DEFINED AS OUTPUT FOR THE
*              QP PROBLEM.
*  II  RNR(NZA) COLUMNS OF THE MATRIX OF CONSTRAINTS.
*  II  PRNR(MCOLS+1) POINTER VECTOR FOR RNR.
*  RI  A(NZA) NUMERICAL VALUES OF THE MATRIX OF CONSTRAINTS.
*  IU  LINP NUMBER OF BASICS AND SUPERBASICS.
*
* COMMON DATA:
*  II  LIT  NUMBER OF THE FIRST ITERATION.
*  II  NIT  ACTUAL NUMBER OF ITERATIONS.
*
* SUBPROGRAMS USED :
*  S   UKQGLL  FILL THE VECTOR LAM WITH THE COST FUNCTION COEFFICIENTS
*         OF THE BASIC COLUMNS.
*  S   UKQGLV  TRANSFORMATION OF THE LAGRANGE MULTIPLIERS.
*
      SUBROUTINE UYTSLM(N,M,MCOLS,MROWS,NZ,NZA,NAU,NUP,NEL,PTL,LENL,CNR,
     & PTR,LENR,ORDC,ORDR,ORDL,LI,LJ,CM,AU,LAM,WR01,GF,BCOL,RNR,PRNR,A,
     & LINP)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER N,M,MCOLS,MROWS,NZ,NZA,NAU,NUP,NEL,PTL(MROWS),LENL(MROWS)
      INTEGER CNR(NZ),PTR(MROWS),LENR(MROWS),ORDC(MROWS),ORDR(MROWS)
      INTEGER ORDL(MROWS),LI(NUP),LJ(NUP)
      INTEGER BCOL(MCOLS),RNR(NZA),PRNR(MCOLS+1),LINP
      REAL*8 CM(NUP),AU(NAU),LAM(MCOLS),WR01(MCOLS),GF(MCOLS)
      REAL*8 A(NZA)
      INTEGER I,J,L
C
C ITERQ IS TESTED TO COMPUTE ALL MULTIPLIERS
C
      IF(ITERQ.NE.2) THEN
        L=LINP
        LINP=N
      ENDIF
C
C SET REDUCED COST AND GRADIENT
C
      IF(NIT.GT.LIT) THEN
C
C RETRIEVE MULTIPLIERS FOR NONBASICS
C
        DO 100 I=LINP+1,N
          J=BCOL(I)
          LAM(I)=LAM(I)+GF(J)
  100   CONTINUE
        CALL UKQGLL(M,MCOLS,BCOL,LAM,GF)
        CALL UKQGLV(N,M,MROWS,MCOLS,NZ,NZA,NAU,NUP,NEL,PTL,LENL,CNR,PTR,
     &  LENR,RNR,PRNR,ORDC,ORDR,ORDL,LI,LJ,CM,AU,A,WR01,BCOL,LAM,GF,
     &  LINP)
      END IF
C
C REDEFINE LINP
C
      IF(ITERQ.NE.2) THEN
        LINP=L
      ENDIF
      RETURN
      END
