$INTERFACE UA1HS4                                             98/12/01

  $HELP
    PRECISION OF THE COMPUTED GRADIENTS
  $CHECK(ETA1,ETA0,REAL,4)
  CALL UA1HS4(NF,NA,NAL,MA,MAH,$X,$XN,$S,$AH,$AG,$AGO,$IAG,$JAG,$AF&
,$AFO,$COLA,JVAR,$ETA1)
  $ADD(REAL,'\$X($NF)\$XN($NF)\$S($NF)\$AH($MAH)\$AG($MA)\$AGO($MA)&
  \$AF($NA)\$AFO($NA)')
  $ADD(INTEGER,'\$IAG($NA+1)\$JAG($MA)\$COLA($NF)')
$REM  $ADD(INTEGER,'\NF\NA\NAL\MA\M\JVAR')
  $ADD(SINID)
    CALL UXVSET(NF,ZERO,$GA)
  $ENDADD
  $ADD(INIT)
    NAHX='UA1HS4'
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UA1HS4                ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* NUMERICAL COMPUTATION OF THE HESSIAN MATRIX OF THE APPROXIMATED
* FUNCTION USING ITS GRADIENTS.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  NA  NUMBER OF FUNCTIONS.
*  II  NAL  NUMBER OF LINEAR FUNCTIONS.
*  II  MA  NUMBER OF ELEMENTS IN THE FIELD AG.
*  II  M  DIMENSION OF THE ARRAY H.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  RI  XN(NF)  VECTOR OF SCALING FACTORS.
*  RA  XO(NF)  AUXILIARY VECTOR.
*  RO  H(M)  SIMULTANEOUSLY COMPUTED SPARSE HESSIAN MATRICES.
*  RI  AG(MA)  SPARSE JACOBIAN MATRIX.
*  RA  AGO(MA)  AUXILIARY ARRAY.
*  RI  IAG(NA+1)  POSITION OF THE FIRST ROWS ELEMENTS IN THE FIELD AG.
*  RI  JAG(MA)  COLUMN INDICES OF ELEMENTS IN THE FIELD AG.
*  RI  AF(NA)  VECTOR CONTAINING VALUES OF THE APPROXIMATED
*         FUNCTIONS.
*  RO  AFO(NA)  SAVED VECTOR CONTAINING VALUES OF THE APPROXIMATED
*         FUNCTIONS.
*  II  IG(NF) VECTOR DEFINING GROUPS CONTAINING JACOBIAN COLUMNS.
*  II  NVAR  NUMBER OF GROUPS IN NUMERICAL DIFFERENTIATION.
*  RI  ETA1  PRECISION OF THE COMPUTED VALUES.
*
* COMMON DATA :
*  II  KBF  TYPE OF BOUNDS. KBF=0-BOUNDS ARE NOT USED. KBF=1-ONE SIDED
*         BOUNDS. KBF=1-TWO SIDED BOUNDS.
*  II  NORMF  SCALING SPECIFICATION. NORMF=0-NO SCALING IS PERFORMED.
*         NORMF=1-SCALING FACTORS ARE DETERMINED AUTOMATICALLY.
*         NORMF=2-SCALING FACTORS ARE SUPPLIED BY USER.
*
* STATUS VARIABLES :
*  NS,ISP,TSS
*
* SUBPROGRAMS USED :
*  S   UXVCOP  COPYING OF A VECTOR.
*  S   UYPRO1  PROLOGUE.
*  S   UYEPI1  EPILOGUE.
*  S   UYSET0  STATUS DEFINITION.
*
      SUBROUTINE UA1HS4(NF,NA,NAL,MA,M,X,XN,XO,H,AG,AGO,IAG,JAG,AF,
     & AFO,IG,NVAR,ETA1)
      $INCLUDE('UMCOMM')
      INTEGER NF,NA,NAL,MA,M,IAG(NA+1),JAG(MA),IG(NF),NVAR
      $FLOAT X(NF),XN(NF),XO(NF),H(M),AG(MA),AGO(MA),AF(NA),AFO(NA),ETA1
      $FLOAT XSTEP,ETA
      INTEGER I,J,K,L,II,IJ,JJ,KA,NB,IND
      INTEGER IVAR
      SAVE ETA
      SAVE IVAR
      IND(I,J)=MAX(I,J)*(MAX(I,J)-1)/2+MIN(I,J)
      IF (TSS(NS).NE.'UAHX') CALL UYPRO1('UAHX',-1)
      GO TO (1,3) ISP(NS)
    1 CONTINUE
      ETA=SQRT(ETA1)
      CALL UXVCOP(NF,X,XO)
      CALL UXVCOP(NA,AF,AFO)
      CALL UXVCOP(MA,AG,AGO)
      IVAR=1
    2 IF (IVAR.GT.NVAR) GO TO 5
C
C     STEP SELECTION
C
      DO 22 I=1,NF
      X(I)=XO(I)
      IF (IG(I).EQ.IVAR) THEN
      IF (NORMF.GT.0) THEN
      XSTEP=XN(I)
      ELSE
      XSTEP=ONE
      ENDIF
      XSTEP=ETA*MAX(ABS(X(I)),XSTEP)*SIGN(ONE,X(I))
      X(I)=X(I)+XSTEP
      ENDIF
   22 CONTINUE
      CALL UYSET0(1,2)
      RETURN
    3 CONTINUE
C
C     NUMERICAL DIFFERENTIATION
C
      NB=0
      DO 77 KA=1,NAL
      K=IAG(KA)
      L=IAG(KA+1)-K
      NB=NB+L*(L+1)/2
   77 CONTINUE
      DO 8 KA=NAL+1,NA
      K=IAG(KA)
      L=IAG(KA+1)-K
      DO 33 II=1,L
      I=ABS(JAG(K))
      IF (IG(I).EQ.IVAR) GO TO 4
      K=K+1
   33 CONTINUE
      GO TO 7
    4 K=IAG(KA)
      DO 44 JJ=1,L
      J=ABS(JAG(K))
      IJ=IND(II,JJ)
      IF(IG(J).GE.IG(I)) THEN
      H(NB+IJ)=(AG(K)-AGO(K))/(X(I)-XO(I))
      ELSE
      H(NB+IJ)=HALF*(H(NB+IJ)+(AG(K)-AGO(K))/(X(I)-XO(I)))
      ENDIF
      K=K+1
   44 CONTINUE
    7 NB=NB+L*(L+1)/2
    8 CONTINUE
      IVAR=IVAR+1
      GO TO 2
    5 CALL UXVCOP(NF,XO,X)
      CALL UXVCOP(NA,AFO,AF)
      CALL UXVCOP(MA,AGO,AG)
      CALL UYEPI1(-1)
      RETURN
      END
