$INTERFACE UASVG2                                             04/12/01

  $IF(NAL>0) $JASA='R'
  $HELP
    STORE OF JACOBIAN MATRIX FOR APPROXIMATED FUNCTIONS
      R - ROWWISE
      C - COLUMNWISE
  $CHECK(JASA,'R','R\C',1,NO)
  CALL UASVG2(NF,NA,KA,KAP,$XN,$AG,$GA,IDECA,ISNA,LDA,$NORMF,'$JASA')
  $ADD(REAL,'\$XN($NF)\$AG($NF*$NA)\$GA($NF)')
$REM  $ADD(INTEGER,'\NF\NA\KA\KAP\IDECA\ISNA\LDA')
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UASVG2             ALL SYSTEMS                   04/12/01
C PORTABILITY : ALL SYSTEMS
C 04/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* COMPUTATION OF THE GRADIENT OF THE LINEAR APPROXIMATED FUNCTION.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  NA  NUMBER OF FUNCTIONS.
*  II  KA  INDEX OF THE SELECTED FUNCTION.
*  II  KAP  SIGNUM OF THE SELECTED FUNCTION.
*  RI  XN(NF)  VECTOR OF SCALING FACTORS.
*  RO  AG(NF*NA)  DENSE JACOBIAN MATRIX.
*  RI  GA(NF)  GRADIENT OF THE SELECTED FUNCTION.
*  IU  IDECA  DECOMPOSITION INDICATOR. IDECA=0-NO DECOMPOSITION.
*  II  ISNA  SAVING SPECIFICATION. ISNA=0-NO SAVING. ISNA=1-FUNCTION
*         VALUES ARE SAVED. ISNA=2-FUNCTION VALUES AND GRADIENTS ARE
*         SAVED.
*  IU  LDA  DEGREE OF PREVIOUSLY COMPUTED DERIVATIVES.
*  II  NORMF  SCALING SPECIFICATION. NORMF=0-NO SCALING IS PERFORMED.
*         NORMF=1-SCALING FACTORS ARE DETERMINED AUTOMATICALLY.
*         NORMF=2-SCALING FACTORS ARE SUPPLIED BY USER.
*  TI  JASA  STORE OF THE JACOBIAN MATRIX. JASA='R'-ROWWISE.
*            JASA='C'-COLUMNWISE.
*
* COMMON DATA :
*
* SUBPROGRAMS USED :
*  S   UXVCOP  COPYING OF A VECTOR.
*  S   UXVMUL  MULTIPLICATION OF A VECTOR BY A DIAGONAL MATRIX.
*
      SUBROUTINE UASVG2(NF,NA,KA,KAP,XN,AG,GA,IDECA,ISNA,LDA,NORMF,JASA)
      INTEGER NF,NA,KA,KAP,IDECA,ISNA,LDA,NORMF
      $FLOAT XN(NF),AG(NF*NA),GA(NF)
      CHARACTER*1 JASA
      INTEGER I,L
      IF (ISNA.LT.2) RETURN
      IF (JASA.EQ.'R') THEN
      IF (NORMF.GT.0.AND.ISNA.GT.2) THEN
      CALL UXVMUL(NF,XN,GA,AG((KA-1)*NF+1),1)
      ELSE
      CALL UXVCOP(NF,GA,AG((KA-1)*NF+1))
      ENDIF
      IF (KAP.LT.0) CALL UXVNEG(NF,AG((KA-1)*NF+1),AG((KA-1)*NF+1))
      ELSE
      L=0
      IF (NORMF.GT.0.AND.ISNA.GT.2) THEN
      IF (KAP.GT.0) THEN
      DO 1 I=1,NF
      AG(L+KA)= XN(I)*GA(I)
      L=L+NA
    1 CONTINUE
      ELSE
      DO 2 I=1,NF
      AG(L+KA)=-XN(I)*GA(I)
      L=L+NA
    2 CONTINUE
      ENDIF
      ELSE
      IF (KAP.GT.0) THEN
      DO 3 I=1,NF
      AG(L+KA)= GA(I)
      L=L+NA
    3 CONTINUE
      ELSE
      DO 4 I=1,NF
      AG(L+KA)=-GA(I)
      L=L+NA
    4 CONTINUE
      ENDIF
      ENDIF
      ENDIF
      IF (KA.EQ.NA) LDA=MAX(LDA,1)
      IDECA=0
      RETURN
      END
