* SUBROUTINE UCCSEC             ALL SYSTEMS                 08/12/01
C PORTABILITY : ALL SYSTEMS
C 08/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* ACTIVE CONSTRAINT PART OF RIGHT HAND SIDE VECTOR.
*
* PARAMETERS :
*  II  NX  NUMBER OF BOX CONSTRAINTS.
*  II  NC  NUMBER OF GENERAL CONSTRAINTS.
*  II  NCA  NUMBER OF ACTIVE CONSTRAINTS.
*  RI  X(NX)  VECTOR OF VARIABLES.
*  II  IX(NX)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XL(NX)  VECTOR CONTAINING LOWER BOUNDS FOR VARIABLES.
*  RI  XU(NX)  VECTOR CONTAINING UPPER BOUNDS FOR VARIABLES.
*  RI  XSL(NX)  VECTOR CONTAINING LOVER SLACKS FOR VARIABLES.
*  RI  XSU(NX)  VECTOR CONTAINING UPPER SLACKS FOR VARIABLES.
*  RI  XZL(NX)  VECTOR CONTAINING LOVER MULTIPLIERS FOR VARIABLES.
*  RI  XZU(NX)  VECTOR CONTAINING UPPER MULTIPLIERS FOR VARIABLES.
*  RI  CF(NC)  VECTOR CONTAINING VALUES OF THE CONSTRAINT FUNCTIONS.
*  II  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  RI  CL(NC)  VECTOR CONTAINING LOWER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CU(NC)  VECTOR CONTAINING UPPER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CSL(NC)  VECTOR CONTAINING LOVER SLACKS FOR CONSTRAINTS.
*  RI  CSU(NC)  VECTOR CONTAINING UPPER SLACKS FOR CONSTRAINTS.
*  RI  CZL(NC)  VECTOR CONTAINING LOVER MULTIPLIERS FOR CONSTRAINTS.
*  RI  CZU(NC)  VECTOR CONTAINING UPPER MULTIPLIERS FOR CONSTRAINTS.
*  II  ICA(NCA)  VECTOR CONTAINING INDICES OF ACTIVE CONSTRAINTS.
*  II  ICCA(NCA)  VECTOR CONTAINING INDICES OF COUPLED COMPLEMENTARITY
*         CONSTRAINTS.
*  IO  D(2*(NX+NC))  ELEMENTS IN THE FIELD D.
*  IO  ID(NCA+1)  POSITION OF THE FIRST ROWS ELEMENTS IN THE FIELD D.
*  IO  JD(2*(NX+NC))  COLUMN INDICES OF ELEMENTS IN THE FIELD D.
*  RO  CP(NCA)  RIGHT HAND SIDE VECTOR.
*  RI  RPF3  BARRIER COEFFICIENT.
*  RI  RPF4  EXACT PENALTY COEFFICIENT.
*  RO  SIG1  SQUARED NORM OF THE PRECISION-MEASURE VECTOR.
*
      SUBROUTINE UCCSEC(NX,NC,NCA,X,IX,XL,XU,XSL,XSU,XZL,XZU,CF,IC,
     & CL,CU,CSL,CSU,CZL,CZU,ICA,ICCA,D,ID,JD,CP,RPF3,RPF4,SIG1)
      INTEGER NX,NC,NCA,IX(NX),IC(NC),ICA(NX+NC),ICCA(NX+NC),
     & ID(NX+NC+1),JD(2*(NX+NC))
      $FLOAT X(NX),XL(NX),XU(NX),XSL(NX),XSU(NX),XZL(NX),XZU(NX),
     & CF(NC),CL(NC),CU(NC),CSL(NC),CSU(NC),CZL(NC),CZU(NC),
     & D(2*(NX+NC)),CP(NX+NC),RPF3,RPF4,SIG1
      INTEGER I,J,K,L
      $FLOAT SK,SL,UK,UL,PK,PL,GK,GL,WK,WL,ZK,ZL,DD
      SIG1=0.0$P 0
      DO 1 I=1,NCA
      J=ICCA(I)
      IF (J.LE.0) THEN
      K=ICA(I)
      IF (K.LT.0) THEN
      K=-K
      IF (IX(K).EQ.-1.OR.IX(K).EQ.-3) THEN
      CP(I)=XL(K)-X(K)+RPF3/XZL(K)
      D(ID(I))=-XSL(K)/XZL(K)
      SIG1=SIG1+(XL(K)-X(K)+XSL(K))**2
      ELSE IF (IX(K).EQ.-2.OR.IX(K).EQ.-4) THEN
      CP(I)=XU(K)-X(K)-RPF3/XZU(K)
      D(ID(I))=-XSU(K)/XZU(K)
      SIG1=SIG1+(XU(K)-X(K)-XSU(K))**2
      ENDIF
      ELSE
      IF (IC(K).EQ.-1.OR.IC(K).EQ.-3) THEN
      CP(I)=CL(K)-CF(K)+RPF3/CZL(K)
      D(ID(I))=-CSL(K)/CZL(K)
      SIG1=SIG1+(CL(K)-CF(K)+CSL(K))**2
      ELSE IF (IC(K).EQ.-2.OR.IC(K).EQ.-4) THEN
      CP(I)=CU(K)-CF(K)-RPF3/CZU(K)
      D(ID(I))=-CSU(K)/CZU(K)
      SIG1=SIG1+(CU(K)-CF(K)-CSU(K))**2
      ELSE IF (IC(K).EQ.-5) THEN
      CP(I)=-CF(K)
      D(ID(I))=0.0$P 0
      SIG1=SIG1+CP(I)**2
      ENDIF
      ENDIF
      ELSE IF (J.GT.I) THEN
      K=ICA(I)
      ZK=1.0$P 0
      IF (K.LT.0) THEN
      K=-K
      IF (IX(K).EQ.-1) THEN
      ZK=-ZK
      SK=XSL(K)
      UK=XZL(K)
      PK=XL(K)-X(K)+SK
      ELSE IF (IX(K).EQ.-2) THEN
      SK=XSU(K)
      UK=XZU(K)
      PK=X(K)-XU(K)+SK
      ENDIF
      ELSE
      IF (IC(K).EQ.-1) THEN
      ZK=-ZK
      SK=CSL(K)
      UK=CZL(K)
      PK=CL(K)-CF(K)+SK
      ELSE IF (IC(K).EQ.-2) THEN
      SK=CSU(K)
      UK=CZU(K)
      PK=CF(K)-CU(K)+SK
      ENDIF
      ENDIF
      L=ICA(J)
      ZL=1.0$P 0
      IF (L.LT.0) THEN
      L=-L
      IF (IX(L).EQ.-1) THEN
      ZL=-ZL
      SL=XSL(L)
      UL=XZL(L)
      PL=XL(L)-X(L)+SL
      ELSE IF (IX(L).EQ.-2) THEN
      SL=XSU(L)
      UL=XZU(L)
      PL=X(L)-XU(L)+SL
      ENDIF
      ELSE
      IF (IC(L).EQ.-1) THEN
      ZL=-ZL
      SL=CSL(L)
      UL=CZL(L)
      PL=CL(L)-CF(L)+SL
      ELSE IF (IC(L).EQ.-2) THEN
      SL=CSU(L)
      UL=CZU(L)
      PL=CF(L)-CU(L)+SL
      ENDIF
      ENDIF
      WK=UK+RPF4*SL
      WL=UL+RPF4*SK
      GK=SK*WK-RPF3
      GL=SL*WL-RPF3
      DD=1.0$P 0/(UK*UL+RPF4*(SK*UK+SL*UL))
      CP(I)=-ZK*(PK-DD*(WL*GK-RPF4*SK*GL))
      CP(J)=-ZL*(PL-DD*(WK*GL-RPF4*SL*GK))
      D(ID(I))=-DD*SK*WL
      D(ID(J))=-DD*SL*WK
      D(ID(I)+1)=ZK*ZL*DD*RPF4*SK*SL
      SIG1=SIG1+PK**2+PL**2
      ENDIF
    1 CONTINUE
      RETURN
      END

