* SUBROUTINE UCFSMM             ALL SYSTEMS                 08/12/01
C PORTABILITY : ALL SYSTEMS
C 08/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* GRADIENT PART OF RIGHT HAND SIDE VECTOR.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  NX  NUMBER OF BOX CONSTRAINTS.
*  II  NC  NUMBER OF GENERAL CONSTRAINTS.
*  II  MC  NUMBER OF NONZERO ELEMENTS OF THE JACOBIAN MATRIX.
*  II  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XL(NF)  VECTOR CONTAINING LOWER BOUNDS FOR VARIABLES.
*  RI  XU(NF)  VECTOR CONTAINING UPPER BOUNDS FOR VARIABLES.
*  RI  CF(NC)  VECTOR CONTAINING VALUES OF THE CONSTRAINT FUNCTIONS.
*  II  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  RI  CL(NC)  VECTOR CONTAINING LOWER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CU(NC)  VECTOR CONTAINING UPPER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CG(MC)  ELEMENTS OF THE CONSTRAINT JACOBIAN MATRIX.
*  II  ICG(NC+1)  POSITIONS OF THE FIRST ROWS ELEMENTS IN THE SPARSE
*         JACOBIAN STRUCTURE.
*  II  JCG(MC)  COLUMN INDICES OF ELEMENTS IN THE SPARSE JACOBIAN
*         STRUCTURE.
*  RI  XO(NF)  INPUT VECTOR.
*  RO  GO(NF)  OUTPUT VECTOR.
*  RI  XSL(NX)  VECTOR CONTAINING LOVER SLACKS FOR VARIABLES.
*  RI  XSU(NX)  VECTOR CONTAINING UPPER SLACKS FOR VARIABLES.
*  RI  XZL(NF)  VECTOR CONTAINING LOVER MULTIPLIERS FOR VARIABLES.
*  RI  XZU(NF)  VECTOR CONTAINING UPPER MULTIPLIERS FOR VARIABLES.
*  RI  CSL(NC)  VECTOR CONTAINING LOVER SLACKS FOR CONSTRAINTS.
*  RI  CSU(NC)  VECTOR CONTAINING UPPER SLACKS FOR CONSTRAINTS.
*  RI  CZL(NC)  VECTOR CONTAINING LOVER MULTIPLIERS FOR CONSTRAINTS.
*  RI  CZU(NC)  VECTOR CONTAINING UPPER MULTIPLIERS FOR CONSTRAINTS.
*
*  COMMON DATA :
*  II  KBF  SPECIFICATION OF SIMPLE BOUNDS. KBF=0-NO SIMPLE BOUNDS.
*         KBF=1-ONE SIDED SIMPLE BOUNDS. KBF=2=TWO SIDED SIMPLE BOUNDS.
*  II  KBC  SPECIFICATION OF LINEAR CONSTRAINTS. KBC=0-NO LINEAR
*         CONSTRAINTS. KBC=1-ONE SIDED LINEAR CONSTRAINTS. KBC=2=TWO
*         SIDED LINEAR CONSTRAINTS.
*
      SUBROUTINE UCFSMM(NF,NX,NC,MC,X,IX,XL,XU,CF,IC,CL,CU,CG,ICG,JCG,
     & XO,GO,XSL,XSU,XZL,XZU,CSL,CSU,CZL,CZU)
      $INCLUDE ('UMCOMM')
      INTEGER NF,NX,NC,MC,IX(NX),IC(NC),ICG(NC+1),JCG(MC)
      $FLOAT X(NF),XL(NX),XU(NX),CF(NC),CL(NC),CU(NC),CG(MC),XO(NF),
     & GO(NF),XSL(NX),XSU(NX),XZL(NX),XZU(NX),CSL(NC),CSU(NC),CZL(NC),
     & CZU(NC)
      INTEGER I,K,KK,KC,L
      $FLOAT DENL,DENU,NUML,NUMU,TEMP,UXWDOT
      IF (KBF.GT.0) THEN
      DO 10 I=1,NX
      KK=IX(I)
      IF (KK.EQ.1.OR.KK.EQ.3.OR.ABS(KK).EQ.4) THEN
      NUML=ABS(XSL(I)-XZL(I))
      DENL=ABS(XSL(I)+XL(I)-X(I))
      GO(I)=GO(I)+XO(I)*DENL/NUML
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.4.OR.ABS(KK).EQ.3) THEN
      NUMU=ABS(XSU(I)-XZU(I))
      DENU=ABS(XSU(I)+X(I)-XU(I))
      GO(I)=GO(I)+XO(I)*DENU/NUMU
      ENDIF
   10 CONTINUE
      ENDIF
      IF (KBC.GT.0) THEN
      DO 20 KC=1,NC
      KK=IC(KC)
      K=ICG(KC)
      L=ICG(KC+1)-K
      IF (KK.EQ.1.OR.KK.EQ.3.OR.ABS(KK).EQ.4) THEN
      NUML=ABS(CSL(KC)-CZL(KC))
      DENL=ABS(CSL(KC)+CL(KC)-CF(KC))
      TEMP=UXWDOT(L,NF,JCG(K),CG(K),XO,2)*DENL/NUML
      CALL UXWDIR(L,NF,JCG(K),TEMP,CG(K),GO,GO,1)
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.4.OR.ABS(KK).EQ.3) THEN
      NUMU=ABS(CSU(KC)-CZU(KC))
      DENU=ABS(CSU(KC)+CF(KC)-CU(KC))
      TEMP=UXWDOT(L,NF,JCG(K),CG(K),XO,2)*DENU/NUMU
      CALL UXWDIR(L,NF,JCG(K),TEMP,CG(K),GO,GO,1)
      ENDIF
   20 CONTINUE
      ENDIF
      RETURN
      END
