* SUBROUTINE UCIDEB             ALL SYSTEMS                 04/12/01
C PORTABILITY : ALL SYSTEMS
C 04/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* DETERMINATION OF THE DIRECTIONAL DERIVATIVE.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  NX  NUMBER OF BOX CONSTRAINTS.
*  II  NC  NUMBER OF GENERAL CONSTRAINTS.
*  II  NCA  NUMBER OF ACTIVE CONSTRAINTS.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XL(NF)  VECTOR CONTAINING LOWER BOUNDS FOR VARIABLES.
*  RI  XU(NF)  VECTOR CONTAINING UPPER BOUNDS FOR VARIABLES.
*  RI  XSL(NX)  VECTOR CONTAINING LOVER SLACKS FOR VARIABLES.
*  RI  XSU(NX)  VECTOR CONTAINING UPPER SLACKS FOR VARIABLES.
*  RI  XZL(NF)  VECTOR CONTAINING LOVER MULTIPLIERS FOR VARIABLES.
*  RI  XZU(NF)  VECTOR CONTAINING UPPER MULTIPLIERS FOR VARIABLES.
*  RI  CF(NC+1)  VECTOR CONTAINING VALUES OF THE CONSTRAINT FUNCTIONS.
*  RI  CFD(NC)  VECTOR CONTAINING INCREMENTS OF CONSTRAINT FUNCTIONS.
*  II  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  RI  CL(NC)  VECTOR CONTAINING LOWER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CU(NC)  VECTOR CONTAINING UPPER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CSL(NC)  VECTOR CONTAINING LOVER SLACKS FOR CONSTRAINTS.
*  RI  CSU(NC)  VECTOR CONTAINING UPPER SLACKS FOR CONSTRAINTS.
*  RI  CZL(NC)  VECTOR CONTAINING LOVER MULTIPLIERS FOR CONSTRAINTS.
*  RI  CZU(NC)  VECTOR CONTAINING UPPER MULTIPLIERS FOR CONSTRAINTS.
*  II  ICA(NCA)  VECTOR CONTAINING INDICES OF ACTIVE CONSTRAINTS.
*  II  S(NF+NCA)  VECTOR CONTAINING NEWTON INCREMENTS.
*  RI  GS(NCA)  VECTOR CONTAINING ACTIVE SLACK INCREMENTS.
*  RI  XO(NX+NC)  VECTOR CONTAINING INCREMENTS FOR LOWER INACTIVE
*         MULTIPLIERS.
*  RI  GO(NX+NC)  VECTOR CONTAINING INCREMENTS FOR UPPER INACTIVE
*         MULTIPLIERS.
*  RU  P  STEPSIZE  PARAMETER.
*  RI  RPF3  BARRIER COEFFICIENT.
*
*  COMMON DATA :
*  II  KBF  SPECIFICATION OF SIMPLE BOUNDS. KBF=0-NO SIMPLE BOUNDS.
*         KBF=1-ONE SIDED SIMPLE BOUNDS. KBF=2=TWO SIDED SIMPLE BOUNDS.
*  II  KBC  SPECIFICATION OF LINEAR CONSTRAINTS. KBC=0-NO LINEAR
*         CONSTRAINTS. KBC=1-ONE SIDED LINEAR CONSTRAINTS. KBC=2=TWO
*         SIDED LINEAR CONSTRAINTS.
*
      SUBROUTINE UCIDEB(NF,NX,NC,NCA,X,IX,XL,XU,XSL,XSU,XZL,XZU,CF,CFD,
     & IC,CL,CU,CSL,CSU,CZL,CZU,ICA,S,GS,XO,GO,RPF3,P)
      $INCLUDE ('UMCOMM')
      INTEGER NF,NX,NC,NCA,IX(NX),IC(NC),ICA(NCA)
      $FLOAT X(NF),XL(NX),XU(NX),XSL(NX),XSU(NX),XZL(NX),XZU(NX),
     & CF(NC),CFD(NC),CL(NC),CU(NC),CSL(NC),CSU(NC),CZL(NC),CZU(NC),
     & S(NF+NCA),GS(NCA),XO(NX+NC),GO(NX+NC),RPF3,P
      INTEGER I,K,KK
      $FLOAT TEMP,TEMP1,TEMP2
      DO 1 I=1,NCA
      TEMP=GS(I)
      K=ICA(I)
      IF (K.LT.0) THEN
      K=-K
      KK=-IX(K)
      IF (KK.EQ.1.OR.KK.EQ.3) THEN
      TEMP1=S(NF+I)
      TEMP2=XL(K)-X(K)+XSL(K)
      P=P-RPF3*TEMP/XSL(K)+TEMP*(XZL(K)+TEMP1)+S(K)*XZL(K)
      ELSE IF (KK.EQ.2.OR.KK.EQ.4) THEN
      TEMP1=S(NF+I)
      TEMP2=X(K)-XU(K)+XSU(K)
      P=P-RPF3*TEMP/XSU(K)+TEMP*(XZU(K)+TEMP1)-S(K)*XZU(K)
      ENDIF
      ELSE
      KK=-IC(K)
      IF (KK.EQ.1.OR.KK.EQ.3) THEN
      TEMP1=S(NF+I)
      TEMP2=CL(K)-CF(K)+CSL(K)
      P=P-RPF3*TEMP/CSL(K)+TEMP*(CZL(K)+TEMP1)+CFD(K)*CZL(K)
      ELSE IF (KK.EQ.2.OR.KK.EQ.4) THEN
      TEMP1=S(NF+I)
      TEMP2=CF(K)-CU(K)+CSU(K)
      P=P-RPF3*TEMP/CSU(K)+TEMP*(CZU(K)+TEMP1)-CFD(K)*CZU(K)
      ELSE IF (KK.EQ.5) THEN
      P=P-CFD(K)*CZL(K)
      ENDIF
      ENDIF
    1 CONTINUE
      IF (KBF.GT.0) THEN
      DO 2 K=1,NX
      KK=IX(K)
      IF (KK.EQ.1.OR.KK.EQ.3.OR.ABS(KK).EQ.4) THEN
      TEMP1=XO(K)
      TEMP2=XL(K)-X(K)+XSL(K)
      TEMP=-(TEMP2-S(K))
      P=P-RPF3*TEMP/XSL(K)+TEMP*(XZL(K)+TEMP1)+S(K)*XZL(K)
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.4.OR.ABS(KK).EQ.3) THEN
      TEMP1=GO(K)
      TEMP2=X(K)-XU(K)+XSU(K)
      TEMP=-(TEMP2+S(K))
      P=P-RPF3*TEMP/XSU(K)+TEMP*(XZU(K)+TEMP1)-S(K)*XZU(K)
      ENDIF
    2 CONTINUE
      ENDIF
      IF (KBC.GT.0) THEN
      DO 3 K=1,NC
      KK=IC(K)
      IF (KK.EQ.1.OR.KK.EQ.3.OR.ABS(KK).EQ.4) THEN
      TEMP1=XO(NX+K)
      TEMP2=CL(K)-CF(K)+CSL(K)
      TEMP=-(TEMP2-CFD(K))
      P=P-RPF3*TEMP/CSL(K)+TEMP*(CZL(K)+TEMP1)+CFD(K)*CZL(K)
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.4.OR.ABS(KK).EQ.3) THEN
      TEMP1=GO(NX+K)
      TEMP2=CF(K)-CU(K)+CSU(K)
      TEMP=-(TEMP2+CFD(K))
      P=P-RPF3*TEMP/CSU(K)+TEMP*(CZU(K)+TEMP1)-CFD(K)*CZU(K)
      ENDIF
    3 CONTINUE
      ENDIF
      RETURN
      END
