* SUBROUTINE UCILAG             ALL SYSTEMS                 06/12/01
C PORTABILITY : ALL SYSTEMS
C 06/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* COMPUTATION OF THE VALUE AND THE GRADIENT OF THE OBJECTIVE FUNCTION.
* COMPUTATION OF THE VALUES AND THE GRADIENTS OF THE CONSTRAINT
* FUNCTIONS.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  NX  NUMBER OF BOX CONSTRAINTS.
*  II  NC  NUMBER OF GENERAL CONSTRAINTS.
*  II  MC  NUMBER OF NONZERO ELEMENTS OF THE SPARSE JACOBIAN MATRIX.
*  II  IX(NX)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XL(NX)  VECTOR CONTAINING LOWER BOUNDS FOR VARIABLES.
*  RI  XU(NX)  VECTOR CONTAINING UPPER BOUNDS FOR VARIABLES.
*  RI  XZL(NX)  VECTOR CONTAINING LOVER MULTIPLIERS FOR VARIABLES.
*  RI  XZU(NX)  VECTOR CONTAINING UPPER MULTIPLIERS FOR VARIABLES.
*  II  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  RI  CL(NC)  VECTOR CONTAINING LOWER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CU(NC)  VECTOR CONTAINING UPPER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CZL(NC)  VECTOR CONTAINING LOVER MULTIPLIERS FOR CONSTRAINTS.
*  RI  CZU(NC)  VECTOR CONTAINING UPPER MULTIPLIERS FOR CONSTRAINTS.
*  RI  CG(MC)  GRADIENT OF THE SELECTED FUNCTION.
*  II  ICG(NC+1)  POSITIONS OF THE FIRST ROWS ELEMENTS IN THE SPARSE
*         JACOBIAN STRUCTURE.
*  II  JCG(MC)  COLUMN INDICES OF ELEMENTS IN THE SPARSE JACOBIAN
*         STRUCTURE.
*  RO  G(NF)  GRADIENT OF THE LAGRANGIAN FUNCTION.
*
* COMMON DATA :
*  II  KBF  SPECIFICATION OF SIMPLE BOUNDS. KBF=0-NO SIMPLE BOUNDS.
*         KBF=1-ONE SIDED SIMPLE BOUNDS. KBF=2=TWO SIDED SIMPLE BOUNDS.
*  II  KBC  SPECIFICATION OF LINEAR CONSTRAINTS. KBC=0-NO LINEAR
*         CONSTRAINTS. KBC=1-ONE SIDED LINEAR CONSTRAINTS. KBC=2=TWO
*         SIDED LINEAR CONSTRAINTS.
*  II  IEXT  TYPE OF EXTREMUM. IEXT=0-MINIMUM. IEXT=1-MAXIMUM.
*
      SUBROUTINE UCILAG(NF,NX,NC,MC,IX,XZL,XZU,GF,IC,CZL,CZU,CG,
     & ICG,JCG,G)
      $INCLUDE ('UMCOMM')
      INTEGER NF,NX,NC,MC,IX(NX),IC(NC),ICG(NC+1),JCG(MC)
      $FLOAT XZL(NX),XZU(NX),GF(NF),CZL(NC),CZU(NC),CG(MC),G(NF)
      INTEGER I,J,K,KK,KC,L,JP
      $FLOAT TEMP
      IF (IEXT.LE.0) THEN
      CALL UXVCOP(NF,GF,G)
      ELSE
      CALL UXVNEG(NF,GF,G)
      ENDIF
      IF (KBF.GT.0) THEN
      DO 7 I=1,NX
      KK=ABS(IX(I))
      IF (KK.EQ.1.OR.KK.EQ.3.OR.KK.EQ.4) THEN
      G(I)=G(I)-XZL(I)
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.3.OR.KK.EQ.4) THEN
      G(I)=G(I)+XZU(I)
      ENDIF
    7 CONTINUE
      ENDIF
      IF (KBC.GT.0) THEN
      DO 9 KC=1,NC
      KK=ABS(IC(KC))
      IF (KK.EQ.3.OR.KK.EQ.4) THEN
      TEMP= CZU(KC)-CZL(KC)
      ELSE IF (KK.EQ.1) THEN
      TEMP=-CZL(KC)
      ELSE IF (KK.EQ.2) THEN
      TEMP= CZU(KC)
      ELSE IF (KK.EQ.5) THEN
      TEMP= CZL(KC)
      ENDIF
      K=ICG(KC)
      L=ICG(KC+1)-K
      DO 8 J=1,L
      JP=ABS(JCG(K))
      G(JP)=G(JP)+TEMP*CG(K)
      K=K+1
    8 CONTINUE
    9 CONTINUE
      ENDIF
      RETURN
      END
