* SUBROUTINE UCIMON             ALL SYSTEMS                 03/12/01
C PORTABILITY : ALL SYSTEMS
C 03/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* DETERMINATION OF THE DIRECTIONAL DERIVATIVE.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  NX  NUMBER OF BOX CONSTRAINTS.
*  II  NC  NUMBER OF GENERAL CONSTRAINTS.
*  II  NCA  NUMBER OF ACTIVE CONSTRAINTS.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XL(NF)  VECTOR CONTAINING LOWER BOUNDS FOR VARIABLES.
*  RI  XU(NF)  VECTOR CONTAINING UPPER BOUNDS FOR VARIABLES.
*  RI  XSL(NX)  VECTOR CONTAINING LOVER SLACKS FOR VARIABLES.
*  RI  XSU(NX)  VECTOR CONTAINING UPPER SLACKS FOR VARIABLES.
*  RI  XZL(NF)  VECTOR CONTAINING LOVER MULTIPLIERS FOR VARIABLES.
*  RI  XZU(NF)  VECTOR CONTAINING UPPER MULTIPLIERS FOR VARIABLES.
*  RI  CF(NC+1)  VECTOR CONTAINING VALUES OF THE CONSTRAINT FUNCTIONS.
*  RI  CFD(NC)  VECTOR CONTAINING INCREMENTS OF CONSTRAINT FUNCTIONS.
*  II  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  RI  CL(NC)  VECTOR CONTAINING LOWER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CU(NC)  VECTOR CONTAINING UPPER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CSL(NC)  VECTOR CONTAINING LOVER SLACKS FOR CONSTRAINTS.
*  RI  CSU(NC)  VECTOR CONTAINING UPPER SLACKS FOR CONSTRAINTS.
*  RI  CZL(NC)  VECTOR CONTAINING LOVER MULTIPLIERS FOR CONSTRAINTS.
*  RI  CZU(NC)  VECTOR CONTAINING UPPER MULTIPLIERS FOR CONSTRAINTS.
*  II  ICA(NCA)  VECTOR CONTAINING INDICES OF ACTIVE CONSTRAINTS.
*  II  S(NF+NCA)  VECTOR CONTAINING NEWTON INCREMENTS.
*  RI  GS(NCA)  VECTOR CONTAINING ACTIVE SLACK INCREMENTS.
*  RI  RAT  RATIO OF TRUST REGION RADIUS.
*  RI  RPF1  OLD PENALTY COEFFICIENT.
*  RI  RPF2  NEW PENALTY COEFFICIENT.
*  RI  RPF3  BARRIER COEFFICIENT.
*  RU  P  STEPSIZE  PARAMETER.
*  RU  PP  SECOND ORDER TERM.
*  II  IPOM  CHOICE OF THE PENALTY COEFFICIENT. IPOM=1-OLD PENALTY
*         COEFFICIENT. IPOM=2-NEW PENALTY COEFFICIENT DETERMINED
*         FROM THE QUADRATIC MODEL.
*  II  JOB  CHOICE OF THE IP FORMULATION. JOB=1-PRIMAL FORMULATION.
*         JOB=2-PRIMAL-DUAL FORMULATION.
*
*  COMMON DATA :
*  II  KBF  SPECIFICATION OF SIMPLE BOUNDS. KBF=0-NO SIMPLE BOUNDS.
*         KBF=1-ONE SIDED SIMPLE BOUNDS. KBF=2=TWO SIDED SIMPLE BOUNDS.
*  II  KBC  SPECIFICATION OF LINEAR CONSTRAINTS. KBC=0-NO LINEAR
*         CONSTRAINTS. KBC=1-ONE SIDED LINEAR CONSTRAINTS. KBC=2=TWO
*         SIDED LINEAR CONSTRAINTS.
*
      SUBROUTINE UCIMON(NF,NX,NC,NCA,X,IX,XL,XU,XSL,XSU,XZL,XZU,CF,CFD,
     & IC,CL,CU,CSL,CSU,CZL,CZU,ICA,S,GS,RAT,RPF1,RPF2,RPF3,P,PP,IPOM,
     & JOB)
      $INCLUDE ('UMCOMM')
      INTEGER NF,NX,NC,NCA,IX(NX),IC(NC),ICA(NCA),IPOM,JOB
      $FLOAT X(NF),XL(NX),XU(NX),XSL(NX),XSU(NX),XZL(NX),XZU(NX),
     & CF(NC),CFD(NC),CL(NC),CU(NC),CSL(NC),CSU(NC),CZL(NC),CZU(NC),
     & S(NF+NCA),GS(NCA),RAT,RPF1,RPF2,RPF3,P,PP
      INTEGER I,K,KK
      $FLOAT TEMP,TEMP1,PNEW,POLD,CON
      PARAMETER (CON=0.9$P 0)
      POLD=ZERO
      PNEW=ZERO
      DO 1 I=1,NCA
      TEMP=GS(I)
      K=ICA(I)
      IF (K.LT.0) THEN
      K=-K
      KK=-IX(K)
      IF (KK.EQ.1.OR.KK.EQ.3) THEN
      TEMP1=XL(K)-X(K)+XSL(K)
      POLD=POLD+TEMP1**2
      PNEW=PNEW+(TEMP1-S(K)+TEMP)**2
      P=P-RPF3*TEMP/XSL(K)
      IF (JOB.EQ.1) THEN
      PP=PP+RPF3*(TEMP/XSL(K))**2
      ELSE
      PP=PP+(XZL(K)/XSL(K))*TEMP**2
      ENDIF
      ELSE IF (KK.EQ.2.OR.KK.EQ.4) THEN
      TEMP1=X(K)-XU(K)+XSU(K)
      POLD=POLD+TEMP1**2
      PNEW=PNEW+(TEMP1+S(K)+TEMP)**2
      P=P-RPF3*TEMP/XSU(K)
      IF (JOB.EQ.1) THEN
      PP=PP+RPF3*(TEMP/XSU(K))**2
      ELSE
      PP=PP+(XZU(K)/XSU(K))*TEMP**2
      ENDIF
      ENDIF
      ELSE
      KK=-IC(K)
      IF (KK.EQ.1.OR.KK.EQ.3) THEN
      TEMP1=CL(K)-CF(K)+CSL(K)
      POLD=POLD+TEMP1**2
      PNEW=PNEW+(TEMP1-CFD(K)+TEMP)**2
      P=P-RPF3*TEMP/CSL(K)
      IF (JOB.EQ.1) THEN
      PP=PP+RPF3*(TEMP/CSL(K))**2
      ELSE
      PP=PP+(CZL(K)/CSL(K))*TEMP**2
      ENDIF
      ELSE IF (KK.EQ.2.OR.KK.EQ.4) THEN
      TEMP1=CF(K)-CU(K)+CSU(K)
      POLD=POLD+TEMP1**2
      PNEW=PNEW+(TEMP1+CFD(K)+TEMP)**2
      P=P-RPF3*TEMP/CSU(K)
      IF (JOB.EQ.1) THEN
      PP=PP+RPF3*(TEMP/CSU(K))**2
      ELSE
      PP=PP+(CZU(K)/CSU(K))*TEMP**2
      ENDIF
      ELSE IF (KK.EQ.5) THEN
      POLD=POLD+CF(K)**2
      PNEW=PNEW+(CF(K)+CFD(K))**2
      ENDIF
      ENDIF
    1 CONTINUE
      IF (KBF.GT.0) THEN
      DO 2 K=1,NX
      KK=IX(K)
      IF (KK.EQ.1.OR.KK.EQ.3.OR.ABS(KK).EQ.4) THEN
      TEMP1=XL(K)-X(K)+XSL(K)
      TEMP=-RAT*(TEMP1-S(K))
      POLD=POLD+TEMP1**2
      P=P-RPF3*TEMP/XSL(K)
      IF (JOB.EQ.1) THEN
      PP=PP+RPF3*(TEMP**2-S(K)**2)/XSL(K)**2
      ELSE
      PP=PP+(XZL(K)/XSL(K))*(TEMP**2-S(K)**2)
      ENDIF
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.4.OR.ABS(KK).EQ.3) THEN
      TEMP1=X(K)-XU(K)+XSU(K)
      TEMP=-RAT*(TEMP1+S(K))
      POLD=POLD+TEMP1**2
      P=P-RPF3*TEMP/XSU(K)
      IF (JOB.EQ.1) THEN
      PP=PP+RPF3*(TEMP**2-S(K)**2)/XSU(K)**2
      ELSE
      PP=PP+(XZU(K)/XSU(K))*(TEMP**2-S(K)**2)
      ENDIF
      ENDIF
    2 CONTINUE
      ENDIF
      IF (KBC.GT.0) THEN
      DO 3 K=1,NC
      KK=IC(K)
      IF (KK.EQ.1.OR.KK.EQ.3.OR.ABS(KK).EQ.4) THEN
      TEMP1=CL(K)-CF(K)+CSL(K)
      TEMP=-RAT*(TEMP1-CFD(K))
      POLD=POLD+TEMP1**2
      P=P-RPF3*TEMP/CSL(K)
      IF (JOB.EQ.1) THEN
      PP=PP+RPF3*(TEMP**2-CFD(K)**2)/CSL(K)**2
      ELSE
      PP=PP+(CZL(K)/CSL(K))*(TEMP**2-CFD(K)**2)
      ENDIF
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.4.OR.ABS(KK).EQ.3) THEN
      TEMP1=CF(K)-CU(K)+CSU(K)
      TEMP=-RAT*(TEMP1+CFD(K))
      POLD=POLD+TEMP1**2
      P=P-RPF3*TEMP/CSU(K)
      IF (JOB.EQ.1) THEN
      PP=PP+RPF3*(TEMP**2-CFD(K)**2)/CSU(K)**2
      ELSE
      PP=PP+(CZU(K)/CSU(K))*(TEMP**2-CFD(K)**2)
      ENDIF
      ENDIF
    3 CONTINUE
      ENDIF
      PP=HALF*PP
      POLD=SQRT(POLD)
      PNEW=SQRT(PNEW)
      IF (IPOM.LE.1.OR.POLD-PNEW.LE.ZERO) THEN
      RPF2=RPF1
      PP=MIN(PP,CON*ABS(P))
      ELSE
      RPF2=MAX(RPF1,1.1$P 1*(P+PP)/(POLD-PNEW))
      ENDIF
      P=P-RPF2*(POLD-PNEW)
      RETURN
      END
