* SUBROUTINE UCINAD             ALL SYSTEMS                 06/12/01
C PORTABILITY : ALL SYSTEMS
C 06/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* GRADIENT PART OF RIGHT HAND SIDE VECTOR.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  NX  NUMBER OF BOX CONSTRAINTS.
*  II  NC  NUMBER OF GENERAL CONSTRAINTS.
*  II  MC  NUMBER OF NONZERO ELEMENTS OF THE JACOBIAN MATRIX.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XL(NF)  VECTOR CONTAINING LOWER BOUNDS FOR VARIABLES.
*  RI  XU(NF)  VECTOR CONTAINING UPPER BOUNDS FOR VARIABLES.
*  RI  XSL(NX)  VECTOR CONTAINING LOVER SLACKS FOR VARIABLES.
*  RI  XSU(NX)  VECTOR CONTAINING UPPER SLACKS FOR VARIABLES.
*  RI  XZL(NF)  VECTOR CONTAINING LOVER MULTIPLIERS FOR VARIABLES.
*  RI  XZU(NF)  VECTOR CONTAINING UPPER MULTIPLIERS FOR VARIABLES.
*  RI  CF(NC)  VECTOR CONTAINING VALUES OF THE CONSTRAINT FUNCTIONS.
*  II  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  RI  CL(NC)  VECTOR CONTAINING LOWER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CU(NC)  VECTOR CONTAINING UPPER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CSL(NC)  VECTOR CONTAINING LOVER SLACKS FOR CONSTRAINTS.
*  RI  CSU(NC)  VECTOR CONTAINING UPPER SLACKS FOR CONSTRAINTS.
*  RI  CZL(NC)  VECTOR CONTAINING LOVER MULTIPLIERS FOR CONSTRAINTS.
*  RI  CZU(NC)  VECTOR CONTAINING UPPER MULTIPLIERS FOR CONSTRAINTS.
*  RI  CG(MC)  ELEMENTS OF THE CONSTRAINT JACOBIAN MATRIX.
*  II  ICG(NC+1)  POSITIONS OF THE FIRST ROWS ELEMENTS IN THE SPARSE
*         JACOBIAN STRUCTURE.
*  II  JCG(MC)  COLUMN INDICES OF ELEMENTS IN THE SPARSE JACOBIAN
*         STRUCTURE.
*  RU  G(NF)  RIGHT HAND SIDE VECTOR.
*  RI  RPF3  BARRIER COEFFICIENT.
*  II  JOB  CHOICE OF THE IP FORMULATION. JOB=1-PRIMAL FORMULATION.
*         JOB=2-PRIMAL-DUAL FORMULATION.
*
*  COMMON DATA :
*  II  KBF  SPECIFICATION OF SIMPLE BOUNDS. KBF=0-NO SIMPLE BOUNDS.
*         KBF=1-ONE SIDED SIMPLE BOUNDS. KBF=2=TWO SIDED SIMPLE BOUNDS.
*  II  KBC  SPECIFICATION OF LINEAR CONSTRAINTS. KBC=0-NO LINEAR
*         CONSTRAINTS. KBC=1-ONE SIDED LINEAR CONSTRAINTS. KBC=2=TWO
*         SIDED LINEAR CONSTRAINTS.
*
      SUBROUTINE UCINAD(NF,NX,NC,MC,X,IX,XL,XU,XSL,XSU,XZL,XZU,CF,IC,
     & CL,CU,CSL,CSU,CZL,CZU,CG,ICG,JCG,G,RPF3,JOB)
      $INCLUDE ('UMCOMM')
      INTEGER NF,NX,NC,MC,IX(NX),IC(NC),ICG(NC+1),JCG(MC),JOB
      $FLOAT X(NF),XL(NX),XU(NX),XSL(NX),XSU(NX),XZL(NX),XZU(NX),
     & CF(NC),CL(NC),CU(NC),CSL(NC),CSU(NC),CZL(NC),CZU(NC),CG(MC),
     & G(NF),RPF3
      INTEGER I,K,KK,KC
      $FLOAT TEMP,TEMP1
      IF (KBF.GT.0) THEN
      DO 10 I=1,NX
      KK=IX(I)
      IF (JOB.EQ.1) THEN
      IF (KK.EQ.1.OR.KK.EQ.3.OR.ABS(KK).EQ.4) THEN
      G(I)=G(I)+RPF3*(TWO+(XL(I)-X(I))/XSL(I))/XSL(I)-XZL(I)
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.4.OR.ABS(KK).EQ.3) THEN
      G(I)=G(I)-RPF3*(TWO+(X(I)-XU(I))/XSU(I))/XSU(I)+XZU(I)
      ENDIF
      ELSE
      IF (KK.EQ.1.OR.KK.EQ.3.OR.ABS(KK).EQ.4) THEN
      G(I)=G(I)+(RPF3+XZL(I)*(XL(I)-X(I)))/XSL(I)
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.4.OR.ABS(KK).EQ.3) THEN
      G(I)=G(I)-(RPF3+XZU(I)*(X(I)-XU(I)))/XSU(I)
      ENDIF
      ENDIF
   10 CONTINUE
      ENDIF
      IF (KBC.GT.0) THEN
      DO 20 KC=1,NC
      KK=IC(KC)
      TEMP=ZERO
      IF (JOB.EQ.1) THEN
      IF (KK.EQ.1.OR.KK.EQ.3.OR.ABS(KK).EQ.4) THEN
      TEMP=RPF3/CSL(KC)**2
      TEMP1= RPF3*(TWO+(CL(KC)-CF(KC))/CSL(KC))/CSL(KC)-CZL(KC)
      IF (TEMP.NE.ZERO) THEN
      K=ICG(KC)
      CALL UXWDIR(ICG(KC+1)-K,NF,JCG(K),TEMP1,CG(K),G,G,1)
      ENDIF
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.4.OR.ABS(KK).EQ.3) THEN
      TEMP=RPF3/CSU(KC)**2
      TEMP1=-RPF3*(TWO+(CF(KC)-CU(KC))/CSU(KC))/CSU(KC)+CZU(KC)
      IF (TEMP.NE.ZERO) THEN
      K=ICG(KC)
      CALL UXWDIR(ICG(KC+1)-K,NF,JCG(K),TEMP1,CG(K),G,G,1)
      ENDIF
      ENDIF
      ELSE
      IF (KK.EQ.1.OR.KK.EQ.3.OR.ABS(KK).EQ.4) THEN
      TEMP=CZL(KC)/CSL(KC)
      TEMP1= (RPF3+CZL(KC)*(CL(KC)-CF(KC)))/CSL(KC)
      IF (TEMP.NE.ZERO) THEN
      K=ICG(KC)
      CALL UXWDIR(ICG(KC+1)-K,NF,JCG(K),TEMP1,CG(K),G,G,1)
      ENDIF
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.4.OR.ABS(KK).EQ.3) THEN
      TEMP=CZU(KC)/CSU(KC)
      TEMP1=-(RPF3+CZU(KC)*(CF(KC)-CU(KC)))/CSU(KC)
      IF (TEMP.NE.ZERO) THEN
      K=ICG(KC)
      CALL UXWDIR(ICG(KC+1)-K,NF,JCG(K),TEMP1,CG(K),G,G,1)
      ENDIF
      ENDIF
      ENDIF
   20 CONTINUE
      ENDIF
      RETURN
      END
