$INTERFACE UD1TL4                                             05/12/01

  $IF(CONSTR='C')
    $ADD(REAL,'\UXUDOT')
    IF (KD.GT.0) P=UXUDOT($N,$GN,$SN,$IX,KBF)
  $ELSE
    $ADD(REAL,'\UXVDOT')
    IF (KD.GT.0) P=UXVDOT($N,$GN,$SN)
  $ENDIF
  CALL UD1TL4($N,$IH($N+1)-1,$GN,$SN,$H,$IH,EPS0,ALF1,ALF2,R,P&
,GNORM,SNORM,RMIN,RMAX,XMAX,XDEL,MES,INITD,INITH)
  $ADD(REAL,'\$GN($NF)\$SN($NF)')
$REM  $ADD(INTEGER,'\NF\$N\MES\INITD\INITH')
$REM  $ADD(REAL,'\EPS0\ALF1\ALF2\R\P\GNORM\SNORM\RMIN\RMAX\XMAX\XDEL')
  $HELP
    CHOICE OF TEST ON DESCENT DIRECTION
     -1 - CHANGE OF DIRECTION
      0 - NO TEST
      1 - UNIFORM CRITERION
  $CHECK(INITD,1,'-1\0\1',4)
  $HELP
    TOLERANCE FOR DESCENT DIRECTION
  $CHECK(EPS0,'1.0$P-6',REAL,4)
  $HELP
    LOWER BOUND FOR STEPSIZE PARAMETER
  $CHECK(ALF1,'1.0$P-10',REAL,4)
  $HELP
    UPPER BOUND FOR STEPSIZE PARAMETER
  $CHECK(ALF2,'1.0$P 10',REAL,4)
  $HELP
    MAXIMUM STEPSIZE
  $CHECK(XMAX,'1.0$P 3',REAL,2)
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UD1TL4                ALL SYSTEMS                05/12/01
C PORTABILITY : ALL SYSTEMS
C 05/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* TEST ON DESCENT DIRECTION AND PREPARATION OF LINE SEARCH.
*
* PARAMETERS :
*  II  N  NUMBER OF VARIABLES.
*  RI  G(N) GRADIENT OF THE OBJECTIVE FUNCTION.
*  RI  S(N) DIRECTION VECTOR.
*  RI  EPS0  TOLERANCE FOR UNIFORM DESCENT DIRECTION.
*  RI  ALF1  LOWER BOUND FOR STEPSIZE PARAMETER.
*  RI  ALF2  UPPER BOUND FOR STEPSIZE PARAMETER.
*  RO  R  STEPSIZE PARAMETER.
*  RO  P  DIRECTIONAL DERIVATE.
*  RI  GNORM  EUCLIDEAN NORM OF THE GRADIENT.
*  RI  SNORM  EUCLIDEAN NORM OF THE DIRECTION VECTOR.
*  RO  RMIN  MINIMUM VALUE OF THE STEPSIZE PARAMETER.
*  RO  RMAX  MAXIMUM VALUE OF THE STEPSIZE PARAMETER.
*  RI  XMAX  MAXIMUM STEPSIZE.
*  RI  XDEL  TRUST REGION RADIUS.
*  II  MES  METHOD SELECTION.
*  II  INITD  CHOICE OF THE RESTART CRITERION. INITD<0-CHANGE OF THE
*         SIGN IF P<0. INITD>0-NO CHANGE IF P<0. ABS(INITD)=0-NO
*         RESTART CRITERION. ABS(INITD)=1-UNIFORM DESCENT CRITERION.
*  II  INITH  CHOICE OF THE INITIAL APPROXIMATION OF THE HESSIAN
*         MATRIX. INITH=1-UNIT INITIAL APPROXIMATION.
*         INITH=2-RANDOM INITIAL APPROXIMATION.
*
* COMMON DATA :
*  IU  ITERD  TERMINATION INDICATOR. ITERD<0-BAD DECOMPOSITION.
*         ITERD=0-DESCENT DIRECTION. ITERD=1-NEWTON LIKE STEP.
*         ITERD=2-INEXACT NEWTON LIKE STEP. ITERD=3-BOUNDARY STEP.
*         ITERD=4-DIRECTION WITH THE NEGATIVE CURVATURE.
*         ITERD=5-MARQUARDT STEP.
*  IU  IREST  RESTART SPECIFICATION. IF IREST>0 THEN RESTART IS
*         PERFORMED.
*  IU  ITERM  TERMINATION INDICATOR. TERMINATION IF ITERM .NE. 0.
*  IU  NRED  ITERATION COUNT IN LINE SEARCH.
*  TO  TDXX  TEXT INFORMATION ON THE DIRECTION VECTOR OBTAINED.
*
* SUBPROGRAMS USED :
*  RF  UXVDOT  DOT PRODUCT OF VECTORS.
*  S   UXVNEG  COPYING OF A VECTOR WITH CHANGE OF THE SIGN.
*  S   UYRES1  DEFINITION OF THE RESTART VARIABLES.
*  S   UOD1D8  INTERMEDIATE PRINT.
*
      SUBROUTINE UD1TL4(N,M,G,S,H,IH,EPS0,ALF1,ALF2,R,P,GNORM,SNORM,
     & RMIN,RMAX,XMAX,XDEL,MES,INITD,INITH)
      $INCLUDE('UMCOMM')
      $INCLUDE('UMCOMD')
      INTEGER N,M,IH(N+1),MES,INITD,INITH
      REAL*8 G(N),S(N),H(M),EPS0,ALF1,ALF2,R,P,GNORM,SNORM,
     & RMIN,RMAX,XMAX,XDEL
      INTEGER I
      $FLOAT SNOR,UXVDOT
      $FLOAT CON,CON1,CON2
      PARAMETER (CON=5.0$P-1,CON1=5.0$P-3,CON2=5.0$P 2)
      SAVE SNOR
      IF(ITERD.LT.0) THEN
        ITERM=ITERD
        RETURN
      ENDIF
      IF (MES.LT.0) THEN
      IF (NIT.LE.LIT+1) THEN
      XDEL=XMAX
      ELSE
      IF (R.LT.CON) THEN
      XDEL=R*SNOR
      ELSE IF (XDEL.LE.SNOR) THEN
      XDEL=TWO*XDEL
      ENDIF
      ENDIF
      IF (SNORM.GT.XDEL) THEN
      CALL UXVSCL(N,XDEL/SNORM,S,S)
      SNORM=XDEL
      ENDIF
      SNOR=SNORM
      ENDIF
      IF (INITD.LT.0.AND.P.GT.ZERO) THEN
C
C     CHANGE OF THE SIGN
C
        CALL UXVNEG(N,S,S)
        P=-P
      ENDIF
C
C     TEST ON DESCENT DIRECTION
C
      IF(SNORM.LE.ZERO) THEN
        CALL UYRES1('ZERO DIR')
      ELSEIF(ABS(INITD).EQ.0.OR.P+EPS0*GNORM*SNORM.LE.ZERO) THEN
        IREST=0
      ELSEIF(IREST.NE.0.OR.INITH.LT.0) THEN
C
C     STEEPEST DESCENT DIRECTION
C
        IREST=0
        DO 1 I=1,N
        S(I)=-G(I)/MIN(MAX(GNORM*ABS(H(IH(I)))/TEN,CON1),CON2)
    1   CONTINUE
        P=UXVDOT(N,G,S)
        SNORM=SQRT(UXVDOT(N,S,S))
        ITERD=0
        TDXX='ST. DES.'
      ELSEIF(ABS(INITD).EQ.1) THEN
C
C     UNIFORM DESCENT CRITERION
C
        CALL UYRES1('UNIFORM ')
      ENDIF
      IF (IREST.EQ.0) THEN
C
C     PREPARATION OF LINE SEARCH
C
        NRED = 0
        RMIN=ALF1*GNORM/SNORM
        RMAX=MIN(ALF2*GNORM/SNORM,XMAX/SNORM)
        RMIN=MIN(RMIN,RMAX/TEN)
      ENDIF
      CALL UOD1D8(N,G,S,GNORM,SNORM,P)
      RETURN
      END
