$INTERFACE UDSLL1                                             98/12/01

  $HELP
    TOLERANCE FOR POSITIVE DEFINITENESS
  $CHECK(ETA2,'ETA0*1.0D-3',REAL,4)
  CALL UDSLL1(NF,NC,MC,MH,MMAX,$CG,$ICG,$JCG,$B,$IB,$JB,$D,$G,$CF,$S&
,$PSL,$PERM,$WN11,$WN12,ETA2,INF)
  $ADD(REAL,'\$CG($MC)\$B($MMAX)\$D($NF)\$G($NF)\$CF($NC)\$S($NF+$NC)')
  $ADD(INTEGER,'\$ICG($NC+1)\$JCG($MC)\$IB($NC+1)\$JB($MMAX)&
  \$PSL($NC+1)\$PERM($NC)\$WN11($NC+1)\$WN12($NC+1)')
  $ADD(INTEGER,'\MH')
  $ADD(INIT)
    NDXX='UDSLL1'
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UDSLL1                ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* DETERMINATION OF THE DIRECTION VECTOR AS A SOLUTION OF THE SYSTEM OF
* NORMAL EQUATIONS USING SPARSE GILL-MURRAY FACTORIZATION WITH THE
* CONTROL OF POSITIVE DEFINITENESS.
*
* PARAMETERS :
*  II  NF NUMBER OF VARIABLES.
*  II  NC NUMBER OF CONSTRAINTS.
*  II  MC  NUMBER OF ELEMENTS IN THE FIELD CG.
*  II  MMAX LENGTH OF THE ARRAYS JB,B.
*  RI  CG(MC)  ELEMENTS OF THE CONSTRAINT JACOBIAN MATRIX.
*  II  ICG(NC+1)  POSITION OF THE FIRST ROWS ELEMENTS IN THE FIELD CG.
*  II  JCG(MC)  COLUMN INDICES OF ELEMENTS IN THE FIELD CG.
*  RU  B(MMAX)  NUMERICAL VALUES OF THE SPARSE MATRIX B.
*         OF THE TRIANGULAR FACTOR.
*  II  IB(NC+1) POINTER VECTOR OF THE SPARSE MATRIX A.
*  II  JB(MMAX) INDICES OF THE TRIANGULAR FACTOR OF THE HESSIAN MATRIX.
*  RI  D(NF)  DIAGONAL MATRIX.
*  RI  G(NF) GRADIENT OF THE OBJECTIVE FUNCTION.
*  RI  CF(NC) CONSTRAINT VECTOR.
*  RO  S(NF+NC) DIRECTION VECTOR.
*  II  PSL(NC+1) POINTER VECTOR OF THE SPARSE TRIANGULAR FACTOR OF THE
*         HESSIAN MATRIX.
*  II  PERM(NC) PERMUTATION VECTOR.
*  IA  WN11(NC+1) AUXILIARY VECTOR.
*  IA  WN12(NC+1) AUXILIARY VECTOR.
*  RI  ETA2  TOLERANCE FOR POSITIVE DEFINITENESS.
*  IU  INF  DECOMPOSITION INDICATOR. INF=0 IF THE MATRIX IS POSITIVE
*         DEFINITE.
*
* COMMON DATA :
*  IO  ITERD  TERMINATION INDICATOR. ITERD<0-BAD DECOMPOSITION.
*         ITERD=0-DESCENT DIRECTION. ITERD=1-NEWTON LIKE STEP.
*         ITERD=2-INEXACT NEWTON LIKE STEP. ITERD=3-BOUNDARY STEP.
*         ITERD=4-DIRECTION WITH THE NEGATIVE CURVATURE.
*         ITERD=5-MARQUARDT STEP.
*  IO  ITERM  TERMINATION INDICATOR. ITERM=0-SUCCESSFUL TERMINATION.
*  IU  IDECC  DECOMPOSITION INDICATOR. IDECC=0-NO DECOMPOSITION.
*         IDECC=1-GILL-MURRAY DECOMPOSITION. IDECC=2-BUNCH-PARLETT
*         DECOMPOSITION. IDECC=3-INVERSION.
*  IU  NDECC  NUMBER OF DECOMPOSITIONS.
*  TO  TDXX  TEXT INFORMATION ON THE DIRECTION VECTOR OBTAINED.
*
* SUBPROGRAMS USED :
*  S   UXSPCF  SPARSE NUMERICAL GILL-MURRAY FACTORIZATION OF A
*          SYMMETRIC MATRIX USING COMPACT FORM FOR FACTORS WITH
*          THE CONTROL OF POSITIVE DEFINITENESS.
*  S   UXSPCB  SPARSE BACK SUBSTITUTION WITH FACTORS IN COMPACT FORM.
*  RF  UXVDOT  DOT PRODUCT OF VECTORS.
*  S   UXVNEG  COPYING OF A VECTOR WITH A CHANGE OF THE SIGN.
*  S   UXVSFP  PERMUTATION OF A VECTOR.
*  S   UXVSBP  INVERSE PERMUTATION OF A VECTOR.
*  S   UOD1D1  PRINT OF ENTRY TO DIRECTION DETERMINATION.
*  S   UOD1D2  PRINT OF EXIT FROM DIRECTION DETERMINATION.
*  S   UOD1D5  PRINT OF INFORMATION DURING DIRECTION DETERMINATION.
*
      SUBROUTINE UDSLL1(NF,NC,MC,MH,MMAX,CG,ICG,JCG,B,IB,JB,D,G,CF,S,
     & PSL,PERM,WN11,WN12,ETA2,INF)
      $INCLUDE('UMCOMM')
      INTEGER NF,NC,MC,MMAX,ICG(NC+1),JCG(MC),IB(NC+1),JB(MMAX),
     & PSL(NC+1),PERM(NC),WN11(NC+1),WN12(NC+1),INF
      INTEGER L,I,MH
      $FLOAT CG(MC),B(MMAX),D(NF),G(NF),CF(NC),S(NF+NC),ETA2
      $FLOAT ALF,TAU
      CALL UOD1D1
      DO 111 I=1,NF
      S(I)=ONE/D(I)
  111 CONTINUE
      CALL UCENS1(NF,NC,MC,MMAX,B,IB,JB,CG,ICG,JCG,S)
      IDECC=0
      L=IB(NC+1)-1
      IF(IDECC.NE.1) THEN
      CALL UXSPCT(NC,L,MH,MMAX,B,JB,PSL,ITERM)
      IF (ITERM.NE.0) THEN
      TDXX='LACK SPC'
      GO TO 3
      ENDIF
C
C     GILL-MURRAY DECOMPOSITION
C
      ALF = ETA2
      CALL UXSPCF(NC,MMAX,B(L+1),PSL,JB(L+1),WN11,WN12,S,INF,ALF,TAU)
      NDECC=NDECC+1
      IDECC=1
      ENDIF
C
C     NEWTON LIKE STEP
C
      CALL UXVMUL(NF,D,G,S,-1)
      CALL UXSRMD(NF,NC,MC,CG,ICG,JCG,S,ONE,CF,S(NF+1))
      CALL UXVNEG(NC,S(NF+1),S(NF+1))
      CALL UXVSFP(NC,PERM,S(NF+1),S)
      CALL UXSPCB(NC,MMAX,B(L+1),PSL,JB(L+1),S(NF+1),0)
      CALL UXVSBP(NC,PERM,S(NF+1),S)
      CALL UXSCMD(NF,NC,MC,CG,ICG,JCG,S(NF+1),ONE,G,S)
      CALL UXVMUL(NF,D,S,S,-1)
      IF(INF.EQ.0) THEN
      ITERD = 1
      TDXX='G-M  POS'
      ELSEIF(INF.LT.0) THEN
      ITERD = 2
      TDXX='G-M  ZER'
      ELSE
      ITERD = 2
      TDXX='G-M  NEG'
      ENDIF
      CALL UOD1D5(ALF,TAU,INF)
    3 CALL UOD1D2(NF,G,S)
      RETURN
      END
