$INTERFACE UF1HS1                                             97/12/01

  $HELP
    PRECISION OF THE COMPUTED GRADIENTS
  $CHECK(ETA1,ETA0,REAL,4)
  CALL UF1HS1(NF,$X,$IX,$XN,$HF,$GF,$GO,FF,$ETA1)
  $ADD(REAL,'\$X($NF)\$XN($NF)\$HF($NF*($NF+1)/2)\$GF($NF)\$GO($NF)')
$REM  $ADD(REAL,'\FF')
  $ADD(INTEGER,'\$IX($NX)')
$REM  $ADD(INTEGER,'\NF')
  $ADD(INIT)
    NFHX='UF1HS1'
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UF1HS1                ALL SYSTEMS                97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* NUMERICAL COMPUTATION OF THE HESSIAN MATRIX OF THE MODEL FUNCTION
* USING ITS GRADIENTS.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XN(NF)  VECTOR OF SCALING FACTORS.
*  RO  HF(NF*(NF+1)/2) HESSIAN MATRIX OF THE MODEL FUNCTION.
*  RI  GF(NF)  GRADIENT OF THE MODEL FUNCTION.
*  RA  GO(NF)  AUXILIARY VECTOR.
*  RI  FF  VALUE OF THE MODEL FUNCTION.
*  RI  ETA1  PRECISION OF THE COMPUTED VALUES.
*
* COMMON DATA :
*  II  KBF  TYPE OF BOUNDS. KBF=0-BOUNDS ARE NOT USED. KBF=1-ONE SIDED
*         BOUNDS. KBF=1-TWO SIDED BOUNDS.
*  II  NORMF  SCALING SPECIFICATION. NORMF=0-NO SCALING IS PERFORMED.
*         NORMF=1-SCALING FACTORS ARE DETERMINED AUTOMATICALLY.
*         NORMF=2-SCALING FACTORS ARE SUPPLIED BY USER.
*
* STATUS VARIABLES :
*  NS,ISP,TSS
*
* SUBPROGRAMS USED :
*  S   UXVCOP  COPYING OF A VECTOR.
*  S   UYPRO1  PROLOGUE.
*  S   UYEPI1  EPILOGUE.
*  S   UYSET0  STATUS DEFINITION.
*
      SUBROUTINE UF1HS1(NF,X,IX,XN,HF,GF,GO,FF,ETA1)
      $INCLUDE('UMCOMM')
      INTEGER NF,IX(NF)
      $FLOAT X(NF),XN(NF),HF(NF*(NF+1)/2),GF(NF),GO(NF),FF,ETA1
      $FLOAT XSTEP,XTEMP,FTEMP,ETA
      INTEGER I,J,IJ,IVAR,IND
      SAVE XSTEP,XTEMP,FTEMP,ETA
      SAVE IVAR
      IND(I,J)=MAX(I,J)*(MAX(I,J)-1)/2+MIN(I,J)
      IF (TSS(NS).NE.'UFHX') CALL UYPRO1('UFHX',-1)
      GO TO (1,3)  ISP(NS)
    1 CONTINUE
      ETA=SQRT(ETA1)
      FTEMP=FF
      CALL UXVCOP(NF,GF,GO)
      IVAR=0
    2 CONTINUE
      IVAR=IVAR+1
      IF (IVAR.GT.NF) GO TO 5
      IF (KBF.GT.0) THEN
      IF (IX(IVAR).LE.-5) GO TO 2
      ENDIF
C
C     STEP SELECTION
C
      IF (NORMF.GT.0) THEN
      XSTEP=XN(IVAR)
      ELSE
      XSTEP=ONE
      ENDIF
      XTEMP=X(IVAR)
      IF (XTEMP.GE.ZERO) THEN
      XSTEP= ETA*MAX(ABS(XTEMP),XSTEP)
      ELSE
      XSTEP=-ETA*MAX(ABS(XTEMP),XSTEP)
      ENDIF
      X(IVAR)=XTEMP+XSTEP
      XSTEP=X(IVAR)-XTEMP
      CALL UYSET0(1,2)
      RETURN
    3 CONTINUE
C
C     NUMERICAL DIFFERENTIATION
C
      DO 4 J=1,NF
      IF (KBF.GT.0) THEN
      IF (IX(J).LE.-5) GO TO 4
      ENDIF
      IJ=IND(IVAR,J)
      IF (J.GE.IVAR) THEN
      HF(IJ)=(GF(J)-GO(J))/XSTEP
      ELSE
      HF(IJ)=HALF*(HF(IJ)+(GF(J)-GO(J))/XSTEP)
      ENDIF
    4 CONTINUE
      X(IVAR)=XTEMP
      GO TO 2
    5 FF=FTEMP
      CALL UXVCOP(NF,GO,GF)
      CALL UYEPI1(-1)
      RETURN
      END
