* SUBROUTINE UKSGLE              ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* FIRST PHASE SIMPLEX EPILOGUE.
*
* PARAMETERS :
*  II  NF INITIAL NUMBER OF VARIABLES.
*  II  N NUMBER OF COLUMNS.
*  II  M  NUMBER OF ROWS (IN FACTORIZATION AND UPDATE PROCEDURES
*         WE USE ANOTHER DESCRIPTION OF VARIABLES N AND M).
*  II  MROWS DECLARED NUMBER OF ROWS.
*  II  MCOLS DECLARED NUMBER OF COLUMNS.
*  II  NZ  SIZE OF VECTORS CNR/INR.
*  II  NZA  SIZE OF ARRAYS A/RNR.
*  II  NAU SIZE OF VECTORS AI/AU.
*  II  NUP SIZE OF ARRAYS LI,LJ,CM.
*  IU  NEL NUMBER OF ELEMENTARY LOWER TRIANGULAR MATRICES USED IN
*          CURRENT FACTORIZATION.
*  IO  IBEG  BEGINNING OF THE ROW SCHEME OF THE FACTOR U IN CNR.
*  IO  IBGR  BEGINNING OF THE COLUMN SCHEME IN CNR.
*  IO  IENDLU  END OF THE ROW INDICES OF THE FACTOR U IN CNR.
*  IU  PTR(N) POINTER VECTOR OF THE ROWS OF THE FACTOR U.
*  IU  LENR(N) LENGTHS OF THE ROWS OF THE FACTOR U.
*  IU  CNR(NZ) CONTAINS THE INDICES OF THE FACTORS.
*  IU  PRNR(N+1) POINTER VECTOR FOR RNR.
*  IU  RNR(NZA) COLUMNS OF THE MATRIX OF CONSTRAINTS.
*  IU  ORDR(N) ROW PERMUTATION.
*  II  IDEB  IDEB=1 FOR LUSOL UPDATE - OTHERWISE USES PERMUTATION
*            OF THE BUMP.
*  IU  ORDC(N) COLUMN PERMUTATION.
*  II  BCOL(N) COLUMN ORDER IN BLOCKS.
*  II  PTL(N) POINTER VECTOR OF THE COLUMNS OF THE L-FACTOR.
*  II  LENL(N) LENGTHS OF THE COLUMNS OF THE FACTOR L.
*  IA  LENC(N) LENGTHS OF THE COLUMNS OF U IF ( ONLY IF IFL.LT.0).
*  IA  PTC(N) POINTERS FOR THE COLUMN U.
*  IU  ORDL(N) ROW PERMUTATION OF THE ORIGINAL FACTOR L.
*  RA  WR01(N) AUXILIARY VECTOR
*  RA  WR01(N) AUXILIARY VECTOR
*  IU  LI(NUP) ROW INDICES OF THE L-UPDATES.
*  IU  LJ(NUP) COLUMN INDICES OF THE L-UPDATES.
*  RI  CM(NUP) UPDATE MULTIPLIERS.
*  IU  RIGHT(N) RIGHTMOST POSITION OF THE ROW SPIKE FOR BASIS.
*  IU  UP(N) UPPERMOST POSITION OF THE COLUMN SPIKE FOR BASIS.
*  RU  A(NZA) NUMERICAL VALUES OF THE MATRIX OF CONSTRAINTS.
*  RU  AU(NAU) NUMERICAL VALUES OF THE FACTORS.
*  RI  STFL  STABILITY FLAG.
*  RU  MMFL  MAXIMUM MULTIPLIER FOUND.
*  RO  MXU   MAXIMUM ELEMENT OF THE U-DIAGONAL.
*  RO  MNU   MINIMUM ELEMENT OF THE U-DIAGONAL.
*  IA  WN01(N) AUXILIARY VECTOR.
*  RA  FCIN(N) AUXILIARY VECTOR.
*  RI  X(N) VECTOR OF SOLUTION.
*  RI  C(N) COST VECTOR.
*  IU  TYPE(MCOLS) TYPES OF THE VARIABLES:
*                                    =1 FOR NONBASIC LOWER BOUNDED
*                                    =2 FOR NONBASIC UPPER BOUNDED
*                                    =3 FOR NONBASIC LOWER AND UPPER
*                     BOUNDED VARIABLE SET TO ITS LOWER BOUND
*                                    =4 FOR NONBASIC LOWER AND UPPER
*                     BOUNDED VARIABLE SET TO ITS UPPER BOUND
*                                    =5 FOR NONBASIC FIXED VARIABLE
*                                    =6 FOR NONBASIC FREE VARIABLE
*                                    =-1 FOR BASIC LOWER BOUNDED
*                                    =-2 FOR BASIC UPPER BOUNDED
*                                    =-3 FOR BASIC LOWER AND UPPER
*                     BOUNDED VARIABLE
*                                    =-6 FOR BASIC FREE VARIABLE.
*  RO  EPS7  BOUND FOR NEGLECTING SMALL SPARSE MATRIX ELEMENTS.
*  RO  EPS9  BOUND FOR NEGLECTING SMALL TOLERANCES.
*  RO  DMAX  MINIMUM ELEMENT OF THE U-DIAGONAL (EARLIER MNU).
*  RO  GMAX  ELEMENT WITH THE MAXIMUM ABSOLUTE VALUE OF THE FACTOR U.
*  RO  UMAX  MAXIMUM ELEMENT OF THE U-DIAGONAL (EARLIER MXU).
*  IO  NREF  NUMBER OF REFACTORIZATIONS.
*  II  INEW INPUT COLUMN NUMBER
*  II  IOLD POSITION OF THE OUTPUT COLUMN IN BCOL.
*  IO  KOLD  NUMBER OF UPDATES AFTER LATEST REFACTORIZATION.
*  IO  KINP NUMBER OF ARTIFICIALS.
*  IO  MINP NUMBER OF SLACKS.
*  IA  ITERL INTERNAL TERMINATION INDICATOR:
*                              =11 IF NAU IS DECLARED TOO SMALL
*                              =12 IF MATRIX IS SINGULAR
*                              =13 IF FACTORS ARE BADLY CONDITIONED.
*                              =14 LITTLE SPACE FOR L-UPDATES.
*                              =15 MATRIX SINGULARITY IS FACED.
*                              =16 NO FEASIBLE SOLUTION EXISTS.
*  IO  ITERL TYPE OF FEASIBLE POINT. ITERL=1-ARBITRARY FEASIBLE POINT-
*         PHASE 1 OUTPUT, ITERL=-1 FEASIBLE POINT DOES
*         NOT EXIST.
*         ITERL=-3 - SMALL NAU, ITERL=-4 - NZ IS SMALL, ITERL=-5 - BADLY
*         CONDITIONED PROBLEM, ITERL=-6 -  SMALL NUP
*
      SUBROUTINE UKSGLE(NF,N,M,MROWS,MCOLS,NZ,NZA,NAU,NUP,NEL,IBEG,IBGR,
     & IENDLU,PTR,LENR,CNR,PRNR,RNR,ORDR,IDEB,ORDC,BCOL,PTL,LENL,LENC,
     & PTC,ORDL,WR01,WR02,LI,LJ,CM,RIGHT,UP,A,AU,STFL,MMFL,MXU,MNU,WN01,
     & FCIN,X,C,TYPE,EPS7,EPS9,DMAX,GMAX,UMAX,NREF,INEW,IOLD,KOLD,KINP,
     & MINP,ITERL)
      INTEGER NF,N,M,MROWS,MCOLS,NZ,NZA,NAU,NUP,NEL,IBEG,IBGR,IENDLU
      INTEGER RIGHT(MROWS),PTR(MROWS),LENR(MROWS),ORDR(MROWS),IDEB
      INTEGER PRNR(MCOLS+1),NREF,INEW,IOLD,KOLD,KINP,MINP,ITERL
      INTEGER ORDC(MROWS),BCOL(MCOLS),PTL(MROWS)
      INTEGER LENL(MROWS),ORDL(MROWS),LENC(MROWS),PTC(MROWS)
      INTEGER CNR(NZ),RNR(NZA),LI(NUP),LJ(NUP),WN01(MCOLS)
      INTEGER TYPE(MCOLS),UP(MROWS)
      $FLOAT WR01(MCOLS),WR02(MCOLS),FCIN(MROWS),C(MCOLS),EPS7,EPS9,
     & DMAX,GMAX,UMAX
      $FLOAT CM(NUP),A(NZA),AU(NAU),X(MCOLS)
      INTEGER COL,IDIF
      $FLOAT SUM,BETA,ALFA,BETA1,HUNDRE
      $FLOAT STFL,MMFL,MXU,MNU,GRFL,GSFL,DRFL
      INTEGER I,J,K,L,L1,L2,IND,JOB,JOPT,JSTRT,JSTOP,NROLD
      INTEGER LACK,P12
      PARAMETER(HUNDRE=100.0$P 0)
      PARAMETER(GRFL=1.0$P 1,GSFL=1.0$P 16)
      BETA=0.0$P 0
      LACK=0
      P12=0
      NROLD=NREF
      KOLD=0
C
C     COST FUNCTION ENUMERATION
C
      SUM=0.0$P 0
      DO 100 I=1,N
      SUM=SUM+ABS(C(I)*X(I))
100   CONTINUE
C
C     IF SUM.GT.TOLERANCE THEN NO FEASIBLE SOLUTION EXISTS
C
      IF(SUM.GT.EPS9) THEN
                      CALL UOERR1('UKSGLE',11)
                      ITERL=-1
                      GO TO 900
      END IF
C
C     IF KINP.EQ.0 THEN WE CAN PROCEED TO THE SECOND PHASE
C
      IF(KINP.EQ.0) THEN
                    ITERL=1
                    N=NF+MINP
                    GO TO 900
      END IF
C
C     ROWS OF ARTIFICIAL VARIABLES ARE INSERTED INTO THE VECTOR
C     WN01 AT THE POSITIONS 1,...,IND.
C
      IND=0
      DO 200 I=M-KINP+1,M
      IND=IND+1
      WN01(IND)=I
200   CONTINUE
C
C     AT THE FOLLOWING LINES WE FILL THE ARRAY FCIN. THEN WE
C     COMPUTE ALFA=B**-1*FCIN, COMPARE WITH ZERO AND TRY TO
C     EXCHANGE THE COLUMNS.
C
C
C     I IS NUMBER OF THE COLUMN
C
C
C     TYPE IS 1,2,3 OR 4
C
      I=1
250   CONTINUE
      IF(NREF.GT.NROLD) THEN
        NROLD=NREF
        CALL UXSGFD(M,M,NZ,NZA,NAU,NEL,P12,N,PTL,LENL,PTC,LENC,PTR,
     *              LENR,CNR,ORDC,ORDR,ORDL,RNR,PRNR,BCOL,RIGHT,WR01,
     *              A,AU,STFL,GMAX,UMAX,DMAX,IBEG,IBGR,IENDLU,LACK,
     *              GSFL,DRFL,ITERL,0)
        IF (ITERL.NE.0) THEN
          CALL UOERR1('UXSGLE',ITERL+20)
          IF(ITERL.EQ.1.OR.ITERL.EQ.6) THEN
            ITERL=-3
          ELSEIF(ITERL.EQ.2.OR.ITERL.EQ.7) THEN
            ITERL=-4
          ELSE
            ITERL=-5
          ENDIF
          GO TO 900
        END IF
        DO 150 I=1,M
          UP(I)=WR01(I)
150     CONTINUE
      ENDIF
        IF(TYPE(I).GE.1) THEN
          COL=I
          CALL UKSGLK(M,MROWS,MCOLS,NZA,COL,RNR,PRNR,A,FCIN)
C
C     FTRAN
C
          JOB=0
          CALL UXSGFB(M,NZ,NAU,NEL,NUP,PTL,LENL,CNR,PTR,LENR,ORDC,
     *                ORDR,ORDL,LI,LJ,CM,AU,FCIN,WR02,JOB)
C
C     BETA IS MAXIMUM ELEMENT OF THE ROW CORRESPONDING TO SOME
C     ARTIFICIAL VARIABLE. ITS ROW NUMBER IS IN WN01(JOPT).
C
          DO 300 J=1,IND
          K=WN01(J)
          BETA1=BETA
          ALFA=ABS(FCIN(K))
          IF(BETA.LT.ALFA) THEN
                           BETA=ALFA
                           JOPT=J
          END IF
300       CONTINUE
          IF(BETA.GT.EPS9) THEN
C
C      COLUMN EXCHANGE
C
              IOLD=WN01(JOPT)
              INEW=I
         CALL UXSGUM(M,M,NZ,NZA,NAU,NEL,NUP,PTR,LENR,CNR,PRNR,RNR,
     *               ORDR,IDEB,ORDC,BCOL,PTL,LENL,ORDL,WR01,WR02,LI,LJ,
     *               CM,RIGHT,UP,A,AU,STFL,MMFL,EPS7,MXU,MNU,INEW,
     *               IOLD,IBGR,IENDLU,IBEG,ITERL)
         KOLD=KOLD+1
         IF(ITERL.GT.10) CALL UOERR1('UKSGLE',ITERL+20)
         IF(ITERL.EQ.11) THEN
           ITERL=-3
           GO TO 900
         ELSEIF(ITERL.GT.0) THEN
           NREF=NREF+1
         IF(KOLD.EQ.0) THEN
           IF(ITERL.EQ.14) THEN
             ITERL=-6
           ELSE
             ITERL=-5
           ENDIF
           GO TO 900
          END IF
          KOLD=0
          BETA=BETA1
          GO TO 250
        END IF
        IF(KOLD.GE.HUNDRE) THEN
          NREF=NREF+1
          KOLD=0
          BETA=BETA1
        END IF
C
C     SHRINKING OF THE ARRAY WN01 AND TEST WHETHER IT HAS
C     LENGTH GREATER OR EQUAL TO ONE. IN THE OPPOSITE CASE
C     RETURN.
              KINP=KINP-1
C
C     SHRINK WN01 - ARRAY OF THE ROW NUMBERS CORRESPONDING TO
C     ARTIFICIAL VARIABLES.
C
              WN01(JOPT)=WN01(IND)
              IND=IND-1
              IF(IND.EQ.0) THEN
                           ITERL=1
                           N=NF+MINP
                           GO TO 900
              END IF
      END IF
      END IF
      I=I+1
      IF(IND.EQ.0) GO TO 900
      IF((NF+MINP).GE.I) GO TO 250
C
C     ROWS CORRESPONDING TO THE ARTIFICIAL VARIABLES
C     ARE LINEARLY DEPENDENT ON THE OTHERS
C     WE MUST THEM EXCLUDE FROM THE PRNR-RNR STRUCTURE.
C
      IF(IND.GT.0) CALL UOKLP5(MCOLS,WN01,IND)
C
C     FUNCTION: WN01 IS ZEROED FOR THE INDICES THAT ARE
C     TO BE OMITTED. OTHER INDICES ARE SHIFTED DOWN.
C
      IDIF=IND
      L2=M
      DO 600 I=1,L2
      WR01(I)=I
600   CONTINUE
      DO 500 I=IND,1,-1
      L1=RNR(PRNR(BCOL(WN01(I))))
          DO 450 K=L1+1,L2
          WR01(K)=WR01(K)-IDIF
450       CONTINUE
      WR01(L1)=0
      IDIF=IDIF-1
      L2=L1-1
500   CONTINUE
C
C     REMOVING THE INDICES CORRESPONDING TO LINEARLY
C     DEPENDENT ROWS.
C
      IDIF=0
      DO 800 I=1,NF+MINP
      JSTRT=PRNR(I)
      JSTOP=JSTRT-IDIF
          DO 700 J=JSTRT,PRNR(I+1)-1
          K=RNR(J)
          L=WR01(K)
          IF(L.EQ.0)THEN
                     IDIF=IDIF+1
                     ELSE
                     RNR(J-IDIF)=L
                     A(J-IDIF)=A(J)
          END IF
700       CONTINUE
          PRNR(I)=JSTOP
800   CONTINUE
      PRNR(I)=PRNR(I)-IDIF
C
C     NEW NUMBER OF ROWS
C
      M=M-IND
C
C NEW NUMBER OF VARIABLES
C
      N=NF+MINP
C
      ITERL=1
900   CONTINUE
      CALL UOKLP4(N,M)
      RETURN
      END
