$INTERFACE ULLPF2                                             97/12/01

    $PAR=S
    $IF(JOB>0) $PAR=G
    $HELP
      TYPE OF EXTREMUM:
         0 - CHEBYSHEV APPROXIMATION
        -1 - MINIMAX
         1 - MAXIMIN
    $CHECK(IEXT,0,'-1\0\1',1)
    $HELP
      PRECISION FOR EITHER LINEAR OR QUADRATIC PROGRAMMING
      SUBROUTINE
    $CHECK(EPS7,'1.0$P-8',REAL,4)
    $HELP
      TOLERANCE FOR ACTIVE CONSTRAINTS
    $CHECK(EPS9,'1.0$P-8',REAL,4)
    CALL ULLPF2(NF,N,MAL,MCL,$X,$IX,$IXA,$XN,$XO,$XL,$XU,$AF,$AFD,$IA&
,$IAA,$AG,$AR,$AZ,$CF,$CFD,$IC,$CL,$CU,$CG,$PAR,$S,$ETA9,EPS7,EPS9&
,XDEL,XNORM,UMAX,GMAX,$JOB)
    $ADD(REAL,'\$X($NF)\$XN($NF)\$XO($NF)\$XL($NF)\$XU($NF)&
    \$AG($NF*($MAL))\$AR(($NF+1)*($NF+2)/2)\$AZ($NF+1)\$CF($MCL)&
    \$CFD($MCL)\$CL($MCL)\$CU($MCL)\$CG($NF*$MCL)\$PAR($NF+1)&
    \$S($NF+1)')
$REM    $ADD(REAL,'\XDEL\XNORM\UMAX\GMAX\EPS7\EPS9')
$REM    $ADD(INTEGER,'\NF\N\MAL\MCL')
    $ADD(INTEGER,'\$IX($NX)\$IXA($NX)\$IAA($NF+1)\$IC($MCL)')
    $IF(IEXT=0)
      $ADD(REAL,'\$AF(2*$MAL)\$AFD(2*$MAL)')
      $ADD(INTEGER,'\$IA(2*$MAL)')
    $ELSE
      $ADD(REAL,'\$AF($MAL)\$AFD($MAL)')
      $ADD(INTEGER,'\$IA($MAL)')
    $ENDIF
    $ADD(INIT)
      NQPX='ULLPF2'
    $ENDADD
    $EXIT

* IMPLEMENTATION

* SUBROUTINE ULLPF2             ALL SYSTEMS                   97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* DETERMINATION OF THE INITIAL FEASIBLE POINT AND THE LINEAR PROGRAMMING
* SUBROUTINE.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  IO  N  DIMENSION OF THE MANIFOLD DEFINED BY ACTIVE CONSTRAINTS.
*  II  NA  NUMBER OF LINEAR APPROXIMATED FUNCTIONS.
*  II  NC  NUMBER OF LINEAR CONSTRAINTS.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  II  IXA(NF)  VECTOR CONTAINING INFORMATION ON TRUST REGION ACTIVITY.
*  RI  XN(NF)  VECTOR OF SCALING FACTORS.
*  RO  XO(NF)  SAVED VECTOR OF VARIABLES.
*  RI  XL(NF)  VECTOR CONTAINING LOWER BOUNDS FOR VARIABLES.
*  RI  XU(NF)  VECTOR CONTAINING UPPER BOUNDS FOR VARIABLES.
*  RI  AF(2*NA)  VECTOR CONTAINING VALUES OF THE APPROXIMATED
*         FUNCTIONS.
*  RO  AFD(2*NA)  VECTOR CONTAINING INCREMENTS OF THE APPROXIMATED
*         FUNCTIONS.
*  II  IA(2*NA)  VECTOR CONTAINING TYPES OF DEVIATIONS.
*  II  IAA(NF+1)  VECTOR CONTAINING INDICES OF ACTIVE FUNCTIONS.
*  RI  AG(NF*NA)  MATRIX WHOSE COLUMNS ARE NORMALS OF THE LINEAR
*         APPROXIMATED FUNCTIONS.
*  RI  AR((NF+1)*(NF+2)/2)  TRIANGULAR DECOMPOSITION OF KERNEL OF THE
*         ORTHOGONAL PROJECTION.
*  RO  AZ(NF+1)  VECTOR OF LAGRANGE MULTIPLIERS.
*  RI  CF(NF)  VECTOR CONTAINING VALUES OF THE CONSTRAINT FUNCTIONS.
*  RO  CFD(NF)  VECTOR CONTAINING INCREMENTS OF THE CONSTRAINT
*         FUNCTIONS.
*  II  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  RI  CL(NC)  VECTOR CONTAINING LOWER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CU(NC)  VECTOR CONTAINING UPPER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CG(NF*NC)  MATRIX WHOSE COLUMNS ARE NORMALS OF THE LINEAR
*         CONSTRAINTS.
*  RO  G(NF+1)  GRADIENT OF THE LAGRANGIAN FUNCTION.
*  RI  S(NF+1)  DIRECTION VECTOR.
*  RI  ETA9  MAXIMUM FOR REAL NUMBERS.
*  RI  EPS7  TOLERANCE FOR LINEAR INDEPENDENCE OF CONSTRAINTS.
*  RI  EPS9  TOLERANCE FOR ACTIVITY OF CONSTRAINTS.
*  RU  XDEL  TRUST REGION BOUND.
*  RO  XNORM  VALUE OF LINEARIZED MINIMAX FUNCTION.
*  RO  UMAX  MAXIMUM ABSOLUTE VALUE OF A NEGATIVE LAGRANGE MULTIPLIER.
*  RO  GMAX  MAXIMUM ABSOLUTE VALUE OF A PARTIAL DERIVATIVE.
*  II  JOB  SPECIFICATION OF THE PROBLEM. JOB=0-QUADRATIC PROGRAMMING
*         PROBLEM. JOB=1-QUADRATIC PROGRAMMING SUBPROBLEM FOR DIRECTION
*         DETERMINATION.
*
* COMMON DATA :
*  II  LCS  VARIABLE WHICH SPECIFIES TRUST REGION BOUNDS.
*  II  KBF  SPECIFICATION OF SIMPLE BOUNDS. KBF=0-NO SIMPLE BOUNDS.
*         KBF=1-ONE SIDED SIMPLE BOUNDS. KBF=2=TWO SIDED SIMPLE BOUNDS.
*  II  KBC  SPECIFICATION OF LINEAR CONSTRAINTS. KBC=0-NO LINEAR
*         CONSTRAINTS. KBC=1-ONE SIDED LINEAR CONSTRAINTS. KBC=2=TWO
*         SIDED LINEAR CONSTRAINTS.
*  II  NORMF  SCALING SPECIFICATION. NORMF=0-NO SCALING PERFORMED.
*         NORMF=1-SCALING FACTORS ARE DETERMINED AUTOMATICALLY.
*         NORMF=2-SCALING FACTORS ARE SUPPLIED BY USER.
*  IO  ITERL  TYPE OF FEASIBLE POINT. ITERQ=1-ARBITRARY FEASIBLE POINT.
*         ITERQ=2-OPTIMUM FEASIBLE POINT. ITERQ=-1 FEASIBLE POINT DOES
*         NOT EXISTS. ITERQ=-2 OPTIMUM FEASIBLE POINT DOES NOT EXISTS.
*  II  IEXT  TYPE OF EXTREMUM. IEXT<0-MINIMAX. IEXT=0-CHEBYSHEV
*         APPROXIMATION, IEXT>0-MAXIMIN.
*
* SUBPROGRAMS USED :
*  S   ULINIT  DETERMINATION OF INITIAL POINT SATISFYING SIMPLE BOUNDS.
*  S   ULAMAX  DETERMINATION OF THE MAXIMUM DEVIATION.
*  S   ULMAXA  MAXIMUM STEPSIZE USING LINEAR APPROXIMATED FUNCTIONS.
*  S   ULMAXL  MAXIMUM STEPSIZE USING LINEAR CONSTRAINTS.
*  S   ULMAXS  MAXIMUM STEPSIZE USING SIMPLE BOUNDS.
*  S   ULMAXT  MAXIMUM STEPSIZE USING TRUST REGION BOUNDS.
*  S   ULNEWA  IDENTIFICATION OF ACTIVE LINEAR APPROXIMATED FUNCTIONS.
*  S   ULNEWL  IDENTIFICATION OF ACTIVE LINEAR CONSTRAINTS.
*  S   ULNEWS  IDENTIFICATION OF ACTIVE SIMPLE BOUNDS.
*  S   ULNEWT  IDENTIFICATION OF ACTIVE TRUST REGION BOUNDS.
*  S   ULDIRA  NEW VALUES OF APPROXIMATED FUNCTIONS.
*  S   ULDIRL  NEW VALUES OF CONSTRAINT FUNCTIONS.
*  S   ULDIRS  NEW VALUES OF VARIABLES.
*  S   ULSETA  INITIAL VALUES OF APPROXIMATED FUNCTIONS.
*  S   ULSETC  INITIAL VALUES OF CONSTRAINT FUNCTIONS.
*  S   ULSETG  DETERMINATION OF THE FIRST PHASE GRADIENT VECTOR.
*  S   ULCLAG  VECTOR OF LAGRANGE MULTIPLIERS IS DETERMINED.
*  S   ULSLAG  NEGATIVE PROJECTED GRADIENT IS DETERMINED.
*  S   ULTLAG  THE OPTIMUM LAGRANGE MULTIPLIER IS DETERMINED.
*  S   ULVLAG  AN AUXILIARY VECTOR IS DETERMINED.
*  S   ULADF1  CONSTRAINT ADDITION.
*  S   ULRMF0  CONSTRAINT DELETION.
*  S   UXDPRB  BACK SUBSTITUTION AFTER CHOLESKI DECOMPOSITION.
*  S   UXDSMI  DETERMINATION OF THE INITIAL UNIT DENSE SYMMETRIC
*         MATRIX.
*  S   UXVCOP  COPYING OF A VECTOR.
*  S   UXVDIF  DIFFERENCE OF TWO VECTORS.
*  S   UXVINA  ABSOLUTE VALUES OF ELEMENTS OF AN INTEGER VECTOR.
*  S   UXVINC  UPDATE OF AN INTEGER VECTOR.
*  S   UXVIND  CHANGE OF THE INTEGER VECTOR FOR CONSTRAINT ADDITION.
*  S   UXVINT  CHANGE OF THE INTEGER VECTOR FOR TRUST REGION BOUND
*         ADDITION.
*  S   UXVMUL  DIAGONAL PREMULTIPLICATION OF A VECTOR.
*  S   UXVNEG  COPYING OF A VECTOR WITH CHANGE OF THE SIGN.
*  S   UXVSET  INITIATION OF A VECTOR.
*  S   UOLLP1  PRINT OF ENTRY TO QUADRATIC PROGRAMMING.
*  S   UOLLP2  PRINT OF EXIT FROM QUADRATIC PROGRAMMING.
*  S   UOLQP5  PRINT OF INFORMATION DURING QUADRATIC PROGRAMMING.
*  S   UOERR1  ERROR MESSAGES.
*
      SUBROUTINE ULLPF2(NF,N,NA,NC,X,IX,IXA,XN,XO,XL,XU,AF,AFD,IA,IAA,
     & AG,AR,AZ,CF,CFD,IC,CL,CU,CG,G,S,ETA9,EPS7,EPS9,XDEL,XNORM,UMAX,
     & GMAX,JOB)
      $INCLUDE ('UMCOMM')
      INTEGER NF,N,NA,NC,IX(NF),IXA(NF),IA(2*NA),IAA(NF+1),IC(NC),JOB
      $FLOAT X(NF),XN(NF),XO(NF),XL(NF),XU(NF),AF(2*NA),AFD(2*NA),
     & AG(NF*NA),AR((NF+1)*(NF+2)/2),AZ(NF+1),CF(NC),CFD(NC),CL(NC),
     & CU(NC),CG(NF*NC),G(NF+1),S(NF+1),ETA9,EPS7,EPS9,XDEL,XNORM,UMAX,
     & GMAX
      $FLOAT POM,CON,E,T,DMAX,SNORM
      INTEGER NAD,NAR,IPOM,I,IOLD,JOLD,KOLD,INEW,JNEW,KNEW,MODE,IER,KA,
     & KREM,KC
      CALL UOLLP1('ULLPF2')
      N=NF
      T=ONE
      CON=ETA9
      IF (IEXT.EQ.0) THEN
      NAD=2*NA
      ELSE
      NAD=NA
      ENDIF
C
C     INITIATION
C
      CALL UXVCOP(NF,X,XO)
      IPOM=0
      JOLD=0
      JNEW=0
      NRED=0
      KREM=0
      ITERL=0
      DMAX=ZERO
      IF (LCS.GT.0) CALL UXVINC(NF,IX,IXA)
      IF (KBF.GT.0) THEN
C
C     SHIFT OF VARIABLES FOR SATISFYING SIMPLE BOUNDS
C
      CALL UXVINA(NF,IX)
      CALL ULINIT(NF,X,IX,XN,XO,XL,XU,XDEL,EPS9,INEW,ITERL)
      IF (ITERL.LT.0) THEN
      CALL UOERR1('ULLPF2',10-ITERL)
      GO TO 6
      ENDIF
      DO 1 I=1,NF
      IF (IX(I).LT.0) THEN
      N=N-1
      IAA(NF-N)=-I
      ENDIF
    1 CONTINUE
      CALL UXDSMI(NF-N,AR)
      ENDIF
      CALL UXVINA(NAD,IA)
      CALL UXVCOP(NA,AF,AF(NA+1))
      IF (NC.GT.0) THEN
C
C     ADDITION OF ACTIVE CONSTRAINTS AND INITIAL CHECK OF FEASIBILITY
C
      CALL UXVINA(NC,IC)
      IF (NF.GT.N) CALL ULSETC(NF,NC,X,XN,XO,CF,IC,CG,S,ISNC,NORMF)
      DO 2 KC=1,NC
      IF (IC(KC).NE.0) THEN
      INEW=0
      CALL ULNEWL(KC,KC,CF,IC,CL,CU,EPS9,INEW)
      CALL ULADF0(NF,N,NA,NC,NC,IA,IAA,AG,AR,CG,S,EPS7,GMAX,UMAX,E,T,
     & INEW,KNEW,NADD,IER)
      CALL UXVIND(NC,IC,KC,IER)
      IF (IC(KC).LT.-10) IPOM=1
      ENDIF
    2 CONTINUE
      IF (IPOM.EQ.1) NAR=NF*(NF+1)/2
      ENDIF
    3 IF (IPOM.EQ.1) THEN
C
C     CHECK OF FEASIBILITY AND UPDATE OF THE FIRST PHASE OBJECTIVE
C     FUNCTION
C
      CALL ULSETG(NF,NC,IC,CG,AR(NAR+1),INEW)
      IF (INEW.EQ.0) IPOM=0
      ENDIF
      IF (IPOM.EQ.0.AND.ITERL.EQ.0) THEN
C
C     FEASIBILITY ACHIEVED
C
      ITERL=1
      IF (NF.GT.N) CALL ULSETA(NF,NA,X,XN,XO,AF,IA,AG,S,NORMF)
C
C     INITIAL MINIMAX VARIABLE
C
      CALL ULAMAX(NA,NAD,AF,IA,XNORM,ETA9)
C
C     ADDITION OF ACTIVE FUNCTIONS
C
      DO 11 KA=1,NAD
      IF (IA(KA).GT.0) THEN
      INEW=0
      CALL ULNEWA(NA,NC,KA,AF,IA,EPS9,XNORM,INEW,KNEW)
      CALL ULADF0(NF,N,NA,NC,NC,IA,IAA,AG,AR,CG,S,EPS7,GMAX,UMAX,E,T,
     & INEW,KNEW,NADD,IER)
      CALL UXVIND(NAD,IA,KA,IER)
      ENDIF
   11 CONTINUE
      ENDIF
    5 IF (IPOM.EQ.0) THEN
      IF (NF.LE.N) THEN
C
C     IMPOSSIBLE SITUATION
C
      ITERL=-5
      CALL UOERR1('ULLPF2',10-ITERL)
      GO TO 6
      ENDIF
C
C     LAGRANGE MULTIPLIERS FOR THE SECOND PHASE
C
      CALL ULCLAG(NF,N,NA,NC,IA,IAA,AR,AZ,SNORM,KOLD)
      IF (KOLD.EQ.0) THEN
C
C     IMPOSSIBLE SITUATION
C
      ITERL=-6
      CALL UOERR1('ULLPF2',10-ITERL)
      GO TO 6
      ENDIF
      CALL ULTLAG(NF,N,NA,NC,NC,IX,IA,IAA,AZ,IC,EPS7,UMAX,IOLD)
C
C     PROJECTED GRADIENT FOR THE SECOND PHASE
C
      CALL UXVSET(NF,ZERO,S)
      CALL ULSLAG(NF,N,NA,NC,NC,IAA,AG,AZ,CG,S,EPS7,GMAX)
      ELSE
C
C     LAGRANGE MULTIPLIERS FOR THE FIRST PHASE
C
      IF (NF.GT.N) THEN
      CALL ULVLAG(NF,N,NA,NC,NC,IAA,AG,CG,AR(NAR+1),AZ)
      CALL UXDPRB(NF-N,AR,AZ,0)
      CALL ULTLAG(NF,N,NA,NC,NC,IX,IA,IAA,AZ,IC,EPS7,UMAX,IOLD)
      ELSE
      IOLD=0
      UMAX=ZERO
      ENDIF
C
C     PROJECTED GRADIENT FOR THE FIRST PHASE
C
      IF (N.GT.0) THEN
      CALL UXVNEG(NF,AR(NAR+1),S)
      CALL ULSLAG(NF,N,NA,NC,NC,IAA,AG,AZ,CG,S,EPS7,GMAX)
      ELSE
      CALL UXVSET(NF,ZERO,S)
      GMAX=ZERO
      ENDIF
      ENDIF
      MODE=1-IPOM
      CALL UOLQP5(NF,N,NC,NA,X,IX,IC,IA,IAA,AF,AR,AZ,DMAX,GMAX,UMAX,
     & XNORM,JOLD,JNEW,MODE)
      INEW=0
      IF (GMAX.EQ.ZERO) THEN
C
C     OPTIMUM ON A LINEAR MANIFOLD OBTAINED
C
      IF (IOLD.EQ.0) THEN
      IF (IPOM.EQ.0) THEN
C
C     OPTIMAL SOLUTION ACHIEVED
C
      IF (JOB.GT.0) THEN
      CALL UXVDIF(NF,X,XO,S)
      CALL UXVCOP(NF,XO,X)
      CALL UXVSET(NF,ZERO,G)
      ENDIF
      ITERL= 2
      GO TO 6
      ELSE
C
C     FEASIBLE SOLUTION DOES NOT EXIST
C
      ITERL=-1
      CALL UOERR1('ULLPF2',10-ITERL)
      GO TO 6
      ENDIF
      ELSE
C
C     CONSTRAINT DELETION
C
      CALL ULRMF0(NF,N,NA,NC,NC,IX,IXA,IA,IAA,AR,IC,S,LCS,IOLD,KREM,
     & NREM,IER)
      DMAX=ZERO
      JOLD=IOLD
      JNEW=0
      GO TO 5
      ENDIF
      ELSE
C
C     STEPSIZE SELECTION
C
      POM=CON
      IF (IPOM.EQ.0) CALL ULMAXA(NF,NA,NAD,NC,AF,AFD,IA,AG,S,POM,XNORM,
     & SNORM,KREM,INEW)
      CALL ULMAXL(NF,NC,CF,CFD,IC,CL,CU,CG,S,POM,KBC,KREM,INEW)
      IF (NORMF.GT.0) CALL UXVMUL(NF,XN,S,S,1)
      IF (LCS.GT.0) CALL ULMAXT(NF,X,IX,IXA,XN,XO,S,POM,XDEL,NORMF,
     & KREM,INEW)
      CALL ULMAXS(NF,X,IX,XL,XU,S,POM,KBF,KREM,INEW)
      IF (INEW.EQ.0) THEN
      IF (IPOM.EQ.0) THEN
C
C     BOUNDED SOLUTION DOES NOT EXIST
C
      ITERL=-2
      CALL UOERR1('ULLPF2',10-ITERL)
      ELSE
C
C     FEASIBLE SOLUTION DOES NOT EXIST
C
      ITERL=-3
      CALL UOERR1('ULLPF2',10-ITERL)
      ENDIF
      GO TO 6
      ELSE
C
C     STEP REALIZATION
C
      CALL ULDIRS(NF,X,IX,S,POM,KBF)
      CALL ULDIRL(NC,CF,CFD,IC,POM,KBC)
      IF (IPOM.EQ.0) CALL ULDIRA(NA,NAD,AF,AFD,IA,POM,XNORM,SNORM)
C
C     CONSTRAINT ADDITION
C
      IF (INEW.GT.NC) THEN
      KA=INEW-NC
      INEW=0
      CALL ULNEWA(NA,NC,KA,AF,IA,EPS9,XNORM,INEW,KNEW)
      CALL ULADF0(NF,N,NA,NC,NC,IA,IAA,AG,AR,CG,S,EPS7,GMAX,UMAX,E,T,
     & INEW,KNEW,NADD,IER)
      CALL UXVIND(NAD,IA,KA,IER)
      ELSE IF (INEW.GT.0) THEN
      KC=INEW
      INEW=0
      CALL ULNEWL(NC,KC,CF,IC,CL,CU,EPS9,INEW)
      CALL ULADF0(NF,N,NA,NC,NC,IA,IAA,AG,AR,CG,S,EPS7,GMAX,UMAX,E,T,
     & INEW,KNEW,NADD,IER)
      CALL UXVIND(NC,IC,KC,IER)
      ELSE IF (INEW+NF.GE.0) THEN
      I=-INEW
      INEW=0
      CALL ULNEWS(NF,X,IX,XN,XL,XU,EPS9,NORMF,I,INEW)
      CALL ULADF0(NF,N,NA,NC,NC,IA,IAA,AG,AR,CG,S,EPS7,GMAX,UMAX,E,T,
     & INEW,KNEW,NADD,IER)
      CALL UXVIND(NF,IX,I,IER)
      ELSE
      I=-(INEW+NF)
      INEW=0
      CALL ULNEWT(NF,X,IX,IXA,XN,XO,XDEL,EPS9,NORMF,I,INEW)
      CALL ULADF0(NF,N,NA,NC,NC,IA,IAA,AG,AR,CG,S,EPS7,GMAX,UMAX,E,T,
     & INEW,KNEW,NADD,IER)
      CALL UXVIND(NF,IX,I,IER)
C      CALL UXVINT(NF,IX,IXA,I,IER)
      ENDIF
      DMAX=POM
      NRED=NRED+1
      JNEW=INEW
      JOLD=0
      GO TO 3
      ENDIF
      ENDIF
    6 CALL UOLLP2
      RETURN
      END


