* SUBROUTINE ULZTGB               ALL SYSTEMS                97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* NORMAL OF THE NEW ACTIVE CONSTRAINT IS PREMULTIPLIED BY TRANSPOSE
* OF THE MATRIX WHOSE COLUMNS ARE BASIC VECTORS FROM CURRENT REDUCED
* SUBSPACE.
*
* PARAMETERS :
*  II  NF  DECLARED NUMBER OF VARIABLES.
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  RI  XN(NF)  VECTOR OF SCALING FACTORS.
*  RI  CR(NF*(NF+1)/2)  TRIANGULAR DECOMPOSITION OF KERNEL OF THE
*         ORTHOGONAL PROJECTION.
*  RI  CZ(NF*NF)  MATRIX WHOSE COLUMNS ARE BASIC VECTORS FROM THE
*         CURRENT REDUCED SUBSPACE.
*  RI  G(NF)  GRADIENT OF THE OBJECTIVE FUNCTION.
*  RO  GN(NF)  OUTPUT VECTOR.
*  II  NORMF  SCALING SPECIFICATION. NORMF=0-NO SCALING PERFORMED.
*         NORMF=1-SCALING FACTORS ARE DETERMINED AUTOMATICALLY.
*         NORMF=2-SCALING FACTORS ARE SUPPLIED BY USER.
*
* SUBPROGRAMS USED :
*  S   UXDRMM  PREMULTIPLICATION OF A VECTOR BY A ROWWISE STORED DENSE
*         RECTANGULAR MATRIX.
*  S   UXVMUL  DIAGONAL PREMULTIPLICATION OF A VECTOR.
*
      SUBROUTINE ULZTGB(NF,N,XN,CR,CZ,G,GN,NORMF)
      INTEGER NF,N,NORMF
      $FLOAT XN(NF),CR(NF*(NF+1)/2),CZ(NF*NF),G(NF),GN(NF)
      INTEGER NCZ
      IF (N.EQ.0) RETURN
      NCZ=N*NF
      IF (NORMF.GT.0) THEN
      IF (NF.GT.N) THEN
      CALL UXVMUL(NF,XN,G,CZ(NCZ+1),1)
      CALL UXDRMM(NF,N,CZ,CZ(NCZ+1),GN)
      ELSE
      CALL UXVMUL(NF,XN,G,CR,1)
      CALL UXDRMM(NF,N,CZ,CR,GN)
      ENDIF
      ELSE
      CALL UXDRMM(NF,N,CZ,G,GN)
      ENDIF
      RETURN
      END
