* SUBROUTINE UNINT1                ALL SYSTEMS                91/12/01
C PORTABILITY : ALL SYSTEMS
C 91/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* EXTRAPOLATION OR INTERPOLATION FOR LINE SEARCH WITH DIRECTIONAL
* DERIVATIVES.
*
* PARAMETERS :
*  RI  RL  LOWER VALUE OF THE STEPSIZE PARAMETER.
*  RI  RU  UPPER VALUE OF THE STEPSIZE PARAMETER.
*  RI  FL  VALUE OF THE OBJECTIVE FUNCTION FOR R=RL.
*  RI  FU  VALUE OF THE OBJECTIVE FUNCTION FOR R=RU.
*  RI  PL  DIRECTIONAL DERIVATIVE FOR R=RL.
*  RI  PU  DIRECTIONAL DERIVATIVE FOR R=RU.
*  RO  R  VALUE OF THE STEPSIZE PARAMETER OBTAINED.
*  II  MODE  MODE OF LINE SEARCH.
*  II  MTYP  METHOD SELECTION. MTYP=1-BISECTION. MTYP=2-QUADRATIC
*         INTERPOLATION (WITH ONE DIRECTIONAL DERIVATIVE).
*         MTYP=3-QUADRATIC INTERPOLATION (WITH TWO DIRECTIONAL
*         DERIVATIVES). MTYP=4-CUBIC INTERPOLATION. MTYP=5-HOMOGENEOUS
*         INTERPOLATION. MTYP=6-CONIC INTERPOLATION.
*  IO  MERR  ERROR INDICATOR. MERR=0 FOR NORMAL RETURN.
*  TO  TEXT  TEXT INFORMATION ON THE STEPSIZE PARAMETER OBTAINED.
*
* METHOD :
* EXTRAPOLATION OR INTERPOLATION WITH STANDARD MODEL FUNCTIONS.
*
      SUBROUTINE UNINT1(RL,RU,FL,FU,PL,PU,R,MODE,MTYP,MERR,TEXT)
      $FLOAT RL, RU, FL, FU, PL, PU, R
      INTEGER MODE,MTYP,MERR,NTYP
      CHARACTER*8 TEXT
      $FLOAT A,B,C,D,DIS,DEN
      $FLOAT ZERO,HALF,ONE,TWO,THREE,C1L,C1U,C2L,C2U,C3L,FOUR
      PARAMETER (ZERO=0.0$P 0,HALF=0.5$P 0,ONE=1.0$P 0,TWO=2.0$P 0,
     *           THREE=3.0$P 0,C1L=1.1$P 0,C1U=1.0$P 3,C2L=1.0$P-2,
     *           C2U=0.9$P 0,C3L=0.1$P 0,FOUR=4.0$P 0)
      MERR=0
      IF (MODE.LE.0) RETURN
      IF (PL.GE.ZERO) THEN
      MERR=2
      TEXT='INT ER-2'
      RETURN
      ELSE IF (RU.LE.RL) THEN
      MERR=3
      TEXT='INT ER-3'
      RETURN
      ENDIF
      DO 1 NTYP=MTYP,1,-1
      IF (NTYP.EQ.1) THEN
C
C     BISECTION
C
      TEXT='BISECT. '
      IF (MODE.EQ.1) THEN
      R=FOUR*RU
      RETURN
      ELSE
      R=HALF*(RL+RU)
      RETURN
      ENDIF
      ELSE IF (NTYP.EQ.MTYP) THEN
      A = (FU-FL)/(PL*(RU-RL))
      B = PU/PL
      ENDIF
      IF (NTYP.EQ.2) THEN
C
C     QUADRATIC EXTRAPOLATION OR INTERPOLATION WITH ONE DIRECTIONAL
C     DERIVATIVE
C
      DEN = TWO*(ONE-A)
      TEXT = 'QUAD-1  '
      ELSE IF (NTYP.EQ.3) THEN
C
C     QUADRATIC EXTRAPOLATION OR INTERPOLATION WITH TWO DIRECTIONAL
C     DERIVATIVES
C
      DEN = ONE - B
      TEXT = 'QUAD-2  '
      ELSE IF (NTYP.EQ.4) THEN
C
C     CUBIC EXTRAPOLATION OR INTERPOLATION
C
      C = B - TWO*A + ONE
      D = B - THREE*A + TWO
      DIS = D*D - THREE*C
      IF (DIS.LT.ZERO) GO TO 1
      IF (D.GE.ZERO) THEN
      DEN = D + SQRT(DIS)
      ELSE
      DEN = THREE*C/(D-SQRT(DIS))
      ENDIF
      TEXT = 'CUBIC   '
      ELSE IF (NTYP.EQ.5) THEN
C
C     HOMOGENEOUS EXTRAPOLATION OR INTERPOLATION
C
      IF (B.EQ.A.OR.B.GE.ONE) GO TO 1
      DEN = (ONE-B)**((A-ONE)/(B-A))
      TEXT = 'HOMOGEN.'
      ELSE IF (NTYP.EQ.6) THEN
C
C     CONIC EXTRAPOLATION OR INTERPOLATION
C
      DIS = A*A - B
      IF (DIS.LT.ZERO) GO TO 1
      DEN = A + SQRT(DIS)
      IF (DEN.LE.ZERO) GO TO 1
      DEN = ONE - B*(ONE/DEN)**3
      TEXT = 'CONIC   '
      ENDIF
      IF (MODE.EQ.1.AND.DEN.GT.ZERO.AND.DEN.LT.ONE) THEN
C
C     EXTRAPOLATION ACCEPTED
C
      R = RL + (RU-RL)/DEN
      R = MAX(R,C1L*RU)
      R = MIN(R,C1U*RU)
      RETURN
      ELSE IF (MODE.EQ.2.AND.DEN.GT.ONE) THEN
C
C     INTERPOLATION ACCEPTED
C
      R = RL + (RU-RL)/DEN
      IF (RL.EQ.ZERO) THEN
      R = MAX(R,RL+C2L*(RU-RL))
      ELSE
      R = MAX(R,RL+C3L*(RU-RL))
      ENDIF
      R = MIN(R,RL+C2U*(RU-RL))
      RETURN
      ENDIF
    1 CONTINUE
      END
