* SUBROUTINE UNINT2             ALL SYSTEMS                89/12/01
C PORTABILITY : ALL SYSTEMS
C 89/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* EXTRAPOLATION OR INTERPOLATION FOR LINE SEARCH WITHOUT DIRECTIONAL
* DERIVATIVES.
*
* PARAMETERS :
*  RI  RO  INITIAL VALUE OF THE STEPSIZE PARAMETER.
*  RI  RL  LOWER VALUE OF THE STEPSIZE PARAMETER.
*  RI  RU  UPPER VALUE OF THE STEPSIZE PARAMETER.
*  RI  FO  VALUE OF THE OBJECTIVE FUNCTION FOR R=RO.
*  RI  FL  VALUE OF THE OBJECTIVE FUNCTION FOR R=RL.
*  RI  FU  VALUE OF THE OBJECTIVE FUNCTION FOR R=RU.
*  RO  PO  INITIAL VALUE OF THE DIRECTIONAL DERIVATIVE.
*  RO  R  VALUE OF THE STEPSIZE PARAMETER OBTAINED.
*  II  MODE  MODE OF LINE SEARCH.
*  II  MTYP  METHOD SELECTION. MTYP=1-BISECTION. MTYP=2-TWO POINT
*         QUADRATIC INTERPOLATION. MTYP=3-THREE POINT QUADRATIC
*         INTERPOLATION. MTYP=4-THREE POINT CUBIC INTERPOLATION.
*  IO  MERR  ERROR INDICATOR. MERR=0 FOR NORMAL RETURN.
*  TO  TEXT  TEXT INFORMATION ON THE STEPSIZE PARAMETER OBTAINED.
*
* METHOD :
* EXTRAPOLATION OR INTERPOLATION WITH STANDARD MODEL FUNCTIONS.
*
      SUBROUTINE UNINT2(RO,RL,RU,FO,FL,FU,PO,R,MODE,MTYP,MERR,TEXT)
      $FLOAT RO,RL,RU,FO,FL,FU,PO,R
      INTEGER MODE,MTYP,MERR,NTYP
      CHARACTER*8 TEXT
      $FLOAT AL,AU,DEN,DIS
      LOGICAL L1
      $FLOAT ZERO,HALF,ONE,TWO,THREE,FOUR,C1L,C1U,C2L,C2U,C3L,C3U
      PARAMETER(ZERO=0.0$P 0,HALF=0.5$P 0,ONE=1.0$P 0,TWO=2.0$P 0,
     & THREE=3.0$P 0,FOUR=4.0$P 0,C1L=1.1$P 0,C1U=1.0$P 2,
     & C2L=1.0$P-2,C2U=0.9$P 0,C3L=1.0$P-1,C3U=1.0$P 3)
      MERR = 0
      IF (MODE .LE. 0)  RETURN
      IF (PO .GE. ZERO)  THEN
      MERR = 2
      TEXT='INT ER-2'
      RETURN
      ELSE  IF (RU .LE. RL)  THEN
      MERR = 3
      TEXT='INT ER-3'
      RETURN
      ENDIF
      L1 = RL .LE. RO
      DO 1  NTYP = MTYP, 1, -1
      IF (NTYP .EQ. 1)  THEN
C
C     BISECTION
C
      TEXT = 'BISECT. '
      IF (MODE .EQ. 1)  THEN
      R = TWO * RU
      RETURN
      ELSE
      R = HALF * (RL + RU)
      RETURN
      ENDIF
      ELSE  IF (NTYP .EQ. MTYP)  THEN
      IF (.NOT. L1)   AL = (FL-FO) / (RL*PO)
      AU = (FU-FO) / (RU*PO)
      ENDIF
      IF (NTYP .EQ. 2.OR.L1)  THEN
C
C     TWO POINT QUADRATIC EXTRAPOLATION OR INTERPOLATION
C
      IF (AU .GE. ONE)  GO TO 1
      R = HALF * RU / (ONE-AU)
      TEXT = 'QUAD-1  '
      ELSE  IF (NTYP .EQ. 3)  THEN
C
C     THREE POINT QUADRATIC EXTRAPOLATION OR INTERPOLATION
C
      IF (AU .GE. AL)  GO TO 1
      R = HALF * (AL*RU - AU*RL) / (AL-AU)
      TEXT = 'QUAD-2  '
      ELSE  IF (NTYP .EQ. 4)  THEN
C
C     THREE POINT CUBIC EXTRAPOLATION OR INTERPOLATION
C
      DIS = (AL-ONE) * (RU/RL)
      DEN = (AU-ONE) * (RL/RU)  -  DIS
      DIS = AU + AL - DEN - TWO * (ONE + DIS)
      DIS = DEN * DEN - THREE * DIS
      IF (DIS .LT. ZERO)  GO TO 1
      DEN = DEN + SQRT( DIS)
      IF (DEN .EQ. ZERO)  GO TO 1
      R = (RU-RL) / DEN
      TEXT = 'CUBIC   '
      ENDIF
      IF (MODE .EQ. 1  .AND.  R .GT. RU)  THEN
C
C     EXTRAPOLATION ACCEPTED
C
      R = MAX( R, C1L*RU)
      IF (NTYP.EQ.4) THEN
      R = MIN( R, C1U*RU)
      ELSE
      R = MIN( R, C3U*RU)
      ENDIF
      RETURN
      ELSE  IF (MODE .EQ. 2  .AND.  R .GT. ZERO  .AND.  R .LT. RU)  THEN
C
C     INTERPOLATION ACCEPTED
C
      IF (RL.EQ.ZERO) THEN
      R = MAX(R,RL+C2L*(RU-RL))
      ELSE
      R = MAX(R,RL+C3L*(RU-RL))
      ENDIF
      R = MIN( R, RL + C2U*(RU-RL))
      RETURN
      ENDIF
    1 CONTINUE
      END
