* SUBROUTINE UNINT8                ALL SYSTEMS                90/12/01
C PORTABILITY : ALL SYSTEMS
C 90/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* EXTRAPOLATION OR INTERPOLATION FOR LINE SEARCH USED IN LEAST SQUARES
* APPROXIMATION.
*
* PARAMETERS :
*  RI  RL  LOWER VALUE OF THE STEPSIZE PARAMETER.
*  RI  RU  UPPER VALUE OF THE STEPSIZE PARAMETER.
*  RI  FL  VALUE OF THE OBJECTIVE FUNCTION FOR R=RL.
*  RI  FU  VALUE OF THE OBJECTIVE FUNCTION FOR R=RU.
*  RI  PL  DIRECTIONAL DERIVATIVE FOR R=RL.
*  RI  PU  DIRECTIONAL DERIVATIVE FOR R=RU.
*
* COMMON DATA :
*
      SUBROUTINE UNINT8(R,RL,RU,RP,FL,FU,FP,PL,PU,PP,ETA9,MES1,MERR,
     & MODE)
      $INCLUDE ('UMCOMM')
      INTEGER MES1,MERR,MODE
      $FLOAT R,RL,RU,RP,FL,FU,FP,PL,PU,PP,ETA9
      INTEGER MTYP,NTYP
      $FLOAT A1,A3,B1,B2,B3,B4,C1,C2,C3,C4,POM,DEN,DIS,DIF,RBEST
      INTEGER K
      $FLOAT C1L,C1U,C2L,C2U,C3L,PI,CON1,CON2,SIX
      PARAMETER (C1L=1.1$P 0,C1U=1.0$P 4,C2L=1.0$P-4,C2U=0.9$P 0,
     * C3L=0.1$P 0,PI=3.14159265358979323846$P 0)
      PARAMETER (CON1=1.2$P 2,CON2=1.0$P-5,SIX=6.0$P 0)
      MERR=0
      IF (MODE.LE.0) RETURN
      IF (PL.GE.ZERO) THEN
      MERR=2
      TSXX='INT ER-2'
      RETURN
      ELSE IF (RU.LE.RL) THEN
      MERR=3
      TSXX='INT ER-3'
      RETURN
      ENDIF
      MTYP=5
      IF (MODE.EQ.1.AND.MES1.EQ.1) MTYP=1
      DO 1 NTYP=MTYP,1,-1
      IF (NTYP.EQ.1) THEN
C
C     BISECTION
C
      TSXX='BISECT. '
      IF (MODE.EQ.1) THEN
      R=FOUR*RU
      RETURN
      ELSE
      R=HALF*(RL+RU)
      RETURN
      ENDIF
      ELSE IF (NTYP.EQ.5) THEN
      DIF=RU-RL
      A1=(FU-FL)/(PL*DIF)
      A3=PU/PL
      ENDIF
      IF (NTYP.EQ.2) THEN
C
C     QUADRATIC EXTRAPOLATION OR INTERPOLATION WITH ONE DIRECTIONAL
C     DERIVATIVE
C
      DEN=TWO*(ONE-A1)
      IF (DEN.LE.ZERO) GO TO 1
      TSXX='QUAD-1  '
      ELSE IF (NTYP.EQ.3) THEN
C
C     QUADRATIC EXTRAPOLATION OR INTERPOLATION WITH TWO DIRECTIONAL
C     DERIVATIVES
C
      DEN=ONE-A3
      IF (DEN.LE.ZERO) GO TO 1
      TSXX='QUAD-2  '
      ELSE IF (NTYP.EQ.4) THEN
C
C     CUBIC EXTRAPOLATION OR INTERPOLATION
C
      C1=A3-TWO*A1+ONE
      C3=A3-THREE*A1+TWO
      DIS=C3*C3-THREE*C1
      IF (DIS.LT.ZERO) GO TO 1
      DEN=C3+SQRT(DIS)
      IF (DEN.LE.ZERO) GO TO 1
      TSXX='CUBIC   '
      ELSE IF (NTYP.EQ.5) THEN
      IF (RP.EQ.RL.OR.RP.EQ.RU) GO TO 1
C
C     BIQUADRATIC EXTRAPOLATION OR INTERPOLATION
C
      RBEST=ZERO
      IF (FU.LT.FL) RBEST=DIF
      TSXX='BIQUAD  '
C
C     DETERMINATION OF COEFFICIENTS OF THE CUBIC EQUATION
C     C1 + C2*R + 3*C3*R**2 + 2*C4*R**3 = 0
C
      DIS=RP-RL
      POM=RP-RU
      B1=(A1-ONE)/DIF
      B2=((FP-FL)/(PL*DIS)-ONE)/DIS
      B3=((A3-ONE)/DIF-TWO*B1)/DIF
      B4=((PP/PL-ONE)/DIS-TWO*B2)/DIS
      C4=HALF*(B4-B3)/POM
      C3=HALF*B3-DIF*C4
      C2=B1-DIF*(TWO*C3+DIF*C4)
      C1=HALF
      B3=THREE*C3
      B4=TWO*C4
C
C     DECISION BETWEEN DIRECT AND ITERATIVE SOLUTION
C
      POM=C1+RBEST*(C2+RBEST*(B3+RBEST*B4))
      DEN=C2+SIX*RBEST*(C3+RBEST*C4)
      IF (DEN.EQ.ZERO) THEN
      DIS=ETA9
      ELSE
      DIS=ABS(POM/DEN)
      ENDIF
      B2=ABS(C3+TWO*C4*RBEST)+FOUR*C4*DIS
      IF (DEN.LE.CON1*B2*MAX(ONE,DIS)) THEN
C
C     DIRECT SOLUTION
C
      B1=C1/B4
      B2=C2/B4
      B3=C3/B4
      POM=(B2-THREE*B3**2)/THREE
      DEN=(B1-B2*B3+TWO*B3**3)/TWO
      DIS=DEN**2+POM**3
      IF (DIS.LT.ZERO) THEN
C
C     THREE REAL ROOTS
C
      DIS=SQRT(-DIS)
      IF (DEN.EQ.ZERO) THEN
      B1=HALF*PI
      ELSE
      B1=ATAN(DIS/ABS(DEN))
      ENDIF
      B2=TWO*SQRT(-POM)
      IF (DEN.GT.ZERO) B2=-B2
      C1=B2*COS(B1/THREE)-B3
      C2=B2*COS((B1+ TWO*PI)/THREE)-B3
      C3=B2*COS((B1+FOUR*PI)/THREE)-B3
      IF (C2.LT.C1) THEN
      C4=C1
      C1=C2
      C2=C4
      ENDIF
      IF (C3.LT.C1) THEN
      C4=C1
      C1=C3
      C3=C4
      ENDIF
      IF (C3.LT.C2) THEN
      C4=C2
      C2=C3
      C3=C4
      ENDIF
      IF (MODE.EQ.1) THEN
      R=C3
      IF (C2.GT.DIF) R=C2
      IF (C1.GT.DIF) R=C1
      ELSE IF (RBEST.EQ.ZERO) THEN
      R=C3
      IF (C2.GT.ZERO) R=C2
      IF (C1.GT.ZERO) R=C1
      ELSE
      R=C1
      IF (C2.LT.DIF) R=C2
      IF (C3.LT.DIF) R=C3
      ENDIF
      ELSE
C
C     ONE REAL ROOT
C
      DIS=SQRT(DIS)
      B1=-DEN-DIS
      B2=-DEN+DIS
      C1=SIGN(ONE,B1)
      C2=SIGN(ONE,B2)
      C3=ONE/THREE
      R=C1*ABS(B1)**C3+C2*ABS(B2)**C3-B3
      ENDIF
      ELSE
C
C     ITERATIVE SOLUTION BY THE NEWTON METHOD
C
      K=0
    2 K=K+1
      DIS=-POM/DEN
      RBEST=RBEST+DIS
      POM=C1+RBEST*(C2+RBEST*(B3+RBEST*B4))
      DEN=C2+SIX*RBEST*(C3+RBEST*C4)
      IF (DEN.EQ.ZERO) THEN
      B1=ETA9
      ELSE
      B1=B2*DIS*DIS/ABS(DEN)
      ENDIF
      IF (B1.GT.CON2.AND.K.LT.3) GO TO 2
      R=RBEST
      ENDIF
      ENDIF
      IF (NTYP.LT.5) R=DIF/DEN
      IF (MODE.EQ.1.AND.R.GT.DIF) THEN
C
C     EXTRAPOLATION ACCEPTED
C
      R=RL+R
      R=MAX(R,C1L*RU)
      R=MIN(R,C1U*RU)
      RETURN
      ELSE IF (MODE.EQ.2.AND.R.GT.ZERO.AND.R.LT.POM) THEN
C
C     INTERPOLATION ACCEPTED
C
      R=RL+R
      IF (RL.EQ.ZERO) THEN
      R=MAX(R,RL+C2L*DIF)
      ELSE
      R=MAX(R,RL+C3L*DIF)
      ENDIF
      R=MIN(R,RL+C2U*DIF)
      RETURN
      ENDIF
    1 CONTINUE
      END



