* SUBROUTINE UOMSCR                ALL SYSTEMS                10/12/01
C PORTABILITY : ALL SYSTEMS
C 10/12/01 MA : ORIGINAL VERSION
*
* PURPOSE :
*  MATLAB SCREEN INTERFACE FOR GRAPHICAL VISUALIZATION.
*
* PARAMETERS :
*  CI  TYP  THE CASE: V - VARIABLES
*                     A - APPROXIMATIONS
*                     C - CONSTRAINTS
*                     X - ITERATIONS OF X
*                     F - ITERATIONS OF F
*                     S - ISOLINES/MAP/SURFACE
*                     I - PATH WITH ISOLINES
*                     T - 1D TRAJECTORIES
*                     O - 2D ORBIT
*                     P - 3D PICTURE
*  II  NN  DECLARED NUMBER OF VARIABLES NF
*                          OR CONSTRAINTS NC
*                          OR APPROXIMATED FUNCTIONS NA
*                          OR IPA IN CASE OF PATH WITH ISOLINES OR MAP (UPDATE)
*  II  TEMP  EITHER KBA IN CASE OF APPROXIMATED FUNCTIONS
*                OR NED IN CASE OF DIFFERENTIAL EQUATIONS
*                OR JOB IN CASE OF PATH OF ITERATIONS
*                OR JOB IN CASE OF PATH WITH ISOLINES/MAP
*  II  II  INDEX OF THE 1-ST VISUALIZED DIFFERENTIAL EQUATION IG
*                         OR PLOTTING VARIABLE IGR
*  II  JJ  INDEX OF THE 2-ND VISUALIZED DIFFERENTIAL EQUATION JG
*                         OR PLOTTING VARIABLE JGR
*  II  KK  INDEX OF THE 3-RD VISUALIZED DIFFERENTIAL EQUATION KG
*                         OR PLOTTING VARIABLE KGR
*  II  GFMT  GRAPHICS FORMAT (1=*.EPS, 2=*.PNG)
*  IO  STA  START INDEX IN THE GRAPH
*  IO  STO  STOP INDEX IN THE GRAPH
*  IO  INFO  THE TYPE OF GRAPH SPECIFIED BY THE USER
*  IU  LEVEL  THE NUMBER OF CONTOUR LEVELS IN CASE OF ISOLINES OR MAP
*  IO  SCALE  LINEAR OR LOGARITHMIC SCALE IN CASE OF ISOLINES, MAP OR SURFACE
*
      SUBROUTINE UOMSCR
     &  (TYP,NN,TEMP,II,JJ,KK,GFMT,STA,STO,INFO,LEVEL,SCALE)
      $OPTION(CHEOL='#')
      INTEGER NN,TEMP,II,JJ,KK,GFMT,STA,STO,INFO,LEVEL,SCALE,MENU,I,J,
     &  GEN
      CHARACTER TYP,ANSWER
      CHARACTER*10 T1,T2,T3,T4,T5,T6
C
      INFO=0
      IF (TYP.EQ.'V'.OR.TYP.EQ.'C') GOTO 10   ! VARIABLES OR CONSTRAINTS
      IF (TYP.EQ.'A') GOTO 20   ! APPROXIMATIONS
      IF (TYP.EQ.'T') GOTO 30   ! DIFF.EQ. 1D TRAJECTORIES
      IF (TYP.EQ.'O') GOTO 40   ! DIFF.EQ. 2D ORBIT
      IF (TYP.EQ.'P') GOTO 50   ! DIFF.EQ. 3D PICTURE
      IF (TYP.EQ.'X') GOTO 60   ! GRAPH OF ITERATIONS
      IF (TYP.EQ.'F') GOTO 70   ! GRAPH OF FUNCTION VALUES
      IF (TYP.EQ.'S') GOTO 80   ! ISOLINES/MAP/SURFACE
      IF (TYP.EQ.'I') GOTO 90   ! PATH WITH ISOLINES/MAP
C
C     VARIABLES / CONSTRAINTS
C
   10 CONTINUE
      IF (TYP.EQ.'V') WRITE(*,'(A)') 'GRAPH OF VARIABLES:'
      IF (TYP.EQ.'C') WRITE(*,'(A)') 'GRAPH OF CONSTRAINTS:'
   15 WRITE(*,'(A)')
     & '  P = POINTS, C = CURVE, R = RETURN  [ENTER = R]'
      READ(*,'(A)',ERR=15) ANSWER
      IF (ANSWER.EQ.'P'.OR.ANSWER.EQ.'p') THEN
        MENU=1
      ELSEIF (ANSWER.EQ.'C'.OR.ANSWER.EQ.'c') THEN
        MENU=2
      ELSE
        MENU=0
      ENDIF
      IF (MENU.EQ.1.OR.MENU.EQ.2) THEN
        WRITE(T2,'(I10)') NN
        T2=ADJUSTL(T2)
   11   CONTINUE
        IF (TYP.EQ.'V') WRITE(*,'(2A)') '  NF = ',TRIM(T2)
        IF (TYP.EQ.'C') WRITE(*,'(2A)') '  NC = ',TRIM(T2)
   16   WRITE(*,'(3A)')
     &    '  START INDEX (1..',TRIM(T2),'):  [ENTER = 1]'
        READ(*,'(I6)',ERR=16) STA
        IF (STA.EQ.0) STA=1
        IF (STA.LT.1.OR.STA.GT.NN) GOTO 16
        WRITE(T1,'(I10)') STA
        T1=ADJUSTL(T1)
   12   WRITE(*,'(7A)') '  STOP INDEX (',TRIM(T1),'..',TRIM(T2),
     &    '):  [ENTER = ',TRIM(T2),']'
        READ(*,'(I6)',ERR=12) STO
        IF (STO.EQ.0) STO=NN
        IF (STO.GT.NN) GOTO 12
        IF (STA.GT.STO) THEN
          WRITE(*,'(A)') 'ERROR: START > STOP'
          GOTO 11
        ENDIF
        WRITE(T2,'(I10)') STO
        T2=ADJUSTL(T2)
        IF (TYP.EQ.'V') THEN
          IF (MENU.EQ.1) THEN
            WRITE(*,'(4A)') 'VARIABLES: GRAPH = POINTS, START = ',
     &        TRIM(T1),', STOP = ',TRIM(T2)
          ELSE
            WRITE(*,'(4A)') 'VARIABLES: GRAPH = CURVE, START = ',
     &        TRIM(T1),', STOP = ',TRIM(T2)
          ENDIF
        ENDIF
        IF (TYP.EQ.'C') THEN
          IF (MENU.EQ.1) THEN
            WRITE(*,'(4A)') 'CONSTRAINTS: GRAPH = POINTS, START = ',
     &        TRIM(T1),', STOP = ',TRIM(T2)
          ELSE
            WRITE(*,'(4A)') 'CONSTRAINTS: GRAPH = CURVE, START = ',
     &        TRIM(T1),', STOP = ',TRIM(T2)
          ENDIF
        ENDIF
        CALL UOMGEN(GFMT,GEN)
        IF (GEN.EQ.0) GOTO 10
        INFO=MENU
      ENDIF
      RETURN
C
C    APPROXIMATIONS
C
   20 CONTINUE
      IF (TEMP.GT.0) THEN
        WRITE(*,'(A)') 'GRAPH OF APPROXIMATIONS:'
   25   WRITE(*,'(A)')
     &    '  F = AF, M = AF,AM, D = AF-AM, R = RETURN  [ENTER = R]'
        READ(*,'(A)',ERR=25) ANSWER
        IF (ANSWER.EQ.'F'.OR.ANSWER.EQ.'f') THEN
          MENU=1
        ELSEIF (ANSWER.EQ.'M'.OR.ANSWER.EQ.'m') THEN
          MENU=2
        ELSEIF (ANSWER.EQ.'D'.OR.ANSWER.EQ.'d') THEN
          MENU=3
        ELSE
          MENU=0
        ENDIF
      ELSE
        WRITE(*,'(A)') 'GRAPH OF APPROXIMATIONS:'
   26   WRITE(*,'(A)') '  F = AF, R = RETURN  [ENTER = R]'
        READ(*,'(A)',ERR=26) ANSWER
        IF (ANSWER.EQ.'F'.OR.ANSWER.EQ.'f') THEN
          MENU=1
        ELSE
          MENU=0
        ENDIF
      ENDIF
      IF (MENU.GT.0) THEN
        WRITE(*,'(A)') 'TYPE OF GRAPH FOR AF: '
   27   WRITE(*,'(A)')
     &    '  P = POINTS, C = CURVE, L = LINES  [ENTER = L]'
        READ(*,'(A)',ERR=27) ANSWER
        IF (ANSWER.EQ.'P'.OR.ANSWER.EQ.'p') THEN
          MENU=3*(MENU-1)+1
        ELSEIF (ANSWER.EQ.'C'.OR.ANSWER.EQ.'c') THEN
          MENU=3*(MENU-1)+2
        ELSE
          MENU=3*(MENU-1)+3
        ENDIF
        WRITE(T2,'(I10)') NN
        T2=ADJUSTL(T2)
   21   CONTINUE
        WRITE(*,'(2A)') '  NA = ',TRIM(T2)
   28   WRITE(*,'(3A)')
     &    '  START INDEX (1..',TRIM(T2),'):  [ENTER = 1]'
        READ(*,'(I6)',ERR=28) STA
        IF (STA.EQ.0) STA=1
        IF (STA.LT.1.OR.STA.GT.NN) GOTO 28
        WRITE(T1,'(I10)') STA
        T1=ADJUSTL(T1)
   22   WRITE(*,'(7A)') '  STOP INDEX (',TRIM(T1),'..',TRIM(T2),
     &    '):  [ENTER = ',TRIM(T2),']'
        READ(*,'(I6)',ERR=22) STO
        IF (STO.EQ.0) STO=NN
        IF (STO.GT.NN) GOTO 22
        IF (STA.GT.STO) THEN
          WRITE(*,'(A)') 'ERROR: START > STOP'
          GOTO 21
        ENDIF
        WRITE(T2,'(I10)') STO
        T2=ADJUSTL(T2)
        IF (MENU.EQ.1) THEN
          WRITE(*,'(4A)')
     &      'APPROXIMATIONS: GRAPH = AF, AF = POINTS, START = ',
     &      TRIM(T1),', STOP = ',TRIM(T2)
        ELSEIF (MENU.EQ.2) THEN
          WRITE(*,'(4A)')
     &      'APPROXIMATIONS: GRAPH = AF, AF = CURVE, START = ',
     &      TRIM(T1),', STOP = ',TRIM(T2)
        ELSEIF (MENU.EQ.3) THEN
          WRITE(*,'(4A)')
     &      'APPROXIMATIONS: GRAPH = AF, AF = LINES, START = ',
     &      TRIM(T1),', STOP = ',TRIM(T2)
        ELSEIF (MENU.EQ.4) THEN
          WRITE(*,'(4A)')
     &      'APPROXIMATIONS: GRAPH = AF,AM, AF = POINTS, START = ',
     &      TRIM(T1),', STOP = ',TRIM(T2)
        ELSEIF (MENU.EQ.5) THEN
          WRITE(*,'(4A)')
     &      'APPROXIMATIONS: GRAPH = AF,AM, AF = CURVE, START = ',
     &      TRIM(T1),', STOP = ',TRIM(T2)
        ELSEIF (MENU.EQ.6) THEN
          WRITE(*,'(4A)')
     &      'APPROXIMATIONS: GRAPH = AF,AM, AF = LINES, START = ',
     &      TRIM(T1),', STOP = ',TRIM(T2)
        ELSEIF (MENU.EQ.7) THEN
          WRITE(*,'(4A)')
     &      'APPROXIMATIONS: GRAPH = AF-AM, AF = POINTS, START = ',
     &      TRIM(T1),', STOP = ',TRIM(T2)
        ELSEIF (MENU.EQ.8) THEN
          WRITE(*,'(4A)')
     &      'APPROXIMATIONS: GRAPH = AF-AM, AF = CURVE, START = ',
     &      TRIM(T1),', STOP = ',TRIM(T2)
        ELSEIF (MENU.EQ.9) THEN
          WRITE(*,'(4A)')
     &      'APPROXIMATIONS: GRAPH = AF-AM, AF = LINES, START = ',
     &      TRIM(T1),', STOP = ',TRIM(T2)
        ENDIF
        CALL UOMGEN(GFMT,GEN)
        IF (GEN.EQ.0) GOTO 20
        INFO=MENU
      ENDIF
      RETURN
C
C     TRAJECTORIES
C
   30 CONTINUE
      WRITE(T1,'(I10)')II
      T1=ADJUSTL(T1)
      IF (TEMP.GE.2) THEN
        WRITE(T2,'(I10)')JJ
        T2=ADJUSTL(T2)
      ENDIF
      IF (TEMP.EQ.3) THEN
        WRITE(T3,'(I10)')KK
        T3=ADJUSTL(T3)
      ENDIF
C
   31 WRITE(*,'(A)') 'DIFFERENTIAL EQUATIONS:'
   35 IF (TEMP.EQ.1) THEN
        WRITE(*,'(3A)')'  1 = TRAJECTORY OF YA(', TRIM(T1), ')'
      ELSEIF (TEMP.EQ.2) THEN
        WRITE(*,'(3A)')'  1 = TRAJECTORY OF YA(', TRIM(T1), ')'
        WRITE(*,'(3A)')'  2 = TRAJECTORY OF YA(', TRIM(T2), ')'
        WRITE(*,'(5A)')'  3 = TRAJECTORIES OF YA(', TRIM(T1),
     &    '), YA(', TRIM(T2), ')'
      ELSE   ! TEMP.EQ.3
        WRITE(*,'(3A)')'  1 = TRAJECTORY OF YA(', TRIM(T1), ')'
        WRITE(*,'(3A)')'  2 = TRAJECTORY OF YA(', TRIM(T2), ')'
        WRITE(*,'(3A)')'  3 = TRAJECTORY OF YA(', TRIM(T3), ')'
        WRITE(*,'(5A)')'  4 = TRAJECTORIES OF YA(', TRIM(T1),
     &    '), YA(', TRIM(T2), ')'
        WRITE(*,'(5A)')'  5 = TRAJECTORIES OF YA(', TRIM(T1),
     &    '), YA(', TRIM(T3), ')'
        WRITE(*,'(5A)')'  6 = TRAJECTORIES OF YA(', TRIM(T2),
     &    '), YA(', TRIM(T3), ')'
        WRITE(*,'(7A)')'  7 = TRAJECTORIES OF YA(', TRIM(T1),
     &    '), YA(', TRIM(T2), '), YA(', TRIM(T3), ')'
      ENDIF
      WRITE(*,'(A)') '  0 = RETURN  [ENTER = 0]'
      READ(*,'(I1)',ERR=35) MENU
C
      IF (TEMP.EQ.1.AND.MENU.EQ.1.OR.
     &    TEMP.EQ.2.AND.MENU.GE.1.AND.MENU.LE.3.OR.
     &    TEMP.EQ.3.AND.MENU.GE.1.AND.MENU.LE.7) THEN
        WRITE(T5,'(I10)') NN
        T5=ADJUSTL(T5)
   32   WRITE(T6,'(I10)') NN-1
        T6=ADJUSTL(T6)
        WRITE(*,'(2A)') '  NA = ',TRIM(T5)
   36   WRITE(*,'(3A)')
     &    '  START INDEX (1..',TRIM(T6),'):  [ENTER = 1]'
        READ(*,'(I6)',ERR=36) STA
        IF (STA.EQ.0) STA=1
        IF (STA.LT.1.OR.STA.GT.NN-1) GOTO 36
        WRITE(T4,'(I10)') STA
        T4=ADJUSTL(T4)
        WRITE(T6,'(I10)') STA+1
        T6=ADJUSTL(T6)
   33   WRITE(*,'(7A)') '  STOP INDEX (',TRIM(T6),'..',TRIM(T5),
     &    '):  [ENTER = ',TRIM(T5),']'
        READ(*,'(I6)',ERR=33) STO
        IF (STO.EQ.0) STO=NN
        IF (STO.GT.NN) GOTO 33
        IF (STA.GE.STO) THEN
          WRITE(*,'(A)') 'ERROR: START >= STOP'
          GOTO 32
        ENDIF
        WRITE(T5,'(I10)') STO
        T5=ADJUSTL(T5)
        IF (TEMP.EQ.1) THEN
          WRITE(*,'(6A)') 'TRAJECTORY OF YA(',TRIM(T1),'), START = ',
     &      TRIM(T4),', STOP = ',TRIM(T5)
        ELSEIF (TEMP.EQ.2.AND.MENU.EQ.1) THEN
          WRITE(*,'(6A)') 'TRAJECTORY OF YA(',TRIM(T1),'), START = ',
     &      TRIM(T4),', STOP = ',TRIM(T5)
        ELSEIF (TEMP.EQ.2.AND.MENU.EQ.2) THEN
          WRITE(*,'(6A)') 'TRAJECTORY OF YA(',TRIM(T2),'), START = ',
     &      TRIM(T4),', STOP = ',TRIM(T5)
        ELSEIF (TEMP.EQ.2.AND.MENU.EQ.3) THEN
          WRITE(*,'(8A)') 'TRAJECTORIES OF YA(', TRIM(T1),'), YA(',
     &      TRIM(T2),'), START = ',TRIM(T4),', STOP = ',TRIM(T5)
        ELSEIF (MENU.EQ.1) THEN
          WRITE(*,'(6A)') 'TRAJECTORY OF YA(',TRIM(T1),'), START = ',
     &      TRIM(T4),', STOP = ',TRIM(T5)
        ELSEIF (MENU.EQ.2) THEN
          WRITE(*,'(6A)') 'TRAJECTORY OF YA(',TRIM(T2),'), START = ',
     &      TRIM(T4),', STOP = ',TRIM(T5)
        ELSEIF (MENU.EQ.3) THEN
          WRITE(*,'(6A)') 'TRAJECTORY OF YA(',TRIM(T3),'), START = ',
     &      TRIM(T4),', STOP = ',TRIM(T5)
        ELSEIF (MENU.EQ.4) THEN
          WRITE(*,'(8A)') 'TRAJECTORIES OF YA(', TRIM(T1),'), YA(',
     &      TRIM(T2),'), START = ',TRIM(T4),', STOP = ',TRIM(T5)
        ELSEIF (MENU.EQ.5) THEN
          WRITE(*,'(8A)') 'TRAJECTORIES OF YA(', TRIM(T1),'), YA(',
     &      TRIM(T3),'), START = ',TRIM(T4),', STOP = ',TRIM(T5)
        ELSEIF (MENU.EQ.6) THEN
          WRITE(*,'(8A)') 'TRAJECTORIES OF YA(', TRIM(T2),'), YA(',
     &      TRIM(T3),'), START = ',TRIM(T4),', STOP = ',TRIM(T5)
        ELSE   ! MENU.EQ.7
          WRITE(*,'(10A)') 'TRAJECTORIES OF YA(', TRIM(T1),'), YA(',
     &      TRIM(T2), '), YA(', TRIM(T3), '), START = ',TRIM(T4),
     &      ', STOP = ',TRIM(T5)
        ENDIF
        CALL UOMGEN(GFMT,GEN)
        IF (GEN.EQ.0) GOTO 31
        INFO=MENU
      ENDIF
      RETURN
C
C     ORBIT
C
   40 CONTINUE
      WRITE(T1,'(I10)')II
      T1=ADJUSTL(T1)
      WRITE(T2,'(I10)')JJ
      T2=ADJUSTL(T2)
      IF (TEMP.EQ.3) THEN
        WRITE(T3,'(I10)')KK
        T3=ADJUSTL(T3)
      ENDIF
C
   41 WRITE(*,'(A)') 'DIFFERENTIAL EQUATIONS:'
   45 IF (TEMP.EQ.2) THEN
        WRITE(*,'(5A)')
     &    '  1 = ORBIT OF YA(', TRIM(T1), '), YA(', TRIM(T2), ')'
      ELSE   ! TEMP.EQ.3
        WRITE(*,'(5A)')
     &    '  1 = ORBIT OF YA(', TRIM(T1), '), YA(', TRIM(T2), ')'
        WRITE(*,'(5A)')
     &    '  2 = ORBIT OF YA(', TRIM(T1), '), YA(', TRIM(T3), ')'
        WRITE(*,'(5A)')
     &    '  3 = ORBIT OF YA(', TRIM(T2), '), YA(', TRIM(T3), ')'
        IF (GFMT.GT.0) THEN
         WRITE(*,'(9A)') '  4 = ORBITS OF YA(', TRIM(T1), '), YA(',
     &    TRIM(T2), '); YA(', TRIM(T1), '), YA(', TRIM(T3), ')'
         WRITE(*,'(9A)')'  5 = ORBITS OF YA(', TRIM(T1), '), YA(',
     &    TRIM(T2), '); YA(', TRIM(T2), '), YA(', TRIM(T3), ')'
         WRITE(*,'(9A)')'  6 = ORBITS OF YA(', TRIM(T1), '), YA(',
     &    TRIM(T3), '); YA(', TRIM(T2), '), YA(', TRIM(T3),')'
         WRITE(*,'(13A)')'  7 = ORBITS OF YA(', TRIM(T1), '), YA(',
     &    TRIM(T2), '); YA(', TRIM(T1), '), YA(', TRIM(T3), '); YA(',
     &    TRIM(T2), '), YA(', TRIM(T3),')'
        ENDIF
      ENDIF
      WRITE(*,'(A)')'  0 = RETURN  [ENTER = 0]'
      READ(*,'(I1)',ERR=45) MENU
      IF (TEMP.EQ.2.AND.MENU.EQ.1.OR.
     &    TEMP.EQ.3.AND.MENU.GE.1.AND.MENU.LE.3.OR.
     &    TEMP.EQ.3.AND.GFMT.GT.0.AND.MENU.GE.4.AND.MENU.LE.7) THEN
        IF (MENU.EQ.1) THEN
          WRITE(*,'(5A)')
     &      'ORBIT OF YA(', TRIM(T1), '), YA(', TRIM(T2), ')'
        ELSEIF (TEMP.EQ.3.AND.MENU.EQ.2) THEN
          WRITE(*,'(5A)')
     &      'ORBIT OF YA(', TRIM(T1), '), YA(', TRIM(T3), ')'
        ELSEIF (TEMP.EQ.3.AND.MENU.EQ.3) THEN
          WRITE(*,'(5A)')
     &      'ORBIT OF YA(', TRIM(T2), '), YA(', TRIM(T3), ')'
        ELSEIF (TEMP.EQ.3.AND.MENU.EQ.4) THEN
          WRITE(*,'(9A)') 'ORBITS OF YA(', TRIM(T1), '), YA(',
     &      TRIM(T2), '); YA(', TRIM(T1), '), YA(', TRIM(T3), ')'
        ELSEIF (TEMP.EQ.3.AND.MENU.EQ.5) THEN
          WRITE(*,'(9A)') 'ORBITS OF YA(', TRIM(T1), '), YA(',
     &      TRIM(T2), '); YA(', TRIM(T2), '), YA(', TRIM(T3), ')'
        ELSEIF (TEMP.EQ.3.AND.MENU.EQ.6) THEN
          WRITE(*,'(9A)') 'ORBITS OF YA(', TRIM(T1), '), YA(',
     &      TRIM(T3), '); YA(', TRIM(T2), '), YA(', TRIM(T3),')'
        ELSEIF (TEMP.EQ.3.AND.MENU.EQ.7) THEN
          WRITE(*,'(13A)') 'ORBITS OF YA(', TRIM(T1), '), YA(',
     &      TRIM(T2), '); YA(', TRIM(T1), '), YA(', TRIM(T3),
     &      '); YA(', TRIM(T2), '), YA(', TRIM(T3),')'
        ENDIF
        CALL UOMGEN(GFMT,GEN)
        IF (GEN.EQ.0) GOTO 41
        INFO=MENU
      ENDIF
      RETURN
C
C     3D PICTURE
C
   50 CONTINUE
      WRITE(T1,'(I10)')II
      T1=ADJUSTL(T1)
      WRITE(T2,'(I10)')JJ
      T2=ADJUSTL(T2)
      WRITE(T3,'(I10)')KK
      T3=ADJUSTL(T3)
C
   51 WRITE(*,'(A)') 'DIFFERENTIAL EQUATIONS:'
   55 WRITE(*,'(7A)') '  1 = PICTURE OF YA(', TRIM(T1),
     &  '), YA(', TRIM(T2), '), YA(', TRIM(T3), ')'
      WRITE(*,'(A)') '  0 = RETURN  [ENTER = 0]'
      READ(*,'(I1)',ERR=55) MENU
      IF (MENU.EQ.1) THEN
        WRITE(*,'(7A)') 'PICTURE OF YA(', TRIM(T1),
     &    '), YA(', TRIM(T2), '), YA(', TRIM(T3), ')'
        CALL UOMGEN(GFMT,GEN)
        IF (GEN.EQ.0) GOTO 51
        INFO=MENU
      ENDIF
      RETURN
C
C     ITERATIONS
C
   60 CONTINUE
      WRITE(T1,'(I10)')II
      T1=ADJUSTL(T1)
      IF (NN.GE.1.AND.TEMP.GE.1) THEN
        WRITE(T2,'(I10)')JJ
        T2=ADJUSTL(T2)
      ENDIF
      IF (NN.GE.1.AND.TEMP.EQ.2) THEN
        WRITE(T3,'(I10)')KK
        T3=ADJUSTL(T3)
      ENDIF
C
   61 WRITE(*,'(A)') 'GRAPH OF ITERATIONS:'
   65 IF (NN.EQ.1) THEN
        WRITE(*,'(3A)')'  1 = ITERATIONS OF X(', TRIM(T1),')'
      ELSEIF (TEMP.EQ.1) THEN
        WRITE(*,'(3A)')'  1 = ITERATIONS OF X(', TRIM(T1), ')'
        WRITE(*,'(3A)')'  2 = ITERATIONS OF X(', TRIM(T2), ')'
        WRITE(*,'(5A)')'  3 = ITERATIONS OF X(', TRIM(T1),
     &    '), X(', TRIM(T2),')'
      ELSE   ! TEMP.EQ.2
        WRITE(*,'(3A)')'  1 = ITERATIONS OF X(', TRIM(T1), ')'
        WRITE(*,'(3A)')'  2 = ITERATIONS OF X(', TRIM(T2), ')'
        WRITE(*,'(3A)')'  3 = ITERATIONS OF X(', TRIM(T3), ')'
        WRITE(*,'(5A)')'  4 = ITERATIONS OF X(', TRIM(T1),
     &    '), X(', TRIM(T2), ')'
        WRITE(*,'(5A)')'  5 = ITERATIONS OF X(', TRIM(T1),
     &    '), X(', TRIM(T3), ')'
        WRITE(*,'(5A)')'  6 = ITERATIONS OF X(', TRIM(T2),
     &    '), X(', TRIM(T3), ')'
        WRITE(*,'(7A)')'  7 = ITERATIONS OF X(', TRIM(T1),
     &    '), X(', TRIM(T2), '), X(', TRIM(T3), ')'
        IF (GFMT.EQ.0) WRITE(*,'(A)')'  9 = NEW INDEXES'
      ENDIF
      WRITE(*,'(A)') '  0 = RETURN  [ENTER = 0]'
      READ(*,'(I1)',ERR=65) MENU
C
      IF (GFMT.EQ.0.AND.TEMP.EQ.2.AND.MENU.EQ.9) THEN
        INFO=-1
        RETURN
      ENDIF
      IF (NN.EQ.1.AND.MENU.EQ.1.OR.
     &    NN.GT.1.AND.TEMP.EQ.1.AND.MENU.GE.1.AND.MENU.LE.3.OR.
     &    NN.GT.1.AND.TEMP.EQ.2.AND.MENU.GE.1.AND.MENU.LE.7) THEN
   66   WRITE(*,'(A)')
     &   '  TYPE OF GRAPH: P = POINTS, C = CURVE  [ENTER = C]'
        READ(*,'(A)',ERR=66) ANSWER
        IF (ANSWER.EQ.'P'.OR.ANSWER.EQ.'p') THEN
          MENU=10*MENU+1
        ELSE
          MENU=10*MENU+2
        ENDIF
        IF (NN.EQ.1.AND.MENU.EQ.11) THEN
          WRITE(*,'(3A)') 'ITERATIONS OF X(',TRIM(T1),
     &      '), GRAPH = POINTS'
        ELSEIF (NN.EQ.1.AND.MENU.EQ.12) THEN
          WRITE(*,'(3A)') 'ITERATIONS OF X(',TRIM(T1),
     &      '), GRAPH = CURVE'
        ELSEIF (NN.GT.1.AND.TEMP.EQ.1.AND.MENU.EQ.11) THEN
          WRITE(*,'(3A)') 'ITERATIONS OF X(',TRIM(T1),
     &      '), GRAPH = POINTS'
        ELSEIF (NN.GT.1.AND.TEMP.EQ.1.AND.MENU.EQ.12) THEN
          WRITE(*,'(3A)') 'ITERATIONS OF X(',TRIM(T1),
     &      '), GRAPH = CURVE'
        ELSEIF (NN.GT.1.AND.TEMP.EQ.1.AND.MENU.EQ.21) THEN
          WRITE(*,'(3A)') 'ITERATIONS OF X(',TRIM(T2),
     &      '), GRAPH = POINTS'
        ELSEIF (NN.GT.1.AND.TEMP.EQ.1.AND.MENU.EQ.22) THEN
          WRITE(*,'(3A)') 'ITERATIONS OF X(',TRIM(T2),
     &      '), GRAPH = CURVE'
        ELSEIF (NN.GT.1.AND.TEMP.EQ.1.AND.MENU.EQ.31) THEN
          WRITE(*,'(5A)') 'ITERATIONS OF X(',TRIM(T1),
     &      '), X(', TRIM(T2),'), GRAPH = POINTS'
        ELSEIF (NN.GT.1.AND.TEMP.EQ.1.AND.MENU.EQ.32) THEN
          WRITE(*,'(5A)') 'ITERATIONS OF X(',TRIM(T1),
     &      '), X(', TRIM(T2),'), GRAPH = CURVE'
        ELSEIF (MENU.EQ.11) THEN
          WRITE(*,'(3A)') 'ITERATIONS OF X(',TRIM(T1),
     &      '), GRAPH = POINTS'
        ELSEIF (MENU.EQ.12) THEN
          WRITE(*,'(3A)') 'ITERATIONS OF X(',TRIM(T1),
     &      '), GRAPH = CURVE'
        ELSEIF (MENU.EQ.21) THEN
          WRITE(*,'(3A)') 'ITERATIONS OF X(',TRIM(T2),
     &      '), GRAPH = POINTS'
        ELSEIF (MENU.EQ.22) THEN
          WRITE(*,'(3A)') 'ITERATIONS OF X(',TRIM(T2),
     &      '), GRAPH = CURVE'
        ELSEIF (MENU.EQ.31) THEN
          WRITE(*,'(3A)') 'ITERATIONS OF X(',TRIM(T3),
     &      '), GRAPH = POINTS'
        ELSEIF (MENU.EQ.32) THEN
          WRITE(*,'(3A)') 'ITERATIONS OF X(',TRIM(T3),
     &      '), GRAPH = CURVE'
        ELSEIF (MENU.EQ.41) THEN
          WRITE(*,'(5A)') 'ITERATIONS OF X(',TRIM(T1),
     &      '), X(', TRIM(T2),'), GRAPH = POINTS'
        ELSEIF (MENU.EQ.42) THEN
          WRITE(*,'(5A)') 'ITERATIONS OF X(',TRIM(T1),
     &      '), X(', TRIM(T2),'), GRAPH = CURVE'
        ELSEIF (MENU.EQ.51) THEN
          WRITE(*,'(5A)') 'ITERATIONS OF X(',TRIM(T1),
     &      '), X(', TRIM(T3),'), GRAPH = POINTS'
        ELSEIF (MENU.EQ.52) THEN
          WRITE(*,'(5A)') 'ITERATIONS OF X(',TRIM(T1),
     &      '), X(', TRIM(T3),'), GRAPH = CURVE'
        ELSEIF (MENU.EQ.61) THEN
          WRITE(*,'(5A)') 'ITERATIONS OF X(',TRIM(T2),
     &      '), X(', TRIM(T3),'), GRAPH = POINTS'
        ELSEIF (MENU.EQ.62) THEN
          WRITE(*,'(5A)') 'ITERATIONS OF X(',TRIM(T2),
     &      '), X(', TRIM(T3),'), GRAPH = CURVE'
        ELSEIF (MENU.EQ.71) THEN
          WRITE(*,'(7A)') 'ITERATIONS OF X(',TRIM(T1),
     &      '), X(', TRIM(T2),'), X(', TRIM(T3),'), GRAPH = POINTS'
        ELSE   ! MENU.EQ.72
          WRITE(*,'(7A)') 'ITERATIONS OF X(',TRIM(T1),
     &      '), X(', TRIM(T2),'), X(', TRIM(T3),'), GRAPH = CURVE'
        ENDIF
        CALL UOMGEN(GFMT,GEN)
        IF (GEN.EQ.0) GOTO 61
        INFO=MENU
      ENDIF
      RETURN
C
C     FUNCTION VALUES
C
   70 CONTINUE
      WRITE(*,'(A)')'GRAPH OF FUNCTION VALUES:'
   75 WRITE(*,'(A)')'  1 = VALUES OF F, 0 = RETURN  [ENTER = 0]'
      READ(*,'(I1)',ERR=75) MENU
      IF (MENU.EQ.1) THEN
   76   WRITE(*,'(A)')
     &    '  TYPE OF GRAPH: P = POINTS, C = CURVE  [ENTER = C]'
        READ(*,'(A)',ERR=76) ANSWER
        IF (ANSWER.EQ.'P'.OR.ANSWER.EQ.'p') THEN
          MENU=10*MENU+1
        ELSE
          MENU=10*MENU+2
        ENDIF
        IF (MENU.EQ.11) THEN
          WRITE(*,'(A)') 'VALUES OF F, GRAPH = POINTS'
        ELSE   ! MENU.EQ.12
          WRITE(*,'(A)') 'VALUES OF F, GRAPH = CURVE'
        ENDIF
        CALL UOMGEN(GFMT,GEN)
        IF (GEN.EQ.0) GOTO 70
        INFO=MENU
      ENDIF
      RETURN
C
C     ISOLINES/MAP/SURFACE
C
   80 CONTINUE
      IF (GFMT.GT.0) THEN
   85   WRITE(*,'(2A)') 'GRAPH OF:  1 = ISOLINES, 2 = MAP,',
     &   '  3 = SURFACE, 9 = NEW INDEXES, 0 = RETURN'
        WRITE(*,'(A)') '  [ 1 2 3 12 13 23 123 9, ENTER = 0 ]'
        READ(*,'(I3)',ERR=85) MENU
      ELSE
   86   WRITE(*,'(A)') 'GRAPH OF:  I = ISOLINES, M = MAP, S = SURFACE,'
        WRITE(*,'(A)')
     &    '           N = NEW INDEXES, R = RETURN  [ENTER = R]'
        READ(*,'(A)',ERR=86) ANSWER
        IF (ANSWER.EQ.'I'.OR.ANSWER.EQ.'i') THEN
          MENU=1
        ELSEIF (ANSWER.EQ.'M'.OR.ANSWER.EQ.'m') THEN
          MENU=2
        ELSEIF (ANSWER.EQ.'S'.OR.ANSWER.EQ.'s') THEN
          MENU=3
        ELSEIF (ANSWER.EQ.'N'.OR.ANSWER.EQ.'n') THEN
          MENU=9
        ELSE
          MENU=0
        ENDIF
      ENDIF
      IF (MENU.EQ.1.OR.MENU.EQ.2.OR.MENU.EQ.3.OR.GFMT.GT.0.AND.
     &  (MENU.EQ.12.OR.MENU.EQ.13.OR.MENU.EQ.23.OR.MENU.EQ.123)) THEN
      ELSEIF (MENU.EQ.9) THEN
        INFO=-1
        RETURN
      ELSE
        INFO=0
        RETURN
      ENDIF
      IF (MENU.NE.3) THEN
        WRITE(T1,'(I10)') LEVEL
        T1=ADJUSTL(T1)
   87   WRITE(*,'(3A)')
     &    '  NUMBER OF CONTOUR LEVELS:  [ENTER = ', TRIM(T1), ']'
        READ(*,'(I3)',ERR=87) I
        IF (I.GE.2) THEN
          WRITE(T1,'(I10)') I
          T1=ADJUSTL(T1)
        ENDIF
      ENDIF
   88 WRITE(*,'(A)') '  SCALE: 1 = LINEAR, 2 = LOGARITHMIC  [ENTER = 1]'
      READ(*,'(I1)',ERR=88) SCALE
      IF (SCALE.NE.2) SCALE=1
C
      IF (MENU.EQ.1.AND.SCALE.EQ.1) THEN
        WRITE(*,'(3A)') 'GRAPH OF ISOLINES, LEVEL = ', TRIM(T1),
     &    ', SCALE = LINEAR'
      ELSEIF (MENU.EQ.1.AND.SCALE.EQ.2) THEN
        WRITE(*,'(3A)') 'GRAPH OF ISOLINES, LEVEL = ', TRIM(T1),
     &    ', SCALE = LOGARITHMIC'
      ELSEIF (MENU.EQ.2.AND.SCALE.EQ.1) THEN
        WRITE(*,'(3A)') 'GRAPH OF MAP, LEVEL = ', TRIM(T1),
     &    ', SCALE = LINEAR'
      ELSEIF (MENU.EQ.2.AND.SCALE.EQ.2) THEN
        WRITE(*,'(3A)') 'GRAPH OF MAP, LEVEL = ', TRIM(T1),
     &    ', SCALE = LOGARITHMIC'
      ELSEIF (MENU.EQ.3.AND.SCALE.EQ.1) THEN
        WRITE(*,'(A)') 'GRAPH OF SURFACE, SCALE = LINEAR'
      ELSEIF (MENU.EQ.3.AND.SCALE.EQ.2) THEN
        WRITE(*,'(A)') 'GRAPH OF SURFACE, SCALE = LOGARITHMIC'
      ELSEIF (MENU.EQ.12.AND.SCALE.EQ.1) THEN
        WRITE(*,'(3A)') 'GRAPH OF ISOLINES, MAP, LEVEL = ',
     &    TRIM(T1), ', SCALE = LINEAR'
      ELSEIF (MENU.EQ.12.AND.SCALE.EQ.2) THEN
        WRITE(*,'(3A)') 'GRAPH OF ISOLINES, MAP, LEVEL = ',
     &    TRIM(T1), ', SCALE = LOGARITHMIC'
      ELSEIF (MENU.EQ.13.AND.SCALE.EQ.1) THEN
        WRITE(*,'(3A)') 'GRAPH OF ISOLINES, SURFACE, LEVEL = ',
     &    TRIM(T1), ', SCALE = LINEAR'
      ELSEIF (MENU.EQ.13.AND.SCALE.EQ.2) THEN
        WRITE(*,'(3A)') 'GRAPH OF ISOLINES, SURFACE, LEVEL = ',
     &    TRIM(T1), ', SCALE = LOGARITHMIC'
      ELSEIF (MENU.EQ.23.AND.SCALE.EQ.1) THEN
        WRITE(*,'(3A)') 'GRAPH OF MAP, SURFACE, LEVEL = ',
     &    TRIM(T1), ', SCALE = LINEAR'
      ELSEIF (MENU.EQ.23.AND.SCALE.EQ.2) THEN
        WRITE(*,'(3A)') 'GRAPH OF MAP, SURFACE, LEVEL = ',
     &    TRIM(T1), ', SCALE = LOGARITHMIC'
      ELSEIF (MENU.EQ.123.AND.SCALE.EQ.1) THEN
        WRITE(*,'(3A)') 'GRAPH OF ISOLINES, MAP, SURFACE, LEVEL = ',
     &    TRIM(T1), ', SCALE = LINEAR'
      ELSEIF (MENU.EQ.123.AND.SCALE.EQ.2) THEN
        WRITE(*,'(3A)') 'GRAPH OF ISOLINES, MAP, SURFACE, LEVEL = ',
     &    TRIM(T1), ', SCALE = LOGARITHMIC'
      ENDIF
      CALL UOMGEN(GFMT,GEN)
      IF (GEN.EQ.0) GOTO 80
      INFO=MENU
      IF (I.GE.2) LEVEL=I   ! UPDATE OF LEVEL
      RETURN
C
C     PATH WITH ISOLINES/MAP
C
   90 CONTINUE
      IF (GFMT.GT.0) THEN
        WRITE(*,'(A)') 'GRAPH OF PATH WITH:'
   93   WRITE(*,'(2A)')'  1 = ISOLINES, 2 = MAP, 0 = RETURN',
     &    '  [ 1 2 12, ENTER = 0 ]'
        READ(*,'(I2)',ERR=93) MENU
      ELSE
   94   IF (TEMP.EQ.2) THEN
          WRITE(*,'(2A)') 'PATH WITH: I = ISOLINES, M = MAP, ',
     &       'N = NEW INDEXES, R = RETURN  [ENTER = R]'
        ELSE
          WRITE(*,'(2A)') 'PATH WITH: I = ISOLINES, M = MAP, ',
     &       'R = RETURN  [ENTER = R]'
        ENDIF
        READ(*,'(A)',ERR=94) ANSWER
        IF (ANSWER.EQ.'I'.OR.ANSWER.EQ.'i') THEN
          MENU=1
        ELSEIF (ANSWER.EQ.'M'.OR.ANSWER.EQ.'m') THEN
          MENU=2
        ELSEIF (TEMP.EQ.2.AND.(ANSWER.EQ.'N'.OR.ANSWER.EQ.'n')) THEN
          MENU=-1
        ELSE
          MENU=0
        ENDIF
      ENDIF
      IF (MENU.EQ.1.OR.MENU.EQ.2.OR.GFMT.GT.0.AND.MENU.EQ.12) THEN
      ELSEIF (MENU.EQ.-1) THEN
        INFO=-1
        RETURN
      ELSE
        INFO=0
        RETURN
      ENDIF
   95 WRITE(*,'(A)')
     &  '  TYPE OF PATH: P = POINTS, C = CURVE, B = BOTH  [ENTER = B]'
      READ(*,'(A)',ERR=95) ANSWER
      IF (ANSWER.EQ.'P'.OR.ANSWER.EQ.'p') THEN
        I=1
      ELSEIF (ANSWER.EQ.'C'.OR.ANSWER.EQ.'c') THEN
        I=2
      ELSE
        I=3
      ENDIF
      MENU=10*MENU+I
      WRITE(T1,'(I10)') LEVEL
      T1=ADJUSTL(T1)
   96 WRITE(*,'(3A)')
     &  '  NUMBER OF CONTOUR LEVELS:  [ENTER = ', TRIM(T1), ']'
      READ(*,'(I3)',ERR=96) I
      IF (I.GE.2) THEN
        WRITE(T1,'(I10)') I
        T1=ADJUSTL(T1)
      ENDIF
   97 WRITE(*,'(A)')
     &  '  SCALE: 1 = LINEAR, 2 = LOGARITHMIC  [ENTER = 1]'
      READ(*,'(I1)',ERR=97) SCALE
      IF (SCALE.NE.2) SCALE=1
      WRITE(T2,'(I10)') NN
      T2=ADJUSTL(T2)
   98 WRITE(*,'(5A)') '  NUMBER OF POINTS: (MAX ',TRIM(T2),
     &  ')  [ENTER = ',TRIM(T2),']'
      READ(*,'(I3)',ERR=98) J
      IF (J.GE.1.AND.J.LT.NN) THEN
        WRITE(T2,'(I10)') J
        T2=ADJUSTL(T2)
      ENDIF
C
      IF (MENU.EQ.11.AND.SCALE.EQ.1) THEN
        WRITE(*,'(5A)') 'PATH WITH ISOLINES, POINTS, LEVEL = ',
     &    TRIM(T1), ', SCALE = LINEAR, ', TRIM(T2), ' POINTS'
      ELSEIF (MENU.EQ.11.AND.SCALE.EQ.2) THEN
        WRITE(*,'(5A)') 'PATH WITH ISOLINES, POINTS, LEVEL = ',
     &    TRIM(T1), ', SCALE = LOGARITHMIC, ', TRIM(T2), ' POINTS'
      ELSEIF (MENU.EQ.12.AND.SCALE.EQ.1) THEN
        WRITE(*,'(5A)') 'PATH WITH ISOLINES, CURVE, LEVEL = ',
     &    TRIM(T1), ', SCALE = LINEAR, ', TRIM(T2), ' POINTS'
      ELSEIF (MENU.EQ.12.AND.SCALE.EQ.2) THEN
        WRITE(*,'(5A)') 'PATH WITH ISOLINES, CURVE, LEVEL = ',
     &    TRIM(T1), ', SCALE = LOGARITHMIC, ', TRIM(T2), ' POINTS'
      ELSEIF (MENU.EQ.13.AND.SCALE.EQ.1) THEN
        WRITE(*,'(5A)') 'PATH WITH ISOLINES, POINTS+CURVE, LEVEL = ',
     &    TRIM(T1), ', SCALE = LINEAR, ', TRIM(T2), ' POINTS'
      ELSEIF (MENU.EQ.13.AND.SCALE.EQ.2) THEN
        WRITE(*,'(5A)') 'PATH WITH ISOLINES, POINTS+CURVE, LEVEL = ',
     &    TRIM(T1), ', SCALE = LOGARITHMIC, ', TRIM(T2), ' POINTS'
      ELSEIF (MENU.EQ.21.AND.SCALE.EQ.1) THEN
        WRITE(*,'(5A)') 'PATH WITH MAP, POINTS, LEVEL = ',
     &    TRIM(T1), ', SCALE = LINEAR, ', TRIM(T2), ' POINTS'
      ELSEIF (MENU.EQ.21.AND.SCALE.EQ.2) THEN
        WRITE(*,'(5A)') 'PATH WITH MAP, POINTS, LEVEL = ',
     &    TRIM(T1), ', SCALE = LOGARITHMIC, ', TRIM(T2), ' POINTS'
      ELSEIF (MENU.EQ.22.AND.SCALE.EQ.1) THEN
        WRITE(*,'(5A)') 'PATH WITH MAP, CURVE, LEVEL = ',
     &    TRIM(T1), ', SCALE = LINEAR, ', TRIM(T2), ' POINTS'
      ELSEIF (MENU.EQ.22.AND.SCALE.EQ.2) THEN
        WRITE(*,'(5A)') 'PATH WITH MAP, CURVE, LEVEL = ',
     &    TRIM(T1), ', SCALE = LOGARITHMIC, ', TRIM(T2), ' POINTS'
      ELSEIF (MENU.EQ.23.AND.SCALE.EQ.1) THEN
        WRITE(*,'(5A)') 'PATH WITH MAP, POINTS+CURVE, LEVEL = ',
     &    TRIM(T1), ', SCALE = LINEAR, ', TRIM(T2), ' POINTS'
      ELSEIF (MENU.EQ.23.AND.SCALE.EQ.2) THEN
        WRITE(*,'(5A)') 'PATH WITH MAP, POINTS+CURVE, LEVEL = ',
     &    TRIM(T1), ', SCALE = LOGARITHMIC, ', TRIM(T2), ' POINTS'
      ELSEIF (MENU.EQ.121.AND.SCALE.EQ.1) THEN
        WRITE(*,'(5A)') 'PATH WITH ISOLINES, MAP, POINTS, LEVEL = ',
     &    TRIM(T1), ', SCALE = LINEAR, ', TRIM(T2), ' POINTS'
      ELSEIF (MENU.EQ.121.AND.SCALE.EQ.2) THEN
        WRITE(*,'(5A)') 'PATH WITH ISOLINES, MAP, POINTS, LEVEL = ',
     &    TRIM(T1), ', SCALE = LOGARITHMIC, ', TRIM(T2), ' POINTS'
      ELSEIF (MENU.EQ.122.AND.SCALE.EQ.1) THEN
        WRITE(*,'(5A)') 'PATH WITH ISOLINES, MAP, CURVE, LEVEL = ',
     &    TRIM(T1), ', SCALE = LINEAR, ', TRIM(T2), ' POINTS'
      ELSEIF (MENU.EQ.122.AND.SCALE.EQ.2) THEN
        WRITE(*,'(5A)') 'PATH WITH ISOLINES, MAP, CURVE, LEVEL = ',
     &    TRIM(T1), ', SCALE = LOGARITHMIC, ', TRIM(T2), ' POINTS'
      ELSEIF (MENU.EQ.123.AND.SCALE.EQ.1) THEN
        WRITE(*,'(5A)')
     &   'PATH WITH ISOLINES, MAP, POINTS+CURVE, LEVEL = ',
     &    TRIM(T1), ', SCALE = LINEAR, ', TRIM(T2), ' POINTS'
      ELSEIF (MENU.EQ.123.AND.SCALE.EQ.2) THEN
        WRITE(*,'(5A)')
     &    'PATH WITH ISOLINES, MAP, POINTS+CURVE, LEVEL = ',
     &    TRIM(T1), ', SCALE = LOGARITHMIC, ', TRIM(T2), ' POINTS'
      ENDIF
      CALL UOMGEN(GFMT,GEN)
      IF (GEN.EQ.0) GOTO 90
      INFO=MENU
      IF (I.GE.2) LEVEL=I   ! UPDATE OF LEVEL
      IF (J.GE.1.AND.J.LT.NN) NN=J   ! UPDATE OF NUMBER OF POINTS
      RETURN
C
      $OPTION(CHEOL=';')
      END