$INTERFACE UP0BP3                                             05/12/01

  CALL UP0BP3(NF,NX,NC,$X,$IX,$XL,$XU,$XSL,$XSU,$CF,$IC,$CL,$CU,$CSL&
,$CSU,F,RPF2,RPF3,MEP4)
  $ADD(REAL,'\$X($NF)\$XL($NX)\$XU($NX)\$XSL($NX)\$XSU($NX)\$CF($NC)&
  \$CL($NC)\$CU($NC)\$CSL($NC)\$CSU($NC)')
  $ADD(INTEGER,'\$IX($NX)\$IC($NC)')
$REM  $ADD(REAL,'\F\RPF3')
$REM  $ADD(INTEGER,'\NF\NX\NC\MEP4')
  $ADD(INIT)
    NPFX='UP0BP3'
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UP0BP3             ALL SYSTEMS                 05/12/01
C PORTABILITY : ALL SYSTEMS
C 05/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* COMPUTATION OF VALUE OF THE AUGMENTED LAGRANGIAN FUNCTION.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  NX  NUMBER OF BOX CONSTRAINTS.
*  II  NC  NUMBER OF GENERAL CONSTRAINTS.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XL(NF)  VECTOR CONTAINING LOWER BOUNDS FOR VARIABLES.
*  RI  XU(NF)  VECTOR CONTAINING UPPER BOUNDS FOR VARIABLES.
*  RI  XSL(NX)  VECTOR CONTAINING LOVER SLACKS FOR VARIABLES.
*  RI  XSU(NX)  VECTOR CONTAINING UPPER SLACKS FOR VARIABLES.
*  RI  CF(NC)  VECTOR CONTAINING VALUES OF APPROXIMATED FUNCTIONS.
*  II  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  RI  CL(NC)  VECTOR CONTAINING LOWER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CU(NC)  VECTOR CONTAINING UPPER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CSL(NC)  VECTOR CONTAINING LOVER SLACKS FOR CONSTRAINTS.
*  RI  CSU(NC)  VECTOR CONTAINING UPPER SLACKS FOR CONSTRAINTS.
*  RO  F  VALUE OF THE BARRIER FUNCTION.
*  RI  RPF2  PENALTY COEFFICIENT.
*  RI  RPF3  BARRIER COEFFICIENT.
*  II  MEP4  COMPUTATION OF THE LAGRANGE MULTIPLIERS. |MEP4|=1-LAGRANGE
*         MULTIPLIERS COMPUTED FROM THE PENALTY TERM. |MEP4|=2-LAGRANGE 
*         MULTIPLIERS COMPUTED FROM THE BARRIER TERM. MEP4>0 TWO BARRIER
*         FUNCTIONS ARE USED. MEP4<0 ONE BARRIER FUNCTION IS USED.
*
*  COMMON DATA :
*  II  KBF  SPECIFICATION OF SIMPLE BOUNDS. KBF=0-NO SIMPLE BOUNDS.
*         KBF=1-ONE SIDED SIMPLE BOUNDS. KBF=2=TWO SIDED SIMPLE BOUNDS.
*  II  KBC  SPECIFICATION OF LINEAR CONSTRAINTS. KBC=0-NO LINEAR
*         CONSTRAINTS. KBC=1-ONE SIDED LINEAR CONSTRAINTS. KBC=2=TWO
*         SIDED LINEAR CONSTRAINTS.
*
      SUBROUTINE UP0BP3(NF,NX,NC,X,IX,XL,XU,XSL,XSU,CF,IC,CL,CU,CSL,
     & CSU,F,RPF2,RPF3,MEP4)
      $INCLUDE ('UMCOMM')
      INTEGER NF,NX,NC,IX(NX),IC(NC),MEP4
      $FLOAT X(NF),XL(NX),XU(NX),XSL(NX),XSU(NX),CF(NC),CL(NC),CU(NC),
     & CSL(NC),CSU(NC),F,RPF2,RPF3
      $FLOAT POM
      INTEGER K,KK
      POM=RPF2*RPF3**(RPF2-ONE)
      IF (KBF.GT.0) THEN
      DO 1 K=1,NX
      KK=ABS(IX(K))
      IF (KK.EQ.1.OR.KK.EQ.3.OR.KK.EQ.4) THEN
      IF (XL(K)-X(K).GE.ZERO.OR.MEP4.LE.0) THEN
      F=F-RPF3*LOG(XSL(K))+(XL(K)-X(K)+XSL(K))**RPF2/POM
      ELSE
      F=F-RPF3*LOG(X(K)-XL(K)+XSL(K))+XSL(K)**RPF2/POM
      ENDIF
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.3.OR.KK.EQ.4) THEN
      IF (X(K)-XU(K).GE.ZERO.OR.MEP4.LE.0) THEN
      F=F-RPF3*LOG(XSU(K))+(X(K)-XU(K)+XSU(K))**RPF2/POM
      ELSE
      F=F-RPF3*LOG(XU(K)-X(K)+XSU(K))+XSU(K)**RPF2/POM
      ENDIF
      ENDIF
    1 CONTINUE
      ENDIF
      IF (KBC.GT.0) THEN
      DO 2 K=1,NC
      KK=ABS(IC(K))
      IF (KK.EQ.1.OR.KK.EQ.3.OR.KK.EQ.4) THEN
      IF (CL(K)-CF(K).GE.ZERO.OR.MEP4.LE.0) THEN
      F=F-RPF3*LOG(CSL(K))+(CL(K)-CF(K)+CSL(K))**RPF2/POM
      ELSE
      F=F-RPF3*LOG(CF(K)-CL(K)+CSL(K))+CSL(K)**RPF2/POM
      ENDIF
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.3.OR.KK.EQ.4) THEN
      IF (CF(K)-CU(K).GE.ZERO.OR.MEP4.LE.0) THEN
      F=F-RPF3*LOG(CSU(K))+(CF(K)-CU(K)+CSU(K))**RPF2/POM
      ELSE
      F=F-RPF3*LOG(CU(K)-CF(K)+CSU(K))+CSU(K)**RPF2/POM
      ENDIF
      ENDIF
    2 CONTINUE
      ENDIF
      RETURN
      END

