$INTERFACE UPLAG4                                             05/12/01

  CALL UPLAG4(NF,NX,NC,$X,$IX,$XL,$XU,$XSL,$XSU,$XZL,$XZU,$CF,$IC,$CL&
,$CU,$CSL,$CSU,$CZL,$CZU,RPF3)
  $ADD(REAL,'\$X($NF)\$XL($NX)\$XU($NX)\$XSL($NX)\$XSU($NX)\$XZL($NX)&
  \$XZU($NX)\$CF($NC)\$CL($NC)\$CU($NC)\$CSL($NC)\$CSU($NC)\$CZL($NC)&
  \$CZU($NC)')
  $ADD(INTEGER,'\$IX($NX)\$IC($NC)')
$REM  $ADD(REAL,'\RPF3')
$REM  $ADD(INTEGER,'\NF\NX\NC')
  $ADD(INIT)
    NPFX='UPLAG4'
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UPLAG4             ALL SYSTEMS                 05/12/01
C PORTABILITY : ALL SYSTEMS
C 05/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* COMPUTATION OF VALUE OF THE AUGMENTED LAGRANGIAN FUNCTION.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  NX  NUMBER OF BOX CONSTRAINTS.
*  II  NC  NUMBER OF GENERAL CONSTRAINTS.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XL(NF)  VECTOR CONTAINING LOWER BOUNDS FOR VARIABLES.
*  RI  XU(NF)  VECTOR CONTAINING UPPER BOUNDS FOR VARIABLES.
*  RI  XSL(NX)  VECTOR CONTAINING LOVER SLACKS FOR VARIABLES.
*  RI  XSU(NX)  VECTOR CONTAINING UPPER SLACKS FOR VARIABLES.
*  RI  XZL(NF)  VECTOR CONTAINING LOVER MULTIPLIERS FOR VARIABLES.
*  RI  XZU(NF)  VECTOR CONTAINING UPPER MULTIPLIERS FOR VARIABLES.
*  RI  CF(NC)  VECTOR CONTAINING VALUES OF APPROXIMATED FUNCTIONS.
*  II  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  RI  CL(NC)  VECTOR CONTAINING LOWER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CU(NC)  VECTOR CONTAINING UPPER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CSL(NC)  VECTOR CONTAINING LOVER SLACKS FOR CONSTRAINTS.
*  RI  CSU(NC)  VECTOR CONTAINING UPPER SLACKS FOR CONSTRAINTS.
*  RI  CZL(NC)  VECTOR CONTAINING LOVER MULTIPLIERS FOR CONSTRAINTS.
*  RI  CZU(NC)  VECTOR CONTAINING UPPER MULTIPLIERS FOR CONSTRAINTS.
*  RI  RPF3  BARRIER COEFFICIENT.
*
*  COMMON DATA :
*  II  KBF  SPECIFICATION OF SIMPLE BOUNDS. KBF=0-NO SIMPLE BOUNDS.
*         KBF=1-ONE SIDED SIMPLE BOUNDS. KBF=2=TWO SIDED SIMPLE BOUNDS.
*  II  KBC  SPECIFICATION OF LINEAR CONSTRAINTS. KBC=0-NO LINEAR
*         CONSTRAINTS. KBC=1-ONE SIDED LINEAR CONSTRAINTS. KBC=2=TWO
*         SIDED LINEAR CONSTRAINTS.
*
      SUBROUTINE UPLAG4(NF,NX,NC,X,IX,XL,XU,XSL,XSU,XZL,XZU,CF,IC,CL,
     & CU,CSL,CSU,CZL,CZU,RPF3)
      $INCLUDE ('UMCOMM')
      INTEGER NF,NX,NC,IX(NX),IC(NC)
      $FLOAT X(NF),XL(NX),XU(NX),XSL(NX),XSU(NX),XZL(NX),XZU(NX),
     & CF(NC),CL(NC),CU(NC),CSL(NC),CSU(NC),CZL(NC),CZU(NC),RPF3
      $FLOAT POM,TMP1,TMP2,DIS
      INTEGER K,KK
      $FLOAT CON
      PARAMETER(CON=1.0$P 0/3.0$P 0)
      IF (KBF.GT.0) THEN
      DO 1 K=1,NX
      KK=ABS(IX(K))
      IF (KK.EQ.1.OR.KK.EQ.3.OR.KK.EQ.4) THEN
      POM=XL(K)-X(K)
      TMP1=HALF*RPF3**2
      TMP2=CON*POM
      DIS=TMP1**2-TMP2**3
      IF (DIS.GE.ZERO) THEN
      DIS=SQRT(DIS)
      XZL(K)=(TMP1+DIS)**CON+SIGN(ABS(TMP1-DIS)**CON,TMP1-DIS)
      XSL(K)=MAX(1.0D-120,XZL(K)**2-POM)
      ELSE
      DIS=ACOS(TMP1/(TMP2*SQRT(TMP2)))
      XZL(K)=TWO*SQRT(TMP2)*COS(DIS/THREE)
      XSL(K)=MAX(1.0D-120,XZL(K)**2-POM)
      ENDIF
      XZL(K)=XZL(K)/RPF3
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.3.OR.KK.EQ.4) THEN
      POM=X(K)-XU(K)
      TMP1=HALF*RPF3**2
      TMP2=CON*POM
      DIS=TMP1**2-TMP2**3
      IF (DIS.GE.ZERO) THEN
      DIS=SQRT(DIS)
      XZU(K)=(TMP1+DIS)**CON+SIGN(ABS(TMP1-DIS)**CON,TMP1-DIS)
      XSU(K)=MAX(1.0D-120,XZU(K)**2-POM)
      ELSE
      DIS=ACOS(TMP1/(TMP2*SQRT(TMP2)))
      XZU(K)=TWO*SQRT(TMP2)*COS(DIS/THREE)
      XSU(K)=MAX(1.0D-120,XZU(K)**2-POM)
      ENDIF
      XZU(K)=XZU(K)/RPF3
      ENDIF
    1 CONTINUE
      ENDIF
      IF (KBC.GT.0) THEN
      DO 2 K=1,NC
      KK=ABS(IC(K))
      IF (KK.EQ.1.OR.KK.EQ.3.OR.KK.EQ.4) THEN
      POM=CL(K)-CF(K)
      TMP1=HALF*RPF3**2
      TMP2=CON*POM
      DIS=TMP1**2-TMP2**3
      IF (DIS.GE.ZERO) THEN
      DIS=SQRT(DIS)
      CZL(K)=(TMP1+DIS)**CON+SIGN(ABS(TMP1-DIS)**CON,TMP1-DIS)
      CSL(K)=MAX(1.0D-120,CZL(K)**2-POM)
      ELSE
      DIS=ACOS(TMP1/(TMP2*SQRT(TMP2)))
      CZL(K)=TWO*SQRT(TMP2)*COS(DIS/THREE)
      CSL(K)=MAX(1.0D-120,CZL(K)**2-POM)
      ENDIF
      CZL(K)=CZL(K)/RPF3
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.3.OR.KK.EQ.4) THEN
      POM=CF(K)-CU(K)
      TMP1=HALF*RPF3**2
      TMP2=CON*POM
      DIS=TMP1**2-TMP2**3
      IF (DIS.GE.ZERO) THEN
      DIS=SQRT(DIS)
      CZU(K)=(TMP1+DIS)**CON+SIGN(ABS(TMP1-DIS)**CON,TMP1-DIS)
      CSU(K)=MAX(1.0D-120,CZU(K)**2-POM)
      ELSE
      DIS=ACOS(TMP1/(TMP2*SQRT(TMP2)))
      CZU(K)=TWO*SQRT(TMP2)*COS(DIS/THREE)
      CSU(K)=MAX(1.0D-120,CZU(K)**2-POM)
      ENDIF
      CZU(K)=CZU(K)/RPF3
      ENDIF
    2 CONTINUE
      ENDIF
      RETURN
      END

