$INTERFACE UPSEB1                                             99/01/23

  CALL UPSEB1(NX,$NN,$IX,$XSL,$XSU,$XZL,$XZU,$IC,$CSL,$CSU,$CZL,$CZU&
,$ETA9,RPF3)
 $EXIT

* IMPLEMENTATION

* SUBROUTINE UPSEB1            ALL SYSTEMS                 98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 MC : ORIGINAL VERSION
*
* PURPOSE :
* BARRIER PARAMETER DETERMINATION (R.J.VANDERBEI AND F.SHANNO
* ALGORITHM).
*
* PARAMETERS :
*  II  NX  NUMBER OF BOX CONSTRAINTS.
*  II  NC  NUMBER OF GENERAL CONSTRAINTS.
*  II  IX(NX)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XSL(NX)  VECTOR CONTAINING LOVER SLACKS FOR VARIABLES.
*  RI  XSU(NX)  VECTOR CONTAINING UPPER SLACKS FOR VARIABLES.
*  RI  XZL(NX)  VECTOR CONTAINING LOVER MULTIPLIERS FOR VARIABLES.
*  RI  XZU(NX)  VECTOR CONTAINING UPPER MULTIPLIERS FOR VARIABLES.
*  II  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  RI  CSL(NC)  VECTOR CONTAINING LOVER SLACKS FOR CONSTRAINTS.
*  RI  CSU(NC)  VECTOR CONTAINING UPPER SLACKS FOR CONSTRAINTS.
*  RI  CZL(NC)  VECTOR CONTAINING LOVER MULTIPLIERS FOR CONSTRAINTS.
*  RI  CZU(NC)  VECTOR CONTAINING UPPER MULTIPLIERS FOR CONSTRAINTS.
*  RI  ETA9  MAXIMUM MAXHINE NUMBER.
*  RI  RPF3  BARRIER COEFFICIENT.
*
*  COMMON DATA :
*  II  KBF  SPECIFICATION OF SIMPLE BOUNDS. KBF=0-NO SIMPLE BOUNDS.
*         KBF=1-ONE SIDED SIMPLE BOUNDS. KBF=2=TWO SIDED SIMPLE BOUNDS.
*  II  KBC  SPECIFICATION OF LINEAR CONSTRAINTS. KBC=0-NO LINEAR
*         CONSTRAINTS. KBC=1-ONE SIDED LINEAR CONSTRAINTS. KBC=2=TWO
*         SIDED LINEAR CONSTRAINTS.
*
      SUBROUTINE UPSEB1(NX,NC,IX,XSL,XSU,XZL,XZU,IC,CSL,CSU,CZL,CZU,
     & ETA9,RPF3)
      $INCLUDE ('UMCOMM')
      INTEGER NX,NC,IX(NX),IC(NC)
      $FLOAT XSL(NX),XSU(NX),XZL(NX),XZU(NX),CSL(NC),CSU(NC),CZL(NC),
     & CZU(NC),ETA9,RPF3
      $FLOAT STEPL,SCALEF,CDOTMIN,CDOT1,SXI
      INTEGER I,K,IIC
      PARAMETER(STEPL=9.5$P-1,SCALEF=1.0$P-1)
      IIC=0
      CDOT1=ZERO
      CDOTMIN=ETA9
      IF (KBF.GT.0) THEN
        DO 2 I=1,NX
          K=ABS(IX(I))
          IF (K.EQ.1.OR.K.EQ.3.OR.K.EQ.4) THEN
            SXI=XSL(I)*ABS(XZL(I))
            IF (SXI.LT.CDOTMIN) THEN
              CDOTMIN=SXI
            ENDIF
            CDOT1=CDOT1+SXI
            IIC=IIC+1
          ENDIF
          IF (K.EQ.2.OR.K.EQ.3.OR.K.EQ.4) THEN
            SXI=XSU(I)*ABS(XZU(I))
            IF (SXI.LT.CDOTMIN) THEN
              CDOTMIN=SXI
            ENDIF
            CDOT1=CDOT1+SXI
            IIC=IIC+1
          ENDIF
    2   CONTINUE
      ENDIF
      IF (KBC.GT.0) THEN
        DO 3 I=1,NC
          K=ABS(IC(I))
          IF (K.EQ.1.OR.K.EQ.3.OR.K.EQ.4) THEN
            SXI=CSL(I)*ABS(CZL(I))
            IF (SXI.LT.CDOTMIN) THEN
              CDOTMIN=SXI
            ENDIF
            CDOT1=CDOT1+SXI
            IIC=IIC+1
          ENDIF
          IF (K.EQ.2.OR.K.EQ.3.OR.K.EQ.4) THEN
            SXI=CSU(I)*ABS(CZU(I))
            IF (SXI.LT.CDOTMIN) THEN
              CDOTMIN=SXI
            ENDIF
            CDOT1=CDOT1+SXI
            IIC=IIC+1
          ENDIF
    3   CONTINUE
      ENDIF
      IF (IIC.EQ.0) THEN
        RPF3=ZERO
        RETURN
      ENDIF
      CDOT1=CDOT1/$DBLE(IIC)
      SXI=CDOTMIN/CDOT1
      CDOTMIN=MIN((ONE-STEPL)*(ONE-SXI)/SXI,TWO)
C      CDOTMIN=ONE
      RPF3=SCALEF*CDOT1*CDOTMIN**3
      END
