$INTERFACE US0G04                                             97/12/01

  $HELP
    STRATEGY FOR STEP DECREASING
      1 - FLETCHER
      2 - POWELL
      3 - SAFEGUARDED INTERPOLATION
  $CHECK(MES1,3,'1\2\3')
  $HELP
    STRATEGY FOR STEP INCREASING
      1 - FLETCHER
      2 - POWELL
  $CHECK(MES2,2,'1\2')
  $HELP
    FIRST TOLERANCE FOR RATIO DF/DFPRED. STEP BOUND IS
    DECREASED IF DF/DFPRED<EPS3
  $CHECK(EPS3,'0.10$P 0',REAL,4)
  $HELP
    SECOND TOLERANCE FOR RATIO DF/DFPRED. STEP BOUND IS
    INCREASED IF IT IS ACTIVE AND DF/DFPRED>EPS4
  $CHECK(EPS4,'0.90$P 0',REAL,4)
  $HELP
    LOWER BOUND FOR STEPSIZE REDUCTION
  $CHECK(BET1,'0.05$P 0',REAL,4)
  $DEFAULT='0.50$P 0'
  $IF(MES1=3) $DEFAULT='0.75$P 0'
  $HELP
    UPPER BOUND FOR STEPSIZE REDUCTION
  $CHECK(BET2,DEFAULT,REAL,4)
  $HELP
    LOWER BOUND FOR STEPSIZE EXPANSION
  $CHECK(GAM1,'2.0$P 0',REAL,4)
  $HELP
    UPPER BOUND FOR STEPSIZE EXPANSION
  $CHECK(GAM2,'1.0$P 6',REAL,4)
  $HELP
    MAXIMUM STEPSIZE
  $CHECK(XMAX,'1.0$P 3',REAL,2)
  $HELP
    MAXIMUM NUMBER OF ZERO STEPS IN ONE ITERATION
  $CHECK(MRED,10,INT,4)
  CALL US0G04(EPS3,EPS4,BET1,BET2,GAM1,GAM2,RO,R,FO,F,PO,PP,XDEL&
,XDELO,XMAX,SNORM,PAR1,RMAX,KTERS,MES1,MES2,MRED,$KDS)
$REM  $ADD(REAL,'\EPS3\EPS4\BET1\BET2\GAM1\GAM2\RO\R\FO\F\PO\PP\XDEL&
$REM  \XDELO\XMAX\SNORM\PAR1\RMAX')
$REM  $ADD(INTEGER,'\KTERS\MES1\MES2\MRED')
  $ADD(INIT)
    NSXX='US0G04'
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE US0G04                ALL SYSTEMS                97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* LINE SEARCH WITH TRUST REGION UPDATE.
*
* PARAMETERS :
*  RI  EPS3  FIRST TOLERANCE FOR RATIO DF/DFPRED. STEP BOUND IS
*         DECREASED IF DF/DFPRED<EPS3.
*  RI  EPS4  SECOND TOLERANCE FOR RATIO DF/DFPRED. STEP BOUND IS
*         INCREASED IF IT IS ACTIVE AND DF/DFPRED>EPS4.
*  RI  BET1  LOWER BOUND FOR STEPSIZE REDUCTION.
*  RI  BET2  UPPER BOUND FOR STEPSIZE REDUCTION.
*  RI  GAM1  LOWER BOUND FOR STEPSIZE EXPANSION.
*  RI  GAM2  UPPER BOUND FOR STEPSIZE EXPANSION.
*  RO  RO  INITIAL VALUE OF THE STEPSIZE PARAMETER.
*  RO  R  VALUE OF THE STEPSIZE PARAMETER.
*  RI  FO  INITIAL VALUE OF THE OBJECTIVE FUNCTION.
*  RO  F  VALUE OF THE OBJECTIVE FUNCTION.
*  RI  PO  INITIAL VALUE OF THE DIRECTIONAL DERIVATIVE.
*  RI  PP  QUADRATIC PART OF THE PREDICTED FUNCTION VALUE.
*  RU  XDEL  TRUST REGION BOUND.
*  RO  XDELO  PREVIOUS TRUST REGION BOUND.
*  RI  XMAX MAXIMUM STEPSIZE.
*  RI  SNORM  EUCLIDEAN NORM OF THE DIRECTION VECTOR.
*  RI  RMAX  MAXIMUM VALUE OF THE STEPSIZE PARAMETER.
*  RA  PAR1  AUXILIARY VARIABLE FOR CONTROLLED SCALING OF VARIABLE
*            METRIC UPDATES.
*  II  KTERS  TERMINATION SELECTION. KTERS=1-NORMAL TERMINATION.
*         KTERS=6-FIRST STEPSIZE.
*  II  MES1  SWITCH FOR EXTRAPOLATION. MES1=1-CONSTANT INCREASING OF
*         THE INTERVAL. MES1=2-EXTRAPOLATION SPECIFIED BY THE PARAMETER
*         MES. MES1=3 SUPPRESSED EXTRAPOLATION.
*  II  MES2  SWITCH FOR TERMINATION. MES2=1-NORMAL TERMINATION.
*         MES2=2-TERMINATION AFTER AT LEAST TWO STEPS (ASYMPTOTICALLY
*         PERFECT LINE SEARCH).
*  II  MRED  MAXIMUM NUMBER OF EXTRAPOLATIONS OR INTERPOLATIONS.
*  II  KDS  DEGREE OF COMPUTED DERIVATIVES.
*
* COMMON DATA :
*  IO  MAXST  MAXIMUM STEPSIZE INDICATOR. MAXST=0 OR MAXST=1 IF MAXIMUM
*         STEPSIZE WAS NOT OR WAS REACHED.
*  IU  IDIR INDICATOR FOR DIRECTION DETERMINATION.
*         IDIR=0-BASIC DETERMINATION. IDIR=1-DETERMINATION
*         AFTER STEPSIZE REDUCTION. IDIR=2-DETERMINATION AFTER
*         STEPSIZE EXPANSION.
*  IO  NRED  ACTUAL NUMBER OF EXTRAPOLATIONS OR INTERPOLATIONS.
*  IO  ITERS  TERMINATION INDICATOR. ITERS=0-ZERO STEP. ITERS=1-STEP
*         BOUND WAS DECREASED. ITERS=2-STEP BOUND WAS UNCHANGED.
*         ITERS=3-STEP BOUND WAS INCREASED. ITERS=6-FIRST STEPSIZE.
*  II  ITERD TERMINATION INDICATOR. ITERD<0-BAD DECOMPOSITION.
*         ITERD=0-DESCENT DIRECTION. ITERD=1-NEWTON LIKE STEP.
*         ITERD=2-INEXACT NEWTON LIKE STEP. ITERD=3-BOUNDARY STEP.
*         ITERD=4-DIRECTION WITH THE NEGATIVE CURVATURE.
*         ITERD=5-MARQUARDT STEP.
*  TO  TXFU  TEXT INFORMATION ON THE CAUSE OF TERMINATION.
*  TO  TSXX  TEXT INFORMATION ON THE STEPSIZE OBTAINED.
*
* STATUS VARIABLES :
*  NS,ISP,TSS
*
* SUBPROGRAM USED :
*  S   UYPRO1  PROLOGUE.
*  S   UYEPI1  EPILOGUE.
*  S   UYSET0  STATUS DEFINITION.
*  S   UYSET2  DEGREE DEFINITION OF THE COMPUTED DERIVATIVES.
*  S   UOS0G1  PRINT OF ENTRY TO STEPSIZE SELECTION.
*  S   UOS0G2  PRINT OF EXIT FROM STEPSIZE SELECTION.
*  S   UOS0G3  PRINT OF INFORMATION DURING STEPSIZE SELECTION.
*  S   UOERR1  ERROR MESSAGES.
*
* METHOD :
* G.A.SCHULTZ, R.B.SCHNABEL, R.H.BYRD: A FAMILY OF TRUST-REGION-BASED
* ALGORITHMS FOR UNCONSTRAINED MINIMIZATION WITH STRONG GLOBAL
* CONVERGENCE PROPERTIES, SIAM J. NUMER.ANAL. 22 (1985) PP. 47-67.
*
      SUBROUTINE US0G04(EPS3,EPS4,BET1,BET2,GAM1,GAM2,RO,R,FO,F,PO,PP,
     & XDEL,XDELO,XMAX,SNORM,PAR1,RMAX,KTERS,MES1,MES2,MRED,KDS)
      $INCLUDE('UMCOMM')
      INTEGER KTERS,MES1,MES2,MRED,KDS
      $FLOAT EPS3,EPS4,BET1,BET2,GAM1,GAM2,RO,R,FO,F,PO,PP,XDEL,
     & XDELO,XMAX,SNORM,PAR1,RMAX
      $FLOAT DF,DFPRED
      $FLOAT C2L
      PARAMETER (C2L=1.0$P-2)
      IF (TSS(NS).NE.'USXX') CALL UYPRO1('USXX',KDS)
      GO TO (1,2,4) ISP(NS)
    1 CONTINUE
      CALL UOS0G1(RO,FO,XDEL,RMAX)
      NRED=0
      IDIR=0
      PAR1=ONE
      XDELO=XDEL
C
C     COMPUTATION OF THE NEW FUNCTION VALUE
C
      R=MIN(ONE,RMAX)
      CALL UYSET2
      CALL UYSET0(1,2)
      RETURN
    2 CONTINUE
      IF(KTERS.LT.0.OR.KTERS.GT.5) THEN
      ITERS=6
      TSXX='FIRST ST'
      GO TO 5
      ENDIF
      IF (FO.LE.F) GO TO 3
      DF=FO-F
      DFPRED=-R*(PO+R*PP)
      IF (DF.LT.EPS3*DFPRED) THEN
C
C     STEP IS TOO LARGE, IT HAS TO BE REDUCED
C
      IF (MES1.EQ.1) THEN
      XDEL=BET2*SNORM
      ELSE IF (MES1.EQ.2) THEN
      XDEL=BET2*MIN(HALF*XDEL,SNORM)
      ELSE
      XDEL=HALF*PO*SNORM/(PO+DF)
      XDEL=MAX(XDEL,BET1*SNORM)
      XDEL=MIN(XDEL,BET2*SNORM)
      ENDIF
      ITERS=1
      TSXX='LARGE   '
      ELSE IF (DF.LE.EPS4*DFPRED) THEN
C
C     STEP IS SUITABLE
C
      ITERS=2
      TSXX='GOOD    '
      ELSE
C
C     STEP IS TOO SMALL, IT HAS TO BE ENLARGED
C
      IF (MES2.EQ.2) THEN
      XDEL=MAX(XDEL,GAM1*SNORM)
      ELSE IF (ITERD.GT.2) THEN
      XDEL=GAM1*XDEL
      ENDIF
      ITERS=3
      TSXX='SMALL   '
      ENDIF
      XDEL=MIN(XDEL,XMAX,GAM2*SNORM)
      GO TO 5
    3 R=R*MAX(C2L,HALF/(ONE+(FO-F)/(R*PO)))
      CALL UYSET2
      CALL UYSET0(1,3)
      RETURN
    4 CONTINUE
      IF (FO.LE.F) THEN
      IF (NRED.GE.MRED) THEN
      ITERS=-1
      TSXX='MAX  RED'
      CALL UOERR1('US0G04',5-ITERS)
      GO TO 5
      ELSE
      NRED=NRED+1
      GO TO 3
      ENDIF
      ELSE
C
C     STEP IS TOO LARGE, IT HAS TO BE REDUCED
C
      XDEL=R*SNORM
      ITERS=1
      TSXX='SEARCH  '
      ENDIF
    5 MAXST=0
      IF (XDEL.GE.XMAX) MAXST=1
      CALL UOS0G3(R,F,FO,PO,PP,XDEL,SNORM)
      CALL UOS0G2(R,F,XDEL)
      CALL UYEPI1(0)
      RETURN
      END

