$INTERFACE US1L10                                             08/12/01

  $HELP
    TERMINATION TOLERANCE FOR LINE SEARCH
    (IN TEST ON THE CHANGE OF THE FUNCTION VALUE)
  $CHECK(EPS1,'1.0$P-1',REAL,4)
  $HELP
    TERMINATION TOLERANCE FOR LINE SEARCH
    (IN TEST ON THE CHANGE OF THE DIRECTIONAL DERIVATES)
  $CHECK(EPS2,'0.9$P 0',REAL,4)
  $HELP
    MAXIMUM NUMBER OF EXTRAPOLATIONS OR INTERPOLATIONS
  $CHECK(MRED,50,INT,4)
  CALL US1L10(NF,$XO,$S,$X,$G,R,F,FO,T,P,PP,TL,EL,PL,TU,EU,PU,PPU,FUB&
,GNORM,RMAX,EPS1,EPS2,MRED,IER,IUP,L1)
  $ADD(REAL,'\PL\PU\PPU')
$REM  $ADD(REAL,'\R\F\FO\T\P\PP\TL\EL\TU\EU\FUB\GNORM\RMAX\EPS1\EPS2')
  $ADD(INTEGER,'\IUP')
$REM  $ADD(INTEGER,'\IER\MRED')
  $ADD(LOGICAL,'\L1')
  $ADD(INIT)
    NSXX='US1L10'
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE US1L10                ALL SYSTEMS                97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 HZ : ORIGINAL VERSION
*
* PURPOSE :
*  STANDARD LINE SEARCH WITH DIRECTIONAL DERIVATIVES.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  RO  XO(NF)  SAVED VECTOR OF VARIABLES.
*  RO  S(NF)  DIRECTION VECTOR.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  RI  G(NF)  GRADIENT OF THE OBJECTIVE FUNCTION.
*  RO  R  VALUE OF THE STEPSIZE PARAMETER.
*  RO  F  VALUE OF THE OBJECTIVE FUNCTION.
*  RI  FO  INITIAL VALUE OF THE OBJECTIVE FUNCTION.
*  RI  T  .
*  RO  P  VALUE OF THE DIRECTIONAL DERIVATIVE.
*  RO  PP  PREVIOUS VALUE OF THE DIRECTIONAL DERIVATIVE.
*  RU  TL  .
*  RU  EL  .
*  RU  PL  .
*  RU  TU  .
*  RU  EU  .
*  RU  PU  .
*  RU  PPU  .
*  RU  FUB  .
*  RI  GNORM  NORM OF THE GRADIENT VECTOR.
*  RI  RMAX  MAXIMUM VALUE OF THE STEPSIZE PARAMETER
*  RI  EPS1  TERMINATION TOLERANCE FOR LINE SEARCH (IN TEST ON THE
*         CHANGE OF THE FUNCTION VALUE).
*  RI  EPS2  TERMINATION TOLERANCE FOR LINE SEARCH (IN TEST ON THE
*         CHANGE OF THE DIRECTIONAL DERIVATIVE).
*  II  MRED  MAXIMUM NUMBER OF EXTRAPOLATIONS OR INTERPOLATIONS.
*  II  IER  .
*  II  IUP  .
*  LI  L1  .
*
* COMMON DATA :
*  II  KIT  NUMBER OF THE ITERATION AFTER LAST RESTART.
*  IO  NRED  ACTUAL NUMBER OF EXTRAPOLATIONS OR INTERPOLATIONS.
*  II  NIT  ACTUAL NUMBER OF ITERATIONS.
*
* STATUS VARIABLES :
*  NS,ISP,TSS
*
* SUBPROGRAM USED :
*  S   UYPRO1  PROLOGUE.
*  S   UYEPI1  EPILOGUE.
*  S   UYSET0  STATUS DEFINITION.
*  S   UYSET2  DEGREE DEFINITION OF THE COMPUTED DERIVATIVES.
*  S   UXVDIR  VECTOR AUGMENTED BY THE SCALED VECTOR.
*
* METHOD :
* SPECIAL LINE SEARCH OF HAGER AND ZHANG.
*
      SUBROUTINE US1L10(NF,XO,S,X,G,R,F,FO,T,P,PP,TL,EL,PL,TU,EU,PU,
     & PPU,FUB,GNORM,RMAX,EPS1,EPS2,MRED,IER,IUP,L1)
      $INCLUDE('UMCOMM')
      INTEGER NF,MRED,IER,IUP
      $FLOAT XO(NF),S(NF),X(NF),G(NF),R,F,FO,T,P,PP,TL,EL,PL,TU,EU,PU,
     & PPU,FUB,GNORM,RMAX,EPS1,EPS2
      LOGICAL L1,L2,L3,L4,L5,L6
      SAVE L2,L3,L4,L5,L6 
      INTEGER NGR,NSH
      SAVE NGR,NSH
      $FLOAT XMAX,GMAX,UXVDOT,RP,CK,QK,DEL,PO,TR,DEN,TO,TLO,ELO,TUO,EUO,
     & RW,FQ,FP,UXVMAX
      SAVE RP,CK,QK,DEL,PO,TR,TO,TLO,ELO,TUO,EUO,RW,FQ,FP
      $FLOAT CON1,CON2,CON3,CON4,CON5,CON6,CON7,CON8,CON9
      PARAMETER (CON1=1.0D-6,CON2=1.0D-3,CON3=6.6D-1,CON4=7.0D-1,
     & CON5=5.0D 0,CON6=1.0D-1,CON7=2.0D 0,CON8=1.0D-2,CON9=1.0D-12)
      IF (TSS(NS).NE.'USXX') CALL UYPRO1('USXX',1)
      GO TO (1,2,3,4,5,6,7,8,9,10,11) ISP(NS)
    1 CONTINUE
      IF (NIT.EQ.1) THEN
C
C     INITIATION
C
        L1 = .FALSE.
        L2 = .FALSE.
        L3 = .TRUE.
        L5 = .TRUE.
        IER = 0
        CK = ZERO
        QK = ZERO
        DEL = TWO*EPS1 - ONE
        GMAX = UXVMAX(NF,G)
        XMAX = UXVMAX(NF,XO)
        R = CON8*XMAX/GMAX
        IF (XMAX.EQ.ZERO) THEN
          IF (FO.NE.ZERO) THEN
            R = CON8*ABS(FO)/GNORM
          ELSE
            R = ONE
          ENDIF
        ENDIF
        T = FO + FO
      ENDIF
      IF (KIT.EQ.NIT) THEN
        PO = -GNORM**2
      ELSE
        PO = UXVDOT(NF,G,S)
      ENDIF
      L4 = .FALSE.
      R = CON7*R
      IF (.NOT.L3) GO TO 700
      IF (FO.NE.ZERO) THEN
        RP = ABS((FO-T)/FO)
      ELSE
        RP = ONE
      ENDIF
      IF (RP.LE.CON9) GO TO 700
      TR = MIN(RMAX,CON6*R)
      CALL UXVDIR(NF,TR,S,XO,X)
      KD=0
      CALL UYSET2
      CALL UYSET0(1,2)
      RETURN
    2 CONTINUE
      IF (F.LT.FO) THEN
        DEN = TWO*(((F-FO)/TR)-PO)
        IF (DEN.GT.ZERO) THEN
          L4 = .TRUE.
          R = -PO*TR/DEN
        ENDIF
      ENDIF
700   CONTINUE
      R = MIN(R,RMAX)
      F = FO
      T = FO
      QK = CON4*QK + ONE
      CK = CK + (ABS(FO) - CK)/QK
      IF (L5) THEN
        FUB = FO + CON1*CK
      ELSE
        FUB = FO + CON1
      ENDIF
      PU = EPS1*PO
      PL = EPS2*PO
      PPU = DEL*PO
      CALL UXVDIR(NF,R,S,XO,X)
      KD=1
      LD=0
      CALL UYSET0(1,3)
      RETURN
    3 CONTINUE
      P = UXVDOT(NF,G,S)
      PP = P
      EL = PO
      IF (.NOT.L1) THEN
        PP = PP - PU
        EL = EL - PU
      ENDIF
C
C     FIND INITIAL INTERVAL [TL,TU] SUCH THAT EL < 0, EU >= 0
C
      TL = ZERO
      NGR = 0
      NSH = 0
750   CONTINUE   
      IF (PP.GE.ZERO) GO TO 100
      KD=0
      CALL UYSET2
      CALL UYSET0(1,4)
      RETURN
    4 CONTINUE
      FP = F
      IF (.NOT.L1) FP = FP - R*PU
C
C     IF QUADSTEP IN EFFECT AND QUADRATIC CONDITIONS HOLD, CHECK WOLFE
C     CONDITION
C
      IF (L4) THEN
        IF (NGR.EQ.0) FQ = MIN(F,T)
        IF (F.LE.FQ) THEN
          L6 = .FALSE.
          IF (P.GE.PL) THEN
C
C     TEST ORIGINAL WOLFE CONDITIONS OR APPROXIMATE WOLFE CONDITIONS
C
            IF (F-T.LE.R*PU.OR.L1.AND.F.LE.FUB.AND.P.LE.PPU)
     &         L6=.TRUE.
          ENDIF
          IF (L6) GO TO 200
        ENDIF
      ENDIF
      IF (FP.LE.FUB) THEN
        TL = R
        EL = PP
        GO TO 300
      ENDIF
C
C     CONTRACTION PHASE
C
      TU = R
      R = (TL+TU)/TWO
      NSH = NSH + 1
      IF (NSH.GT.MRED) THEN
        IER = 6
        GO TO 200
      ENDIF
      CALL UXVDIR(NF,R,S,XO,X)
      KD=1
      LD=0
      CALL UYSET0(1,5)
      RETURN
    5 CONTINUE
      P = UXVDOT(NF,G,S)
      PP = P
      IF (.NOT.L1) PP = PP - PU
      IF (PP.GE.ZERO) GOTO 100
      KD=0
      CALL UYSET2
      CALL UYSET0(1,6)
      RETURN
    6 CONTINUE
      FP = F
      IF (.NOT.L1) FP = FP - R*PU
      IF (L4.AND.(F.LE.FQ)) THEN
        L6 = .FALSE.
        IF (P.GE.PL) THEN
C
C     TEST ORIGINAL WOLFE CONDITIONS OR APPROXIMATE WOLFE CONDITIONS
C
          IF (F-T.LE.R*PU.OR.L1.AND.F.LE.FUB.AND.P.LE.PPU) L6=.TRUE.
        ENDIF
        IF (L6) GO TO 200
      ENDIF
      IF (FP.LE.FUB) THEN
        TL = R
        EL = PP
      ELSE
        TU = R
      ENDIF
C
C EXPANSION PHASE
C
300   CONTINUE
      NGR = NGR + 1
      IF (NGR.GT.MRED) THEN
        IER = 3
        GO TO 200
      ENDIF
      R = CON5*R
      CALL UXVDIR(NF,R,S,XO,X)
      KD=1
      LD=0
      CALL UYSET0(1,7)
      RETURN
    7 CONTINUE
      P = UXVDOT(NF,G,S)
      PP = P
      IF (.NOT.L1) PP = PP - PU
      GO TO 750
100   CONTINUE
      TU = R
      EU = PP
      IF (.NOT.L4) GO TO 400
      KD=0
      CALL UYSET2
      CALL UYSET0(1,8)
      RETURN
    8 CONTINUE
      IF (NGR + NSH.EQ.0) FQ = MIN(F,T)
      IF (F.LE.FQ) THEN
        L6 = .FALSE.
        IF (P.GE.PL) THEN
C
C TEST ORIGINAL WOLFE CONDITIONS OR APPROXIMATE WOLFE CONDITIONS
C
          IF (F-T.LE.R*PU.OR.L1.AND.F.LE.FUB.AND.P.LE.PPU) L6=.TRUE.
        ENDIF
        IF (L6) GO TO 200
      ENDIF
400   CONTINUE
      NRED=1
500   CONTINUE
      IF (NRED.GT.MRED) GO TO 600
      RW = CON3*(TU - TL)
      IF (EU-EL.NE.ZERO) THEN
        IF (-EL.LE.EU) THEN
          R = TL - (TL-TU)*(EL/(EL-EU))
        ELSE
          R = TU - (TL-TU)*(EU/(EL-EU))
        ENDIF
      ELSE
        R = ONE
      ENDIF
      TO = R
      TLO = TL
      TUO = TU
      ELO = EL
      EUO = EU
      CALL UYSET0(2,9)
      RETURN
    9 CONTINUE
      IF (IUP.GT.0) GO TO 200
      IF (IUP.LT.0) GO TO 800
      IF (TO.EQ.TL) THEN
        IF (PP.GT.ELO) THEN
          R = TO - (TO-TLO)*(PP/(PP-ELO))
        ELSE
          R = TL
        ENDIF
      ELSE
        IF (PP.LT.EUO) THEN
          R = TO - (TO-TUO)*(PP/(PP-EUO))
        ELSE
          R = TU
        ENDIF
      ENDIF
      IF ((R.LE.TL).OR.(R.GE.TU)) GO TO 800
      CALL UYSET0(2,10)
      RETURN
   10 CONTINUE
      IF (IUP.GT.0) GO TO 200
C
C     BISECTION ITERATION
C
800   CONTINUE
      IF ((TU-TL).LT.RW) THEN
        IF (TU.LE.TL) THEN
          IER = 7
          GO TO 200
        ENDIF
        NRED = NRED + 1
        GO TO 500
      ENDIF
      R = (TU+TL)/TWO
      CALL UYSET0(2,11)
      RETURN
   11 CONTINUE
      IF (IUP.GT.0) GO TO 200
      NRED = NRED + 1
      GO TO 500
600   CONTINUE
      IER = 4
200   CONTINUE
      IF (.NOT.L1.AND.ABS(F-T).LT.CON2*CK) L1 = .TRUE.
      LD=1
      CALL UYEPI1(1)
      RETURN
      END
