$INTERFACE US1LUP                                             08/12/01

  CALL US1LUP(NF,$XO,$X,$S,$G,R,F,T,P,PP,TL,EL,PL,TU,EU,PU,PPU,FUB&
,MRED,IER,IUP,L1)
  $ADD(INIT)
    NSXX='US1L10'
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE US1LUP               ALL SYSTEMS                97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 HZ : ORIGINAL VERSION
*
* PURPOSE :
*  STANDARD LINE SEARCH WITH DIRECTIONAL DERIVATIVES.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  RO  XO(NF)  SAVED VECTOR OF VARIABLES.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  RO  S(NF)  DIRECTION VECTOR.
*  RI  G(NF)  GRADIENT OF THE OBJECTIVE FUNCTION.
*  RO  R  VALUE OF THE STEPSIZE PARAMETER.
*  RO  F  VALUE OF THE OBJECTIVE FUNCTION.
*  RI  T  .
*  RO  P  VALUE OF THE DIRECTIONAL DERIVATIVE.
*  RO  PP  PREVIOUS VALUE OF THE DIRECTIONAL DERIVATIVE.
*  RU  TL  .
*  RU  EL  .
*  RU  PL  .
*  RU  TU  .
*  RU  EU  .
*  RU  PU  .
*  RU  PPU  .
*  RU  FUB  .
*  II  MRED  MAXIMUM NUMBER OF EXTRAPOLATIONS OR INTERPOLATIONS.
*  II  IER  .
*  II  IUP  .
*  LI  L1  .
*
* STATUS VARIABLES :
*  NS,ISP,TSS
*
* SUBPROGRAM USED :
*  S   UYPRO1  PROLOGUE.
*  S   UYEPI1  EPILOGUE.
*  S   UYSET0  STATUS DEFINITION.
*  S   UYSET2  DEGREE DEFINITION OF THE COMPUTED DERIVATIVES.
*  S   UXVDIR  VECTOR AUGMENTED BY THE SCALED VECTOR.
*
* METHOD :
* SPECIAL LINE SEARCH OF HAGER AND ZHANG.
*
      SUBROUTINE US1LUP(NF,XO,X,S,G,R,F,T,P,PP,TL,EL,PL,TU,EU,PU,PPU,
     & FUB,MRED,IER,IUP,L1)
      $INCLUDE('UMCOMM')
      INTEGER NF,MRED,IER,IUP
      $FLOAT XO(NF),X(NF),S(NF),G(NF),R,F,T,P,PP,TL,EL,PL,TU,EU,PU,PPU,
     & FUB
      LOGICAL L1,L6
      INTEGER NSH
      $FLOAT TOL,ETA,FP,UXVDOT
      IF (TSS(NS).NE.'USSX') CALL UYPRO1('USSX',1)
      GO TO (1,2,3) ISP(NS)
    1 CONTINUE
      CALL UXVDIR(NF,R,S,XO,X)
      CALL UYSET2
      CALL UYSET0(1,2)
      RETURN
    2 CONTINUE
      P = UXVDOT(NF,G,S)
      FP = F
      PP = P
      IF (.NOT.L1) THEN
        FP = FP - R*PU
        PP = PP - PU
      ENDIF
      IUP = 0
      L6 = .FALSE.
      IF (P.GE.PL) THEN
C
C TEST ORIGINAL WOLFE CONDITIONS OR APPROXIMATE WOLFE CONDITIONS
C
        IF (F-T.LE.R*PU.OR.L1.AND.F.LE.FUB.AND.P.LE.PPU) L6=.TRUE.
      ENDIF
      IF (L6) THEN
        IUP = 1
        GO TO 200
      ELSEIF (PP.GE.ZERO) THEN
        TU = R
        EU = PP
        GO TO 200
      ELSEIF (FP.LE.FUB) THEN
        TL = R
        EL = PP
        GO TO 200
      ENDIF
      NSH = 0
      TU = R
      R = (TL+TU)/TWO
      NSH = NSH + 1
      IF (NSH.GT.MRED) THEN
        IER = 8
        IUP = 1
        GO TO 200
      ENDIF
      CALL UXVDIR(NF,R,S,XO,X)
      CALL UYSET2
      CALL UYSET0(1,3)
      RETURN
    3 CONTINUE
      P = UXVDOT(NF,G,S)
      FP = F
      PP = P
      IF (.NOT.L1) THEN
        FP = FP - R*PU
        PP = PP - PU
      ENDIF
      L6 = .FALSE.
      IF (P.GE.PL) THEN
C
C TEST ORIGINAL WOLFE CONDITIONS OR APPROXIMATE WOLFE CONDITIONS
C
        IF (F-T.LE.R*PU.OR.L1.AND.F.LE.FUB.AND.P.LE.PPU) L6=.TRUE.
      ENDIF
      IF (L6) THEN
        IUP = 1
         GO TO 200
      ELSEIF (PP.GE.ZERO) THEN
        TU = R
        EU = PP
        GOTO 100
      ELSEIF (FP.LE.FUB) THEN
        TL = R
        EL = PP
      ELSE
        TU = R
      ENDIF
100   CONTINUE
      IUP = -1
200   CONTINUE
      CALL UYEPI1(1)
      RETURN
      END
