$INTERFACE UUBSD1                                             97/12/01

  $HELP
    MAXIMUM SIZE OF THE PARTITIONED HESSIAN MATRIX
  $CHECK(MAH,'25*$NA',INT,1)
  $HELP
    CHOICE OF THE INITIAL APPROXIMATION OF THE SPARSE
    HESSIAN  MATRIX:
      1 - UNIT INITIAL APPROXIMATION
      2 - RANDOM INITIAL APPROXIMATION
      3 - NUMERICAL INITIAL APPROXIMATION
  $CHECK(INITH,1,'1\2\3',4)
  $IF(INITH>2)
    IF (IREST.LE.0) GO TO $20
    CALL UYPRO1('UUXX',2)
    ASSIGN $10 TO IMD
    $GO TO $LMD
$10 CONTINUE
    TUXX='NEWTON  '
    IDECF=0
    CALL UYSET3
    CALL UYEPI1(2)
$20 CONTINUE
  $ELSE
    $N='NF'
    $ADD(REAL,'\$AH($MAH)')
    $ADD(INTEGER,'\$IAG($NA+1)')
$REM    $ADD(INTEGER,'\$N\NA\NAL\MAH')
    CALL UUBSD1($N,NA,NAL,MAH,$AH,$IAG,$INITH)
  $ENDIF
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UUBSD1                ALL SYSTEMS                97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* INITIATION OF A PARTITIONED SYMMETRIC POSITIVE DEFINITE MATRIX.
*
* PARAMETERS :
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  II  NA  NUMBER OF APPROXIMATED FUNCTIONS.
*  II  NAL  NUMBER OF LINEAR APPROXIMATED FUNCTIONS.
*  II  MB  NUMBER OF NONZERO ELEMENTS OF THE PARTITIONED HESSIAN MATRIX.
*  RU  AH(MB)  NUMERICAL VALUES OF ELEMENTS OF THE PARTITIONED HESSIAN
*         MATRIX.
*  II  IAG(NA+1)  POINTERS OF THE JACOBIAN MATRIX.
*  II  INITH  CHOICE OF THE INITIAL APPROXIMATION OF THE HESSIAN
*         MATRIX. INITH=1-UNIT INITIAL APPROXIMATION.
*         INITH=2-RANDOM INITIAL APPROXIMATION.
*
* COMMON DATA :
*  RU  RAN  RANDOM NUMBER.
*  II  IDECF  DECOMPOSITION INDICATOR. IDECF=0-NO DECOMPOSITION.
*  II  IREST  RESTART SPECIFICATION. IF IREST=0 DOES NOT HOLD THEN A
*         RESTART IS PERFORMED.
*  IU  LD  DEGREE OF PREVIOUSLY COMPUTED DERIVATIVES.
*  TO  TUXX   TEXT INFORMATION ON THE RESTART USED.
*
* SUBPROGRAMS USED :
*  S   UXBSMI  GENERATION OF THE UNIT MATRIX.
*  S   UXSSDO  INITIATION OF A DIAGONAL MATRIX.
*  RF  UNRAN1  RANDOM NUMBER GENERATOR.
*  S   UYSET3  DEFINITION OF THE RESTART VARIABLES.
*
      SUBROUTINE UUBSD1(N,NA,NAL,MB,AH,IAG,INITH)
      $INCLUDE('UMCOMM')
      INTEGER N,NA,NAL,MB,IAG(NA+1),INITH
      $FLOAT AH(MB)
      INTEGER K
      $FLOAT POM,UNRAN1
      $FLOAT CON
      PARAMETER (CON=1.0$P 1)
      IF(IREST.LE.0) RETURN
      IF(INITH.LE.0) RETURN
      GOTO (1,2) INITH
    1 CALL UXBSMI(MB,NA,AH,IAG)
      TUXX='UNIT    '
      LD=MIN(LD,1)
      GOTO 6
    2 POM=(CON-ONE)/(CON+ONE)
      POM=POM/$DBLE(N+1)
      DO 7 K=1,MB
      AH(K)=POM*(TWO*UNRAN1(RAN)-ONE)
    7 CONTINUE
      CALL UXBSDO(MB,NA,ONE,AH,IAG)
      TUXX='RANDOM  '
      LD=MIN(LD,1)
    6 IDECF=0
      IF (NAL.GT.0) CALL UXBSMO(MB,NAL,ZERO,AH,IAG)
      CALL UYSET3
      RETURN
      END
