$INTERFACE UUBSD3                                             97/12/01

  $HELP
    MAXIMUM SIZE OF THE PARTITIONED HESSIAN MATRIX
  $CHECK(MAH,'25*$NA',INT,1)
  $HELP
    CHOICE OF THE INITIAL APPROXIMATION OF THE HESSIAN
    MATRIX:
      1 - UNIT INITIAL APPROXIMATION
      2 - RANDOM INITIAL APPROXIMATION
  $CHECK(INITH,1,'1\2',4)
  $ADD(REAL,'\$AH($MAH)\$AB($MAH)')
  $ADD(INTEGER,'\$IA($NA)\$IAG($NA+1)')
$REM  $ADD(INTEGER,'\$N\NA\NAL\MAH\IPOM1\IPOM2')
  CALL UUBSD3($N,NA,NAL,MAH,$IA,$AH,$AB,$IAG,IPOM1,IPOM2,$INITH)
  $ADD(SINIT)
    CALL UXVINS(NA,3,$IA)
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UUBSD3                ALL SYSTEMS                97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* INITIATION OR MODIFICATION OF A PARTITIONED SYMMETRIC POSITIVE
* DEFINITE MATRIX.
*
* PARAMETERS :
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  II  NA  NUMBER OF APPROXIMATED FUNCTIONS.
*  II  NAL  NUMBER OF LINEAR APPROXIMATED FUNCTIONS.
*  II  MB  NUMBER OF NONZERO ELEMENTS OF THE PARTITIONED HESSIAN MATRIX.
*  II  IA(NA)  TYPES OF APPROXIMATED FUNCTIONS.
*  RU  AH(MB)  NUMERICAL VALUES OF ELEMENTS OF THE PARTITIONED HESSIAN
*         MATRIX.
*  RU  AB(MB)  NUMERICAL VALUES OF ELEMENTS OF THE PARTITIONED HESSIAN
*         MATRIX.
*  II  IAG(NA+1)  POINTERS OF THE JACOBIAN MATRIX.
*         MATRIX.
*  II  IPOM1  METHOD INDICATOR.
*  IO  IPOM2  INDICATOR FOR SCALING.
*  II  INITH  CHOICE OF THE INITIAL APPROXIMATION OF THE HESSIAN
*         MATRIX. INITH=1-UNIT INITIAL APPROXIMATION.
*         INITH=2-RANDOM INITIAL APPROXIMATION.
*
* COMMON DATA :
*  RU  RAN  RANDOM NUMBER.
*  II  IDECF  DECOMPOSITION INDICATOR. IDECF=0-NO DECOMPOSITION.
*  II  IREST  RESTART SPECIFICATION. IF IREST=0 DOES NOT HOLD THEN A
*         RESTART IS PERFORMED.
*  IU  LD  DEGREE OF PREVIOUSLY COMPUTED DERIVATIVES.
*  TO  TUXX   TEXT INFORMATION ON THE RESTART USED.
*
* SUBPROGRAMS USED :
*  S   UXBSMI  GENERATION OF THE UNIT MATRIX.
*  S   UXBSDO  INITIATION OF A DIAGONAL MATRIX.
*  S   UXDSMC  COPYING OF A DENSE SYMMETRIC MATRIX.
*  S   UXVCOP  COPYING OF A VECTOR.
*  RF  UNRAN1  RANDOM NUMBER GENERATOR.
*  S   UYSET3  DEFINITION OF THE RESTART VARIABLES.
*
      SUBROUTINE UUBSD3(N,NA,NAL,MB,IA,AH,AB,IAG,IPOM1,IPOM2,INITH)
      $INCLUDE('UMCOMM')
      INTEGER N,NA,NAL,MB,IA(NA),IAG(NA+1),IPOM1,IPOM2,INITH
      $FLOAT AH(MB),AB(MB)
      INTEGER K,L,KA,NB
      $FLOAT POM,UNRAN1
      $FLOAT CON
      PARAMETER (CON=1.0$P 1)
      IF(IREST.EQ.0.OR.IREST.GT.0.AND.IPOM1.EQ.0) THEN
      ELSEIF(INITH.EQ.1) THEN
      CALL UXBSMI(MB,NA,AB,IAG)
      TUXX='UNIT    '
      IPOM2=1
      ELSEIF(INITH.EQ.2) THEN
      POM=(CON-ONE)/(CON+ONE)
      POM=POM/$DBLE(N+1)
      DO 1 K=1,MB
      AB(K)=POM*(TWO*UNRAN1(RAN)-ONE)
    1 CONTINUE
      CALL UXBSDO(MB,NA,ONE,AB,IAG)
      TUXX='RANDOM  '
      IPOM2=1
      ENDIF
      IF (NAL.GT.0) CALL UXBSMO(MB,NAL,ZERO,AB,IAG)
      IF (IPOM1.EQ.1) THEN
      IF (ITERS.GT.0.OR.IREST.GT.0) THEN
      NB=0
      DO 2 KA=1,NA
      K=IAG(KA)
      L=IAG(KA+1)-K
      IF (IA(KA).GT.0) THEN
      CALL UXDSMC(L,AB(NB+1),AH(NB+1))
      ENDIF
      NB=NB+L*(L+1)/2
    2 CONTINUE
      LD=MIN(LD,1)
      IF(IREST.GT.0) CALL UYSET3
      ENDIF
      ELSE
      IF(IREST.LE.0) THEN
      TUXX='NEWTON  '
      ELSE
      IPOM1=1
      CALL UXVCOP(MB,AB,AH)
      LD=MIN(LD,1)
      ENDIF
      ENDIF
      IDECF=0
      RETURN
      END
