$INTERFACE UUBSD5                                             97/12/01

  $HELP
    MAXIMUM SIZE OF THE PARTITIONED HESSIAN MATRIX
  $CHECK(MAH,'25*$NA',INT,1)
  $HELP
    CHOICE OF THE INITIAL APPROXIMATION OF THE HESSIAN
    MATRIX:
      1 - UNIT INITIAL APPROXIMATION
      2 - RANDOM INITIAL APPROXIMATION
  $CHECK(INITH,1,'1\2',4)
  $DEFAULT='5.0D-4'
  $IF(TYPE='L') $DEFAULT='4.0D-2'
  $HELP
    TOLERANCE FOR APPLICATION OF THE BFGS METHOD
  $CHECK(ETA5,DEFAULT,REAL,4)
  $ADD(REAL,'\$AF($NA)\$AH($MAH)\$AB($MAH)')
$REM  $ADD(REAL,'\F\FO\ETA5')
  $ADD(INTEGER,'\$IA($NA)\$IAG($NA+1)')
$REM  $ADD(INTEGER,'\$N\NA\NAL\MAH')
  $N='NF'
  CALL UUBSD5($N,NA,NAL,MAH,$IA,$AF,$AH,$AB,$IAG,F,FO,ETA5,$INITH)
  $ADD(SINIT)
    CALL UXVINS(NA,3,$IA)
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UUBSD5                ALL SYSTEMS                97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* INITIATION OR MODIFICATION OF A PARTITIONED SYMMETRIC POSITIVE
* DEFINITE MATRIX.
*
* PARAMETERS :
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  II  NA  NUMBER OF BLOCKS OF THE MATRIX H.
*  II  NAL  NUMBER OF LINEAR APPROXIMATED FUNCTIONS.
*  II  MB  NUMBER OF NONZERO ELEMENTS OF THE MATRIX H.
*  II  IA(NA)  TYPES OF APPROXIMATED FUNCTIONS.
*  RI  AF(NA)  VECTOR CONTAINING VALUES OF THE APPROXIMATED
*         FUNCTIONS.
*  RU  AH(MB)  POSITIVE DEFINITE APPROXIMATION OF THE HESSIAN
*         MATRIX
*  RU  AB(MB)  POSITIVE DEFINITE APPROXIMATION OF THE HESSIAN
*         MATRIX
*  II  IAG(NA+1)  POINTERS OF THE DIAGONAL ELEMENTS OF THE HESSIAN
*         MATRIX.
*  RI  F  CURRENT VALUE OF THE OBJECTIVE FUNCTION.
*  RI  FO  PREVIOUS VALUE OF THE OBJECTIVE FUNCTION.
*  RI  ETA5  TOLERANCE FOR A HYBRID METHOD.
*  II  INITH  CHOICE OF THE INITIAL APPROXIMATION OF THE HESSIAN
*         MATRIX. INITH=1-UNIT INITIAL APPROXIMATION.
*         INITH=2-RANDOM INITIAL APPROXIMATION.
*
* COMMON DATA :
*  RU  RAN  RANDOM NUMBER.
*  II  IDECF  DECOMPOSITION INDICATOR. IDECF=0-NO DECOMPOSITION.
*  II  IREST  RESTART SPECIFICATION. IF IREST=0 DOES NOT HOLD THEN A
*         RESTART IS PERFORMED.
*  IU  LD  DEGREE OF PREVIOUSLY COMPUTED DERIVATIVES.
*  TO  TUXX   TEXT INFORMATION ON THE RESTART USED.
*
* SUBPROGRAMS USED :
*  S   UXBSMI  GENERATION OF THE UNIT MATRIX.
*  S   UXBSDO  INITIATION OF A DIAGONAL MATRIX.
*  S   UXDSMA  MODIFICATION OF A DENSE SYMMETRIC MATRIX.
*  RF  UNRAN1  RANDOM NUMBER GENERATOR.
*  S   UYSET3  DEFINITION OF THE RESTART VARIABLES.
*
      SUBROUTINE UUBSD5(N,NA,NAL,MB,IA,AF,AH,AB,IAG,F,FO,ETA5,INITH)
      $INCLUDE('UMCOMM')
      INTEGER N,NA,NAL,MB,IA(NA),IAG(NA+1),INITH
      $FLOAT AF(NA),AH(MB),AB(MB),F,FO,ETA5
      INTEGER K,L,LL,KA,NB
      $FLOAT POM,UNRAN1
      $FLOAT CON
      PARAMETER (CON=1.0$P 1)
      IF (IREST.LT.0) THEN
      CALL UXBSMI(MB,NA,AB,IAG)
      IF (NAL.GT.0) CALL UXBSMO(MB,NAL,ZERO,AB,IAG)
      ELSE IF (IREST.EQ.0) THEN
      ELSEIF(INITH.EQ.1) THEN
      CALL UXBSMI(MB,NA,AH,IAG)
      IF (NAL.GT.0) CALL UXBSMO(MB,NAL,ZERO,AH,IAG)
      TUXX='UNIT    '
      LD=MIN(LD,1)
      CALL UYSET3
      ELSEIF(INITH.EQ.2) THEN
      POM=(CON-ONE)/(CON+ONE)
      POM=POM/$DBLE(N+1)
      DO 1 K=1,MB
      AH(K)=POM*(TWO*UNRAN1(RAN)-ONE)
    1 CONTINUE
      CALL UXBSDO(MB,NA,ONE,AH,IAG)
      IF (NAL.GT.0) CALL UXBSMO(MB,NAL,ZERO,AH,IAG)
      TUXX='RANDOM  '
      LD=MIN(LD,1)
      CALL UYSET3
      ENDIF
      IF (IDIR.EQ.0.AND.FO-F.LE.ETA5*FO) THEN
      NB=0
      DO 2 KA=1,NA
      K=IAG(KA)
      L=IAG(KA+1)-K
      LL=L*(L+1)/2
      IF (IA(KA).GT.0) THEN
      CALL UXDSMA(L,AF(KA),AB(NB+1),AH(NB+1),AH(NB+1))
      LD=MIN(LD,1)
      ENDIF
      NB=NB+LL
    2 CONTINUE
      ENDIF
      IDECF=0
      RETURN
      END
