$INTERFACE UUDFA1                                             97/12/01

  $HELP
    SELECTION OF SELF SCALING:
      1 - SELF SCALING SUPPRESSED
      2 - SELF SCALING IN THE FIRST ITERATION AFTER RESTART
      3 - SELF SCALING IN EACH ITERATION
  $CHECK(MET1,2,'1\2\3',4)
  $DEFAULT='1.0D-3'
  $IF(TYPE='L') $DEFAULT='2.0D-2'
  $HELP
    TOLERANCE FOR APPLICATION OF THE BFGS METHOD
  $CHECK(ETA5,DEFAULT,REAL,4)
  $ADD(REAL,'\$AGH($NF*$NA)\$XO($NF)\$GO($NF)\$AFO($NA)')
$REM  $ADD(REAL,'\F\FO\ETA5')
$REM  $ADD(INTEGER,'\NA\IPOM1\IPOM2')
  $N='N'
  CALL UUDFA1($N,NA,$AGH,$XO,$GO,$AFO,F,FO,ETA5,IPOM1,IPOM2&
,$MET,$MET1)
  $ADD(INIT)
    NUXX='UUDFA1'
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UUDFA1                ALL SYSTEMS                97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* VARIABLE METRIC UPDATE OF A DENSE RECTANGULAR MATRIX.
*
* PARAMETERS :
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  II  NA  NUMBER OF APPROXIMATED FUNCTIONS.
*  RU  BG(N*NA)  APPROXIMATION OF THE JACOBIAN MATRIX.
*  RI  G(N)  GRADIENT OF THE OBJECTIVE FUNCTION.
*  RU  XO(N)  VECTORS OF VARIABLES DIFFERENCE.
*  RI  GO(N)  GRADIENTS DIFFERENCE.
*  RA  AFO(NA)  AUXILIARY VECTOR.
*  RI  F  CURRENT VALUE OF THE OBJECTIVE FUNCTION.
*  RI  FO  PREVIOUS VALUE OF THE OBJECTIVE FUNCTION.
*  RI  ETA5  TOLERANCE FOR A HYBRID METHOD.
*  RI  ETA9  MAXIMUM FOR REAL NUMBERS.
*  II  IPOM1  METHOD INDICATOR.
*  IO  IPOM2  INDICATOR FOR SCALING.
*  II  MET  METHOD SELECTION. MET=0-NO UPDATE. MET=1-BFGS UPDATE.
*  II  MET1  SELECTION OF SELF SCALING.  MET1=1-SELF SCALING SUPPRESSED.
*         MET1=2 SELF SCALING IN THE FIRST ITERATION AFTER RESTART.
*         MET1=3-SELF SCALING IN EACH ITERATION.
*
* COMMON DATA :
*  IO  ITERH  TERMINATION INDICATOR. ITERH<0-BAD DECOMPOSITION.
*         ITERH=0-SUCCESSFUL UPDATE. ITERH>0-NONPOSITIVE PARAMETERS.
*  II  IDECA  DECOMPOSITION INDICATOR. IDECF=0-NO DECOMPOSITION.
*  TO  TUXX  TEXT INFORMATION ON THE CORRECTION USED.
*
* SUBPROGRAMS USED :
*  S   UXDCMM  MATRIX-VECTOR PRODUCT.
*  S   UXDRMM  MATRIX-VECTOR PRODUCT.
*  S   UXDRMU  CORRECTION OF A RECTANGULAR MATRIX.
*  S   UXVDIR  VECTOR AUGMENTED BY THE SCALED VECTOR.
*  RF  UXVDOT  DOT PRODUCT OF VECTORS.
*  S   UXVSCL  SCALING OF A VECTOR.
*  S   UOERR1  ERROR MESAGES.
*  S   UOU1D1  PRINT OF ENTRY TO VARIABLE METRIC UPDATE.
*  S   UOU1D2  PRINT OF EXIT FROM VARIABLE METRIC UPDATE.
*
* METHOD :
*  FLETCHER'S COMBINATION OF THE GAUSS-NEWTON AND THE BFGS METHODS.
*
      SUBROUTINE UUDFA1(N,NA,BG,XO,GO,AFO,F,FO,ETA5,ETA9,IPOM1,IPOM2,
     & MET,MET1)
      $INCLUDE('UMCOMM')
      INTEGER N,NA,IPOM1,IPOM2,MET,MET1
      $FLOAT BG(N*NA),XO(N),GO(N),AFO(N),F,FO,ETA5,ETA9
      $FLOAT UXVDOT
      $FLOAT AB,BB,CB,DB,GAM,PAR
      LOGICAL L1
      IF (MET.LE.0) RETURN
      CALL UOU1D1(N,XO,GO)
      IF(IDECA.NE.0) THEN
      ITERH=-1
      TUXX='BAD DEC0'
      CALL UOERR1('UUDFA1',1)
      GOTO 2
      ENDIF
      PAR=ZERO
C
C     DETERMINATION OF THE PARAMETERS A,B,C
C
      BB=UXVDOT(N,XO,GO)
      IF (BB.LE.ZERO)  THEN
      ITERH=2
      TUXX='B - NEG.'
      IPOM1=0
      GOTO 2
      ENDIF
      AB=ZERO
      CALL UXDRMM(N,NA,BG,XO,AFO)
      CB=UXVDOT(NA,AFO,AFO)
      IF (CB.LE.ZERO) THEN
      ITERH=3
      TUXX='C - NEG.'
      GOTO 2
      ENDIF
      DB=SQRT(CB/BB)
      L1=MET1.EQ.4.OR.MET1.EQ.3.AND.IPOM2.GE.1.OR.MET1.EQ.2
     & .AND.IPOM2.EQ.1
      IF (FO-F.GE.ETA5*FO) THEN
      IPOM1=0
      ELSE
      IPOM1=1
      ENDIF
      IF (L1)  THEN
C
C     DETERMINATION OF THE PARAMETER GAM (SELF SCALING)
C
      GAM=DB
      ELSE
      GAM=ONE
      ENDIF
C
C     BFGS UPDATE
C
      CALL UXDCMM(N,NA,BG,AFO,XO)
      CALL UXVDIR(N,-DB,GO,XO,XO)
      CALL UXDRMU(N,NA,BG,-ONE/CB,AFO,XO)
      TUXX='BFGS    '
      NUPD=NUPD+1
      ITERH=0
      IPOM2=0
      IF (L1) THEN
      CALL UXVSCL(N*NA,ONE/GAM,BG,BG)
      NSCL=NSCL+1
      ENDIF
    2 CALL UOU1D2(N,BG,XO,GAM,GAM,PAR,AB,BB,CB,GAM,ETA9)
      RETURN
      END
