$INTERFACE UULSP2                                             02/12/01

  $GR='GR'
  $HELP
    METHOD OF UPDATE OF HESSIAN MATRIX:
      1 - BFGS UPDATE
      2 - DFP UPDATE
      3 - HOSHINO UPDATE
      4 - SAFEGUARDED RANK ONE UPDATE
      5 - VARIATIONALLY DERIVED PRECONVEX UPDATE
  $CHECK(MET,1,'1\2\3\4\5',3)
  $HELP
    SELECTION OF SELF SCALING:
      ABS(MET1)=1 - SELF SCALING SUPPRESSED
      ABS(MET1)=2 - INITIAL SELF SCALING
      ABS(MET1)=3 - CONTROLLED SELF SCALING
      ABS(MET1)=4 - SIMPLIFIED CONTROLLED SELF SCALING
      ABS(MET1)=5 - SELF SCALING IN EACH ITERATION
           MET1>0 - BASIC INITIAL SELF SCALING
           MET1<0 - MODIFIED INITIAL SELF SCALING
  $CHECK(MET1,4,'-5\-4\-3\-2\-1\1\2\3\4\5',4)
  $NET1=MET1
  $IF(NET1=-1) $NET1=1
  $IF(NET1=-2) $NET1=2
  $IF(NET1=-3) $NET1=3
  $IF(NET1=-4) $NET1=4
  $IF(NET1=-5) $NET1=5
  $HELP
    SELECTION OF THE LINE SEARCH MODEL:
      ABS(MET2)=1 - QUADRATIC MODEL
      ABS(MET2)=2 - BASIC USE OF TAYLOR EXPANSION
      ABS(MET2)=3 - MODIFIED USE OF TAYLOR EXPANSION
      ABS(MET2)=4 - HOMOGENEOUS MODEL
      ABS(MET2)=5 - CUBIC MODEL
           MET2>0 - BASIC VARIABLE METRIC UPDATE
           MET2<0 - MODIFIED VARIABLE METRIC UPDATE
  $CHECK(MET2,2,'-5\-4\-3\-2\-1\1\2\3\4\5',4)
  $IF($DEF(MET3DEF))
  $ELSE
    $MET3DEF='MET3'
    $HELP
      COMPUTATION OF THE WEIGHTING PARAMETER:
        0 - PREVIOUS CHOICE
        1 - SIMPLE CHOICE WITH A CONSTANT RELATIVE PARAMETER
        2 - USING QUADRATIC EQUATION
        3 - HEURISTIC CHOICE WITH A RELATIVE PARAMETER LESS
            THEN HALF
        4 - HEURISTIC CHOICE WITH THE OPTIMUM CONDITION NUMBER
            IN THE FIRST ITERATION (FIRST FORMULA)
        5 - HEURISTIC CHOICE WITH THE OPTIMUM CONDITION NUMBER
            IN THE FIRST ITERATION (SECOND FORMULA)
        6 - HEURISTIC CHOICE DERIVED FROM THE LOWEST EIGENVALUE
    $CHECK(MET3,4,'0\1\2\3\4\5\6',4)
  $ENDIF
  $IF(MET= 1) $DEFAULT=1
  $IF(MET= 2) $DEFAULT=2
  $IF(MET= 3) $DEFAULT=3
  $IF(MET= 4) $DEFAULT=1
  $IF(MET= 5) $DEFAULT=3
  $IF(MET1=5) $DEFAULT=3
  $HELP
    METHOD CORRECTION:
      1 - BFGS SCALING
      2 - DFP SCALING
      3 - GEOMETRIC MEAN
      4 - HARMONIC MEAN
      5 - ARITHMETIC MEAN
  $CHECK(MET4,DEFAULT,'1\2\3\4\5',4)
  $ADD(REAL,'\$XM($NF*$MF)\$GR($MF)\$S($NF)\$XO($NF)\$GO($NF)')
$REM  $ADD(INTEGER,'\NF\MF\INF')
  CALL UULSP2(NF,NN,MF,$XM,$GR,$S,$XO,$GO,R,F,FO,P,PO,PAR&
,PAR1,PAR2,$CON1,MET,MET1,MET2,$MET3DEF,MET4)
  $ADD(INIT)
    NUXX='UULSP2'
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UULSP2                ALL SYSTEMS                02/12/01
C PORTABILITY : ALL SYSTEMS
C 02/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* LIMITED STORAGE VARIABLE METRIC UPDATE.
*
* PARAMETERS :
*  II  N  NUMBER OF VARIABLES.
*  II  M  NUMBER OF VARIABLE METRIC STEPS.
*  II  MF MAXIMUM NUMBER OF VARIABLE METRIC STEPS.
*  RU  XM(N*M)  SET OF VECTOR OF VARIABLES DIFFERENCES.
*  RU  GR(M)  MATRIX TRANS(GM)*XM STORED IN PACKED FORM.
*  RA  S(N)  AUXILIARY VECTOR.
*  RU  XO(N)  VECTOR OF VARIABLES DIFFERENCE.
*  RI  GO(N)  GRADIENT DIFFERENCE.
*  RI  R  STEPSIZE PARAMETER.
*  RI  F  NEW FUNCTION VALUE.
*  RI  FO  OLD FUNCTION VALUE.
*  RI  P  NEW DIRECTIONAL DERIVATIVE (MULTIPLIED BY R)
*  RI  PO  OLD DIRECTIONAL DERIVATIVE (MULTIPLIED BY R)
*  RU  SIG  SHIFT PARAMETER (ZETA AND SIGMA).
*  RI  PAR1  PARAMETER FOR SCALING.
*  RI  PAR2  PARAMETER FOR SCALING.
*  RI  CON  CONSTANT FOR THE SHIFT PARAMETER DETERMINATION.
*  II  MET  METHOD SELECTION. MET=1-BFGS UPDATE.
*  II  MET1  SCALING OPTION. MET1=1-NO SCALING.
*  II  MET2  CORRECTION OPTION. MET1=1-NO CORRECTION.
*  II  MET3  CHOICE OF SIGMA (1-CONSTANT, 2-QUADRATIC EQUATION).
*  II  MET4  SCALING OPTION. MET4=1 - RHO/GAM=A/B.
*
* COMMON DATA :
*  IO  ITERH  TERMINATION INDICATOR. ITERH<0-BAD DECOMPOSITION.
*         ITERH=0-SUCCESSFUL UPDATE. ITERH>0-NONPOSITIVE PARAMETERS.
*  II  NIT  ACTUAL NUMBER OF ITERATIONS.
*  II  KIT  NUMBER OF THE ITERATION AFTER LAST RESTART.
*  TO  TUXX  TEXT INFORMATION ON THE CORRECTION USED.
*
* SUBPROGRAMS USED :
*  RF  UXVDOT  DOT PRODUCT OF VECTORS.
*  S   UXVSCL  SCALING OF A VECTOR.
*  S   UOERR1  ERROR MESAGES.
*
* METHOD :
* LIMITED STORAGE VARIABLE METRIC METHODS.
*
      SUBROUTINE UULSP2(N,M,MF,XM,GR,S,XO,GO,R,F,FO,P,PO,SIG,
     & PAR1,PAR2,CON,MET,MET1,MET2,MET3,MET4)
      $INCLUDE('UMCOMM')
      INTEGER N,M,MF,MET,MET1,MET2,MET3,MET4
      $FLOAT XM(N*MF),GR(MF),S(N),XO(N),GO(N),
     & R,F,FO,P,PO,SIG,PAR1,PAR2,CON
      $FLOAT UXVDOT
      $FLOAT POM,A,B,C,AA,BB,AH,GAM,RHO,PAR
      INTEGER METS
      IF (MET.LE.0.OR.M.GE.MF) RETURN
      CALL UOU1D1(N,XO,GO)
      METS=MET
      IF (MET.GT.5) MET=1
      B=UXVDOT(N,XO,GO)
      IF (B.LE.ZERO) THEN
      ITERH=2
      TUXX='B - NEG.'
      GO TO 22
      ENDIF
      CALL UXDRMM(N,M,XM,GO,GR)
      AH=UXVDOT(N,GO,GO)
      AA=UXVDOT(M,GR,GR)
      A=AA+AH*SIG
      C=-R*PO
C
C     DETERMINATION OF THE PARAMETER RHO (NONQUADRATIC PROPERTIES)
C
      CALL UURHO1(F,FO,P,PO,B,RHO,MET2)
C
C     DETERMINATION OF THE PARAMETER GAM (SELF SCALING)
C
      CALL UUGAM1(A,B,C,RHO,GAM,PAR,PAR1,PAR2,MET1,MET4)
C
C     DETERMINATION OF THE PARAMETER SIG (SHIFT)
C
      CALL UUSIG1(N,XO,A,B,C,AA,AH,PAR,SIG,SIG,CON,MET3)
C
C     COMPUTATION OF SHIFTED XO AND SHIFTED B
C
      BB=B-AH*SIG
      CALL UXVDIR(N,-SIG,GO,XO,XO)
      IF (RHO.NE.ONE) SIG=RHO*SIG
      IF (M.EQ.0.OR.AA.LE.ZERO) THEN
C
C     BFGS-BASED SHIFTED BFGS UPDATE
C
      POM=ONE
      TUXX='BFGS    '
      NUPD=NUPD+1
      CALL UXDCMU(N,M,XM,-ONE/BB,XO,GR)
      CALL UXVSCL(N,SQRT(PAR/BB),XO,XM(N*M+1))
      GO TO 21
      ENDIF
C
C     DFP-BASED SHIFTED UPDATES
C
C
C     DETERMINATION OF THE BROYDEN PARAMETER POM=ETA
C
      CALL UUPOM1(AA,BB,PAR,POM,MET)
C
C     DFP-BASED SHIFTED UPDATES
C
      CALL UXDCMD(N,M,XM,GR,-SQRT(PAR*AA/BB),XO,S)
      CALL UXDCMD(N,M,XM,GR,-AA/BB,XO,XO)
      CALL UXDCMU(N,M,XM,-ONE/AA,S,GR)
      CALL UXVSCL(N,SQRT(POM/AA),XO,XM(N*M+1))
   21 M=M+1
      IF (GAM.EQ.ONE) GO TO 22
C
C     SCALING
C
      CALL UXVSCL(N*M,SQRT(GAM),XM,XM)
      NSCL=NSCL+1
   22 CONTINUE
      ITERH=0
      MET=METS
C      CALL UOU1D3(N,H,S,RHO,GAM,PAR,A,B,C,POM,ETA9)
      RETURN
      END
