$INTERFACE UULVP2                                             02/12/01

  $XR='XR'
  $GR='GR'
  $SO='SO'
  $GG=G
  $IF(CONSTR='C') $GG=SO
  $HELP
    METHOD OF UPDATE OF HESSIAN MATRIX:
      1 - BFGS UPDATE
      2 - DFP UPDATE
      3 - HOSHINO UPDATE
      4 - SAFEGUARDED RANK ONE UPDATE
      5 - VARIATIONALLY DERIVED PRECONVEX UPDATE
      6 - SPECIAL UPDATE FOR SSBC METHOD
  $CHECK(MET,1,'1\2\3\4\5\6',3)
  $HELP
    VALUE OF THE CORRECTION PARAMETER:
      MET2=1 - UNIT VALUE
      MET2=2 - BALANCING VALUE EQUAL TO THE PREVIOUS RELATIVE 
               PARAMETER
      MET2=3 - VALUE DERIVED FROM THE GRADIENT DIFFERENCES 
      MET2=4 - GEOMETRIC MEAN OF THE PREVIOUS VALUES
      MET2=5 - GEOMETRIC MEAN OF THE PREVIOUS RELATIVE PARAMETER
               AND THE RATIO ZET/(ZET+SIG)
      MET2=6 - RATIO ZET/(ZET+SIG) 
      MET2=7 - RATIO SQRT(ZET)/(SQRT(ZET)+SQRT(SIG))
  $CHECK(MET2,6,'1\2\3\4\5\6\7',4)
  $ADD(REAL,'\$XM($NF*$MF)\$XR($MF)\$GR($MF)\$SO($NF)\$XO($NF)&
  \$GO($NF)')
$REM  $ADD(INTEGER,'\NF\MF\INF')
  CALL UULVP2(NF,NN,$XM,$XR,$GR,$S,$SO,$XO,$GO,R,PO,PAR,MET,MET2,MET3&
,MET5)
  $ADD(INIT)
    NUXX='UULVP2'
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UULVP2                ALL SYSTEMS                02/12/01
C PORTABILITY : ALL SYSTEMS
C 02/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* RANK-TWO LIMITED-STORAGE VARIABLE-METRIC METHODS IN THE PRODUCT FORM.
*
* PARAMETERS :
*  II  N  NUMBER OF VARIABLES (NUMBER OF ROWS OF XM).
*  II  M  NUMBER OF COLUMNS OF XM.
*  RI  XM(N*M)  RECTANGULAR MATRIX IN THE PRODUCT FORM SHIFTED BROYDEN
*         METHOD (STORED COLUMNWISE): H-SIGMA*I=XM*TRANS(XM)
*  RO  XR(M)  VECTOR TRANS(XM)*H**(-1)*XO.
*  RO  GR(M)  MATRIX TRANS(XM)*GO.
*  RA  S(N)  AUXILIARY VECTORS (H**(-1)*XO AND U).
*  RA  SO(N)  AUXILIARY VECTORS ((H-SIGMA*I)*H**(-1)*XO AND V).
*  RU  XO(N)  VECTORS OF VARIABLES DIFFERENCE XO AND VECTOR XO-TILDE.
*  RU  GO(N)  GRADIENT DIFFERENCE GO AND VECTOR XM*TRANS(XM)*GO.
*  RI  R  STEPSIZE PARAMETER.
*  RI  PO  OLD DIRECTIONAL DERIVATIVE (MULTIPLIED BY R)
*  RU  SIG  SCALING PARAMETER (ZETA AND SIGMA).
*  II  MET  METHOD SELECTION. MET=1-BFGS UPDATE.
*  II  MET2  CORRECTION OPTION. MET2=1-NO CORRECTION.
*  II  MET3  CHOICE OF SIGMA (1-CONSTANT, 2-QUADRATIC EQUATION).
*  II  MET5  CHOICE OF Q. (1 AND 2 SIMPLE CHOICE, 3 - APPROXIMATION
*         OF THE SHIFTED BROYDEN CLASS.
*
* COMMON DATA :
*  IO  ITERH  TERMINATION INDICATOR. ITERH<0-BAD DECOMPOSITION.
*         ITERH=0-SUCCESSFUL UPDATE. ITERH>0-NONPOSITIVE PARAMETERS.
*  II  NIT  ACTUAL NUMBER OF ITERATIONS.
*  II  KIT  NUMBER OF THE ITERATION AFTER LAST RESTART.
*  TO  TUXX  TEXT INFORMATION ON THE CORRECTION USED.
*
* SUBPROGRAMS USED :
*  S   UXDRMM  MULTIPLICATION OF A ROWWISE STORED DENSE RECTANGULAR
*         MATRIX BY A VECTOR.
*  S   UXDCMM  MULTIPLICATION OF A COLUMNWISE STORED DENSE RECTANGULAR
*         MATRIX BY A VECTOR.
*  S   UXDCMU  UPDATE OF A COLUMNWISE STORED DENSE RECTANGULAR MATRIX.
*         WITH CONTROLLING OF POSITIVE DEFINITENESS.
*  S   UXVDIF  DIFFERENCE OF TWO VECTORS.
*  S   UXVDIR  VECTOR AUGMENTED BY A SCALED VECTOR.
*  RF  UXVDOT  DOT PRODUCT OF VECTORS.
*  RF  UXVNOR  EUCLIDEAN NORM OF A VECTOR.
*  S   UXVSCL  SCALING OF A VECTOR.
*  S   UXVSET  INITIATION OF A VECTOR.
*
* METHOD :
* RANK-TWO LIMITED-STORAGE VARIABLE-METRIC METHODS IN THE PRODUCT FORM.
*
      SUBROUTINE UULVP2(N,M,XM,XR,GR,S,SO,XO,GO,R,PO,SIG,MET,MET2,MET3,
     & MET5)
      $INCLUDE('UMCOMM')
      INTEGER N,M,MET,MET2,MET3,MET5
      $FLOAT XM(N*M),XR(M),GR(M),S(N),SO(N),XO(N),GO(N),R,PO,SIG
      $FLOAT UXVDOT,UXVNOR,UU,UV,VV
      $FLOAT DEN,DIS,POM,A,B,C,AA,BB,CC,AH,BH,GAM,RHO,PAR,ZET
      INTEGER METS
      $FLOAT CON
      PARAMETER(CON=4.0$P 0)
      IF (MET.LE.0) RETURN
      CALL UOU1D1(N,XO,GO)
      METS=MET
      IF (MET.GT.6) MET=1
C
C     COMPUTATION OF B
C
      B=UXVDOT(N,XO,GO)
      IF (B.LE.ZERO) THEN
      ITERH=2
      TUXX='B - NEG.'
      GO TO 22
      ENDIF
      ZET=SIG
C
C     COMPUTATION OF GR=TRANS(XM)*GO, XR=TRANS(XM)*H**(-1)*XO
C     AND S=H**(-1)*XO, SO=(H-SIGMA*I)*H**(-1)*XO. COMPUTATION
C     OF AA=GR*GR, BB=GR*XR, CC=XR*XR. COMPUTATION OF A AND C.
C
      CALL UXDRMM(N,M,XM,GO,GR)
C      CALL UXVDIF(N,GO,G,S)
      CALL UXVSCL(N,R,S,S)
      CALL UXDRMM(N,M,XM,S,XR)
      CALL UXVDIR(N,-SIG,S,XO,SO)
      AH=UXVDOT(N,GO,GO)
      AA=UXVDOT(M,GR,GR)
      BB=UXVDOT(M,GR,XR)
      CC=UXVDOT(M,XR,XR)
      A=AA+AH*SIG
      C=-R*PO
      RHO=ONE
C
C     DETERMINATION OF THE PARAMETER GAM (SELF SCALING)
C
      GAM=ONE
C
C     DETERMINATION OF THE PARAMETER SIG (SHIFT)
C
      CALL UUSIG1(N,XO,A,B,C,AA,AH,PAR,SIG,SIG,2.5$P-1,MET3)
C
C     COMPUTATION OF SHIFTED XO AND SHIFTED B
C
      B=B-AH*SIG
      CALL UXVDIR(N,-SIG,GO,XO,XO)
C
C     COMPUTATION OF THE PARAMETER RHO (CORRECTION)
C
      IF (MET2.LE.1) THEN
      PAR=ONE
      ELSE IF (MET2.EQ.2) THEN
      PAR=SIG*AH/B
      ELSE IF (MET2.EQ.3) THEN
      PAR=SQRT(ONE-AA/A)
      ELSE IF (MET2.EQ.4) THEN
      PAR=SQRT(SQRT(ONE-AA/A)*(SIG*AH/B))
      ELSE IF (MET2.EQ.5) THEN
      PAR=SQRT(SQRT(HALF*ZET/(SIG+ZET))*SIG*AH/DEN)
      ELSE IF (MET2.EQ.6) THEN
      PAR=ZET/(ZET+SIG)
      ELSE
      PAR=SQRT(ZET)/(SQRT(ZET)+SQRT(SIG))
      ENDIF
C      IF (RHO.NE.ONE) SIG=RHO*SIG
      IF (ABS(MET5).EQ.1) THEN
C
C     COMPUTATION OF Q AND P
C
      PAR=SQRT(SQRT(ONE-AA/A)*(SIG*AH/B))
      POM=SIGN(ONE,BB)*SQRT((CC/B)/PAR)
      CALL UXVDIR(M,POM,GR,XR,XR)
      CALL UXVDIR(N,PAR*POM,XO,SO,S)
C
C     RANK ONE UPDATE OF XM
C
      CALL UXDCMU(N,M,XM,-ONE/(CC+POM*BB),S ,XR)
      TUXX='RANK ONE'
      NUPD=NUPD+1
      ELSE
C
C     DETERMINATION OF THE BROYDEN PARAMETER POM=ETA
C
      IF (MET.LT.6) CALL UUPOM1(A,B,PAR,POM,MET)
      VV=ONE
C
C     COMPUTATION OF XM*GR=XM*TRANS(XM)*GO
C
      CALL UXDCMM(N,M,XM,GR,GO)
C
C     COMPUTATION OF THE VALUE AA*CC-BB*BB WITH TEST ON POSITIVITY
C
      DEN=AA*CC-BB**2
      IF (DEN.LE.ZERO) THEN
      CALL UXVSET(N,ZERO,S)
      ELSE
      IF (ABS(MET5).GE.5) DIS=(UXVDOT(N,XO,S)*AA/B-BB)/DEN
C
C     COMPUTATION OF AUXILIARY VECTORS S=U AND SO=V
C
      CALL UXVDIR(N,-BB/AA,GO,SO,SO)
      IF (ABS(MET5).EQ.3.AND.MET.EQ.6) THEN
      BH=UXVDOT(N,XO,S)
      POM=ONE+MIN(DEN/((AA/B)*BH-BB)**2,CON)
      ENDIF
      IF (ABS(MET5).GE.3) CALL UXVDIR(N,-AA/B,XO,GO,S)
C
C     COMPUTATION OF Q2 AND P2
C
      IF (ABS(MET5).EQ.2) THEN
      DIS=POM*(AA/BB)/(BB+SQRT(POM*DEN+BB**2))
      CALL UXVSCL(N,DIS,SO,S)
      ELSE IF (ABS(MET5).EQ.3) THEN
      CALL UXVDIR(N,SQRT(POM*DEN)/AA,S,SO,S)
      VV=-AA/DEN
      ELSE IF (ABS(MET5).EQ.4) THEN
      CALL UXVSCL(N,-SQRT(POM*DEN)/AA,S,S)
      UU=UXVDOT(N,S ,S )
      UV=UXVDOT(N,S ,SO)
      VV=UXVDOT(N,SO,SO)
      DIS=HALF*(UU+VV+SQRT((UU-VV)**2+FOUR*UV**2))
      IF (.TRUE.) THEN
      CALL UXVLIN(N,UV,S,DIS-UU,SO,S)
      ELSE
      CALL UXVLIN(N,DIS-VV,S,UV,SO,S)
      ENDIF
      CALL UXVDIR(N,-SQRT(DIS)/UXVNOR(N,S),S,SO,S)
      VV=-AA/DEN
      ELSE IF (ABS(MET5).EQ.5) THEN
      UU=SQRT(ONE+POM*DEN*DIS**2)
      CALL UXVDIR(N,AA*DIS/(ONE+UU),SO,S,S)
      VV=-POM*DIS/UU
      ENDIF
      ENDIF
C
C     COMPUTATION OF P1
C
      CALL UXVDIR(N,BB*VV,S,GO,SO)
      CALL UXVDIR(N,-SQRT(PAR*AA/B),XO,SO,SO)
C
C     RANK TWO UPDATE OF XM
C
      CALL UXDCMV(N,M,XM,VV,S,XR,-ONE/AA,SO,GR)
      TUXX='RANK TWO'
      NUPD=NUPD+1
      ENDIF
      IF (GAM.EQ.ONE) GO TO 22
C
C     SCALING
C
      CALL UXVSCL(N*M,SQRT(GAM),XM,XM)
      NSCL=NSCL+1
   22 CONTINUE
      ITERH=0
C      CALL UOU1D3(N,H,S,RHO,GAM,PAR,A,B,C,POM,ETA9)
      RETURN
      END

