* SUBROUTINE UULVPS                ALL SYSTEMS                02/12/01
C PORTABILITY : ALL SYSTEMS
C 02/12/01 VL : ORIGINAL VERSION
*
* PURPOSE :
* LIMITED-STORAGE VARIABLE-METRIC UPDATES IN THE PRODUCT FORM.
*
* PARAMETERS :
*  II  N  NUMBER OF VARIABLES (NUMBER OF ROWS OF XM).
*  II  M  NUMBER OF COLUMNS OF XM.
*  RI  UU(N*M)  RECTANGULAR MATRIX IN THE PRODUCT FORM SHIFTED BROYDEN
*         METHOD (STORED COLUMNWISE): H-SIGMA*I=XM*TRANS(XM)
*  RA  ZZ(3*N) AUXILIARY ARRAY.
*  RU  S(N)  VECTORS OF VARIABLES DIFFERENCE.
*  RA  AY(N) VECTOR XM*TRANS(XM)*GO.
*  RA  BS(N)  VECTOR H**(-1)*XO.
*  RU  Y(N)  GRADIENT DIFFERENCE.
*  RO  UY(M)  MATRIX TRANS(XM)*GO.
*  RO  UBS(M)  VECTOR TRANS(XM)*H**(-1)*XO.
*  RI  ZET  SHIFT PARAMETER ZETA.
*  RU  SIG  SHIFT PARAMETER SIGMA.
*  RO  ETA  BROYDEN PARAMETER.
*  RO  GAM  SCALING PARAMETER.
*  RI  RHO  NONQUADRATIC CORRECTION PARAMETER.
*  RI  A  INPUT PARAMETER.
*  RI  AH  INPUT PARAMETER.
*  RI  AP  INPUT PARAMETER.
*  RI  B  INPUT PARAMETER.
*  RI  BP  INPUT PARAMETER.
*  RI  C  INPUT PARAMETER.
*  RI  CP  INPUT PARAMETER.
*  RI  MET5  CHOICE OF THE LIMITED MEMORY VARIABLE METRIC METHOD.
*  IO  IER  ERROR INDICATOR.
*
* SUBPROGRAMS USED :
*
* METHOD :
* J.VLCEK, L.LUKSAN: NEW VARIABLE METRIC METHODS FOR UNCONSTRAINED
* MINIMIZATION COVERING THE LARGE-SCALE CASE. ICS REPORT V876-02,
* INSTITUTE OF COMPUTER SCIENCE, AVCR, PRAGUE 2002.
*
      SUBROUTINE UULVPS(N,M,UU,ZZ,S,AY,BS,Y,UY,UBS,ZET,SIG,ETA,GAM,
     & RHO,A,AH,AP,B,BP,C,CP,MET5,IER)
      INTEGER N,M,MET5,IER
      $FLOAT UU(N*M),ZZ(N*3),S(N),AY(N),BS(N),Y(N),UY(M),UBS(M),
     & ZET,SIG,ETA,GAM,RHO,A,AH,AP,B,BP,C,CP,ROV
      $FLOAT AQ(15),ET,LAM,OM,Q,R,THE,W,Z
      INTEGER I,J,K,L
      IER=0
      GOTO(5,10,20,30,40,50,60),MET5+1
C  *****  TYPE 1 METHOD  *****
   5  IF(CP.LE.0D0.OR.B-SIG*AH.LE.0D0)GOTO90
      ROV=SQRT(SQRT(ZET*AH/A)*SIG*AH/(B-SIG*AH))
      THE=SIGN(SQRT(RHO*ROV/GAM*(B-SIG*AH)/CP),BP)
      Q=SIGN(SQRT(GAM/RHO/ROV*(B-SIG*AH)/CP),BP)
      DO 6 I=1,N
      S(I)=-(S(I)-SIG*Y(I)+Q*(S(I)-ZET*BS(I)))/(B-SIG*AH+Q*BP)
   6  AY(I)=S(I)*THE
      GOTO99
C  *****  TYPE 1 METHOD  *****
  10  IF(CP.LT.0.0$P 0.OR.B-SIG*AH.LE.0.0$P 0)GOTO90
      ROV=SQRT(SQRT(1.0$P 0-AP/A)*SIG*AH/(B-SIG*AH))
      THE=SIGN(SQRT(GAM/RHO/ROV*CP/(B-SIG*AH)),BP)
      DO11 I=1,N
      AY(I)=(ZET*BS(I)-(S(I)-SIG*Y(I))*THE*RHO*ROV/GAM-S(I))/(CP+THE*BP)
  11  S(I)=THE*AY(I)
      GOTO99
C  *****  SIMPLE TYPE 2 METHOD  *****
  20  IF(AP.LE.0.0$P 0)GOTO90
C     IF(SQRT(A*C)-B.LE.0.0$P 0)GOTO90
      SIG=(B+B)/(AH+SQRT(AH*AH+4.0$P 0*B*AP*RHO/(GAM*ZET*ZET)))
      DO21 I=1,N
      W=AY(I)
      AY(I)=(W+ZET*Y(I)-A/B*S(I))/MAX(SQRT(A*C)-B,1.0$P-14)
  21  S(I)=ZET/(AP*SIG)*S(I)-(W+ZET*Y(I)+BP*AY(I))/AP
C 21  S(I)=((S(I)-SIG*Y(I))*SQRT(RHO/GAM*AP/(B-SIG*AH))-W-BP*AY(I))/AP
      GOTO99
C  *****  SIMPLIFIED METHOD NEAREST TO THE SHIFTED BROYDEN CLASS  *****
  30  IF(AP*CP-BP*BP.LE.0.0$P 0.OR.AP.LE.0.0$P 0)GOTO90
      DO31 I=1,N
      W=AY(I)
      AY(I)=(SQRT(ETA*(AP*CP-BP*BP))*(AP/(B-SIG*AH)*(S(I)-SIG*Y(I))
     & -AY(I))-AP*(S(I)-ZET*BS(I))+BP*W)/(AP*CP-BP*BP)
  31  S(I)=((S(I)-SIG*Y(I))*SQRT(RHO/GAM*AP/(B-SIG*AH))-W-BP*AY(I))/AP
      GOTO99
C  *****  METHOD NEAREST TO THE SHIFTED BROYDEN CLASS  *****
  40  IF(AP*CP-BP*BP.LE.0.0$P 0.OR.AP.LE.0.0$P 0)GOTO90
      Q=0.0$P 0
      W=0.0$P 0
      Z=0.0$P 0
      DO41 I=1,N
      ZZ(I)=AP*(S(I)-ZET*BS(I))-BP*AY(I)
      ZZ(N+I)=SQRT(ETA*(AP*CP-BP*BP))*(AP/(B-SIG*AH)*(S(I)-SIG*Y(I))
     & -AY(I))
      Q=Q+ZZ(I)*ZZ(I)
      W=W+ZZ(N+I)*ZZ(N+I)
  41  Z=Z+ZZ(I)*ZZ(N+I)
      LAM=(Q+W+SQRT((Q-W)**2+(Z+Z)**2))*0.5$P 0
      R=0.0$P 0
      IF(W.GT.Q)THEN
        DO42 I=1,N
        ZZ(N+N+I)=Z*ZZ(I)+(LAM-Q)*ZZ(N+I)
  42    R=R+ZZ(N+N+I)*ZZ(N+N+I)
      ELSE
        DO43 I=1,N
        ZZ(N+N+I)=(LAM-W)*ZZ(I)+Z*ZZ(N+I)
  43    R=R+ZZ(N+N+I)*ZZ(N+N+I)
      ENDIF
      DO44 I=1,N
      W=AY(I)
      AY(I)=(SQRT(LAM/R)*ZZ(N+N+I)-AP*(S(I)-ZET*BS(I))+BP*W)/
     & (AP*CP-BP*BP)
  44  S(I)=((S(I)-SIG*Y(I))*SQRT(RHO/GAM*AP/(B-SIG*AH))-W-BP*AY(I))/AP
      GOTO99
C  *****  METHOD WITH DIRECTION VECTOR AFTER SHIFTED BROYDEN CLASS  ****
  50  IF(AP*CP-BP*BP.LE.0.0$P 0.OR.AP.LE.0.0$P 0)GOTO90
      OM=AP/(B-SIG*AH)*(C-SIG*(B-BP)/ZET)-BP
      R=SQRT(1.0$P 0+ETA*OM*OM/(AP*CP-BP*BP))
      DO51 I=1,N
      W=AY(I)
      AY(I)=(ETA*OM*(AP/(B-SIG*AH)*(S(I)-SIG*Y(I))-W)+
     & AP*(1.0$P 0-R)*(S(I)-ZET*BS(I)-W*BP/AP))/((AP*CP-BP*BP)*R)
  51  S(I)=(S(I)-SIG*Y(I))/SQRT(AP*(B-SIG*AH)*GAM/RHO)-(W+BP*AY(I))/AP
      GOTO99
C  *****  METHOD NEAREST TO THE BROYDEN CLASS  *****
  60  ET=1.0$P 0
      IF(AP*CP-BP*BP.LE.0.0$P 0.OR.AP.LE.0.0$P 0)GOTO90
      DO61 I=1,N
      ZZ(I)=S(I)-B/AH*Y(I)
      ZZ(N+I)=AY(I)-AP/AH*Y(I)
  61  ZZ(N+N+I)=BS(I)-(B-BP)/(AH*ZET)*Y(I)
      CALL UXVJOR(N,ZZ,AQ,K)
      IF(K.EQ.0)GOTO90
      CALL UUZMZ1(N,K,AQ,ZZ,S,AY,BS,ZET,GAM,SIG,ETA,RHO,A,B,AH,AP,BP,CP)
      DO63 I=1,N
      Q=0.0$P 0
      DO62 J=1,K
  62  Q=Q+AQ(6+J)*ZZ((J-1)*N+I)
      W=AY(I)
      AY(I)=(Q*SQRT(AP*(AP*CP-BP*BP))-AP*(S(I)-ZET*BS(I))+BP*W)/
     & (AP*CP-BP*BP)
  63  S(I)=((S(I)-SIG*Y(I))*SQRT(RHO/GAM*AP/(B-SIG*AH))-W-BP*AY(I))/AP
      GOTO99
  90  IF(AP*(B-SIG*AH).GT.0.0$P 0)THEN
        DO1 I=1,N
        S(I)=((S(I)-SIG*Y(I))*SQRT(RHO/GAM*AP/(B-SIG*AH))-AY(I))/AP
   1    AY(I)=0.0$P 0
      ELSE
        DO2 I=1,N
        IF(CP*(B-SIG*AH).LE.0.0$P 0)GOTO91
        AY(I)=((S(I)-SIG*Y(I))*SQRT(RHO/GAM*CP/(B-SIG*AH))-
     &   AY(I)*SQRT(GAM/RHO*CP/(B-SIG*AH))-S(I)+ZET*BS(I))/CP
   2    S(I)=0.0$P 0
      ENDIF
  99  L=-N
      DO9 J=1,M
      L=L+N
      W=UY(J)
      Z=UBS(J)
      DO9 I=1,N
   9  UU(L+I)=SQRT(GAM)*(UU(L+I)+S(I)*W+AY(I)*Z)
      ZET=MAX(1D-9,SIG*RHO)
      RETURN
  91  IER=IER+1
      RETURN
      END
