$INTERFACE UUSMM1                                             97/12/01

  $HELP
    SELECTION OF SELF SCALING:
      1 - SELF SCALING SUPPRESSED
      2 - INITIAL SELF SCALING
      3 - SELF SCALING IN EACH ITERATION
  $CHECK(MET1,1,'1\2\3',4)
  $ADD(REAL,'\$H($M)\$GN($NF)\$XS($NF)\$S($NF)\$XO($NF)\$GO($NF)')
$REM  $ADD(REAL,'\R\PO')
  $ADD(INTEGER,'\$IH($NF+1)\$JH($M)\$IX($NX)')
$REM  $ADD(INTEGER,'\NF\M')
  CALL UUSMM1(NF,$IH(NF+1)-1,$H,$IH,$JH,$GN,$XS,$S,$XO,$GO,$IX,R,PO&
,$MET1)
  $ADD(INIT)
    NUXX='UUSMM1'
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UUSMM1                ALL SYSTEMS                97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* VARIABLE METRIC UPDATE OF A SPARSE SYMMETRIC POSITIVE DEFINITE MATRIX.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  M  NUMBER OF NONZERO ELEMENTS OF THE MATRIX H.
*  RU  H(M)  POSITIVE DEFINITE APPROXIMATION OF THE SPARSE HESSIAN
*         MATRIX.
*  II  IH(NF)  POINTERS OF THE DIAGONAL ELEMENTS OF H.
*  II  JH(M)  INDICES OF THE NONZERO ELEMENTS OF H.
*  RI  G(NF)  GRADIENT OF THE OBJECTIVE FUNCTION.
*  RA  XS(NF)  AUXILIARY WECTOR.
*  RA  S(NF)  AUXILIARY VECTOR.
*  RU  XO(NF)  VECTORS OF VARIABLES DIFFERENCE.
*  RI  GO(NF)  GRADIENTS DIFFERENCE.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RO  R  VALUE OF THE STEPSIZE PARAMETER.
*  RI  PO  INITIAL VALUE OF THE DIRECTIONAL DERIVATIVE.
*  II  MET1  SELECTION OF SELF SCALING. MET1=1-SELF SCALING SUPPRESSED.
*         MET1=2-INITIAL SELF SCALING. MET1=3-SELF SCALING IN EACH
*         ITERATION.
*
* COMMON DATA :
*  IO  ITERH  TERMINATION INDICATOR. ITERH<0-BAD DECOMPOSITION.
*         ITERH=0-SUCCESSFUL UPDATE. ITERH>0-NONPOSITIVE PARAMETERS.
*  II  ITERD  TERMINATION INDICATOR. ITERD<0-BAD DECOMPOSITION.
*         ITERD=0-DESCENT DIRECTION. ITERD=1-NEWTON LIKE STEP.
*         ITERD=2-INEXACT NEWTON LIKE STEP. ITERD=3-BOUNDARY STEP.
*         ITERD=4-DIRECTION WITH THE NEGATIVE CURVATURE.
*         ITERD=5-MARQUARDT STEP.
*  II  IDECF  DECOMPOSITION INDICATOR. IDECF=0-NO DECOMPOSITION.
*  II  NIT  ACTUAL NUMBER OF ITERATIONS.
*  II  KIT  NUMBER OF THE ITERATION AFTER LAST RESTART.
*  TO  TUXX  TEXT INFORMATION ON THE CORRECTION USED.
*
* SUBPROGRAMS USED :
*  S   UXSSMM  MATRIX-VECTOR PRODUCT.
*  S   UXUDIF  DIFFERENCE OF TWO VECTORS.
*  S   UXUDIR  VECTOR AUGMENTED BY THE SCALED VECTOR.
*  RF  UXUDOT  DOT PRODUCT OF VECTORS.
*  S   UXVSCL  SCALING OF A VECTOR.
*  S   UOERR1  ERROR MESAGES.
*
      SUBROUTINE UUSMM1(NF,M,H,IH,JH,G,XS,S,XO,GO,IX,R,PO,MET1)
      $INCLUDE('UMCOMM')
      INTEGER NF,M,IH(NF+1),JH(M),IX(NF),MET1
      $FLOAT H(M),G(NF),S(NF),XO(NF),GO(NF),XS(NF),R,PO
      INTEGER MM,I,J,JSTRT,JSTOP,K
      $FLOAT UXUDOT
      $FLOAT A,B,C,GAM,TEMP,SIGMA
      LOGICAL L1
      IF (IDECF.NE.0) THEN
      ITERH=-1
      TUXX='BAD DEC0'
      CALL UOERR1('UUSMM1',1)
      GO TO 500
      ENDIF
      MM=IH(NF+1)-1
C
C     DETERMINATION OF THE PARAMETER C AND THE VECTOR S
C
      A=ZERO
      L1=MET1.GE.3.OR.MET1.GE.2.AND.NIT.EQ.KIT
      IF (ITERD.NE.1) THEN
      CALL UXSSMM(NF,MM,H,IH,JH,XO,S)
      IF (L1) C=UXUDOT(NF,XO,S,IX,KBF)
      ELSE
      CALL UXUDIF(NF,GO,G,S,IX,KBF)
      CALL UXVSCL(NF,R,S,S)
      IF (L1) C=-R*PO
      ENDIF
      GAM=ONE
      IF (L1) THEN
C
C     SELF SCALING
C
      B=UXUDOT(NF,XO,GO,IX,KBF)
      IF (B.GT.ZERO.AND.C.GT.ZERO) THEN
      GAM=C/B
      CALL UXVSCL(MM,ONE/GAM,H,H)
      CALL UXVSCL(NF,ONE/GAM,S,S)
      NSCL=NSCL+1
      ENDIF
      ENDIF
      CALL UXUDIR(NF,-ONE,S,GO,S,IX,KBF)
      CALL UXVSET(NF,ZERO,XS)
C
C  COMPUTE SIGMA(I)
C
      JSTOP=0
      DO 200 I=1,NF
        JSTRT=JSTOP+1
        JSTOP=IH(I+1)-1
        IF (JH(JSTRT).GT.0) THEN
          SIGMA=ZERO
          DO 100 J=JSTRT,JSTOP
            K=JH(J)
            IF (K.GT.0) THEN
              SIGMA=SIGMA+XO(K)*XO(K)
              IF(K.NE.I) XS(K)=XS(K)+XO(I)*XO(I)
            ENDIF
100       CONTINUE
          XS(I)=XS(I)+SIGMA
        ENDIF
200   CONTINUE
C
C  UPDATE MATRIX
C
      JSTOP=0
      DO 400 I=1,NF
        JSTRT=JSTOP+1
        JSTOP=IH(I+1)-1
        IF (JH(JSTRT).GT.0) THEN
          IF(XS(I).EQ.ZERO) THEN
            TEMP=ZERO
          ELSE
            TEMP=S(I)/XS(I)
          ENDIF
          DO 300 J=JSTRT,JSTOP
            K=JH(J)
            IF (K.GT.0) THEN
              IF(XS(K).EQ.ZERO) THEN
              H(J)=H(J)+HALF*TEMP*XO(K)
              ELSE
              H(J)=H(J)+HALF*(TEMP*XO(K)+XO(I)*S(K)/XS(K))
              ENDIF
            ENDIF
300       CONTINUE
        ENDIF
400   CONTINUE
      TUXX='MARWIL  '
      NUPD=NUPD+1
      ITERH=0
500   CONTINUE
      RETURN
      END
