$INTERFACE UUSSD6                                             97/12/01

  $HELP
    SWITCH FOR THE HYBRID METHOD:
      0 - THE FLETCHER AND XU SWITCH WITH GENERAL UPDATE
      1 - THE FLETCHER AND XU SWITCH WITH SPECIAL UPDATE
      2 - THE DENNIS AND WELSCH SWITCH
      3 - THE RAMSIN AND WEDIN SWITCH
  $CHECK(MOT2,0,'0\1\2\3',3)
  $HELP
    CHOICE OF THE INITIAL APPROXIMATION OF THE HESSIAN
    MATRIX:
      1 - UNIT INITIAL APPROXIMATION
      2 - RANDOM INITIAL APPROXIMATION
  $CHECK(INITH,1,'1\2',4)
  $DEFAULT='5.0D-4'
  $IF(TYPE='L') $DEFAULT='4.0D-2'
  $HELP
    TOLERANCE FOR APPLICATION OF THE BFGS METHOD
  $CHECK(ETA5,DEFAULT,REAL,4)
  $N='NF'
  $ADD(REAL,'\$AF($NA)\$H($M)\$AH($MAH)\PPJ\PPB')
$REM  $ADD(REAL,'\R\F\FO\PO\ETA5')
  $ADD(INTEGER,'\$IA($NA)\$IH($NF+1)\$JH($M)\$IAG($NA+1)\$JAG($MA)&
  \IPOM')
$REM  $ADD(INTEGER,'\$N\NA\M\MA\MAH')
  CALL UUSSD6($N,NA,M,MA,MAH,$IA,$AF,$H,$IH,$JH,$AH,$IAG,$JAG,R,F,FO,PO&
,ETA5,PPJ,PPB,IPOM,$MOT2,$INITH)
  $ADD(SINIT)
    CALL UXVINS(NA,3,$IA)
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UUSSD6                ALL SYSTEMS                97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* INITIATION OF A DENSE SYMMETRIC POSITIVE DEFINITE MATRIX
*
* PARAMETERS :
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  II  NA  NUMBER OF APPROXIMATED FUNCTIONS.
*  II  MA  NUMBER OF NONZERO ELEMENTS OF THE JACOBIAN MATRIX.
*  II  M  NUMBER OF NONZERO ELEMENTS OF THE SPARSE HESSIAN MATRIX.
*  II  MB  NUMBER OF NONZERO ELEMENTS OF THE PARTITIONED HESSIAN
*         MATRIX.
*  II  IA(NA)  TYPES OF APPROXIMATED FUNCTIONS.
*  RI  AF(NA)  VECTOR CONTAINING VALUES OF THE APPROXIMATED
*         FUNCTIONS.
*  RU  H(M)  POSITIVE DEFINITE APPROXIMATION OF THE SPARSE HESSIAN
*         MATRIX
*  II  IH(NF+1)  POINTERS OF THE DIAGONAL ELEMENTS OF THE SPARSE
*         HESSIAN MATRIX.
*  II  JH(M)  INDICES OF THE NONZERO ELEMENTS OF THE SPARSE HESSIAN
*             MATRIX IN THE PACKED ROW FORM.
*  RU  AH(MB)  POSITIVE DEFINITE APPROXIMATION OF THE PARTITIONED
*         HESSIAN MATRIX.
*  II  IAG(NA+1)  POINTERS OF THE SPARSE JACOBIAN MATRIX.
*  II  JAG(MA)  COLUMN INDICES OF THE SPARSE JACOBIAN MATRIX.
*  RI  R  CURRENT STEPSIZE PARAMETER.
*  RI  F  CURRENT VALUE OF THE OBJECTIVE FUNCTION.
*  RI  FO  PREVIOUS VALUE OF THE OBJECTIVE FUNCTION.
*  RI  PO  PREVIOUS VALUE OF THE DIRECTIONAL DERIVATIVE.
*  RI  ETA5  TOLERANCE FOR A HYBRID METHOD.
*  RU  PPJ  QUADRATIC TERM WITH THE NORMAL EQUATION MATRIX.
*  RU  PPB  QUADRATIC TERM WITH A SECOND ORDER INFORMATION.
*  II  IPOM  INDICATION FOR SECOND ORDER INFORMATION.
*  II  MOT2  SWITCH FOR THE HYBRID METHOD. MOT2=0-THE FLETCHER AND XU
*         SWITCH WITH GENERAL UPDATE. MOT2=1-THE FLETCHER AND XU SWITCH
*         WITH SPECIAL UPDATE. MOT2=2-THE DENNIS AND WELSCH SWITCH.
*         MOT2=3-THE RAMSIN AND WEDIN SWITCH.
*  II  INITH  CHOICE OF THE INITIAL APPROXIMATION OF THE SPARSE
*         HESSIAN MATRIX. INITH=1-UNIT INITIAL APPROXIMATION.
*         INITH=2-RANDOM INITIAL APPROXIMATION.
* COMMON DATA :
*  RU  RAN  RANDOM NUMBER.
*  II  IDECF  DECOMPOSITION INDICATOR. IDECF=0-NO DECOMPOSITION.
*  II  IREST  RESTART SPECIFICATION. IF IREST=0 DOES NOT HOLD THEN A
*         RESTART IS PERFORMED.
*  II  LIT  NUMBER OF THE FIRST ITERATION.
*  IU  NIT  ACTUAL NUMBER OF ITERATIONS.
*  IU  LD  DEGREE OF PREVIOUSLY COMPUTED DERIVATIVES.
*  TO  TUXX   TEXT INFORMATION ON THE RESTART USED.
*
* SUBPROGRAMS USED :
*  S   UXBSMI  GENERATION OF THE UNIT MATRIX.
*  S   UXSSMI  GENERATION OF THE SPARSE UNIT MATRIX.
*  S   UXSSDO  INITIATION OF A SPARSE DIAGONAL MATRIX.
*  S   UASSH2  COMPUTATION OF THE SPARSE HESSIAN MATRIX FROM THE
*         PARTITIONED HESSIAN MATRIX.
*  RF  UNRAN1  RANDOM NUMBER GENERATOR.
*  S   UYSET3  DEFINITION OF THE RESTART VARIABLES.
*
      SUBROUTINE UUSSD6(N,NA,M,MA,MB,IA,AF,H,IH,JH,AH,IAG,JAG,R,F,FO,PO,
     & ETA5,PPJ,PPB,IPOM,MOT2,INITH)
      $INCLUDE('UMCOMM')
      INTEGER N,NA,M,MA,MB,IA(NA),IH(N+1),JH(M),IAG(NA+1),JAG(MA),
     & IPOM,MOT2,INITH
      $FLOAT AF(NA),H(M),AH(MB),R,F,FO,PO,ETA5,PPJ,PPB
      LOGICAL L1
      INTEGER K,L,LL,KA,NB
      $FLOAT POM,RHO,UNRAN1
      $FLOAT CON,CON2,CON3,CON4
      PARAMETER(CON=1.0$P 1,CON2=0.2$P 0,CON3=5.0$P-3,CON4=0.5$P 0)
      IF (IREST.LT.0) THEN
      CALL UXBSMI(MB,NA,AH,IAG)
      ELSE IF (IREST.EQ.0) THEN
      ELSEIF(INITH.EQ.1) THEN
      CALL UXSSMI(N,M,H,IH)
      TUXX='UNIT    '
      LD=MIN(LD,1)
      CALL UYSET3
      ELSEIF(INITH.EQ.2) THEN
      POM=(CON-ONE)/(CON+ONE)
      POM=POM/$DBLE(N+1)
      DO 1 K=1,IH(N+1)-1
      H(K)=POM*(TWO*UNRAN1(RAN)-ONE)
    1 CONTINUE
      CALL UXSSDO(N,M,ONE,H,IH,JH)
      TUXX='RANDOM  '
      LD=MIN(LD,1)
      CALL UYSET3
      ENDIF
      IF (NIT.LE.LIT+1) THEN
      IPOM=0
      PPJ=ZERO
      PPB=ZERO
      RETURN
      ELSE IF (MOT2.EQ.2) THEN
      RHO=R*R
      PPJ=PPJ*RHO
      PPB=PPB*RHO
      ENDIF
      IF (IDIR.EQ.0) THEN
      IPOM=0
      IF (MOT2.EQ.3) THEN
      L1=IPOM.EQ.1.OR.PPJ.LT.CON3*FO.AND.PPJ.LT.CON4*PPB
      ELSE IF (MOT2.EQ.2) THEN
      L1=ABS((F-FO)/(PO+PPB)-ONE).LE.CON2*ABS((F-FO)/(PO+PPJ)-ONE)
      ELSE
      L1=FO-F.LE.ETA5*FO
      ENDIF
      IF (L1) IPOM=1
      IF (IPOM.EQ.1) THEN
      NB=0
      DO 2 KA=1,NA
      K=IAG(KA)
      L=IAG(KA+1)-K
      LL=L*(L+1)/2
      IF (IA(KA).GT.0) THEN
      CALL UASSH2(N,NA,MA,LL,M,H,IH,JH,AH(NB+1),IAG,JAG,KA,AF(KA))
      LD=MIN(LD,1)
      ENDIF
      NB=NB+LL
    2 CONTINUE
      ENDIF
      IDECF=0
      ENDIF
      RETURN
      END
