$INTERFACE UUTBM1                                             10/12/01

  $HELP
    SELECTION OF SELF SCALING:
      1 - SELF SCALING SUPPRESSED
      2 - INITIAL SELF SCALING
      3 - CONTROLLED SELF SCALING
      4 - SIMPLIFIED CONTROLLED SELF SCALING
      5 - SELF SCALING IN EACH ITERATION
  $CHECK(MET1,1,'1\2\3\4\5',4)
  $ADD(REAL,'\$DD($NF)\$DDU($NF)\$DDV($NF)\$G($NF)\$S($NF)\$XO($NF)&
  \$GO($NF)')
$REM  $ADD(REAL,'\R\PO\PAR1\PAR2')
$REM  $ADD(INTEGER,'\MET\MET1\MOS2')
  CALL UUTBM1($N,$DD,$DDU,$DDV,$G,$S,$XO,$GO,R,PO,PAR1,PAR2,MET,MET1&
,MOS2)
  $ADD(INIT)
    NUXX='UUTBM1'
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UUTBM1                ALL SYSTEMS                10/12/01
C PORTABILITY : ALL SYSTEMS
C 10/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* VARIABLE METRIC UPDATE OF A DENSE SYMMETRIC POSITIVE DEFINITE MATRIX.
*
* PARAMETERS :
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  RU  D(N)  THE DIAGONAL OF THE BAND PRECONDITIONER.
*  RU  DU(N)  THE FIRST SUPERDIAGONAL OF THE BAND PRECONDITIONER.
*  RU  DV(N)  THE SECOND SUPERDIAGONAL OF THE BAND PRECONDITIONER.
*  RA  G(NF)  GRADIENT VECTOR.
*  RA  S(NF)  AUXILIARY VECTOR.
*  RU  XO(NF)  VECTORS OF VARIABLES DIFFERENCE.
*  RI  GO(NF)  GRADIENTS DIFFERENCE.
*  RI  R
*  RI  PO
*  RI  PAR1
*  RI  PAR2
*  II  MET1  SELECTION OF SELF SCALING.  MET1=1-SELF SCALING SUPPRESSED.
*         MET1=2 SELF SCALING IN THE FIRST ITERATION AFTER RESTART.
*         MET1=3-SELF SCALING IN EACH ITERATION.
*  MOS2
*
* COMMON DATA :
*  II  ITRAN
*  IO  ITERH  TERMINATION INDICATOR. ITERH<0-BAD DECOMPOSITION.
*         ITERH=0-SUCCESSFUL UPDATE. ITERH>0-NONPOSITIVE PARAMETERS.
*  TO  TUXX  TEXT INFORMATION ON THE CORRECTION USED.
*
* SUBPROGRAMS USED :
*  S   UXVSMU  CORRECTION OF A DIAGONAL MATRIX.
*  S   UXTSMU  CORRECTION OF A SYMMETRIC TRIDIAGONAL MATRIX.
*  S   UXPSMU  CORRECTION OF A SYMMETRIC PENTADIAGONAL MATRIX.
*  S   UXVDIF
*  RF  UXVDOT  DOT PRODUCT OF VECTORS.
*  S   UXVNEG
*  S   UXVSCL
*
* METHOD :
* BFGS UPDATE FOR DIAGONAL, TRIDIAGONAL AND PENTADIAGONAL MATRICES.
*
      SUBROUTINE UUTBM1(N,D,DU,DV,G,S,XO,GO,R,PO,PAR1,PAR2,MET,MET1,
     & MOS2)
      $INCLUDE('UMCOMM')
      INTEGER N,MET,MET1,MOS2
      $FLOAT D(N),DU(N),DV(N),G(N),S(N),XO(N),GO(N),R,PO,PAR1,PAR2
      $FLOAT UXVDOT
      $FLOAT A,B,C,RHO,GAM,PAR,POM,POM3,POM4,CON3
      LOGICAL L1,L2,L3
      PARAMETER (CON3=0.2$P 0)
      IF (MET.LE.0) RETURN
      L1=MET1.GE.3.OR.MET1.EQ.2.AND.NIT.EQ.KIT
      L3=.NOT.L1
C
C     DETERMINATION OF THE PARAMETERS A,B,C
C
      A=ZERO
      B=UXVDOT(N,XO,GO)
      IF (B.LE.ZERO) THEN
      ITERH=2
      TUXX='B - NEG.'
      GO TO 2
      ENDIF
      IF (ITRAN.EQ.0) THEN
      CALL UXVDIF(N,GO,G,S)
      ELSE
      CALL UXVNEG(N,G,S)
      ENDIF
      CALL UXVSCL(N,R,S,S)
      C=-R*PO
      IF (C.LE.ZERO) THEN
      ITERH=3
      TUXX='C - NEG.'
      GO TO 2
      ENDIF
      RHO=ONE
      IF (L1)  THEN
C
C     DETERMINATION OF THE PARAMETER GAM (SELF SCALING)
C
      PAR=C/B
      POM3=0.6$P 0
      POM4=4.0$P 0
      GAM=PAR*RHO
      IF (MET1.EQ.3) THEN
      IF (NIT.NE.KIT) THEN
      L2=PAR2.LE.ZERO
      L3=L2.AND.ABS(PAR1).LE.CON3
      L3=L3.OR.(.NOT.L2.AND.GAM.GT.ONE)
      L3=L3.OR.(L2.AND.PAR1.LT.ZERO.AND.GAM.GT.ONE)
      L3=L3.OR.(L2.AND.PAR1.GT.ZERO.AND.GAM.LT.ONE)
      L3=L3.OR.GAM.LT.POM3
      L3=L3.OR.GAM.GT.POM4
      ENDIF
      ELSE IF (MET1.EQ.4) THEN
      IF (NIT.NE.KIT) THEN
      L3=GAM.LT.POM3.OR.GAM.GT.POM4
      ENDIF
      ENDIF
      ENDIF
      IF (L3) THEN
      GAM=ONE
      PAR=GAM/RHO
      ENDIF
C
C     BFGS UPDATE
C
      POM=ZERO
      IF (MOS2.EQ.3) THEN
      CALL UXVSMU(N,D,PAR/B,GO)
      CALL UXVSMU(N,D,-ONE/C,S)
      ELSE IF (MOS2.EQ.4) THEN
      CALL UXTSMU(N,D,DU,PAR/B,GO)
      CALL UXTSMU(N,D,DU,-ONE/C,S)
      ELSE IF (MOS2.EQ.5) THEN
      CALL UXPSMU(N,D,DU,DV,PAR/B,GO)
      CALL UXPSMU(N,D,DU,DV,-ONE/C,S)
      ENDIF
      TUXX='BFGS    '
      NUPD=NUPD+1
      ITERH=0
      IF (GAM.EQ.ONE) GO TO 2
      NSCL=NSCL+1
      IF (MOS2.GE.3) CALL UXVSCL(N,ONE/GAM,D,D)
      IF (MOS2.GE.4) CALL UXVSCL(N,ONE/GAM,DU,DU)
      IF (MOS2.EQ.5) CALL UXVSCL(N,ONE/GAM,DV,DV)
    2 CONTINUE
      RETURN
      END
