* SUBROUTINE UXDCHC               ALL SYSTEMS                16/12/01
C PORTABILITY : ALL SYSTEMS
C 16/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* QR DECOMPOSITION OF TRANSPOSE OF A DENSE RECTANGULAR MATRIX A USING
* HOUSEHOLDER TRANSFORMATIONS WITH INCREMENTAL CONDITION ESTIMATION.
*
* PARAMETERS :
*  II  N  NUMBER OF COLUMNS OF THE MATRIX A.
*  II  M  NUMBER OF ROWS OF THE MATRIX A.
*  RU  A(M*N)  RECTANGULAR MATRIX STORED COLUMNWISE IN THE
*         ONE-DIMENSIONAL ARRAY.
*  IO  IA(N)  A PERMUTATION FIELD WHICH CONTAINS AN INFORMATION ABOUT
*         PERMUTATION MATRIX.
*  RI  TOL  TOLERANCE FOR PSEUDORANK DETERMINATION.
*  RO  D(N)  VECTOR CONTAINING INFORMATION ABOUT HOUSEHOLDER
*         TRANSFORMATIONS.
*  RA  E(N)  AUXILIARY VECTOR.
*  RA  V(N)  AUXILIARY VECTOR.
*  IO  NM  ASSUMED RANK OF THE MATRIX A.
*  IO  NP  PSEUDORANK OF THE MATRIX A.
*
* METHOD :
* C.H.BISCHOF : INCREMENTAL CONDITION ESTIMATION. SIAM J. MATRIX ANAL.
* APPL. 11 (1990) 312-322.
*
      SUBROUTINE UXDCHC(N,M,A,IA,TOL,D,E,V,NM,NP)
      INTEGER N,M,IA(N),NM,NP
      $FLOAT A(N*M),TOL,D(N),E(N),V(N)
      $FLOAT POM,DEN,XEST,XPOM,XNEW,ALF,BET,ETA,ETAI,EPS,EPSI,S,C,
     & TEMP,TOL1,UXVDOT,UXVNOR
      INTEGER I,J,K,L,IP,JP,KP,MODE
      $FLOAT ZERO,ONE,TWO,HALF,CON
      PARAMETER (ZERO=0.0$P 0,ONE=1.0$P 0,TWO=2.0$P 0,HALF=5.0$P-1,
     * CON=5.0$P-2)
      NM=MIN(N,M)
      DO 1 I=1,N
      IA(I)=I
      D(I)=UXVNOR(M,A((I-1)*M+1))
      E(I)=D(I)
      V(I)=ZERO
    1 CONTINUE
      XEST=ZERO
      DO 4 K=1,NM
      L=0
      XNEW=ZERO
      DO 2 I=K,N
      J=IA(I)
      ALF=V(J)
      DEN=D(J)**2
      IF (ALF.EQ.ZERO) THEN
      IF (XEST*DEN.GT.ONE) THEN
      XPOM=ONE/XEST
      JP=1
      ELSE
      XPOM=DEN
      JP=2
      ENDIF
      IF (XNEW.LT.XPOM) THEN
      XNEW=XPOM
      MODE=JP
      L=I
      ENDIF
      ELSE
      BET=DEN*XEST+ALF**2-ONE
      ETAI=HALF*BET/ABS(ALF)
      EPSI=SQRT(ETAI**2+ONE)
      XPOM=DEN/(ONE+ABS(ALF)*(ETAI+EPSI))
      IF (XNEW.LT.XPOM) THEN
      XNEW=XPOM
      ETA=ETAI
      EPS=EPSI
      MODE=0
      L=I
      ENDIF
      ENDIF
    2 CONTINUE
      IF (L.EQ.0) THEN
      NM=K-1
      GO TO 5
      ENDIF
      J=IA(L)
      IA(L)=IA(K)
      IA(K)=J
      L=J
      KP=(J-1)*M
      IF (MODE.EQ.1) THEN
      S=ONE
      C=ZERO
      ELSE IF (MODE.EQ.2) THEN
      S=ZERO
      C=ONE
      ELSE
      S=SQRT((ONE+ETA/EPS)/TWO)
      C=SQRT((ONE-ETA/EPS)/TWO)
      ENDIF
      XEST=ONE/XNEW
      POM=UXVNOR(M-K+1,A(KP+K))
      IF (A(KP+K).NE.ZERO) POM=SIGN(POM,A(KP+K))
C
C     HOUSEHOLDER TRANSFORMATION
C
      CALL UXVSCL(M-K+1,ONE/POM,A(KP+K),A(KP+K))
      A(KP+K)=A(KP+K)+ONE
      DO 3 I=K+1,N
      J=IA(I)
      IP=(J-1)*M
      ALF=UXVDOT(M-K+1,A(KP+K),A(IP+K))/A(KP+K)
      CALL UXVDIR(M-K+1,-ALF,A(KP+K),A(IP+K),A(IP+K))
C
C     COLUMN NORMS UPDATE
C
      V(J)=S*V(J)+D(J)*(C-V(L)*S)/D(L)
      IF (D(J).NE.ZERO) THEN
      ALF=ONE-(A(IP+K)/D(J))**2
      ALF=MAX(ALF,ZERO)
      BET=ONE+CON*ALF*(D(J)/E(J))**2
      IF (BET.GT.ONE) THEN
      D(J)=D(J)*SQRT(ALF)
      ELSE
      D(J)=UXVNOR(M-K,A(IP+K+1))
      E(J)=D(J)
      ENDIF
      ENDIF
    3 CONTINUE
      V(L)=SQRT(XNEW)
      D(L)=A(KP+K)
      A(KP+K)=-POM
    4 CONTINUE
    5 TOL1=SQRT(TOL)
      TEMP=TOL1
      NP=0
      DO 6 K=1,NM
      J=IA(K)
      POM=ABS(V(J))
      IF (K.EQ.1) TEMP=TOL1*MAX(TEMP,POM)
      IF (POM.LE.TEMP) RETURN
      NP=K
    6 CONTINUE
      RETURN
      END
