* SUBROUTINE UXDGGV                ALL SYSTEMS                16/12/01
C PORTABILITY : ALL SYSTEMS
C 16/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* RANK ONE UPDATE OF FACTORIZATION P*A=L*U OBTAINED BY THE SUBROUTINE
* UXDGGF.
*
* PARAMETERS :
*  II  N ORDER OF THE MATRIX A.
*  RU  A(N*N) THE FACTORIZATION P*A=L*U COMPUTED BY THE SUBROUTINE 
*         UXDGGF.
*  IU  IA(N) A PERMUTATION FIELD WHICH CONTAINS AN INFORMATION ABOUT
*         PERMUTATION MATRIX P.
*  RU  X(N) INPUT VECTOR.
*  RU  Y(N) INPUT VECTOR.
*  RU  D(N) AUXILIARY VECTOR
*  RU  Z(N) AUXILIARY VECTOR
*  IO  INF INDICATOR. INF=0 - CORRECT COMPUTATION. INF=1 - DIVISION BY 
*         ZERO.       
*
* METHOD :
* BACK SUBSTITUTION.
*
      SUBROUTINE UXDGGV(N,A,IA,X,Y,D,Z,INF)
      INTEGER N,IA(N),INF
      $FLOAT A(N*N),X(N),Y(N),D(N),Z(N)
      INTEGER J,JP,K,KP,L,LP
      $FLOAT Q,TAU,ZERO,ONE
      PARAMETER (ZERO=0.0$P 0,ONE=1.0$P 0)
      INF=0
C      
C     TRANSFORMATION OF THE INPUT VECTOR
C
      CALL UXVGFP(N,IA,X,Z)
      DO 20 K=1,N
      KP=IA(K)
      DO 10 J=1,K-1
      X(K)=X(K)-A(KP+J)*X(J)
   10 CONTINUE
   20 CONTINUE      
C      
C     THE BACKWARD CYCLE - ZEROING ELEMENTS OF THE INPUT VECTOR
C
      D(N)=ONE
      DO 30 K=N-1,1,-1
      D(K)=ONE
      L=K+1
      KP=IA(K)
      LP=IA(L)
      IF (X(L).NE.ZERO) THEN
C      IF (ABS(X(K)).LT.1.0$P-1*ABS(X(L))) THEN
      IF (ABS(X(K)).LT.1.0$P-1*ABS(A(LP+K)*X(K)+X(L))) THEN 
C      
C     POSSIBLE PERMUTATION
C
      IF (L.LT.N) THEN
      DO 22 J=K+2,N
      JP=IA(J)
      CALL UXVSWP(1,A(JP+K),A(JP+L),1)
   22 CONTINUE   
      ENDIF
      CALL UXVSWP(1,D(K),D(L),1)
      CALL UXVSWP(1,X(K),X(L),1)
      Q=A(LP+K)
      A(LP+K)=ZERO
      Z(K)=A(KP+K)
      A(KP+K)=ZERO
      JP=KP
      KP=LP
      LP=JP
      IA(K)=KP
      IA(L)=LP
      ELSE
      Q=ZERO
      Z(K)=ZERO
      ENDIF 
C      
C     ZEROING THE ELEMENT X(K+1)
C
      TAU=-X(L)/X(K)
      CALL UXVELT(N-K,A(KP+L),A(LP+L),1,TAU)
      Z(K)=Z(K)+TAU*A(KP+K)
      IF (L.LT.N) THEN
      DO 24 J=K+2,N
      JP=IA(J)
      CALL UXVELT(1,A(JP+L),A(JP+K),1,-TAU)
   24 CONTINUE   
      ENDIF
      D(K)=D(K)-TAU*Q
      A(LP+K)=A(LP+K)-TAU*D(L)  
C      
C     ZEROING THE ELEMENT L(K,K+1)
C
      IF (Q.NE.ZERO) THEN
      TAU=-Q/D(K)
      D(L)=D(L)+TAU*A(LP+K)
      IF (L.LT.N) THEN
      DO 26 J=K+2,N
      JP=IA(J)
      A(JP+L)=A(JP+L)+TAU*A(JP+K)
C      CALL UXVELT(1,A(JP+K),A(JP+L),1,TAU)
   26 CONTINUE   
      ENDIF
      CALL UXVELT(N-K,A(LP+L),A(KP+L),1,-TAU)
      A(KP+K)=A(KP+K)-TAU*Z(K)
      ENDIF
      ELSE
      Z(K)=ZERO
      ENDIF
   30 CONTINUE
C      
C     UPDATE OF THE UPPER HESSENBERG MATRIX
C
      KP=IA(1)
      DO 40 K=1,N
      A(KP+K)=A(KP+K)+X(1)*Y(K)
   40 CONTINUE     
C      
C     THE FORWARD CYCLE - ZEROING OFF-DIAGONAL ELEMENTS OF THE UPPER 
C     HESSENBERG MATRIX
C
      DO 50 K=1,N-1
      L=K+1
      KP=IA(K)
      LP=IA(L)
      IF (Z(K).NE.ZERO) THEN 
C      IF (ABS(A(KP+K)).LT.1.0$P-1*ABS(Z(K))) THEN
      IF (ABS(A(KP+K)).LT.1.0$P-1*ABS(A(LP+K)*A(KP+K)+Z(K))) THEN 
C      
C     POSSIBLE PERMUTATION
C
      IF (L.LT.N) THEN
      DO 42 J=K+2,N
      JP=IA(J)
      CALL UXVSWP(1,A(JP+K),A(JP+L),1)
   42 CONTINUE   
      ENDIF
      CALL UXVSWP(1,D(K),D(L),1)
      Q=A(LP+K)
      A(LP+K)=Z(K)
      Z(K)=A(KP+K)
      A(KP+K)=ZERO
      JP=KP
      KP=LP
      LP=JP
      IA(K)=KP
      IA(L)=LP
      ELSE
      Q=ZERO
      ENDIF 
C      
C     ZEROING THE ELEMENT U(K+1,K)
C
      TAU=-Z(K)/A(KP+K)
      CALL UXVELT(N-K,A(KP+L),A(LP+L),1,TAU)
      IF (L.LT.N) THEN
      DO 44 J=K+2,N
      JP=IA(J)
      A(JP+K)=A(JP+K)-TAU*A(JP+L)
C      CALL UXVELT(1,A(JP+L),A(JP+K),1,-TAU)
   44 CONTINUE   
      ENDIF
      D(K)=D(K)-TAU*Q
      A(LP+K)=A(LP+K)-TAU*D(L)  
C      
C     ZEROING THE ELEMENT L(K,K+1)
C
      IF (Q.NE.ZERO) THEN
      TAU=-Q/D(K)
      D(L)=D(L)+TAU*A(LP+K)
      IF (L.LT.N) THEN
      DO 46 J=K+2,N
      JP=IA(J)
      A(JP+L)=A(JP+L)+TAU*A(JP+K)
C      CALL UXVELT(1,A(JP+K),A(JP+L),1,TAU)
   46 CONTINUE   
      ENDIF
      CALL UXVELT(N-K,A(LP+L),A(KP+L),1,-TAU)
      ENDIF
      ENDIF
  50  CONTINUE       
C      
C     FINAL SCALING
C
      DO 60 K=1,N
      IF (D(K).NE.ONE) THEN
      KP=IA(K)
      IF (K.LT.N) THEN
      Q=ONE/D(K)
      DO 52 J=K+1,N
      JP=IA(J)
      A(JP+K)=A(JP+K)*Q
C      CALL UXVSCG(1,ONE/D(K),A(JP+K),A(JP+K),1)
   52 CONTINUE   
      ENDIF
      CALL UXVSCG(N-K+1,D(K),A(KP+K),A(KP+K),1)
      ENDIF
   60 CONTINUE
      RETURN
      END       
        
      
