* SUBROUTINE UXDLBB                ALL SYSTEMS                92/12/01
C PORTABILITY : ALL SYSTEMS
C 92/12/01 MA : ORIGINAL VERSION
*
* PURPOSE :
* SOLUTION OF A SYSTEM OF LINEAR EQUATIONS WITH A DENSE SYMMETRIC
* MATRIX A USING THE FACTORIZATION A=P*U*D*TRANS(U)*TRANS(P) OBTAINED
* BY THE SUBROUTINE UXDLBF.
*
* PARAMETERS :
*  II  N ORDER OF THE MATRIX A.
*  RI  A(N*(N+1)/2)  FACTORIZATION A=P*U*D*TRANS(U)*TRANS(P) OBTAINED
*         BY THE SUBROUTINE UXDLBF.
*  II  IA(N)  PERMUTATION FIELD OBTAINED BY THE SUBROUTINE UXDLBF.
*  RU  X(N)  ON INPUT THE RIGHT HAND SIDE OF A SYSTEM OF LINEAR
*         EQUATIONS. ON OUTPUT THE SOLUTION OF A SYSTEM OF LINEAR
*         EQUATIONS.
*  II  JOB  OPTION. IF JOB=0 THEN X:=A**(-1)*X. IF JOB>0 THEN
*         X:=(P*U)**(-1)*X. IF JOB<0 THEN X:=(TRANS(U)*TRANS(P))**(-1)*X
*
* METHOD :
* BACK SUBSTITUTION.
*
      SUBROUTINE UXDLBB(N,A,IA,X,JOB)
      $FLOAT ONE
      PARAMETER ( ONE = 1.0D+0 )
      INTEGER N, IA(N), JOB
      $FLOAT A(N*(N+1)/2), X(N)
      INTEGER K, KC, KP
      $FLOAT AK, AKM1, AKM1K, BK, BKM1, DENOM
*
*     Solve A*X = B, where A = U*D*U**T.
*
      IF (JOB .GE. 0) THEN
*
*         First solve U*D*X = B, overwriting B with X.
*
*         K is the main loop index, decreasing from N to 1 in steps of
*         1 or 2, depending on the size of the diagonal blocks.
*
          K = N
          KC = N*( N+1 ) / 2 + 1
   10     CONTINUE
*
*         If K < 1, exit from loop.
*
          IF( K.LT.1 ) GO TO 30
          KC = KC - K
          IF( IA( K ).GT.0 ) THEN
*
*             1 x 1 diagonal block, interchange elements K <-> IA(K).
*
              KP = IA( K )
              IF( KP.NE.K ) CALL UNSWAP(X(K),X(KP))
*
*             Multiply by inv(U(K)), where U(K) is the transformation
*             stored in column K of A.
*
              IF (K.GT.1) CALL UXVDIR(K-1,-X(K),A(KC),X(1),X(1))
              IF (JOB .EQ. 0) THEN
*
*                 Multiply by the inverse of the diagonal block.
*
                  X(K) = X(K) / A( KC+K-1 )
              END IF
              K = K - 1
          ELSE
*
*             2 x 2 diagonal block, interchange elements
*             K <-> -IA(K) and K-1 <-> -IA(K-1).
*
              KP = -IA( K )
              IF( KP.NE.K ) CALL UNSWAP(X(K),X(KP))
              KP = -IA( K-1 )
              IF( KP.NE.K-1 ) CALL UNSWAP(X(K-1),X(KP))
*
*             Multiply by inv(U(K)), where U(K) is the transformation
*             stored in columns K-1 and K of A.
*
              IF (K.GT.2) THEN
                  CALL UXVDIR(K-2,-X(K),A(KC),X(1),X(1))
                  CALL UXVDIR(K-2,-X(K-1),A(KC-(K-1)),X(1),X(1))
              END IF
              IF (JOB .EQ. 0) THEN
*
*                 Multiply by the inverse of the diagonal block.
*
                  AKM1K = A( KC+K-2 )
                  AKM1 = A( KC-1 ) / AKM1K
                  AK = A( KC+K-1 ) / AKM1K
                  DENOM = AKM1*AK - ONE
                  BKM1 = X( K-1 ) / AKM1K
                  BK = X( K ) / AKM1K
                  X( K-1 ) = ( AK*BKM1-BK ) / DENOM
                  X( K ) = ( AKM1*BK-BKM1 ) / DENOM
              END IF
              KC = KC - K + 1
              K = K - 2
          END IF
          GO TO 10
      END IF
   30 CONTINUE
      IF (JOB .LE. 0) THEN
*
*         Next solve U**T*X = B, overwriting B with X.
*
*         K is the main loop index, increasing from 1 to N in steps of
*         1 or 2, depending on the size of the diagonal blocks.
*
          K = 1
          KC = 1
   40     CONTINUE
*
*         If K > N, exit from loop.
*
          IF( K.GT.N ) GO TO 50
          IF( IA( K ).GT.0 ) THEN
*
*             1 x 1 diagonal block
*
*             Multiply by inv(U**T(K)), where U(K) is the transformation
*             stored in column K of A.
*
C              IF (K.GT.1) X(K) = X(K) - UXVDOT(K-1,X,A(KC))
              IF (K.GT.1) X(K) = X(K)-DOT_PRODUCT(X(1:K-1),A(KC:KC+K-2))
*
*             Interchange elements K <-> IA(K).
*
              KP = IA( K )
              IF( KP.NE.K ) CALL UNSWAP(X(K),X(KP))
              KC = KC + K
              K = K + 1
          ELSE
*
*             2 x 2 diagonal block
*
*             Multiply by inv(U**T(K+1)), where U(K+1) is 
*             the transformation stored in columns K and K+1 of A.
*
              IF (K.GT.1) THEN
C                  X(K) = X(K) - UXVDOT(K-1,X,A(KC))
C                  X(K+1) = X(K+1) - UXVDOT(K-1,X,A(KC+K))
                  X(K) = X(K) - DOT_PRODUCT(X(1:K-1),A(KC:KC+K-2))
                  X(K+1) = X(K+1)-DOT_PRODUCT(X(1:K-1),A(KC+K:KC+K+K-2))
              END IF
*
*             Interchange elements K <-> -IA(K) AND K+1 <-> -IA(K+1).
*
              KP = -IA( K )
              IF( KP.NE.K ) CALL UNSWAP(X(K),X(KP))
              KP = -IA( K+1 )
              IF( KP.NE.K+1 ) CALL UNSWAP(X(K+1),X(KP))
              KC = KC + 2*K + 1
              K = K + 2
          END IF
          GO TO 40
      END IF
   50 CONTINUE
      RETURN
      END