* SUBROUTINE UXDLBF                ALL SYSTEMS                91/12/01
C PORTABILITY : ALL SYSTEMS
C 91/12/01 MA : ORIGINAL VERSION
*
* PURPOSE :
* FACTORIZATION A=P*U*D*TRANS(U)*TRANS(P) OF A DENSE SYMMETRIC MATRIX
* A WHERE P IS A PERMUTATION MATRIX, D IS A BLOCK DIAGONAL MATRIX (WITH
* 1*1 OR 2*2 BLOCKS) AND U IS AN UPPER TRIANGULAR MATRIX.
*
* PARAMETERS :
*  II  N ORDER OF THE MATRIX A.
*  RU  A(N*(N+1)/2)  ON INPUT A GIVEN DENSE SYMMETRIC MATRIX A STORED
*         IN PACKED FORM. ON OUTPUT THE COMPUTED FACTORIZATION
*         A=P*U*D*TRANS(U)*TRANS(P).
*  IO  IA(N)  A PERMUTATION FIELD WHICH CONTAINS AN INFORMATION ABOUT
*         PERMUTATION MATRIX P.
*  IO  INF  AN INFORMATION OBTAINED IN THE FACTORIZATION PROCESS. IF
*         INF=0 THEN A IS POSITIVE SEMIDEFINITE. IF INF>0 THEN A IS
*         INDEFINITE.
*  RU  ALF  ON INPUT DESIRED TOLERANCE FOR NONSINGULARITY. ON OUTPUT
*         A NEGATIVE NUMBER IF A IS SINGULAR OR NEARLY SINGULAR.
*  II  BUNCH  IF BUNCH=1 THEN BUNCH-PARLETT FACTORIZATION,
*             IF BUNCH=2 THEN BUNCH-KAUFMAN FACTORIZATION.
*
      SUBROUTINE UXDLBF(N,A,IA,INF,ALF,BUNCH)
      $FLOAT ZERO, DEL, EPS, ONE, EIGHT, SEVTEN
      PARAMETER ( ZERO = 0.0$P+0, DEL=1.0$P-30, EPS = 1.0$P-6 )
      PARAMETER ( ONE = 1.0$P+0, EIGHT = 8.0$P+0, SEVTEN = 1.7$P+1 )
      INTEGER N,IA(N),INF,BUNCH
      $FLOAT A(N*(N+1)/2),ALF
      INTEGER I,IMAX,J,JMAX,K,KC,KK,KNC,KP,KPC,KSTEP,KX,UXVIMX
      INTEGER KD,KP1,KPC1,KMAX(1),DMAX
      $FLOAT ABSAKK,ALPHA,COLMAX,D11,D12,D22,R1,ROWMAX,T,WK,WKM1
      $FLOAT DIAGMAX,NONDMAX
      $FLOAT TALF,TOL
      LOGICAL TWOPERM
*
      ALF = MAX(ABS(ALF),DEL)
      TALF = ALF
      INF = 0
      ALPHA = ( ONE+SQRT( SEVTEN ) ) / EIGHT
      GOTO (1,2) BUNCH
*
*     ===========================
*     BUNCH-PARLETT FACTORIZATION
*     ===========================
*
    1 CONTINUE
      K = N
      KC = ( K-1 )*K / 2
      KD = KC + K
      KC = KC + 1
   10 CONTINUE
      KNC = KC
      TWOPERM = .FALSE.
      IF( K.LT.1 ) RETURN
*
*     Determine rows and columns to be interchanged and whether
*     a 1-by-1 or 2-by-2 pivot block will be used:
*     1-by-1: interchange rows and columns KK = K and KP,
*             KC corresponds to values KNC and KPC.
*     2-by-2: potential second permutation:
*             interchange rows and columns KK = K-1 and KP1,
*             KC corresponds similarly.
*
      KMAX = MAXLOC(ABS(A(1:KD)))
      IF (K.EQ.N) TOL=TALF*MAX(TALF,ABS(A(KMAX(1))))
      IF (ABS(A(KMAX(1))).LE.TOL) THEN
*
*         Zero submatrix, no interchange, use 1-by-1 pivot block
*
          ALF = -DBLE(K)
          A(KD) = TOL
          KSTEP = 1
          IA(K) = K
          GOTO 14
      ENDIF
      JMAX = INT( ( 1 + SQRT( DBLE( 8*KMAX(1)-7 ) ) ) / 2 + EPS )
   11 IMAX = KMAX(1) - ((JMAX-1)*JMAX)/2
      IF (IMAX.GT.0.AND.IMAX.LT.JMAX) THEN
*
*         It holds DIAGMAX <= NONDMAX, find max diagonal element
*
          NONDMAX = ABS(A(KMAX(1)))
          GOTO 12
      ELSEIF (IMAX.EQ.JMAX.AND.JMAX.LT.K) THEN
*
*         It holds DIAGMAX >= NONDMAX > ALPHA*NONDMAX
*         interchange rows and columns K and DMAX,
*         use 1-by-1 pivot block
*
          KSTEP = 1
          KP = JMAX
          KPC = KMAX(1)-JMAX+1
          IA(K) = KP
          GOTO 13
      ELSEIF (IMAX.EQ.JMAX.AND.JMAX.EQ.K) THEN
*
*         It holds DIAGMAX >= NONDMAX > ALPHA*NONDMAX
*         no interchange, use 1-by-1 pivot block
*
          KSTEP = 1
          IA(K) = K
          GOTO 14
      ELSEIF (IMAX.LE.0) THEN
          JMAX = JMAX-1
          GOTO 11
      ELSE
          JMAX = JMAX+1
          GOTO 11
      ENDIF
   12 CONTINUE
      DIAGMAX=ZERO
      DO J=K,1,-1
          I=J*(J+1)/2
          IF (ABS(A(I)).GT.DIAGMAX) THEN
              DIAGMAX = ABS(A(I))
              DMAX = J
          ENDIF
      ENDDO
      IF (DIAGMAX.GE.ALPHA*NONDMAX.AND.DMAX.LT.K) THEN
*
*         interchange rows and columns K and DMAX,
*         use 1-by-1 pivot block
*
          KSTEP = 1
          KP = DMAX
          KPC = ( DMAX-1 )*DMAX / 2 + 1
          IA(K) = KP
      ELSEIF (DIAGMAX.GE.ALPHA*NONDMAX.AND.DMAX.EQ.K) THEN
*
*         no interchange, use 1-by-1 pivot block
*
          KSTEP = 1
          IA(K) = K
          GOTO 14
      ELSEIF (JMAX.LT.K) THEN
*
*         interchange rows and columns K and JMAX and K-1 and IMAX,
*         use 2-by-2 pivot block
*
          TWOPERM = .TRUE.
          KSTEP = 1   ! in 2nd interchange K-1 and IMAX set KSTEP=2
          KP = JMAX
          KPC = ( JMAX-1 )*JMAX / 2 + 1
          KP1 = IMAX
          KPC1 = ( IMAX-1 )*IMAX / 2 + 1
          IA(K) = -KP
          IA(K-1) = -KP1
      ELSEIF (IMAX.LT.K-1) THEN
*
*         interchange rows and columns K-1 and IMAX,
*         use 2-by-2 pivot block
*
          KSTEP = 2
          KP = IMAX
          KPC = ( IMAX-1 )*IMAX / 2 + 1
          IA(K) = -K
          IA(K-1) = -KP
      ELSE
*
*         no interchange, use 2-by-2 pivot block
*
          KSTEP = 2
          KNC = KNC - K + 1
          IA(K) = -K
          IA(K-1) = -(K-1)
          GOTO 14
      ENDIF
   13 CONTINUE
*
*     Interchange rows and columns KK and KP in the leading
*     submatrix A(1:k,1:k)
*
      KK = K - KSTEP + 1
      IF( KSTEP.EQ.2 ) KNC = KNC - K + 1
      CALL UXVSWP(KP-1,A(KNC),A(KPC),1)
      KX = KPC + KP - 1
      DO J = KP + 1, KK - 1
          KX = KX + J - 1
          CALL UNSWAP( A( KNC+J-1 ), A(KX) )
      END DO
      CALL UNSWAP( A( KNC+KK-1 ), A( KPC+KP-1 ) )
      IF( KSTEP.EQ.2 ) CALL UNSWAP( A( KC+K-2 ), A( KC+KP-1 ) )
      IF (TWOPERM) THEN
          KSTEP = 2
          KP = KP1
          KPC = KPC1
          TWOPERM = .FALSE.
          GOTO 13
      ENDIF
   14 CONTINUE
*
*     Update the leading submatrix
*
      IF( KSTEP.EQ.1 ) THEN
*
*         1-by-1 pivot block D(k): column k now holds
*             W(k) = U(k)*D(k)
*         where U(k) is the k-th column of U
*
*         Perform a rank-1 update of A(1:k-1,1:k-1) as
*             A := A - U(k)*D(k)*U(k)**T = A - W(k)*1/D(k)*W(k)**T
*
          IF( A( KC+K-1 ).LT.ZERO ) INF = 1
          R1 = ONE / A( KC+K-1 )
          CALL UXDSMU(K-1,A,-R1,A(KC))
*
*         Store U(k) in column k
*
          CALL UXVSCL(K-1,R1,A(KC),A(KC))
      ELSE
*
*         2-by-2 pivot block D(k): columns k and k-1 now hold
*             ( W(k-1) W(k) ) = ( U(k-1) U(k) )*D(k)
*         where U(k) and U(k-1) are the k-th and (k-1)-th columns of U
*
*         Perform a rank-2 update of A(1:k-2,1:k-2) as
*             A := A - ( U(k-1) U(k) )*D(k)*( U(k-1) U(k) )**T
*                = A - ( W(k-1) W(k) )*inv(D(k))*( W(k-1) W(k) )**T
*
          IF( K.GT.2 ) THEN
              D11 = A( K+( K-1 )*K / 2 )
              D12 = A( K-1+( K-1 )*K / 2 )
              D22 = A( K-1+( K-2 )*( K-1 ) / 2 )
              IF( D11+D22 .LT. ZERO ) INF = 1
              IF( D11*D22-D12*D12 .LT. ZERO ) INF = 1
              D22 = D22 / D12
              D11 = D11 / D12
              T = ONE / ( D11*D22-ONE )
              D12 = T / D12
              DO J = K - 2, 1, -1
                  WKM1 = D12*( D11*A( J+( K-2 )*( K-1 ) / 2 )-
     &                   A( J+( K-1 )*K / 2 ) )
                  WK = D12*( D22*A( J+( K-1 )*K / 2 )-
     &                 A( J+( K-2 )*( K-1 ) / 2 ) )
                  DO I = J, 1, -1
                      A( I+( J-1 )*J / 2 ) = A( I+( J-1 )*J / 2 ) -
     &                    A( I+( K-1 )*K / 2 )*WK -
     &                    A( I+( K-2 )*( K-1 ) / 2 )*WKM1
                  END DO
                  A( J+( K-1 )*K / 2 ) = WK
                  A( J+( K-2 )*( K-1 ) / 2 ) = WKM1
              END DO
          END IF
      END IF
*
*     Decrease K and return to the start of the main loop
*
      K = K - KSTEP
      KC = KNC - K
      KD = KNC - 1
      GO TO 10
*
*     ===========================
*     BUNCH-KAUFMAN FACTORIZATION
*     ===========================
*
    2 CONTINUE
      K = N
      KC = ( N-1 )*N / 2 + 1
   20 CONTINUE
      KNC = KC
      IF( K.LT.1 ) RETURN
      KSTEP = 1
*
*     Determine rows and columns to be interchanged and whether
*     a 1-by-1 or 2-by-2 pivot block will be used
*
      ABSAKK = ABS( A( KC+K-1 ) )
*
*     IMAX is the row-index of the largest off-diagonal element in
*     column K, and COLMAX is its absolute value
*
      IF( K.GT.1 ) THEN
          IMAX = UXVIMX(K-1,A(KC),COLMAX)
      ELSE
          COLMAX = ZERO
      END IF
      TOL=TALF*MAX(TALF,COLMAX,ABSAKK)
*
      IF( MAX( ABSAKK, COLMAX ).LE.TOL ) THEN
*
*         Column K is zero
*
          ALF = -DBLE(K)
          ABSAKK = TOL
          COLMAX = TOL
          KP = K
          A( KC+K-1 ) = TOL
      ELSE
          IF( ABSAKK.GE.ALPHA*COLMAX ) THEN
*
*             no interchange, use 1-by-1 pivot block
*
              KP = K
          ELSE
*
              ROWMAX = ZERO
              JMAX = IMAX
              KX = IMAX*( IMAX+1 ) / 2 + IMAX
              DO J = IMAX + 1, K
                  IF( ABS( A( KX ) ).GT.ROWMAX ) THEN
                      ROWMAX = ABS( A( KX ) )
                      JMAX = J
                  END IF
                  KX = KX + J
              END DO
              KPC = ( IMAX-1 )*IMAX / 2 + 1
              IF( IMAX.GT.1 ) THEN
                  JMAX = UXVIMX(IMAX-1,A(KPC),T)
                  ROWMAX = MAX(ROWMAX,T)
              ENDIF
              IF( ABSAKK.GE.ALPHA*COLMAX*( COLMAX / ROWMAX ) ) THEN
*
*                 no interchange, use 1-by-1 pivot block
*
                  KP = K
              ELSE IF( ABS( A( KPC+IMAX-1 ) ).GE.ALPHA*ROWMAX ) THEN
*
*                 interchange rows and columns K and IMAX,
*                 use 1-by-1 pivot block
*
                  KP = IMAX
              ELSE
*
*                 interchange rows and columns K-1 and IMAX,
*                 use 2-by-2 pivot block
*
                  KP = IMAX
                  KSTEP = 2
              END IF
          END IF
*
          KK = K - KSTEP + 1
          IF( KSTEP.EQ.2 ) KNC = KNC - K + 1
          IF( KP.NE.KK ) THEN
*
*             Interchange rows and columns KK and KP in the leading
*             submatrix A(1:k,1:k)
*
              CALL UXVSWP(KP-1,A(KNC),A(KPC),1)
              KX = KPC + KP - 1
              DO J = KP + 1, KK - 1
                  KX = KX + J - 1
                  CALL UNSWAP( A( KNC+J-1 ), A(KX) )
              END DO
              CALL UNSWAP( A( KNC+KK-1 ), A( KPC+KP-1 ) )
              IF( KSTEP.EQ.2 ) CALL UNSWAP( A( KC+K-2 ), A( KC+KP-1 ) )
          END IF
*
*         Update the leading submatrix
*
          IF( KSTEP.EQ.1 ) THEN
*
*             1-by-1 pivot block D(k): column k now holds
*                 W(k) = U(k)*D(k)
*             where U(k) is the k-th column of U
*
*             Perform a rank-1 update of A(1:k-1,1:k-1) as
*                 A := A - U(k)*D(k)*U(k)**T = A - W(k)*1/D(k)*W(k)**T
*
              IF( A( KC+K-1 ).LT.ZERO ) INF = 1
              R1 = ONE / A( KC+K-1 )
              CALL UXDSMU(K-1,A,-R1,A(KC))
*
*             Store U(k) in column k
*
              CALL UXVSCL(K-1,R1,A(KC),A(KC))
          ELSE
*
*             2-by-2 pivot block D(k): columns k and k-1 now hold
*                 ( W(k-1) W(k) ) = ( U(k-1) U(k) )*D(k)
*             where U(k) and U(k-1) are the k-th and (k-1)-th columns of U
*
*             Perform a rank-2 update of A(1:k-2,1:k-2) as
*                 A := A - ( U(k-1) U(k) )*D(k)*( U(k-1) U(k) )**T
*                    = A - ( W(k-1) W(k) )*inv(D(k))*( W(k-1) W(k) )**T
*
              IF( K.GT.2 ) THEN
                  D11 = A( K+( K-1 )*K / 2 )
                  D12 = A( K-1+( K-1 )*K / 2 )
                  D22 = A( K-1+( K-2 )*( K-1 ) / 2 )
                  IF( D11+D22 .LT. ZERO ) INF = 1
                  IF( D11*D22-D12*D12 .LT. ZERO ) INF = 1
                  D22 = D22 / D12
                  D11 = D11 / D12
                  T = ONE / ( D11*D22-ONE )
                  D12 = T / D12
                  DO J = K - 2, 1, -1
                      WKM1 = D12*( D11*A( J+( K-2 )*( K-1 ) / 2 )-
     &                       A( J+( K-1 )*K / 2 ) )
                      WK = D12*( D22*A( J+( K-1 )*K / 2 )-
     &                     A( J+( K-2 )*( K-1 ) / 2 ) )
                      DO I = J, 1, -1
                          A( I+( J-1 )*J / 2 ) = A( I+( J-1 )*J / 2 ) -
     &                        A( I+( K-1 )*K / 2 )*WK -
     &                        A( I+( K-2 )*( K-1 ) / 2 )*WKM1
                      END DO
                      A( J+( K-1 )*K / 2 ) = WK
                      A( J+( K-2 )*( K-1 ) / 2 ) = WKM1
                  END DO
              END IF
          END IF
      END IF
*
*     Store details of the interchanges in IA
*
      IF( KSTEP.EQ.1 ) THEN
          IA( K ) = KP
      ELSE
          IA( K ) = -K
          IA( K-1 ) = -KP
      END IF
*
*     Decrease K and return to the start of the main loop
*
      K = K - KSTEP
      KC = KNC - K
      GO TO 20
      END