* SUBROUTINE UXDLBM                ALL SYSTEMS                91/12/01
C PORTABILITY : ALL SYSTEMS
C 91/12/01 MA : ORIGINAL VERSION
*
* PURPOSE :
* MULTIPLICATION OF A GIVEN VECTOR X BY A DENSE SYMMETRIC MATRIX
* A USING THE FACTORIZATION A=P*U*D*TRANS(U)*TRANS(P) OBTAINED BY THE
* SUBROUTINE UXDLBF.
*
* PARAMETERS :
*  II  N ORDER OF THE MATRIX A.
*  RI  A(N*(N+1)/2)  FACTORIZATION A=P*U*D*TRANS(U)*TRANS(P) OBTAINED
*         BY THE SUBROUTINE UXDLBF.
*  II  IA(N)  PERMUTATION FIELD OBTAINED BY THE SUBROUTINE UXDLBF.
*  RU  X(N)  ON INPUT THE GIVEN VECTOR. ON OUTPUT THE RESULT OF
*         MULTIPLICATION.
*  II  JOB  OPTION. IF JOB=0 THEN X:=A*X. IF JOB>0 THEN
*         X:=TRANS(U)*TRANS(P)*X. IF JOB<0 THEN X:=P*U*X.
*
      SUBROUTINE UXDLBM(N,A,IA,X,JOB)
      INTEGER N,IA(N),JOB
      $FLOAT A(N*(N+1)/2),X(N)
      INTEGER K,KC,KP
      $FLOAT TEMP
*
*     Compute A*X, where A = U*D*U**T.
*
      IF (JOB .GE. 0) THEN
*
*         First compute D*U**T*X.
*
*         K is the main loop index, decreasing from N to 1 in steps of
*         1 or 2, depending on the size of the diagonal blocks.
*
          K = N
          KC = N*( N+1 ) / 2 + 1
   10     CONTINUE
*
*         If K < 1, exit from loop.
*
          IF( K.LT.1 ) GO TO 30
          KC = KC - K
          IF( IA( K ).GT.0 ) THEN
*
*             1 x 1 diagonal block, interchange elements K <-> IA(K).
*
              KP = IA( K )
              IF( KP.NE.K ) CALL UNSWAP(X(K),X(KP))
*
*             Multiply by U**T(K), where U(K) is the transformation
*             stored in column K of A.
*
C              IF (K.GT.1) X(K) = X(K) + UXVDOT(K-1,X,A(KC))
              IF (K.GT.1) X(K) = X(K)+DOT_PRODUCT(X(1:K-1),A(KC:KC+K-2))
              IF (JOB .EQ. 0) THEN
*
*                 Multiply by the diagonal block.
*
                  X(K) = X(K) * A( KC+K-1 )
              END IF
              K = K - 1
          ELSE
*
*             2 x 2 diagonal block, interchange elements
*             K <-> -IA(K) and K-1 <-> -IA(K-1).
*
              KP = -IA( K )
              IF( KP.NE.K ) CALL UNSWAP(X(K),X(KP))
              KP = -IA( K-1 )
              IF( KP.NE.K-1 ) CALL UNSWAP(X(K-1),X(KP))
*
*             Multiply by U**T(K), where U(K) is the transformation
*             stored in columns K-1 and K of A.
*
              IF (K.GT.2) THEN
C                  X(K) = X(K) + X(K-1) + UXVDOT(K-2,X,A(KC))
C                  X(K-1) = X(K-1) + UXVDOT(K-2,X,A(KC-(K-1)))
                  X(K) = X(K)+DOT_PRODUCT(X(1:K-2),A(KC:KC+K-3))
                  X(K-1) = X(K-1) +
     &                    DOT_PRODUCT(X(1:K-2),A(KC-(K-1):KC-(K-1)+K-3))
              END IF
              IF (JOB .EQ. 0) THEN
*
*                 Multiply by the diagonal block.
*
                  TEMP = A(KC+K-2)*X(K)
                  X(K) = A(KC+K-2)*X(K-1) + A(KC+K-1)*X(K)
                  X(K-1) = A(KC-1)*X(K-1) + TEMP
              END IF
              KC = KC - K + 1
              K = K - 2
          END IF
          GO TO 10
      END IF
   30 CONTINUE
      IF (JOB .LE. 0) THEN
*
*         Next compute U*X.
*
*         K is the main loop index, increasing from 1 to N in steps of
*         1 or 2, depending on the size of the diagonal blocks.
*
          K = 1
          KC = 1
   40     CONTINUE
*
*         If K > N, exit from loop.
*
          IF( K.GT.N ) GO TO 50
          IF( IA( K ).GT.0 ) THEN
*
*             1 x 1 diagonal block
*
*             Multiply by U(K), where U(K) is the transformation
*             stored in column K of A.
*
              IF (K.GT.1) CALL UXVDIR(K-1,X(K),A(KC),X,X)
*
*             Interchange elements K <-> IA(K).
*
              KP = IA( K )
              IF( KP.NE.K ) CALL UNSWAP(X(K),X(KP))
              KC = KC + K
              K = K + 1
          ELSE
*
*             2 x 2 diagonal block
*
*             Multiply by U(K+1), where U(K+1) is
*             the transformation stored in columns K and K+1 of A.
*
              IF (K.GT.1) THEN
                  CALL UXVDIR(K-1,X(K),A(KC),X,X)
                  CALL UXVDIR(K-1,X(K+1),A(KC+K),X,X)
              END IF
*
*             Interchange elements K <-> -IA(K) and K+1 <-> -IA(K+1).
*
              KP = -IA( K )
              IF( KP.NE.K ) CALL UNSWAP(X(K),X(KP))
              KP = -IA( K+1 )
              IF( KP.NE.K+1 ) CALL UNSWAP(X(K+1),X(KP))
              KC = KC + 2*K + 1
              K = K + 2
          END IF
          GO TO 40
      END IF
   50 CONTINUE
      RETURN
      END