* SUBROUTINE UXDLGD                ALL SYSTEMS                92/12/01
C PORTABILITY : ALL SYSTEMS
C 92/12/01 MA : ORIGINAL VERSION
*
* PURPOSE :
* COMPUTATION OF A DIRECTION OF NEGATIVE CURVATURE WITH RESPECT
* TO A DENSE SYMMETRIC MATRIX A USING THE FACTORIZATION
* A+E=P*U*D*TRANS(U)*TRANS(P) OBTAINED BY THE SUBROUTINE UXDLGF.
*
* PARAMETERS :
*  II  N ORDER OF THE MATRIX A.
*  RI  A(N*(N+1)/2) FACTORIZATION A+E=P*U*D*TRANS(U)*TRANS(P) OBTAINED
*         BY THE SUBROUTINE UXDLGF.
*  II  IA(N)  A PERMUTATION FIELD WHICH CONTAINS AN INFORMATION ABOUT
*         PERMUTATION MATRIX P.
*  RO  X(N)  COMPUTED DIRECTION OF NEGATIVE CURVATURE (I.E.
*         TRANS(X)*A*X<0) IF IT EXISTS.
*  II  INF  INFORMATION OBTAINED IN THE FACTORIZATION PROCESS. THE
*         DIRECTION OF NEGATIVE CURVATURE EXISTS ONLY IF INF>0.
*
* METHOD :
* P.E.GILL, W.MURRAY : NEWTON TYPE METHODS FOR UNCONSTRAINED AND
* LINEARLY CONSTRAINED OPTIMIZATION, MATH. PROGRAMMING 28 (1974)
* PP. 311-350.
*
      SUBROUTINE UXDLGD(N,A,IA,X,INF)
      INTEGER N, IA(N), INF
      $FLOAT  A(N*(N+1)/2), X(N)
      INTEGER K, KC, KD, KP, J
      $FLOAT UXVDOT, ZERO, ONE
      PARAMETER (ZERO = 0.0$P 0, ONE = 1.0$P 0)
C
C     RIGHT HAND SIDE FORMATION
C
      X = ZERO
      IF (INF .LE. 0) RETURN
      INF = N - INF + 1
      X(INF) = ONE
C
      IF (INF.LT.N) THEN
C
C       PERMUTATION OF ROWS IN MATRIX A
C
          KD = N*(N+1)/2
          DO K = N, 3, -1
              KD = KD - K
              DO J = K-1, 2, -1
                  KP = IA(J)
                  IF (KP.NE.J) CALL UNSWAP(A(KD+KP),A(KD+J))
              ENDDO
          ENDDO
C
C         BACK SUBSTITUTION
C
          KC = INF*(INF+1)/2+1
          DO K = INF + 1, N
              X(K) = X(K) - UXVDOT(K-INF,X(INF),A(KC+INF-1))
              KC = KC + K
          ENDDO
C
C         PERMUTATION OF ROWS IN MATRIX A
C
          KD = 0
          DO K = 1, N
              DO J = 2, K-1
                  KP = IA(J)
                  IF (KP.NE.J) CALL UNSWAP(A(KD+KP),A(KD+J))
              ENDDO
              KD = KD+K
          ENDDO
C
C         PERMUTATION OF VECTOR X
C
          DO K = 2, N
              KP = IA(K)
              IF (KP.NE.K) CALL UNSWAP(X(K),X(KP))
          ENDDO
      ENDIF
C
      RETURN
      END
