* SUBROUTINE UXDLGI                ALL SYSTEMS                91/12/01
C PORTABILITY : ALL SYSTEMS
C 91/12/01 MA : ORIGINAL VERSION
*
* PURPOSE :
*  INVERSION OF A DENSE SYMMETRIC MATRIX A+E USING THE DECOMPOSITION
*  A+E=P*U*D*TRANS(U)*TRANS(P) OBTAINED BY THE SUBROUTINE UXDLGF.
*
* PARAMETERS :
*  II  N  ORDER OF THE MATRIX A
*  RU  A(N*(N+1)/2)  ON INPUT THE DECOMPOSITION
*         A+E=P*U*D*TRANS(U)*TRANS(P) OBTAINED BY THE SUBROUTINE
*         UXDLGF. ON OUTPUT THE INVERSION (A+E)**(-1).
*  II  IA(N)  A PERMUTATION FIELD WHICH CONTAINS AN INFORMATION ABOUT
*         PERMUTATION MATRIX P.
*  RA  B(N)  AUXILIARY VECTOR.
*
* METHOD :
*  INVERSION OF THE UPPER TRIANGULAR MATRIX U AND BACK MULTIPLICATION
*  (A+E)**(-1) = (TRANS(U)*TRANS(P))**(-1)*D**(-1)*(P*U)**(-1).
*
      SUBROUTINE UXDLGI(N,A,IA,B)
      INTEGER  N, IA(N)
      $FLOAT  A(N*(N+1)/2), B(N)
      INTEGER K, KC, KD, KP, KPC, KPD, KX, J
      $FLOAT UXVDOT
      $FLOAT ONE
      PARAMETER (ONE = 1.0$P 0)
*
*     K is the main loop index, increasing from 1 to N.
*
      KC = 1
   30 CONTINUE
      DO K = 1, N
          KD = KC + K - 1
*
*         Invert the diagonal block.
*
          A(KD) = ONE / A(KD)
*
*         Compute column K of the inverse.
*
          IF( K.GT.1 ) THEN
              CALL UXVCOP(K-1,A(KC),B)
              CALL UXDSMM(K-1,A,-B,A(KC))
              A(KD) = A(KD) - UXVDOT(K-1,B,A(KC))
          END IF
*
          KP = IA(K)
          IF( KP.NE.K ) THEN
*
*             Interchange rows and columns K and KP in the leading
*             submatrix A(1:k+1,1:k+1)
*
              KPC = ( KP-1 )*KP / 2 + 1
              CALL UXVSWP(KP-1,A(KC),A(KPC),1)
              KPD = KPC + KP - 1
              KX = KPD
              DO J = KP + 1, K - 1
                  KX = KX + J - 1
                  CALL UNSWAP(A(KC+J-1),A(KX))
              ENDDO
              CALL UNSWAP(A(KD),A(KPD))
          END IF
          KC = KC + K
      ENDDO
      RETURN
      END
