* SUBROUTINE UXDPRN                ALL SYSTEMS               91/12/01
C PORTABILITY : ALL SYSTEMS
C 91/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* ESTIMATION OF THE MINIMUM EIGENVALUE AND THE CORRESPONDING EIGENVECTOR
* OF A DENSE SYMMETRIC POSITIVE DEFINITE MATRIX A USING THE
* FACTORIZATION A=TRANS(R)*R.
*
* PARAMETERS :
*  II  N ORDER OF THE MATRIX A.
*  RI  A(N*(N+1)/2) FACTORIZATION A=TRANS(R)*R.
*  RO  X(N)  ESTIMATED EIGENVECTOR.
*  RO  ALF  ESTIMATED EIGENVALUE.
*  II  JOB  OPTION. IF JOB=0 THEN ONLY ESTIMATED EIGENVALUE IS
*         COMPUTED. IS JOB>0 THEN BOTH ESTIMATED EIGENVALUE AND
*         ESTIMATED EIGENVECTOR ARE COMPUTED BY JOB ITERATIONS.
*
* SUBPROGRAMS USED :
*  S   UXDPRB  BACK SUBSTITUTION.
*  RF  UXVDOT  DOT PRODUCT OF TWO VECTORS.
*
* METHOD :
* A.K.CLINE, C.B.MOLER, G.W.STEWART, J.H.WILKINSON : AN ESTIMATE
* FOR THE CONDITION NUMBER OF A MATRIX. SIAM J. NUMER. ANAL. 16
* (1979) 368-373.
*
      SUBROUTINE UXDPRN(N,A,X,ALF,JOB)
      INTEGER N,JOB
      $FLOAT A(N*(N+1)/2),X(N),ALF
      $FLOAT XP,XM,FP,FM,UXVDOT
      INTEGER I,K,IK,KK
      $FLOAT ZERO,ONE
      PARAMETER (ZERO=0.0$P 0,ONE=1.0$P 0)
C
C     COMPUTATION OF THE VECTOR V WITH POSSIBLE MAXIMUM NORM SUCH
C     THAT  TRANS(R)*V=U  WHERE U HAS ELEMENTS +1 OR -1
C
      DO 2 I=1,N
      X(I)=ZERO
    2 CONTINUE
      KK=0
      DO 6 K=1,N
      KK=KK+K
      XP=-X(K)+ONE
      XM=-X(K)-ONE
      FP=ABS(XP)
      FM=ABS(XM)
      XP=XP/A(KK)
      XM=XM/A(KK)
      IK=KK
      DO 3 I=K+1,N
      IK=IK+I-1
      FP=FP+ABS(X(I)+A(IK)*XP)
      FM=FM+ABS(X(I)+A(IK)*XM)
    3 CONTINUE
      IF(FP.GE.FM) THEN
      X(K)=XP
      IK=KK
      DO 4 I=K+1,N
      IK=IK+I-1
      X(I)=X(I)+A(IK)*XP
    4 CONTINUE
      ELSE
      X(K)=XM
      IK=KK
      DO 5 I=K+1,N
      IK=IK+I-1
      X(I)=X(I)+A(IK)*XM
    5 CONTINUE
      ENDIF
    6 CONTINUE
      FP=$DBLE(N)
      IF (JOB.LE.0) THEN
C
C     FIRST ESTIMATION OF THE MINIMUM EIGENVALUE BY THE FORMULA
C     ALF=(TRANS(U)*U)/(TRANS(V)*V)
C
      FM=UXVDOT(N,X,X)
      ALF=FP/FM
      RETURN
      ENDIF
C
C     COMPUTATION OF THE VECTOR X SUCH THAT R*X=V
C
      FM=ZERO
      KK=N*(N+1)/2
      DO 9 K=N,1,-1
      IK=KK
      DO 8 I=K+1,N
      IK=IK+I-1
      X(K)=X(K)-A(IK)*X(I)
    8 CONTINUE
      X(K)=X(K)/A(KK)
      FM=FM+X(K)*X(K)
      KK=KK-K
    9 CONTINUE
      FM=SQRT(FM)
      IF (JOB.LE.1) THEN
C
C     SECOND ESTIMATION OF THE MINIMUM EIGENVALUE BY THE FORMULA
C     ALF=SQRT(TRANS(U)*U)/SQRT(TRANS(X)*X)
C
      ALF=SQRT(FP)/FM
      ELSE
C
C     INVERSE ITERATIONS
C
      DO 11 K=2,JOB
C
C     SCALING THE VECTOR X BY ITS NORM
C
      DO 10 I=1,N
      X(I)=X(I)/FM
   10 CONTINUE
      CALL UXDPRB(N,A,X,0)
      FM=SQRT(UXVDOT(N,X,X))
   11 CONTINUE
      ALF=ONE/FM
      ENDIF
C
C     SCALING THE VECTOR X BY ITS NORM
C
      DO 12 I=1,N
      X(I)=X(I)/FM
   12 CONTINUE
      RETURN
      END
