* SUBROUTINE UXDPRR               ALL SYSTEMS                02/12/01
C PORTABILITY : ALL SYSTEMS
C 02/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* PLANE ROTATION IS APPLIED TO A DENSE POSITIVE DEFINITE MATRIX A
* FACTORED IN THE FORM A=TRANS(R)*R. EITHER L=K+1 OR K=L+1 IS REQUIRED.
*
* PARAMETERS :
*  II  N  ORDER OF THE MATRIX A.
*  RU  A(N*(N+1)/2) FACTORIZATION A=TRANS(R)*R STORED IN THE PACKED
*         FORM.
*  II  K  FIRST INDEX OF PLANE ROTATION.
*  II  L  SECOND INDEX OF PLANE ROTATION.
*  RI  CK  DIAGONAL ELEMENT OF THE ELEMENTARY ORTHOGONAL MATRIX.
*  RI  CL  OFF-DIAGONAL ELEMENT OF THE ELEMENTARY ORTHOGONAL MATRIX.
*  IU  IER  INFORMATION ON THE TRANSFORMATION. IER<0-K OR L OUT OF
*         RANGE. IER=0-GENERAL PLANE ROTATION. IER=1-PERMUTATION.
*         IER=2-TRANSFORMATION SUPPRESSED.
*
* SUBPROGRAMS USED :
*  S   UXVORT  COMPUTATION OF THE PLANE ROTATION MATRIX.
*  S   UXVROT  PLANE ROTATION IS APPLIED TO TWO NUMBERS.
*
      SUBROUTINE UXDPRR(N,A,K,L,CK,CL,IER)
      INTEGER N,K,L,IER
      $FLOAT A(N*(N+1)/2),CK,CL
      $FLOAT AKJ,ALJ,CKK,CLL
      INTEGER J,KJ,LJ,IERR
      $FLOAT ZERO
      PARAMETER (ZERO=0.0$P 0)
      IF (IER.NE.0.AND.IER.NE.1) RETURN
      IF (L.EQ.K+1) THEN
      KJ=K*(K-1)/2
      DO 1 J=1,K
      KJ=KJ+1
      LJ=KJ+K
      CALL UXVROT(A(KJ),A(LJ),CK,CL,IER)
    1 CONTINUE
      LJ=LJ+1
      AKJ=ZERO
      CALL UXVROT(AKJ,A(LJ),CK,CL,IER)
      IF (AKJ.EQ.ZERO) RETURN
      CALL UXVORT(A(KJ),AKJ,CKK,CLL,IERR)
      DO 2 J=L,N
      KJ=KJ+J-1
      CALL UXVROT(A(KJ),A(KJ+1),CKK,CLL,IERR)
    2 CONTINUE
      ELSE IF (K.EQ.L+1) THEN
      LJ=L*(L-1)/2
      DO 3 J=1,L
      LJ=LJ+1
      KJ=LJ+L
      CALL UXVROT(A(KJ),A(LJ),CK,CL,IER)
    3 CONTINUE
      KJ=KJ+1
      ALJ=ZERO
      CALL UXVROT(A(KJ),ALJ,CK,CL,IER)
      IF (ALJ.EQ.ZERO) RETURN
      CALL UXVORT(A(LJ),ALJ,CKK,CLL,IERR)
      DO 4 J=K,N
      LJ=LJ+J-1
      CALL UXVROT(A(LJ),A(LJ+1),CKK,CLL,IERR)
    4 CONTINUE
      ELSE
      IER=-1
      ENDIF
      RETURN
      END

