* SUBROUTINE UXDRSV                ALL SYSTEMS                06/11/08
C PORTABILITY : ALL SYSTEMS
C 06/11/08 VL : ORIGINAL VERSION
*
* PURPOSE :
* IMPLICIT VM MATRIX BY VECTOR MULTIPLICATION BY ADAPTED STRANG RECURENCES.
*
* PARAMETERS :
*  II  N  NUMBER OF ROWS OF  MATRICES XM, GM.
*  II  M  NUMBER OF COLUMNS OF MATRICES XM, GM.
*  RU  XM(N*M)  TRANSFORMED RECTANGULAR MATRIX OF VARIABLES DIFFERENCES
*  RU  GM(N*M)  TRANSFORMED RECTANGULAR MATRIX OF GRADIENTS DIFFERENCES
*  RI  U(N)  INPUT PREMULTIPLIED VECTOR.
*  RO  V(N)  OUTPUT PREMULTIPLIED VECTOR.
*  RO  GR(M)  AUXILIARY VECTOR.
*  RI  XR(M)  TRANSFORMED NONQUADRATIC CORRECTION PARAMETERS VECTOR.
*  RI  PAR  INPUT SCALING PARAMETER.
*
* METHOD :
* H.MATTHIES, G.STRANG: THE SOLUTION OF NONLINEAR FINITE ELEMENT
* EQUATIONS. INT.J.NUMER. METHODS ENGN. 14 (1979) 1613-1626.
*
      SUBROUTINE UXDRSV(N,M,XM,GM,U,V,GR,XR,PAR)
      INTEGER M,N
      $FLOAT XM(N*M),GM(N*M),U(N),V(N),GR(M),XR(M),PAR
      $FLOAT TEMP,UXVDOT
      INTEGER I,L
C
C     STRANG RECURRENCES
C
      CALL UXVSCL(N,PAR,U,V)
      L=M*N
      DO 1 I=M,1,-1
        L=L-N
        TEMP=UXVDOT(N,V,XM(L+1))
        GR(I)=TEMP
        CALL UXVDIR(N,-TEMP,GM(L+1),V,V)
    1 CONTINUE
      DO 2 I=1,M
        TEMP=XR(I)*GR(I)/PAR-UXVDOT(N,V,GM(L+1))
        CALL UXVDIR(N,TEMP,XM(L+1),V,V)
        L=L+N
    2 CONTINUE
      RETURN
      END
