* SUBROUTINE UXDSBB                ALL SYSTEMS                92/12/01
C PORTABILITY : ALL SYSTEMS
C 92/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* SOLUTION OF A SYSTEM OF LINEAR EQUATIONS WITH A DENSE SYMMETRIC
* MATRIX A USING THE FACTORIZATION A=P*L*D*TRANS(L)*TRANS(P) OBTAINED
* BY THE SUBROUTINE UXDSBF.
*
* PARAMETERS :
*  II  N ORDER OF THE MATRIX A.
*  RI  A(N*(N+1)/2)  FACTORIZATION A=P*L*D*TRANS(L)*TRANS(P) OBTAINED
*         BY THE SUBROUTINE UXDSBF.
*  II  IA(N)  PERMUTATION FIELD OBTAINED BY THE SUBROUTINE UXDSBF.
*  RU  X(N)  ON INPUT THE RIGHT HAND SIDE OF A SYSTEM OF LINEAR
*         EQUATIONS. ON OUTPUT THE SOLUTION OF A SYSTEM OF LINEAR
*         EQUATIONS.
*  II  JOB  OPTION. IF JOB=0 THEN X:=A**(-1)*X. IF JOB>0 THEN
*         X:=(P*L)**(-1)*X. IF JOB<0 THEN X:=(TRANS(L)*TRANS(P))**(-1)*X
*
* METHOD :
* BACK SUBSTITUTION.
*
      SUBROUTINE UXDSBB(N,A,IA,X,JOB)
      INTEGER N, IA(N), JOB
      $FLOAT  A(N*(N+1)/2), X(N)
      $FLOAT  XI, XJ, DET
      INTEGER  IS, IP, JP, I, J, II, JJ, IJ, IB, IND
      $FLOAT ZERO,CON
      PARAMETER (ZERO=0.0$P 0,CON=1.0$P-20)
      IND(I,J) = MAX(I,J) * (MAX(I,J) - 1) / 2 + MIN(I,J)
      IF (JOB .GE. 0) THEN
C
C     PHASE 1 : X:=(P*L)**(-1)*X
C
      IS = 1
      DO 2 IP=1,N
      IF (IS.GT.0) IB=IP-1
      I = IA(IP)
      IF (I .LT. 0) THEN
      I = -I
      IS = -IS
      ENDIF
      DO 1 JP=1,IB
      J = IA(JP)
      IF (J .LT. 0)  J = -J
      IJ = IND(I,J)
      X(I) = X(I) - A(IJ)*X(J)
    1 CONTINUE
    2 CONTINUE
      ENDIF
      IF (JOB .EQ. 0) THEN
C
C     PHASE 2 : X:=D**(-1)*X
C
      IP=1
    3 IF (IP.LE.N) THEN
      I = IA(IP)
      IF (I .GE. 0) THEN
      II = IND(I,I)
      DET = A(II)
      IF (DET .EQ. ZERO) DET = CON
      X(I) = X(I) / DET
      ELSE
      IP = IP + 1
      I = -I
      J = -IA(IP)
      II = IND(I,I)
      IJ = IND(I,J)
      JJ = IND(J,J)
      DET = A(II)*A(JJ) - A(IJ)*A(IJ)
      IF (DET .EQ. ZERO) DET = CON
      XI = A(JJ)*X(I) - A(IJ)*X(J)
      XJ = A(II)*X(J) - A(IJ)*X(I)
      X(I) = XI / DET
      X(J) = XJ / DET
      ENDIF
      IP=IP+1
      GO TO 3
      ENDIF
      ENDIF
      IF (JOB .LE. 0) THEN
C
C     PHASE 3 : X:=(TRANS(L)*TRANS(D))**(-1)*X
C
      IS = 1
      DO 5 IP = N, 1, -1
      IF (IS.GT.0) IB=IP+1
      I = IA(IP)
      IF (I .LT. 0) THEN
      I = -I
      IS = -IS
      ENDIF
      DO 4 JP=IB,N
      J = IA(JP)
      IF (J .LT. 0)  J = -J
      IJ = IND(I,J)
      X(I) = X(I) - A(IJ)*X(J)
    4 CONTINUE
    5 CONTINUE
      ENDIF
      RETURN
      END
