* SUBROUTINE UXDSBD                ALL SYSTEMS                91/12/01
C PORTABILITY : ALL SYSTEMS
C 91/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* COMPUTATION OF A DIRECTION OF NEGATIVE CURVATURE WITH RESPECT TO A
* DENSE SYMMETRIC MATRIX A USING THE FACTORIZATION
* A=P*L*D*TRANS(L)*TRANS(P) OBTAINED BY THE SUBROUTINE UXDPGF.
*
* PARAMETERS :
*  II  N ORDER OF THE MATRIX A.
*  RI  A(N*(N+1)/2)  FACTORIZATION A=P*L*D*TRANS(L)*TRANS(P) OBTAINED
*         BY THE SUBROUTINE UXDSBF.
*  II  IA(N)  PERMUTATION FIELD OBTAINED BY THE SUBROUTINE UXDSBF.
*  RO  X(N)  COMPUTED DIRECTION OF NEGATIVE CURVATURE (I.E.
*         TRANS(X)*A*X<0) IF IT EXISTS.
*  RO  ALF  VALUE OF TRANS(X)*A*X
*  II  INF  INFORMATION OBTAINED IN THE FACTORIZATION PROCESS. THE
*         DIRECTION OF NEGATIVE CURVATURE EXISTS ONLY IF INF>0.
*
* METHOD :
* R.FLETCHER, T.L.FREEMAN: A MODIFIED NEWTON METHOD FOR MINIMIZATION,
* J.OPTIMIZATION THEORY APPL. 23 (1977) PP. 357-372.
*
      SUBROUTINE UXDSBD(N,A,IA,X,ALF,INF)
      INTEGER N, IA(N), INF
      $FLOAT  A(N*(N+1)/2), X(N), ALF
      $FLOAT  DET, DEN, AII, AIJ, AJJ
      $FLOAT UXDSBQ
      INTEGER  IP, I, J, IND
      $FLOAT ZERO, ONE, HALF
      PARAMETER (ZERO = 0.0$P 0, ONE = 1.0$P 0, HALF = 0.5$P 0)
      IND(I,J) = MAX(I,J) * (MAX(I,J) - 1) / 2 + MIN(I,J)
      IF (INF .LE. 0) RETURN
C
C     RIGHT HAND SIDE FORMATION
C
      IP=1
    1 IF (IP.LE.N) THEN
        I = IA(IP)
        IF (I .GE. 0) THEN
          X(I) = ZERO
          IF (A( IND(I,I) ).LE.ZERO) X(I)=ONE
        ELSE
          IP = IP + 1
          I = -I
          J = -IA(IP)
          AII = A(IND(I,I))
          AIJ = A(IND(I,J))
          AJJ = A(IND(J,J))
          X(I) = ZERO
          X(J) = ZERO
          IF (AII+AJJ.LT.ZERO .OR. AII*AJJ.LT.AIJ*AIJ) THEN
            IF (AIJ.NE.ZERO) THEN
              DEN=HALF*(AJJ-AII)
              DET=DEN-SQRT(DEN*DEN+AIJ*AIJ)
              DEN=    SQRT(DET*DET+AIJ*AIJ)
              X(I)=AIJ/DEN
              X(J)=DET/DEN
            ELSE
              IF(AII.GE.AJJ) X(J)=ONE
              IF(AII.LE.AJJ) X(I)=ONE
            ENDIF
          ENDIF
        ENDIF
        IP=IP+1
        GOTO 1
      ENDIF
C
C     BACK SUBSTITUTION
C
      ALF=UXDSBQ(N,A,IA,X,X)
      CALL UXDSBB(N,A,IA,X,-1)
      RETURN
      END
