* SUBROUTINE UXDSGB                ALL SYSTEMS                91/12/01
C PORTABILITY : ALL SYSTEMS
C 91/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* SOLUTION OF A SYSTEM OF LINEAR EQUATIONS WITH A DENSE SYMMETRIC
* POSITIVE DEFINITE MATRIX A+E USING THE FACTORIZATION
* A+E=P*L*D*TRANS(L)*TRANS(P) OBTAINED BY THE SUBROUTINE UXDSGF.
*
* PARAMETERS :
*  II  N ORDER OF THE MATRIX A.
*  RI  A(N*(N+1)/2) FACTORIZATION A+E=P*L*D*TRANS(L)*TRANS(P) OBTAINED
*         BY THE SUBROUTINE UXDSGF.
*  II  IA(N)  A PERMUTATION FIELD WHICH CONTAINS AN INFORMATION ABOUT
*         PERMUTATION MATRIX P.
*  RU  X(N)  ON INPUT THE RIGHT HAND SIDE OF A SYSTEM OF LINEAR
*         EQUATIONS. ON OUTPUT THE SOLUTION OF A SYSTEM OF LINEAR
*         EQUATIONS.
*  II  JOB  OPTION. IF JOB=0 THEN X:=(A+E)**(-1)*X. IF JOB>0 THEN
*         X:=(P*L)**(-1)*X. IF JOB<0 THEN X:=(TRANS(L)*TRANS(P))**(-1)*X
*
* METHOD :
* BACK SUBSTITUTION.
*
      SUBROUTINE UXDSGB(N,A,IA,X,JOB)
      INTEGER  N, IA(N), JOB
      $FLOAT  A(N*(N+1)/2), X(N)
      INTEGER  I, J, IP, JP, II, IJ, IND
      $FLOAT ZERO,CON
      PARAMETER (ZERO=0.0$P 0,CON=1.0$(P)120)
      IND(I,J) = MAX(I,J) * (MAX(I,J) - 1) / 2 + MIN(I,J)
      IF (JOB .GE. 0) THEN
C
C     PHASE 1 : X:=L**(-1)*X
C
      DO 3 I=1,N
      IP=IA(I)
      DO 2 J=1,I-1
      JP=IA(J)
      IJ=IND(IP,JP)
      X(IP) = X(IP) - A(IJ)*X(JP)
    2 CONTINUE
    3 CONTINUE
      ENDIF
      IF (JOB .EQ. 0) THEN
C
C     PHASE 2 : X:=D**(-1)*X
C
      DO 4 I=1,N
      IP=IA(I)
      II=IND(IP,IP)
      X(IP) = X(IP) / A(II)
    4 CONTINUE
      ENDIF
      IF (JOB .LE. 0) THEN
C
C     PHASE 3 : X:=TRANS(L)**(-1)*X
C
      DO 6  I = N-1, 1, -1
      IP=IA(I)
      DO 5  J = I+1, N
      JP=IA(J)
      IJ=IND(IP,JP)
      X(IP) = X(IP) - A(IJ)*X(JP)
    5 CONTINUE
    6 CONTINUE
      ENDIF
      RETURN
      END
