* SUBROUTINE UXDTGC                ALL SYSTEMS               92/12/01
C PORTABILITY : ALL SYSTEMS
C 92/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* RANK REVEALING QR FACTORIZATION.
*
* PARAMETERS :
*  II  N  ORDER OF THE MATRIX A.
*  IU  M  NUMERICAL RANK OF THE MATRIX A.
*  RU  A(N*(N+1)/2) UPPER TRIANGULAR MATRIX.
*  IO  IA(N)  PERMUTATION VECTOR.
*  RU  B(N)  RIGHT HAND SIDE VECTOR OBTAINED USING ORTHOGONAL
*         TRANSFORMATION OF A SET OF LINEAR EQUATIONS.
*  RA  X(N)  AUXILIARY.
*  RI  TOL  TOLERANCE FOR NUMERICAL RANK.
*
* SUBPROGRAMS USED :
*  S   UXDSRO  INCREMENTAL CONDITION ESTIMATION.
*  S   UXDTGV  COLUMN PERMUTATION OF A PERMUTED UPPER TRIANGULAR
*         MATRIX.
*  S   UXVINP  INITIATION OF THE PERMUTATION VECTOR.
*
* METHOD :
* T.F.CHAN : RANK REVEALING QR FACTORIZATIONS. LINEAR ALGEBRA APPL.
* 88/89 (1987) 67-82.
*
      SUBROUTINE UXDTGC(N,M,A,IA,B,X,TOL)
      INTEGER N,M,IA(N)
      $FLOAT A(N*(N+1)/2),B(N),X(N),TOL
      $FLOAT DEL,TEMP,TOL1
      INTEGER I,K,IP,KP,II,IK,IND
      $FLOAT ZERO
      PARAMETER (ZERO=0.0$P 0)
      IND(I,K)=MAX(I,K)*(MAX(I,K)-1)/2+MIN(I,K)
      CALL UXVINP(N,IA)
      IF(M.LT.N) THEN
C
C     CORRECTION OF THE INPUT MATRIX
C
      DO 2 I=M+1,N
      IP=IA(I)
      DO 1 K=I,N
      KP=IA(K)
      IK=IND(IP,KP)
      A(IK)=ZERO
    1 CONTINUE
    2 CONTINUE
      ENDIF
      TOL1=SQRT(TOL)
      TEMP=TOL1
      DO 3 I=1,N*(N+1)/2
      TEMP=MAX(TEMP,ABS(A(I)))
    3 CONTINUE
      TEMP=TEMP*TOL1
      DO 5 M=N,1,-1
C
C     INCREMENTAL CONDITION ESTIMATION
C
      CALL UXDSRO(M,A,IA,X,DEL,2)
      DEL=SQRT(DEL)
      IF (DEL.GT.TEMP) GO TO 6
C
C     DETERMINATION OF THE NEW PERMUTATION
C
      K=M
      DEL=ZERO
      DO 4 I=1,M
      IP=IA(I)
      IF (DEL.LE.ABS(X(IP))) THEN
      DEL=ABS(X(IP))
      K=I
      ENDIF
    4 CONTINUE
C
C     NEW PERMUTATION
C
      CALL UXDTGV(N,A,IA,B,K,M)
    5 CONTINUE
    6 IF(M.LT.N) THEN
C
C     CORRECTION OF THE OUTPUT MATRIX
C
      DO 7 I=M+1,N
      IP=IA(I)
      II=IND(IP,IP)
      A(II)=SIGN(MAX(TEMP,ABS(A(II))),A(II))
    7 CONTINUE
      ENDIF
      RETURN
      END
