* SUBROUTINE UXSGDF                 ALL SYSTEMS                95/12/01
C PORTABILITY : ALL SYSTEMS
C 95/12/01 DA : ORIGINAL VERSION
*
* PURPOSE :
* SPARSE SYMBOLIC AND NUMERICAL LU FACTORIZATION OF NONSYMMETRIC
* MATRIX FROM UMFPACK.
*
* PARAMETERS :
*  RA  XMEM(MMAX)  AUXILIARY REAL STORAGE.
*  IA  IMEM(MMAX)  AUXILIARY INTEGER STORAGE.
*  IO  PMEM(7,2)  STATUS OF AUXILIARY REAL AND INTEGER STORAGE
*  RI  AVALUE(NZ)  NONZERO ELEMENTS OF A SPARSE MATRIX.
*  II  AINDC(NZ)  COLUMN INDICES OF THE NONZERO ELEMENTS.
*  II  AINDR(NZ)  ROW INDICES OF THE NONZERO ELEMENTS.
*  II  N  ORDER OF A SPARSE MATRIX.
*  II  NZ NUMBER OF NONZERO ELEMENTS OF A SPARSE MATRIX.
*  IO  LU(2,2)  POINTERS OF LU FACTOR IN AN AUXILIARY STORAGE.
*  IO  ERROR  ERROR MESSAGE.
*  RI  ABSPT  ABSOLUTE TOLERANCE FOR A PIVOT.
*  II  NB  BLOCK SIZE FOR NUMERICAL FACTORIZATION (USUALLY 16 - 64).
*
      SUBROUTINE UXSGDF(XMEM,IMEM,PMEM,AVALUE,AINDC,AINDR,N,NZ,LU,
     & ERROR,ABSPT,NB)
      INTEGER N,NZ,IMEM(*),PMEM(7,2),AINDC(NZ),AINDR(NZ),NINVLD,
     & LU(2,2),ERROR,NB,NPIV,XGAR
      $FLOAT XMEM(*),AVALUE(NZ),ABSPT,PMIN
      $FLOAT ABSPT2
       INTEGER LU2(2,2),MMEM(7,2),XP,IP,XSIZE,ISIZE,K,
     & NB2,AN,ANZ2,ANZDIA,ANZOFF,ABLKS,USAGE,BARE,
     & LUNS,LUPP,AOFFXP,PERMP,LUBLPP,ABLKPP,AOFFPP,AOFFIP
      LOGICAL TRANS,DEALLO
      TRANS=.FALSE.
      DEALLO=.TRUE.
      IF (NZ.LT.0) GOTO 995
      IF (LU (1,1) .LE. 0 .OR. (LU(2,1)-LU(1,1)).LT.8 .OR.
     & LU (1,2) .LE. 0 .OR. (LU(2,2)-LU(1,2)).LT.0 ) GOTO 999
      CALL UXIMCO(PMEM,ERROR)
      IF (ERROR .NE. 0) GOTO 998
      IF (ABSPT .LE. 0.0) THEN
      ABSPT2 = 0.0
      ELSE
      ABSPT2 = ABSPT
      ENDIF
      NB2 = MAX (1, NB)
      NB2 = MIN (NB2, N)
      CALL UXSGF7(IMEM(LU(1,1)),AN,ABLKS,LUNS,ANZOFF,ANZ2,
     & ANZDIA,ERROR,LUPP,AOFFXP,PERMP,LUBLPP,
     & ABLKPP,AOFFPP,AOFFIP,LU)
      IF (ERROR .NE. 0) GOTO 999
      IF (AN .NE. N) GOTO 997
      IF (DEALLO) THEN
C
C     DEALLOCATE OLD AOFFX AND LU NUMERICAL VALUES
C
      CALL UXIFRE(PMEM,ERROR,2,
     & LU(2,2)-LU(1,2),LU(1,2))
      IF (ERROR .NE. 0) GOTO 998
      ENDIF
C
C     INITALIZE ALLOCABLE MEMORY: XX/II, WHICH ARE THE SIZE OF
C     THE CURRENTLY AVAILABLE FREE MEMORY IN XMEM AND IMEM. USE
C     XX AND II AS ALLOCABLE MEMORY, AND  LUPAT AS STATIC LU PATTERN
C     (LUPAT IS IMEM (LU(1,1),...,LU(1,2)-1)).
C
      XGAR = 0
      XP = PMEM (1,2)
      IP = PMEM (1,1)
      XSIZE = PMEM (2,2) - XP
      ISIZE = PMEM (2,1) - IP
      CALL UXIINI(MMEM,ERROR,1,
     & ISIZE+1,1,XSIZE+1)
      IF (ERROR .NE. 0) GOTO 998
      LU2 (1,1) = 1
      LU2 (2,1) = LU (2,1) - LU (1,1) + 1
C
C     INITIALIZATIONS AND FACTORIZATION
C
      CALL UXSGF1(LU2,AINDC,AINDR,AVALUE,XMEM(XP),IMEM(IP),ERROR,
     & MMEM,XGAR,IMEM(LU(1,1)),
     & IMEM(LU(1,1)-1+PERMP),IMEM(LU(1,1)-1+PERMP+AN),
     & IMEM(LU(1,1)-1+LUBLPP),IMEM(LU(1,1)-1+ABLKPP),
     & IMEM(LU(1,1)-1+LUPP),IMEM(LU(1,1)-1+AOFFPP),
     & IMEM(LU(1,1)-1+AOFFIP),NB2,NPIV,AN,NZ,
     & ANZ2,ANZOFF,ABLKS,LUNS,ABSPT2,
     & PMIN,NINVLD,TRANS)
C
C     ADJUST PMEM TO REFLECT USAGE OF IMEM/XMEM DURING FACTORIZATION
C
      DO 10 K = 1, 2
      USAGE = (PMEM(1,K) - PMEM(3,K)) + (PMEM(4,K) - PMEM(2,K))
      BARE = PMEM (6,K)
      PMEM (5,K) = MAX (PMEM (5,K), USAGE + MMEM (5,K))
      PMEM (7,K) = MAX (PMEM (7,K), BARE + MMEM (7,K))
10    CONTINUE
      IF (ERROR .NE. 0) RETURN
C
C     THE LU FACTORS ARE ALLOCATED IN MMEM, BUT NOT IN PMEM
C
      CALL UXIGET(PMEM,ERROR,2,
     & LU2(2,2)-LU2(1,2),1,LU(1,2))
      IF (ERROR .NE. 0) GOTO 998
      IF (LU (1,2) .EQ. 0) GOTO 996
      LU (2,2) = LU (1,2) + (LU2 (2,2) - LU2 (1,2))
      RETURN
C
C     ERROR CONDITIONS
C
  999 ERROR = 67
      CALL UOERR1('UXSGDF',ERROR)
      RETURN
  998 ERROR = 64
      CALL UOERR1('UXSGDF',ERROR)
      RETURN
  997 ERROR = 61
      CALL UOERR1('UXSGDF',ERROR)
      RETURN
  996 ERROR = 66
      CALL UOERR1('UXSGDF',ERROR)
      RETURN
  995 ERROR = 62
      CALL UOERR1('UXSGDF',ERROR)
      RETURN
      END
