* SUBROUTINE UXSGFC                 ALL SYSTEMS                91/12/01
C PORTABILITY : ALL SYSTEMS
C 91/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* COMPRESSION OF CNR USED THROUGHOUT THE FACTORIZATION.
*
* PARAMETERS :
*  II  N  NUMBER OF ROWS OF THE SPARSE MATRIX
*  II  M  NUMBER OF PROCESSED COLUMNS THAT FORM THE COLUMS OF THE MATRIX
*         THAT DESCRIBES THE ACTIVE SET. M <= N.
*  II  NZ SIZE OF THE INDEX ARRAY CNR
*  II  NAU SIZE OF THE ARRAY THAT STORES THE NUMERICAL ELEMENTS OF
*          THE PROCESSED MATRIX.
*  IU  PTC(N) POINTER VECTOR OF THE COLUMNS OF THE SPARSE MATRIX.
*  II  LENC(N) LENGTHS OF THE COLUMNS.
*  IU  CNR(NZ) CONTAINS THE INDICES OF THE COLUMN AND ROW SCHEME
*              OF THE PROCESSED MATRIX.
*  IU  PTR(N) POINTER VECTOR OF THE ROWS OF THE SPARSE MATRIX.
*  IO  LENR(N) LENGTHS OF THE ROWS.
*  RU  AU(NAU) ELEMENTS OF THE PROCESSED SPARSE MATRIX.
*  IO  IBGR BEGINNING OF THE ROW SCHEME IN CNR.
*  IO  IENDLU END OF THE COLUMN SCHEME IN CNR.
*
      SUBROUTINE UXSGFC(N,M,NZ,NAU,PTC,LENC,CNR,PTR,LENR,
     *                 AU,IBGR,IENDLU)
      INTEGER N,M,NZ,NAU
      INTEGER PTC(N),LENC(N),PTR(N),LENR(N)
      INTEGER CNR(NZ)
      INTEGER IBGR,IENDLU
      $FLOAT  AU(NAU)
      $FLOAT AUI
      INTEGER I,J,K,L,ISTRT,IDIST,ISIG
C
C     INITIALIZATION OF THE DISTANCE PARAMETER
C
      IDIST=0
C
C     MARK THE COLUMNS' BEGINNINGS
C
      DO 100 I=1,M
      K=PTC(I)
      IF(LENC(I).NE.0) THEN
          IF(K.LT.0) THEN
                     K=-K
                     ISIG=-1
                     ELSE
                     ISIG=1
          END IF
                       L=CNR(K)
                       CNR(K)=-I
                       PTC(I)=ISIG*L
                       ELSE
                       PTC(I)=SIGN(1,K)
      END IF
100   CONTINUE
C
C     MARK THE ROWS' BEGINNINGS
C
      DO 200 I=1,N
      K=PTR(I)
      IF (K.EQ.0) THEN
                  GO TO 200
            ELSE
            K=ABS(K)+LENR(I)-1
            IF(LENR(I).NE.0) THEN
                             L=CNR(K)
                             CNR(K)=-I
                             PTR(I)=L
                             ELSE
                             PTR(I)=NZ
            END IF
      END IF
200   CONTINUE
C
C     COLUMNS' GARBAGE COLLECTION
C
      ISTRT=1
      I=1
300   K=CNR(I)
      IF(K.LT.0) THEN
                 AUI=AU(I)
                 DO 400 J=1,LENC(-K)-1
                 CNR(ISTRT+J)=CNR(I+J)
                 AU(ISTRT+J)=AU(I+J)
400              CONTINUE
                 AU(ISTRT)=AUI
                 IF(PTC(-K).LT.0) THEN
                            CNR(ISTRT)=-PTC(-K)
                            PTC(-K)=-ISTRT
                            ISTRT=ISTRT+LENC(-K)
                                   ELSE
                            CNR(ISTRT)=PTC(-K)
                            PTC(-K)=ISTRT
                            ISTRT=ISTRT+LENC(-K)+IDIST
                 END IF
                 I=I+LENC(-K)
                 ELSE
                 I=I+1
      END IF
      IF(I.LT.IENDLU) GO TO 300
      DO 450 I=ISTRT,IENDLU
      CNR(I)=0
450   CONTINUE
      IENDLU=ISTRT-1
C
C     ROWS' COMPRESSION
C
      ISTRT=NZ-1
      I=ISTRT
500   K=CNR(I)
      IF(K.LT.0) THEN
                 DO 600 J=1,LENR(-K)-1
                 CNR(ISTRT-J)=CNR(I-J)
600              CONTINUE
                 CNR(ISTRT)=PTR(-K)
                 PTR(-K)=ISTRT-J+1
                 ISTRT=ISTRT-LENR(-K)-IDIST
                 I=I-LENR(-K)
                 ELSE
                 I=I-1
      END IF
      IF(I.GT.IBGR) GO TO 500
C
C     SET THE ZEROES IN THE FREED SPACE
C
      DO 800 I=IBGR,ISTRT
      CNR(I)=0
800   CONTINUE
      IBGR=ISTRT+1
      RETURN
      END
