* SUBROUTINE UXSGUR                 ALL SYSTEMS                91/12/01
C PORTABILITY : ALL SYSTEMS
C 91/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* NUMERICAL PART OF SPARSE COLUMN EXCHANGE.
*
* PARAMETERS :
*  II  N  NUMBER OF ROWS OF THE SPARSE MATRIX
*  II  M  NUMBER OF PROCESSED COLUMNS THAT FORM THE COLUMS OF THE MATRIX
*         THAT DESCRIBES THE ACTIVE SET. M <= N.
*  II  NZ SIZE OF THE INDEX ARRAY CNR
*  II  NAU SIZE OF THE ARRAY THAT STORES THE NUMERICAL ELEMENTS OF
*          THE PROCESSED MATRIX.
*  II  NEL NUMBER OF L-UPDATES IN THE ARRAYS LI,LJ,CM.
*  II  NUP SIZE OF THE L-UPDATE ARRAYS (LI,LJ,CM)
*  IU  PTR(N) POINTER VECTOR OF THE ROWS OF THE FACTOR U.
*  IU  LENR(N) LENGTHS OF THE ROWS OF THE FACTOR U.
*  IU  CNR(NZ) CONTAINS THE INDICES OF THE FACTORS.
*  IU  ORDR(N) ROW PERMUTATION OF THE FACTOR U.
*  IU  ORDC(N) COLUMN PERMUTATION OF THE FACTOR U.
*  IA  WR01(N) AUXILIARY VECTOR
*  IA  WR02(N) AUXILIARY VECTOR
*  IU  LI(NUP) ROW INDICES OF THE L-UPDATES.
*  IU  LJ(NUP) COLUMN INDICES OF THE L-UPDATES.
*  RU  CM(NUP) UPDATE MULTIPLIERS.
*  RU  AU(NAU) NUMERICAL VALUES OF THE FACTORS.
*  RI  STFL  STABILITY FLAG.
*  RU  MMFL  MAXIMUM MULTIPLIER FOUND.
*  RI  DRFL  DROPPING PARAMETER.
*  RU  MXU   MAXIMUM ELEMENT OF THE U-DIAGONAL.
*  RU  MNU   MINIMUM ELEMENT OF THE U-DIAGONAL.
*  IU  IBGR  BEGINNING OF THE COLUMN SCHEME IN CNR(IF IFL.EQ.0 AFTER
*            FACTORIZATION).
*  IU  IENDLU  END OF THE ROW INDICES OF THE FACTOR U IN CNR.
*  II  IBEG  BEGINNING OF THE FACTOR U IN CNR.
*  IO  IFL  OUTPUT INFORMATION:
*                              =11 IF NAU IS DECLARED TOO SMALL
*                              =12 IF MATRIX IS SINGULAR
*                              =13 IF FACTORS ARE BADLY CONDITIONED.
*                              =14 LITTLE SPACE FOR L-UPDATES.
*                              =15 MATRIX SINGULARITY IS FACED.
*
      SUBROUTINE UXSGUR(N,M,NZ,NAU,NEL,NUP,ROWUP,PTR,LENR,CNR,ORDR,
     *                  WR01,WR02,LI,LJ,CM,AU,STFL,MMFL,
     *                  DRFL,MXU,MNU,IENDLU,IBEG,IFL)
      INTEGER N,M,NZ,NAU,NEL,NUP,ROWUP
      INTEGER IENDLU,IBEG,IFL
      INTEGER PTR(N),LENR(N),ORDR(N)
      INTEGER CNR(NZ),LI(NUP),LJ(NUP)
      INTEGER RM,RI,RRM,RJ,PIVEL
      $FLOAT PT
      REAL*8 STFL,MMFL,DRFL,MXU,MNU,MI,EPS,ALFA,BETA,ZERO,TEMP
      REAL*8 WR01(N),WR02(N)
      REAL*8 CM(NUP),AU(NAU)
      INTEGER I,J,K,L,L2,I1,I2,K1,J1,J2,J3,J4,JSTRT,JSTOP,IND
      INTEGER IQCMPR,ISTR1,ISTRT,ISTOP,JR
      PARAMETER (EPS=1.0D-6,ZERO=0.0D0)
C
C     INITIALIZATION
C
      IND=0
      PT=0.05
      RI=ORDR(ROWUP)
      RM=N+1
      RRM=RM
      ISTRT=PTR(RI)
      ISTOP=ISTRT+LENR(RI)-1
      DO 50 I=1,N
      WR01(I)=0
50    CONTINUE
C
C     FIND THE ELEMENT THAT IS TO BE ELIMINATED
C
      DO 100 I=ISTRT,ISTOP
      K=CNR(I)
      WR01(K)=I
      IF(K.LE.ROWUP) THEN
                     IND=IND+1
                     WR02(IND)=K
          IF(RRM.GT.K) THEN
                       RM=IND
                       RRM=WR02(RM)
          END IF
      END IF
100   CONTINUE
C
C     MAIN CYCLE ON THE ELEMENTS OF WR02(1),...,WR02(IND)
C     DURING WHICH IND IS UPDATED.
C
C
C     TEST ON THE END
C
140   IF(RRM.GE.ROWUP) GO TO 750
C
C     FIND THE MULTIPLIER
C
      PIVEL=WR01(RRM)
      RJ=ORDR(RRM)
      JSTRT=PTR(RJ)
      JSTOP=JSTRT+LENR(RJ)-1
      ALFA=AU(JSTRT)
      BETA=AU(PIVEL)
      MI=BETA/ALFA
C     IF(MI.NE.0) MI=100
C
C     MULTIPLIER SIZE TEST
C
C     IF(ABS(MI).GT.STFL) THEN
C  *****
      IF(ABS(MI).GT.STFL.OR.ABS(MI).LT.1.AND.ABS(MI).GT.PT
     *   .AND.LENR(RJ).GT.LENR(RI)) THEN
C  *****
C
C     SET THE DIAGONAL ELEMENT OF THE NEW DIAGONAL
C
      L=CNR(PIVEL)
      TEMP=AU(PIVEL)
      CNR(PIVEL)=CNR(ISTRT)
      AU(PIVEL)=AU(ISTRT)
      CNR(ISTRT)=L
      AU(ISTRT)=TEMP
C
C     EMPTY WR01
C
      DO 200 J=ISTRT,ISTOP
      K1=CNR(J)
      WR01(K1)=0
200   CONTINUE
C
C     DROPPING ELEMENTS
C
      J=ISTRT+1
      K1=ISTRT+1
      IF(J.GT.ISTOP) GO TO 260
250   IF(ABS(AU(J)).GT.DRFL) THEN
                             AU(K1)=AU(J)
                             CNR(K1)=CNR(J)
                             K1=K1+1
      END IF
      J=J+1
      IF(J.LE.ISTOP) GOTO 250
260   LENR(RI)=K1-ISTRT
      ISTOP=K1-1
C
C     ROW EXCHANGE
C
      IND=0
      ORDR(ROWUP)=RJ
      ORDR(RRM)=RI
      JR=RJ
      RJ=RI
      RI=JR
C
C     SET THE ROW BOUNDS
C
      L=JSTRT
      K=JSTOP
      JSTRT=ISTRT
      JSTOP=ISTOP
      ISTRT=L
      ISTOP=K
C
C     SET THE DIAGONAL OF THE NEW PIVOT ROW
C
C
C     SEARCH THROUGH THE ROW
C
      I1=0
      DO 270 I=ISTRT,ISTOP
      IF(CNR(I).EQ.ROWUP) THEN
                          I1=1
                          PIVEL=I
      END IF
270   CONTINUE
C
C     FILL THE DIAGONAL
C
      IF(I1.EQ.0) THEN
C
280   IF(CNR(ISTOP+1).EQ.0) THEN
                            IQCMPR=0
                            ISTOP=ISTOP+1
                            CNR(ISTOP)=CNR(ISTRT)
                            AU(ISTOP)=AU(ISTRT)
                            AU(ISTRT)=ZERO
                            CNR(ISTRT)=ROWUP
                            IENDLU=MAX(IENDLU,ISTOP)
                            LENR(RI)=LENR(RI)+1
                            ISTR1=ISTOP
                            ELSE
                            J1=ISTOP-ISTRT+1
          IF((IENDLU+J1).LT.(NAU-1))       THEN
                                  IQCMPR=0
                                  DO 290 J2=1,J1
                                  J3=ISTRT+J2-1
                                  CNR(J2+IENDLU)=CNR(J3)
                                  AU(J2+IENDLU)=AU(J3)
                                  CNR(J3)=0
290                               CONTINUE
                                  ISTRT=IENDLU+1
                                  PTR(RI)=ISTRT
                                  ISTOP=IENDLU+J1
                                  GO TO 280
                                           ELSE
              IF(IQCMPR.EQ.1) THEN
                              IFL=11
                              RETURN
              END IF
              IQCMPR=1
              CALL UXSGUG(N,M,NZ,NAU,PTR,LENR,CNR,
     *                    AU,IBEG,IENDLU)
              ISTRT=PTR(RI)
              ISTOP=ISTRT+LENR(RI)-1
              JSTRT=PTR(RJ)
              JSTOP=JSTRT+LENR(RJ)-1
              GO TO 280
          END IF
      END IF
      END IF
      IF(I1.EQ.1) THEN
                  K=CNR(PIVEL)
                  TEMP=AU(PIVEL)
                  CNR(PIVEL)=CNR(ISTRT)
                  AU(PIVEL)=AU(ISTRT)
                  CNR(ISTRT)=K
                  AU(ISTRT)=TEMP
      END IF
C
C     FILL WR01,WR02
C
      DO 300 I=ISTRT,ISTOP
      K1=CNR(I)
      WR01(K1)=I
      IF(K1.LE.ROWUP) THEN
                      IND=IND+1
                      WR02(IND)=K1
      END IF
300   CONTINUE
C
C     NEW MULTIPLIER
C
      MI=ALFA/BETA
      IF(I1.EQ.1) THEN
                  RM=N+1
                  RRM=RM
                  DO 310 J=1,IND
                  K=WR02(J)
                  IF(RRM.GT.K) THEN
                               RM=J
                               RRM=K
                  END IF
310               CONTINUE
                  ISTR1=PIVEL
                  ELSE
                  RM=IND
                  RRM=WR02(IND)
      END IF
                     ELSE
      ISTR1=PIVEL
      END IF
C
C     L-FACTOR UPDATE
C
      NEL=NEL+1
      IF(NEL.GT.NUP) THEN
                     IFL=14
                     RETURN
      END IF
      CM(NEL)=MI
      LI(NEL)=RI
      LJ(NEL)=RJ
C
C     MMFL UPDATE
C
      MMFL=MAX(ABS(MI),MMFL)
C
C     ELIMINATION
C
      L=CNR(ISTOP)
      CNR(ISTR1)=L
      CNR(ISTOP)=0
      AU(ISTR1)=AU(ISTOP)
      WR01(L)=ISTR1
      WR01(RRM)=0
      ISTOP=ISTOP-1
      LENR(RI)=LENR(RI)-1
C
C     INNER LOOP
C
      DO 1000 I1=2,JSTOP-JSTRT+1
      I2=CNR(I1+JSTRT-1)
      L2=WR01(I2)
      IF(L2.GT.0) THEN
          AU(L2)=AU(L2)-MI*AU(I1+JSTRT-1)
                  ELSE
C
C     ADD FILL-IN
C
400   IF(CNR(ISTOP+1).EQ.0) THEN
                            IQCMPR=0
                            ISTOP=ISTOP+1
                            CNR(ISTOP)=I2
                            AU(ISTOP)=-MI*AU(I1+JSTRT-1)
                            IENDLU=MAX(IENDLU,ISTOP)
                            LENR(RI)=LENR(RI)+1
                            IND=IND+1
                            WR02(IND)=I2
                            WR01(I2)=ISTOP
                            ELSE
                            J1=ISTOP-ISTRT+1
          IF((IENDLU+J1).LT.(NAU-1))       THEN
                                  DO 600 J2=1,J1
                                  J3=ISTRT+J2-1
                                  J4=CNR(J3)
                                  CNR(J2+IENDLU)=J4
                                  AU(J2+IENDLU)=AU(J3)
                                  WR01(J4)=J2+IENDLU
                                  CNR(J3)=0
600                               CONTINUE
                                  ISTRT=IENDLU+1
                                  PTR(RI)=ISTRT
                                  ISTOP=IENDLU+J1
                                  GO TO 400
                                           ELSE
              IF(IQCMPR.EQ.1) THEN
                              IFL=11
                              RETURN
              END IF
              IQCMPR=1
              CALL UXSGUG(N,M,NZ,NAU,PTR,LENR,CNR,
     *                    AU,IBEG,IENDLU)
              ISTRT=PTR(RI)
              ISTOP=ISTRT+LENR(RI)-1
              JSTRT=PTR(ORDR(K))
              JSTOP=JSTRT+LENR(RJ)-1
              GO TO 400
          END IF
      END IF
      END IF
1000  CONTINUE
C
C     WR02 UPDATE
C
      WR02(RM)=WR02(IND)
      IND=IND-1
C
C     RETURN TEST
C
      IF(IND.EQ.0) THEN
                   IFL=15
                   RETURN
      END IF
      RM=N+1
      RRM=RM
      DO 700 J=1,IND
      K=WR02(J)
      IF(RRM.GT.K) THEN
                  RM=J
                  RRM=K
      END IF
700   CONTINUE
      GO TO 140
C
C     DROPPING ELEMENTS
C
750   J=ISTRT
      K1=ISTRT
      DRFL=1.0D-25
800   IF(ABS(AU(J)).GT.DRFL) THEN
                             AU(K1)=AU(J)
                             CNR(K1)=CNR(J)
                             K1=K1+1
      END IF
      J=J+1
      IF(J.LE.ISTOP) GOTO 800
      LENR(RI)=K1-ISTRT
C
C     MNU,MXU UPDATE
C
      MNU=MIN(MNU,ABS(AU(INT(WR01(INT(WR02(1)))))))
      MXU=MAX(MXU,ABS(AU(INT(WR01(INT(WR02(1)))))))
C      IF(MNU.LT.EPS) IFL=13
      RETURN
      END
