* SUBROUTINE UXSIBB                 ALL SYSTEMS                95/12/01
C PORTABILITY : ALL SYSTEMS
C 95/12/01 DU : ORIGINAL VERSION
*
* PURPOSE :
* THIS SUBROUTINE USES THE FACTORISATION OF THE MATRIX IN A,IW TO
* SOLVE A SYSTEM OF EQUATIONS.
*
* PARAMETERS :
*  II  N  MUST BE SET TO THE ORDER OF THE MATRIX.  IT IS NOT ALTERED.
*  RI  A(LA)  HOLD INFORMATION ON THE FACTORS AND MUST BE UNCHANGED
*         SINCE THE CALL TO UXSIBF. THEY ARE NOT ALTERED BY UXSIBB.
*  II  LA  MUST BE SET TO THE LENGTHS OF A. IT IS NOT ALTERED.
*  II  IW(MW)  HOLD INFORMATION ON THE FACTORS AND MUST BE UNCHANGED
*         SINCE THE CALL TO UXSIBF. THEY ARE NOT ALTERED BY UXSIBB.
*  II  MW  MUST BE SET TO THE LENGTHS OF IW. IT IS NOT ALTERED.
*  RA  W(MF)  USED AS A WORK ARRAY.
*  RU  RHS(N)  MUST BE SET TO THE RIGHT HAND SIDE FOR THE EQUATIONS
*         BEING SOLVED. ON EXIT, THIS ARRAY WILL HOLD THE SOLUTION.
*  IA  IW1(NS)  USED AS A WORK ARRAY.
*  II  NS  IS THE LENGTH OF IW1 WHICH MUST BE AT LEAST THE ABSOLUTE
*      VALUE OF IW(1). IT IS NOT ALTERED.
*  II  MF  IS THE LENGTH OF W AND MUST BE PASSED UNCHANGED FROM THE
*      CALL TO UXSIBF. IT IS NOT ALTERED.
*
      SUBROUTINE UXSIBB(N, A, LA, IW, MW, W, RHS, IW1, NS, MF)
      INTEGER N, LA, MW, NS, MF
      INTEGER IW1(NS)
      INTEGER IW(MW)
      DOUBLE PRECISION A(LA), W(MF), RHS(N)
      INTEGER I, LATOP, LDIAG, LP, MP, NBLK
      DOUBLE PRECISION U
      INTRINSIC MIN0, IABS
C
C     SEE BLOCK DATA FOR COMMENTS ON VARIABLES IN COMMON
C
      COMMON /UMMAT1/ U, LP, MP, LDIAG
      IF (IW(1).LT.0) GO TO 130
      NBLK = IW(1)
      IF (NBLK.GT.0) GO TO 140
C
C     WE HAVE A ZERO MATRIX
C
      DO 120 I=1,N
        RHS(I) = 0.0D0
  120 CONTINUE
      RETURN
  130 NBLK = -IW(1)
C
C     FORWARD SUBSTITUTION
C
  140 CALL UXSIB1(N, A, LA, IW(2), MW-1, W, MF, RHS, IW1, NBLK,
     & LATOP)
C
C     BACK SUBSTITUTION.
C
      CALL UXSIB2(N, A, LA, IW(2), MW-1, W, MF, RHS, IW1, NBLK,
     & LATOP)
      RETURN
      END
