* SUBROUTINE UXSIF1                 ALL SYSTEMS                95/12/01
C PORTABILITY : ALL SYSTEMS
C 95/12/01 DU : ORIGINAL VERSION
*
* PURPOSE :
* AUXILIARY SUBROUTINE FOR UXSIBF.
* SORT PRIOR TO FACTORIZATION USING UXSIF2.
*
      SUBROUTINE UXSIF1(N, M, M1, A, LA, KA, JA, IW, MW, PERM,
     & IW2, INF)
C
C     THIS SUBROUTINE REORDERS THE USER'S INPUT SO THAT THE UPPER
C     TRIANGLE OF THE PERMUTED MATRIX, INCLUDING THE DIAGONAL, IS
C     HELD ORDERED BY ROWS AT THE END OF THE STORAGE FOR A AND IW.
C     IT IGNORES ENTRIES WITH ONE OR BOTH INDICES OUT OF RANGE AND
C     ACCUMULATES DIAGONAL ENTRIES.
C     IT ADDS EXPLICIT ZEROS ON THE DIAGONAL WHERE NECESSARY.
C
      INTEGER N, M, LA, MW, INF
      INTEGER IW2(N)
      INTEGER KA(*),JA(*),IW(MW),PERM(N)
      DOUBLE PRECISION A(LA), ZERO, ANEXT, ANOW
C
C     N - MUST BE SET TO THE ORDER OF THE MATRIX.
C         IT IS NOT ALTERED BY UXSIF1.
C     M - ON ENTRY M MUST BE SET TO THE NUMBER
C         OF NON-ZEROS INPUT.  NOT ALTERED BY UXSIF1.
C     M1- ON EXIT M1 WILL BE EQUAL TO THE NUMBER OF ENTRIES IN THE
C         SORTED MATRIX.
C     A - ON ENTRY A(I) MUST
C         HOLD THE VALUE OF THE ORIGINAL MATRIX ELEMENT IN POSITION
C         (KA(I),JA(I)),I=1,M.  ON EXIT A(LA-M1+I),I=1,M1 HOLDS
C         THE UPPER TRIANGLE OF THE PERMUTED MATRIX BY ROWS WITH
C         THE DIAGONAL ENTRY FIRST ALTHOUGH THERE IS NO FURTHER
C         ORDERING WITHIN THE ROWS THEMSELVES.
C     LA- LENGTH OF ARRAY A. MUST BE AT LEAST N+MAX0(M,M1).
C         IT IS NOT ALTERED BY UXSIF1.
C     KA- KA(I) MUST BE SET TO
C         HOLD THE ROW INDEX OF ENTRY A(I),I=1,M.  KA WILL BE
C         UNALTERED BY UXSIF1, UNLESS IT IS EQUIVALENCED WITH IW.
C     JA- JA(I) MUST BE SET TO
C         HOLD THE COLUMN INDEX OF ENTRY A(I),I=1,M.  JA WILL BE
C         UNALTERED BY UXSIF1, UNLESS IT IS EQUIVALENCED WITH IW.
C     IW- USED AS WORKSPACE AND ON
C         EXIT, ENTRIES IW(MW-M1+I),I=1,M1 HOLD THE COLUMN INDICES
C         (THE ORIGINAL UNPERMUTED INDICES) OF THE CORRESPONDING ENTRY
C         OF A WITH THE FIRST ENTRY FOR EACH ROW FLAGGED NEGATIVE.
C         KA(1) MAY BE EQUIVALENCED WITH IW(1) AND JA(1) MAY BE
C         EQUIVALENCED WITH IW(K) WHERE K.GT.M.
C     MW- LENGTH OF ARRAY IW. MUST BE AT LEAST AS
C         GREAT AS THE MAXIMUM OF M AND M1.
C         NOT ALTERED BY UXSIF1.
C     PERM- PERM(I) HOLDS THE
C         POSITION IN THE TENTATIVE PIVOT ORDER OF ROW I IN THE
C         ORIGINAL MATRIX,I=1,N. NOT ALTERED BY UXSIF1.
C     IW2 - USED AS WORKSPACE.
C         SEE COMMENTS IN CODE IMMEDIATELY PRIOR TO
C          EACH USE.
C     INF - ON EXIT FROM UXSIF1, A ZERO VALUE OF
C         INF INDICATES THAT NO ERROR HAS BEEN DETECTED.
C         POSSIBLE NON-ZERO VALUES ARE ..
C         +1 WARNING.  INDICES OUT OF RANGE.  THESE ARE IGNORED,
C            THEIR NUMBER IS RECORDED IN IERROR OF UMMAT2 AND
C            MESSAGES IDENTIFYING THE FIRST TEN ARE OUTPUT ON UNIT MP.
C         -3 INTEGER ARRAY IW IS TOO SMALL.
C         -4 REAL ARRAY A IS TOO SMALL.
C
      INTEGER I,IA,ICH, IERROR,  II, IIW, INEW, IOLD, IPOS,
     &        J1, J2, JJ, JNEW, JOLD, JPOS, K,
     &        LDIAG,  LP,
     &        MP,
     &        NCMPA, NCMPBI, NCMPBR,
     &        NEIG, NIRADU,  NIRBDU, NIRNEC, NIRTOT,
     &        NRLADU, NRLBDU,
     &        NRLNEC, NRLTOT, NTWO, M1
      DOUBLE PRECISION U, OPS
      INTRINSIC MIN0
      COMMON /UMMAT1/ U, LP, MP, LDIAG
      COMMON /UMMAT2/ OPS, IERROR, NRLTOT, NIRTOT, NRLNEC, NIRNEC,
     & NRLADU, NIRADU, NRLBDU, NIRBDU, NCMPA, NCMPBR, NCMPBI, NTWO,NEIG
      DATA ZERO /0.0D0/
      INF = 0
C
C     INITIALIZE WORK ARRAY (IW2) IN PREPARATION FOR
C     COUNTING NUMBERS OF NON-ZEROS IN THE ROWS AND INITIALIZE
C     LAST N ENTRIES IN A WHICH WILL HOLD THE DIAGONAL ENTRIES
C
      IA = LA
      DO 10 IOLD=1,N
        IW2(IOLD) = 1
        A(IA) = ZERO
        IA = IA - 1
   10 CONTINUE
C
C     SCAN INPUT COPYING ROW INDICES FROM KA TO THE FIRST M POSITIONS
C     IN IW.  THE NEGATIVE OF THE INDEX IS HELD TO FLAG ENTRIES FOR
C     THE IN-PLACE SORT.  ENTRIES IN IW CORRESPONDING TO DIAGONALS AND
C     ENTRIES WITH OUT-OF-RANGE INDICES ARE SET TO ZERO.
C     FOR DIAGONAL ENTRIES, REALS ARE ACCUMULATED IN THE LAST N
C     LOCATIONS OF A.
C     THE NUMBER OF ENTRIES IN EACH ROW OF THE PERMUTED MATRIX IS
C     ACCUMULATED IN IW2.
C     INDICES OUT OF RANGE ARE IGNORED  AFTER BEING COUNTED AND
C     AFTER APPROPRIATE MESSAGES HAVE BEEN PRINTED.
C
      IERROR = 0
C
C     M1 IS THE NUMBER OF NON-ZEROS HELD AFTER INDICES OUT OF RANGE HAVE
C     BEEN IGNORED AND DIAGONAL ENTRIES ACCUMULATED.
C
      M1 = N
      IF (M.EQ.0) GO TO 80
      DO 70 K=1,M
        IOLD = KA(K)
        IF (IOLD.GT.N .OR. IOLD.LE.0) GO TO 30
        JOLD = JA(K)
        IF (JOLD.GT.N .OR. JOLD.LE.0) GO TO 30
        INEW = PERM(IOLD)
        JNEW = PERM(JOLD)
        IF (INEW.NE.JNEW) GO TO 20
        IA = LA - N + IOLD
        A(IA) = A(IA) + A(K)
        GO TO 60
   20   INEW = MIN0(INEW,JNEW)
C
C     INCREMENT NUMBER OF ENTRIES IN ROW INEW.
C
        IW2(INEW) = IW2(INEW) + 1
        IW(K) = -IOLD
        M1 = M1 + 1
        GO TO 70
C
C     ENTRY OUT OF RANGE.  IT WILL BE IGNORED AND A FLAG SET.
C
   30   INF = 1
        IERROR = IERROR + 1
        IF (IERROR.LE.1) CALL UOERR1('UXSIF1',73)
   60   IW(K) = 0
   70 CONTINUE
C
C     CALCULATE POINTERS (IN IW2) TO THE POSITION IMMEDIATELY AFTER THE
C     END OF EACH ROW.
C
   80 IF (M.LT.M1 .AND. M1.NE.N) GO TO 100
C
C     ROOM IS INCLUDED FOR THE DIAGONALS.
C
      K = 1
      DO 90 I=1,N
        K = K + IW2(I)
        IW2(I) = K
   90 CONTINUE
      GO TO 120
C ROOM IS NOT INCLUDED FOR THE DIAGONALS.
  100 K = 1
      DO 110 I=1,N
        K = K + IW2(I) - 1
        IW2(I) = K
  110 CONTINUE
C
C     FAIL IF INSUFFICIENT SPACE IN ARRAYS A OR IW.
C
  120 IF (M1.GT.MW) GO TO 210
      IF (M1+N.GT.LA) GO TO 220
C
C     NOW RUN THROUGH NON-ZEROS IN ORDER PLACING THEM IN THEIR NEW
C     POSITION AND DECREMENTING APPROPRIATE IW2 ENTRY.  IF WE ARE
C     ABOUT TO OVERWRITE AN ENTRY NOT YET MOVED, WE MUST DEAL WITH
C     THIS AT THIS TIME.
C
      IF (M1.EQ.N) GO TO 180
      DO 140 K=1,M
        IOLD = -IW(K)
        IF (IOLD.LE.0) GO TO 140
        JOLD = JA(K)
        ANOW = A(K)
        IW(K) = 0
        DO 130 ICH=1,M
          INEW = PERM(IOLD)
          JNEW = PERM(JOLD)
          INEW = MIN0(INEW,JNEW)
          IF (INEW.EQ.PERM(JOLD)) JOLD = IOLD
          JPOS = IW2(INEW) - 1
          IOLD = -IW(JPOS)
          ANEXT = A(JPOS)
          A(JPOS) = ANOW
          IW(JPOS) = JOLD
          IW2(INEW) = JPOS
          IF (IOLD.EQ.0) GO TO 140
          ANOW = ANEXT
          JOLD = JA(JPOS)
  130   CONTINUE
  140 CONTINUE
      IF (M.GE.M1) GO TO 180
C
C     MOVE UP ENTRIES TO ALLOW FOR DIAGONALS.
C
      IPOS = M1
      JPOS = M1 - N
      DO 170 II=1,N
        I = N - II + 1
        J1 = IW2(I)
        J2 = JPOS
        IF (J1.GT.JPOS) GO TO 160
        DO 150 JJ=J1,J2
          IW(IPOS) = IW(JPOS)
          A(IPOS) = A(JPOS)
          IPOS = IPOS - 1
          JPOS = JPOS - 1
  150   CONTINUE
  160   IW2(I) = IPOS + 1
        IPOS = IPOS - 1
  170 CONTINUE
C
C     RUN THROUGH ROWS INSERTING DIAGONAL ENTRIES AND FLAGGING BEGINNING
C     OF EACH ROW BY NEGATING FIRST COLUMN INDEX.
C
  180 DO 190 IOLD=1,N
        INEW = PERM(IOLD)
        JPOS = IW2(INEW) - 1
        IA = LA - N + IOLD
        A(JPOS) = A(IA)
        IW(JPOS) = -IOLD
  190 CONTINUE
C
C     MOVE SORTED MATRIX TO THE END OF THE ARRAYS.
C
      IPOS = M1
      IA = LA
      IIW = MW
      DO 200 I=1,M1
        A(IA) = A(IPOS)
        IW(IIW) = IW(IPOS)
        IPOS = IPOS - 1
        IA = IA - 1
        IIW = IIW - 1
  200 CONTINUE
      GO TO 230
C
C     ERROR RETURN
C
  210 INF = -3
      IERROR = M1
      GO TO 230
  220 INF = -4
      IERROR = M1 + N
  230 RETURN
      END
