* SUBROUTINE UXSIS6                 ALL SYSTEMS                95/12/01
C PORTABILITY : ALL SYSTEMS
C 95/12/01 DU : ORIGINAL VERSION
*
* PURPOSE :
* AUXILIARY SUBROUTINE FOR UXSIBS.
* TREE SEARCH
*
      SUBROUTINE UXSIS6(N, IPE, NV, IPS, NE, NA, ND, NS)
C
C     GIVEN SON TO FATHER TREE POINTERS, PERFORM DEPTH-FIRST
C     SEARCH TO FIND PIVOT ORDER AND NUMBER OF ELIMINATIONS
C     AND ASSEMBLIES AT EACH STAGE.
C
      INTEGER N, NS
      INTEGER IPE(N)
      INTEGER NV(N),IPS(N),NE(N),NA(N),ND(N)
C
C     N MUST BE SET TO THE MATRIX ORDER. IT IS NOT ALTERED.
C     IPE(I) MUST BE SET EQUAL TO -(FATHER OF NODE I) OR ZERO IF
C     NODE IS A ROOT. IT IS ALTERED TO POINT TO ITS NEXT
C     YOUNGER BROTHER IF IT HAS ONE, BUT OTHERWISE IS NOT
C     CHANGED.
C     NV(I) MUST BE SET TO ZERO IF NO VARIABLES ARE ELIMINATED AT NODE
C     I AND TO THE DEGREE OTHERWISE. ONLY LEAF NODES CAN HAVE
C     ZERO VALUES OF NV(I). NV IS NOT ALTERED.
C     IPS(I) NEED NOT BE SET. IT IS USED TEMPORARILY TO HOLD
C     -(ELDEST SON OF NODE I) IF IT HAS ONE AND 0 OTHERWISE. IT IS
C     EVENTUALLY SET TO HOLD THE POSITION OF NODE I IN THE ORDER.
C     NE(IS) NEED NOT BE SET. IT IS SET TO THE NUMBER OF VARIABLES
C     ELIMINATED AT STAGE IS OF THE ELIMINATION.
C     NA(IS) NEED NOT BE SET. IT IS SET TO THE NUMBER OF ELEMENTS
C     ASSEMBLED AT STAGE IS OF THE ELIMINATION.
C     ND(IS) NEED NOT BE SET. IT IS SET TO THE DEGREE AT STAGE IS OF
C     THE ELIMINATION.
C     NS NEED NOT BE SET. IT IS SET TO  THE NUMBER OF ELIMINATION
C     STEPS.
      INTEGER I,IB, IF, IFRLVL, IL, IOVFLO, IS, ISON,
     &        K,
     &        L,
     &        NEMIN, NR
      COMMON /UMMAT3/ IOVFLO, NEMIN, IFRLVL(20)
C
C     INITIALIZE IPS AND NE.
C
      DO 10 I=1,N
        IPS(I) = 0
        NE(I) = 0
   10 CONTINUE
C
C     SET IPS(I) TO -(ELDEST SON OF NODE I) AND IPE(I) TO NEXT YOUNGER
C     BROTHER OF NODE I IF IT HAS ONE.
C     FIRST PASS IS FOR NODES WITHOUT ELIMINATIONS.
C
      DO 20 I=1,N
        IF (NV(I).GT.0) GO TO 20
        IF = -IPE(I)
        IS = -IPS(IF)
        IF (IS.GT.0) IPE(I) = IS
        IPS(IF) = -I
   20 CONTINUE
C
C     NR IS DECREMENTED FOR EACH ROOT NODE. THESE ARE STORED IN
C     NE(I),I=NR,N.
C
      NR = N + 1
C
C     SECOND PASS TO ADD NODES WITH ELIMINATIONS.
C
      DO 50 I=1,N
        IF (NV(I).LE.0) GO TO 50
C
C     NODE IF IS THE FATHER OF NODE I.
C
        IF = -IPE(I)
        IF (IF.EQ.0) GO TO 40
        IS = -IPS(IF)
C
C     JUMP IF NODE IF HAS NO SONS YET.
C
        IF (IS.LE.0) GO TO 30
C
C     SET POINTER TO NEXT BROTHER
C
        IPE(I) = IS
C
C     NODE I IS ELDEST SON OF NODE IF.
C
   30   IPS(IF) = -I
        GO TO 50
C
C     WE HAVE A ROOT
C
   40   NR = NR - 1
        NE(NR) = I
   50 CONTINUE
C
C     DEPTH-FIRST SEARCH.
C     IL HOLDS THE CURRENT TREE LEVEL. ROOTS ARE AT LEVEL N, THEIR SONS
C     ARE AT LEVEL N-1, ETC.
C     IS HOLDS THE CURRENT ELIMINATION STAGE. WE ACCUMULATE THE NUMBER
C     OF ELIMINATIONS AT STAGE IS DIRECTLY IN NE(IS). THE NUMBER OF
C     ASSEMBLIES IS ACCUMULATED TEMPORARILY IN NA(IL), FOR TREE
C     LEVEL IL, AND IS TRANSFERED TO NA(IS) WHEN WE REACH THE
C     APPROPRIATE STAGE IS.
C
      IS = 1
C
C     I IS THE CURRENT NODE.
C
      I = 0
      DO 160 K=1,N
        IF (I.GT.0) GO TO 60
C
C     PICK UP NEXT ROOT.
C
        I = NE(NR)
        NE(NR) = 0
        NR = NR + 1
        IL = N
        NA(N) = 0
C
C     GO TO SON FOR AS LONG AS POSSIBLE, CLEARING FATHER-SON POINTERS
C     IN IPS AS EACH IS USED AND SETTING NA(IL)=0 FOR ALL LEVELS
C     REACHED.
C
   60   DO 70 L=1,N
          IF (IPS(I).GE.0) GO TO 80
          ISON = -IPS(I)
          IPS(I) = 0
          I = ISON
          IL = IL - 1
          NA(IL) = 0
   70   CONTINUE
C
C     RECORD POSITION OF NODE I IN THE ORDER.
C
   80   IPS(I) = K
        NE(IS) = NE(IS) + 1
C
C     JUMP IF NODE HAS NO ELIMINATIONS.
C
        IF (NV(I).LE.0) GO TO 120
        IF (IL.LT.N) NA(IL+1) = NA(IL+1) + 1
        NA(IS) = NA(IL)
        ND(IS) = NV(I)
C
C     CHECK FOR STATIC CONDENSATION
C
        IF (NA(IS).NE.1) GO TO 90
        IF (ND(IS-1)-NE(IS-1).EQ.ND(IS)) GO TO 100
C
C     CHECK FOR SMALL NUMBERS OF ELIMINATIONS IN BOTH LAST TWO STEPS.
C
   90   IF (NE(IS).GE.NEMIN) GO TO 110
        IF (NA(IS).EQ.0) GO TO 110
        IF (NE(IS-1).GE.NEMIN) GO TO 110
C
C     COMBINE THE LAST TWO STEPS
C
  100   NA(IS-1) = NA(IS-1) + NA(IS) - 1
        ND(IS-1) = ND(IS) + NE(IS-1)
        NE(IS-1) = NE(IS) + NE(IS-1)
        NE(IS) = 0
        GO TO 120
  110   IS = IS + 1
  120   IB = IPE(I)
        IF (IB.GE.0) THEN
C
C     NODE I HAS A BROTHER OR IS A ROOT
C
        IF (IB.GT.0) NA(IL) = 0
        I = IB
        ELSE
C
C     GO TO FATHER OF NODE I
C
        I = -IB
        IL = IL + 1
        ENDIF
  160 CONTINUE
      NS = IS - 1
      RETURN
      END
