* SUBROUTINE UXSPCN                ALL SYSTEMS               93/12/01
C PORTABILITY : ALL SYSTEMS
C 93/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* ESTIMATION OF THE MINIMUM EIGENVALUE AND THE CORRESPONDING EIGENVECTOR
* OF A SPARSE SYMMETRIC POSITIVE DEFINITE MATRIX A+E USING THE
* FACTORIZATION A+E=L*D*TRANS(L) OBTAINED BY THE SUBROUTINE UXSPCF.
*
* PARAMETERS :
*  II  N ORDER OF THE MATRIX A.
*  II  MMAX  LENGTH OF THE PRINCIPAL MATRIX ARRAYS (SL,A).
*  RI  A(MMAX)  FACTORS L,D OF THE FACTORIZATION A+E=L*D*TRANS(L)
*                STORED USING THE COMPACT SCHEME OF STORING.
*  II  PSL(N+1) POINTER ARRAY OF THE FACTORIZED SPARSE MATRIX
*  II  SL(MMAX)  ARRAY OF COLUMN INDICES OF THE FACTORS L AND D
*         STORED USING THE COMPACT SCHEME.
*         SUBROUTINE UXDPGF.
*  RO  X(N)  ESTIMATED EIGENVECTOR.
*  RO  ALF  ESTIMATED EIGENVALUE.
*  II  JOB  OPTION. IF JOB=0 THEN ONLY ESTIMATED EIGENVALUE IS
*         COMPUTED. IS JOB>0 THEN BOTH ESTIMATED EIGENVALUE AND
*         ESTIMATED EIGENVECTOR ARE COMPUTED.
*
      SUBROUTINE UXSPCN(N,MMAX,A,PSL,SL,X,ALF,JOB)
      INTEGER N,MMAX
      $FLOAT A(MMAX),X(N),ALF
      INTEGER PSL(N+1),SL(MMAX),JOB
      $FLOAT XP,XM,FP,FM,UXVDOT
      INTEGER I,K,IS
      $FLOAT ZERO,ONE
      PARAMETER (ZERO=0.0$P 0,ONE=1.0$P 0)
C
C     COMPUTATION OF THE VECTOR V WITH POSSIBLE MAXIMUM NORM SUCH
C     THAT  L*D**(1/2)*V=U  WHERE U HAS ELEMENTS +1 OR -1
C
      DO 2 I=1,N
      X(I)=ZERO
    2 CONTINUE
      DO 6 K=1,N
      XP=-X(K)+ONE
      XM=-X(K)-ONE
      FP=ABS(XP)
      FM=ABS(XM)
      IS=SL(K)+N+1
      DO 3 I=PSL(K)+K,PSL(K+1)+K-1
      FP=FP+ABS(X(SL(IS))+A(I)*XP)
      FM=FM+ABS(X(SL(IS))+A(I)*XM)
      IS=IS+1
    3 CONTINUE
      IF(FP.GE.FM) THEN
      X(K)=XP
      IS=SL(K)+N+1
      DO 4 I=PSL(K)+K,PSL(K+1)+K-1
      X(SL(IS))=X(SL(IS))+A(I)*XP
      IS=IS+1
    4 CONTINUE
      ELSE
      X(K)=XM
      IS=SL(K)+N+1
      DO 5 I=PSL(K)+K,PSL(K+1)+K-1
      X(SL(IS))=X(SL(IS))+A(I)*XM
      IS=IS+1
    5 CONTINUE
      ENDIF
    6 CONTINUE
C
C     COMPUTATION OF THE VECTOR X SUCH THAT
C     D**(1/2)*TRANS(L)*X=V
C
      FM=ZERO
      DO 7 K=1,N
      IF (JOB.LE.0) THEN
      FP=SQRT(A(PSL(K)+K-1))
      X(K)=X(K)/FP
      FM=FM+X(K)*X(K)
      ELSE
      X(K)=X(K)/A(PSL(K)+K-1)
      ENDIF
    7 CONTINUE
      FP=$DBLE(N)
      IF (JOB.LE.0) THEN
C
C     FIRST ESTIMATION OF THE MINIMUM EIGENVALUE BY THE FORMULA
C     ALF=(TRANS(U)*U)/(TRANS(V)*V)
C
      ALF=FP/FM
      RETURN
      ENDIF
      FM=ZERO
      DO 9 K=N,1,-1
      IS=SL(K)+N+1
      DO 8 I=PSL(K)+K,PSL(K+1)+K-1
      X(K)=X(K)-A(I)*X(SL(IS))
      IS=IS+1
    8 CONTINUE
      FM=FM+X(K)*X(K)
    9 CONTINUE
      FM=SQRT(FM)
      IF (JOB.LE.1) THEN
C
C     SECOND ESTIMATION OF THE MINIMUM EIGENVALUE BY THE FORMULA
C     ALF=SQRT(TRANS(U)*U)/SQRT(TRANS(X)*X)
C
      ALF=SQRT(FP)/FM
      ELSE
C
C     INVERSE ITERATIONS
C
      DO 11 K=2,JOB
C
C     SCALING THE VECTOR X BY ITS NORM
C
      DO 10 I=1,N
      X(I)=X(I)/FM
   10 CONTINUE
      CALL UXSPCB(N,MMAX,A,PSL,SL,X,0)
      FM=SQRT(UXVDOT(N,X,X))
   11 CONTINUE
      ALF=ONE/FM
      ENDIF
C
C     SCALING THE VECTOR X BY ITS NORM
C
      DO 12 I=1,N
      X(I)=X(I)/FM
   12 CONTINUE
      RETURN
      END
