* SUBROUTINE UXSPCZ                ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* SOLUTION OF A SYSTEM OF LINEAR EQUATIONS WITH A SPARSE SYMMETRIC
* POSITIVE DEFINITE MATRIX A+E USING THE FACTORIZATION A+E=L*D*TRANS(L)
* STORED IN THE COMPACT SCHEME. THIS FACTORIZATION CAN BE OBTAINED
* USING THE SUBROUTINE UXSPCF.
*
* PARAMETERS :
*  II  N  ORDER OF THE MATRIX A.
*  II  MMAX  LENGTH OF THE PRINCIPAL MATRIX ARRAYS (SL,A).
*  II  ND  NUMBER OF DENSE ROWS.
*  RI  A(MMAX)  FACTORS L,D OF THE FACTORIZATION A+E=L*D*TRANS(L)
*                STORED USING THE COMPACT SCHEME OF STORING.
*  II  PSL(N+1) POINTER ARRAY OF THE FACTORIZED SPARSE MATRIX
*  II  SL(MMAX)  ARRAY OF COLUMN INDICES OF THE FACTORS L AND D
*         STORED USING THE COMPACT SCHEME.
*  II  PERM(N)  PERMUTATION VECTOR
*  RI  U(N*ND)  DENSE ROWS.
*  RA  D(ND)  AUXILIARY ARRAY.
*  RO  B(ND*(ND+1)/2))  DECOMPOSITION OF THE UPDATED MATRIX.
*  RA  X(N)  AUXILIARY ARRAY.
*  RA  Y(N)  VECTOR MULTIPLIED BY THE INVERSE OF THE UPDATED MATRIX.
*  RA  Z(N)  AUXILIARY ARRAY.
*
* METHOD :
* BACK SUBSTITUTION
*
      SUBROUTINE UXSPCZ(N,MMAX,ND,A,PSL,SL,PERM,U,D,B,X,Y,Z)
      INTEGER N,MMAX,ND
      $FLOAT A(MMAX),U(N*ND),D(ND),B(ND*(ND+1)/2),X(N),Y(N),Z(N)
      INTEGER PSL(N+1),SL(MMAX),PERM(N)
      INTEGER I
      $FLOAT UXVDOT,ZERO
      PARAMETER (ZERO=0.0$P 0)
      CALL UXVCOP(N,Y,Z)
      CALL UXVSFP(N,PERM,Y,X)
      CALL UXSPCB(N,MMAX,A,PSL,SL,Y,0)
      CALL UXVSBP(N,PERM,Y,X)
      IF (ND.LE.0) RETURN
      DO 1 I=1,ND
      D(I)=UXVDOT(N,U((I-1)*N+1),Y)
    1 CONTINUE
      CALL UXDPGB(ND,B,D,0)
      CALL UXVCOP(N,Z,Y)
      DO 2 I=1,ND
      CALL UXVDIR(N,-D(I),U((I-1)*N+1),Y,Y)
    2 CONTINUE
      CALL UXVSFP(N,PERM,Y,X)
      CALL UXSPCB(N,MMAX,A,PSL,SL,Y,0)
      CALL UXVSBP(N,PERM,Y,X)
      RETURN
      END
