* SUBROUTINE UXSPGF                ALL SYSTEMS                94/12/01
C PORTABILITY : ALL SYSTEMS
C 94/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* INCOMPLETE FACTORIZATION A+E=L*D*TRANS(L) OF A SPARSE SYMMETRIC
* POSITIVE DEFINITE MATRIX A+E WHERE D AND E ARE DIAGONAL POSITIVE
* DEFINITE MATRICES AND L IS A LOWER TRIANGULAR MATRIX.
*
* PARAMETERS :
*  II  N ORDER OF THE MATRIX A.
*  II  M NUMBER OF NONZERO ELEMENTS OF THE MATRIX A.
*  RU  A(M)  ON INPUT A GIVEN SPARSE SYMMETRIC (USUALLY POSITIVE
*         DEFINITE) MATRIX. ON OUTPUT THE COMPUTED INCOMPLETE
*         FACTORIZATION A+E=L*D*TRANS(L).
*  II  IA(N+1)  POINTERS OF THE DIAGONAL ELEMENTS OF A.
*  II  JA(M)  INDICES OF THE NONZERO ELEMENTS OF A.
*  IO  INF  AN INFORMATION OBTAINED IN THE FACTORIZATION PROCESS. IF
*         INF=0 THEN A IS SUFFICIENTLY POSITIVE DEFINITE AND E=0. IF
*         INF<0 THEN A IS NOT SUFFICIENTLY POSITIVE DEFINITE AND E>0. IF
*         INF>0 THEN A IS INDEFINITE AND INF IS AN INDEX OF THE
*         MOST NEGATIVE DIAGONAL ELEMENT USED IN THE FACTORIZATION
*         PROCESS.
*  RU  ALF  ON INPUT A DESIRED TOLERANCE FOR POSITIVE DEFINITENESS. ON
*         OUTPUT THE MOST NEGATIVE DIAGONAL ELEMENT USED IN THE
*         FACTORIZATION PROCESS (IF INF>0).
*  RO  TAU  MAXIMUM DIAGONAL ELEMENT OF THE MATRIX E.
*
* METHOD :
* P.E.GILL, W.MURRAY : NEWTON TYPE METHODS FOR UNCONSTRAINED AND
* LINEARLY CONSTRAINED OPTIMIZATION, MATH. PROGRAMMING 28 (1974)
* PP. 311-350.
*
      SUBROUTINE UXSPGF(N,M,A,IA,JA,INF,ALF,TAU,X)
      INTEGER N,M,IA(N+1),JA(M),INF
      $FLOAT A(M),ALF,TAU,X(N)
      $FLOAT BET,GAM,DEL,RHO,SIG,TOL,POM
      INTEGER I,J,K,L,IJ,KJ,KK,KN
      $FLOAT ZERO
      PARAMETER (ZERO=0.0$P 0)
      L=0
      INF=0
      TOL=ALF
C
C     ESTIMATION OF THE MATRIX NORM
C
      ALF=ZERO
      BET=ZERO
      GAM=ZERO
      TAU=ZERO
      CALL UXVSET(N,ZERO,X)
      DO 1 K=1,N
      BET=MAX(BET,ABS(A(IA(K))))
    1 CONTINUE
      DO 2 K=1,IA(N+1)-1
      GAM=MAX(GAM,ABS(A(K)))
    2 CONTINUE
      BET=MAX(TOL,BET,GAM/$DBLE(N))
      DEL=TOL*BET
      DO 7 K=1,N
      KK=IA(K)
      KN=IA(K+1)-1
C
C     DETERMINATION OF A DIAGONAL CORRECTION
C
      SIG=A(KK)
      IF (ALF.GT.SIG) THEN
      ALF=SIG
      L=K
      ENDIF
      GAM=ZERO
      DO 3 KJ=KK+1,KN
      J=JA(KJ)
      X(J)=A(KJ)
      GAM=MAX(GAM,ABS(A(KJ)))
    3 CONTINUE
      GAM=GAM*GAM
      RHO=MAX(ABS(SIG),GAM/BET,DEL)
      IF (TAU.LT.RHO-SIG) THEN
      TAU=RHO-SIG
      INF=-1
      ENDIF
C
C     GAUSSIAN ELIMINATION
C
      A(KK)=RHO
      DO 5 I=K+1,N
      IF (X(I).EQ.ZERO) GO TO 5
      POM=X(I)/RHO
      DO 4 IJ=IA(I),IA(I+1)-1
      J=JA(IJ)
      IF (X(J).EQ.ZERO) GO TO 4
      A(IJ)=A(IJ)-POM*X(J)
    4 CONTINUE
      X(I)=POM
    5 CONTINUE
      DO 6 KJ=KK+1,KN
      J=JA(KJ)
      A(KJ)=X(J)
      X(J)=ZERO
    6 CONTINUE
    7 CONTINUE
      IF (L.GT.0.AND.ABS(ALF).GT.DEL) INF = L
      RETURN
      END
