* SUBROUTINE UXSPRB                ALL SYSTEMS                93/12/01
C PORTABILITY : ALL SYSTEMS
C 93/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* SOLUTION OF A SYSTEM OF LINEAR EQUATIONS WITH A SPARSE SYMMETRIC
* POSITIVE DEFINITE MATRIX A USING THE FACTORIZATION A=TRANS(R)*R.
*
* PARAMETERS :
*  II  N ORDER OF THE MATRIX A.
*  RI  A(M) ELEMENTS OF THE SPARSE SYMMETRIC MATRIX STORED IN THE
*      PACKED FORM.
*  II  IA(N+1)  POINTERS OF THE FIRST COLUMN ELEMENTS OF A.
*  II  JA(M)  INDICES OF THE NONZERO ELEMENTS OF A.
*  RU  X(N)  ON INPUT THE RIGHT HAND SIDE OF A SYSTEM OF LINEAR
*         EQUATIONS. ON OUTPUT THE SOLUTION OF A SYSTEM OF LINEAR
*         EQUATIONS.
*  II  JOB  OPTION. IF JOB=0 THEN X:=A**(-1)*X. IF JOB>0 THEN
*         X:=TRANS(R)**(-1)*X. IF JOB<0 THEN X:=R**(-1)*X.
*
* METHOD :
* BACK SUBSTITUTION
*
      SUBROUTINE UXSPRB(N,M,A,IA,JA,X,JOB)
      INTEGER N,M,IA(N+1),JA(M),JOB
      $FLOAT A(M),X(N)
      INTEGER I,J,K,L,IP,JP
      IF (JOB.GE.0) THEN
C
C     PHASE 1 : X:=TRANS(R)**(-1)*X
C
      IP=MAX(JOB,1)
      K=IA(IP)
      DO 3 I=IP,N
      L=IA(I+1)-1
      DO 2 J=K,L-1
      JP=JA(J)
      X(I)=X(I)-A(J)*X(JP)
    2 CONTINUE
      X(I)=X(I)/A(L)
      K=L+1
    3 CONTINUE
      ENDIF
      IF (JOB.LE.0) THEN
C
C     PHASE 2 : X:=R**(-1)*X
C
      L=IA(N+1)-1
      DO 6 I=N,1,-1
      X(I)=X(I)/A(L)
      K=IA(I)
      DO 5 J=K,L-1
      JP=JA(J)
      X(JP)=X(JP)-A(J)*X(I)
    5 CONTINUE
      L=K-1
    6 CONTINUE
      ENDIF
      RETURN
      END
