* SUBROUTINE UXSPTS                ALL SYSTEMS                03/12/01
C PORTABILITY : ALL SYSTEMS
C 03/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* INCOMPLETE CHOLESKY FACTORIZATION A+E=L*D*TRANS(L) OF A SPARSE
* SYMMETRIC POSITIVE DEFINITE MATRIX A+E WHERE D AND E ARE DIAGONAL
* POSITIVE DEFINITE MATRICES AND L IS A LOWER TRIANGULAR MATRIX.
* ELEMENTS LARGER THAN A GIVEN DROP TOLERANCE ARE REMOVED
* FROM THE FACTOR. SPARSE LEFT-LOOKING IMPLEMENTATION.

* PARAMETERS :
*  II  N ORDER OF THE MATRIX A.
*  II  NJA NUMBER OF NONZERO ELEMENTS OF THE MATRIX A.
*  II  MMAX DIMENSION OF THE ARRAY AP.
*  II  IA(N+1)  POINTERS OF THE DIAGONAL ELEMENTS OF A.
*  II  JA(M)  INDICES OF THE NONZERO ELEMENTS OF A.
*  IA  LIST(N+1)  AUXILIARY ARRAY.
*  IA  STRT(N+1)  AUXILIARY ARRAY.
*  IA  WN01(N+1)  AUXILIARY ARRAY.
*  IA  WN02(N+1)  AUXILIARY ARRAY.
*  IO  IFIL fILL-IN INDICATOR. IFIL=0-NO FILL-IN. IFIL=1-NEW NONZERO
*         ELEMENT ARISEN.
*
* METHOD :
* P.E.GILL, W.MURRAY : NEWTON TYPE METHODS FOR UNCONSTRAINED AND
* LINEARLY CONSTRAINED OPTIMIZATION, MATH. PROGRAMMING 28 (1974)
* PP. 311-350.
*
      SUBROUTINE UXSPTS(N,NJA,MMAX,IA,JA,LIST,START,WN01,WN02,IFIL)
      INTEGER N,NJA,MMAX,IA(N+1),JA(MMAX),LIST(N+1), START(N+1),WN01(N+1),
     & WN02(N+1),IFIL
      INTEGER JNDIAG,INDOLD,COLNEX,COLCUR,NXTROW
      INTEGER J,K,II,IND,IND2,ISTRT,ISTOP
      IFIL=0
      NJA=IA(N+1)-1
C
C     INITIALIZE AUXILIARY VECTOR
C
      CALL UXVINS(N,0,WN01)
C
C     INTIALIZE WORKING VECTORS
C
      CALL UXVINS(N,0,LIST)
C
C     LOOP FOR COLUMNS
C
      DO 400 J=1,N
        IND2=0
C
C     ADD CONTRIBUTIONS FROM THE ORIGINAL J-TH COLUMN
C     COMPUTE AVERAGE SIZE OF AN ORIGINAL NONZERO
C
        DO 500 II=IA(J)+1,IA(J+1)-1
          K=JA(II)
          IF(WN01(K).EQ.0) THEN
            IND2=IND2+1
            WN02(IND2)=K
            WN01(K)=IND2
          END IF
 500    CONTINUE
        COLNEX=LIST(J)
 200    COLCUR=COLNEX
        IF(COLCUR.NE.0) THEN
          COLNEX=LIST(COLCUR)
          JNDIAG=START(COLCUR)
          ISTRT=JNDIAG+1
          ISTOP=IA(COLCUR+1)-1
C
C     UPDATE LIST FOR THE ROW NXTROW
C
          IF(ISTRT.LE.ISTOP) THEN
            START(COLCUR)=ISTRT
            NXTROW=JA(ISTRT)
            LIST(COLCUR)=LIST(NXTROW)
            LIST(NXTROW)=COLCUR
          END IF
C
C     GATHER UPDATES FROM PREVIOUS COLUMNS
C     MUST BE TESTED WHETHER USING A VECTOR IN NODAL VERSION
C     IS NECESSARY FOR MORE EFFICIENT IMPLEMENTATION SINCE
C     CACHE CAN ADOPT IN THE SAME WAY A PART OF VECTOR A.
C
          DO 300 II=ISTRT,ISTOP
            K=JA(II)
            IF(WN01(K).EQ.0) THEN
              IFIL=1
              RETURN
            END IF
 300      CONTINUE
C
C     BACK TO NEXT COLUMN HAVING A NONZERO IN A ROW I
C
          GO TO 200
        END IF
C
C     SORT CONTRIBUTIONS IN WN02
C
        CALL UXVSRT(IND2,WN02)
C
C     DEFINE NEXT UNUSED POSITION
C
        IND=IA(J)+1
C
C     A QUICKER BUT SOMETIMES SLIGHTLY GENEROUS SPACE TEST
C
C
C     DROP ELEMENTS SMALLER THAN A GIVEN DROP TOLERANCE
C
        INDOLD=IND
        DO 510 II=1,IND2
          WN01(WN02(II))=0
          IND=IND+1
 510    CONTINUE
C        TADY VEN
C
C     DEFINE START,TOP,IP(J+1)
C
        IF(IND.NE.IA(J)+1) THEN
          START(J)=IA(J)+1
          COLNEX=JA(IA(J)+1)
          LIST(J)=LIST(COLNEX)
          LIST(COLNEX)=J
        END IF
  400 CONTINUE
      RETURN
      END

