* SUBROUTINE UXSSMT                ALL SYSTEMS                06/12/01
C PORTABILITY : ALL SYSTEMS
C 06/12/01 MA : ORIGINAL VERSION
*
* PURPOSE :
* COMPUTATION OF THE LANCZOS ORTHONORMAL VECTORS
*
* PARAMETERS :
*  II  N  ORDER OF THE SPARSE SYMMETRIC MATRIX A.
*  II  M  NUMBER OF NONZERO ELEMENTS OF THE MATRIX A.
*  RI  A(M) ELEMENTS OF THE SPARSE SYMMETRIC MATRIX STORED IN THE
*      PACKED FORM.
*  II  IA(N+1) POINTERS OF THE HESSIAN MATRIX STORED IN A COMPACT FORM.
*  II  JA(M) COLUMN INDICES OF NONZEROES IN THE MATRIX A.
*  RI  U(N) STARTING VECTOR
*  RO  V(N*L) COMPUTING LANCZOS VECTORS STORED COLUMNWISE
*      (THERE IS ONE AUXILIARY COLUMN MORE).
*  RO  D(N)  DIAGONAL ELEMENTS OF THE TRIDIAGONAL MATRIX T.
*  RO  E(N)  SUBDIAGONAL ELEMENTS OF THE TRIDIAGONAL MATRIX T
*      STORED IN POSITIONS 2..N (E(1) IS SET TO ONE)
*  RI  LI  NUMBER OF REQUESTED LANCZOS VECTORS.
*  RO  LO  NUMBER OF COMPUTED LANCZOS VECTORS.
*
* SUBPROGRAMS USED :
*  S   UXVSCL  SCALING OF A VECTOR.
*  S   UXVDIR  VECTOR AUGMENTED BY THE SCALED VECTOR.
*  S   UXSSMM  MULTIPLICATION OF A MATRIX BY A VECTOR.
*  RF  UXVNOR  EUCLIDEAN NORM OF A VECTOR.
*  RF  UXVDOT  DOT PRODUCT OF TWO VECTORS.
*
      SUBROUTINE UXSSMT(N,M,A,IA,JA,U,V,D,E,LI,LO)
      INTEGER N,M,LI,LO,IA(N+1),JA(M),I
      $FLOAT A(M),U(N),V(N*(LI+1)),D(LI),E(LI),
     & TEMPR,ONE,EPS,UXVNOR,UXVDOT
      PARAMETER (ONE=1.0$P 0, EPS=1.0$P-6)
      TEMPR=UXVNOR(N,U)
      CALL UXVSCL(N,ONE/TEMPR,U,V)
      E(1)=ONE
      DO I=1,LI-1
      CALL UXSSMM(N,M,A,IA,JA,V((I-1)*N+1),V(I*N+1))
      D(I)=UXVDOT(N,V(I*N+1),V((I-1)*N+1))
      CALL UXVDIR(N,-D(I),V((I-1)*N+1),V(I*N+1),V(I*N+1))
      IF (I.NE.1) CALL UXVDIR(N,-E(I),V((I-2)*N+1),V(I*N+1),V(I*N+1))
      E(I+1)=UXVNOR(N,V(I*N+1))
      IF (E(I+1).LE.EPS) GO TO 1
      CALL UXVSCL(N,ONE/E(I+1),V(I*N+1),V(I*N+1))
      ENDDO
      CALL UXSSMM(N,M,A,IA,JA,V((I-1)*N+1),V(I*N+1))
      D(I)=UXVDOT(N,V(I*N+1),V((I-1)*N+1))
    1 LO=I
      RETURN
      END
