* SUBROUTINE UXVCON               ALL SYSTEMS                16/12/01
C PORTABILITY : ALL SYSTEMS
C 16/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* ONE STEP OF INCREMENTAL CONDITION ESTIMATION.
*
* PARAMETERS :
*  II  NP  DIMENSION OF VECTORS X AND W.
*  RI  X(NP)  INPUT VECTOR.
*  RI  W(NP)  INPUT VECTOR.
*  RI  GAM  INPUT PARAMETER
*  RI  SEST  INITIAL ESTIMATED SINGULAR VALUE.
*  RO  SESTPR  NEW ESTIMATED SINGULAR VALUE.
*  RO  S  SINE NEEDED FOR INCREMENTAL CONDITION ESTIMATION.
*  RO  C  COSINE NEEDED FOR INCREMENTAL CONDITION ESTIMATION.
*  II  JOB  OPTION. JOB=1 - AN ESTIMATE FOR THE LARGEST SINGULAR VALUE
*         IS COMPUTED. JOB=2 - AN ESTIMATE FOR THE SMALLEST SINGULAR
*         VALUE IS COMPUTED.     
*
* METHOD :
* P.A.BUSSINGER, G.H.GOLUB : LINEAR LEAST SQUARES SOLUTION BY
* HOUSEHOLDER TRANSFORMATION. NUMER. MATH. 7 (1965) 269-276.
*
      SUBROUTINE UXVCON(NP,X,W,GAM,SEST,SESTPR,S,C,JOB)
      INTEGER NP,JOB
      $FLOAT GAM,SEST,SESTPR,S,C
      $FLOAT X(NP),W(NP)
      $FLOAT ABSALF,ABSEST,ABSGAM,ALF,B,CS,EPS,POM,S1,S2,SN,
     & T,TEST,TMP,ZET1,ZET2,UXVDOT
      $FLOAT ZERO,ONE,TWO,HALF,FOUR
      PARAMETER (ZERO=0.0$P 0,ONE=1.0$P 0,TWO=2.0$P 0,HALF=0.5$P 0, 
     & FOUR=4.0$P 0)
      EPS=1.0$P-16
      ALF=UXVDOT(NP,X,W)
      ABSALF=ABS(ALF)
      ABSGAM=ABS(GAM)
      ABSEST=ABS(SEST)
      IF(JOB.EQ.1) THEN
*
*     ESTIMATING LARGEST SINGULAR VALUE
*
*     SPECIAL CASES
*
        IF(SEST.EQ.ZERO) THEN
          S1=MAX(ABSGAM, ABSALF)
          IF(S1.EQ.ZERO) THEN
            S=ZERO
            C=ONE
            SESTPR=ZERO
          ELSE
            S=ALF/S1
            C=GAM/S1
            TMP=SQRT(S*S+C*C)
            S=S/TMP
            C=C/TMP
            SESTPR=S1*TMP
          END IF
          RETURN
        ELSE IF(ABSGAM.LE.EPS*ABSEST) THEN
          S=ONE
          C=ZERO
          TMP=MAX(ABSEST,ABSALF)
          S1=ABSEST/TMP
          S2=ABSALF/TMP
          SESTPR=TMP*SQRT(S1*S1+S2*S2)
          RETURN
        ELSE IF(ABSALF.LE.EPS*ABSEST) THEN
          S1=ABSGAM
          S2=ABSEST
          IF(S1.LE.S2) THEN
            S=ONE
            C=ZERO
            SESTPR=S2
          ELSE
            S=ZERO
            C=ONE
            SESTPR=S1
          END IF
          RETURN
        ELSE IF(ABSEST.LE.EPS*ABSALF.OR.ABSEST.LE.EPS*ABSGAM) THEN
          S1=ABSGAM
          S2=ABSALF
          IF(S1.LE.S2) THEN
            TMP=S1/S2
            S=SQRT(ONE+TMP*TMP)
            SESTPR=S2*S
            C=(GAM/S2)/S
            S=SIGN(ONE,ALF)/S
          ELSE
            TMP=S2/S1
            C=SQRT(ONE+TMP*TMP)
            SESTPR=S1*C
            S=(ALF/S1)/C
            C=SIGN(ONE,GAM)/C
          END IF
          RETURN
        ELSE
*
*        NORMAL CASE
*
          ZET1=ALF/ABSEST
          ZET2=GAM/ABSEST
          B=(ONE-ZET1*ZET1-ZET2*ZET2)*HALF
          C=ZET1*ZET1
          IF(B.GT.ZERO) THEN
            T=C/(B+SQRT(B*B+C))
          ELSE
            T=SQRT(B*B+C)-B
          END IF
          SN=-ZET1/T
          CS=-ZET2/(ONE+T)
          TMP=SQRT(SN*SN+CS*CS)
          S=SN/TMP
          C=CS/TMP
          SESTPR=SQRT(T+ONE)*ABSEST
          RETURN
        END IF
      ELSE IF(JOB.EQ.2) THEN
*
*     ESTIMATING SMALLEST SINGULAR VALUE
*
*     SPECIAL CASES
*
        IF(SEST.EQ.ZERO) THEN
          SESTPR=ZERO
          IF(MAX(ABSGAM,ABSALF).EQ.ZERO) THEN
             SN=ONE
             CS=ZERO
          ELSE
             SN=-GAM
             CS=ALF
          END IF
          S1=MAX(ABS(SN),ABS(CS))
          S=SN/S1
          C=CS/S1
          TMP=SQRT(S*S+C*C)
          S=S/TMP
          C=C/TMP
          RETURN
        ELSE IF(ABSGAM.LE.EPS*ABSEST) THEN
          S=ZERO
          C=ONE
          SESTPR=ABSGAM
          RETURN
        ELSE IF(ABSALF.LE.EPS*ABSEST) THEN
          S1=ABSGAM
          S2=ABSEST
          IF(S1.LE.S2) THEN
            S=ZERO
            C=ONE
            SESTPR=S1
          ELSE
            S=ONE
            C=ZERO
            SESTPR=S2
          END IF
          RETURN
        ELSE IF(ABSEST.LE.EPS*ABSALF.OR.ABSEST.LE.EPS*ABSGAM) THEN
          S1=ABSGAM
          S2=ABSALF
          IF(S1.LE.S2) THEN
            TMP=S1/S2
            C=SQRT(ONE+TMP*TMP)
            SESTPR=ABSEST*(TMP/C)
            S=-(GAM/S2)/C
            C=SIGN(ONE,ALF)/C
          ELSE
            TMP=S2/S1
            S=SQRT(ONE+TMP*TMP)
            SESTPR=ABSEST/S
            C=(ALF/S1)/S
            S=-SIGN(ONE,GAM)/S
          END IF
          RETURN
        ELSE
*
*        NORMAL CASE
*
          ZET1=ALF/ABSEST
          ZET2=GAM/ABSEST
          POM=MAX(ONE+ZET1*ZET1+ABS(ZET1*ZET2),ABS(ZET1*ZET2)+
     &    ZET2*ZET2)
*
*     SEE IF ROOT IS CLOSER TO ZERO OR TO ONE
*
          TEST=ONE+TWO*(ZET1-ZET2)*(ZET1+ZET2)
          IF(TEST.GE.ZERO) THEN
*
*     ROOT IS CLOSE TO ZERO: DIRECT COMPUTATION
*
            B=(ZET1*ZET1+ZET2*ZET2+ONE)*HALF
            C=ZET2*ZET2
            T=C/(B+SQRT(ABS(B*B-C)))
            SN=ZET1/(ONE-T)
            CS=-ZET2/T
            SESTPR=SQRT(T+FOUR*EPS*EPS*POM)*ABSEST
          ELSE
*
*     ROOT IS CLOSE TO ONE: USE A SHIFT
*
            B=(ZET2*ZET2+ZET1*ZET1-ONE)*HALF
            C=ZET1*ZET1
            IF(B.GE.ZERO) THEN
              T=-C/(B+SQRT(B*B+C))
            ELSE
              T=B-SQRT(B*B+C)
            END IF
            SN=-ZET1/T
            CS=-ZET2/(ONE+T)
            SESTPR=SQRT(ONE+T+FOUR*EPS*EPS*POM)*ABSEST
          END IF
          TMP=SQRT(SN*SN+CS*CS)
          S=SN/TMP
          C=CS/TMP
          RETURN
        END IF
      END IF
      RETURN
      END
