$INTERFACE UYABU1                                             04/12/01

  CALL UYABU1(NF,MMAX-MM,$H(MM+1),$JH(MM+1),$PSL,$PERM,$G,$GO,$GP&
,$S,$GS,$XS,ALFN,ALFV,ETA3,JC)
  $ADD(INTEGER,'\$JH($MMAX)\$PSL($NF+1)\$PERM($NF)')
  $ADD(REAL,'\$H($MMAX)\$AM($MAM)\$(AM)O($MAM)\$G($NF)\$GO($NF)&
  \$GP($NF)\$S($NF)\$GS($NF)\$XS($NF)')
  $ADD(REAL,'\ALFN\ALFV')
$REM  $ADD(INTEGER,'\NF\MA\MMAX')
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UYABU1                ALL SYSTEMS                04/12/01
C PORTABILITY : ALL SYSTEMS
C 04/12/01 VL : ORIGINAL VERSION
*
* PURPOSE :
*  SUBGRADIENT AGGREGATION FOR NONSMOOTH VARIABLE METRIC METHOD.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  M  NUMBER OF NONZERO ELEMENTS OF THE SPARSE HESSIAN MATRIX.
*  RI  H(M)  POSITIVE DEFINITE APPROXIMATION OF THE SPARSE HESSIAN
*         MATRIX.
*  IO  JH(M)  INDICES OF THE NONZERO ELEMENTS OF H.
*  II  PSL(NF+1) POINTER ARRAY OF THE FACTORIZED SPARSE MATRIX
*  II  PERM(NF)  PERMUTATION VECTOR
*  RI  G(NF)  NEW SUBGRADIENT OF THE OBJECTIVE FUNCTION.
*  RI  GO(NF)  OLD SUBGRADIENT OF THE OBJECTIVE FUNCTION.
*  RU  GV(NF)  AGGREGATED SUBGRADIENT OF THE OBJECTIVE FUNCTION.
*  RI  S(NF)  DIRECTION VECTOR.
*  RA  U(NF)  AUXILIARY VECTOR.
*  RA  V(NF)  AUXILIARY VECTOR.
*  RO  ALF  LINEARIZATION TERM.
*  RU  ALFV  AGGREGATED LINEARIZATION TERM.
*  RI  RHO  CORRECTION PARAMETER.
*  II  JC  CORRECTION INDICATOR.
*
      SUBROUTINE UYABU1(NF,M,H,JH,PSL,PERM,G,GO,GV,S,U,V,ALF,
     & ALFV,RHO,JC)
      INTEGER NF,M,JH(M),PSL(NF+1),PERM(NF),JC
      $FLOAT H(M),G(NF),GO(NF),GV(NF),S(NF),U(NF),V(NF),ALF,ALFV,RHO
      $FLOAT A,B,ALFM,LAM1,LAM2,PQ,PR,PRQR,QQP,QR,RR,RRP,RRQ,W,W1
      INTEGER I
      $FLOAT ZERO,ONE,UXVDOT
      PARAMETER (ZERO=0.0$P 0,ONE=1.0$P 0)
      ALFM=ZERO
C
C     General routine - here always input parameter ALFM=0
C
      RR=ALFV+ALFV
      RRP=ALFV-ALFM
      RRQ=ALFV-ALF
      DO 1 I=1,NF
        A=S(I)
        U(I)=GO(I)-GV(I)
        S(I)=G(I)-GV(I)
        RR=RR-A*GV(I)
        RRP=RRP+A*U(I)
        RRQ=RRQ+A*S(I)
    1 CONTINUE
      PQ=ZERO
      PR=ZERO
      QR=ZERO
      PRQR=ZERO
      QQP=ZERO
      IF (JC.GE.1) THEN
      DO 2 I=1,NF
      PQ=PQ+RHO*(S(I)-U(I))**2
      PR=PR+RHO*U(I)**2
      QR=QR+RHO*S(I)**2
      PRQR=PRQR+RHO*U(I)*S(I)
      QQP=QQP+RHO+G(I)*(S(I)-U(I))
    2 CONTINUE
      ENDIF
      QQP=QQP+ALF-ALFM
      CALL UXVSFP(NF,PERM,U,V)
      CALL UXSPCB(NF,M,H,PSL,JH,U,1)
      CALL UXVSFP(NF,PERM,S,V)
      CALL UXSPCB(NF,M,H,PSL,JH,S,1)
      DO 4 I=1,NF
        W1=ONE/H(PSL(I)+I-1)
        PQ=PQ+W1*(S(I)-U(I))**2
        PR=PR+W1*U(I)**2
        QR=QR+W1*S(I)**2
        PRQR=PRQR+W1*U(I)*S(I)
        S(I)=W1*(S(I)-U(I))
    4 CONTINUE
      CALL UXSPCB(NF,M,H,PSL,JH,S,-1)
      CALL UXVSBP(NF,PERM,S,V)
      QQP=QQP+UXVDOT(NF,G,S)
      IF (PR.LE.ZERO.OR.QR.LE.ZERO) GO TO 10
      A=RRQ/QR
      B=PRQR/QR
      W=PRQR*B-PR
      IF (W.EQ.ZERO) GO TO 10
      LAM1=(A*PRQR-RRP)/W
      LAM2=A-LAM1*B
      IF (LAM1*(LAM1-ONE).LT.ZERO.AND.LAM2*(LAM1+LAM2-ONE).LT.ZERO)
     & GO TO 40
C
C     MINIMUM ON THE BOUNDARY
C
   10 LAM1=ZERO
      LAM2=ZERO
      IF (ALF.LE.ALFV) LAM2=ONE
      IF (QR.GT.ZERO) LAM2=MIN(ONE,MAX(ZERO,RRQ/QR))
      W=(LAM2*QR-RRQ-RRQ)*LAM2
      A=ZERO
      IF (ALFM.LE.ALFV) A=ONE
      IF (PR.GT.ZERO) A=MIN(ONE,MAX(ZERO,RRP/PR))
      B=(A*PR-RRP-RRP)*A
      IF(B.LT.W)THEN
        W=B
        LAM1=A
        LAM2=ZERO
      ENDIF
      IF (QQP*(QQP-PQ).GE.ZERO) GO TO 40
      IF (QR-RRQ-RRQ-QQP*QQP/PQ.GE.W) GO TO 40
      LAM1=QQP/PQ
      LAM2=ONE-LAM1
   40 IF (LAM1.EQ.ZERO.AND.LAM2*(LAM2-ONE).LT.ZERO.AND.RRP-LAM2*PRQR
     & .GT.ZERO.AND.PR.GT.ZERO) LAM1=MIN(ONE-LAM2,(RRP-LAM2*PRQR)/PR)
      A=ONE-LAM1-LAM2
      ALFV=LAM1*ALFM+LAM2*ALF+A*ALFV
      DO 5 I=1,NF
        GV(I)=LAM1*GO(I)+LAM2*G(I)+A*GV(I)
    5 CONTINUE
      RETURN
      END
