$INTERFACE UYADB0                                             98/12/01

  $HELP
    SPECIFICATION OF THE CONSTRAINT ADDITION
      1 - ONLY ONE ACTIVE CONSTRAINT IS ADDED TO THE ACTIVE SET
      2 - ALL ACTIVE CONSTRAINTS ARE ADDED TO THE ACTIVE SET
  $CHECK(MLC1,2,'1\2',4)
  $HELP
    PRECISION FOR LINEAR INDEPENDENCE OF CONSTRAINTS.
  $CHECK(EPS7,'1.0$P-8',REAL,4)
  $HELP
    TOLERANCE FOR ACTIVE CONSTRAINTS
  $CHECK(EPS9,'1.0$P-8',REAL,4)
  CALL UYADB0(NF,N,MCL,$X,$IX,$XN,$XL,$XU,$CF,$CFD,$IC,$ICA,$CL,$CU&
,$CG,$CR,$CZ,$S,R,$EPS7,$EPS9,GMAX,UMAX,$MLC1,INEW,IER)
  $ADD(REAL,'\$X($NF)\$XN($NF)\$XL($NF)\$XU($NF)\$CF($NC)\$CFD($NC)&
  \$CL($NC)\$CU($NC)\$CG($NF*$MCL)\$CR($NF*($NF+1)/2)\$CZ($NF*$NF)&
  \$S($NF)')
  $ADD(INTEGER,'\$IX($NX)\$IC($NC)\$ICA($NF)')
$REM  $ADD(INTEGER,'\$NF\N\MCL\INEW\IER')
$REM  $ADD(REAL,'\R\GMAX\UMAX\')
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UYADB0             ALL SYSTEMS                   98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* NEW LINEAR CONSTRAINTS OR NEW SIMPLE BOUNDS ARE ADDED TO THE ACTIVE
* SET.
*
* PARAMETERS :
*  II  NF  DECLARED NUMBER OF VARIABLES.
*  IU  N  ACTUAL NUMBER OF VARIABLES.
*  II  NC  NUMBER OF LINEARIZED CONSTRAINTS.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XN(NF)  VECTOR OF SCALING FACTORS.
*  RI  XL(NF)  VECTOR CONTAINING LOWER BOUNDS FOR VARIABLES.
*  RI  XU(NF)  VECTOR CONTAINING UPPER BOUNDS FOR VARIABLES.
*  RI  CF(NC)  VECTOR CONTAINING VALUES OF THE CONSTRAINT FUNCTIONS.
*  RI  CFD(NC) VECTOR CONTAINING INCREMENTS OF THE CONSTRAINT FUNCTIONS.
*  II  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  II  ICA(NF)  VECTOR CONTAINING INDICES OF ACTIVE CONSTRAINTS.
*  RI  CL(NC)  VECTOR CONTAINING LOWER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CU(NC)  VECTOR CONTAINING UPPER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CG(NF*NC)  MATRIX WHOSE COLUMNS ARE NORMALS OF THE LINEAR
*         CONSTRAINTS.
*  RI  CR(NR*(NF+1)/2)  TRIANGULAR DECOMPOSITION OF KERNEL OF THE
*         ORTHOGONAL PROJECTION.
*  RI  CZ(NF*NF)  MATRIX WHOSE COLUMNS ARE BASIC VECTORS FROM THE
*         CURRENT REDUCED SUBSPACE.
*  RA  S(NF)  AUXILIARY VECTOR.
*  RO  R  VALUE OF THE STEPSIZE PARAMETER.
*  RI  EPS7  TOLERANCE FOR LINEAR INDEPENDENCE OF CONSTRAINTS.
*  RI  EPS9  TOLERANCE FOR ACTIVE CONSTRAINTS.
*  RO  GMAX  MAXIMUM ABSOLUTE VALUE OF A PARTIAL DERIVATIVE.
*  RO  UMAX  MAXIMUM ABSOLUTE VALUE OF A NEGATIVE LAGRANGE MULTIPLIER.
*  II  MLC1  SPECIFICATION OF THE CONSTRAINT ADDITION. MLC1=1-ONLY ONE
*         ACTIVE CONSTRAINT IS ADDED TO THE ACTIVE SET. MLC1=2-ALL
*         ACTIVE CONSTRAINTS ARE ADDED TO THE ACTIVE SET.
*  II  INEW  INDEX OF THE NEW ACTIVE CONSTRAINT.
*  IO  IER  ERROR INDICATOR.
*
* COMMON DATA :
*  II  KBF  TYPE OF SIMPLE BOUNDS. KBF=0-NO SIMPLE BOUNDS. KBF=1-ONE
*         SIDED SIMPLE BOUNDS. KBF=2-TWO SIDED SIMPLE BOUNDS.
*  II  KBC  TYPE OF CONSTRAINTS. KBC=0-NO CONSTRAINTS. KBC=1-CONSTRAINTS
*         WITH ONE SIDED BOUNDS. KBC=2-CONSTRAINTS WITH TWO SIDED
*         BOUNDS.
*  II  NORMF  SCALING SPECIFICATION. NORMF=0-NO SCALING PERFORMED.
*         NORMF=1-SCALING FACTORS ARE DETERMINED AUTOMATICALLY.
*         NORMF=2-SCALING FACTORS ARE SUPPLIED BY USER.
*  IU  NADD  NUMBER OF CONSTRAINT ADDITIONS.
*  IO  ITERM  TERMINATION INDICATOR.
*  TO  TXFU  TEXT INFORMATION ON THE CAUSE OF TERMINATION.
*
* SUBPROGRAMS USED :
*  S   ULADB0  ADDITION OF A NEW ACTIVE CONSTRAINT.
*  S   ULNEWS  IDENTIFICATION OF ACTIVE UPPER BOUNDS.
*  S   ULNEWL  IDENTIFICATION OF ACTIVE LINEAR CONSTRAINRS.
*  S   ULDIRL  NEW VALUES OF CONSTRAINT FUNCTIONS.
*  S   UXVIND  CHANGE OF THE INTEGER VECTOR FOR CONSTRAINT ADDITION.
*  S   UOYAD1  PRINT OF ENTRY TO CONSTRAINT ADDITION.
*  S   UOYAD5  PRINT OF INFORMATION ON CONSTRAINT ADDITION.
*  S   UOERR1  ERROR MESSAGES.
*
      SUBROUTINE UYADB0(NF,N,NC,X,IX,XN,XL,XU,CF,CFD,IC,ICA,CL,CU,CG,
     & CR,CZ,S,R,EPS7,EPS9,GMAX,UMAX,MLC1,INEW,IER)
      $INCLUDE ('UMCOMM')
      INTEGER NF,N,NC,IX(NF),IC(NC),ICA(NF),MLC1,INEW,IER
      $FLOAT X(NF),XN(NF),XL(NF),XU(NF),CF(NC),CFD(NC),CL(NC),CU(NC),
     & CG(NF*NC),CR(NF*(NF+1)/2),CZ(NF*NF),S(NF),R,EPS7,EPS9,GMAX,UMAX
      INTEGER I,KC
      IF (R.NE.ZERO) CALL ULDIRL(NC,CF,CFD,IC,R,KBC)
      IF (MLC1.EQ.2.AND.INEW.NE.0) THEN
      IF (KBF.GT.0) THEN
      DO 1 I=1,NF
      INEW=0
      CALL ULNEWS(NF,X,IX,XN,XL,XU,EPS9,NORMF,I,INEW)
      CALL ULADB0(NF,N,NC,ICA,CG,CR,CZ,S,EPS7,GMAX,UMAX,INEW,NADD,IER)
      CALL UXVIND(NF,IX,I,IER)
      CALL UOYAD1(IER,'UYADB0')
      CALL UOYAD4(NF,N,NC,IX,IC,ICA,CR,CZ,UMAX,GMAX,INEW,IER)
      IF (IER.LT.0) THEN
      ITERM=-15
      TXFU='BAD ADD.'
      CALL UOERR1('UYADB0',15)
      RETURN
      ENDIF
    1 CONTINUE
      ENDIF
      IF (KBC.GT.0) THEN
      DO 2 KC=1,NC
      INEW=0
      CALL ULNEWL(NC,KC,CF,IC,CL,CU,EPS9,INEW)
      CALL ULADB0(NF,N,NC,ICA,CG,CR,CZ,S,EPS7,GMAX,UMAX,INEW,NADD,IER)
      CALL UXVIND(NC,IC,KC,IER)
      CALL UOYAD1(IER,'UYADB0')
      CALL UOYAD4(NF,N,NC,IX,IC,ICA,CR,CZ,UMAX,GMAX,INEW,IER)
      IF (IER.LT.0) THEN
      ITERM=-15
      TXFU='BAD ADD.'
      CALL UOERR1('UYADB0',15)
      RETURN
      ENDIF
    2 CONTINUE
      ENDIF
      ELSE IF (MLC1.EQ.1.AND.INEW.GT.0) THEN
      KC=INEW
      INEW=0
      CALL ULNEWL(NC,KC,CF,IC,CL,CU,EPS9,INEW)
      CALL ULADB0(NF,N,NC,ICA,CG,CR,CZ,S,EPS7,GMAX,UMAX,INEW,NADD,IER)
      CALL UXVIND(NC,IC,KC,IER)
      CALL UOYAD1(IER,'UYADB0')
      CALL UOYAD4(NF,N,NC,IX,IC,ICA,CR,CZ,UMAX,GMAX,INEW,IER)
      IF (IER.LT.0) THEN
      ITERM=-15
      TXFU='BAD ADD.'
      CALL UOERR1('UYADB0',15)
      RETURN
      ENDIF
      ELSE IF (MLC1.EQ.1.AND.INEW.LT.0) THEN
      I=-INEW
      INEW=0
      CALL ULNEWS(NF,X,IX,XN,XL,XU,EPS9,NORMF,I,INEW)
      CALL ULADB0(NF,N,NC,ICA,CG,CR,CZ,S,EPS7,GMAX,UMAX,INEW,NADD,IER)
      CALL UXVIND(NF,IX,I,IER)
      CALL UOYAD1(IER,'UYADB0')
      CALL UOYAD4(NF,N,NC,IX,IC,ICA,CR,CZ,UMAX,GMAX,INEW,IER)
      IF (IER.LT.0) THEN
      ITERM=-15
      TXFU='BAD ADD.'
      CALL UOERR1('UYADB0',15)
      RETURN
      ENDIF
      ENDIF
      RETURN
      END
