$INTERFACE UYGCC1                                             98/12/01

  $HELP
    TOLERANCE FOR CONSTRAINTS TO BE REMOVED
  $CHECK(EPS8,'1.0$P 0',REAL,4)
  CALL UYGCC1(NF,M,$X,$XO,$IX,$XN,$XL,$XU,$H,$IH,$JH,$S,$G,$GO,$GS&
,EPS9,$ETA9,XDEL,XDELS)
  $ADD(INTEGER,'\$IX($NX)\$IH($NF+1)\$JH($M)')
  $ADD(REAL,'\$X($NF)\$XO($NF)\$XN($NF)\$XL($NF)\$XU($NF)\$H($M)&
  \$S($NF)\$G($NF)\$GO($NF)\$GS($NF)')
$REM  $ADD(INTEGER,'\NF\M')
$REM  $ADD(REAL,'\EPS9\XDEL\XDELS')
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UYGCC1                ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* COMPUTATION OF THE GENERALIZED CAUCHY POINT.
*
* PARAMETERS :
*  II  NF  DECLARED NUMBER OF VARIABLES.
*  II  M  NUMBER OF NONZERO ELEMENTS OF THE HESSIAN MATRIX.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  RI  XO(NF)  SHIFTED VECTOR OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XN(NF)  VECTOR OF SCALING FACTORS.
*  RI  XL(NF)  VECTOR CONTAINING LOWER BOUNDS FOR VARIABLES.
*  RI  XU(NF)  VECTOR CONTAINING UPPER BOUNDS FOR VARIABLES.
*  RI  H(M)  HESSIAN MATRIX OR ITS APPROXIMATION.
*  II  IH(NF+1) POINTERS OF THE HESSIAN MATRIX STORED IN A COMPACT
*         FORM.
*  II  JH(MMAX) INDEX VECTOR OF THE HESSIAN MATRIX.
*  RI  S(NF)  DIRECTION VECTOR.
*  RI  G(NF)  GRADIENT OF THE OBJECTIVE FUNCTION.
*  RU  GO(NF)  GRADIENT OF THE QUADRATIC APPROXIMATION.
*  RU  GS(NF)  DIRECTION VECTOR MULTIPLIED BY THE HESSIAN MATRIX.
*  RI  EPS9  TOLERANCE FOR ACTIVE CONSTRAINTS.
*  RI  ETA9  MAXIMUM FOR REAL NUMBERS.
*  RI  XDEL  TRUST REGION RADIUS.
*  RI  XDELS  STORED TRUST REGION RADIUS.
*
* COMMON DATA :
*  II  KBF  SPECIFICATION OF SIMPLE BOUNDS. KBF=0-NO SIMPLE BOUNDS.
*         KBF=1-ONE SIDED SIMPLE BOUNDS. KBF=2=TWO SIDED SIMPLE BOUNDS.
*
* SUBPROGRAMS USED :
*  S   UYADC1  ADDITION OF A NEW ACTIVE CONSTRAINT WITH VECTOR UPDATES.
*  S   ULMAXS  MAXIMUM STEPSIZE USING SIMPLE BOUNDS.
*  S   UNSTEP  DETERMINATION OF A SCALING FACTOR FOR THE BOUNDARY STEP.
*  S   UXSSMM  MULTIPLICATION OF A SPARSE SYMMETRIC MATRIX BY A VECTOR.
*  S   UXUDIF  DIFFERENCE OF TWO VECTORS.
*  S   UXUDIR  VECTOR AUGMENTED BY THE SCALED VECTOR.
*  RF  UXUDDT  DOT PRODUCT WITH VECTOR DIFFERENCE.
*  RF  UXUDOT  DOT PRODUCT OF VECTORS.
*  RF  UXUNOR  EUCLIDEAN NORM OF A VECTOR.
*  S   UXVCOP  COPYING OF A VECTOR.
*  S   UXVNEG  COPYING OF A VECTOR WITH CHANGE OF THE SIGN.
*  RF  UXVNM2  EUCLIDEAN NORM OF VECTOR DIFFERENCE.
*
      SUBROUTINE UYGCC1(NF,M,X,XO,IX,XN,XL,XU,H,IH,JH,S,G,GO,GS,EPS9,
     & ETA9,XDEL,XDELS)
      $INCLUDE ('UMCOMM')
      INTEGER NF,M,IX(NF),IH(NF+1),JH(M)
      $FLOAT X(NF),XO(NF),XN(NF),XL(NF),XU(NF),H(M),S(NF),G(NF),GO(NF),
     & GS(NF),EPS9,ETA9,XDEL,XDELS
      $FLOAT ALF,ALF1,ALF2,RHO1,RHO2,SNORM
      $FLOAT UXUDDT,UXUDOT,UXUNOR,UXVNM2
      INTEGER KREM,INEW,I
      LOGICAL FIRST
      FIRST=.TRUE.
      CALL UXVNEG(NF,G,S)
      CALL UXVCOP(NF,X,XO)
      CALL UXVCOP(NF,G,GO)
      CALL UXSSMM(NF,M,H,IH,JH,S,GS)
    1 ALF2=ETA9
      CALL ULMAXS(NF,XO,IX,XL,XU,S,ALF2,KBF,KREM,INEW)
      IF (FIRST) THEN
      ALF1=XDEL/UXUNOR(NF,S,IX,KBF)
      ELSE
      SNORM=UXVNM2(NF,XO,X)
      RHO1=UXUDDT(NF,S,XO,X,IX,KBF)
      RHO2=UXUDOT(NF,S,S,IX,KBF)
      CALL UNSTEP(XDEL,SNORM,RHO1,RHO2,ALF1)
      ENDIF
      ALF=MIN(ALF1,ALF2,-UXUDOT(NF,S,GS,IX,KBF)/UXUDOT(NF,S,GO,IX,KBF))
      CALL UXUDIR(NF,ALF,S,XO,XO,IX,KBF)
      CALL UCBS04(NF,XO,IX,XN,XL,XU,EPS9)
      IF (ALF.GE.ALF2) THEN
      CALL UYADC1(NF,M,XO,IX,XL,XU,H,IH,S,GO,GS,INEW)
      FIRST=.FALSE.
      GO TO 1
      ENDIF
      SNORM=ZERO
      IF (FIRST) THEN
      CALL UXVSET(NF,ZERO,S)
      ELSE
      DO 2 I=1,NF
      IF (IX(I).GE.0) THEN
      S(I)=XO(I)-X(I)
      SNORM=SNORM+S(I)*S(I)
      ELSE
      S(I)=ZERO
      ENDIF
    2 CONTINUE
      XDELS=XDEL
      XDEL=SQRT(MAX(ZERO,XDEL**2-SNORM))
      CALL UXSSMD(NF,M,H,IH,JH,S,ONE,G,G)
      CALL UXVINB(NF,M,IX,JH)
      ENDIF
      RETURN
      END
