$INTERFACE UYRMB0                                            98/12/01

  $HELP
    OPTION FOR CONSTRAINT DELETION
      0 - SINGLE CONSTRAINT DELETION
      1 - SIMULTANEOUS CONSTRAINT DELETION
  $CHECK(IREM,0,'0\1',4,NO)
  $HELP
    TOLERANCE FOR CONSTRAINTS TO BE REMOVED
  $CHECK(EPS8,'0.5$P 0',REAL,4)
  $IF(IREM>0)
$00 CONTINUE
  $ENDIF
  CALL UYRMB0(NF,N,MCL,$IX,$XN,$IC,$ICA,$CG,$CR,$CZ,$G,$GN,EPS8,UMAX,
     & GMAX,MOT,IOLD,KREM,IER)
  $IF(IREM>0)
    IF (IOLD.NE.0) THEN
    CALL UYTRBG(NF,N,MCL,$IX,$XN,$IC,$ICA,$CG,$CR,$CZ,$G,$GN,UMAX,GMAX&
,IOLD)
    GO TO $00
    ENDIF
  $ENDIF
  $ADD(REAL,'\$XN($NF)\$CG($NF*$MCL)\$CR($NF*($NF+1)/2)\$CZ($NF*$NF)&
  \$G($NF)\$GN($NF)')
  $ADD(INTEGER,'\$IX($NX)\$IC($NC)\$ICA($NF)')
$REM  $ADD(INTEGER,'\NF\N\MCL\MOT\IOLD\KREM\IER')
$REM  $ADD(REAL,'\UMAX\GMAX')
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UYRMB0               ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* OLD LINEAR CONSTRAINT OR AN OLD SIMPLE BOUND IS REMOVED FROM THE
* ACTIVE SET. TRANSFORMED GRADIENT OF THE OBJECTIVE FUNCTION IS
* UPDATED.
*
* PARAMETERS :
*  II  NF  DECLARED NUMBER OF VARIABLES.
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  II  NC  NUMBER OF CONSTRAINTS.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XN(NF)  VECTOR OF SCALING FACTORS.
*  II  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  II  ICA(NF)  VECTOR CONTAINING INDICES OF ACTIVE CONSTRAINTS.
*  RI  CG(NF*NC)  MATRIX WHOSE COLUMNS ARE NORMALS OF THE LINEAR
*         CONSTRAINTS.
*  RI  CR(NF*(NF+1)/2)  TRIANGULAR DECOMPOSITION OF KERNEL OF THE
*         ORTHOGONAL PROJECTION.
*  RI  CZ(NF*NF)  MATRIX WHOSE COLUMNS ARE BASIC VECTORS FROM THE
*         CURRENT REDUCED SUBSPACE.
*  RI  G(NF)  GRADIENT OF THE OBJECTIVE FUNCTION.
*  RU  GN(NF)  TRANSFORMED GRADIENT OF THE OBJECTIVE FUNCTION.
*  RI  EPS8  TOLERANCE FOR CONSTRAINT TO BE REMOVED.
*  RI  GMAX  NORM OF THE TRANSFORMED GRADIENT.
*  RI  UMAX  MAXIMUN ABSOLUTE VALUE OF THE NEGATIVE LAGRANGE MULTIPLIER.
*  II  MOT  SPECIFICATION OF RESTART AFTER CHANGE OF ACTIVE
*         CONSTRAINTS. MOT=1-RESTART AFTER EACH CHANGE OF ACTIVE
*         CONSTRAINTS. MOT=2-NO RESTART.
*  II  IOLD  INDEX OF THE REMOVED CONSTRAINT.
*  IA  KREM  AUXILIARY VARIABLE.
*  IO  IER  ERROR INDICATOR.
*
* COMMON DATA :
*  II  NORMF  SCALING SPECIFICATION. NORMF=0-NO SCALING PERFORMED.
*         NORMF=1-SCALING FACTORS ARE DETERMINED AUTOMATICALLY.
*         NORMF=2-SCALING FACTORS ARE SUPPLIED BY USER.
*  IU  NREM  NUMBER OF CONSTRAINT DELETIONS.
*  IO  ITERM  TERMINATION INDICATOR.
*  TO  TXFU  TEXT INFORMATION ON THE CAUSE OF TERMINATION.
*
* SUBPROGRAMS USED :
*  S   UYRES1  DEFINITION OF THE RESTART VARIABLES.
*  S   ULRMB0  CONSTRAINT DELETION.
*  S   UOYRM1  PRINT OF ENTRY TO CONSTRAINT DELETION.
*  S   UOYRM4  PRINT OF INFORMATION ON CONSTRAINT DELETION.
*  S   UOERR1  ERROR MESSAGES.
*
      SUBROUTINE UYRMB0(NF,N,NC,IX,XN,IC,ICA,CG,CR,CZ,G,GN,EPS8,UMAX,
     & GMAX,MOT,IOLD,KREM,IER)
      $INCLUDE ('UMCOMM')
      INTEGER NF,N,NC,IX(NF),IC(NC),ICA(NF),MOT,IOLD,KREM,IER
      $FLOAT XN(NF),CG(NF*NC),CR(NF*(NF+1)/2),CZ(NF*NF),G(NF),GN(NF),
     & EPS8,UMAX,GMAX
      INTEGER K,KC
      IF (UMAX.GT.EPS8*GMAX) THEN
      CALL UOYRM1(N,GMAX,UMAX,IOLD,'UYRMB0')
      CALL ULRMB0(NF,N,NC,XN,ICA,CG,CR,CZ,G,GN,NORMF,IOLD,KREM,NREM,IER)
      IF (IER.LT.0) THEN
      ITERM=-16
      TXFU='BAD REM.'
      CALL UOERR1('UYRMB0',16)
      ELSE IF (IER.GT.0) THEN
      IOLD=0
      ELSE
      KC=ICA(NF-N+1)
      IF (KC.GT.0) THEN
      IC(KC)=-IC(KC)
      ELSE
      K=-KC
      IX(K)=-IX(K)
      ENDIF
      ENDIF
      CALL UOYRM4(NF,N,NC,IX,IC,ICA,CR,CZ,IER)
      IF (MOT.EQ.1) CALL UYRES1('REM CONS')
      ELSE
      IOLD=0
      ENDIF
      RETURN
      END
