$INTERFACE UYRMP0                                            98/12/01

  $HELP
    TOLERANCE FOR CONSTRAINTS TO BE REMOVED
  $CHECK(EPS8,'1.0$P 0',REAL,4)
  CALL UYRMP0(NF,N,MCL,$MMAX,$IX,$IC,$ICA,$CR,$ICR,$JCR,$CZ,$S,EPS8&
,UMAX,GMAX,MOT,IOLD,KREM,IER)
  $ADD(REAL,'\$CR($MMAX)\$CZ($NF)\$S($NF)')
  $ADD(INTEGER,'\$IX($NX)\$IC($NC)\$ICA($NF)\$ICR($NF+1)\$JCR($MMAX)')
$REM  $ADD(INTEGER,'\NF\N\MCL\MOT\IOLD\KREM\IER')
$REM  $ADD(REAL,'\UMAX\GMAX')
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UYRMP0               ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* OLD LINEAR CONSTRAINT OR AN OLD SIMPLE BOUND IS REMOVED FROM THE
* ACTIVE SET. TRANSFORMED GRADIENT OF THE OBJECTIVE FUNCTION IS
* UPDATED.
*
* PARAMETERS :
*  II  NF  DECLARED NUMBER OF VARIABLES.
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  II  NC  NUMBER OF CONSTRAINTS.
*  II  MMAX  MAXIMUM NUMBER OF NONZERO ELEMENTS IN THE FIELD CR.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  II  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  II  ICA(NF)  VECTOR CONTAINING INDICES OF ACTIVE CONSTRAINTS.
*  RI  CR(MMAX)  TRIANGULAR DECOMPOSITION OF KERNEL OF THE
*         ORTHOGONAL PROJECTION.
*  II  ICR(NF+1)  POSITION OF THE FIRST RORWS ELEMENTS IN THE FIELD CR.
*  II  JCR(MMAX) COLUMN INDICES OF ELEMENTS IN THE FIELD CR.
*  RO  CZ(NF)  VECTOR OF LAGRANGE MULTIPLIERS.
*  RI  EPS8  TOLERANCE FOR CONSTRAINT TO BE REMOVED.
*  RI  GMAX  NORM OF THE TRANSFORMED GRADIENT.
*  RI  UMAX  MAXIMUN ABSOLUTE VALUE OF THE NEGATIVE LAGRANGE MULTIPLIER.
*  II  MOT  SPECIFICATION OF RESTART AFTER CHANGE OF ACTIVE
*         CONSTRAINTS. MOT=1-RESTART AFTER EACH CHANGE OF ACTIVE
*         CONSTRAINTS. MOT=2-NO RESTART.
*  II  IOLD  INDEX OF THE REMOVED CONSTRAINT.
*  IA  KREM  AUXILIARY VARIABLE.
*  IO  IER  ERROR INDICATOR.
*
* COMMON DATA :
*  II  NORMF  SCALING SPECIFICATION. NORMF=0-NO SCALING PERFORMED.
*         NORMF=1-SCALING FACTORS ARE DETERMINED AUTOMATICALLY.
*         NORMF=2-SCALING FACTORS ARE SUPPLIED BY USER.
*  IU  NREM  NUMBER OF CONSTRAINT DELETIONS.
*  IO  ITERM  TERMINATION INDICATOR.
*  TO  TXFU  TEXT INFORMATION ON THE CAUSE OF TERMINATION.
*
* SUBPROGRAMS USED :
*  S   UYRES1  DEFINITION OF THE RESTART VARIABLES.
*  S   UMRMB0  CONSTRAINT DELETION.
*  S   UOYRM1  PRINT OF ENTRY TO CONSTRAINT DELETION.
*  S   UOYRM6  PRINT OF INFORMATION ON CONSTRAINT DELETION.
*  S   UOERR1  ERROR MESSAGES.
*
      SUBROUTINE UYRMP0(NF,N,NC,MMAX,IX,IC,ICA,CR,ICR,JCR,CZ,S,EPS8,
     & UMAX,GMAX,MOT,IOLD,KREM,IER)
      $INCLUDE ('UMCOMM')
      INTEGER NF,N,NC,MMAX,IX(NF),IC(NC),ICA(NF),ICR(NF+1),JCR(MMAX),
     & MOT,IOLD,KREM,IER
      $FLOAT CR(MMAX),CZ(NF),S(NF),EPS8,UMAX,GMAX
      INTEGER K,KC
      IF (UMAX.GT.EPS8*GMAX) THEN
      CALL UOYRM1(N,GMAX,UMAX,IOLD,'UYRMP0')
      IER=0
      IF (N.EQ.NF) IER=2
      IF (IOLD.EQ.0) IER=3
      IF (IER.GT.0) THEN
      IOLD=0
      ELSE
      CALL UMRMR0(NF,N,MMAX,ICA,CR,ICR,JCR,CZ,S,IOLD,KREM)
      N=N+1
      NREM=NREM+1
      KC=ICA(NF-N+1)
      IF (KC.GT.0) THEN
      IC(KC)=-IC(KC)
      ELSE
      K=-KC
      IX(K)=-IX(K)
      ENDIF
      ENDIF
      CALL UOYRM6(NF,N,NC,MMAX,IX,IC,ICA,CR,ICR,JCR,CZ,IER)
      IF (MOT.EQ.1) CALL UYRES1('REM CONS')
      ELSE
      IOLD=0
      ENDIF
      RETURN
      END
