* SUBROUTINE UYSIDO                ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* DETERMINATION OF AN INCIDENCE-DEGREE ORDERING OF THE COLUMNS OF A.
*
* PARAMETERS :
*  II  NF DECLARED NUMBER OF VARIABLES.
*  II  NA  NUMBER OF APPROXIMATED FUNCTIONS.
*  II  MA  NUMBER OF NONZERO ELEMENTS OF THE JACOBIAN MATRIX.
*  II  IA(NA+1) ROW POINTERS OF THE SPARSE JACOBIAN MATRIX.
*  II  JA(MA) COLUMN INDICES OF THE SPARSE JACOBIAN MATRIX.
*  II  IB(NF+1) COLUMN POINTERS OF THE SPARSE JACOBIAN MATRIX.
*  II  JB(MA) ROW INDICES OF THE SPARSE JACOBIAN MATRIX.
*  II  IDEG(NF) VECTOR SPECIFYING DEGREE SEQUENCE.
*  IO  LIST(NF) VECTOR SPECIFYING INCIDENCE-DEGREE ORDERING.
*  IA  IW1(NF)  AUXILIARY VECTOR.
*  IA  IW2(NF)  AUXILIARY VECTOR.
*  IA  IW3(NF)  AUXILIARY VECTOR.
*  IA  IW4(NF)  AUXILIARY VECTOR.
*  IO  MAXC  SIZE OF THE LARGEST CLIQUE FOUND DURING THE ORDERING.
*
      SUBROUTINE UYSIDO(NF,NA,MA,IA,JA,IB,JB,IDEG,LIST,IW1,IW2,IW3,IW4,
     & MAXC)
      INTEGER NF,NA,MA,IA(NA+1),JA(MA),IB(NF+1),JB(MA),IDEG(NF),
     & LIST(NF),IW1(0:NF-1),IW2(NF),IW3(NF),IW4(NF),MAXC
      INTEGER IC,IP,IR,JC,JP,MAXI,MAXL,NCOM,NUMI,NUML,NUMO,NUMW
C
C     SORT THE DEGREE SEQUENCE
C
      CALL UXVSR1(NF,NF-1,IDEG,IW4,IW2,IW3,-1)
C
C     INITIALIZATION BLOCK
C
      MAXI = 0
      DO 10 JP = NF, 1, -1
      IC = IW4(JP)
      IW1(NF-JP) = 0
      IW2(IC) = 0
      IW3(IC) = IW1(0)
      IF (IW1(0) .GT. 0) IW2(IW1(0)) = IC
      IW1(0) = IC
      IW4(JP) = 0
      LIST(JP) = 0
   10 CONTINUE
C
C     DETERMINE THE MAXIMAL SEARCH LENGTH FOR THE LIST
C     OF COLUMNS OF MAXIMAL INCIDENCE
C
      MAXL = 0
      DO 20 IR = 1, NA
      MAXL = MAXL + (IA(IR+1) - IA(IR))**2
   20 CONTINUE
      MAXL = MAXL/NF
      MAXC = 0
      NUMO = 1
C
C     BEGINNING OF ITERATION LOOP
C
   30 CONTINUE
C
C     UPDATE THE SIZE OF THE LARGEST CLIQUE
C     FOUND DURING THE ORDERING
C
      IF (MAXI .EQ. 0) NCOM = 0
      NCOM = NCOM + 1
      IF (MAXI + 1 .EQ. NCOM) MAXC = MAX(MAXC,NCOM)
C
C     CHOOSE A COLUMN JC OF MAXIMAL DEGREE AMONG THE
C     COLUMNS OF MAXIMAL INCIDENCE MAXI
C
   40 CONTINUE
      JP = IW1(MAXI)
      IF (JP .GT. 0) GO TO 50
      MAXI = MAXI - 1
      GO TO 40
   50 CONTINUE
      NUMW = -1
      DO 60 NUML = 1, MAXL
      IF (IDEG(JP) .GT. NUMW) THEN
      NUMW = IDEG(JP)
      JC = JP
      END IF
      JP = IW3(JP)
      IF (JP .LE. 0) GO TO 70
   60 CONTINUE
   70 CONTINUE
      LIST(JC) = NUMO
      NUMO = NUMO + 1
C
C     TERMINATION TEST
C
      IF (NUMO .GT. NF) GO TO 100
C
C     DELETE COLUMN JC FROM THE MAXI LIST
C
      IF (IW2(JC) .EQ. 0) THEN
      IW1(MAXI) = IW3(JC)
      ELSE
      IW3(IW2(JC)) = IW3(JC)
      END IF
      IF (IW3(JC) .GT. 0) IW2(IW3(JC)) = IW2(JC)
C
C     FIND ALL COLUMNS ADJACENT TO COLUMN JC
C
      IW4(JC) = NF
C
C     DETERMINE ALL POSITIONS (IR,JC) WHICH CORRESPOND
C     TO NON-ZEROES IN THE MATRIX
C
      DO 90 JP = IB(JC), IB(JC+1)-1
      IR = JB(JP)
C
C     FOR EACH ROW IR, DETERMINE ALL POSITIONS (IR,IC)
C     WHICH CORRESPOND TO NON-ZEROES IN THE MATRIX
C
      DO 80 IP = IA(IR), IA(IR+1)-1
      IC = JA(IP)
C
C     ARRAY IW4 MARKS COLUMNS WHICH ARE ADJACENT TO
C     COLUMN JC
C
      IF (IW4(IC) .LT. NUMO) THEN
      IW4(IC) = NUMO
C
C     UPDATE THE POINTERS TO THE CURRENT INCIDENCE LISTS
C
      NUMI = LIST(IC)
      LIST(IC) = LIST(IC) + 1
      MAXI = MAX(MAXI,LIST(IC))
C
C     DELETE COLUMN IC FROM THE NUMI LIST
C
      IF (IW2(IC) .EQ. 0) THEN
      IW1(NUMI) = IW3(IC)
      ELSE
      IW3(IW2(IC)) = IW3(IC)
      END IF
      IF (IW3(IC) .GT. 0) IW2(IW3(IC)) = IW2(IC)
C
C     ADD COLUMN IC TO THE NUMI+1 LIST
C
      IW2(IC) = 0
      IW3(IC) = IW1(NUMI+1)
      IF (IW1(NUMI+1) .GT. 0) IW2(IW1(NUMI+1)) = IC
      IW1(NUMI+1) = IC
      END IF
   80 CONTINUE
   90 CONTINUE
C
C     END OF ITERATION LOOP
C
      GO TO 30
  100 CONTINUE
C
C     INVERT THE ARRAY LIST
C
      DO 110 JC = 1, NF
      IW2(LIST(JC)) = JC
  110 CONTINUE
      DO 120 JP = 1, NF
      LIST(JP) = IW2(JP)
  120 CONTINUE
      RETURN
      END
