$INTERFACE UYTRAD                                             98/12/01

    CALL UYTRAD(NF,N,MAL,$X,$XN,$XO,$IAA,$AG,$AZ,$AZS,$G,$GO,R,F,FO&
,P,PO,CMAX,CMAXO,DMAX,IPOM3)
    $ADD(REAL,'\$X($NF)\$XN($NF)\$XO($NF)\$AG($NF*$MAL)\$AZ($NF+1)&
    \$AZS($NF+1)\$G($NF)\$GO($NF)')
    $ADD(INTEGER,'\$IAA($NF+1)')
$REM  $ADD(REAL,'\R\FO\F\PO\P\DMAX\CMAX\CMAXO')
$REM  $ADD(INTEGER,'\NF\N\MAL\IPOM3')
    $EXIT

* IMPLEMENTATION

* SUBROUTINE UYTRAD             ALL SYSTEMS                   98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* DUAL RANGE SPACE QUADRATIC PROGRAMMING METHOD FOR MINIMAX
* APPROXIMATION.
*
* PARAMETERS :
*  II  NF DECLARED NUMBER OF VARIABLES.
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  II  NA NUMBER OF APPROXIMATED FUNCTIONS.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  RI  XN(NF)  VECTOR OF SCALING FACTORS.
*  RO  XO(NF)  SAVED VECTOR OF VARIABLES.
*  II  IAA(NF+1)  VECTOR CONTAINING INDICES OF ACTIVE FUNCTIONS.
*  RI  AG(NF*NA)  VECTOR CONTAINING SCALING PARAMETERS.
*  RO  AZ(NF+1)  VECTOR OF LAGRANGE MULTIPLIERS.
*  RI  AZS(NF+1)  SAVED VECTOR OF LAGRANGE MULTIPLIERS.
*  RI  G(NF)  GRADIENT OF THE LAGRANGIAN FUNCTION.
*  RI  GO(NF)  SAVED GRADIENT OF THE LAGRANGIAN FUNCTION.
*  RO  R  VALUE OF THE STEPSIZE PARAMETER.
*  RO  F  NEW VALUE OF THE OBJECTIVE FUNCTION.
*  RI  FO  OLD VALUE OF THE OBJECTIVE FUNCTION.
*  RO  P  NEW VALUE OF THE DIRECTIONAL DERIVATIVE.
*  RI  PO  OLD VALUE OF THE DIRECTIONAL DERIVATIVE.
*  RI  CMAX  VALUE OF THE CONSTRAINT VIOLATION.
*  RO  CMAXO  SAVED VALUE OF THE CONSTRAINT VIOLATION.
*  RO  DMAX  MAXIMUM RELATIVE DIFFERENCE OF VARIABLES.
*
* COMMON DATA :
*  II  NORMF  SCALING SPECIFICATION. NORMF=0-NO SCALING PERFORMED.
*         NORMF=1-SCALING FACTORS ARE DETERMINED AUTOMATICALLY.
*         NORMF=2-SCALING FACTORS ARE SUPPLIED BY USER.
*  II  ITERS  TERMINATION INDICATOR FOR STEPLENGTH DETERMINATION.
*         ITERS=0 FOR ZERO STEP.
*
* SUBPROGRAMS USED :
*  S   UXVCOP  COPYING OF A VECTOR.
*  S   UXVDIR  VECTOR AUGMENTED BY THE SCALED VECTOR.
*  S   UXVSET  INITIATION OF A VECTOR.
*  S   UXVSAV  DIFFERENCE OF TWO VECTORS WITH COPYING AND SAVING THE
*         SUBSTRACTED ONE.
*
      SUBROUTINE UYTRAD(NF,N,NA,X,XN,XO,IAA,AG,AZ,AZS,G,GO,R,F,FO,
     & P,PO,CMAX,CMAXO,DMAX,IPOM3)
      $INCLUDE ('UMCOMM')
      INTEGER NF,N,NA,IPOM3
      INTEGER IAA(NF+1)
      $FLOAT X(NF),XN(NF),XO(NF),AG(NF*NA),AZ(NF+1),AZS(NF+1),G(NF),
     & GO(NF),R,F,FO,P,PO,CMAX,CMAXO,DMAX
      INTEGER I,J,L
      IF (IPOM3.EQ.2) CALL UXVCOP(NF-N,AZS,AZ)
      CALL UXVSET(NF,ZERO,G)
      DO 2 J=1,NF-N
      L=IAA(J)
      IF (L.GT. 0) THEN
      IF (L.GT.NA) L=L-NA
      CALL UXVDIR(NF,-AZ(J),AG((L-1)*NF+1),G,G)
      ELSE
      L=-L
      G(L)=G(L)-AZ(J)
      ENDIF
    2 CONTINUE
      IF (ITERS.GT.0) THEN
      CALL UXVDIF(NF,X,XO,XO)
      CALL UXVDIF(NF,G,GO,GO)
      PO=R*PO
      P=R*P
      ELSE
      F = FO
      P = PO
      CMAX = CMAXO
      CALL UXVSAV(NF,X,XO)
      CALL UXVSAV(NF,G,GO)
      CALL UYSET1
      ENDIF
      DMAX = ZERO
      DO 1 I=1,NF
      IF ( NORMF .GT. 0) THEN
      DMAX=MAX(DMAX,ABS(XO(I))/MAX(ABS(X(I)),XN(I)))
      XO(I)=XO(I)/XN(I)
      ELSE
      DMAX=MAX(DMAX,ABS(XO(I))/MAX(ABS(X(I)),ONE))
      ENDIF
    1 CONTINUE
      N=NF
      RETURN
      END




