$INTERFACE UYTRBB                                             98/12/01

  CALL UYTRBB(NF,N,$X,$XN,$XO,$G,$GO,$CZ,$SN,$S,$H,R,F,FO,P,PO,DMAX)
  $ADD(REAL,'\$X($NF)\$XN($NF)\$XO($NF)\$G($NF)\$GO($NF)&
\$CZ($NF*$NF)\$SN($NF)\$S($NF)\$H($NF*($NF+1)/2)')
$REM  $ADD(REAL,'\R\FO\F\PO\P\DMAX')
$REM  $ADD(INTEGER,'\NF\N')
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UYTRBB             ALL SYSTEMS                   98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* VECTORS OF VARIABLES DIFFERENCE AND GRADIENTS DIFFERENCE ARE COMPUTED
* AND TRANSFORMED. TEST VALUE DMAX IS DETERMINED.
*
* PARAMETERS :
*  II  NF DECLARED NUMBER OF VARIABLES.
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  RI  XN(NF)  VECTOR OF SCALING FACTORS.
*  RU  XO(NF)  VECTORS OF VARIABLES DIFFERENCE.
*  RI  G(NF)  GRADIENT OF THE OBJECTIVE FUNCTION.
*  RU  GO(NF)  GRADIENTS DIFFERENCE.
*  RI  CZ(NF*NF)  MATRIX WHOSE COLUMNS ARE BASIC VECTORS FROM CURRENT
*         REDUCED SUBSPACE.
*  RU  SN(NF)  TRANSFORMED DIRECTION VECTOR.
*  RA  S(NF)  AUXILIARY VECTOR.
*  RI  H(NF*(NF+1)/2)  HESSIAN MATRIX OF THE OBJECTIVE FUNCTION OR ITS
*         APPROXIMATION.
*  RO  R  VALUE OF THE STEPSIZE PARAMETER.
*  RO  F  NEW VALUE OF THE OBJECTIVE FUNCTION.
*  RI  FO  OLD VALUE OF THE OBJECTIVE FUNCTION.
*  RO  P  NEW VALUE OF THE DIRECTIONAL DERIVATIVE.
*  RI  PO  OLD VALUE OF THE DIRECTIONAL DERIVATIVE.
*  RO  DMAX  MAXIMUM RELATIVE DIFFERENCE OF VARIABLES.
*
* COMMON DATA :
*  II  NORMF  SCALING SPECIFICATION. NORMF=0-NO SCALING IS PERFORMED.
*         NORMF=1-SCALING FACTORS ARE DETERMINED AUTOMATICALLY.
*         NORMF=2-SCALING FACTORS ARE SUPPLIED BY USER.
*  II  ITERS  TERMINATION INDICATOR FOR STEPLENGTH DETERMINATION.
*         ITERS=0 FOR ZERO STEP.
*
* SUBPROGRAMS USED :
*  S   UYSET1  DEGREE DEFINITION OF THE COMPUTED DERIVATIVES.
*  S   UXDRMM  PREMULTIPLICATION OF A VECTOR BY TRANSPOSE OF A DENSE
*         RECTANGULAR MATRIX.
*  S   UXDSMM  MATRIX VECTOR PRODUCT.
*  S   UXVCOP  COPYING OF A VECTOR.
*  S   UXVDIF  DIFFERENCE OF TWO VECTORS.
*  S   UXVMUL  DIAGONAL PREMULTIPLICATION OF A VECTOR.
*  S   UXVSAV  DIFFERENCE OF TWO VECTORS WITH COPYING AND SAVING THE
*         SUBSTRACTED ONE.
*  S   UXVSCL  SCALING OF A VECTOR.
*
      SUBROUTINE UYTRBB(NF,N,X,XN,XO,G,GO,CZ,SN,S,H,R,F,FO,P,PO,DMAX)
      $INCLUDE ('UMCOMM')
      INTEGER NF,N
      $FLOAT X(NF),XN(NF),XO(NF),G(NF),GO(NF),CZ(NF*NF),SN(NF),S(NF),
     &  H(NF*(NF+1)/2),R,F,FO,P,PO,DMAX
      INTEGER I
      IF (ITERS.GT.0) THEN
      CALL UXVDIF(NF,X,XO,XO)
      CALL UXVDIF(NF,G,GO,GO)
      PO=R*PO
      P=R*P
      ELSE
      F = FO
      P = PO
      CALL UXVSAV(NF,X,XO)
      CALL UXVSAV(NF,G,GO)
      CALL UYSET1
      ENDIF
      CALL UXDSMM(NF,H,XO,S)
      DMAX = ZERO
      DO 1 I=1,NF
      IF ( NORMF .GT. 0) THEN
      DMAX=MAX(DMAX,ABS(XO(I))/MAX(ABS(X(I)),XN(I)))
      ELSE
      DMAX=MAX(DMAX,ABS(XO(I))/MAX(ABS(X(I)),ONE))
      ENDIF
    1 CONTINUE
      IF (N.GT.0) THEN
      CALL UXVSCL(N,R,SN,XO)
      IF (NORMF.GT.0) THEN
      CALL UXVMUL(NF,XN,GO,SN,1)
      ELSE
      CALL UXVCOP(NF,GO,SN)
      ENDIF
      CALL UXDRMM(NF,N,CZ,SN,GO)
      IF (NORMF.GT.0) THEN
      CALL UXVMUL(NF,XN,S,SN,1)
      ELSE
      CALL UXVCOP(NF,S,SN)
      ENDIF
      CALL UXDRMM(NF,N,CZ,SN,S)
      CALL UXVDIF(N,GO,S,GO)
      ENDIF
      IF (ITERS.EQ.0) R=ZERO
      RETURN
      END
