$INTERFACE UYTRRG                                            16/12/01

  $HELP
    COMPUTATION OF THE GRADIENT NORM FOR TERMINATION CRITERIA:
      1 - MAXIMUM NORM OF THE MODIFIED REDUCED GRADIENT
      2 - EUCLIDEAN NORM OF THE REDUCED GRADIENT COMPUTED
          USING LAGRANGE MULTIPLIERS
      3 - MAXIMUM NORM OF THE PROJECTED GRADIENT COMPUTED
          USING LAGRANGE MULTIPLIERS
  $CHECK(KTERM,1,'1\2\3',4)
  $IF(KTERM=1)
    CALL UYTRBG(NF,N,MCL,$IX,$XN,$IC,$ICA,$CG,$CR,$CZ,$G,$GN,UMAX,GMAX&
,IOLD)
  $ELSE
    CALL UYTRRG(NF,N,MCL,$IX,$XN,$IC,$ICA,$CG,$CR,$CZ,$G,$GN,$S,UMAX&
,GMAX,IOLD,KTERM)
  $ENDIF
  $ADD(REAL,'\$XN($NF)\$CG($NF*$MCL)\$CR($NF*($NF+1)/2)&
  \$CZ($NF*$NF)\$G($NF)\$GN($NF)\$S($NF)')
  $ADD(INTEGER,'\$IX($NX)\$IC($NC)\$ICA($NF)')
$REM  $ADD(INTEGER,'\NF\N\MCL\IOLD\KTERM')
$REM  $ADD(REAL,'\UMAX\GMAX')
 $ADD(INIT)
    ITRAN=2
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UYTRRG               ALL SYSTEMS                16/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* GRADIENT OF THE OBJECTIVE FUNCTION IS SCALED AND REDUCED.
* TEST VALUES GMAX AND UMAX ARE COMPUTED.
*
* PARAMETERS :
*  II  NF  DECLARED NUMBER OF VARIABLES.
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  II  NC  NUMBER OF CONSTRAINTS.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XN(NF)  VECTOR OF SCALING FACTORS.
*  II  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  II  ICA(NF)  VECTOR CONTAINING INDICES OF ACTIVE CONSTRAINTS.
*  RI  CG(NF*NC)  MATRIX WHOSE COLUMNS ARE NORMALS OF THE LINEAR
*         CONSTRAINTS.
*  RI  CR(NF*(NF+1)/2)  TRIANGULAR DECOMPOSITION OF KERNEL OF THE
*         ORTHOGONAL PROJECTION.
*  RI  CZ(NF*NF)  MATRIX WHOSE COLUMNS ARE BASIC VECTORS FROM THE
*         CURRENT REDUCED SUBSPACE.
*  RI  G(NF)  GRADIENT OF THE OBJECTIVE FUNCTION.
*  RO  GN(NF)  TRANSFORMED GRADIENT OF THE OBJECTIVE FUNCTION.
*  RA  S(NF)  AUXILIARY VECTOR.
*  RI  EPS7  TOLERANCE FOR LINEAR INDEPENDENCE OF CONSTRAINTS.
*  RI  UMAX  MAXIMUM ABSOLUTE VALUE OF THE NEGATIVE LAGRANGE MULTIPLIER.
*  RI  GMAX  NORM OF THE TRANSFORMED GRADIENT.
*  II  IOLD  INDEX OF THE REMOVED CONSTRAINT.
*  II  KTERM COMPUTATION OF THE GRADIENT NORM FOR TERMINATION CRITERIA.
*         KTERM=1-MAXIMUM NORM OF THE MODIFIED REDUCED GRADIENT.
*         KTERM=2-EUCLIDEAN NORM OF THE REDUCED GRADIENT COMPUTED
*         USING LAGRANGE MULTIPLIERS. KTERM=3-MAXIMUM NORM OF THE
*         PROJECTED GRADIENT COMPUTED USING LAGRANGE MULTIPLIERS.
*
* COMMON DATA :
*  II  KBF  SPECIFICATION OF SIMPLE BOUNDS. KBF=0-NO SIMPLE BOUNDS.
*         KBF=1-ONE SIDED SIMPLE BOUNDS. KBF=2=TWO SIDED SIMPLE BOUNDS.
*  II  KBC  SPECIFICATION OF LINEAR CONSTRAINTS. KBC=0-NO LINEAR
*         CONSTRAINTS. KBC=1-ONE SIDED LINEAR CONSTRAINTS. KBC=2=TWO
*         SIDED LINEAR CONSTRAINTS.
*  II  NORMF  SCALING SPECIFICATION. NORMF=0-NO SCALING IS PERFORMED.
*         NORMF=1-SCALING FACTORS ARE DETERMINED AUTOMATICALLY.
*         NORMF=2-SCALING FACTORS ARE SUPPLIED BY USER.
*
* SUBPROGRAMS USED :
*  S   UXDRMM  PREMULTIPLICATION OF A VECTOR BY A ROWWISE STORED DENSE
*         RECTANGULAR MATRIX.
*  S   UXDPRB  BACK SUBSTITUTION.
*  S   UXVCOP  COPYING OF A VECTOR.
*  RF  UXVDOT  DOT PRODUCT OF TWO VECTORS.
*  RF  UXVMAX  L-INFINITY NORM OF A VECTOR.
*  S   UXVMUL  DIAGONAL PREMULTIPLICATION OF A VECTOR.
*
      SUBROUTINE UYTRRG(NF,N,NC,IX,XN,IC,ICA,CG,CR,CZ,G,GN,S,UMAX,GMAX,
     & IOLD,KTERM)
      $INCLUDE ('UMCOMM')
      INTEGER NF,N,NC,IX(NF),IC(NC),ICA(NF),IOLD,KTERM
      $FLOAT XN(NF),CG(NF*NC),CR(NF*(NF+1)/2),CZ(NF*NF),G(NF),GN(NF),
     & S(NF),UMAX,GMAX
      $FLOAT TEMP,UXVMAX,UXVDOT
      INTEGER NCA,NCR,NCZ,I,J,K,KC
      IOLD=0
      UMAX=0.0$P 0
      GMAX=0.0$P 0
      IF (NORMF.GT.0) THEN
      CALL UXVMUL(NF,XN,G,GN,1)
      ELSE
      CALL UXVCOP(NF,G,GN)
      ENDIF
      CALL UXVCOP(NF,GN,S)
      IF (NF.GT.N) THEN
      NCA=NF-N
      NCR=NCA*(NCA+1)/2
      NCZ=N*NF
      DO 1 J=1,NCA
      K=ICA(J)
      IF (K.GT.0) THEN
      CZ(NCZ+J)=UXVDOT(NF,CG((K-1)*NF+1),GN)
      ELSE
      I=-K
      CZ(NCZ+J)=GN(I)
      ENDIF
    1 CONTINUE
      CALL UXDPRB(NCA,CR,CZ(NCZ+1),0)
      DO 2 J=1,NCA
      TEMP=CZ(NCZ+J)
      KC=ICA(J)
      IF (KC.GT.0) THEN
      K=IC(KC)
      ELSE
      I=-KC
      K=IX(I)
      ENDIF
      IF (K.LE.-5) THEN
      ELSE IF ((K.EQ.-1.OR.K.EQ.-3).AND.UMAX+TEMP.GE.0.0$P 0) THEN
      ELSE IF ((K.EQ.-2.OR.K.EQ.-4).AND.UMAX-TEMP.GE.0.0$P 0) THEN
      ELSE
      IOLD=J
      UMAX=ABS(TEMP)
      ENDIF
    2 CONTINUE
      DO 3 J=1,NCA
      K=ICA(J)
      IF (K.GT.0) THEN
      CALL UXVDIR(NF,-CZ(NCZ+J),CG((K-1)*NF+1),S,S)
      ELSE
      I=-K
      S(I)=S(I)-CZ(NCZ+J)
      ENDIF
    3 CONTINUE
      ENDIF
      IF (KTERM.EQ.3) THEN
      GMAX=UXVMAX(NF,S)
      ELSE
      GMAX=SQRT(ABS(UXVDOT(NF,S,GN)))
      ENDIF
      IF (N.GT.0) THEN
      IF (NORMF.GT.0) THEN
      IF (NF.GT.N) THEN
      CALL UXVCOP(NF,GN,CZ(NCZ+1))
      CALL UXDRMM(NF,N,CZ,CZ(NCZ+1),GN)
      ELSE
      CALL UXVMUL(NF,XN,G,CR,1)
      CALL UXDRMM(NF,N,CZ,CR,GN)
      ENDIF
      ELSE
      CALL UXDRMM(NF,N,CZ,G,GN)
      ENDIF
C      GMAX=UXVMAX(N,GN)
      ENDIF
      RETURN
      END
